# $Date: 1995/08/25 19:00:54 $ $Author: kg $ $Revision: 1.10 $ #
#++
Cat::AbelianMonoid -- the category of abelian monoids

An Cat::AbelianMonoid is a Cat::AbelianSemiGroup with a neutral element "zero"
according to the operation '+'.

Use axiom 'Ax::normalRep' to state that "zero" is always represented
canonically.

If a domain has not the axion 'Ax::normalRep' then "zero" is only one
possible representation of the neutral element. Use "iszero" to
test for 0 in such a case.

Entries:-
zero         - the neutral Element according to '_plus'

Methods:-
iszero(x)    - returns TRUE iff x is zero
intmult(x,i) - returns x+x+...+x (i times) for a positive integer i and
	       zero for i=0
++#

Cat::AbelianMonoid:= CategoryConstructor(
    Cat::AbelianMonoid,
    [],
    [],
    NIL,
    [ Cat::AbelianSemiGroup ],
    [ (if this::hasProp(Ax::canonicalRep) then Ax::normalRep end_if) ],

    "zero",

    "iszero" = ( if this::hasProp(Ax::normalRep) then
	proc(x) begin bool(x = this::zero) end_proc
    else
	proc(x) begin bool(this::equal(x, this::zero)) end_proc
    end_if ),

    "intmult" = domains::repeatedSquaring(this, "_plus", "zero")

)():

