# $Date: 1995/11/02 09:22:57 $ $Author: kg $ $Revision: 1.9 $
#
#++
Cat::FactorialDomain -- the category of factorial domains (unique factorisation
		   domains)

Please note the following inconsistency: An Dom::Integer is represented
as DOM_INT and factored by 'ifactor'. If 'factor' is called with an
integer it returns that integer unfactored (assuming it is a trivial
polynomial). So the method this::factor MUST be used to get a
factorization for any factorial domain. Do not rely on overloading!

Methods:
irreducible(x) - returns TRUE iff x is prime
sqrfree(x) - returns the square-free factorisation of x in form
	     of a list [u, f1, e1, ... ,fn, en] where u is a unit and fi are
	     square-free factors with exponents ei, ie.
	     x = u * f1^e1 * ... * fn^en.
factor(x)  - returns the factorisation of x in form of a list
             [u, f1, e1, ... ,fn, en] where u is a unit and fi are
	     irreducible elements with exponents ei, ie.
	     x = u * f1^e1 * ... * fn^en.
Factor(x)  - returns the expression u * f1^e1 * ... * fn^en, with the 
	     factorisation given by this::factor
++#

Cat::FactorialDomain:= CategoryConstructor(
    Cat::FactorialDomain,
    [],
    [],
    NIL,
    [ Cat::GcdDomain ],
    [],

    "factor",

    "sqrfree" = proc(x) local i, j, f, e; begin
	x:= this::factor(x);
	f:= table();
	for i from 3 to nops(x) step 2 do
	    if contains(f, x[i]) then
		f[x[i]]:= this::_mult(f[x[i]], x[i-1])
	    else
		f[x[i]]:= x[i-1]
	    end_if;
	end_for;
	e:= sort(map([op(f)], op, 1));
	[x[1], (f[e[j]], e[j]) $ j=1..nops(e) ]
    end_proc,

    "irreducible" = proc(x) begin
	x:= this::factor(x);
	if nops(x) = 3 then
	    if x[3] = 1 then
		return(TRUE)
	    end_if
	end_if;
	FALSE
    end_proc,

    "Factor" = proc(x) local i; begin
        x:= this::factor(x);
        if nops(x) = 1 then
            x[1]
        else
	    hold(_mult)(x[1], hold(_power)(x[2*i], x[2*i+1])
	    			$ i=1..(nops(x) div 2))
        end_if
    end_proc

)():

