# #
# $Date: 1997/03/05 12:08:08 $ $Author: olli $ $Revision: 1.18 $ #
# #
# frankp, 13/06/94 #

#++

	Cat::MatrixCat  --  represents the category of matrices

	Cat::MatrixCat(R)

	R - the coefficient domain (a Cat::Rng)

	The category Cat::MatrixCat(R) represents all matrices over 
	the Cat::Rng R.

	The method "dimen" has to return the dimension of a matrix 
	in form of a list [nrows,ncols].
	The method "new" of a respected domain of these category has 
	to define a m times n matrix by "this::new( m,n )". This is 
	necessary for method "_mult" of this category to define a 
	new matrix of a certain dimension.

	The methods "_index", "set_index" and "dimen" need the internal
	representation of a matrix. Therefore these methods have to be 
	implemented in a respected domain.

	Methods:
	--------
	coeffRing, newThis, _mult, _plus, _negate, _subtract, iszero
	equal, row, col, setRow, setCol, delRow, delCol, swapRow,
	swapCol, delRow, delCol, concatMatrix, stackMatrix,
	transpose
++#

Cat::MatrixCat := CategoryConstructor(

# name #
    Cat::MatrixCat,
# arguments #
    [ R ],
# local variables #
    [],
# initialisation #
    (if args(0) <> 1 then error("wrong no of args") end_if;
     if R::hasProp(Cat::Rng) <> TRUE then
	error("coefficients must be from a Cat::Rng")
    end_if),
# super-categories #
    [ Cat::SetCat ],
# axioms #
    [ ],
# basis methods #
    "_index", "set_index", "dimen", 
# further methods #
    "newThis" = this::new,
# #
    "coeffRing" = R,
# #
    "_plus" = proc(x)
        local y, i, j, d;
    begin
        if args(0) = 1 then return( x ) end_if;
	for y in args() do
	    if y::hasProp( this ) <> TRUE then
		return( FAIL )
	    end_if
	end_for;

	d:= x::dimen( x );
        for i from 2 to args(0) do
            y:= args(i);
            if y::dimen( y ) <> d then
                error ("dimensions don't match")
            end_if;
	    ((x[i,j]:= R::_plus( x[i,j],y[i,j] )) $ j=1..d[2]) $ i=1..d[1];
        end_for;

	x
    end_proc,
# #
    "_negate" = proc(x)
	local d, i, j;
    begin
	d:= x::dimen( x );
	((x[i,j]:= R::_negate( x[i,j] )) $ j=1..d[2]) $ i=1..d[1];

	a
    end_proc,
# #
    "_subtract" = fun( this::_plus( args(1),this::_negate(args(2)) ) ),
# #
    "iszero" = proc(x)
	local i, j, d;
    begin
	d:= x::dimen( x );
	for i from 1 to d[1] do
	    for j from 1 to d[2] do
		if not R::iszero( x[i,j] ) then return( FALSE ) end_if
	    end_for
	end_for;

	TRUE
    end_proc,
# #
    "equal" = fun(
	(if this::dimen(args(1)) <> this::dimen(args(2)) then 
	     FALSE
	 else
	     this::iszero( this::_plus( args(1),this::_negate(args(2)) ) )
	 end_if)
    ),
# #
    "row" = proc(x,p)
	local d, r, j;
    begin
	d:= x::dimen( x );
	r:= x::new( 1,d[2] );
	(r[1,j]:= x[p,j]) $ j=1..d[2];

	r
    end_proc,
# #
    "col" = proc(x,p)
        local d, c, i;
    begin
        d:= x::dimen( x );
        c:= x::new( d[1],1 );
        (c[i,1]:= x[i,p]) $ i=1..d[1];

        c
    end_proc,
# #
    "delRow" = proc(x,p)
	local i, j, ii, d, rx;
    begin
	d:= x::dimen( x );
	rx:= x::new( d[1]-1,d[2] );
	ii:= 1;
	for i from 1 to d[1] do
	    if i <> p then
		(rx[ii,j]:= x[i,j]) $ j=1..d[2];
		ii:= ii + 1
	    end_if
	end_for;

	rx
    end_proc,
# #
    "delCol" = proc(x,p)
        local i, j, jj, d, cx;
    begin
        d:= x::dimen( x );
        cx:= x::new( d[1],d[2]-1 );
        jj:= 1;
        for j from 1 to d[2] do
            if j <> p then
                (cx[i,jj]:= x[i,j]) $ i=1..d[1];
                jj:= jj + 1
            end_if
        end_for;

        cx
    end_proc,
# #
    "swapRow" = proc(x,k,l)
	local d, j, t;
    begin
	d:= x::dimen( x );
	for j from 1 to d[2] do
	    t:= x[k,j]; x[k,j]:= x[l,j]; x[l,j]:= t
	end_for;

	x
    end_proc,	
# #
    "swapCol" = proc(x,k,l)
        local d, i, t;
    begin
        d:= x::dimen( x );
        for i from 1 to d[1] do
            t:= x[i,k]; x[i,k]:= x[i,l]; x[i,l]:= t
        end_for;

        x
    end_proc,  
# #
    "setRow" = proc(x,p,y)
	local d, j;
    begin
	d:= x::dimen(x);
	(x[p,j]:= y[1,j]) $ j=1..d[2];

	x
    end_proc,
# #
    "setCol" = proc(x,p,y)
        local d, i;
    begin
        d:= x::dimen(x);
        (x[i,p]:= y[i,1]) $ i=1..d[1];
        
        x
    end_proc,
# #
    "transpose" = proc(x)
        local d, i, j, t;
    begin
	d:= x::dimen( x );
	t:= x::new( d[2],d[1] );
        ((t[j,i]:= x[i,j]) $ j=1..d[2]) $ i=1..d[1];

	t
    end_proc,
# #
    "concatMatrix" = proc(x,y)
	local dx, dy, r, cm, i, j;
    begin
	dx:= x::dimen( x );
	dy:= y::dimen( y );
	if dx[1] <> dy[1] then
	    error("incompatible operands (different no of rows)")
	end_if;

	r:= dx[1];
	cm:= x::new( r,dx[2]+dy[2] );
	((cm[i,j]:= x[i,j]) $ j=1..dx[2]) $ i=1..r;
	((cm[i,j+dx[2]]:= y[i,j]) $ j=1..dy[2]) $ i=1..r;

	cm
    end_proc,
# #
    "stackMatrix" = proc(x,y)
        local dx, dy, c, cm, i, j;
    begin
        dx:= x::dimen( x );
        dy:= y::dimen( y );
        if dx[2] <> dy[2] then
            error("incompatible operands (different no of columns)")
        end_if;

        c:= dx[2]; 
        cm:= x::new( dx[1]+dy[1],c );
        ((cm[i,j]:= x[i,j]) $ j=1..c) $ i=1..dx[1];
        ((cm[i+dx[1],j]:= y[i,j]) $ j=1..c) $ i=1..dy[1];

        cm
    end_proc

):

# end of file #
