# $Date: 1999/09/06 14:10:01 $ $Author: frankp $ $Revision: 1.19.2.1 $ #
#++
Cat::PolynomialCat -- the category of polynomials

Cat::PolynomialCat(R)

R - coefficient ring

Methods:
indets(p)        - returns list of indeterminates of p
mainvar(p)	 - returns the main variable of p
degree(p)        - returns total degree of p
degree(p,x)      - returns degree of variable x
degreevec(p)     - returns exponent list of leading term
evalp(p,x=v...)  - evaluates p at x = v
lcoeff(p)        - returns leading coefficient of p
lmonomial(p)     - returns leading monomial of p
lterm(p)         - returns leading term of p
nterms(p)        - returns number of non-zero terms of p
nthterm(p,n)     - returns term number n of p, the order may be not represent
		   the term ordering given by 'lcoeff'.
nthcoeff(p,n)    - returns coefficient number n of p
nthmonomial(p,n) - returns monomial number n of p
tcoeff(p)        - returns the trailing coefficient of p
coeff(p)         - returns a list of all non-zero coefficients
coeff(p,x,n)     - returns the coefficient of x^n
coeff(p,n)       - returns the coefficient of x^n where x is the main variable
multcoeffs(p,c)  - multiplies the coefficients of p by c
mapcoeffs(p,f,a...) - replaces the coefficients ci of p by f(ci,a,...)
content(p)       - returns the content of p
primpart(p)      - returns the primitive part of p
solve(p,x[,...]) - finds roots of p


Entries:
coeffRing	 - the coefficient ring
++#

Cat::PolynomialCat:= CategoryConstructor(
    Cat::PolynomialCat,
    [ R ],
    [ ],
    ( if args(0) <> 1 then error("wrong no of args") end_if;
      if R::hasProp(Cat::CommutativeRing) <> TRUE then
	error("no commutative ring")
      end_if ),
    [ ( if R::hasProp(Cat::FactorialDomain) then
	    Cat::FactorialDomain
        elif R::hasProp(Cat::GcdDomain) then
	    Cat::GcdDomain
        elif R::hasProp(Cat::IntegralDomain) then
	    Cat::IntegralDomain
	end_if ),
      Cat::PartialDifferentialRing,
      Cat::Algebra(R) ],
    [ ( if R::hasProp(Ax::canonicalUnitNormal) then
	    Ax::canonicalUnitNormal
	end_if ),
      ( if R::hasProp(Ax::closedUnitNormals) then
	    Ax::closedUnitNormals
	end_if ) ],

    "indets", "mainvar", "degree", "degreevec", "lcoeff", "lmonomial",
    "lterm", "nterms", "nthterm", "nthcoeff", "nthmonomial", "tcoeff",
    "coeff", "multcoeffs", "mapcoeffs", "evalp",

    "coeffRing" = R,

    "characteristic" = R::characteristic,
    
    "content" = (if R::hasProp(Cat::Field) then
	R::one
    elif R::hasProp(Cat::GcdDomain) then
	R::gcd @ this::coeff
    end_if),

    "primpart" = (if R::hasProp(Cat::Field) then
	this::unitNormal
    elif R::hasProp(Cat::GcdDomain) then
	proc(x) begin
	    this::unitNormal(mapcoeffs(x, R::_divide, this::content(x)))
	end_proc
    end_if),

    "unitNormalRep" = (if R::hasProp(Ax::canonicalUnitNormal) then
	proc(x) local n; begin
	    n:= R::unitNormalRep(this::lcoeff(x));
	    [ this::multcoeffs(x, n[2]), this::new(n[2]), this::new(n[3]) ]
	end_proc
    end_if),

    "unitNormal" = (if R::hasProp(Ax::canonicalUnitNormal) then
	proc(x) begin
	    this::multcoeffs(x, (R::unitNormalRep(this::lcoeff(x)))[2])
	end_proc
    end_if),

    "isUnit" = proc(x) begin
        if this::degree(x) > 0 then FALSE
        else R::isUnit(this::lcoeff(x))
        end_if
    end_proc,

#--
    solve( p,x [=T] [,lev] [,Multiple] )

    p: polynomial of category this
    x: DOM_IDENT
    T: (optional) Domain
    lev: (optional) positive integer
    Multiple: (optional) DOM_IDENT
--#
   "solve" = proc(p,x,lev)
           local T, opt;
       begin
           if args(0) < 1 or args(0) > 4 then error("expecting 2, 3 or 4 arguments") end_if;
           if p::hasProp( this ) <> TRUE then error("expecting polynomial") end_if; 
           if type(x) = "_equal" then
               if domtype(op(x,1)) <> DOM_IDENT then error("invalid unknown") end_if;
               if domtype(op(x,2)) <> DOM_DOMAIN then error("invalid computation domain") end_if;
               T:= op(x,2); x:= op(x,1)
           else
               if domtype(x) <> DOM_IDENT then error("invalid unknown") end_if;
               T:= R
           end_if;
           case args(0)
           of 1 do
           of 2 do lev:= 4; opt:= FALSE; break
           of 3 do 
               if domtype(lev) = DOM_INT then opt:= FALSE 
               elif lev <> hold(Multiple) then error("expecting 'Multiple' as option")
               else
                   lev:= 4; opt:= TRUE
               end_if;
               break
           of 4 do
               opt:= TRUE;
               if domtype(lev) <> DOM_INT then error("third argument must be a positive integer") end_if;
               if args(4) <> hold(Multiple) then error("expecting 'Multiple' as option") end_if
           end_case;

           if opt then x:= Dom::Multiset( stdlib::solve_poly( poly(p::expr(p),[x]),x,lev ) )
           else x:= {stdlib::solve_poly( poly(p::expr(p),[x]),x,lev )}
           end_if;

           select( map(x,T::coerce),_not@_equal,FAIL )
       end_proc
):

