#++
Cat::RestrictedDifferentialFunctionCat -- differential functions that depend
	only on certain types of differential variables


This category is a specialization of Cat::DifferentialFunctionCat. It does
not provide any further methods but should actually be used more like an
axiom. Domains in this category represent functions that depend only on
prescribed types of differential variables. Such domains are needed for
example as coefficient rings for various domains in
Cat::DifferentialFunctionCat like linear functions or differential
polynomial.


Author: Werner M. Seiler, Uni Karlsruhe
Created: September 30th, 1997
Last update: October 6th, 1997
++#

Cat::RestrictedDifferentialFunctionCat := CategoryConstructor(

	Cat::RestrictedDifferentialFunctionCat,

	[ DV, Types ], [],

	(if args(0)<>2 then error("wrong no of args") end_if;
	 if domtype(DV)<>DOM_DOMAIN or DV::hasProp=FAIL or
	    not(DV::hasProp(Cat::DifferentialVariableCat)) then
		error("first argument must be Cat::DifferentialVariableCat");
	 end_if;
	 if domtype(Types)=DOM_STRING then Types := {Types} end_if;
	 if not(testtype(Types,Type::SetOf(DOM_STRING))) then
		error("second argument must be set of strings");
	 end_if;
	 if nops(Types intersect {"Const","Indep","Dep","Deriv"})<>
	    nops(Types) then
		error("unknown type of differential variable");
	 end_if),

	[ Cat::DifferentialFunctionCat(DV) ], [],

	# polynomial domains want this function to be implemented
	  in their coefficient rings ... #
	"intmult" = fun(this::mult(args(2),args(1)))

):


