# $Date: 1997/10/10 15:55:31 $ $Author: tonner $ $Revision: 1.5 $ #

#++
DomainConstructor::getCategory -- get category by index

DomainConstructor::getCategory(this, i)

this - the domain to get the category of
i    - index

This function is used to iterate the categories of a domain. It returns
the 'i'th category of the domain 'this', where the categories of the
categories graph of 'this' are orderd in a breath-first manner. FAIL is
returned if less than 'i' categories exist for 'this'.

Note that categories are created on demand during the iteration, using the
function DomainConstructor::makeCategory. Categories created so far are
stored in 'this::categories', the entry 'this::categories_idx' is the index
of the next category for which to create the direct super-categories.

The axioms of 'this' defined by the categories are als created as a side-
effect of creating the categories (not very nice).
++#

DomainConstructor::getCategory:= proc( # this, i # )
    option noDebug;
begin
    if args(2) <= nops((args(1))::categories) then
    	(args(1))::categories[args(2)]
    elif (args(1))::categories_idx > nops((args(1))::categories) then
    	FAIL
    else
    	DomainConstructor::makeCategory(args())
    end_if
end_proc:

DomainConstructor::makeCategory:= proc(this #, i #)
    option noDebug;
    local _0_C, _0_S, _0_A;
begin
    while (args(1))::categories_idx <= nops((args(1))::categories) do
	_0_S:= (args(1))::categories[(args(1))::categories_idx];
	for _0_C in extop(extop(_0_S,1),4) do
	    # make super-category #
	    _0_C:= CategoryConstructor::subsArgs(_0_C, _0_S, args(1));
	    if _0_C = hold(_0_C) then next end_if;
	    if contains((args(1))::categories, _0_C) <> 0 then next end_if;
	    if domtype(_0_C) <> Category then
		error("invalid category")
	    end_if;
	    sysassign(domattr(args(1), "categories"), append((args(1))::categories, _0_C));
	    # make axioms #
	    _0_A:= { op(map(extop(extop(_0_C,1),5),
			    CategoryConstructor::subsArgs, _0_C, args(1))) }
		   minus { NIL };
	    if _0_A <> {} then
		if map(_0_A, domtype) <> {Axiom} then
		    error("invalid axiom")
		end_if;
		sysassign(domattr(args(1), "axioms"), (args(1))::axioms union _0_A);
	    end_if;
	end_for;
	sysassign(domattr(args(1), "categories_idx"), (args(1))::categories_idx+1);
	if args(2) <= nops((args(1))::categories) then 
	    return((args(1))::categories[args(2)]) 
	end_if;
    end_while;
    FAIL
end_proc:

# end of file #
