# $Date: 1997/10/10 15:55:32 $ $Author: tonner $ $Revision: 1.9 $ #

#++
hasProp -- check if a domain has a certain property

hasProp(this, attr)

this - domain
attr - attribute to check for (super-domain, category, axiom or constructor)

hasProp returns TRUE, if the domain has super-domain, category or axiom
'attr' or a constructor of one of these objects. The method normally is not
called directly but rather by the entry 'hasProp' of a domain. (This method
is defined by the domain 'Dom::BaseDomain'.)
++#

DomainConstructor::hasProp:= proc(this #, attr# )
    option noDebug;
    local _0_i, _0_C;
begin
    if testargs() then
        if args(0) <> 2 then error("wrong no of args") end_if;
        if domtype(args(1)) <> DOM_DOMAIN then
            error("invalid domain")
        end_if;
        if (args(1))::axioms = FAIL or (args(1))::categories = FAIL then
            error("not created by constructor")
        end_if;
    end_if;

    case domtype(args(2))
    of Axiom do
        # incrementally loading categories, thereby creating their axioms #
        _0_i:= 1;
	while TRUE do
	    if contains((args(1))::axioms, args(2)) then 
	        return(TRUE) 
	    end_if;
	    if (args(1))::categories_idx > nops((args(1))::categories) then
	        return(FALSE);
	    end_if;
	    if DomainConstructor::getCategory(args(1), _0_i) = FAIL then
	        return(FALSE)
	    end_if;
	    _0_i:= _0_i+1;
	end_while;
	break;
    of AxiomConstructor do
        _0_i:= 1;
	while TRUE do
	    if contains(map((args(1))::axioms, extop, 1), args(2)) then
	        return(TRUE) 
	    end_if;
	    if (args(1))::categories_idx > nops((args(1))::categories) then
	        return(FALSE);
	    end_if;
	    if DomainConstructor::getCategory(args(1), _0_i) = FAIL then
	        return(FALSE)
	    end_if;
	    _0_i:= _0_i+1;
	end_while;
	break;
    of Category do
	if contains((args(1))::categories, args(2)) <> 0 then
	    TRUE
	elif (args(1))::categories_idx > nops((args(1))::categories) then
	    FALSE
	else
	    # incrementelly loading categories #
	    _0_i:= 1;
	    while (_0_C:= DomainConstructor::getCategory(args(1), _0_i)) <> FAIL do
	        if _0_C = args(2) then return(TRUE) end_if;
	        _0_i:= _0_i+1
	    end_while;
	    FALSE
	end_if;
	break;
    of CategoryConstructor do
	if contains(map((args(1))::categories, extop, 1), args(2)) <> 0 then
	    TRUE
	elif (args(1))::categories_idx > nops((args(1))::categories) then
	    FALSE
	else
	    _0_i:= 1;
	    while (_0_C:= DomainConstructor::getCategory(args(1), _0_i)) <> FAIL do
	        if extop(_0_C,1) = args(2) then return(TRUE) end_if;
	        _0_i:= _0_i+1
	    end_while;
	    FALSE
	end_if;
	break;
    of DOM_DOMAIN do
        if args(1) = args(2) then
            TRUE
        else
            bool(contains((args(1))::super_domains, args(2)) <> 0)
        end_if;
        break;
    of DomainConstructor do
        if (args(1))::constructor = args(2) then
            TRUE
        else
            bool(contains(map((args(1))::super_domains, domattr, "constructor"),
            		  args(2)) <> 0)
        end_if;
        break;
    otherwise error("invalid attribute");
    end_case
end_proc:

domains::hasProp:= DomainConstructor::hasProp:

# end of file #
