# $Date: 1996/09/13 09:17:07 $ $Author: kg $ $Revision: 1.17 $ #
#++
Dom::BaseDomain -- the base domain of the 'domains'-library

Methods:
equal(x,y)	   - tests if x and y are equal (for Ax::canonicalRep only)
TeX(x)		   - returns string for object x in TeX-format
subs(x,...)	   - returns x
subsex(x,...)	   - returns x

hasProp(a)         - tests if this domain has super-domain, category, axiom
		     or constructor a
getAxioms()        - returns set with direct axioms
allAxioms()        - returns set with all axioms
getSuperDomain()   - returns direct super-domain
allSuperDomains()  - returns list with all super-domains
getCategories()    - returns list with direct categories
allCategories()    - returns list with all categories
whichEntry(e)      - returns domain or category defining entry e
allEntries()       - returns set with all entry names
undefinedEntries() - returns set with all entrys not defined yet
info()		   - prints some information
++#

Dom::BaseDomain:= DomainConstructor(
    Dom::BaseDomain,
    [],
    [],
    NIL,
    NIL,
    [ Cat::SetCat ],
    [],

    "convert_to" = fun((
	case args(2)
	  of this do return(args(1));
	  of Dom::Expression do return(this::expr(args(1)));
	end_case;
	FAIL
    )),

    "equal" = (if this::hasProp(Ax::canonicalRep) then
	bool @ _equal
    end_if),

    "TeX" = fun(generate::TeX(this::expr(args(1)))),

    "subs" = fun(args(1)),

    "subsex" = fun(args(1)),

    "hasProp" = fun(DomainConstructor::hasProp(this, args(1))),

    "getAxioms" = fun(DomainConstructor::axioms(this)),

    "allAxioms" = fun((
    	# force loading all categories now #
    	this::allCategories();
    	this::axioms)),

    "getSuperDomain" = fun(this::super_domains[1]),

    "allSuperDomains" = fun(this::super_domains),

    "getCategories" = fun(DomainConstructor::categories(this)),

    "allCategories" = proc() local _0_i; begin
	# force loading all categories now #
        _0_i:= 1;
        while DomainConstructor::getCategory(this, _0_i) <> FAIL do
            _0_i:= _0_i+1
        end_while;
        this::categories
     end_proc,

    "whichEntry" = fun(DomainConstructor::whichEntry(this, args(1))),

    "allEntries" = fun(DomainConstructor::allEntries(this)),

    "undefinedEntries" = fun(DomainConstructor::undefinedEntries(this)),
    
    "info" = proc() local PRETTY_PRINT, t, p, nam; begin
        PRETTY_PRINT:= FALSE;
        p:= fun(print(Unquoted, args()));
        p("-- Domain:");
	nam:= (if this::name = FAIL then this::key else this::name end_if);
        if this::info_str = FAIL then
	    print(nam)
        else
	    p(expr2text(nam)." -- ".this::info_str)
        end_if;
        p("");
        p("-- Constructor:");
        print(this::constructor);
        p("");
        p("-- Super-Domains:");
        print(op(this::allSuperDomains()));
        p("");
        p("-- Categories:");
        print(op(this::allCategories()));
        p("");
        t:= this::allAxioms();
        if nops(t) = 0 then
	    p("-- No Axioms.")
        else
	    p("-- Axioms:");
	    print(op(t))
        end_if;
        p("");
        p("-- Entries:");
        p(op(sort([op(this::allEntries())])));
    end_proc

)():

