# #
# frankp, 2.04.1995 #
# #

#++
	Dom::Complex  --  the field of complex numbers

	The test whether an expression is complex is performed by 
	calling 'rectform'.

	The domain has category Cat::Field, which is very optimistic 
	(indeed false, because for example bool(1.0 = float(3) / float(3))
	FALSE. The category is assumed only because of pragmatism.
	Note that elements may not have an unic representation, for example
	bool( 0 = sin(2)^2 + cos(2)^2 - 1 ) returns FALSE.

	Note: Unknowns defined in "globalReal" of 'rectform' do not 
        influence the process of the testing of Complex, i.e. the set is 
        redefined to the empty set during the test.
++#

Dom::Complex := DomainConstructor::new(

# Name = # Dom::Complex,

# Args = # [ ],

# Local Paramameters = # [ ],

# Init = # (
    if args(0) <> 0 then
	error("no arguments expected")
    end_if),

# Super Domains = # Dom::BaseDomain,

# Categories = # [ Cat::DifferentialRing, Cat::Field ],

# Axioms = # [ ],

# Methods = #

"new" = proc(x)
begin
    x:= this::convert(x);
    if x = FAIL then error("invalid argument") else x end_if
end_proc,

"D" = 0,

"diff" = fun((if args(0) = 1 then args(1) else 0 end_if)),

"characteristic" = 0,

"convert" = proc(x)
    local d, t;
begin
    case domtype(x)
    of DOM_INT do
    of DOM_RAT do
    of DOM_FLOAT do
    of DOM_COMPLEX do
        return( new(this,x) )
    end_case;

    d:= rectform::globalReal({});
    t:= rectform( x,{} );
    rectform::globalReal(d);

    t:= rectform( expr(x),{} );
    if op(t,3) = 0 and not has(t,{hold(Re),hold(Im)}) then new(this,x)
    else FAIL
    end_if
end_proc,

"convert_to" = proc(x,T)
begin
    if domtype(T) <> DOM_DOMAIN then T:= domtype(T) end_if;

    case T
    of this 	     do
    of Dom::Numeric  do
    of DOM_COMPLEX   do
    of DOM_EXPR	     do return( T::convert(extop(x,1)) )
    otherwise
	if T::constructor = Dom::ExpressionField then
	    return( T::new( extop(x,1) ) )
	else
	    return( FAIL )
	end_if
    end_case
end_proc,

"random" = proc()
begin
    if args(0) = 0 then new(this,random(args())+I*random(args()))
    else 
        subsop( proc() begin new(this,X()+I*X()) end_proc,[4,2,1,0]=random(args()),[4,2,2,2,0]=random(args()) )
    end_if
end_proc,

"_plus" = proc()
    local t, o, i;
begin
    t:= [args()];
    i:= 1; o:= [];
    while i <= nops(t) and domtype(t[i]) = this do o:= append(o,extop(t[i],1)); i:= i+1 end_while;
    if i > nops(t) then new( this,_plus(op(o)) )
    else (t[i])::_plus( new( this,_plus(op(o)) ),op(t,i..nops(t)) )
    end_if
end_proc,


"_mult" = proc()
    local t, o, i;
begin
    t:= [args()];
    o:= map(t, this::coerce);
    if (i:=contains(o, FAIL)) > 0 then
	(t[i])::_mult(args())
    else
	new(this, _mult(op(map(o, extop, 1))))
    end_if 
end_proc, 

"_power" = proc(x,y)
begin
    if domtype(x) = this and domtype(y) = this then
	this::convert( extop(x,1)^extop(y,1) )
    elif domtype(x) = this then
	this::convert( extop(x,1)^y )
    else
	this::convert( x^extop(y,1) )
    end_if
end_proc,

"_invert" = fun(new(this,1/extop(args(1),1))),

"_negate" = fun(new(this,-extop(args(1),1))),

"zero" = new( this,0 ),

"one" = new( this,1 ),

"expr" = fun(extop(args(1),1)),

"Re" = fun(new(this,Re(extop(args(1),1)))),

"Im" = fun(new(this,Im(extop(args(1),1)))),

"conjugate" = fun(new(this,conjugate(extop(args(1),1)))),

"iszero" = fun((
    if iszero(extop(args(1),1)) or iszero(normal(extop(args(1),1))) then TRUE
    else FALSE
    end_if
)),

"equal" = fun(_equal(extop(args(1),1),extop(args(2),1))),

"norm" = fun( new( this,abs(extop(args(1),1)) ) ),

"abs" = fun( new( this,abs(extop(args(1),1)) ) ),

"_divide" = fun( new( this,normal(extop(args(1),1)/extop(args(2),1)) ) )

)():				

# end of file #
