#++
Dom::DifferentialExpression(DV) -- general expressions in differential variables

Dom::DifferentialExpression(DV) is an instance of Cat::DifferentialFunctionCat.
It implements general expression in the differential variables from the domain
DV (an element of Cat::DifferentialVariableCat). The implementation is based
on DOM_EXPR.


Author: Werner M. Seiler, Uni Karlsruhe
Created: September 30th, 1997
Last update: January 21st, 1998
++#

alias(REP(x)=extop(x,1),
      VARS(x)=extop(x,2),
      TYPE_ERR=error("wrong type of argument"),
      ARG_ERR=error("wrong no of args")):


Dom::DifferentialExpression := DomainConstructor(

	Dom::DifferentialExpression,

	[ DV ],

	[ DVcoerce, DVconvert, DVconvto, DVexpr, DVdiff, DVindex, DVtype, nn ],

	(if args(0)<>1 then ARG_ERR end_if;
	 if domtype(DV)<>DOM_DOMAIN or DV::hasProp=FAIL or
	    not(DV::hasProp(Cat::DifferentialVariableCat)) then
		error("argument must be Cat::DifferentialVariableCat");
	 end_if;

        DVcoerce := DV::coerce;
	DVconvert := DV::convert;
	DVconvto := DV::convert_to;
	DVexpr := DV::expr;
	DVdiff := DV::totalDiff;
	DVindex := DV::index;
	DVtype := DV::vartype;
	nn := DV::numIndVar;
	),

	Dom::BaseDomain,

	[ Cat::DifferentialFunctionCat(DV), Cat::PartialDifferentialRing ],

	[ ],


        # =============== #
        # "local" methods #
        # =============== #

        "generate" = proc(x,vars)
        local i,v;
        begin
            i := 1;
            for v in vars do
                if has(x,v) then i := i+1
                else vars[i] := NIL end_if;
            end_for;
            new(this,x,vars);
        end_proc,
	
	
	"collectVars" = proc()
	local v,res;
	begin
	    res := args(1);
	    for v in [args(2..args(0))] do
		res := listtools::singleMerge(v,res,DV::greater);
	    end_for;
	    res;
	end_proc,
#	"collectVars" = fun((fp::fold(listtools::singleMerge,[]);
			     %(args()))),#


        # ========================= #
        # representation, in/output #
        # ========================= #

        "convert" = fun((
	    case domtype(args(1))
            of this do return(args(1));
            of DV do return(new(this,args(1),[args(1)]));
            of DOM_EXPR do return(this::convertExpr(args(1)));
	    of DOM_IDENT do
		DVconvert(args(1));
		if %=FAIL then return(new(this,args(1),[]))
		else return(new(this,%,[%])) end_if;
	    of DOM_POLY do return(this::convert(expr(args(1))));
	    of DOM_INT do
	    of DOM_RAT do
	    of DOM_FLOAT do
	    of DOM_COMPLEX do return(new(this,args(1),[]));
	    otherwise
		if (args(1))::expr<>FAIL then
		    return(this::convert((args(1))::expr(args(1))));
		end_if;
		DVcoerce(args(1));
		if %=FAIL then return(FAIL)
		else return(new(this,%,[%])) end_if;
            end_case)),
	
	
	"convertExpr" = proc(x)
	local r,xop,xargs,nargs,nvars;
	begin
	    r := DVconvert(x);
	    if r<>FAIL then return(new(this,r,[r])) end_if;
	    xop := op(x,0);
	    xargs := map([op(x)],this::convert);
	    if contains(xargs,FAIL)<>0 then return(FAIL) end_if;
	    nargs := map(xargs,extop,1);
	    nvars := this::collectVars(op(map(xargs,extop,2)));
	    new(this,xop(op(nargs)),nvars);
	end_proc,


        "convert_to" = fun((case args(2)
            of this do return(args(1));
            of DOM_EXPR do return(this::expr(args(1)));
            of DV do
                if domtype(REP(args(1)))=DV then return(REP(args(1)))
                else return(FAIL) end_if;
            of "diff" do
            of "D" do
                if domtype(REP(args(1)))=DV then
                    return(DVconvto(REP(args(1)),args(2)));
                else
                    map(VARS(args(1)),
                        fun(args(1)=DVconvto(args(1),args(2))),
                        args(2));
                    return(subs(REP(args(1)),op(%)));
                end_if;
            otherwise FAIL
            end_case)),


        "expr" = fun((
            if domtype(REP(args(1)))=DV then DVexpr(REP(args(1)))
            else
            	map(VARS(args(1)),fun(args(1)=DVexpr(args(1))));
            	subs(REP(args(1)),op(%));
            end_if)),


	"equal" = fun(bool((VARS(args(1))=VARS(args(2))) and
			   (REP(args(1))=REP(args(2))))),


	"diffVars" = fun(VARS(args(1))),


        "indets" = proc(x)
        local res,opts,flag,pos;
        begin
	    if testargs() then
		if args(0)<1 or args(0)>3 then ARG_ERR end_if;
		if domtype(x)<>this then TYPE_ERR end_if;
	    end_if;

	    if args(0)>1 then
		opts := [args(2..args(0))];
		pos := contains(opts,hold(DiffVars));
		flag := bool(pos=0);
		if not(flag) then opts[pos] := NIL end_if;
	    else flag := TRUE end_if;

            res := {op(VARS(x))};
	    if flag then
		if args(0)>1 and opts<>[] then
		    res := res union indets(REP(x),op(opts))
		else res := res union indets(REP(x)) end_if;
	    end_if;
	    res;
        end_proc,


        # =========== #
        # arithmetics #
        # =========== #

        "zero" = new(this,0,[]),


        "one" = new(this,1,[]),


        "iszero" = fun(iszero(REP(args(1)))),


        "isone" = fun(bool(REP(args(1))=1)),


        "_plus" = fun(this::generate(
                _plus(map(args(),
			  fun((if domtype(args(1))=this then REP(args(1))
			       else args(1) end_if)))),
                this::collectVars(map(args(),
			fun((if domtype(args(1))=this then VARS(args(1))
			     else [] end_if)))))),


        "_mult" = fun(new(this,
                _mult(map(args(),
			  fun((if domtype(args(1))=this then REP(args(1))
			       else args(1) end_if)))),
                this::collectVars(map(args(),
			fun((if domtype(args(1))=this then VARS(args(1))
			     else [] end_if)))))),


        "_negate" = fun(new(this,_negate(REP(args(1))),VARS(args(1)))),


        "_invert" = fun(new(this,_invert(REP(args(1))),VARS(args(1)))),


        # ==================== #
        # differential methods #
        # ==================== #

	"coerceArgs" = fun((DVcoerce(args(1));
			  if %=FAIL then args(1) else % end_if)),


        "diff" = func((
            if args(0)=1 then return(x) end_if;
            if domtype(v)=DV then    # differentiation wrt diff. variable #
                if contains(VARS(x),v)=0 then return(this::zero) end_if;
                subs(REP(x),v=hold(_dummy_0),Unsimplified);
                diff(%,hold(_dummy_0));
                subs(%,hold(_dummy_0)=v,Unsimplified);
            else                     # differentiation wrt parameter #
		if testargs() then
		    DVcoerce(v);
		    if %<>FAIL then
			return(this::diff(x,%,op(map([args(3..args(0))],
						     this::coerceArgs))));
		    end_if;
		end_if;
                diff(REP(x),v);
            end_if;
	    if iszero(%) then return(this::zero)
            else this::generate(%,VARS(x)) end_if;
            if args(0)=2 then %
            else
		if testargs() then
		    this::diff(%,op(map([args(3..args(0))],this::coerceArgs)))
		else this::diff(%,args(3..args(0))) end_if;
	    end_if), x,v),


	"totalDiffJM" = proc(sys,mu,JM)
            local n,vars,nvars,dvars,tdvars,rvars,rows,i,j,k,pos,res,dv,t;
            begin 
                if testargs() then
                    if args(0)<>3 then ARG_ERR end_if;
                    sys := map(sys,this::coerce);
                    if domtype(mu)=DOM_INT then
                        if mu<1 or mu>nn then TYPE_ERR end_if;
                    else
                        mu := DVcoerce(mu);
                        if mu=FAIL then TYPE_ERR end_if;
                        mu := DVindex(mu);
                    end_if;
                    if domtype(JM)<>Dom::SparseMatrix(DV,this,DV::greater) then
                        TYPE_ERR
                    end_if;
                    if nops(sys)<>JM::nrows(JM) then
                        error("incompatible jacobian");
                    end_if;
                end_if;

                n := JM::nrows(JM);
                vars := JM::allIndices(JM);
                if vars=[] then
                    return(([this::zero $ n], [[] $ n]));
                end_if;
                rows := JM::rows(JM);
                nvars := select(vars,fun(contains({"Deriv","Dep"},
                                                  DVtype(args(1)))));
                dvars := map(nvars,DVdiff,mu);
		tdvars := map(dvars,this::convert);

                for i from 1 to n do         # for each equation #
		    if rows[i]=[] then
			sys[i] := this::zero;
			next;
		    end_if;
                    rvars := map(rows[i],op,1);
		    k := 0;
                    res := [ this::zero $ nops(rows[i]) ];
                    for t in rows[i] do      # compute derivative #
                        case DVtype(t[1])
                        of "Dep" do
                        of "Deriv" do
                            pos := contains(nvars,t[1]);
			    k := k+1;
                            res[k] := this::_mult(t[2],tdvars[pos]);
			    break;
                        of "Indep" do
                            if DVindex(t[1])=mu then
				k := k+1;
                                res[k] := t[2];
                            end_if;
                            break;
                        end_case;
                    end_for;
		    sys[i] := this::_plus(op(res,1..k));
                    vars[i] := VARS(sys[i]);
                end_for;
                (sys,vars);
            end_proc,

	
       	# ============== #
	# simplification #
	# ============== #

	"normal" = fun(this::generate(normal(REP(args(1))),VARS(args(1)))),


	"simplify" = fun((if domtype(args(1))=DOM_LIST then
		map(args(1),this::simplify)
	    else this::generate(simplify(REP(args(1))),VARS(args(1)))
	    end_if)),


        # ============= #
        # miscellaneous #
        # ============= #

        "freeOf" = fun(bool(contains(VARS(args(1)),args(2))=0)),


        "leader" = fun((if VARS(args(1))=[] then DV::one
                        else VARS(args(1))[1] end_if)),


	"subs" = proc(de,eq)
	local x,v,vars,res,pos;
	begin
	    if testargs() then
		if args(0)<>2 then ARG_ERR end_if;
		de := this::coerce(de);
		if (de=FAIL) then TYPE_ERR end_if;
		v := DVcoerce(op(eq,1));
		if (v=FAIL) then TYPE_ERR end_if;
		x := DVcoerce(op(eq,2));
		if (x=FAIL) then
		    x := this::coerce(op(eq,2));
		    if (x=FAIL) then x := op(eq,2) end_if;
		end_if;
	    else
	        v := op(eq,1);
	    	x := op(eq,2);
	    end_if;
	    
	    vars := VARS(de);
	    pos := contains(vars,v);
	    if pos=0 then return(de) end_if;
	    vars[pos] := NIL;
	    
	    case domtype(x)
	    of this do
		vars := this::collectVars(VARS(x),vars);
		res := subs(REP(de),v=REP(x));
		break;
	    of DV do
	    	if x=v then return(de) end_if;
		if contains(vars,x)=0 then
		    vars := insert_ordered(x,vars,DV::greater);
		end_if;
		res := subs(REP(de),v=x);
		break;
	    otherwise 
	    	res := subs(REP(de),v=x);
	    end_case;
	    this::generate(res,vars);
	end_proc,


	"solve" = proc(de,dv)
	local sol,vars,pos;
	begin
	    if testargs() then
		if args(0)<>2 then ARG_ERR end_if;
		de := this::coerce(de);
		dv := DVcoerce(dv);
		if (de=FAIL) or (dv=FAIL) then TYPE_ERR end_if;
	    end_if;
	    
	    vars := VARS(de);
	    pos := contains(vars,dv);
	    if pos=0 then return(FAIL) end_if;
	    vars[pos] := NIL;
	    sol := solve(REP(de),{dv});
	    if nops(sol)<>1 then return(FAIL) end_if;
	    sol := op(sol,1);
	    this::generate(sol,vars);
	end_proc,


	"random" = fun((random(1..6);
			this::convert(expr(
				randpoly([DV::random() $ hold(i)=1..%()],
					 hold(Expr)))))),


	"characteristic" = 0

):

unalias(REP,VARS,TYPE_ERR,ARG_ERR):

