#++
Dom::DifferentialProduct(DV,order) -- abelian monoid of power products over
        the domain DV of differential variables. The optional parameter
	order is a string naming the used ordering.

Dom::DifferentialProduct(DV,order) is a special case of
Dom::PowerProduct(DV,order). The only difference is that independent
variables are not accepted in the products (i.e. those for which DV::vartype
yields "Indep"). Hence only the convert method must be redefined.

Additional methods are
        order : returns the maximum of the orders of the variables contained
                in the product
The method convert_to is extended; its second argument can now also be
diff or D.


Author: Werner M. Seiler, Uni Karlsruhe
Created: September 24th, 1997
Last updated: October 15th, 1997
++#

alias(REP(x)=extop(x,1)):


Dom::DifferentialProduct := DomainConstructor(

    Dom::DifferentialProduct,

    [ DV, order ],

    [ DVcoerce, DVconvto, DVorder, DVtype ],

    (if args(0)<1 or args(0)>2 then error("wrong number of arguments") end_if;
     if domtype(DV)=DOM_DOMAIN and (DV::hasProp<>FAIL) and
	  DV::hasProp(Cat::DifferentialVariableCat) then
	 DVcoerce := DV::coerce;
	 DVconvto := DV::convert_to;
	 DVorder := DV::order;
	 DVtype := DV::vartype;
     else error("first argument must be Cat::DifferentialVariableCat") end_if;

     if args(0)<2 then order := LexOrder;
     elif contains({ LexOrder, RevLexOrder, InvLexOrder,
		     DegreeOrder, DegRevLexOrder, DegInvLexOrder },
                   order)=0 then error("undefined ordering") end_if;
     ),

    Dom::PowerProduct(DV,order),

    [ Cat::OrderedAbelianMonoid, Cat::OrderedMonoid ],

    [ Ax::canonicalRep, Ax::normalRep ],


    # ========================= #
    # representation, in/output #
    # ========================= #

    "convert" = proc(x)
        local s,res,e,f,facts;
        begin
            case domtype(x)
            of this do return(x);
            of DOM_LIST do
		if nops(x)=2 and (s:=DVcoerce(x[1]))<>FAIL and
		   testtype(x[2],Type::PosInt) then
		    if DVtype(s)="Indep" then return(FAIL) end_if;
		    return(new(this,[[s,x[2]]]));
		elif {op(map(x,domtype))}<>{DOM_LIST} then return(FAIL)
		elif {op(map(x,nops))}<>{2} then return(FAIL)
		else
		    x := map(x,this::coerceFactor);
		    if has(x,FAIL) then return(FAIL)
		    else
			for s in x do
			    if DVtype(s[1])="Indep" then return(FAIL) end_if;
			end_for;
			return(new(this,this::normalize(x)))
		    end_if;
		end_if;
            of DOM_EXPR do
                case type(x)
                of "_mult" do
                    res := [];
                    facts := op(x);
                    for f in facts do
                        case type(f)
                        of "_power" do
                            s := DVcoerce(op(f,1));
                            if s=FAIL or DVtype(s)="Indep" then
				return(FAIL)
			    end_if;
                            e := op(f,2);
                            if testtype(e,Type::PosInt) then
                                res := append(res,[s,e]);
                            elif e<>0 then return(FAIL) end_if;
                            break;
                        otherwise
                            s := DVcoerce(f);
                            if s=FAIL or DVtype(s)="Indep" then
				return(FAIL)
			    end_if;
			    res := append(res,[s,1]);
                            break;
                        end_case;
                    end_for;
                    return(new(this,this::normalize(res)));
                of "_power" do
                    s := DVcoerce(op(x,1));
                    if s=FAIL or DVtype(s)="Indep" then return(FAIL) end_if;
                    e := op(x,2);
                    if testtype(e,Type::PosInt) then
                        return(new(this,[[s,e]]));
                    else return(FAIL) end_if;
		otherwise
                    s := DVcoerce(x);
                    if s=FAIL or DVtype(s)="Indep" then return(FAIL) end_if;
                    return(new(this,[[s,1]]));
                end_case;
                return(FAIL);
            otherwise
                s := DVcoerce(x);
                if s=FAIL or DVtype(s)="Indep" then return(FAIL) end_if;
                return(new(this,[[s,1]]));
            end_case;
            FAIL;
        end_proc,


    "convert_to" = fun((case args(2)
        of DOM_LIST do return(REP(args(1)));
        of "diff" do
        of "D" do
            if this::isone(args(1)) then return(1) end_if;
            map(REP(args(1)),fun(subsop(args(1),
                                 1=DVconvto(args(1)[1],args(2)))),args(2));
	    if nops(%)=1 then
		%[1];
		if %[2]=1 then %[1] else hold(_power)(op(%)) end_if;
	    else
                map(%,fun((if args(1)[2]=1 then args(1)[1]
                           else hold(_power)(op(args(1))) end_if)));
                hold(_mult)(op(%));
	    end_if;
	    break;
        of this do return(args(1));
        of DOM_EXPR do return(this::expr(args(1)));
        otherwise return(FAIL);
        end_case)),


    # ============= #
    # miscellaneous #
    # ============= #

    "order" = fun(max(op(map(REP(args(1)),fun(DVorder(args(1)[1]))))))

):

unalias(REP):
