#  $Date $      $Author $     $Revision $  #    

# stefanw, 12.03.1996 #

#++

Dom::DihedralGroup(n) - dihedral group of order n, i.e. the group generated by
elements s and d such that o(s)=2, o(d)=n and ds=sd^(-1)

n - integer >= 1

An element is represented as a domain element with operands:

0 - domain Dom::DihedralGroup(n)
1 - a list [a,b ] of two integers, with 0<=a<=1 and 0<=b<=n-1
    representing s^a d^b

++#

Dom::DihedralGroup:= DomainConstructor(
    Dom::DihedralGroup,
    [ n ],
    [ ],
    (if args(0) <> 1 then error("wrong no of args") end_if;
     if domtype(n) <> DOM_INT then
	 error("argument must be integer")
     end_if;
     if n < 1 then
	 error("argument must be positive")
     end_if),
    Dom::BaseDomain,
    [ Cat::Group ],
    [ Ax::canonicalRep ],

    "size" = 2*n,

    "one" = new(this, [0,0]),


    "convert" = fun((
	case domtype(args(1))
	of this do 
		return(args(1));
	of DOM_ARRAY do 
		return(this::convert([op(args(1))]));
	of DOM_LIST do
	 if nops(args(1)) <> 2 then 
		error("List of length two expected");
	 elif {op(map(args(1),domtype)) } <> { DOM_INT } then
		error("List must consist of integers");
	 else
	 	return(new(this, [op(args(1),1)mod 2,op(args(1),2) mod n]));
	 end_if;
	end_case;
        FAIL
    )),

    "convert_to" = fun((
	case args(2)
	of DOM_LIST do	
		 return(extop(args(1),1));
	of this do return(args(1));
	end_case;
	FAIL
    )),

    
	

    "expr" = fun( extop(args(1),1) ),  # gibt nicht DOM_EXPR, sondern
					 DOM_LIST zurueck # 
					 

    "print" = fun( extop(args(1),1) ),



    "_mult" = fun((
    	if args(0) = 2 then
	    if domtype(args(2)) <> this or 
	       domtype(args(1)) <> this then
	    	    FAIL
	    elif op(extop(args(2),1),1) mod 2=0 then
		    new(this,[op(extop(args(1),1),1) mod 2,
			op(extop(args(1),1),2)+op(extop(args(2),1),2) mod n])
	            # s^a*d^b*d^c gives s^a*d^(b+c) #
	    else  
		    new(this,[op(extop(args(1),1),1)+1 mod 2,
			op(extop(args(2),1),2)-op(extop(args(1),1),2) mod n])
	            # s^a*d^b*s*d^c= s^a*d^(b-1)*(d*s)*d^c
				   = s^a*d^(b-1)*s*d^(c-1)
				   =  ...
				   = s^(a+1)*d(c-b)     #
	    end_if
    	elif args(0) = 1 then
    	    args(1)
    	else
    	    _mult(args(1..(args(0) div 2)));
    	    _mult(args(((args(0) div 2) + 1)..args(0)));
	    if (domtype(%2))::_mult <> FAIL then (domtype(%2))::_mult(%2, %1)
	    elif (domtype(%1))::_mult <> FAIL then (domtype(%1))::_mult(%2, %1)
	    else _mult(%2, %1)
	    end_if
    	end_if
    )),

    "_power" = fun((
        if testargs() then
            if not testtype(args(2), Type::NonNegInt) then
                  error("illegal power")
            end_if
        end_if;
	if op(extop(args(1),1),1)=1 then
		if args(2) mod 2 = 0 then 
			new(this,[0,0])
		else
		   args(1)
		end_if
	else
		new(this,[0,op(extop(args(1),1),2)*args(2) mod n])
	end_if
     )),



    "_invert" = fun((
	if op(extop(args(1),1),1)=1 then
		args(1)
	else
		new(this,[0, n-op(extop(args(1),1),2) mod n])
	end_if
	 )),



#    "_divide" : defined in Cat::Group  # 

	"order" = fun
	((	
	 if op(extop(args(1),1),1)=1 then 2
	 else n/igcd(n,op(extop(args(1),1),2))
	 end_if
	)),
	 		

    "TeX" = fun(
	expr2text(extop(args(1),1))
    ),

    "random" = fun
	(
		new(this,[random() mod 2,random() mod n])
	)
			
		
):

# end of file #
