# domain of streams:
  Dom::DiscreteSet(e,x=t) represents the discrete (possibly infinity) set of 
  			  all values of e where x is in t
#

alias(EXPRESSION(x)=extop(x,1)): alias(EXPIND=1):
alias(VARIABLE(x)=extop(x,2)):
alias(SET(x)=extop(x,3)):
alias(CONDITION(x)=extop(x,4)): alias(CONDIND=4):
alias(DOTS=stdlib::dots):

Dom::DiscreteSet := DomainConstructor(
# name #
    Dom::DiscreteSet,
# arguments #
    [ ],
# local variables #
    [ ],
# initialisation #
    (if args(0) <> 0 then error("wrong no of args") end_if;
     sysassign(DOTS, domain("Dom::DiscreteSet...")); 
     sysassign(DOTS::name,"...")),
# super-domains #
    Dom::BaseDomain,
# categories #
    [ Cat::SetCat ],
# axioms #
    [ ],
# methods #
    # "elemattr" = proc(x,f) begin extsubsop(x,EXPIND=f(EXPRESSION(x),args(3..args(0)))) end_proc, #

    "_negate" = fun(extsubsop(args(1),EXPIND=-EXPRESSION(args(1)))),

    "expr" = proc(x) local i; begin
       # it is a pity that extop(x,4..args(0)) does not work !
         otherwise we would not need a local variable and we could use a function #
       hold(Dom::DiscreteSet)(extop(x,1),extop(x,2)=extop(x,3),extop(x,i)$i=4..extnops(x))
    end_proc,

    "new" = proc(f,eq)
    begin
       if args(0)<2 then error("at least two arguments expected")
       elif type(eq)<>"_equal" then error("invalid arguments")
       else new(this,f,op(eq),TRUE) # use select to put a condition #
       end_if
    end_proc,

    "nops" = proc(x) local s;
    begin
       s:=SET(x); op(s,2)-op(s,1)+1
    end_proc,

    "domnops" = proc(t) begin
       if contains({DOM_INT,DOM_FLOAT},t) then infinity
       elif type(t)=DOM_SET then nops(t)
       else FAIL
       end_if
    end_proc,

    # the method length must be defined otherwise the standard length fails #
    "length" = fun(length([extop(args(1))])),

    "op" = proc(x,i) local p,j,f,s,k,e,n; begin
       if args(0)<>2 then error("two arguments expected") end_if;
       p:=this::index(SET(x)); f:=CONDITION(x); n:=VARIABLE(x); e:=EXPRESSION(x);
       if p=FAIL then error("no method op for the underlying set")
       elif testtype(i,DOM_INT) then
          if f=TRUE then s:=p(i) # no condition #
          else
             j:=0; while TRUE do
                j:=j+1;
                # if bool(f(eval(subs(e,n=p(j)))))=TRUE then #
                if bool(f(subs(e,n=p(j),Simplified)))=TRUE then
                   i:=i-1; if i=0 then s:=p(j); break end_if 
                end_if;
             end_while
          end_if;
          subs(e,n=s)
       elif testtype(i,DOM_INT..DOM_INT) then
          if f=TRUE then s:=p(j)$j=i
          elif (k:=op(i,2)-op(i,1))<=0 then s:=null()
          else
             i:=op(i,1);
             j:=0; while TRUE do
                j:=j+1;
                # if bool(f(eval(subs(e,n=p(j)))))=TRUE then #
                if bool(f(subs(e,n=p(j),Simplified)))=TRUE then
                   i:=i-1; if i=0 then s:=p(j); break end_if
                end_if;
             end_while;
             while k>0 do j:=j+1;
		# if bool(f(eval(subs(e,n=p(j)))))=TRUE then #
		if bool(f(subs(e,n=p(j),Simplified)))=TRUE then
                k:=k-1; s:=s,p(j) end_if 
             end_while;
          end_if;
          s:=[s];
          subs(e,n=s[i])$hold(i)=1..nops(s)
       else error("invalid arguments")
       end_if
    end_proc,
    "_index" = this::op,

    # indexing function of the underlying countable set,
      index(i) should return the i-th element (i>=1) #
    "index" = proc(t) local X; begin
       case type(t)
       of "_range" do
          case type(op(t,1)),type(op(t,2))
          of DOM_INT,DOM_INT do
          of DOM_INT,stdlib::Infinity do return(subs(fun(args(1)-1+X),X=op(t,1)))
          of stdlib::Infinity,DOM_INT do return(subs(fun(-args(1)+1+X),X=op(t,2)))
          of stdlib::Infinity,stdlib::Infinity do return(fun((-1)^args(1)*(args(1) div 2)))
          end_case; break
       end_case; FAIL
    end_proc,

    "ORDER" = 3,
    "print" = proc(x) local n; begin
       if testtype( (n:=this::nops(x)),Type::RealNum ) then
           if n<=this::ORDER then return( this::op(x,1..n) ) end_if
       end_if;
       return( this::op(x,1..this::ORDER),DOTS )
    end_proc,

    # converts to an expression sequence if finite, unless remains unchanged #
    "expand" = proc(x) begin
       this::nops(x);
       if testtype(%,Type::NonNegInt) then this::op(x,1..this::nops(x))
       elif testtype(%,Type::NegInt) then null()
       else x end_if
    end_proc,

    "Re" = proc(x) begin
       new(this,Re(rectform(extop(x,1),{extop(x,2)})),extop(x,2..extnops(x)))
    end_proc,

    "Im" = proc(x) begin
       new(this,Im(rectform(extop(x,1),{extop(x,2)})),extop(x,2..extnops(x)))
    end_proc,

    "select" = proc(x,f) local e,n,d,r,l; begin
       if f=_leequal or f=(not _less) then
          # assume must be real #
          l:={select(x,Type::RealNum)};

          if nops(l)<>1 or type(op(l,1))<>this then
             return(op(select(l,args(2..args(0)))))
          else x:=op(l,1)
          end_if;
       end_if;
       e:=EXPRESSION(x); n:=VARIABLE(x); r:=SET(x);
       if f=_leequal then
          if args(3)=infinity then return(x)
          elif (l:=Type::Linear(e,[n]))<>FALSE and is(args(3),Type::RealNum) = TRUE then
             if sign(l[1])=1 then
                if is(l[2],Type::RealNum)=TRUE then
		   # a*n+b <= x <==> n <= (x-b)/a #
                   return(extsubsop(x,3=subsop(r,2=min(op(r,2),
			floor((args(3)-l[2])/l[1]))))
                   )
                elif testtype(l[2],Type::Constant) and Im(float(l[2]))<>0.0 then 
                   return(null())
                end_if
             elif sign(l[1])=-1 then
                if is(l[2],Type::RealNum)=TRUE then
		   # -a*n+b <= x <==> n >= (x-b)/(-a) #
                   return(extsubsop(x,3=subsop(r,1=max(op(r,1),
			ceil((args(3)-l[2])/l[1]))))
                   )
                elif testtype(l[2],Type::Constant) and Im(float(l[2]))<>0.0 then 
                   return(null())
                end_if
             end_if
          end_if; 
          return(hold(select)(args()))
       elif f=(not _less) then
          if args(3)=-infinity then return(x)
          elif (l:=Type::Linear(e,[n]))<>FALSE then
             if sign(l[1])=1 and (is(l[2],Type::RealNum)=TRUE) then
		# a*n+b >= x <==> n >= (x-b)/a #
                return(extsubsop(x,3=subsop(r,1=max(op(r,1),ceil((args(3)-l[2])/l[1])))))
             elif sign(l[1])=-1 and (is(l[2],Type::RealNum)=TRUE) then
		# -a*n+b >= x <==> n <= (x-b)/(-a) #
                return(extsubsop(x,3=subsop(r,2=min(op(r,2),floor((args(3)-l[2])/l[1])))))
             end_if
          end_if; return(hold(select)(args()))
       elif f=Type::RealNum then
          # first check if real #
          l:=op(rectform(e,{n}),2);
          if has(l,n) then
             l:=solve(l,n=DOM_INT);
             if l={} then return(null())
             # else return(op(map(l,eval(subsop(hold(func(X,Y)),1=e,2=n))))) #
             else return(op(map(l,subsop(hold(func(X,Y)),1=e,2=n,Simplified))))
             end_if
          elif l<>0 then return(null()) # empty set #
          else return(x)
          end_if;
       end_if;
       if args(0)<>2 then f:=fp::fixargs(args(2),1,args(3..args(0))) end_if;
       extsubsop(x,CONDIND=(CONDITION(x) and f))
    end_proc,

    "subs" = fun(new(this,subs(extop(args(1)),args(2..args(0))))),

    "_mult" = proc() local a,b,c,x; begin
       a:=[]; b:=1; c:=1;
       for x in [args()] do
          if type(x)=this then a:=append(a,x)
          elif testtype(x,Type::Constant) then c:=c*x
          else b:=b*x
          end_if
       end_for;
       if nops(a)<>1 then return(FAIL) else a:=op(a) end_if;
       a:=extsubsop(a,EXPIND=c*EXPRESSION(a));
       if b=1 then a else hold(_mult)(b,a) end_if
    end_proc,

    "_plus" = proc() local a,b,c,x; begin
       a:=[]; b:=0; c:=0;
       for x in [args()] do
          if type(x)=this then a:=append(a,x)
          elif testtype(x,Type::Constant) then c:=c+x
          else b:=b+x
          end_if
       end_for;
       if nops(a)<>1 then return(FAIL) else a:=op(a) end_if;
       a:=extsubsop(a,EXPIND=c+EXPRESSION(a));
       if b=0 then a else hold(_plus)(b,a) end_if
    end_proc,

    "shift"= proc(S,k)

    begin
    if k=0 then return(S) end_if;
    new(this,expand(subs(EXPRESSION(S),VARIABLE(S)=VARIABLE(S)+k)), VARIABLE(S),
    map(SET(S),_subtract,k), CONDITION(S))
    end_proc,        

    # normal(S) - try to write S as {f(k), k=range} with minimal f(0) #
    # at the moment: proceed to the next local minimum #

    "normal" = proc(S)
    local minimum,mink,minkidx,increment;

    begin
    if op(SET(S),1)<0 then
	if op(SET(S),2)>=0 then
           mink:=0;
           if float(abs(subs(EXPRESSION(S),VARIABLE(S)=0)))>
              float(abs(subs(EXPRESSION(S),VARIABLE(S)=-1))) then
		increment:=-1
	   else
           	increment:=1
           end_if
        else
	   mink:=op(SET(S),2);
           increment:=-1
	end_if 
    else
        mink:=op(SET(S),1);
	increment:=1;
    end_if;
    minimum:=float(abs(subs(EXPRESSION(S),VARIABLE(S)=mink)));
    #
    breakcond:=(if increment=1 then 
			hold(mink)=op(SET(S),2)
		else
			hold(mink)=op(SET(S),1)
		end_if);
    while not eval(breakcond) and
    #
    if increment=1 then minkidx:=2 else minkidx:=1 end_if;
    while not hold(mink)=op(SET(S),minkidx) and
       (value:=float(abs(subs(EXPRESSION(S), VARIABLE(S)=mink+increment)))) < minimum do
        	minimum:=value;
                mink:=mink+increment 
    end_while;
    this::shift(S,mink)
    end_proc,
       
    "contains" = proc(x,v) begin
       contains({op(x,1..x::ORDER)},v) or UNKNOWN
    end_proc
)():

unalias(EXPRESSION, EXPIND, VARIABLE, SET, CONDITION, CONDIND, DOTS):
