# $Date: 1997/03/05 12:08:18 $ $Author: olli $ $Revision: 1.48 $ #
#++
Dom::DistributedPolynomial -- domain of distributed polynomials

Dom::DistributedPolynomial(Ind,R)

Ind - list of indeterminates
R   - coefficient ring

Distributed polynomials may have 2 different representations:

- If the coefficient ring R has normal form (axiom normalRep) and is not
  constructed by Dom::IntegerMod, then the polynomials are represented 
  as expressions of type DOM_POLY.
  
- Otherwise they are represented explicitly having a DOM_POLY as operand.

If the coefficient ring R has no normal form then the polynomials may
also have no normal form, i.e. coefficients may be 0 without beeing
'physically' equal to R::zero.

Methods:-
poly(p,X,R) - converts a polynomial p from this domain into a DOM_POLY
	with variables X and coeff. ring R
++#

Dom::DistributedPolynomial:= DomainConstructor(
    Dom::DistributedPolynomial,
    [ Ind, R ],
    [ NumInd, SysRep ],
    ( if args(0) <> 2 then error("wrong no of args") end_if;
      if R::hasProp(Cat::CommutativeRing) <> TRUE then
	  error("no commutative ring")
      end_if;
      if traperror(poly(1,Ind)) <> 0 then
	  error("illegal indeterminates")
      end_if;
      if poly(1,Ind) = FAIL then
	  error("illegal indeterminates")
      end_if;
      NumInd:= nops(Ind);
      SysRep:= bool(R::constructor <> Dom::IntegerMod and 
      		    R::hasProp(Ax::normalRep))
    ),
    ( if SysRep then 
          Dom::ArithmeticalExpression 
      else 
          Dom::BaseDomain
      end_if),
    [ ( if NumInd = 1 then
	    Cat::UnivariatePolynomialCat(R)
	else
	    Cat::PolynomialCat(R)
	end_if ) ],
    [ ( if SysRep then Ax::systemRep end_if),
      ( if R::hasProp(Ax::canonicalRep) then
	    Ax::canonicalRep
	elif R::hasProp(Ax::normalRep) then
	    Ax::normalRep
	end_if )
    ],

    "convert" = (if R = Dom::Integer then
	proc(x) begin
	    x:= poly(x, hold(Ind), hold(Expr));
	    if x <> FAIL then
		if map({coeff(x), 1}, domtype) <> {DOM_INT} then
		    x:= FAIL
		end_if
	    end_if;
	    x
	end_proc
    elif R = Dom::Rational then
	proc(x) begin
	    x:= poly(x, hold(Ind), hold(Expr));
	    if x <> FAIL then
		if map({coeff(x), 1, 1/2}, domtype) <> {DOM_INT, DOM_RAT} then
		    x:= FAIL
		end_if
	    end_if;
	    x
	end_proc
    elif R::hasProp(Ax::systemRep) then
        if R::hasProp(Ax::normalRep) then
	    proc(x) begin
		x:= poly(x, hold(Ind), hold(Expr));
		if x <> FAIL then
		    x:= mapcoeffs(x, R::convert);
		end_if;
		x
	    end_proc
	else
	    proc(x) begin
		x:= poly(x, hold(Ind), hold(Expr));
		if x <> FAIL then
		    x:= mapcoeffs(x, R::convert);
		end_if;
		if x = FAIL then x else new(this, x) end_if
	    end_proc
	end_if
    elif R::constructor = Dom::IntegerMod then
	proc(x) begin
	    x:= poly(mapcoeffs(mapcoeffs(poly(x, hold(Ind), hold(Expr)),
					 R::convert),
			       extop, 1),
		     hold(IntMod)((R::constructor_args)[1]));
	    if x = FAIL then x else new(this, x) end_if
	end_proc
    elif not R::hasProp(Ax::normalRep) then
	proc(x) begin
	    x:= poly(x, hold(Ind), R);
	    if x = FAIL then x else new(this, x) end_if
	end_proc
    else
	proc(x) begin poly(x, hold(Ind), R) end_proc
    end_if),

    "expr" = (if not SysRep then
	fun(expr(extop(args(1),1)))
    else
	expr
    end_if),

    "poly" = (if not SysRep then
	fun(poly(extop(args(1),1), args(2..args(0))))
    else
	poly
    end_if),

    "TeXindet" = proc(x) begin
	"{".expr2text(x)."}"
    end_proc,

    "TeX" = proc(x) local i, j, d, s; begin
	if iszero(x) then
	    R::TeX(R::zero)
	else
	    d:= degreevec(lterm(x));
	    if _plus(op(d)) = 0 or not R::equal(lcoeff(x), R::one) then
		s:= "{".R::TeX(lcoeff(x))."}";
	    else
		s:= "";
	    end_if;
	    for i from 1 to NumInd do
		if d[i] = 1 then
		    s:= s.this::TeXindet(hold(Ind)[i])
		elif d[i] > 0 then
		    s:= s.this::TeXindet(hold(Ind)[i])."^{".
			expr2text(d[i])."}"
		end_if;
	    end_for;
	    for j from 2 to nterms(x) do
		d:= degreevec(nthterm(x,j));
		if _plus(op(d)) = 0 or not R::equal(nthcoeff(x,j), R::one) then
		    s:= s." + {".R::TeX(nthcoeff(x,j))."}";
		else
		    s:= s." + ";
		end_if;
		for i from 1 to NumInd do
		    if d[i] = 1 then
			s:= s.this::TeXindet(hold(Ind)[i])
		    elif d[i] > 0 then
			s:= s.this::TeXindet(hold(Ind)[i])."^{".
				expr2text(d[i])."}"
		    end_if;
		end_for;
	    end_for;
	    s
	end_if
    end_proc,

    "zero" = this::convert(R::zero),

    "one" = this::convert(R::one),

    "iszero" = (if R::constructor = Dom::IntegerMod then
	fun(iszero(extop(args(1),1)))
    elif not R::hasProp(Ax::normalRep) then
        proc(x) local c; begin
            if iszero(extop(x,1)) then return(TRUE) end_if;
            for c in [ coeff(extop(x,1)) ] do
                if not R::iszero(c) then return(FALSE) end_if
            end_for;
            TRUE
        end_proc
    else
        iszero
    end_if),

    "_plus" = (if not SysRep then
	fun((if map({args()}, domtype) <> {this} then FAIL
	     else new(this, _plus(op(map([args()], extop, 1))))
	     end_if))
    end_if),

    "_subtract" = (if not SysRep then
	fun(new(this, extop(args(1),1) - extop(args(2),1)))
    end_if),

    "_negate" = (if not SysRep then
	fun(new(this, - extop(args(1),1)))
    end_if),

    "_mult" = (if not SysRep then
        fun((if args(0) = 2 then
		if domtype(args(2)) <> this then
		    case domtype(args(2))
		    of DOM_INT do this::intmult(args()); break;
		    of R do this::multcoeffs(args()); break;
		    otherwise (domtype(args(2)))::_mult(args());
		    end_case
		elif domtype(args(1)) <> this then
		    case domtype(args(1))
		    of DOM_INT do this::intmult(args(2), args(1)); break;
		    of R do this::multcoeffs(args(2), args(1)); break;
		    otherwise FAIL;
		    end_case
		else
		    new(this, extop(args(1),1) * extop(args(2),1))
		end_if
	    elif args(0) = 1 then
		args(1)
	    else
		_mult(args(1..(args(0) div 2)));
		_mult(args(((args(0) div 2) + 1)..args(0)));
		if (domtype(%2))::_mult <> FAIL then (domtype(%2))::_mult(%2, %1)
		elif (domtype(%1))::_mult <> FAIL then (domtype(%1))::_mult(%2, %1)
		else _mult(%2, %1)
		end_if
	    end_if))
    end_if),

    "_power" = (if not SysRep then
	fun((
	    if testargs() then
		if not testtype(args(2), Type::NonNegInt) then
		      error("illegal power")
		end_if
	    end_if;
	    new(this, extop(args(1),1) ^ args(2))
	))
    end_if),

    "_invert" = proc(x) begin
	x:= this::normalPoly(x);
	if degree(x) > 0 then return(FAIL) end_if;
	x:= R::_invert(lcoeff(x));
	if x = FAIL then FAIL else this::convert(x) end_if;
    end_proc,

    "intmult" = (if not SysRep then
        if R::constructor = Dom::IntegerMod then
            fun( new(this, multcoeffs(extop(args(1),1), args(2))) )
        else
	    fun( new(this, mapcoeffs(extop(args(1),1), R::intmult, args(2))) )
	end_if
    else
	fun( mapcoeffs(args(1), R::intmult, args(2)) )
    end_if),

    "_divide" = (if not SysRep then
	fun((
	    divide(extop(args(1),1), extop(args(2),1), hold(Exact));
	    if % = FAIL then FAIL else new(this, %) end_if
	))
    else
	fun( divide(args(1), args(2), hold(Exact)) )
    end_if),

    "euclideanDegree" = (if NumInd = 1 then degree end_if),

    "divide" = (if NumInd = 1 then
	if not SysRep then
	    proc(x,y,o) begin
		if args(0) = 2 then
		    x:= divide(extop(x,1), extop(y,1));
		else
		    x:= divide(extop(x,1), extop(y,1), o);
		end_if;
		case domtype(x)
		of DOM_POLY do new(this, x); break;
		of DOM_FAIL do FAIL; break;
		otherwise (new(this, x[1]), new(this, x[2]));
		end_case
	    end_proc
	else
	    divide
	end_if
    end_if),
 
    "quo" = (if NumInd = 1 then
	fun( divide(args(1), args(2), hold(Quo)) )
    end_if),

    "rem" = (if NumInd = 1 then
	fun( divide(args(1), args(2), hold(Rem)) )
    end_if),

    "gcd" = (if R = Dom::Integer or R = Dom::Rational then
	gcd
    elif R::constructor = Dom::IntegerMod and R::hasProp(Cat::Field) then
	fun( new(this, gcd(op(map([args()], extop, 1)))) )
    elif R::hasProp(Cat::GcdDomain) then
        if SysRep then
	    gcd
	else
	    fun( new(this, gcd(op(map([args()], extop, 1)))) )
	end_if
    end_if),

    "lcm" = (if R = Dom::Integer or R = Dom::Rational then
	lcm
    elif R::constructor = Dom::IntegerMod and R::hasProp(Cat::Field) then
	fun( new(this, lcm(op(map([args()], extop, 1)))) )
    elif R::hasProp(Cat::GcdDomain) then
        if SysRep then
	    lcm
	else
	    fun( new(this, lcm(op(map([args()], extop, 1)))) )
	end_if
    end_if),

    "factor" = (if R = Dom::Integer or R = Dom::Rational then
	factor
    elif R::constructor = Dom::IntegerMod and R::hasProp(Cat::Field) and NumInd = 1 then
	proc(x) local i; begin
	    x:= factor(extop(x,1));
	    [ R::convert(x[1]),
	      (new(this, x[2*i]), x[2*i+1]) $ hold(i)=1..(nops(x) div 2) ]
	end_proc
    end_if),

    "sqrfree" = (if R = Dom::Integer or R = Dom::Rational then
	sqrfree
    elif R::constructor = Dom::IntegerMod and R::hasProp(Cat::Field) and NumInd = 1 then
	proc(x) local i; begin
	    x:= sqrfree(extop(x,1));
	    [ R::convert(x[1]),
	      (new(this, x[2*i]), x[2*i+1]) $ hold(i)=1..(nops(x) div 2) ]
	end_proc
    elif R::hasProp(Cat::FactorialDomain) then
        if R::characteristic = 0 then
            if SysRep then
                sqrfree
            else
                fun( new(this, sqrfree(extop(args(1),1))) )
            end_if
        end_if
    end_if),

    "pdivide" = (if NumInd = 1 then
	if not SysRep then
	    proc(x,y,o) begin
		if args(0) = 2 then
		    x:= pdivide(extop(x,1), extop(y,1));
		    (R::convert(x[1]), new(this, x[2]), new(this, x[3]))
		else
		    new(this, pdivide(extop(x,1), extop(y,1), o))
		end_if;
	    end_proc
	else
	    pdivide
	end_if
    end_if),

    "pquo" = (if NumInd = 1 then
	fun( pdivide(args(1), args(2), hold(Quo)) )
    end_if),

    "prem" = (if NumInd = 1 then
	fun( pdivide(args(1), args(2), hold(Rem)) )
    end_if),

    "indets" = (if not SysRep then
	fun( op(extop(args(1),1), 2) )
    else
	fun( op(args(1), 2) )
    end_if),

    "mainvar" = (if not SysRep then
	fun( op(extop(args(1),1), 2)[1] )
    else
	fun( op(args(1), 2)[1] )
    end_if),

    "degree" = (if not SysRep then
	fun( degree(extop(args(1),1), args(2..args(0))) )
    else
	degree
    end_if),

    "degreevec" = (if not SysRep then
        fun( degreevec(extop(args(1),1), args(2..args(0))) )
    else
        degreevec
    end_if),

    "lcoeff" = (if R::constructor = Dom::IntegerMod then
        fun( R::convert(lcoeff(extop(args(1),1), args(2..args(0)))) )
    elif not SysRep then
        fun( lcoeff(extop(args(1),1), args(2..args(0))) )
    else
        lcoeff
    end_if),

    "lmonomial" = (if not SysRep then
        fun( new(this, lmonomial(extop(args(1),1), args(2..args(0)))) )
    else
        lmonomial
    end_if),

    "lterm" = (if not SysRep then
        fun( new(this, lterm(extop(args(1),1), args(2..args(0)))) )
    else
        lterm
    end_if),

    "nterms" = (if not SysRep then
        fun( nterms(extop(args(1),1)) )
    else
        nterms
    end_if),

    "nthterm" = (if not SysRep then
        fun( new(this, nthterm(extop(args(1),1), args(2))) )
    else
        nthterm
    end_if),

    "nthcoeff" = (if R::constructor = Dom::IntegerMod then
        fun( R::convert( nthcoeff(extop(args(1),1), args(2))) )
    elif not SysRep then
        fun( nthcoeff(extop(args(1),1), args(2)) )
    else
        nthcoeff
    end_if),

    "nthmonomial" = (if not SysRep then
        fun( new(this, nthmonomial(extop(args(1),1), args(2))) )
    else
        nthmonomial
    end_if),

    "tcoeff" = (if R::constructor = Dom::IntegerMod then
        fun( R::convert(tcoeff(extop(args(1),1))) )
    elif not SysRep then 
        fun( tcoeff(extop(args(1),1)) )
    else
        tcoeff
    end_if),

    "coeff" = (if R::constructor = Dom::IntegerMod then
        proc(p,x,n) local c; begin
            p:= extop(p,1);
            case args(0)
            of 1 do return(map(coeff(p), R::convert));
            of 2 do c:= coeff(p,x); break;
            of 3 do c:= coeff(p,x,n); break;
            otherwise error("wrong no of args")
            end_case;
            if nops(op(p,2)) = 1 then
                R::convert(c)
            else
		Dom::DistributedPolynomial(op(c,2), R)(c)
            end_if
        end_proc
    elif not SysRep then
        proc(p,x,n) local c; begin
            p:= extop(p,1);
            case args(0)
            of 1 do return(coeff(p));
            of 2 do c:= coeff(p,x); break;
            of 3 do c:= coeff(p,x,n); break;
            otherwise error("wrong no of args")
            end_case;
            if nops(op(p,2)) = 1 then
                c
            else
		Dom::DistributedPolynomial(op(c,2), R)(c)
            end_if
        end_proc
    else
	coeff
    end_if),

    "multcoeffs" = ( if not SysRep then
        fun( new(this, multcoeffs(extop(args(1),1), extop(args(2),1))) )
    else
	multcoeffs
    end_if),

    "evalp" = (if R::constructor = Dom::IntegerMod then
        proc(p) local i; begin
            p:= evalp(extop(p,1), (op(args(i),1) = extop(op(args(i),2),1))
                                        $ hold(i)=2..args(0));
            if domtype(p) = DOM_POLY then
                Dom::DistributedPolynomial(op(p,2), R)(p)
            else 
                R::convert(p) 
            end_if
        end_proc
    elif not SysRep then
        proc(p) begin
            p:= evalp(extop(p,1), args(2..args(0)));
            if domtype(p) = DOM_POLY and domtype(p) <> R then
                Dom::DistributedPolynomial(op(p,2), R)(p)
            else 
                p
            end_if
        end_proc
    else
	evalp
    end_if),

    "mapcoeffs" = (if R::constructor = Dom::IntegerMod then
        fun( new(this, poly(mapcoeffs(poly(extop(args(1),1), R),
                                          args(2..args(0))),
                            hold(IntMod)((R::constructor_args)[1]))) )
    elif not SysRep then
        fun( new(this, mapcoeffs(extop(args(1),1), args(2..args(0)))) )
    else
	mapcoeffs
    end_if),

    "content" = (if R = Dom::Integer then
	icontent
    end_if),

    "primpart" = (if R = Dom::Integer then
	fun( this::unitNormal(multcoeffs(args(1), 1/icontent(args(1)))) )
    end_if),

    "unitNormal" = (if R = Dom::Integer then
	proc(x) begin
	    if iszero(x) then x 
	    elif sign(lcoeff(x)) = 1 then x 
	    else multcoeffs(x, -1) end_if
	end_proc
    elif R = Dom::Rational then
	proc(x) begin
	    if iszero(x) then x else multcoeffs(x, 1/lcoeff(x)) end_if
	end_proc
    elif R::constructor = Dom::IntegerMod and R::hasProp(Cat::Field) then
	proc(x) begin
	    x:= extop(x,1);
	    if iszero(x) then 
		this::zero 
	    else
		new(this, multcoeffs(x, 1/lcoeff(x)))
	    end_if
	end_proc
    end_if),

    "unitNormalRep" = (if R = Dom::Integer then
	proc(x) local o; begin
	    o:= poly(1, hold(Ind));
	    if iszero(x) then
	        [ x, o, o ]
	    elif sign(lcoeff(x)) = 1 then
		[ x, o, o ]
	    else
		[ multcoeffs(x, -1),  -o, -o ]
	    end_if
	end_proc
    elif R = Dom::Rational then
	proc(x) local l; begin
	    if iszero(x) then
	        [ x, this::one, this::one ]
	    else
		l:= lcoeff(x);
		[ multcoeffs(x, 1/l), poly(1/l,op(x,2..3)), poly(l,op(x,2..3)) ]
	    end_if
	end_proc
    elif R::constructor = Dom::IntegerMod and R::hasProp(Cat::Field) then
	proc(x) local l; begin
	    x:= extop(x,1);
	    if iszero(x) then
	        [ this::zero, this::one, this::one ]
	    else
		l:= lcoeff(x);
		[ new(this, multcoeffs(x, 1/l)),
		  new(this, poly(1/l,op(x,2..3))),
		  new(this, poly(l,op(x,2..3))) ]
	    end_if
	end_proc
    end_if),

    "D" = (if not SysRep then
	fun(new(this, (if args(0) = 1 then D(extop(args(1),1))
		       else D(args(1), extop(args(2),1)) end_if)))
    else
	D
    end_if),

    "Dpoly" = (if not SysRep then
	fun(new(this, (if args(0) = 1 then Dpoly(extop(args(1),1))
		       else Dpoly(args(1), extop(args(2),1)) end_if)))
    else
	Dpoly
    end_if), 
   
    "diff" = (if not SysRep then
        fun(new(this, diff(extop(args(1),1), args(2..args(0)))))
    else
        diff
    end_if),

    "random" = (if not SysRep then
        if R::constructor = Dom::IntegerMod then
            fun(new(this, randpoly(hold(Ind), hold(IntMod)((R::constructor_args)[1]))))
        elif R::hasProp(Ax::systemRep) then
            fun(new(this, randpoly(hold(Ind), hold(Expr), hold(Coeffs)=R::random)))
        else
            fun(new(this, randpoly(hold(Ind), R)))
        end_if
    elif R = Dom::Integer then
        fun(randpoly(hold(Ind), hold(Expr)))
    elif R::hasProp(Ax::systemRep) then
        fun(randpoly(hold(Ind), hold(Expr), hold(Coeffs)=R::random))
    else
        fun(randpoly(hold(Ind), R))
    end_if),
    
    "pivotSize" = this::degree,

    "solve" = (if NumInd=1 and R::constructor = Dom::IntegerMod then
	fun((map([faclib::roots(poly(expr(args(1)),hold(Ind), hold(IntMod)((R::constructor_args)[1])))],R::convert);
             if contains({args(2..args(0))},hold(Multiple)) then Dom::Multiset(op(%))
             else {op(%)}
             end_if))
    end_if),
    
    "normalPoly" = (if not SysRep then
        if R::constructor = Dom::IntegerMod then
    	    fun(extop(args(1),1))
        else
            fun(mapcoeffs(extop(args(1),1),
        	       fun((if R::iszero(args(1)) then R::zero else args(1) end_if))))
        end_if
    else
        id
    end_if)

):

