#++
Dom::IndependentDifferentialExpression(DV) -- general expression in the
	independent variables of DV

This domain is a simple specialization of Dom::DifferentialExpression(DV).
The only difference is in the convert method which does not accept any
differential variable which is not an independent variable.


Author: Werner M. Seiler, Uni Karlsruhe
Created: September 30th, 1997
Last update: October 8th, 1997
++#

Dom::IndependentDifferentialExpression := DomainConstructor(

	Dom::IndependentDifferentialExpression,

	[ DV ],

	[ DVcoerce, DVconvert, DVtype ],

	(if args(0)<>1 then error("wrong no of args") end_if;
	 if domtype(DV)<>DOM_DOMAIN or DV::hasProp=FAIL or
	    not(DV::hasProp(Cat::DifferentialVariableCat)) then
	     error("argument must be Cat::DifferentialVariableCat");
	 end_if;

	 DVcoerce := DV::coerce;
	 DVconvert := DV::convert;
	 DVtype := DV::vartype;
	),

	Dom::DifferentialExpression(DV),

	[ Cat::RestrictedDifferentialFunctionCat(DV,"Indep"),
	  Cat::PartialDifferentialRing ],

	[],

        "convert" = proc(x)
	local r,xop,xargs,nargs,nvars;
        begin
	    case domtype(x)
            of this do return(x);
            of DV do
		if DVtype(x)="Indep" then return(new(this,x,[x]))
		else return(FAIL) end_if;
            of DOM_EXPR do
		r := DVconvert(x);
		if r<>FAIL then
		    if DVtype(r)="Indep" then return(new(this,r,[r]))
		    else return(FAIL) end_if;
		end_if;
		xop := op(x,0);
		xargs := map([op(x)],this::convert);
		if contains(xargs,FAIL)<>0 then return(FAIL) end_if;
		nargs := map(xargs,extop,1);
		nvars := this::collectVars(op(map(xargs,this::diffVars)));
		return(new(this,xop(op(nargs)),nvars));
	    of DOM_IDENT do
		r := DVconvert(x);
		if r=FAIL then return(new(this,x,[]))
		elif DVtype(r)="Indep" then return(new(this,r,[r]))
		else return(FAIL) end_if;
            of DOM_POLY do return(this::convert(expr(x)));
	    of DOM_INT do
	    of DOM_RAT do
	    of DOM_FLOAT do
	    of DOM_COMPLEX do return(new(this,x,[]));
	    otherwise
		if x::expr<>FAIL then
		    return(this::convert(x::expr(x)));
		end_if;
		x := DVcoerce(x);
		if x=FAIL or DVtype(x)<>"Indep" then return(FAIL)
		else return(new(this,x,[x])) end_if;
            end_case;
        end_proc

):







