# domains of intervals: Dom::Interval(a..b) represents the interval [a,b].
  It forms a field, but the representation of zero is not unique
  (any interval [a,b] such that a<=0<=b represents 0).

  The available methods are:
  _plus, sign, _mult, _negate, _invert, _power, iszero, convert,
  random, exp, _union, _intersect.
#

Dom::Interval := DomainConstructor(
# name #
    Dom::Interval,
# arguments #
    [ ],
# local variables #
    [ ],
# initialisation #
    (if args(0) <> 0 then error("wrong no of args") end_if),
# super-domains #
    Dom::BaseDomain,
# categories #
    [ Cat::Field ],
# axioms #
    [ ],
# methods #
    "characteristic" = 0,
    
    "expr" = fun(hold(_range)(extop(args(1)))),
  
    "convert" = fun((
       case domtype(args(1))
       of this do return(args(1))
       of DOM_INT do # use float because otherwise we get ln(2) #
       of DOM_RAT do return(new(this,float(args(1)),float(args(1))))
       of DOM_FLOAT do return(new(this,args(1),args(1)))
       of DOM_EXPR do
           map(args(1),float);
	   if testtype(%, hold(_range)(Type::RealNum, Type::RealNum)) then
               if op(%,1)<=op(%,2) then return(new(this,op(%)))
               else return(new(this,op(%,2),op(%,1)))
               end_if
	   end_if;
       end_case;
       FAIL
    )),

    "_plus" = proc(r) local i,s,t; begin
        i:=this::convert(r);
        if i=FAIL then t:=r; r:=this::zero else r:=i; t:=0 end_if;
        for i from 2 to args(0) do
           s:=this::convert(args(i));
           if s=FAIL then t:=t+args(i)
	   else r:=new(this,extop(r,1)+extop(s,1),extop(r,2)+extop(s,2))
           end_if
        end_for;
        if t=0 then r else hold(_plus)(t,r) end_if
    end_proc,

    "sign" = proc(r) begin 
	if extop(r,1)>0 then 1 elif extop(r,2)<0 then -1 else 0 end_if
    end_proc,

    "_mult" = proc(r) local a,b,c,d,s,i,t; begin
        a:=this::convert(r);
        if a=FAIL then t:=r; r:=this::one else r:=a; t:=1 end_if;
        for i from 2 to args(0) do
	   s:=this::convert(args(i));
           if s=FAIL then t:=t*args(i)
           else
	      a:=extop(r,1); b:=extop(r,2); c:=extop(s,1); d:=extop(s,2);
	      case this::sign(r),this::sign(s)
	      of 1,1 do r:=new(this,a*c,b*d); break
	      of 1,0 do r:=new(this,b*c,b*d); break
	      of 1,-1 do r:=new(this,b*c,a*d); break
	      of 0,1 do r:=new(this,a*d,b*d); break
	      of 0,0 do r:=new(this,min(a*d,b*c),max(a*c,b*d)); break
	      of 0,-1 do r:=new(this,b*c,a*c); break
	      of -1,1 do r:=new(this,a*d,b*c); break
	      of -1,0 do r:=new(this,a*d,a*c); break
	      of -1,-1 do r:=new(this,b*d,a*c)
	      end_case
           end_if
        end_for;
        if t=1 then r else hold(_mult)(t,r) end_if
    end_proc,

    "_negate" = fun(new(this,-extop(args(1),2),-extop(args(1),1))),

    "_subtract" = fun(this::_plus(args(1),this::_negate(this::convert(args(2))))),

    "_invert" = proc(r) begin
	if this::sign(r)=0 then error("division by zero")
	else new(this,1/extop(r,2),1/extop(r,1))
	end_if
    end_proc,

    "_power" = proc(r,n) local a,b; begin
        if type(n)=this then this::exp(this::_mult(n,this::ln(this::convert(r))))
	elif n<0 then 1/r^(-n)
	elif n=1 then r
	else
           a:=extop(r,1)^n; b:=extop(r,2)^n;
           case this::sign(r)
           of 1 do return(new(this,a,b))
           of 0 do
              if n mod 2 = 0 then this::convert(0..max(a,b))
              else new(this,a,b)
              end_if; break
           of -1 do if a<b then new(this,a,b) else new(this,b,a) end_if
           end_case
        end_if
    end_proc,

    "iszero" = fun(
	bool(iszero(extop(args(1),1)) and iszero(extop(args(1),2)))
    ),

    "random" = proc() local a,b; begin 
       a:=tan(float(PI) * (random()/(10.0^12) - 0.5)); # -infinity < a < infinity #
       b:=tan(float(PI/2) * (random()/(10.0^12))); # 0 <= b < infinity #
       new(this,a,a+b)
    end_proc,

    "one" = this::convert(1..1),

    "zero" = this::convert(0..0),
    
    "exp" = fun(new(this,exp(extop(args(1),1)),exp(extop(args(1),2)))),

    "_union" = proc() begin
       if args(0)=0 then error("wrong number of arguments") end_if;
       new(this,min(op(map([args()],extop,1))),max(op(map([args()],extop,2))))
    end_proc,

    "_intersect" = proc() begin
       if args(0)=0 then error("wrong number of arguments") end_if;
       new(this,max(op(map([args()],extop,1))),min(op(map([args()],extop,2))))
    end_proc,

    "map" = fun(new(this,map(extop(args(1)),args(2..args(0))))),

    "float" = fun(new(this,float(extop(args(1),1)),float(extop(args(1),2)))),

    "ln" = fun(new(this,ln(extop(args(1),1)),ln(extop(args(1),2)))),
    "abs" = fun((case sign(args(1))
                of 1 do return(args(1))
                of -1 do return(this::_negate(args(1)))
                of 0 do return(this::convert(0..max(-extop(args(1),1),extop(args(1),2))))
                end_case)),
    "acos" = proc(x) begin
       if extop(x,1)<-1 or 1<extop(x,2) then error("argument outside range") end_if;
       new(this,acos(extop(x,2)),acos(extop(x,1)))
    end_proc,
    "acosh" = proc(x) begin
       if extop(x,1)<1 then error("argument outside range") end_if;
       new(this,acosh(extop(x,1)),acosh(extop(x,2)))
    end_proc,
    "acot" = fun((if sign(args(1))=0 then new(this,-PI/2,PI/2)
                  else new(this,acot(extop(x,2)),acot(extop(x,1))) end_if)),
    "acoth" = proc(x) begin
       if extop(x,2)<-1 or 1<extop(x,1) then new(this,acoth(extop(x,2)),acoth(extop(x,1)))
       else error("singularity encountered") end_if
    end_proc,
    "asin" = proc(x) begin
       if extop(x,1)<-1 or 1<extop(x,2) then error("argument outside range") end_if;
       new(this,asin(extop(x,1)),asin(extop(x,2)))
    end_proc,
    "asinh" = fun(new(this,asinh(extop(args(1),1)),asinh(extop(args(1),2)))),
    "atan" = fun(new(this,atan(extop(args(1),1)),atan(extop(args(1),2)))),
    "atanh" = proc(x) begin
       if extop(x,1)<-1 or 1<extop(x,2) then error("argument outside range") end_if;
       new(this,atanh(extop(x,1)),atanh(extop(x,2)))
    end_proc,
    "cos" = proc(x) local lo,hi; begin
       hi:=2*PI*ceil(extop(x,1)/2/PI); # next maxima following the left bound #
       lo:=2*PI*ceil((extop(x,1)-PI)/2/PI)+PI; # next minima to the right #
       if float(hi-extop(x,2))<=0 then hi:=1
       elif cos(extop(x,1))>=cos(extop(x,2)) then hi:=cos(extop(x,1))
       else hi:=cos(extop(x,2))
       end_if;
       if float(lo-extop(x,2))<=0 then lo:=-1
       elif cos(extop(x,1))<=cos(extop(x,2)) then lo:=cos(extop(x,1))
       else lo:=cos(extop(x,2))
       end_if;
       this::convert(lo..hi)
    end_proc,
    "cosh" = fun((case sign(args(1))
                of 1 do return(map(args(1),cosh))
                of -1 do return(map(-args(1),cosh))
                of 0 do return(this::convert(1..cosh(max(-extop(args(1),1),extop(args(1),2)))))
                end_case)),
    "cot" = fun((if sign(args(1))=0 then error("singularity encountered")
                else new(this,cot(extop(args(1),2)),cot(extop(args(1),1))) end_if)),
    "coth" = fun((if sign(args(1))=0 then error("singularity encountered")
                else new(this,coth(extop(args(1),2)),coth(extop(args(1),1))) end_if)),
    "sin" = proc(x) local lo,hi; begin
       hi:=2*PI*ceil((extop(x,1)-PI/2)/2/PI)+PI/2; # next maxima following the left bound #
       lo:=2*PI*ceil((extop(x,1)+PI/2)/2/PI)-PI/2; # next minima to the right #
       if float(hi-extop(x,2))<=0 then hi:=1
       elif sin(extop(x,1))>=sin(extop(x,2)) then hi:=sin(extop(x,1))
       else hi:=sin(extop(x,2))
       end_if;
       if float(lo-extop(x,2))<=0 then lo:=-1
       elif sin(extop(x,1))<=sin(extop(x,2)) then lo:=sin(extop(x,1))
       else lo:=sin(extop(x,2))
       end_if;
       this::convert(lo..hi)
    end_proc,
    "sinh" = fun(map(args(1),sinh)),
    "tan" = proc(x) local a,b,k; begin
       a:=extop(x,1); b:=extop(x,2);
       # tan(x+k*PI)=tan(x) #
       k:=round(a/PI); a:=float(a-k*PI); b:=float(b-k*PI);
       if float(PI/2-b)<=0
       then error("singularity encountered")
       else new(this,tan(a),tan(b))
       end_if
    end_proc,
    "tanh" = fun(map(args(1),tanh))
)():
