#++
Dom::LinearDifferentialOperator(R,S) -- linear differential operators with
	coefficients from the partial differential ring R in variables
	given by the list S

This is a simple application of Dom::MonidOperatorAlgebra. Currently no extra
methods are implemented besides a "commutator" method. Of course one should add
a few like "adjoint" etc.

Dom::LinearDifferentialOperator(R,S) has in addition the structure of a partial
differential ring. The differentiations are applied to the coefficients of the
operator. However, we can use the category Cat::PartialDifferentialRing, as it
assumes that we have a commutative ring.


Author: Werner M. Seiler, Uni Karlsruhe
Date created: September 16th, 1997
Last update: October 8th, 1997
++#

Dom::LinearDifferentialOperator := DomainConstructor(

	Dom::LinearDifferentialOperator,

	[ R, S ],

	[ DP ],

	(if args(0)<1 or args(0)>2 then error("wrong no of args") end_if;
	if args(0)=1 then
	    S := R;
	    R := Dom::ExpressionField(normal);
        end_if;
	if domtype(R)<>DOM_DOMAIN or R::hasProp=FAIL or
	   not(R::hasProp(Cat::PartialDifferentialRing)) then
	    error("first argument must be Cat::PartialDifferentialRing");
	end_if;
	if domtype(S)<>DOM_LIST then
	    error("second argument must be list")
	end_if;
	DP := Dom::DerivativeProduct(S);
	),

	Dom::MonoidOperatorAlgebra(R,DP),

	[ Cat::Ring, Cat::LeftModule(R) ],

    	[ (if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if),
      	  (if R::hasProp(Ax::normalRep) then Ax::normalRep end_if) ],


	# ================== #
	# "operator methods" #
	# ================== #

	"commutator" = 
	    fun(this::_subtract(
		    this::mapcoeffs(args(2),
				    fun(this::func_call(args(2),args(1))),
				    args(1)),
		    this::mapcoeffs(args(1),
				    fun(this::func_call(args(2),args(1))),
				    args(2)))),


	# ============= #
	# miscellaneous #
	# ============= #

	"order" = this::degree,


	"ordervec" = this::degreevec,


	"diff" = fun(this::mapcoeffs(args(1),R::diff,args(2..args(0)))),

	"D" = fun(this::mapcoeffs(args(2),fun(R::D(args(2),args(1))),args(1)))

):

