
#++ ---------------- Lodo.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 22 June     1996
Last Update : 16 April    1999


Description:
This domain represents linear ordinary differential operators over an 
arbitrary commutative differential ring of charakteristic zero.

Call:

 Dom::LinearOrdinaryDifferentialOperator(Var, Dvar, < Ring >) 
 where
   Var        => indeterminate of type DOM_INDET; 
   Dvar       => differential indeterminate of type DOM_INDET; 
   opt: Ring  => arbitrary commutative differential ring 
                 of the Domains-package;
                 otherwise Dom::ExpressionField(normal) is choosen
                 automatically

Methods: 

 - used Parameter:
    ++ a,b   : this Domain
    ++ c     : Ring
    ++ f     : DOM_EXPR
    ++ m     : DOM_INT
    ++ v,l,la: DOM_LIST
    ++ e     : any of {DOM_EXPR, DOM_LIST}
    ++ a,b  = differential operators
    ++ c    = element of the coefficient ring 
    ++ f    = expression or any function
    ++ m    = exponent of the symmetric power 
    ++ v    = list of coefficient ring elements
    ++ e    = e is an ordinary linear differential equation (in f) or
    ++        its coefficients vector or a polynomial expression in Var
    ++ l    = list of non negative integers
    ++ la   = list of (irreducible) differential operators

 - new(e)
   new(e, f)
    ++ creates a Domain element.
 - adjoint(a)
    ++ returns the adjoint of a=c_0+c_1*Var+...+c_n*Var^n, which is 
    ++ a^*=c_0-Var*c_1+...+(-1)^n*Var^n*c_n.
 - exponentialZeros(a)
    ++ computes a fundamental set of all exponential zeros 
    ++ (i.e. {y | a(y)=0 and D(y)/y in R}) of a, 
    ++ if all coefficients are rational functions in Dvar. 
    ++ Note: This function works only properly on operators for which
    ++ ode::expsols works properly.
 - D(a)
    ++ calculates the derivative of a, which is var*a.
 - Dpoly(<l>, a)
    ++ calculates the derivative of a. When optional a list l is given, then
    ++ l=[1,...,1] with length(l)=n computes the n-th derivative of a. If 
    ++ l=[], then a is returned unchanged.
 - Factor(a)
    ++ top-level procedure for factor (see below), returns an expression
    ++ instead of a list.
 - factor(a)
    ++ computes a (complete) factorization [a_1,...,a_m] of a=a_1(...(a_m)), 
    ++ if all coefficient are rational functions and deg(a)<4. For operators 
    ++ of higher degree, currently only left and right factors of degree 1 
    ++ will be found.
 - func_call(a, f, < Unsimplified >)
    ++ applies f on a using the standard derivative with respect to Dvar
    ++ and simplifies the recieved expression.
    ++ When the option Unsimplified is given, the recieved expression will not 
    ++ be simplified.
 - leftDivide(a, b)
    ++ computes q, r, such that a=b*q+r with deg(r) < deg(b) and 
    ++ returns a table of quotient=q, remainder=r.
 - leftExtendedEuclid(a, b)
    ++ computes [[r0, s0, t0], [s1, t1]] such that
    ++ leftGcd(a,b) = r0 = a*s0+b*t0 and rightLcm(a,b) = -a*s1 = b*t1. 
 - leftExtendedGcd(a, b)
    ++ computes [r0, s0, t0] such that leftGcd(a,b) = r0 = a*s0+b*t0.
 - leftGcd(a,b)
    ++ computes the value g of highest degree such that a=g*a0 and b=g*b0
    ++ for some values a0 and b0.
 - leftLcm(a,b)
    ++ computes the value s of lowest degree such that s=a0*a=b0*b
    ++ for some values a0 and b0.
 - leftQuotient(a, b)
    ++ computes q such that a=b*q+r with deg(r)=0 or deg(r) < deg(b).
 - leftRemainder(a, b)
    ++ computes r such that a=b*q+r with deg(r)=0 or deg(r) < deg(b).
 - liouvillianZeros(a, <"Transform" | "Irreducible">)
 - liouvillianZeros(la)
    ++ see 'solve'. 
 - mkLODO(v)
   mkLODO(Ly, y(x))
    ++ returns the corresponding linear differential operator
    ++ v[1]+v[2]*Var+...+v[n]*Var^(n-1) or
    ++ a_n*Var^n+...+a_0, where Ly = a_n*diff(y(x), x$n)+...+a_0*y(x).
 - polynomialZeros(a) 
    ++ computes a fundamental set of all polynomial zeros of a,  
    ++ if all coefficients are rational functions in Dvar. 
 - rationalZeros(a) 
    ++ computes a fundamental set of all rational zeros of a,  
    ++ if all coefficients are rational functions in Dvar. 
 - rightDivide(a, b)
    ++ computes q, r, such that a=q*b+r with deg(r) < deg(b) and 
    ++ returns a table of quotient=q, remainder=r.
 - rightExtendedEuclid(a, b)
    ++ computes [[r0, s0, t0], [s1, t1]] such that
    ++ rightGcd(a,b) = r0 = s0*a+t0*b and leftLcm(a,b) = -s1*a = t1*b. 
 - rightExtendedGcd(a, b)
    ++ computes [r0, s0, t0] such that rightGcd(a,b) = r0 = s0*a+t0*b.
 - rightGcd(a,b)
    ++ computes the value g of highest degree such that a=a0*g and b=b0*g
    ++ for some values a0 and b0.
 - rightLcm(a,b)
    ++ computes the value s of lowest degree such that s=a*a0=b*b0
    ++ for some values a0 and b0.
 - rightQuotient(a, b)
    ++ computes q such that a=q*b+r with deg(r)=0 or deg(r) < deg(b).
 - rightRemainder(a, b)
    ++ computes r such that a=q*b+r with deg(r)=0 or deg(r) < deg(b).
 - rlfactor(a)
    ++ this is procedure is intended to be a subprocedure of factor. It 
    ++ computes at least one right and one left factor of a, if them exits.
 - solve(a, <"Transform" | "Irreducible">)
 - solve(la)
    ++ computes the liouvillian solutions of a first degree operator a and
    ++ of a second degree operator a with rational function coefficients
    ++ or liouvillian solutions of a (factor) list la of irreducible 
    ++ operators, whereby the last operator of the list corresponds to the  
    ++ right-hand factor (note, maybe not *all* liouvillian solutions of la  
    ++ will be found).
    ++ Higher degree operators are currently only partially treated. 
    ++ The two options only affects by second degree operators:
    ++ When option "Transform" is given, the unimodular transform is
    ++ executed unconditionally.
    ++ When option "Irreducible" is given, a is assumed to be irreducible.  
    ++ NOTE: currently this function only works properly for operators
    ++       for which the function ode::expsols works properly.
    ++ Otherwise: no warranty! 
 - symmetricPower(a, m)
    ++ computes the m-th symmetric power of a. This is the lowest degree
    ++ linear differential operator whose solution space consists 
    ++ exactly of all possible m-th powerproducts of solutions of a.

All Entries:

D, Dpoly, Factor, Rep, TeX, TeXCoeff, TeXTerm, TeXindet, _invert, _mult, _\
negate, _plus, _power, _subtract, adjoint, allAxioms, allCategories, allEn\
tries, allSuperDomains, characteristic, coeff, coeffRing, coerce, content,\
 convert, convert_to, degree, degreevec, equal, equiv, euclideanDegree, ev\
alLODO, exponentialZeros, expr, factor, func_call, getAxioms, getCategorie\
s, getSuperDomain, getTermOrder, hasProp, indets, info, intmult, isone, is\
zero, lcoeff, leftDivide, leftExactQuotient, leftExtendedEuclid, leftGcd, \
leftLcm, leftQuotient, leftRemainder, leftShift, liouvillianZeros, lmonomi\
al, lterm, mainvar, mapcoeffs, mkLODO, monic, mult, multcoeffs, negate, ne\
w, norm, nterms, nthcoeff, nthmonomial, nthterm, one, ore_mult, plus, poly\
, polynomialZeros, pquo, print, printElem, printMonomial, printTerm, pseud\
oFactor, random, rationalZeros, reductum, rightDivide, rightExactQuotient,\
 rightExtendedEuclid, rightGcd, rightLcm, rightQuotient, rightRemainder, r\
ightShift, ringmult, rlfactor, sign, solve, subs, subsex, symmetricPower, \
tcoeff, testtype, undefinedEntries, vectorize, whichEntry, zero

See: 
 - M. Bronstein & M. Petkovsek (1993). On Ore Rings, Linear Operators 
   and Factorisation. Bericht Nr. 200  ETH Zuerich 
 - M. Bronstein & M. Petkovsek (1996). An introduction to pseudo-linear
   algebra. Theoretical Computer Science 157 No. 1, 3-33. 
 - Fakler, W. (1997). Algorithms for Solving Linear Ordinary Differential
   Equations. mathPAD 7 No.1, 50-59.

Example:
>> lodo := Dom::LinearOrdinaryDifferentialOperator(Df, x);
>> L1 := lodo(Df^2+(x+1)*Df+2*x);
>> L2 := lodo(Df+1);
>> L1::leftDivide(L1, L2);
>> Factor(L1);

++#

# Dependencies:

  OrePolyCat.mu, PolyExplicit.mu, UniPoly.mu, OrePoly.mu,
  sympower.mu, LiouvSol.mu, ExpSol.mu, PolySol.mu, tools.mu

#

sysassign(Dom::LinearOrdinaryDifferentialOperator, DomainConstructor(

  # name: #            Dom::LinearOrdinaryDifferentialOperator,
  # parameter: #       [ var, dvar, R ],
  # static value: #    [ UP, coerce, hasField, INT, EF ],
  # initialization: #  (if iszero(args(0)) or args(0) > 3 then 
                          error("wrong no of args")
                        end_if;
                        if domtype(var) <> DOM_IDENT or
                           domtype(dvar) <> DOM_IDENT
                          then error("not a variable")
                        end_if;
                        if args(0) = 3 then
                          if R::hasProp(Dom::BaseDomain) = FAIL 
                            then error("illegal coefficient ring") end_if;
                          if not(R::hasProp(Cat::CommutativeRing) and
                                 R::hasProp(Cat::DifferentialRing) and
                                 iszero( R::characteristic() ) and
                                 R::hasProp(Ax::normalRep)  or
                                 R::hasProp(
                                   Dom::ExpressionField(id, iszero@normal))) 
			     then error("illegal coefficient ring") end_if
                         else R := Dom::ExpressionField(normal) 
                        end_if;
                        UP := Dom::UnivariatePolynomial(var, R);
                        coerce := UP::new;
                        hasField := bool(R::hasProp(Cat::Field));
                        INT := Dom::Integer;
                        EF := Dom::ExpressionField(); ) ,
  # super-domain: #     Dom::UnivariateSkewPolynomial(var,id,
                                                  fun(diff(args(1),dvar)),R),
  # categories: #       [ Cat::UnivariateSkewPolynomialCat(R) ],
  # axioms: #           [ Ax::normalRep, 
                         (if R::hasProp(Ax::canonicalRep) 
                           then Ax::canonicalRep end_if) ],
  # entries: #

  "new" = proc(e, f)
    local i;
    begin
      if args(0)=0 then error("wrong no of args") end_if;
      if type(e)=DOM_LIST then
        e := map(e, expr);
        e := _plus(e[i]*var^(i-1) $i=1..nops(e))
      elif args(0)=2 then
        e := ode::isLODE(expr(e), f, "Hlode");
        if nops(e)=4 then 
          e := expand(subs(e[1],[diff(f,dvar$i)=var^i $ hold(i)=0..e[4]]))
        end_if
      else e := expr(e) end_if;
      poly(e, [var], R);
      if % = FAIL then return(FAIL)
       else new(this, %) end_if
    end_proc,     


  "Dpoly" = fun( 
   (if args(0) = 1 then this::ore_mult(this::new(var), args(1))
     elif args(1)=[] then args(2)
      else 
          case op(args(1), 1) 
           of 1 do this::Dpoly([op(args(1), 2..nops(args(1)))], 
                                                  this::Dpoly(args(2))); break;
           otherwise 
            if op(args(1),1) > 1 then return( this::zero )
              else return(error("Illegal argument [Dpoly]")) end_if
          end_case;
     end_if) ),

  "D" = this::Dpoly,
   
  "adjoint" = proc(a)
     local r;
     begin
       r := this::zero;
       while not iszero(a) do
         r := r + this::intmult(this::_mult(lterm(a), coerce(lcoeff(a))),
                                (-1)^degree(a) );
         a := this::reductum(a);
       end_while;
       r
     end_proc,   

  "func_call" = proc(_a_, f, o) # Option o: Unsimplified # 
     local r, da;
     begin
       f  := expr(context(f));
       if o=Unsimplified then 
         r  := 0;
         _a_  := this::poly(_a_);
         da := degree(_a_);
         while not iszero(_a_) do
           r  := r + expr(lcoeff(_a_))*diff(f, dvar $ da);
           #~  a  := this::reductum(a); ~#
           _a_  := _a_ - lmonomial(_a_);
           da := degree(_a_);
         end_while;
         r
       else return(this::evalLODO(_a_, f)) end_if
     end_proc,

  "evalLODO" = proc(_a_, f)
    local y, eq, n, r, i, Int, p, u,s;
    begin
      eq := this::func_call(_a_, y(dvar), Unsimplified);
      n  := this::degree(_a_);
      #f  := expr(context(f));#
                                                                     
      # substituting 'Int' for 'int'; this gives a speed up #

      f := maprec(f, {"int"}=fun(Int(op(args()))));
      Int := funcattr(Int, "diff", 
           fun( (if op(args(1), 2)=dvar then op(args(1), 1) else 0 end_if) ));

      #suppress information about the arguments of RootOf#
      f := eval(text2expr(expr2text(f))); 

      # case: f contains RootOf #
      if hastype(f, RootOf) then 
         r  := op(Indets(f, RootOf));
         p  := expr(op(r, 1)); 
         u  := op(r, 2);
         # eq := subs(eval(subs(eq, y(dvar)=subs(f, r=u(dvar)))),
                    [diff(u(dvar),dvar$i)=ode::derivationOfRootOf(p,u,dvar,i) 
                     $hold(i)=0..n]);  #
         eq := subs(subs(eq, y(dvar)=subs(f, r=u(dvar)), Simplified), 
                    [diff(u(dvar),dvar$i)=ode::derivationOfRootOf(p,u,dvar,i) 
                     $hold(i)=0..n], Simplified); 
         y  := u
       else 
         # eq := eval(subs(eq, y(dvar)=f)) end_if; #
         eq := subs(eq, y(dvar)=f,Simplified) end_if;

       # case: f contains exp(...) #
       # (speeds up simplification in an exponential case) # 
      case type(f)  
        of "_plus" do
        of "_mult" do select(f, fun(type(args(1))="exp"));
                      if type(%)<>DOM_INT then
                        # eq := subsex(eq, split(f, has, exp)[1]=1) #
                        eq := subsex(eq, split(f, has, exp)[1]=1, Simplified)
                      end_if; break;
        # of "exp"   do eq := subsex(eq, f=1); break; #
        of "exp"   do eq := subsex(eq, f=1, Simplified); break;
      end_case;

      # now we can simplify eq #
      # eq := simplify(numer(eval(eq))); #
      s := simplify(numer(eq));
      if iszero(s) then return(s)
      else
        normal(s/simplify(denom(eq)))
      end_if;
    end_proc,

  "mkLODO" = proc(a, f)
    local i;
    begin
      case type(a)
        of DOM_LIST do a := map(a, expr);
                       this::new(_plus(a[i]*var^(i-1) $i=1..nops(a))); break;
        otherwise
          a := ode::isLODE(expr(a), f, "Hlode");
          if nops(a)=4 then
            this::new( expand(subs(a[1], [diff(f,dvar$i)=var^i $ i=0..a[4]])) )
           else FAIL end_if
      end_case;
      if %=FAIL then error("cannot convert this expression") 
        else % end_if
    end_proc,

  "symmetricPower" = 
   (if hasField then 
    proc(a, m)
     local y, i;
     begin
       a := ode::symPower(this::func_call(a,y(dvar)),y,dvar,this::degree(a),m);
       this::new(
         expand(subs(a,[diff(y(dvar),dvar$i)=var^i $i=0..ode::order(a,{y})])));
     end_proc
    end_if ),
 
  "solve" = proc(a)
     local o, y;
     begin
       if domtype(a)=DOM_LIST then 
         if map({op(a)}, domtype)<>{this} then 
           error("list of ".expr2text(this)." expected") end_if;
         return(ode::combineSolutionsOfFactors(
                map(a,this::func_call,y(dvar),Unsimplified),y,dvar,
                _plus(op(map(a, this::degree)))))
         end_if;  
       case degree(a)
         of 0 do return({0});
         of 1 do return(
                 {exp(int(-expr(this::reductum(a))/expr(lcoeff(a)), dvar))});
         #of 2 do o := {args(2..args(0))};
                 if not contains(o, "Irreducible") then
                   print(Unquoted, "Warning: Reducible case is not ".
                         "completely implemented! \n".
                         "When you are not sure that your operator, say L, ".
                         "is irreducible,\n"."please test the result e.g. ".
                         "with L(op(%, 1)) ") end_if; #
         otherwise
           o := {args(2..args(0))};
           return(ode::liouvillianSolutions(
                  this::func_call(a,y(dvar),Unsimplified),y(dvar),o));
       end_case 
     end_proc,

  "polynomialZeros" = proc(a)
    local y;
    begin
      if iszero(degree(a)) then return({0}) end_if;
       ode::polysols(map(this::vectorize(a), expr), dvar)
      #ode::polysols(this::vectorize(a), dvar)#
    end_proc,

  "rationalZeros" = proc(a)
    local y;
    begin
      if iszero(degree(a)) then return({0}) end_if;
       ode::ratsols(map(this::vectorize(a), expr), dvar)
      #ode::ratsols(this::vectorize(a), dvar)#
    end_proc,

  "exponentialZeros" = proc(a)
    local y;
    begin
      if iszero(degree(a)) then return({0}) end_if;
      ode::exponentialSolutions(this::func_call(a,y(dvar),Unsimplified),
                                  y(dvar))
    end_proc,

  "liouvillianZeros" = this::solve,

  "Factor" =
  (if hasField then
    proc(a)
      local faclist;
      begin
        faclist := this::factor(a);
        if nops(faclist)=1 then
          faclist[1]
        else
          hold(_mult)(op(faclist))
        end_if
      end_proc
    end_if),

  "factor" = 
  (if hasField then
    proc(a)
      local i,fl;
      begin
        i:=1;
        fl:=[a];
        if degree(a)>3 then 
          print(Unquoted,"Warning: Only right and left factors of degree 1 ".
                         "will be found \n")
        end_if;
        while i <= nops(fl) do
          if degree(fl[i])>1 then fl[i]:=op(this::rlfactor(fl[i])) end_if;
          i:=i+1;
        end_while;
        fl
      end_proc
    end_if),

  "rlfactor" =
  (if hasField then
    proc(a)
      local da, faclist, adjlist, n, i;
      begin
        da := degree(a);
        if (da=0 or da=1) then return([a]) end_if;
        adjlist := [];
        faclist := this::pseudoFactor(a,this::exponentialZeros(a));
        if degree(faclist[1])>1 and da>2 then
          a := this::adjoint(faclist[1]);
          adjlist := this::pseudoFactor(a,this::exponentialZeros(a));
          n := nops(adjlist);
          adjlist := map([adjlist[n-i] $ hold(i)=0..(n-1)], this::adjoint);
          faclist[1] := NIL;
        end_if;
        _concat(adjlist,faclist)
      end_proc
    end_if),

  "pseudoFactor" =
  (if hasField then
    proc(a, l)    # can be improved! #
      local faclist, i, a2, lasta2, y;
      begin
        if iszero(nops(l)) or args(0)=1 then return([a]) end_if;
        l := [op(map(l,expr))];
        faclist := [];
        if degree(a)=nops(l) then l[nops(l)]:=NIL end_if;
        lasta2:=lodo::one;
        for i from 1 to nops(l) do
          a2:=ode::Wronskian([op(l,1..i),y(dvar)],dvar,R);
          a2:=this::mkLODO(map(ode::normalize(a2,y,dvar,i), normal), y(dvar));
          faclist:=[this::rightQuotient(a2,lasta2), op(faclist)];
          lasta2:=a2;
        end_for;
      [this::rightQuotient(a,a2),op(faclist)]
      end_proc
    end_if)        
         

) ):

