# #
# $Date: 1997/03/05 12:08:24 $  $Author: olli $   $Revision: 1.7 $ #
# #
# frankp, 7.09.95 #
# #

#++
        Dom::MatrixGroup  --  the abelian group of m x n matrices

        MatrixGroup( m,n [,R] )

	m,n: positive integers
	R  : (optional) a Cat::CommutativeRing

        This contructor creates a domain representing the set of 
	m x n matrices over a commutative ring R. If R is not given then
	the domain Dom::ExpressionField() will be used.

        This domain constructor is simply build up by the domain
        constructor 'Dom::Matrix'.
++#

Dom::MatrixGroup:= DomainConstructor(

# Name = # Dom::MatrixGroup,

# Args = # [ m,n,R ],

# Local Parameters = # [ MR ],

# Init = # (
    if args(0) <> 2 and args(0) <> 3 then
        error("wrong no of args")
    end_if;
    if not testtype(m,Type::PosInt) or not testtype(n,Type::PosInt) then
	error("no of rows and cols must be positive integers")
    end_if;
    if args(0) = 2 then
        R:= Dom::ExpressionField()
    else
	if R::hasProp( Cat::CommutativeRing ) <> TRUE then
            error("coefficients must be from a Cat::CommutativeRing")
        end_if
    end_if;
    MR:= Dom::Matrix( R )
),

# Super Domain = # Dom::Matrix( R ),

# Categories = # [
    (if R::hasProp(Cat::Field) then
	Cat::VectorSpace(R)
    end_if),
    Cat::MatrixCat( R ), Cat::AbelianGroup
],

# Axiome = # [
    (if R::hasProp( Ax::canonicalRep ) then
	Ax::canonicalRep
    end_if)
],

# Methods = #

"new" = proc(r,c)
    local A, t;
begin
    t:= testargs(TRUE);
    if args(0) > 1 then
	if testtype( r,Type::PosInt ) and testtype( c,Type::PosInt ) then
	    if [r,c] <> [m,n] then
		error("wrong no of rows and/or columns")
	    elif traperror( (A:=MR::new(args())) ) <> 0 then
		error("wrong argument(s)")
    	    end_if
	elif traperror( (A:=MR::new(m,n,args())) ) <> 0 then
	     error("wrong argument(s)")
	end_if
    elif r::hasProp( Dom::Matrix ) = TRUE then
	A:= this::convert(r);
        if A = FAIL then
             error("invalid argument, unable to convert")
        end_if
    elif traperror( (A:=MR::new(m,n,args())) ) <> 0 then
         error("wrong argument(s)")
    end_if;

    if A::dimen(A) <> [m,n] then
	error("wrong no of rows and/or columns")
    end_if;

    extsubsop(A,0=this)
end_proc,

"convert" = proc(A)
    local B;
begin
    if args(0) <> 1 then FAIL
    elif domtype(args(1)) = this then A
    elif A::hasProp( Dom::Matrix ) = TRUE then
        B:= MR::convert( A );
        if B = FAIL then FAIL
        elif B::dimen(B) <> [m,n] then FAIL
        else extsubsop(B,0=this)
        end_if
    elif traperror( (A:=MR::new(m,n,A)) ) <> 0 then
        FAIL
    else
        A
    end_if
end_proc,

"_mult" = proc(r,x)
    local t;
begin
    if args(0) = 1 then return( args(1) )
    elif args(0) > 2 then
        r:= _mult( args(1..(args(0) div 2)) );
        x:= _mult( args(((args(0) div 2) + 1)..args(0)) );
	if (domtype(r))::_mult <> FAIL then r:= (domtype(r))::_mult(r, x)
	elif (domtype(x))::_mult <> FAIL then r:= (domtype(x))::_mult(r, x)
	else r:= _mult(r, x)
	end_if;
	return(r);
    end_if;

    if domtype(x) = this then
        if domtype(r) = DOM_INT then
            if r = 1 then x
            elif r = -1 then this::_negate(x)
            elif r = 0 then this::zero
            elif r > 1 then this::_plus(x $ r)
            else this::_negate(this::_plus(x $ -r))
            end_if
	elif testtype( r,R ) then
	    # component wise multiplication #
	    this::map( x,R::_mult,R::convert(r) )
        elif r::hasProp( Dom::Matrix ) = TRUE then
	    t:= MR::_mult(extsubsop(r,0=MR),extsubsop(x,0=MR));
	    if t <> FAIL then
		if t::dimen(t) <> [m,n] then t
                else extsubsop( t,0=this )
                end_if
            else FAIL
            end_if
	else
	    FAIL
        end_if
    elif domtype(x) = DOM_INT or testtype( x,R ) then
	this::_mult(x,r)
    elif x::hasProp( Dom::Matrix ) = TRUE then
	t:= MR::_mult(extsubsop(r,0=MR),extsubsop(x,0=MR));
	if t <> FAIL then
	    if t::dimen(t) <> [m,n] then t
	    else extsubsop( t,0=this )
	    end_if
	else FAIL
	end_if
    else
	FAIL
    end_if
end_proc,

"zero" = extsubsop(MR::new( m,n ),0=this),

"one" = (if m = n then
            extsubsop(MR::new( m,n,1,Diagonal ),0=this)
        end_if),

"delRow" = domattr(MR,"delRow"),

"delCol" = domattr(MR,"delCol"),

"stackMatrix" = domattr(MR,"stackMatrix"),

"concatMatrix" = domattr(MR,"concatMatrix"),

"row" = domattr(MR,"row"),

"col" = domattr(MR,"col"),

"random" = (if R::random <> FAIL then
    fun( this::newThis( m,n,
            [[R::random() $ hold(j)=1..n] $ hold(i)=1..m]
          ) )
end_if)

):

# end of file #
