# #
# $Date: 1999/06/02 12:07:09 $  $Author: frankp $   $Revision: 1.161.2.4 $ #

# #
# frankp, 11/10/94 #

#++
  Dom::Matrix.mu

	Dom::Matrix -- the domains of matrices 

	Dom::Matrix([R])

	R - (optional) coefficient domain (a Cat::Rng)

	A domain Dom::Matrix(R) represents all matrices over the 
	Cat::Rng R.  If the argument 'R' is missing then the coefficient 
	domain will be the domain 'Dom::ExpressionField()'. 

	An element of a domain created by Dom::Matrix (a matrix), has three
	operands 'r, c, a'. The first two operands give the dimension 
	[r,c] of the matrix and the third operand is a two dimensional 
	array a which holds the entries of the matrix.

	Methods:
	--------
	new, newThis, mkDense, _index, set_index,
	convert, convert_to, dimen, nonZeros, transpose, has,
	stackMatrix, concatMatrix, setRow, setCol, row, col,
	delRow, delCol, swapRow, swapCol, _mult, _plus, _power,
	_negate, iszero, equal, _invert, _divide, gaussElim, isDense,
	isSparse, TeX, map, subs, subsex, length, expr, _concat,
	randomDimen, random, tr, norm, exp, range_index, op, subsop,
	nops, assign_elems, float.
++#

Dom::Matrix := DomainConstructor(
# name #
    Dom::Matrix,
# arguments #
    [ R ],
# local variables #
    [ RcanonicalRep, Riszero, Rmult, Rplus, Rzero ],
# initialisation #
    (if args(0) > 1 then error("wrong no of args") end_if;
     if args(0) = 0 then R := Dom::ExpressionField()
     elif R::hasProp( Cat::Rng ) <> TRUE then
	error("coefficients must be from a Cat::Rng")
     end_if;
     RcanonicalRep := R::hasProp( Ax::canonicalRep );
     Riszero := R::iszero; Rzero := R::zero;
     Rplus := R::_plus; Rmult := R::_mult
    ),
# super-domains #
    Dom::BaseDomain,
# categories #
    [ Cat::MatrixCat(R) ],
# axioms #
    [ (if R::hasProp( Ax::canonicalRep ) then Ax::canonicalRep end_if) ],
# #
    "isSparse" = FALSE, "isDense" = TRUE,
#--
  new  --  return a new matrix
--#
    "new" = proc(r,c,f)
        local i, j, a;
    begin
        case args(0)
	of 1 do
	    if testargs() then
		if r::hasProp( Cat::MatrixCat ) <> TRUE and
		   domtype(r) <> DOM_LIST and
		   (domtype(r) = DOM_ARRAY and op(r,[0,1]) > 2)
		then
		    error("invalid argument")
		end_if
	    end_if;
	    if (r:= this::coerce( r )) = FAIL then
		error("unable to define matrix over ".expr2text(R))
	    else
		return( r )
	    end_if
        of 2 do
            if not testtype( r, Type::PosInt ) or not testtype( c, Type::PosInt ) then
                error("expecting dimension as positive integers")
	    else
                return( new(this,r,c,array(1..r,1..c,[[Rzero $ c] $ r])) )
	    end_if
	of 3 do
	    if not testtype( r, Type::PosInt ) or not testtype( c, Type::PosInt ) then
                error("expecting dimension as positive integers")
            elif domtype( f ) <> DOM_LIST then
                # consider f as a function of (i,j) #
                a:= array(1..r,1..c, [[R::coerce( f(i,j) ) $ j=1..c] $ i=1..r]);
                if has( a, FAIL ) then
                    error("unable to convert function value");
                else
                    return( new(this,r,c,a) )
                end_if
	    else
                a:= this::mkDense( r,c,f );
		if a = FAIL then
		    error("unable to define matrix over ".expr2text(R))
		else
		    return( new(this,op(a)) )
		end_if
	    end_if
        of 4 do # create diagonal matrix #
            case args(4)
            of hold(Diagonal) do
	        if domtype( f ) <> DOM_LIST then
                    a:= array(1..r,1..c,
                      [[Rzero $ i-1, R::coerce(f(i,i)), Rzero $ c-i] $ i=1..min(r,c), 
                       [Rzero $ c] $ r-min(r,c)]
		    )
	        else
	            a:= assign_elems( array(1..r,1..c,[ [Rzero $ c] $ r ]),
		      [i,i] = R::coerce( f[i] ) $ i=1..min(r,c,nops(f)) 
		    )
	        end_if;

                if has( a, FAIL ) then
                    error("unable to define matrix over ".expr2text(R))
                else
                    return( new(this,r,c,a) )
                end_if
            of hold(Banded) do
                if r <> c then error("matrix must be square")
                elif domtype( f ) <> DOM_LIST or nops( f ) mod 2 = 0 or nops( f ) > r then
                    error("expecting list with a number of elements which is odd and not greater than ".expr2text(r))
                else
                    c:= nops(f) div 2;
                    a:= array(1..r,1..r,
                        [[ (if abs(j-i) > c then Rzero else R::coerce(f[c+j-i+1]) end_if) $ j=1..r] $ i=1..r]
                    );
                    if has( a,FAIL ) then 
                        error("unable to define matrix over ".expr2text(R))
                    else
                        return( new(this,r,r,a) )
                    end_if
                end_if
            otherwise
                error("expecting 'Diagonal' or 'Banded' as options")
            end_case 
        otherwise
            error("wrong no of args")
        end_case
    end_proc,
#--
  newThis  --  create matrices without testing the arguments
--# 
    "newThis" = proc(r,c,f)
	local i, j, a;
    begin
	case args(0)
	of 2 do
	    return( new(this,r,c,array(1..r,1..c,[[Rzero $ c] $ r])) )
	of 1 do
	    if domtype(r) = DOM_ARRAY then
	        return( new(this,op(r,[0,2,2]) - op(r,[0,2,1]) + 1,
                            op(r,[0,3,2]) - op(r,[0,3,1]) + 1,r) )
            elif domtype(r) = DOM_LIST then
                f:= r; r:= nops(f); c:= 1;
                if domtype(f[1]) = DOM_LIST then
                    c:= max( op(map(f,nops)) )
                end_if
            elif r::hasProp(Cat::MatrixCat) = TRUE then
                return( new(this,extop(r)) )
	    else
	        error("invalid argument")
            end_if
	of 4 do
	of 3 do
	    if domtype(f) <> DOM_LIST then
		# consider f as a function of (i,j) #
		if args(0) = 4 and args(4) = hold(Diagonal) then 
		    return( new(this,r,c,array(1..r,1..c,
                      [[Rzero $ i-1, f(i,i), Rzero $ c-i] $ i=1..min(r,c), 
		       [Rzero $ c] $ r-min(r,c)]
		    )) )
		else
		    return( new(this,r,c,array(1..r,1..c,
		      [[f(i,j) $ j=1..c] $ i=1..r]
		    )) )
		end_if
	    end_if;

	    if args(0) = 4 then
                if args(4) = hold(Diagonal) then
                    a := array( 1..r,1..c,[ [Rzero $ c] $ r ] );
		    a:= assign_elems( a,([i,i] = f[i]) $ i=1..min(r,c,nops(f)) )
                else # option Banded #
                    c:= nops(f) div 2;
                    a:= array(1..r,1..r,
                        [[ (if abs(j-i) > c then Rzero else f[c+j-i+1] end_if) $ j=1..r] $ i=1..r]
                    )
                end_if
            elif r > 1 and c > 1 then
                # define matrix by list of list #
                a:= array( 1..r,1..c,[ [Rzero $ c] $ r ] );
                a:= assign_elems( a,( ([i,j] = op(f,[i,j])) $ j=1..nops(f[i]) ) $ i=1..nops(f) );
	    elif r = 1 then
		if domtype(f[1]) = DOM_LIST then f:= f[1] end_if;
                a:= array( 1..r,1..c,[ [Rzero $ c] $ r ] );
		a:= assign_elems( a,([1,i] = f[i]) $ i=1..nops(f) )
	    elif domtype(f[1]) = DOM_LIST then
		# case [[1],[2],...] #
                a:= array( 1..r,1..c,[ [Rzero $ c] $ r ] );
		a:= assign_elems( a,([i,1] = f[i][1]) $ i=1..nops(f) )
	    else
                a:= array( 1..r,1..c,[ [Rzero $ c] $ r ] );
		a:= assign_elems( a,([i,1] = f[i]) $ i=1..nops(f) )
	    end_if;

	    new( this,r,c,a )
	end_case
    end_proc,
#--
   mkDense  --  convert an expression to an array 
--#
    "mkDense" = proc(__x)
	local i, j, r, c, __a, t, offsi, offsj, aux;
    begin
	case args(0)
	of 1 do
	    case domtype(__x)
	    of DOM_ARRAY do
		if has(__x,NIL) then
		    userinfo(1,"unable to define matrix over ",R);
		    return( FAIL )
		end_if;
	        __a:= map( __x,R::coerce );
	        if has( __a,FAIL ) then
		    userinfo(1,"unable to define matrix over ",R);
		    return( FAIL )
	  	end_if;  
	        r:= op(__x,[0,2,2]) - op(__x,[0,2,1]) + 1;
	    	if op(__x,[0,1]) = 2 then 
		    c:= op(__x,[0,3,2]) - op(__x,[0,3,1]) + 1;
		    if op(__x,[0,2,1])=1 and op(__x,[0,3,1])=1 then
			return([r,c,__a]);
		    end_if;
		    // return( [r,c,subsop(array(1..r,1..c),i=op(a,i) $ i=1..nops(a))] )
		    t:=array(1..r,1..c);
		    offsi:=op(__x,[0,2,1])-1;
		    offsj:=op(__x,[0,3,1])-1;
		    for i from 1 to r do
			aux:= i+offsi;
			for j from 1 to c do
				t[i,j]:=__a[aux, j+offsj]
			end_for
		    end_for;
		    return([r,c,t])
		else
 		    return( [r,1, array(1..r,1..1, map([op(__a)], DOM_LIST)) ])
		end_if;

	    of DOM_LIST do
	        r:= nops(__x);
	        if r = 0 then return( FAIL )
	        elif domtype(__x[1]) <> DOM_LIST then
		    c := 1;
		    __x:= map(__x, fun([args(1)]))
		elif nops({op(map(__x,domtype))} minus {DOM_LIST}) > 0 then
		    return( FAIL )
		else
		    c:= max(op(map(__x,nops)))
		end_if;

		break
	    otherwise
		return( FAIL )
	    end_case;
	    break
	of 3 do
	    r := __x; c := args(2); 
	    if not testtype( r,Type::PosInt ) or not testtype( c,Type::PosInt ) then
		userinfo(1,"expecting positive integers as dimension");
		return( FAIL ) 
	    end_if;
	    __x := args(3);
	    # handle special cases for vectors  

		M( 1,2,[1,2] ) -> [1,2]

		M( 2,1,[1,2] ) -> +- -+
		                  | 1 |
			          |   |
			          | 2 |
		                  +- -+
	    #
	    if r = 1 then
		if nops(__x) > c then
		    error("number of columns does not match")
		end_if;
		if domtype(op(__x)) <> DOM_LIST then __x:= [__x] end_if;
	    elif c = 1 then
		if nops(__x) > r then
		    error("number of rows does not match")
		elif domtype(op(__x)) = DOM_LIST then
		    error("expecting list of entries")
		end_if
	    elif nops(__x) > r then
		error("number of rows does not match")
	    else
		t := map({op(__x)},domtype);
	        if t = {DOM_LIST} then
	            if max(op(map(__x, nops))) > c then
		        error("number of columns does not match")
	            end_if
	        elif contains(t,DOM_LIST) then
		    return( FAIL )
	        end_if
	    end_if;
	    break
	otherwise
	    return( FAIL )
	end_case;

	__a:= array(1..r,1..c,[[ Rzero $ c] $ r]);
	# define matrix by list of list #
	for i from 1 to nops(__x) do
	    t := __x[i];
	    if domtype(t) <> DOM_LIST then
               t:=[t] # allows to write matrix([a,b,c]) instead 
			of matrix([[a],[b],[c]]) #
            end_if;
	    t:= [ ([i,j] = R::coerce( t[j] )) $ j=1..nops(t) ];
	    if has( t,FAIL ) then 
		return( FAIL )
	    else
	        __a:= assign_elems( __a,op(t) );
	    end_if
	end_for;

	[r,c,__a]
    end_proc,
#--
  _concat  --  appends matrices horizontallt (see concatMatrix)
--#
    "_concat" = this::concatMatrix,
#--
  _index  --  indexing of matrices 
--#
    "_index" = fun( 
	(if args(0) = 3 then
	    if domtype(args(2)) = DOM_EXPR then
		this::range_index(args())
	    else
	        _index(extop(args(1),3),args(2),args(3))
	    end_if
	 elif extop(args(1),1) = 1 then
	    _index(extop(args(1),3),1,args(2))
	 else
	    _index(extop(args(1),3),args(2),1)
	 end_if)
	),
    "range_index" = proc(x,r1,r2)
	local a, __i, __j, ur1, ur2;
    begin
	x:= extop(x,3);
	ur1:= op(r1,1); ur2:= op(r2,1);
	a:= array(1..op(r1,2)-ur1+1,1..op(r2,2)-ur2+1);
	((a[__i-ur1+1,__j-ur2+1]:= x[__i,__j]) $ __j=r2) $ __i=r1;

	new( this,op(a,[0,2,2]),op(a,[0,3,2]),a )
    end_proc,
	     	

   
#--
  set_index  --  indexing of matrices 
--#
    "set_index" = fun(
	(if args(0) = 4 then
	    extsubsop( args(1),3=subsop( 
	    extop(args(1),3),(args(2)-1)*extop(args(1),2)+args(3)=args(4)) )
	 elif extop(args(1),1) = 1 then
	    extsubsop( args(1),3=subsop(extop(args(1),3),args(2)=args(3)) ) 
	 else
	    extsubsop( args(1),3=subsop(
		extop(args(1),3),(args(2)-1)*extop(args(1),2)+1=args(3)) ) 
	 end_if)
	),
#--
  print  --  print matrices in form of an array 
--#
   "print" = fun(( 
		   map(extop(args(1),3),context);
		   if has(map(%,type),type(args(1)))
			then
				extop(args(1),3) 
		   else
				%
		   end_if
		)),
#--
   expr -- returns the matrix as an element of DOM_ARRAY, whereby
           each entry was converted to an expression (using R::expr).
--#
    "expr" = fun(map(extop(args(1),3),R::expr)),

#--
   float -- maps float to the elements of the matrix, but only if
	    the domain of the elements has a float method.
--#

    "float" = (if R::float <> FAIL then
		   fun( 
			extsubsop(args(1),3=map(extop(args(1),3),R::float))
		   )
	       else FAIL end_if),

#--
   convert -- convert an expression to a matrix 
--#
    "convert" = proc(x)
    begin
        if args(0) <> 1 then return( FAIL ) end_if;

        case domtype(x)
        of this do
            return( x )
        of DOM_ARRAY do
        of DOM_LIST do
            x:= this::mkDense( x );
            if x = FAIL then return( FAIL ) else return(new( this,op(x) )) end_if
        otherwise
            case x::constructor
            of Dom::SquareMatrix do
            of this::constructor do
                if x::coeffRing = R then
                    return(extsubsop( x,0=this ))
                elif x::isSparse then
                    // not yet implemented!
                    return(FAIL)
                else
                    x:= this::mkDense( extop( x,3 ) );
                    if x = FAIL then return( FAIL ) else return(new( this,op(x) )) end_if
                end_if
            otherwise
                return( FAIL )
            end_case
        end_case
    end_proc,

#--
  convert_to  --  convert matrices 
--#
    "convert_to" = proc(e,F)
        local i, j, t;
    begin
        if args(0) <> 2 then return( FAIL ) end_if;
        if domtype(e) = F then return(e) end_if;
        if domtype(F) <> DOM_DOMAIN then F:= domtype(F) end_if;

        case F
        of DOM_ARRAY do
            return( extop( e,3 ) )
        of DOM_LIST do
            t:= extop(e,3);
            return( [[t[i,j] $ j=1..extop(e,2)] $ i=1..extop(e,1)] )
        otherwise
            if testtype( F,DOM_ARRAY ) then return(F::convert( extop(e,3) )) end_if
        end_case;

        case F::constructor
        of Dom::SquareMatrix do
            if F::dimen <> e::dimen(e) then return( FAIL ) end_if
        of Dom::Matrix do
            // same internal representation of matrices
            if F::isSparse then
                // not yet implemented!
                return(FAIL)
            elif F::coeffRing = e::coeffRing then
                return( extsubsop( e,0=F ) )
            else
                t:= F::mkDense( extop( e,3 ) );
                if t <> FAIL then
                    return( new( F,op(t) ) )
                end_if
            end_if
        otherwise
            return( FAIL )
        end_case
    end_proc,

#--
  dimen  --  return the dimension of a matrix 
--#
    "dimen" = fun( [extop(args(1),1),extop(args(1),2)] ),
#--
  _mult  --  multiply matrices

  Syntax:

  _mult(x1 [,x2,x3])

  Synopsis:

  Multiplies two or more matrices.
  If an argument is not a matrix then it will be converted to an element 
  of R if possible, and scalar multiplication is performed. Otherwise 
  the domain of the arument is asked to perform the multiplication
  (which could return FAIL).
--#
    "_mult" =
# Has R the axiom systemRep ? (multiplication would become faster) #
(if R::hasProp( Ax::systemRep ) then
    proc(x,y)
	local r, c, i, j, cy, k;
    begin
	# Only one argument given? Useful for direct use of A::_mult. #
	if args(0) = 1 then return( x )
	elif args(0) > 2 then
            x:= _mult( args(i) $ i=1..(args(0) div 2) );
            y:= _mult( args(i) $ i=((args(0) div 2) + 1)..args(0) );
            if (domtype(x))::_mult <> FAIL then x:= (domtype(x))::_mult(x, y)
            elif (domtype(y))::_mult <> FAIL then x:= (domtype(y))::_mult(x, y)
            else x:= _mult(x, y)
            end_if;
            return(x);
        end_if;

	case domtype(x) 
	of this do
	    case domtype(y)
	    of this do
                r:= extop(x,1);
                c:= extop(x,2);
                x:= extop(x,3);
                if c <> extop(y,1) then
                    error("dimensions don't match")
                end_if;
                cy:= extop(y,2);
                y:= extop(y,3);
                return( new(this,r,cy,array(1..r,1..cy,[
                  [Rplus(Rmult(x[k,i],y[i,j]) $ i=1..c ) $ j=1..cy]
                        $ k=1..r]
                )) )

	    # Is y an integer? Note, that R has system representation. #
	    of DOM_INT do
		case y 
		of 1 do return( x )
		of 0 do
		    r:= extop(x,1);
		    c:= extop(x,2);
		    return( extsubsop(x,3=array(1..r,1..c,[[0 $ c] $ r])) )
		otherwise
		    return( extsubsop(x,3=map(extop(x,3),_mult,y)) )
	 	end_case

	    # Is y an element of R ? #
	    of R do return( extsubsop(x,3=map(extop(x,3),_mult,expr(y))) )

	    # Could y be converted into this or R ? #
	    otherwise
		if (k:= this::coerce(y)) <> FAIL then
		    return(this::_mult( x,k ))
		elif testtype( y,R ) then
		    return( extsubsop(x,3=map(extop(x,3),_mult,expr(y))) )
			# do not use y::expr, y could be an element #
			# of a base domain #
	        else 
		    # Is the domain of y be able to perform #
		    # the multiplication? #
	            return( y::_mult(x, y) )
		end_if
	    end_case


	# Is x an integer? Note, that R has system representation. #
	of DOM_INT do
	    case x 
            of 1 do return( y )
            of 0 do
                r:= extop(y,1);
		c:= extop(y,2);
                return( extsubsop(y,3=array(1..r,1..c,[[0 $ c] $ r])) )
            otherwise
                return( extsubsop(y,3=map(extop(y,3),_mult,x)) )
            end_case

	# Is x an element of R ? #
        of R do return( extsubsop(y,3=map(extop(y,3),_mult,expr(x))) )

	# Could x be converted into this or R ? #
	otherwise
	    if (k:= this::coerce(x)) <> FAIL then
		return(this::_mult( k,y ))
	    elif testtype( x,R ) then
		return( 
                    extsubsop(y,3=map(extop(y,3),_mult,expr(x)))
                )
	    else
		# unable to perform the multiplication #
                return( FAIL )
	    end_if
	end_case
    end_proc
elif R::hasProp( Cat::CommutativeRing ) then
    proc(x,y)
        local r, c, i, j, cy, k;
    begin
	# Only one argument given? Useful for direct use of A::_mult. #
        if args(0) = 1 then return( x )
        elif args(0) > 2 then
            x:= _mult( args(i) $ i=1..(args(0) div 2) );
            y:= _mult( args(i) $ i=((args(0) div 2) + 1)..args(0) );
            if (domtype(x))::_mult <> FAIL then x:= (domtype(x))::_mult(x, y)
            elif (domtype(y))::_mult <> FAIL then x:= (domtype(y))::_mult(x, y)
            else x:= _mult(x, y)
            end_if;
            return(x);
        end_if;

	case domtype(x)
	of this do
	    case domtype(y)
	    of this do
                r:= extop(x,1);
		c:= extop(x,2);
                x:= extop(x,3);
                if c <> extop(y,1) then
                    error("dimensions don't match")
                end_if;
                cy:= extop(y,2);
                y:= extop(y,3);
                return( new(this,r,cy,array(1..r,1..cy,[
                  [Rplus(Rmult(x[k,i],y[i,j]) $ i=1..c ) $ j=1..cy] $ k=1..r]
                )) );

            of DOM_INT do
		case y 
                of 1 do return( x )
                of -1 do return( this::_negate(x) )
                of 0 do
		    r:= extop(x,1);
		    c:= extop(x,2);
		    return( extsubsop(x,3=array(1..r,1..c,[[Rzero $ c] $ r])) )
		otherwise
		    # Is R a ring with unit ? #
		    if (i:=R::one) <> FAIL then
			# c = i+i+...+i (|y| times) becomes an #
			# element of R #
		        c:= Rplus( i $ abs(y) );
			x:= extsubsop(x,3=map(extop(x,3),Rmult,c));
			if y > 0 then return( x )
			else return( this::_negate(x) )
			end_if
		    elif y > 0 then return( this::_plus(x $ y) )
                    else return( this::_negate(this::_plus(x $ -y)) )
                    end_if;
		end_case

	    of R do return( extsubsop(x,3=map(extop(x,3),Rmult,y)) )

	    otherwise
		if (k:= this::coerce( y )) <> FAIL then
		    return(this::_mult( x,k ))
		elif (k:= R::coerce(y)) <> FAIL then
		    return( extsubsop(x,3=map(extop(x,3),Rmult,k)) )
		else
		    # Is the domain of y be able to perform #
		    # the multiplication ? #
		    return( y::_mult(x, y) )
		end_if
	    end_case


        of DOM_INT do
	    case x
            of 1 do return( y )
            of -1 do return( this::_negate(y) )
            of 0 do
                r:= extop(y,1);
		c:= extop(y,2);
                return( extsubsop(y,3=array(1..r,1..c,[[Rzero $ c] $ r])) )
            otherwise
		# Is R a ring with unit ? #
                if (i:=R::one) <> FAIL then
                    # c = i+i+...+i (|x| times) becomes an element of R #
                    c:= Rplus( i $ abs(x) );
                    y:= extsubsop(y,3=map(extop(y,3),Rmult,c));
                    if x > 0 then return( y )
                    else return( this::_negate(y) )
                    end_if
                elif x > 0 then return( this::_plus(y $ x) )
                else return( this::_negate(this::_plus(y $ -x)) )
                end_if;
            end_case

	# scalar * Matrix ? Use of map is possible #
	# because R is commutative #
 	of R do return( extsubsop(y,3=map(extop(y,3),Rmult,x)) )

	otherwise
	    if (k:= this::coerce(x)) <> FAIL then
		return(this::_mult( k,y ))
	    elif (k:= R::coerce(x)) <> FAIL then
		return( extsubsop(y,3=map(extop(y,3),Rmult,k)) )
	    else
		# unable to perform the multiplication #
                return( FAIL )
            end_if
        end_case;
    end_proc
else
    proc(x,y)
        local r, c, i, j, cy, k;
    begin
        # Only one argument given? Useful for direct use of A::_mult. #
        if args(0) = 1 then return( x )
        elif args(0) > 2 then
            x:= _mult( args(i) $ i=1..(args(0) div 2) );
            y:= _mult( args(i) $ i=((args(0) div 2) + 1)..args(0) );
            if (domtype(x))::_mult <> FAIL then x:= (domtype(x))::_mult(x, y)
            elif (domtype(y))::_mult <> FAIL then x:= (domtype(y))::_mult(x, y)
            else x:= _mult(x, y)
            end_if;
            return(x);
        end_if;

        case domtype(x)
        of this do
            case domtype(y)
            of this do
                r:= extop(x,1);
                c:= extop(x,2);
                x:= extop(x,3);
                if c <> extop(y,1) then
                    error("dimensions don't match")
                end_if;
                cy:= extop(y,2);
                y:= extop(y,3);
                return( new(this,r,cy,array(1..r,1..cy,[
                  [Rplus(Rmult(x[k,i],y[i,j]) $ i=1..c ) $ j=1..cy] $ k=1..r]
                )) );

            of DOM_INT do
                case y
                of 1 do return( x )
                of -1 do return( this::_negate(x) )
                of 0 do
                    r:= extop(x,1);
                    c:= extop(x,2);
                    return( extsubsop(x,3=array(1..r,1..c,[[Rzero $ c] $ r])) )
                otherwise
                    # Is R a ring with unit ? #
                    if (i:=R::one) <> FAIL then
                        # c = i+i+...+i (|y| times) becomes an #
			# element of R #
                        c:= Rplus( i $ abs(y) );
                        x:= extsubsop(x,3=map(extop(x,3),Rmult,c));
                        if y > 0 then return( x )
                        else return( this::_negate(x) )
                        end_if
                    elif y > 0 then return( this::_plus(x $ y) )
                    else return( this::_negate(this::_plus(x $ -y)) )
                    end_if;
                end_case

            of R do return( extsubsop(x,3=map(extop(x,3),Rmult,y)) )

            otherwise
		if (k:= this::coerce(y)) <> FAIL then
		    return(this::_mult( x,k ))
                elif (k:= R::coerce(y)) <> FAIL then
                    return( extsubsop(x,3=map(extop(x,3),Rmult,k)) )
                else
                    # Is the domain of y be able to perform #
		    # the multiplication? #
                    return( y::_mult(x, y) )
                end_if
            end_case


        of DOM_INT do
            case x
            of 1 do return( y )
            of -1 do return( this::_negate(y) )
            of 0 do
                r:= extop(y,1);
                c:= extop(y,2);
                return( extsubsop(y,3=array(1..r,1..c,[[Rzero $ c] $ r])) )
            otherwise
                # Is R a ring with unit ? #
                if (i:=R::one) <> FAIL then
                    # c = i+i+...+i (|x| times) becomes an element of R #
                    c:= Rplus( i $ abs(x) );
                    y:= extsubsop(y,3=map(extop(y,3),Rmult,c));
                    if x > 0 then return( y )
                    else return( this::_negate(y) )
                    end_if
                elif x > 0 then return( this::_plus(y $ x) )
                else return( this::_negate(this::_plus(y $ -x)) )
                end_if;
            end_case

        of R do
            # scalar * Matrix; use of map is not possible #
	    # because R is not commutative #
            r:= extop(y,1);
            c:= extop(y,2);
            cy:= extop(y,3);
            ((cy[i,j] := Rmult(x,cy[i,j])) $ j=1..c) $ i=1..r;
            return( extsubsop(y,3=cy) )

        otherwise
            if (k:= this::coerce(x)) <> FAIL then
                return(this::_mult( k,y ))
            elif (k:= R::coerce(x)) <> FAIL then
                r:= extop(y,1);
                c:= extop(y,2);
                cy:= extop(y,3);
                ((cy[i,j] := Rmult(k,cy[i,j])) $ j=1..c) $ i=1..r;
                return( extsubsop(y,3=cy) )
            else
		# unable to perform the multiplication #
                return( FAIL )
            end_if
        end_case;
    end_proc
end_if),
#--
   _plus  --  add matrices 
--#
    "_plus" = proc(x)
	local y, r, c, k, l, argu;
    begin
	if args(0) = 1 then return( x ) end_if;
        argu:= [args()];
        for c from 1 to args(0) do
            if domtype( argu[c] ) <> this then
		r:= argu[c];
		if (r:= this::coerce(r)) <> FAIL then
		    argu[c]:= r
	        else
	            userinfo(1,"operands are not compatible");
                    return( FAIL )
		end_if
	    end_if
	end_for;

        r:= extop(x,1);
	c:= extop(x,2);
        x:= extop(x,3);
        unassign( argu[1] );
        for y in argu do
            if [extop(y,1),extop(y,2)] <> [r,c] then
                error ("dimensions don't match")
            end_if;
	    l:= zip([op(x)],[op(extop(y,3))],Rplus);
            x:= subsop( array(1..r,1..c),k=l[k] $ k=1..r*c )
        end_for;

	extsubsop(y,3=x)
    end_proc,
#--
  _power  --  multiply a matrix with itselfes 

  Syntax:

  _power(A,i)

  A  -- matrix
  i  -- integer

  Synopsis:

  Multiplies A i times. There are three cases:
  - i > 0: The product A*A*...*A (i times) is computed.
  - i = 0: The identity matrix is returned if R has the unit 1.
  - i < 0: The product 1/A*...*1/A (-i times) is computed if
           the inverse of A can be computed.

  Returns FAIL if i is not an integer or in the case of
  negative i and the inverse of A can not be computed.
  If i = 0 and R has not the unit 1 then FAIL is returned.
--#
    "_power" = proc(x,i)
	local n, j;
    begin
	if extop(x,1) <> extop(x,2) then
	    userinfo(1,"matrix is not square");
	    return( FAIL )
	elif domtype(i) <> DOM_INT then
	    userinfo(1,"exponent is not an integer");
	    return( FAIL )
	end_if;

	case i
	of 0 do
	    if R::one = FAIL then
		userinfo(1,"coefficient domain is without the unit");
		return( FAIL )
	    end_if;
	    n:= extop( x,1 );
	    return( new( this,n,n,
	      array(1..n,1..n,[[Rzero $ j-1,R::one,Rzero $ n-j]
		$ j=1..n])
	    ) )
	of 1 do return( x )
 	of -1 do return( this::_invert( x ) )
	otherwise
	    if i > 1 then this::_mult( x $ i )
	    # case i < -1 #
	    elif (x:= this::_invert(x)) = FAIL then
		userinfo(1,"the inverse of the matrix can not be computed");
		FAIL
	    else this::_mult( x $ -i )
	    end_if
	end_case
    end_proc,
#--
  _negate  --  _negate a matrix 
--#
    "_negate" =
	fun( extsubsop( args(1),3=map(extop( args(1),3 ),R::_negate) )
    ),
#--
  _invert  --  compute the invert of a matrix 

  Syntax:

  _invert(A)

  A  --  matrix

  Synopsis:

  Returns A^(-1) if A is regular, FAIL otherwise.
--#
    "_invert" = (if R::hasProp( Cat::IntegralDomain ) then 
    # in MuPAD an Cat::IntegralDomain has the unit element #
    proc(x)
	local n, i, j, t, k, R_divide, R_negate;
    begin
	if extop( x,1 ) <> extop( x,2 ) then
	    userinfo(1,"expecting a square matrix");
	    return( FAIL )
	end_if;

    # append the n x n identity matrix to x #
	n := extop( x,1 ); x := extop( x,3 );
	t := array( 1..n,1..2*n,
             [[ x[i,j]$j=1..n,Rzero$i-1,R::one,Rzero$n-i ] $ i=1..n]
	);

    # rank(x) < n ? #
	x := extop( op(this::gaussElim( new(this,n,2*n,t) ),1),3 );
	t := { Riszero( x[n,i] ) $ i=1..n };
	if t = {TRUE} then
	    userinfo(1,"matrix is not invertible");
	    return( FAIL )
 	end_if;

	R_divide := R::_divide; R_negate := R::_negate;

    # transform the submatrix x(1..n,1..n) to identity matrix 
      by backsolving
    #
	for k from n downto 1 do
	    t := x[k,k];
	    for j from n+1 to 2*n do
		x[k,j] := R_divide( x[k,j],t );
		if x[k,j] = FAIL then
		    userinfo(1,"computation fails over its coefficient domain");
		    return( FAIL )
		end_if
	    end_for;
	    for i from k-1 downto 1 do
		t := R_negate( x[i,k] );
		for j from n+1 to 2*n do
		    if not Riszero( x[k,j] ) then
			x[i,j] := Rplus(x[i,j],Rmult(x[k,j],t))
		    end_if
		end_for
	    end_for
	end_for;

	new( this,n,n,
	    array(1..n,1..n,[[x[i,j] $ hold(j)=n+1..2*n] $ hold(i)=1..n])
	)
    end_proc
    end_if ),
#--
  _divide  --  compute a/b for a matrix a.
--#
    "_divide" = ( if R::hasProp( Cat::IntegralDomain ) then
        fun((
          1/args(2);
	  if % <> FAIL then
	      this::_mult( args(1),% )
	  else
	      userinfo(1,"the inversion of the 2nd matrix can not be computed");
	      FAIL
	  end_if
	))
    end_if ),
#--
  iszero  --  test if a matrix is the zero matrix 
--#
    "iszero" = fun(
	(map({op(extop(args(1),3))},Riszero);	
	 if nops(%) > 1 then	
	     FALSE 
	 else 
	     op(%,1) 
	 end_if) 
	       ),
#--
  equal  --  test if two matrices are equal 
--#
    "equal" = proc(x,y)
	local r, c, i, j, t;
    begin
	r := extop(x,1); c := extop(x,2);
	if [r,c] <> [extop(y,1),extop(y,2)] then
	    return( FALSE )
	end_if;
	x := extop(x,3); 
	y := extop(y,3);
	t := R::equal;
	for i from 1 to r do
	    for j from 1 to c do
		if not t(x[i,j],y[i,j]) then return( FALSE ) end_if
	    end_for
	end_for;
	return( TRUE )
    end_proc,
#--
  gaussElim  --  Gaussian elimination

  Syntax:

  gaussElim(A)

  A  --  matrix

  Synopsis:

  Performs Gaussian elimination on A and returns a list with 
  a row echolon form of A, the rank of A, the determinant of 
  A if this is defined (otherwise FAIL will be the third 
  operand of the list) and a set of indices which represent 
  the column vectors that belong to the kernel of A.

  If R has a method "pivotSize" then the pivot element of
  smallest size will be choosen, whereby pivotSize must return
  a natural number representing 'the size' of an element.
--#
    "gaussElim" = (if R::hasProp( Cat::Field ) and 
		      R::hasProp( Ax::effectiveOperation("_invert") )
		   then
    # ordinary gaussian elimination #
    proc(x)
	option remember;
	local det, m, n, i, j, l, o, cl, k, t, ps, p, sdet,
	      R_invert, R_negate, Rone;
    begin
        m:= extop(x,1); n:= extop(x,2);
	x:= extop(x,3);

        ps:= R::pivotSize;
        # if ps = FAIL then no pivot strategy will be used, otherwise
          the pivot in respect to < will be choosen
        #
	R_negate:= R::_negate;
	R_invert:= R::_invert;
	Rone:= R::one;

	det:= [Rone];
	sdet:= 1;
        userinfo(1,"perform (ordinary) Gaussian elimination");

	cl:= {};
	j:= 1;
        for i from 1 to n do
            if j = m then break #i# end_if;

	    userinfo(2,"search for pivot");
            userinfo(3,"  in column ",i);
	    p:= 0; k:= -1;
            for l from j to m do
                if not Riszero(x[l,i]) then 
		    if ps = FAIL then
			p:= l;
			break #l#
		    elif k = -1 then
			k:= ps(x[l,i]);
			p:= l
		    elif (t:= ps(x[l,i])) < k then
			k:= t;
		  	p:= l
		    end_if
		end_if
            end_for;
            if p = 0 then
                userinfo(3,"no pivot found");
                cl:= cl union {i};
                det:= [Rzero];
                next # i #
            end_if;

            if p <> j then
                userinfo(3,"swap rows ",p," and ",j);
		x:= assign_elems( x,([j,o]=x[p,o],
                                     [p,o]=x[j,o]) $ o=i..n
		);
                sdet:= -sdet
            end_if;

            t:= x[j,i];
            userinfo(3,"pivot element is ",t);

	    det:= det . [t];
	    t:= R_invert(t);

	    # row operation #
	    for k from j+1 to m do
		x:= assign_elems( x,
		    ([k,o]=Rplus( x[k,o],
		      R_negate(Rmult(t,x[j,o],x[k,i])) )) $ o=i+1..n
		);
		x[k,i]:= Rzero
	    end_for;

	    # next row #
	    j:= j + 1
        end_for;
	
	while i < n+1 and Riszero(x[j,i]) do 
	    cl:= cl union {i}; i:= i + 1
	end_while;
	cl:= cl union {o $ o=i+1..n};

	if m = n then 
	    det:= Rmult( op(det),x[n,n] );
	    if sdet = -1 then [new(this,m,n,x), n-nops(cl), R_negate(det), cl]
	    else [new(this,m,n,x), n-nops(cl), det, cl]
 	    end_if
	else [new(this,m,n,x), n-nops(cl), FAIL, cl]
	end_if
    end_proc
	elif R::hasProp( Cat::IntegralDomain ) then
    # two-step fraction free gaussian elimination #
    proc(x)
	option remember;
	local n, m, i, j, l, f, f0, f1, f2, r, k, sig, p, l0, ns, o,
	      R_negate, R_divide, ps;
    begin
        R_negate := R::_negate; 
        R_divide := (if R::constructor=Dom::DistributedPolynomial
                   and R::quo<>FAIL then R::quo
		   # we know the division is always exact here,
                     and R::_divide may return FAIL for Dom::Numerical #
                   else R::_divide end_if);
	ps := R::pivotSize; 
	# if ps = FAIL then no pivot strategy will be used, otherwise
	  the pivot in respect to < will be choosen
	#

	m := extop(x,1);
	n := extop(x,2); 
	x := extop(x,3);
	sig := 1;
	f0 := R::one;
	k := 2; r := 2;
	userinfo(1,"perform 2-step fraction free Gaussian elimination");

	for k from 2 to 2*(n div 2) step 2 do
	    if r > m then break end_if;
	    userinfo(3,"row ",r,";  column ",k);

	    userinfo(2,"search for non-zero determinant");
	    unassign(l0, p);
	    for i from r-1 to m do
		f1 := R_negate( x[i,k] );
		f2 := x[i,k-1];
		for j from i+1 to m do 
		    f := Rplus(
			Rmult( f2,x[j,k] ),
                        Rmult( f1,x[j,k-1] )
		    );
		    if not Riszero(f) then
			if ps = FAIL then
			    p := [i,j,f];
			    break
			elif l0 = hold(l0) then
			    l0 := ps(f);
			    p := [i,j,f]
			elif (l:=ps(f)) < l0 then
			    l0 := l;
			    p := [i,j,f]
			end_if
		    end_if
		end_for;
		if j <= m and ps = FAIL then break end_if
	    end_for;
	    if p = hold(p) then
		userinfo(2,"  non non-zero determinant found");
		userinfo(3,"  in step ",k);
		# initialize single step #
		if r = 2 then f0 := R::one
		else f0:= x[r-2,k-2]
		end_if;
		break
	    end_if;
	    if p[1] <> r-1 then 
		i := p[1];
		userinfo(3,"swap rows ",r-1," and ",i);
		x:= assign_elems( x, ( [r-1,o]=x[i,o],
			               [i,o]=x[r-1,o]) $ o=k-1..n 
		);
		sig := -sig
	    end_if;
	    if p[2] <> r then
		j := p[2];
		userinfo(3,"swap rows ",j," and ",r);
		x:= assign_elems( x, ( [r,o]=x[j,o],
				       [j,o]=x[r,o] ) $ o=k-1..n
		);
		sig := -sig
	    end_if;
	    # x[r-1,k-1] = 0 ? #
            if Riszero( x[r-1,k-1] ) then
                userinfo(3,"swap rows ",r-1," and ",r);
		x:= assign_elems( x, ( [r-1,o]=x[r,o],
				       [r,o]=x[r-1,o] ) $ o=k-1..n
		);
		p[3] := R_negate( p[3] ); # |0 a\\c d| = -ac = -|c d\\0 a| #
                sig := -sig
            end_if;

	    f := R_divide(p[3],f0);
	    for i from r+1 to m do
		f1 := R_divide( Rplus( Rmult( x[r-1,k],x[i,k-1] ),
		      R_negate(Rmult( x[r-1,k-1],x[i,k] )) ),f0 );
		f2 := R_divide( Rplus( Rmult( x[r,k-1],x[i,k] ),
		      R_negate(Rmult( x[r,k],x[i,k-1] )) ),f0 );
		for j from k+1 to n do
		    x[i,j] := R_divide(
			Rplus(Rmult(f,x[i,j]),Rmult(f1,x[r,j]),
                              Rmult(f2,x[r-1,j]) ),f0
	            );
		end_for;
		x[i,k] := Rzero; x[i,k-1] := Rzero
	    end_for;
	    for j from k+1 to n do
		x[r,j] := R_divide( Rplus( Rmult( x[r-1,k-1], x[r,j] ),
			  R_negate(Rmult( x[r-1,j],x[r,k-1] )) ),f0 )
	    end_for;
	    x[r,k-1] := Rzero; x[r,k] := f;
	    # new divisor #
	    f0 := f; 
	    # next step #
	    r := r + 2
	end_for;
	r:= r - 2;

        # ns -- indices representing the colvects that belong to the kernel #
	ns:= {};

	if n mod 2 = 1 or r <= n then
	    userinfo(1,"perform single-step algorithm");
	    f1 := k - 1; 
	    r := r + 1;
	    for k from f1 to n do
		if r > m then 
		    break 
		end_if;
		userinfo(3,"row ",r,";  column ",k);
		userinfo(2,"searching for pivot element");
                p := m+1; unassign(l0);
                for i from r to m do
                    if not Riszero(x[i,k]) then
			if ps = FAIL then p := i; break
			elif l0 = hold(l0) then l0 := ps(x[i,k]); p := i
			elif (l:=ps(x[i,k])) < l0 then l0 := l; p := i 
			end_if
                    end_if
                end_for;
		if p <= m then
		    if r <> p then 
			userinfo(3,"swap rows ",r," and ",p);
			x:= assign_elems( x,( [r,l]=x[p,l],
					      [p,l]=x[r,l] ) $ hold(l)=k..n
			);
			sig := -sig
		    end_if;
		    for i from r+1 to m do
			f2:= R_negate( x[i,k] );
			for j from k+1 to n do
			    x[i,j] := R_divide( Rplus( Rmult( x[r,k],x[i,j] ),
				      Rmult( x[r,j],f2 ) ),f0 )
			end_for;
			x[i,k] := Rzero
		    end_for;
		    f0 := x[r,k]; 
		    r := r + 1
		else
		    ns:= ns union {k}
		end_if
	    end_for;
	    r := r - 1
	end_if;
	if n = m then
	    if r = m then if sig = -1 then f0 := R_negate(f0) end_if
	    else f0 := Rzero
	    end_if
	else f0 := FAIL
	end_if;

	ns:= ns union {i $ hold(i)=k..n };  # k is already increased by 1 #
	[new(this,m,n,x),r,f0,ns]
    end_proc
end_if ),
#--
  transpose  --  compute the transpose of a matrix 
--#
    "transpose" = proc(x)
	local r, c, i, j, t;
    begin
	r := extop( x,1 ); c := extop( x,2 ); x := extop( x,3 );
	t := array( 1..c,1..r );
	((t[j,i] := index_val(x,i,j)) $ j=1..c) $ i=1..r;

	new( this,c,r,t )
    end_proc,
#--
  nonZeros  --  return the number of non-zero elements of a matrix 
--#
    "nonZeros" = fun(
	nops( select([op(extop(args(1),3))],_not@Riszero) )
    ),
#--
  stackMatrix  --  concat two matrices vertically 
--#
    "stackMatrix" = proc(a,b)
	local rb, r, c, t, j, i;
    begin
	if args(0) = 1 then 
	    return( a )
	elif args(0) > 2 then
            return(
                this::stackMatrix(a,this::stackMatrix(b,args(3..args(0))))
            )
        end_if;

	r := extop( a,1 ); c := extop( a,2 ); a := extop( a,3 );
	rb := extop( b,1 ); b := extop( b,3 );
	t := array(1..r+rb,1..c);

        (( t[i,j] := index_val(a,i,j) ) $ i=1..r;
         ( t[r+i,j] := index_val(b,i,j) ) $ i=1..rb) $ j=1..c;

	new( this,r+rb,c,t )
    end_proc,
#--
  concatMatrix  --  append two matrices horizontally 
--#
    "concatMatrix" = proc(a,b)
	local cb, r, c, t, i, j;
    begin
	if args(0) = 1 then
	    return( a )
	elif args(0) > 2 then
            return(
                this::concatMatrix(a,this::concatMatrix(b,args(3..args(0))))
            )
        end_if;

	r := extop( a,1 ); c := extop( a,2 ); a := extop( a,3 );
	cb := extop( b,2 ); b := extop( b,3 );
	t := array(1..r,1..c+cb);

        (( t[i,j] := index_val(a,i,j) ) $ j=1..c;
         ( t[i,c+j] := index_val(b,i,j) ) $ j=1..cb ) $ i=1..r;

	new( this,r,c+cb,t )
    end_proc,
#--
  setRow  --  replace a row of a matrix by a new one 
--#
    "setRow" = proc(x,p,row)
	local c, j, pp;
    begin
	row := extop( row,3 );
	c := extop( x,2 );
	pp:= (p-1)*c;
	extsubsop( x,3=subsop(extop(x,3),
	    pp+j=index_val(row,1,j) $ j=1..c,hold(Unsimplified)
        ))
end_proc,
#--
  setCol  --  replace a column of a matrix by a new one 
--#
    "setCol" = proc(x,p,col)
	local c, i;
    begin
	c := extop(x,2);
	col := extop( col,3 );
	extsubsop( x,3=subsop(
	    extop(x,3),p+i*c = index_val(col,i+1,1) $ i=0..extop(x,1)-1
	    ,hold(Unsimplified)
	))
    end_proc,
#--
  row  --  extract a row of a matrix 
--#
    "row" = proc(x,p)
	local c, j;
    begin
	c := extop( x,2 ); x := extop( x,3 );
	new( this,1,c,array( 1..1,1..c,[[index_val(x,p,j) $ j=1..c]] ) )
    end_proc,
#--
  col  --  extract a column of a matrix 
--#
    "col" = proc(x,p)
	local r, i;
    begin
	r:= extop( x,1 );x := extop( x,3 );
	new( this,r,1,array( 1..r,1..1,[ [index_val(x,i,p)] $ i=1..r ] ) )
    end_proc,
#--
  delRow  --  delete a row of a matrix 
--#
    "delRow" = proc(x,p)
	local r, c, t, j, i;
    begin
	r := extop( x,1 ); c := extop( x,2 );
	if r = 1 then return( NIL ) end_if;
	x := extop( x,3 );

	t := array( 1..r-1,1..c );
	( ( t[i,j] := index_val(x,i,j) ) $ j=1..c ) $ i=1..p-1;
	( ( t[i-1,j] := index_val(x,i,j) ) $ j=1..c ) $ i=p+1..r;

	new( this,r-1,c,t )
    end_proc,
#--
  delCol  --  delete a column of a matrix 
--#
    "delCol" = proc(x,p)
	local r, c, t, i, j;
    begin
	r := extop( x,1 ); c := extop( x,2 );
	if c = 1 then return( NIL ) end_if;
	x := extop( x,3 );

	t := array( 1..r,1..c-1 );
	( ( t[i,j] := index_val(x,i,j) ) $ i=1..r ) $ j=1..p-1;
	( ( t[i,j-1] := index_val(x,i,j) ) $ i=1..r ) $ j=p+1..c;

	new( this,r,c-1,t )
    end_proc,
#--
  swapRow  --  swap two rows of a matrix 
--#
    "swapRow" = proc(x,k,l,b)
	local a, d, j;
    begin
	if k = l then return( x ) end_if;

	a := extop( x,3 );
 	if args(0) = 3 then
            a := assign_elems(a, ([[l,j],index_val(a,k,j)],
                                  [[k,j],index_val(a,l,j)]) $ j=1..extop(x,2)) ;
	else
            a := assign_elems(a, ([[l,j],index_val(a,k,j)],
                                  [[k,j],index_val(a,l,j)]) $ j=op(b,1)..op(b,2)) ;
	end_if;

	extsubsop( x,3=a )
    end_proc,
#--
  swapCol  --  swap two columns of a matrix 
--#
    "swapCol" = proc(x,k,l,b)
	local a, d, i;
    begin
	if k = l then return( x ) end_if;
	a := extop( x,3 );

	if args(0) = 3 then
            a := assign_elems(a, ([[i,l], index_val(a,i,k)],
                                  [[i,k], index_val(a,i,l)]) $ i=1..extop(x,1)) ;
	else
            a := assign_elems(a, ([[i,l], index_val(a,i,k)], 
                                  [[i,k], index_val(a,i,l)]) $ i=op(b,1)..op(b,2)) ;
	end_if;

	extsubsop( x,3=a )
    end_proc,

#--
  assign_elems  --  overloads the function 'assign_elems'
--#
"assign_elems" = 
    fun(
        extsubsop( args(1),3=assign_elems( extop(args(1),3),args(2..args(0)) ) )
    ),

#--
  TeX  --  'TeX' for matrices 
--#
    "TeX" = fun( generate::TeX(extop(args(1),3)) ),
#--
  map  --  'map' for matrices 
--#
    "map" = fun( 
	extsubsop(args(1),3=map(extop(args(1),3),args(2..args(0))))
    ),
#--
  subs  --  'subs' for matrices 
--#
    "subs" = fun( extsubsop(args(1),3=
	subs(extop(args(1),3),args(2..args(0)))
    )),
#--
  subsex  --  'subsex' for matrices 
--#
    "subsex" = fun( extsubsop(args(1),3=
	subsex(extop(args(1),3),args(2..args(0)))
    )),
#--
  has  --  the function has for matrices
--#
    "has" = fun( has(extop(args(1),3),args(2)) ),
#--
  length  --  returns the lenght of a matrix 

  The length of a matrix is defined by the length of its
  3rd operand, i.e. the length of an array.
--#
    "length" = fun( length(extop(args(1),3)) ),
#--
  conjugate  --  compute the complex conjugate of a matrix 
--#
    "conjugate" = (if R::conjugate <> FAIL then 
	fun( extsubsop(args(1),3=map(extop(args(1),3),R::conjugate)) )
    end_if),
#--
  randomDimen 
--#
    "randomDimen" = [10,10],
#--
  random  --  create random matrices

  Syntax:

  random()

  Synopsis:

  Returns a random matrix whereby the dimension is randomly choosen
  in the interval [1..this::randomDimen[1],1..this::randomDimen[2]]
  The elements of the random matrix are created by the method
  "random" of R. If such a method does not exist, then FAIL is 
  returned.
--#
    "random" = (if R::random <> FAIL then 
    proc()
	local m, n, j, i;
    begin
	m := (random() mod this::randomDimen[1]) + 1;
	n := (random() mod this::randomDimen[2]) + 1;
	new( this,m,n,array( 1..m,1..n,
	    [[R::random() $ j=1..n] $ i=1..m]
	))
    end_proc
    end_if),
#--
  tr  --  compute the trace of a square matrix

  Syntax:

  tr(A)

  A  --  matrix

  Synopsis:

  A trace of a square matrix A is defined to be the sum of the
  diagonal elements of A.
--#
    "tr" = proc(x)
	local n, i;
    begin
	n := extop(x,1);
        if n <> extop(x,2) then
            error("expecting a square matrix")
        end_if;
	x := extop(x,3);
	Rplus( Rzero,x[i,i] $ i=1..n )
    end_proc,
#--
  norm()  --  compute the norm of a matrix

  Syntax: 

  norm(A)
  norm(A,Frobenius)
  norm(v,k)

  A                    --  matrix
  v                    --  vector
  k                    --  positive integer
  Frobenius, Infinity  --  identifier

  Synopsis:

  norm(A) and norm(A,Infinity) compute the infinity norm of a matrix.
  That is the maximum row sum, where the row sum is the sum of norms
  of each element.
  The infinity norm of a vector is the maximum norm of all
  elements.

  norm(A,1) computes the 1-norm of a matrix which is the maximum sum 
  of the norm of the elements of each column.

  norm(A,Frobenius) computes the Frobenius norm of a matrix which is 
  the square root of the sum of the squares of the norm of each element.

  norm(v,k) computes the k-norm of a vector for k >= 2.
  This norm is defined to be the kth root of the sum of the norm of each 
  element raised to the kth power.
  For the 2-norm the function 'linalg::scalarProduct' will be used
  if she is defined. Otherwise the Euclidian scalar product of v will
  be returned, which is defined by

	v[1]*conjugate(v[1]) + v[2]*conjugate(v[2]) + ...

  If the coeffRing of v does not provide a method "conjugate"
  (which must return the complex conjugate of an element) then 
  the sum

            v[1]*v[1] + v[2]*v[2] + ...

  will be returned.


  This method returns FAIL if the result is not longer an element
  of the coefficient ring of A and v, respectively.

  For all cases except of the 2-norm, the coefficient ring of the
  matrix has to provide the method 'norm', which result for any element 
  has to be a number. FAIL will be returned if such a method not exists.
--#
    "norm" = proc(x)
	local i, j, c, r, n, Rnorm, nt;
    begin
        if args(0) < 1 or args(0) > 2 then
            error("wrong no of args")
        end_if;

        if args(0) = 2 then nt := args(2) end_if;
	if nt <> 2 then
	    Rnorm := R::norm;
	    if Rnorm = FAIL then
		userinfo(1,"the coefficient domain has not the method \"norm\"");
	        return( FAIL )
	    end_if
	end_if;

	r := extop(x,1); c := extop(x,2);
	case nt
	of 1 do # compute the 1-norm #
	    x := this::transpose(x);
            n := c; c := r; r := n

	of hold(Infinity) do
	of hold(nt) do # compute the infinity norm #
	    x := extop(x,3);
	    n := max(_plus(Rzero,Rnorm(x[i,j]) $ j=1..c)
		$ i=1..r
	    );
	    break

	of hold(Frobenius) do # compute the Frobenius norm #
	    x := extop(x,3);
            n := Rzero;
            (n := _plus(n,_mult(Rnorm(x[i,j]) $ 2) $ j=1..c)) $ i=1..r;
            n := n^(1/2);
            break
	of 2 do # compute the 2-norm of vector x #
	    if min(r,c) <> 1 then
                error("invalid matrix norm (expecting a vector)")
            end_if;

	    nt := linalg::scalarProduct;
	    if nt <> FAIL then
	        n := linalg::scalarProduct(x,x)^(1/2)
	    else
		x := extop(x,3);
		nt := R::conjugate;
		if nt <> FAIL then
		    if r = 1 then
        	        n := Rplus( Rmult(x[1,i],nt(x[1,i])) $ i=1..c )
		    else
			n := Rplus( Rmult(x[i,1],nt(x[i,1])) $ i=1..r )
		    end_if
    		elif r = 1 then
		    n := Rplus( Rmult(x[1,i],x[1,i]) $ i=1..c )
		else # c = 1 #
		    n := Rplus( Rmult(x[i,1],x[i,1]) $ i=1..r )
    		end_if;
		n := n^(1/2)
	    end_if;
	    break

	otherwise # compute the k-norm of vector x #
	    if not testtype( nt,Type::PosInt ) then
		error("invalid matrix norm")
	    end_if;
	    if r = 1 then
		x := extop(x,3);
	        n := _plus( Rnorm(x[1,i])^nt $ i=1..c )^(1/nt)
	    elif c = 1 then
		x := extop(x,3);
		n := _plus( Rnorm(x[i,1])^nt $ i=1..r )^(1/nt)
	    else
		error("invalid matrix norm (expecting a vector)")
            end_if
	end_case;

	R::coerce(n)
    end_proc,
#--
	exp  --  compute the exponential of a matrix

	Syntax:

        exp(A [,x])

        A:  square matrix
        x:  undeterminate

	Synopsis:

        exp(A,x) computes the matrix exponential defined by

            exp(A,x) = I + A*x + 1/2!*A^2*x^2 + ...

        If the second argument is not given then 1 is set for x.
        When the eigenvalues of A can not be computed then an 
	error message will be returned.

        When the sum of the algebraic multiplicities of the
        eigenvalues of A is not equal to the dimension of A
        then FAIL will be returned.

	NOTE: This method uses the linalg functions
	      linalg::jordanForm and linalg::eigenValues.
--#
    "exp" = (if R::hasProp( Cat::Field ) then proc(A)
	local ev, l, k, i, j, r, x, J, P, t;
    begin
	if args(0) < 1 or args(0) > 2 then
	    error("wrong no of args")
	end_if;
	t := extop(A,1);
	if t <> extop(A,2) then
	    error("expecting a square matrix")
	end_if;
	if args(0) = 2 then
	    if poly(args(2)) = FAIL then
		error("invalid indeterminate")
	    end_if;
	    x := args(2);
	    if domtype(x) <> R then
		x := R::coerce( x );
		if x = FAIL then
		    userinfo(1,"unable to compute over the coefficient domain");
                    return( FAIL )
                end_if
	    end_if
	else
	    x := R::one
	end_if;

	userinfo(2,"compute the Jordan form");
	ev:= linalg::eigenValues( A,hold(Multiple) );
	if ev = FAIL then
	    userinfo(1,"can't compute eigenvalues");
	    return( FAIL )
	end_if;

	l:= linalg::jordanForm( A,hold(All) );
	if l = FAIL then
	    userinfo(1,"function linalg::jordanForm fails");
	    return( FAIL )
	else
	    J := l[1]; P:= l[2]
	end_if;

	# exp(A) = exp(J)*exp(P) #
	#     [ 0, 0, 0 ]                 [   1   , 0, 0]
          P = [ 1, 0, 0 ]   ->   exp(P) = [   x   , 1, 0]
              [ 0, 1, 0 ]                 [1/2 x^2, x, 1]
        #
        k := 0;
        for l from 1 to nops(ev) do
            r := op(ev,[l,2]);
            for i from k+1 to k+r do
                J[i,i] := R::coerce(exp(expr( Rmult(x,op(ev,[l,1])) )));
		if J[i,i] = FAIL then
		    userinfo(1,"unable to compute over the coefficient domain");
		    return( FAIL )
	 	end_if;
                if i > 1 and not Riszero( J[i,i-1] ) then
		    t:= Rmult( J[i,i],J[i,i-1] );
                    (J[i,i-j] := Rmult( t,x^j/fact(j) ))
			$ j=1..i-k-1
                end_if
            end_for;
            k:= k + r
        end_for;

        A::_mult( P,J,P::_invert(P) )
    end_proc
end_if),
#--
	zip  --  overload the function zip for matrices
--#
"zip" = proc(a,b,f,d)
        local ia, ib, ic, ra, rb, rm, ca, cb, cm, i, j;
    begin
        ia := extop( a,3 ); ib := extop( b,3 );
        ra := extop( a,1 ); ca := extop( a,2 );
        rb := extop( b,1 ); cb := extop( b,2 );
        rm := min(ra,rb); cm := min(ca,cb);
        if args(0) = 4 then
            ic := array( 1..max(ra,rb),1..max(ca,cb) )
        else
            ic := array( 1..rm,1..cm )
        end_if;

        for i from 1 to rm do
            (ic[i,j] := f(ia[i,j],ib[i,j])) $ hold(j)=1..cm
        end_for;
        if args(0) = 4 then
            for i from 1 to cm do
                (ic[j,i] := f(ia[j,i],d)) $ hold(j)=(rm+1)..ra;
                (ic[j,i] := f(d,ib[j,i])) $ hold(j)=(rm+1)..rb
            end_for;

            for i from 1 to rm do
                (ic[i,j] := f(ia[i,j],d)) $ hold(j)=(cm+1)..ca;
                (ic[i,j] := f(d,ib[i,j])) $ hold(j)=(cm+1)..cb
            end_for;

            for i from rm+1 to max(ra,rb) do
                (ic[i,j] := f(d,d)) $ hold(j)=(cm+1)..max(ca,cb)
            end_for
        end_if;

        new( this,op(ic,[0,2,2]),op(ic,[0,3,2]),ic )
    end_proc,
#--
        expr2text  --  convert matrix to a string
--#
"expr2text" = proc(x)
    begin
	expr2text(this)."(".expr2text(extop(x,3)).")"
    end_proc,
#--
        op  --  overload the function op as it operates on arrays
--#
"op" = fun( op(extop(args(1),3),args(2..args(0))) ),

"subsop" = fun( 
	    extsubsop(args(1),3=
	        subsop(extop(args(1),3),args(2..args(0)))
	    )),

"nops" = fun( nops(extop(args(1),3)) ),

"expand"= fun( extsubsop(args(1),3=
		 map(extop(args(1),3), expand, args(2..args(0)))
	     )),

"simplify"= fun( extsubsop(args(1),3=
                 map(extop(args(1),3), simplify, args(2..args(0)))
             )),

"combine"= fun( extsubsop(args(1),3=
                 map(extop(args(1),3), combine, args(2..args(0)))
             )),

"rewrite"= fun( extsubsop(args(1),3=
                 map(extop(args(1),3), rewrite, args(2..args(0)))
             )),

"diff"= fun( extsubsop(args(1),3=
		map(extop(args(1),3), diff, args(2..args(0)))
	     ))

):

# end of file #
