#++
Dom::MonoidAlgebra(R,S) -- free algebra over the ring R with (finite) support
	in the ordered monoid S where one assumes that the elements of R and
	S commutes (otherwise see Dom::FreeOperatorAlgebra(R,S))

This domain is a specialization of Dom::FreeModule(R,S). Especially, it
inherits its representation. The only additional operation is the
multiplication of elements of the domain. Furthermore we have now a
bimodule.


Author: Werner M. Seiler, Uni Karlsruhe
Created: July 3rd, 1997
Last update: October 14th, 1997
++#

alias(REP(x)=extop(x,1),
      TYPE_ERR=error("wrong type of argument")):

Dom::MonoidAlgebra := DomainConstructor(

    Dom::MonoidAlgebra,

    [ R, S ],

    [ Rmult, Rcoerce, Riszero, Rone, Smult, Sone ],

    (if args(0)<>2 then error("wrong number of arguments") end_if;
     if (domtype(R)<>DOM_DOMAIN) or (domtype(S)<>DOM_DOMAIN) or
	(R::hasProp=FAIL) or (S::hasProp=FAIL) then
	 error("arguments must be domains");
     end_if;
     if R::hasProp(Cat::Ring) then
         Rmult := R::_mult;
	 Rcoerce := R::coerce;
         Riszero := R::iszero;
         Rone := R::one;
     else error("first argument must be a Cat::Ring") end_if;
     if S::hasProp(Cat::OrderedMonoid) then
	 Smult := S::_mult;
         Sone := S::one;
     elif S::hasProp(Cat::OrderedAbelianMonoid) then
	 Smult := S::_plus;
         Sone := S::zero;
     else error("second argument must be ordered monoid") end_if),

    Dom::FreeModule(R,S),

    [ (if R::hasProp(Cat::CommutativeRing) and
	  S::hasProp(Cat::AbelianMonoid) then Cat::CommutativeRing end_if),
	Cat::Module(R), Cat::FiniteMonoidRing(R,S) ],

    [ (if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if),
      (if R::hasProp(Ax::normalRep) then Ax::normalRep end_if) ],


    "one" = this::generate1(Rone,Sone),


    "_mult" = proc()
    local res,x1,x2,i;
    begin
	case args(0)
	of 0 do return(this::one);
	of 1 do return(args(1));
	of 2 do
	    if testargs() then
	        x1 := Rcoerce(args(1));
		if x1=FAIL then
                    x1 := this::convert(args(1));
                    if x1=FAIL then TYPE_ERR end_if;
		else return(this::smult(x1,this::convert(args(2)))) end_if;
	        x2 := Rcoerce(args(2));
                if x2=FAIL then
                    x2 := this::convert(args(2));
                    if x2=FAIL then TYPE_ERR end_if;
                else return(this::smult(x2,x1)) end_if;
	    else
		x1 := args(1);
		x2 := args(2);
	    end_if;
	    if domtype(x1)=R or domtype(x1)=DOM_INT then
		return(this::smult(x1,x2))
	    elif domtype(x2)=R then return(this::smult(x2,x1)) end_if;
            res := [];
            for i in REP(x1) do
                res := res . map(REP(x2),fun([Rmult(args(1)[1],args(2)[1]),
            		                      Smult(args(1)[2],args(2)[2])]),i);
            end_for;
            return(this::generate(this::normalizeRep(res)));
	otherwise
	    i := args(0) div 2;
	    this::_mult(this::_mult(args(1..i)),
			this::_mult(args(i+1..args(0))));
	end_case;
    end_proc

):

unalias(REP,TYPE_ERR):
