#++
Dom::MonoidOperatorAlgebra(R,S,f) -- free algebra over the ring R with support
                in the ordered monoid S. The function f defines the
                commutator of an operator in S and an element of R (if they
                commute, use Dom::MonoidAlgebra(R,S)).

This domain is a specialization of Dom::FreeModule(R,S). Especially, it
inherits its representation. There are two additional operations: the
multiplication of elements of the domain and the application of an element
of the operator algebra to an element of the coefficient ring.


Author: Werner M. Seiler, Uni Karlsruhe
Created: July 6th, 1997
Last update: October 8th, 1997
++#

alias(REP(x)=extop(x,1)):


Dom::MonoidOperatorAlgebra := DomainConstructor(

    Dom::MonoidOperatorAlgebra,

    [ R, S, sigma ],

    [ Rmult, Rplus, Riszero, Rone, Smult, Sone, Scall ],

    (if args(0)<2 or args(0)>3 then error("wrong number of arguments") end_if;
     if (domtype(R)<>DOM_DOMAIN) or (domtype(S)<>DOM_DOMAIN) or
	(R::hasProp=FAIL) or (S::hasProp=FAIL) then
	 error("first two arguments must be domains");
     end_if;
     if R::hasProp(Cat::Ring) then
         Rmult := R::_mult;
	 Rplus := R::_plus;
         Riszero := R::iszero;
         Rone := R::one;
     else error("first argument must be a Cat::Ring") end_if;
     if S::hasProp(Cat::OrderedMonoid) then
	 Smult := S::_mult;
         Sone := S::one;
     elif S::hasProp(Cat::OrderedAbelianMonoid) then
	 Smult := S::_plus;
         Sone := S::zero;
     else error("second argument must be ordered monoid") end_if;
     Scall := S::func_call;
     if args(0)=2 then sigma := Scall end_if;
     if sigma=FAIL then error("commutator not defined") end_if;
    ),

    Dom::FreeModule(R,S),

    [ Cat::FiniteMonoidRing(R,S) ],

    [ (if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if),
      (if R::hasProp(Ax::normalRep) then Ax::normalRep end_if) ],


    "one" = this::generate1(Rone,Sone),


    "_mult" = proc()
    local res,x1,x2,i;
    begin
	case args(0)
	of 0 do error("too few arguments");
	of 1 do return(args(1));
	of 2 do
	    x1 := args(1);
	    x2 := args(2);
	    if domtype(x1)=R then
                if Riszero(x1) then return(this::zero) end_if;
		return(this::mapcoeffs(x2,fun(Rmult(x1,args(1)))));
	    elif domtype(x2)=R then
                if Riszero(x2) then return(this::zero) end_if;
		return(this::mapcoeffs(x1,Rmult,x2));
	    else
		res := [];
		for i in REP(x1) do
		    res := res . map(REP(x2),
                        fun(op([[Rmult(i[1],args(1)[1]),Smult(i[2],args(1)[2])],
                                [Rmult(i[1],sigma(i[2],args(1)[1])),
                                 args(1)[2]]])));
		end_for;
		return(this::generate(this::normalizeRep(res)));
	    end_if;
	otherwise
	    i := args(0) div 2;
	    this::_mult(this::_mult(args(1..i)),
			this::_mult(args(i+1..args(0))));
	end_case;
    end_proc,


    "applyMonomial" = (if Scall<>FAIL then
	fun(Rmult(args(1)[1],Scall(args(1)[2],args(2))))
    else NIL end_if),


    "func_call" = (if Scall<>FAIL then
	fun(Rplus(op(map(REP(args(1)),this::applyMonomial,args(2)))))
    else NIL end_if)

):

unalias(REP):
