
#++ ---------------- MultiPoly.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 23 June      1996
Last Update : 12 September 1997

Description:
This domain represents multivariate polynomials in some identifiers over 
commutative coefficient rings.
Most of the functions for DOM_POLY are available.

Call:

 Dom::MultivariatePolynomial(Vars, < Ring >, < Order >) 
 where
   Vars       => list of indeterminates of type DOM_INDET; 
   opt: Ring  => arbitrary commutative ring of the Domains-package;
                 otherwise Dom::ExpressionField(normal) is choosen
                 automatically
   opt: Order => set a term order. Currently is possible: 
                 LexOrder, DegreeOrder, DegInvLexOrder; 
                 otherwise LexOrder is choosen automatically

All Entries:
D, Dpoly, Factor, Rep, SPolynomial, TeX, TeXCoeff, TeXTerm, TeXindet, _div\
ide, _invert, _mult, _negate, _plus, _power, _subtract, allAxioms, allCate\
gories, allEntries, allSuperDomains, associates, characteristic, coeff, co\
effRing, coerce, content, convert, convert_to, decompose, degree, degreeve\
c, diff, dimension, divides, equal, equiv, evalp, expr, factor, gcd, getAx\
ioms, getCategories, getSuperDomain, getTermOrder, groebner, hasProp, inde\
ts, info, intmult, irreducible, isUnit, isone, iszero, lcm, lcoeff, lmonom\
ial, lterm, mainvar, mapcoeffs, monic, mult, multcoeffs, negate, new, norm\
, normalForm, nterms, nthcoeff, nthmonomial, nthterm, one, plus, poly, pri\
mpart, print, printElem, printMonomial, printTerm, random, reductum, resul\
tant, ringmult, sign, solve, sqrfree, subs, subsex, tcoeff, testtype, unde\
finedEntries, unitNormal, unitNormalRep, whichEntry, zero


Example:
>> MP := Dom::MultivariatePolynomial([x,y,z], Dom::Integer);

     Dom::MultivariatePolynomial([x, y, z], Dom::Integer, LexOrder)

>> mp1:= MP(3*x-2);
                                  3 x - 2

>> mp2:= MP(3*x^2-2*x+1);
                                 2
                              3 x  - 2 x + 1

>> mp3:= MP(5*x^3*y^2*z-z^3+5*y^2+1+x);

                          3  2            2    3
                       5 x  y  z + x + 5 y  - z  + 1

>> 5*mp3+2*mp1^2*3*mp2;

          4       3  2          3        2               2      3
     162 x  + 25 x  y  z - 324 x  + 270 x  - 115 x + 25 y  - 5 z  + 29


++#

# Dependencies:

  read("PolyExplicit.mu"):  #

loadlib("domains"):


sysassign(Dom::MultivariatePolynomial, DomainConstructor(

  # name: #            Dom::MultivariatePolynomial,
  # parameter: #       [ Vars, R, Order ],
  # static value: #    [ ],
  # initialization: #  (if iszero(args(0)) or args(0) > 3 then 
                          error("wrong no of args")
                        end_if;
                        if domtype(Vars) <> DOM_LIST then
                          error("illegal indeterminates")
                        end_if;
                        if not(testtype(Vars, Type::ListOf(DOM_IDENT))) then
                          error("not a variable")
                        end_if;
                        if R::hasProp(Dom::BaseDomain) = FAIL then
                           R := Dom::ExpressionField(normal) 
                         elif not(R::hasProp(Cat::CommutativeRing) and
                                  R::hasProp(Ax::normalRep) or
                                  R::hasProp(
                                   Dom::ExpressionField(id, iszero@normal)))
                             then error("illegal coefficient ring")
                        end_if;
                        if contains({LexOrder, DegreeOrder, DegInvLexOrder},
                                     args(args(0))) 
                           then Order := args(args(0))                     
                           else Order := LexOrder end_if; ),
  # super-domain: #     Dom::PolynomialExplicit(Vars, R, Order),
  # categories: #       [ (if nops(Vars) = 1 then
                             Cat::UnivariatePolynomialCat(R)
                           else Cat::PolynomialCat(R) end_if) ],
  # axioms: #           [ Ax::normalRep, 
                        (if R::hasProp(Ax::canonicalRep) 
                           then Ax::canonicalRep end_if) ]
  # entries: #

) ):


