# domain of multisets (sets with repetitions).

  Available methods: contains, convert, expr, has, intersect, nops, op, print, union

  Example:

>> m:=Dom::Multiset(a,a,b): n:=Dom::Multiset(a,c,c):
>> m,n;

                    {[a, 2], [b, 1]}, {[a, 1], [c, 2]}

>> m union n;

                         {[b, 1], [a, 2], [c, 2]}

>> m intersect n;

                                 {[a, 1]}
#

Dom::Multiset := DomainConstructor(
# name #
    Dom::Multiset,
# arguments #
    [ ],
# local variables #
    [ ],
# initialisation #
    (if args(0) <> 0 then error("wrong no of args") end_if),
# super-domains #
    Dom::BaseDomain,
# categories #
    [ Cat::SetCat ],
# axioms #
    [ ],
# methods #
    # contains(m,x) returns 0 if x is not in m, otherwise the index j
      such that op(extop(m,j),1)=x #
    "contains" = fun(contains(map([extop(args(1))],op,1),args(2))),

    "normal" = fun(new(this,op({extop(args(1))}))),

    "convert" = proc(x) local s,i,j,n; begin
       if type(x)=this then x
       else
          x:=sort([args()]); n:=nops(x); s:=null();
          i:=1; while i<=n do
             for j from i+1 to n do if x[i]<>x[j] then break end_if end_for;
             s:=s,[x[i],j-i];
             i:=j
          end_while;
          new(this,op({s}))
       end_if
     end_proc,

    "expand" = fun(map(extop(args(1)),fun(op(args(1),1)$op(args(1),2)))),
    "expr" = fun(hold(Dom::Multiset)(this::expand(args(1)))),

    "has" = fun(has([this::op(args(1))],args(2))),

    "_intersect" = proc(m) local s,l,p,j,news,ss;
    begin
       case args(0)
       of 0 do return(_intersect())
       of 1 do return(m)
       otherwise
          s:=[extop(m)]; l:=[args(2..args(0))];
          while l<>[] do # intersect s with l[1] #
             if s=[] then break end_if;
             news:=null(); ss:=map(s,op,1);
             for p in [extop(l[1])] do # p = [x,i] #
                if (j:=contains(ss,p[1]))<>0 then
                   news:=news,[p[1],min(p[2],s[j][2])]
                end_if
             end_for;
             s:=[news]; l[1]:=NIL
          end_while;
          this::normal(new(this,op(s)))
       end_case
    end_proc,

    "nops" = extnops,
    "op" = extop,
    "_index" = extop,

    "print" = fun({extop(args(1))}),

    "_union" = proc(m) local s,l,p,j,ss;
    begin
       case args(0)
       of 0 do return(_union())
       of 1 do return(m)
       otherwise
          s:=[extop(m)]; l:=[args(2..args(0))];
          while l<>[] do # union of s with l[1] #
             ss:=map(s,op,1);
             for p in [extop(l[1])] do # p = [x,i] #
                if (j:=contains(ss,p[1]))<>0 then
                   s:=subsop(s,j=[p[1],max(p[2],s[j][2])])
                else s:=append(s,p); ss:=append(ss,p[1])
                end_if
             end_for;
             l[1]:=NIL
          end_while;
          this::normal(new(this,op(s)))
       end_case
    end_proc,

    "select" = fun(new(this,select(extop(args(1)),args(2..args(0))))),

    # the method map maps f on the elements of the multiset #
    "map" = proc(x,f) local F,R; begin
       this::normal(new(this,
	map(extop(x),subs(fun([F(args(1)[1],R),args(1)[2]]),F=f,R=args(3..args(0))))))
    end_proc,

    "_subtract" = proc(s,t) local u,j,l; begin
       if type(t)=this then t:=[expand(t)] end_if;
       for u in t do
          if (j:=contains(s,u))<>0 then
             s:=extsubsop(s,j=[u,max(0,op(extop(s,j),2)-1)])
          end_if
       end_for;
       new(this,op(map([extop(s)],fun((if args(1)[2]=0 then null() else
	args(1) end_if)))))
    end_proc
)():
