# domains of Pade approximations. Author: Paul Zimmermann, 23 Feb 1996

  Available methods: 
  new(f,x) - creates a Pade approximant

  Example:

>> Dom::Pade(exp(-x),x,3);

                                  - x + 2
                                  -------
                                   x + 2
#

Dom::Pade := DomainConstructor(
# name #
    Dom::Pade,
# arguments #
    [ ],
# local variables #
    [ ],
# initialisation #
    (if args(0) <> 0 then error("wrong no of args") end_if),
# super-domains #
    Dom::BaseDomain,
# categories #
    [ Cat::Field ],
# axioms #
    [ ],
# methods #
    "characteristic" = 0,
    
    "new" = proc(f) local ldeg,ord,m,n,p,q,a,b,x,i,j,sola,solb,unk;
    begin
       if type(f)<>Puiseux then
          if args(0)<2 then error("point of expansion not given") end_if;
          f:=series(f,args(2..args(0)));
          if type(f)<>Puiseux then return(FAIL) end_if
       end_if;
       ldeg:=f::ldegree(f); ord:=f::order(f)-ldeg-1;
       n:=ord div 2; m:=ord-n; x:=f::indet(f);
       f:=extop(f,4); b[0]:=1;
       # (f0+f1*x+...+f(m+n)*x^(m+n))*(1+b1*x+...+bn*x^n) = (a0+a1*x+...+am*x^m) #
       # the following code avoids the call to linsolve #
       for i from 0 to m do a[i]:=_plus(f[j+1]*b[i-j]$j=max(0,i-n)..i) end_for;
       unk:={b[j]$j=1..n}; solb:=null();
       for i from m+1 to m+n do
          eq:=subs(_plus(f[j+1]*b[i-j]$j=max(0,i-n)..i),solb);
          p:=indets(eq,PolyExpr) intersect unk;
          if p={} then if eq<>0 then return(FAIL) end_if
          else
             p:=op(p,1); unk:=unk minus {p};
             eq:=p=-coeff(eq,[p],0)/coeff(eq,[p],1);
             solb:=solb,eq
          end_if
       end_for;
      p:=_plus(a[i]*x^i$hold(i)=0..m); q:=_plus(1,b[i]*x^i$hold(i)=1..n);
      # f:=x^ldeg*eval(subs(p/q,solb)); #
      f:=x^ldeg*subs(p/q,solb,Simplified);
       new(this,normal(f));
     end_proc,

    "print" = proc(r) begin extop(r,1) end_proc,
  
    "convert" = proc(r) begin
       case domtype(r)
       of this do return(r)
       of Puiseux do return(new(this,r))
       otherwise FAIL
       end_case
    end_proc,

    "expr" = proc(r) begin extop(r,1) end_proc
)():
