#  $Date: 1998/01/08 10:42:37 $      $Author: stefanw $     $Revision: 1.7 $      #

# stefanw, 12.02.1996 #

#++

Dom::PermutationGroup(n) - symmetric group of order n, i.e. the group of bijective mappings of { 1..n} onto itself

n - integer >= 1

An element is represented as a domain element with operands:

0 - domain Dom::PermutationGroup(n)
1 - a list [a_1, ... ,a_n ] of n integers, representing the permutation that maps
    i to a_i for each i between 1 and n 

++#

Dom::PermutationGroup:= DomainConstructor(
    Dom::PermutationGroup,
    [ n ],
    [ ],
    (if args(0) <> 1 then error("wrong no of args") end_if;
     if domtype(n) <> DOM_INT then
	 error("argument must be integer")
     end_if;
     if n < 1 then
	 error("argument must be positive")
     end_if),
    Dom::BaseDomain,
    [ Cat::Group ],
    [ Ax::canonicalRep ],

    "size" = fact(n),

    "one" = new(this, [ $ 1..n ]),


    "convert" = fun((
	case domtype(args(1))
	of this do 
		return(args(1));
	of DOM_ARRAY do 
		return(this::convert([op(args(1))]));
	of DOM_LIST do
	 if {op(map(args(1),domtype)) } <> { DOM_INT } then
		error("List must consist of integers");
	 elif has(map(args(1),fun( bool(args(1)<= n and args(1) > 0 ) )),
	   FALSE) then
		error("List contains integers outside the range");
	 elif nops(args(1)) <> n then
		error("List has incorrect length");
	 elif nops({ op(args(1)) } ) <> n then
		error("No permutation (not bijective)");
	 else
	 	return(new(this, args(1)));
	 end_if;
	end_case;
        FAIL
    )),

    "convert_to" = fun((
	case args(2)
	of DOM_LIST do	
		 return(extop(args(1),1));
	of this do return(args(1));
	end_case;
	FAIL
    )),

    "cycles" = proc(a)
	# returns a list of cycles c_1,..,c_k such that a=c_1*..*c_k, each
 	  cycle is written as a list #
	
	local l,i,k,cyc,result;
	# l[i] is TRUE iff i is already involved in a cycle #

	begin
	l:=[ FALSE $ n ];
	cyc:=[];
	result:=[];
	for i from 1 to n do	
		if not l[i] then
			k:=i; 
			repeat
				l[k]:=TRUE;
				cyc:=append(cyc,k);
				k:=op(extop(a,1),k);				
			until i=k end_repeat;
			result:=append(result,cyc);
			cyc:=[];
		end_if;
	end_for;
	result
	end_proc,

    "expr" = fun( extop(args(1),1) ),  # gibt nicht DOM_EXPR, sondern
					 DOM_LIST zurueck # 
					 

    "print" = fun( extop(args(1),1) ),



    "_mult" = fun((
    	if args(0) = 2 then
	    if domtype(args(2)) <> this or 
	       domtype(args(1)) <> this then
	    	    FAIL
	    else
	        new(this, [ op(extop(args(1),1),op(extop(args(2),1),i)) $ hold(i)=1..n])
	    end_if
    	elif args(0) = 1 then
    	    args(1)
    	else
    	    _mult(args(1..(args(0) div 2)));
    	    _mult(args(((args(0) div 2) + 1)..args(0)));
	    if (domtype(%2))::_mult <> FAIL then (domtype(%2))::_mult(%2, %1)
	    elif (domtype(%1))::_mult <> FAIL then (domtype(%1))::_mult(%2, %1)
	    else _mult(%2, %1)
	    end_if
    	end_if
    )),

 #    "_power" : inherited from category constructor #



    "_invert" = proc(x)
	local result,i;

	begin
	 for i from 1 to n do 
		result[op(extop(x,1),i)]:=i;
	 end_for;
 	 new(this,[ result[i] $ hold(i)=1..n ])
	end_proc,

#    "_divide" : defined in Cat::Group  # 

	"order" = proc(a)
	local c,i;

	begin
	 c:=this::cycles(a);
	 ilcm(nops(op(c,i)) $ hold(i)=1..nops(c))
	end_proc,
	 		

    "func_call" = proc(sigma:this, l)
	local cp;

	begin
	l:=context(l);
	if type(l)=DOM_INT then
		if l<1 or l>n then
			error("Permutation is only defined for arguments between 1 and ".
				expr2text(n));
		else
			return(op(extop(sigma,1),l))
		end_if
	elif type(l)=DOM_LIST then
		if nops(l)<>n then
			error("List must have ".expr2text(n)." elements") 
		end_if;
		cp:=l;
		map([$1..n], fun((cp[op(extop(sigma,1),args(1))]:=l[args(1)])) );
		return (cp) 		
	else
		error("Second argument has wrong type")
	end_if
	end_proc,


   "inversions"=proc(sigma:this)
   local i,result;

   begin
   sigma:=extop(sigma,1);
   result:=0;
   for i from 1 to n-1 do
      result:=result+nops(select([$i+1..n], fun(sigma[args(1)]<sigma[i])))
   end_for;
   end_proc,
         

   "sign"=proc(sigma:this)

	begin
	map(this::cycles(sigma),nops);
	map(%,_plus, -1);
	(-1)^_plus(op(%))
	end_proc,


    "TeX" = fun(
	expr2text(extop(args(1),1))
    ),

    "random" = proc()
	local beenthere,rnd,result,i,a;

	begin
	 beenthere:= [ FALSE $ n ];
         rnd:=random(n);    # random generator for 0..n-1 #
	 result:=[];
	 for i from 1 to n do
		repeat a:=rnd()+1 until beenthere[a]=FALSE end_repeat;
		result:=append(result,a);
		beenthere[a]:=TRUE;
	 end_for;
	 new(this,result);
	end_proc		
		
):

# end of file #
