
#  $Date: 1998/01/21 18:54:28 $      $Author: stefanw $     $Revision: 1.3 $  #    

# stefanw, 20.03.1997 #

#++

Dom::Quaternion - 
the skew field of quaternions

An element is represented as a domain element with operands:

0 - domain Dom::Quaternion
1 - a list [a,b,c,d] of four real numbers, representing a+bI+cJ+dK 

[a,b,c,d] may also be regarded as a 2x2 - matrix 

	(  a+bI   -c-dI )
	(	        )
	(  c-dI    a-bI )

over the field of complex numbers.      

++#

Dom::Quaternion:= DomainConstructor(
    Dom::Quaternion,
    [],
    [],
    (
	userinfo(4, "Creating domain Dom::Quaternion")
    ),
    Dom::BaseDomain,
    [ Cat::SkewField ],
    [ Ax::canonicalRep ],

    "size" = infinity,

    "zero" = new(this,[0,0,0,0]),

    "one" = new(this, [1,0,0,0]),

    "characteristic" = 0,

    "convert" = fun((
	case domtype(args(1))
	of this do 
		return(args(1))
	of DOM_INT do
	of DOM_FLOAT do
	of DOM_RAT do
		return(new(this,[args(1),0,0,0]))
	of DOM_COMPLEX do
		return(new(this,[Re(args(1)),Im(args(1)),0,0]))
	of DOM_LIST do
	 if nops(args(1)) <> 4 then 
		error("List of length four expected");
	 elif {op(map(args(1), is, Type::RealNum)) } <> {TRUE} then
		error("List must consist of real numbers");
	 else
	 	return(new(this, args(1)));
	 end_if
	of DOM_IDENT do
		if args(1)=PI then 
			return(new(this,[PI,0,0,0]))
		elif args(1)=hold(i) then
			return(new(this,[0,1,0,0]))
		elif args(1)=hold(J) or args(1)=hold(j) then
			return(new(this,[0,0,1,0]))
		elif args(1)=hold(K) or args(1)=hold(k) then 
			return(new(this,[0,0,0,1]))
		else
			return(FAIL)
		end_if
	of DOM_EXPR do
		case type(args(1)) 
		 of "_plus" do
		      return(this::_plus(map(op(args(1)),this::convert)))
		 of "_mult" do
		      return(this::_mult(map(op(args(1)),this::convert)))
		 of "_power" do
		      return(this::_power(this::convert(op(args(1),1)),
			     op(args(1),2)))
		end_case;
		if is(args(1),Type::RealNum)=TRUE then 
		      return(new(this,[args(1),0,0,0]))
		end_if;
		return(FAIL)
	of Dom::Matrix(Dom::Complex) do
		if normal(conjugate(args(1)[1,1])<>args(1)[2,2]) or
		  normal(-conjugate(args(1)[1,2])<>args(1)[2,1]) then
			return(FAIL)
		else
		   return(new(this,map([Re((args(1)[1,1])), Im((args(1)[1,1])),
			 Re((args(1)[2,1])), -Im((args(1)[2,1]))],expr)))
		end_if		
	end_case;
        FAIL
    )),

    "convert_to" = fun((
	case args(2)
	of DOM_LIST do	
		 return(extop(args(1),1));
	of this do return(args(1))
	of DOM_COMPLEX do
		if op(extop(args(1),1),3..4)<>(0,0) then
			return(FAIL)
		else
			return(op(extop(args(1),1),1)+I*op(extop(args(1),1),2))
		end_if
	of DOM_FLOAT do
		if op(extop(args(1),1),2..4)<>(0,0,0) then
			return(FAIL)
		else
			return(float(op(extop(args(1),1),1)))
		end_if	
        of Dom::Matrix(Dom::Complex) do
		return(this::matrixform(args(1)))
	end_case;
	FAIL
    )),

    
	

    "expr" = fun( (
		extop(args(1),1); 
		select([%[1],%[2]*I,%[3]*hold(J),
			%[4]*hold(K)],not iszero);
		case nops(%)
			of 0 do return(0) 
			of 1 do return(op(%))
		end_case;
		_plus(op(%))
		 )),  
					 

    "print" = fun( this::expr(args(1)) ),


    "_plus"=fun((
	 map([args()],this::coerce);
	 if contains(%,FAIL)=0 then
		new(this, [_plus(op(op((map(map(%, extop, 1),op,i))
			 $hold(i)=1..4 ,j) )) $hold(j)=1..4])
	 else
		FAIL
	 end_if
	)),

     "_negate"= fun((
	 new(this,[-op(extop(args(1),1),i) $hold(i)=1..4])
	)),

     "_subtract"= fun(new(this,[op(extop(args(1),1),i)-op(extop(args(2),1),i) 
		  $hold(i)=1..4])),

    "_mult" = fun((
	 if args(0)=1 then
		args(1)
	 elif args(0)=2 then
		if domtype(args(2)) <> this then
	                if is(args(2), Type::RealNum) then
        	            this::scalarmult(args(2),args(1))
                	else
                    	    (domtype(args(2)))::_mult(args())
                	end_if
		elif domtype(args(1)) <> this then
                	if is(args(1), Type::RealNum) then
        	            this::scalarmult(args())
                	else
                    		FAIL
                	end_if
		else
			extop(args(1),1);
			extop(args(2),1);
			new(this,[
				%2[1]*%[1]-%2[2]*%[2]-%2[3]*%[3]-%2[4]*%[4],
				%2[1]*%[2]+%2[2]*%[1]+%2[3]*%[4]-%2[4]*%[3],
				%2[1]*%[3]-%2[2]*%[4]+%2[3]*%[1]+%2[4]*%[2],
				%2[1]*%[4]+%2[2]*%[3]-%2[3]*%[2]+%2[4]*%[1]
				])
		end_if
	 else
		_mult(args(1..(args(0) div 2)));
                _mult(args(((args(0) div 2) + 1)..args(0)));
                if (domtype(%2))::_mult <> FAIL then 
			(domtype(%2))::_mult(%2, %1)
            	elif (domtype(%1))::_mult <> FAIL then 
			(domtype(%1))::_mult(%2, %1)
            	else 
			_mult(%2, %1)
            	end_if
         end_if 
    )),

    "scalarmult"=fun(new(this,[args(1)*op(extop(args(2),1),i) $hold(i)=1..4])),

    "scalarprod"=fun( _plus(extop(args(1),1)[i]*extop(args(2),1)[i]
		 $hold(i)=1..4)), 


    "conjugate"=fun(new(this,[extop(args(1),1)[1],-extop(args(1),1)[i] 
			$hold(i)=2..4])),

    "matrixform"=fun(Dom::Matrix(Dom::Complex)(
	 [[extop(args(1),1)[1]+extop(args(1),1)[2]*I,
	   -extop(args(1),1)[3]-extop(args(1),1)[4]*I],
	  [extop(args(1),1)[3]-extop(args(1),1)[4]*I,
	   extop(args(1),1)[1]-extop(args(1),1)[2]*I]])
	),   	 

    "_invert" = fun( 	
	 this::convert(this::matrixform(args(1))^(-1))	        
	 ),

    "Re"= fun(extop(args(1),1)[1]),

    "Im"= fun([extop(args(1),1)[j] $hold(j)=2..4]),


#    "_divide" : defined in Cat::Group  # 

    "norm"= fun(_plus(extop(args(1),1)[i]^2 $hold(i)=1..4)),
	 		

    "TeX" = fun(
	expr2text(expr(args(1)))
    ),

    "random" = fun
	(
		new(this,[random(), random(), random(), random()])
	)
			
		
)():

# end of file #
