# $Date: 1997/03/05 12:08:29 $ $Author: olli $ $Revision: 1.20 $ #
#++
Dom::Rational -- the domain of rational numbers

A rational number is represented by an expression of type DOM_INT
or DOM_RAT.

Methods:-
convert(x)         - returns the value of x converted to Dom::Rational
numer(x)	   - returns the numerator of x
denom(x)	   - returns the denominator of x
retract(x)	   - returns x if it is an integer and FAIL otherwise
testtype(x,D)      - returns TRUE if x is of type DOM_INT or DOM_RAT
min(x,...)	   - returns the minimal argument
max(x,...)	   - returns the maximal argument
++#

Dom::Rational:= DomainConstructor(

    Dom::Rational,
    [ ],
    [ ],
    NIL,
    Dom::Numerical,
    [ Cat::QuotientField(Dom::Integer) , Cat::DifferentialRing, Cat::OrderedSet ],
    [ Ax::canonicalRep, Ax::systemRep, Ax::canonicalOrder,
      Ax::effectiveOperation("_divide"), Ax::effectiveOperation("_mult"),
      Ax::effectiveOperation("_invert")
    ],

    "testtype" = fun((
	if contains({DOM_INT, DOM_RAT}, domtype(args(1))) then
	    TRUE
	else
	    FAIL
	end_if
    )),

    "numer" = fun((
	if domtype(args(1)) = DOM_RAT then op(args(1),1) else args(1) end_if
    )),

    "denom" = fun((
	if domtype(args(1)) = DOM_RAT then op(args(1),2) else 1 end_if
    )),

    "retract" = fun((
	if domtype(args(1)) = DOM_INT then args(1) else FAIL end_if
    )),

    "convert" = fun((
       if contains({DOM_INT, DOM_RAT}, domtype(args(1))) then
          args(1)
       else
          FAIL
       end_if
    )),

    "convert_to" = fun((
	if not contains({DOM_INT, DOM_RAT}, domtype(args(1))) then FAIL else
	    case args(2)
	    of Dom::Rational do
	    of Dom::Numerical do
		args(1);
		break;
	    of DOM_RAT do
		if domtype(args(1)) = DOM_RAT then args(1) else FAIL end_if;
		break;
	    of Dom::Integer do
	    of DOM_INT do
		if domtype(args(1)) = DOM_INT then args(1) else FAIL end_if;
		break;
	    of DOM_FLOAT do
	    of Dom::Float do
		float(args(1));
		break;
	    otherwise FAIL;
	    end_case
	end_if
    )),

    "min" = min,

    "max" = max,
    
    "random" = fun((
        (Dom::Integer)::random();
        if % = 0 then 0 else (Dom::Integer)::random() / % end_if
    ))

)():

