
#++ ---------------- UniPoly.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 29 May       1996
Last Update : 12 September 1997

Description:
This domain represents univariate polynomials in some identifier over 
commutative coefficient rings.
Most of the functions for DOM_POLY are available.

Call:

 Dom::UnivariatePolynomial(Var, < Ring >) 
 where
   Var        => indeterminate of type DOM_INDET; 
   opt: Ring  => arbitrary commutative ring of the Domains-package;
                 otherwise Dom::ExpressionField(normal) is choosen
                 automatically

Methods: 

 - used Parameter:
    ++ a : this Domain
    ++ n : DOM_INT

 - vectorize(a, < n >)
    ++ returns a in it's vectorized form, i.e. *all* coefficients of a in
    ++ increasing order. When n is given (and long enough) a list of n entries
    ++ maybe filled up with zeros is returned.  

All Entries:
D, Dpoly, Factor, Rep, SPolynomial, TeX, TeXCoeff, TeXTerm, TeXindet, _div\
ide, _invert, _mult, _negate, _plus, _power, _subtract, allAxioms, allCate\
gories, allEntries, allSuperDomains, associates, characteristic, coeff, co\
effRing, coerce, content, convert, convert_to, decompose, degree, degreeve\
c, diff, dimension, divide, divides, equal, equiv, euclideanDegree, evalp,\
 expr, factor, gcd, gcdex, getAxioms, getCategories, getSuperDomain, getTe\
rmOrder, groebner, hasProp, idealGenerator, indets, info, intmult, irreduc\
ible, isUnit, isone, iszero, lcm, lcoeff, lmonomial, lterm, mainvar, mapco\
effs, monic, mult, multcoeffs, negate, new, norm, normalForm, nterms, nthc\
oeff, nthmonomial, nthterm, one, pdivide, plus, poly, pquo, prem, primpart\
, print, printElem, printMonomial, printTerm, quo, random, reductum, rem, \
resultant, ringmult, sign, solve, sqrfree, subs, subsex, tcoeff, testtype,\
 undefinedEntries, unitNormal, unitNormalRep, vectorize, whichEntry, zero


Example:
>> UP := Dom::UnivariatePolynomial(x, Dom::Integer);
>> up1:= UP(3*x-2);

++#

# Dependencies:

  PolyExplicit.mu  #

alias(
      rep(x) = extop(args(x),1)
      ):

sysassign(Dom::UnivariatePolynomial, DomainConstructor(

  # name: #            Dom::UnivariatePolynomial,
  # parameter: #       [ var, R ],
  # static value: #    [ ],
  # initialization: #  (if iszero(args(0)) or args(0) > 2 then 
                          error("wrong no of args")
                        end_if;
                        if domtype(var) <> DOM_IDENT then
                          error("not a variable")
                        end_if;
                        if R::hasProp(Dom::BaseDomain) = FAIL then
                           R := Dom::ExpressionField(normal) 
                         elif not(R::hasProp(Cat::CommutativeRing) and
                                  R::hasProp(Ax::normalRep) or
                                  R::hasProp(
                                   Dom::ExpressionField(id, iszero@normal)))
                             then error("illegal coefficient ring")
                        end_if; ),
  # super-domain: #     Dom::PolynomialExplicit([var], R),
  # categories: #       [ Cat::UnivariatePolynomialCat(R) ],
  # axioms: #           [ Ax::normalRep, 
                        (if R::hasProp(Ax::canonicalRep) 
                           then Ax::canonicalRep end_if) ],
  # entries: #

  "coeff" = 
        proc(p, x, n)
          local cl;
          begin
            if this::iszero(p) then return( R::zero ) end_if;
            cl := [];
            case args(0)
              of 1 do while not this::iszero(p) do 
                        cl := append(cl, this::lcoeff(p));
                        p  := this::reductum(p)
                      end_while;
                      return( op(cl) );
              otherwise  
                R::new( coeff(rep(1), args(2..args(0))) )
            end_case
          end_proc,

  "euclideanDegree" = this::degree,

  "vectorize" = proc(p, n)
     local i, v;
     begin
       if args(0) = 1 then n := this::degree(p) + 1
         elif this::degree(p) > n-1 then
           error("too short vector to represent this polynomial") end_if;
       v := [];
       for i from 0 to n-1 do 
         v := append(v, this::coeff(p, i))
       end_for;
     end_proc

) ):

unalias( rep ):

