# $Date $ $Author $ $Revision $ #



# faclib::findfactor --- detection of a bivariate factor
  by root refinement #


faclib::refroots:=proc(f,y,x,h,d)

# linear lifting #
# finds a in F[x,y] such that f(a,y) is 
  in the ideal generated by h and y^{d+1} #

# f(x,0) must be squarefree #
# f must be monic with respect to x #

#   f: polynomial #
#   y: main variable #
#   x: other variable #
#   h: irreducible factor of f(x,0) #
#   d: end of iteration #

local F,f0,fx,s,k,a,S,i,j,l,increment;

begin
F:=op(f,3);
f0:=evalp(f,y=0);
fx:=D([1],f);
a:=poly(divide(poly(x,[x],F),h,Rem),[x,y],F);
userinfo(4,"a0=",op(a,1));
s:=op(gcdex(evalp(fx,y=0),h),2);

S:=array(1..degree(f,x),0..d );



for k from 1 to degree(f,x) do
	S[k,0]:=divide(poly(x^k,[x],F),h,Rem);
end_for;

for k from 1 to d do
	 
	S[1,k]:=poly(0,[x],F);
	for j from 2 to degree(f,x) do  
		S[j,k]:=divide(_plus(S[j-1,l]*S[1,k-l] $ hold(l)=0..k),h,Rem);
        end_for; 

	a:=a - poly(y^k,[x,y],F)* poly(divide( (_plus (_plus 
		( multcoeffs(S[i,k-j], coeff(coeff(f,y,j),x,i)) 
		$ hold(i)=1..degree(f,x) ) 
		 $ hold(j)=0..k )  + poly(coeff(coeff(f,y,k),x,0),[x],F))*s,
		h,hold(Rem)) ,[x,y],F); 
	
	userinfo(4,"a".expr2text(k)." = ".expr2text(expr(a)));
	S[1,k]:=coeff(a,y,k);
        increment:=S[1,k];
	for j from 2 to degree(f,x) do
                # modify S[j,k] such that it becomes correct for
		  the new value of S[1,k] and S[j-1,k] #
		increment:=divide(increment*S[1,0]+S[j-1,0]*S[1,k],h,hold(Rem));
                S[j,k]:=S[j,k]+increment;
        end_for; 
end_for;
userinfo(2,"Root found:",a);
return(a);
end_proc:


faclib::trunc:=proc(p,y,k)
local i;
# reduces p modulo y^k #

begin
if iszero(p) then return(p); end_if;
if nterms(p)=1 then 
        if degree(p,y)<k then return(p) else return(p-p);  end_if;
end_if;
_plus(select(nthmonomial(p, i) $ hold(i)=1..nterms(p),
        fun(degree(args(1), y) < k)));

end_proc:



faclib::horner:=proc(f,g,d,y)
# computes f(g) modulo y^d #
local x,result,i;

begin
x:=op(f,[2,1]);
result:=poly(coeff(f,x,degree(f,x)),op(f,2..3));
for i from degree(f,x)-1 downto 0 do
	result:=faclib::trunc(result*g,y,d);
	result:=result+poly(coeff(f,x,i),op(f,2..3));		
end_for;
result
end_proc:



faclib::quadref:=proc(f,y,x,h,d)

# quadratic refinement #

# f(x,0) must be squarefree #
# f must be monic with respect to x #

#   f: polynomial #
#   y: main variable #
#   x: other variable #
#   h: irreducible factor of f(x,0) #
#   d: end of iteration #

local F,f0,fx,s,k,a;


begin
F:=op(f,3);
f0:=coeff(f,y,0);
fx:=D([1],f);
a:=poly(divide(poly(x,[x],F),h,Rem),[x,y],F);
userinfo(3,"a0=",op(a,1));
s:=poly(op(gcdex(coeff(fx,y,0),h),2),[x,y],F);
k:=1;



while k <= d do

        a:=a - faclib::trunc(s* faclib::horner(f,a,min(2*k,d+1),y),
			y,min(2*k,d+1));
	s:=multcoeffs(s,2) - faclib::trunc(faclib::horner(fx,a,min(2*k,d+1),y)
			*s^2,y,min(2*k,d+1));    
	s:=poly(s,[y],Poly([x],F));
	s:=mapcoeffs(s,divide,h,Rem);
	s:=poly(s,[x,y],F); 
	a:=poly(a,[y],Poly([x],F));
	a:=mapcoeffs(a,divide,h,Rem);
	a:=poly(a,[x,y],F);
        userinfo(3,"a[",k,"]=",expr(a));     
	k:=2*k; 
end_while;

return(a);
end_proc:




faclib::coeffmatrix:=proc(b,n,m,d,y,h,F)
local M,bpow,i,ii,j,k,A,dh;
 
# gives back the coeff. matrix, each row ordered as
  follows: u00 u01 u02 .. u10 .. umd with ujk corresp. to x^j y^k #

begin
dh:=degree(h);
M:=Dom::Matrix(F);
bpow:=array(0..m);
bpow[0]:=poly(1,op(b,2..3));
for i from 1 to m do
	bpow[i]:=mapcoeffs(faclib::truncate(bpow[i-1]*b,d+1),divide,h,Rem);
end_for;
A:=M(dh*(d+1),(n+1)*(m+1));
userinfo(2,"Constructing a ".expr2text(dh*(d+1))."x".expr2text((n+1)*(m+1)).
	" matrix over ".expr2text(F));
for i from 0 to d do
    for ii from 0 to dh-1 do
	for j from 0 to m do
		for k from 0 to min(i,n) do
			 A[i*dh+ii+1,j*(n+1)+k+1]:=coeff(coeff(bpow[j],y,i-k),ii);
		end_for;
	end_for;
    end_for;
end_for;
return(A);

end_proc:



faclib::maxnonzero:=proc(x)
# Input: a vector x #
# Output: the maximal i for which x[i] is nonzero #

begin
max(op(select([i $hold(i)=1..nops(args(1))], fun(not iszero(x[args(1)])))))
end_proc:

faclib::reconvert:=proc(v,m,n,y,x,F)
local s,j;
# converts a row of the matrix into a polynomial #
begin
poly([([v[j*(n+1)+s+1],[j,s]] $hold(j)=0..m) $hold(s)=0..n],[x,y],F)
end_proc:

faclib::findfactor:=proc(f)
local g,h,y,x,m,n,mnz,F,c,pt,rnd,A,b,l,st,i,j,d,q,ok;

# first argument: f \in F[x,y], where F is a finite field #
# if a second argument is given, count operations #
# Output: an irreducible factor of f #                 

begin	
if args(0)=2 then
	if args(2)<>hold(Quadratic) then
		error("Illegal option");
	end_if;
end_if;
if has(op(f,3), IntMod) then
	f:=subsop(f,3=Dom::IntegerMod(op(f,[3,1])));
end_if;


F:=op(f,3);

if iszero(f) then
	 return(F(0))
elif degree(f)=0 then
	 return(coeff(f))
end_if;

q:=(if F::constructor=Dom::GaloisField then
		F::size
    else 
		F::characteristic
    end_if
   );




x:=op(f,[2,1]);
y:=op(f,[2,2]);

st:=time();
l:=content(poly(f,[x],Poly([y],F)));
if degree(l)>0 then 
	userinfo(2,"Factor found: polynomial was not primitive");
	return(poly(l,[x,y],F));
end_if;
l:=content(poly(f,[y],Poly([x],F)));
if degree(l)>0 then
	userinfo(2,"Factor found: polynomial was not primitive");
	return(poly(l,[x,y],F));
end_if;	
userinfo(1,"Time for checking primitivity:".expr2text(time()-st));





# choice of main variable: derivative with respect to main variable
  must be nonzero  #

st:=time();
if iszero((l:=D([1],f))) then
	if iszero(D([2],f)) then
		userinfo(2,"Polynomial is a ".expr2text(F::characteristic).
			"th-power");
		return(faclib::recursmap(f,F::characteristic,q))
	end_if;
	x:=op(f,[2,2]);
	y:=op(f,[2,1]);
elif degree((l:=gcd(l,f)))>0 then
        userinfo(2,"Non-trivial gcd of polynomial and partial derivative");	 	return(l)
	#else x:=op(f,[2,1]);	y:=op(f,[2,2]) as it has been set before #
end_if;
userinfo(1,"Time for checking squarefreeness:".expr2text(time()-st));


# now f is squarefree #
# choice of main variable: derivative with respect to main variable
  must be nonzero  #

userinfo(4,"Main variable chosen: ".expr2text(x));




# Choice of evaluation point #
# try zero first #

st:=time();

pt:=(op(f,3))::zero;

h:=evalp(f,y=pt);
l:=sqrfree(h);
if nops(l)=3 and op(l,3)=1 and degree(h)=degree(f,x) then
	userinfo(2,"Lucky case: zero is suitable evaluation point"); 
else
	userinfo(2, "Zero is not a suitable evaluation point");
	userinfo(2,"Looking for evaluation point");
	
	rnd:=(op(f,3))::random;

	if q > 3*degree(f,x)*degree(f,y) then
		# there must be lots of good evaluation points #
		repeat
			pt:=rnd();
			h:=evalp(f,y=pt);
			l:=sqrfree(h);
		until nops(l)=3 and op(l,3)=1 and degree(h)=degree(f,x)
		end_repeat
	else
		# deterministic algorithm #
		ok:=FALSE;
		for i from 1 to 10 do # some tries first #
                	pt:=rnd();
			h:=evalp(f,y=pt);
                	
                	if degree(h)=degree(f,x) then
                        	l:=sqrfree(h);
                        	if nops(l)=3 and op(l,3)=1 then
                                	ok:=TRUE;
                                	userinfo(3,"Found evaluation point by\
					    random choice in ".expr2text(i).
					    "th try");
                                	break;
                        	end_if;
                	end_if; 
                	userinfo(5,"No success in ".expr2text(i)."th try");
        	end_for;
		if ok = FALSE then
			l:=poly(f,[x],Dom::DistributedPolynomial([y],F));
			h:=poly(resultant(l,Dpoly(l))*lcoeff(l),[y],F);
			j:=0;
			repeat
				j:=j+1;
				g:=poly(y^(q^j)-y,[y],F);
				g:=divide(g,gcd(g,h),hold(Exact));
				# only suitable evaluation points remain #
			until degree(g)>0 end_repeat;
			if j>1 then 
			# extend field #
				userinfo(2,"Working over field extension ".
					"of degree ".expr2text(j));
				l:=faclib::findfactor(poly(f,Dom::GaloisField(F,j)));
				st:=time();
				# Multiply l by its conjugates #
				g:=l; # store original l #
				h:=poly(1,op(l,2..3)); 
					# product of all conjugates so far # 
				repeat
					h:=h*l;
					l:=mapcoeffs(l,_power,q);
				until l=g end_repeat;
				
				userinfo(1,"Time for multiplying by conjugates:"
					.expr2text(time()-st));	
				return(subsop(h,3=F)); 
			elif degree(g)> q div 4 then
			# many good points, find one by random choice #
				repeat
					pt:=rnd()
				until iszero(evalp(g,y=pt)) end_repeat;
                                userinfo(3,"Found evaluation point by random
                                         choice");
			else
                                h:=faclib::eqdegr(poly(g,[y],F),1,1);
                                pt:=-coeff(h,0);
                                userinfo(3,"Choosing evaluation point by \
					factorization");
                                # simply choose the first
                                 factor found (anyone would be o.k) #					
			end_if;			
		h:=evalp(f,y=pt);  
		end_if	
	end_if
end_if;
userinfo(1,"Time for choosing an evaluation point:".expr2text(time()-st));
userinfo(3,"Evaluation point used",pt);

st:=time();
h:=factor(h);
userinfo(1,"Time for univariate factorization:".expr2text(time()-st));

if nops(h)=3 then return(f); end_if;
h:=op(h,2);
userinfo(3,"Univariate factor used:",h);


if not iszero(pt) then
	f:=poly(subs(expr(f),y=y+expr(pt)),op(f,2..3));
	userinfo(1,"Factoring ",f,"instead");
else
	userinfo(1,"No substitution necessary");
end_if;

m:=degree(f,x);
n:=degree(f,y);
d:=ceil((2*degree(f,x)*degree(f,y))/degree(h));
userinfo(2,"Lifting up to degree ".expr2text(d));
st:=time();
if args(0)=1 then
	b:=faclib::refroots(f,y,x,h,d)
else
	b:=faclib::quadref(f,y,x,h,d)
end_if;
userinfo(1,"Time for Hensel lifting:".expr2text(time()-st));

userinfo(3,"Computing coefficient matrix"); 

b:=mapcoeffs(poly(b,[y],Poly([x],op(b,3))),divide,h,hold(Rem));

# reduce modulo h #

A:=faclib::coeffmatrix(b,n,m,d,y,h,F);
userinfo(3,"Solving linear system");
st:=time();
l:=linalg::nullSpace(A);
userinfo(1,"Time for solving big linear system:".expr2text(time()-st));


# now l is a basis of the nullspace of A, given  as a list #
# the first entries correspond to the members of J_d of
  lowest degree (with respect to x) #
if nops(l)=0 then error("Only trivial solution?!"); end_if;

c:=map(l,faclib::maxnonzero);
mnz:=min(op(c));
userinfo(4,"Irreducible factor has at least degree ".
	expr2text((mnz-1) div(n+1)).
	" with respect to ".expr2text(x));

l:=l[contains(c,mnz)];


st:=time();

l:=subsop(faclib::reconvert(l,m,n,y,x,F),3=op(f,3));
userinfo(1,"Time for reconversion:".
		expr2text(time()-st));


if iszero(pt) then
	l
else
	# back substitution #
	poly(subs(expr(l),y=y-expr(pt)),op(f,2..3));
end_if

end_proc:	

