# $ Date $ $ Author $ $ Revision $ #

# faclib::quadfactor --- bivariate factorization
  by quadratic lifting #

faclib::quadfactor:=proc(f)
local i,j,l,F,xx,factlist,result;
begin
 if type(f)<>DOM_POLY then error("not a polynomial") end_if;
 xx:=op(f,2);
 if nops(xx)<>2 then error("bivariate polynomial expected"); end_if;
 F:=op(f,3);
   if F::hasProp(Cat::Field)<>TRUE then
         error("coefficient ring should be a field")
   end_if;
l:=faclib::pre_factor(f);
result:=[op(l,1)];
for i from 2 to nops(l) step 2 do
	if nops(op(l,[i,2]))=1 then 
		factlist:=factor(op(l,i));
	else
		factlist:=faclib::bivfactor(op(l,i));
	end_if;
	result[1]:=result[1]*factlist[1];
	for j from 3 to nops(factlist) step 2 do
		factlist[j]:=factlist[j]*op(l,i+1);
	end_for;
	if op(l,[i,2])<>xx then
		for j from 2 to nops(factlist) step 2 do
			factlist[j]:=subsop(factlist[j],2=xx);
		end_for;
	end_if;
	result:=result.[op(factlist,2..nops(factlist))];
end_for;
result
end_proc:


# faclib::bivfactor - factor primitive bivariate polynomial #

faclib::bivfactor:=proc(f)
local l,F,factlist,result,i,j,st;

begin
 F:=op(f,3);  
  st:=time();
  l:=sqrfree(f);
  userinfo(2,"Time for squarefree factorization:".expr2text(time()-st));
   result:=[op(l,1)];
   for i from 2 to nops(l) step 2 do
        factlist:=faclib::bihensel(op(l,i));
        result:=result.[(op(factlist,j),op(l,i+1)) $hold(j)=1..nops(factlist) ];
    end_for;
    result
end_proc:


# If f \in F[x,y] , f primitive and squarefree,
  faclib::bihensel computes a factorization of f #
# by quadratic Hensel lifting #
 
faclib::bihensel:=proc(f)
local ff,f0,fi,R,X,Y,df,q,a,e,r,s,t,u,fprod,g,h,i,j,k,evpoint,
	l,iindets,F,d,lc,st,factorsleft,setleft,factlist,sg;

begin
iindets:=op(f,2);

# as f is primitive, if it has degree =1 with respect to one variable,
  it must be irreducible #
if has([degree(f,iindets[i]) $hold(i)=1..2],1) then
	 userinfo(2,"Irreducible by degree argument");
	 return([f]);
end_if;

# exchange variables if f has zero derivative #
# as f is squarefree, this cannot happen for both variables #

X:=op(iindets,1);
Y:=op(iindets,2);
if iszero(D([1],f)) then 
	iindets[1]:=Y;
	iindets[2]:=X;
	g:=subsop(f,2=iindets);
	return(map(faclib::multihensel(g),
		fun(subsop(args(1),2=[X,Y]))));
end_if;	
userinfo(2,"Using ".expr2text(X)." as main variable");
F:=op(f,3);
q:=(if F::constructor=Dom::GaloisField then
         F::size
    elif F::constructor=Dom::IntegerMod then
         F::characteristic
    else
	 infinity
    end_if
    );

if degree((ff:=gcd(f,D([1],f)))) > 0 then
	return(ff.faclib::bihensel(divide(f,ff,hold(Exact))))
end_if;

R:=Dom::DistributedPolynomial([Y],F);
# try whether zero is good evaluation point #
f0:=evalp(f,Y=F::zero);
ff:=sqrfree(f0);
if degree(f0)=degree(f,X) and nops(ff)=3 and op(ff,3)=1 then
	
	evpoint:=F::zero; 
	userinfo(2,"Zero is suitable evaluation point");
	ff:=poly(f,[X],R);
elif q > 3*degree(f,X) then
	st:=time();
	repeat
		evpoint:=F::random();
		f0:=evalp(f,Y=evpoint);
		ff:=sqrfree(f0);
	until degree(f0)=degree(f,X) and nops(ff)=3 and op(ff,3)=1 end_repeat;
	userinfo(2,"Time needed for choosing evaluation point:".expr2text(time()-st));

	ff:=poly(subs(expr(f),Y=Y+expr(evpoint)),[X],R);
else  # finite field #
	st:=time();
	a:=FALSE;   # evaluation point found ? #
        for i from 1 to 10 do # some tries first #
		evpoint:=F::random();
		f0:=evalp(f,Y=evpoint);
		if degree(f0)=degree(f,X) then
			ff:=sqrfree(f0);
			if nops(ff)=3 and op(ff,3)=1 then
				a:=TRUE;
				userinfo(3,"Found evaluation point by random
					choice in ".expr2text(i)."th try");
				userinfo(3,"Evaluation point is ".expr2text(
					 evpoint));
				break;
			end_if;
		end_if; 
	end_for;
	if a=FALSE then
		ff:=poly(f,[X],R);
	 	a:=lcoeff(ff);
		df:=Dpoly(ff);
	 	g:=resultant(ff,df);

		g:=g*a;

                # as y^q-y is the product of all linear polynomials,
                  (y^q-y)/g is the product
                   of all (y-c) where c runs through all suitable
                   evaluation points; they can be found by factoring #

                j:=0;
                repeat
                        j:=j+1;
                        h:=R(Y^(q^j)-Y);
                        h:=divide(h,gcd(g,h),hold(Exact));
                until degree(h)>0 end_repeat;

		if j>1 then
                # bad luck : there is no good evaluation point #
                # field has to be extended #
        
                        userinfo(2,"Field does not contain good evaluation point
");
                        userinfo(2,"Constructing extension of degree",j);
                
                        f:=poly(f,Dom::GaloisField(F,j));
                        h:=faclib::bihensel(f);
                        # multiply conjugates #
                        g:=[];
                        repeat
                                l:=poly(1,op(f,2..3));
                                userinfo(5,"Collecting conjugates of ".
                                        expr2text(h[1]));
                                repeat 
                                        l:=l*h[1];
                                        h[1]:=mapcoeffs(h[1],_power,q);
                                        i:=contains(h,h[1],2);  
                                        if i<>0 then unassign(h[i]) end_if;
                                until i=0 end_repeat;
                                unassign(h[1]);
                                g:=append(g,l); 
                        until h=[] end_repeat;
                        for i from 1 to nops(g) do
                                g[i]:=subsop(op(g,i),3=F);
                        end_for;
                        return(g)


        	else     
			userinfo(3,"Number of good evaluation points: "
				.expr2text(degree(h)));
			if degree(h) > q/4 then  
				# many good points, try random choice  #
				repeat 
					evpoint:=F::random()
				until evalp(h,Y=evpoint)=F::zero end_repeat;
				userinfo(3,"Found evaluation point by random
					 choice");
				userinfo(3,"Evaluation point is ".
					expr2text(evpoint));
			else    
                		h:=faclib::eqdegr(poly(h,[Y],F),1,1);
                		evpoint:=-coeff(h,0);
				userinfo(3,"Choosing evaluation point: "
				.expr2text(evpoint));
                		# simply choose the first
				 factor found (anyone would be o.k) #
			end_if;
		end_if;
	end_if;
	f0:=evalp(f,Y=evpoint);
	ff:=poly(subs(expr(f),Y=
		Y+expr(evpoint)
 		),	[X],R);

end_if;
lc:=lcoeff(ff);
st:=time();
fi:=factor(f0);
userinfo(2,"Time for univariate factorization:".expr2text(time()-st));
if nops(fi)=3 then 
	userinfo(2,"Even modular image is irreducible");
	return([f]); 
else
	userinfo(2,"Modular image has "
		.expr2text(nops(fi) div 2)." factors");
end_if;
t:=op(fi,1);
fi:=[op(fi,2*j) $hold(j)=1..nops(fi) div 2];
fi[1]:=multcoeffs(fi[1],t);
# find s1 ... sn #

# if u*(f1*..fk)+v*(f(k+1))=1 and s1*(f2* ...*fk)+ ... + sk*(f1*...*f(k-1))=1
  then set s(k+1):=u and multiply the other si by v #

st:=time();
fprod:=fi[1]*fi[2];
t:=gcdex(fi[2],fi[1]);  # sic! #
s:=[ op(t,2..3) ];
# loop invariant: fprod = f1 * ... * fj #
for j from 3 to nops(fi) do 
	t:=gcdex(fprod,fi[j]);
	for k from 1 to j-1 do
		s[k]:=s[k]*op(t,3);
	end_for;
	s:=append(s,op(t,2)); 
	fprod:=fprod*fi[j];
end_for;
 s:=zip(s,fi,fun(divide(args(1),args(2),hold(Rem)))); 

userinfo(2,"Time for determining the si: ".expr2text(time()-st));

# transform to polynomials over R #
for j from 1 to nops(fi) do 
	fi[j]:=poly(fi[j],[X],R);
	s[j]:=poly(s[j],[X],R);
end_for;


d:=max(map(coeff(ff),degree))+1;
userinfo(2,"Lifting up to order ".expr2text(d));
st:=time();
j:=1;
q:=(r:=(t:=(u:=(g:=[0 $nops(fi)]))));


# quadratic lifting #
repeat
	j:=min(j*2,d);
		userinfo(3,"Lifting to order ".expr2text(j));
		e:=mapcoeffs(ff-_mult(op(fi)),fun(faclib::truncate(args(1),j)));  
	for i from 2 to nops(fi) do 
		q[i]:=[divide(e*s[i],fi[i])];
		r[i]:=mapcoeffs(op(q[i],2),fun(faclib::truncate(args(1),j)));
		q[i]:=mapcoeffs(op(q[i],1),fun(faclib::truncate(args(1),j)));
		fi[i]:=fi[i]+r[i];
	end_for;
	# i = 1 #
	r[1]:=mapcoeffs(s[1]*e+fi[1]*(_plus(q[k] $hold(k)=2..nops(fi))),
		fun(faclib::truncate(args(1),j)));
	fi[1]:=fi[1]+r[1];
	for i from 1 to nops(fi) do 
			g[i]:=mapcoeffs(_mult(op(fi,k) $hold(k)=1..i-1, 
			op(fi,k)$hold(k)=i+1..nops(fi)),
			fun(faclib::truncate(args(1),j))); 
	end_for; 
	sg:=map(zip(s,g,_mult),mapcoeffs,faclib::truncate,j);
	for i from 2 to nops(fi) do
		u[i]:=[divide(s[i]*(sg[i]-poly(1,op(fi[1],2..3))),fi[i])];
		t[i]:=mapcoeffs(op(u[i],2),fun(faclib::truncate(args(1),j)));
		u[i]:=mapcoeffs(op(u[i],1),fun(faclib::truncate(args(1),j)));
	end_for;
	# i=1 #
	t[1]:=mapcoeffs(s[1]*(_plus(sg[k] $hold(k)=1..nops(fi)) -
		poly(1,[X],R)) +
		fi[1]*(_plus(u[k] + s[k] * divide
		(_plus(sg[l] $hold(l)=1..k-1,
		 sg[l] $hold(l)=k+1..nops(fi)),fi[k],hold(Quo)) 
		$hold(k)=2..nops(fi))),fun(faclib::truncate(args(1),j))); 
	s:=zip(s,t,fun(args(1)-args(2)));
	# determine s[1]#
	# s[i]*g[i] is not sg[i] any more #
	#s[1]:=poly(bimoddivideexact(poly(1,[X,Y],F)-
		_plus(poly(faclib::truncate(s[i]*g[i],j),[X,Y],F) 
		$hold(i)=2..nops(fi)),poly(g[1],[X,Y],F),j),[X],R); #		        
until j>=d end_repeat;
userinfo(2,"Time for Hensel lifting: ".expr2text(time()-st));

userinfo(3,"Result of lifting:",fi);	

# try to combine operands of fi to obtain a factorization #

st:=time();
factorsleft:=[op(fi,2..nops(fi))];
factlist:=[];

repeat
	r:=faclib::tryall(ff,factorsleft,lc,d);
	if r=FAIL then
		factorsleft:=[];
	else
		factlist := append(factlist,op(r,1));
		userinfo(2,"Factor found:",op(r,1));
		ff:=divide(ff,op(r,1),hold(Exact));
		if ff=FAIL then error("factor not correct"); end_if;
		setleft:= {$1..nops(factorsleft)} minus op(r,2);
		factorsleft:=[op(factorsleft,op(setleft,j))
		 $hold(j)=1..nops(setleft)];
	end_if
until factorsleft=[] end_repeat;
userinfo(1,"Time needed for recombination:".expr2text(time()-st));
if factlist=[] then 
	return([f])
else
   	factlist:=append(factlist,ff); 
	return(map(factlist,fun(poly(subs(expr(args(1)),
		Y=Y-expr(evpoint)),
		op(f,2..3)))));
end_if
end_proc:




