# $Date: 1996/12/20 14:08:33 $ $Author: stefanw $ $Revision: 1.10 $ #

# kg, 30/07/94 #

#--
gcdlib::dom_gcd -- compute gcd of 2 polynomials over a domain

gcdlib::dom_gcd(p1, p2)

p1,p2 - non-zero polynomials over a domain
--#

gcdlib::dom_gcd:= proc(a, b)
    local x, X, T, g, r, i, j;
begin
    X:= op(a,2);
    T:= op(a,3);
    if T::hasProp(Dom::IntegerMod)=TRUE then  # mod_gcd is faster #
	r:= IntMod(T::constructor_args[1]);
	g:= gcdlib::mod_gcd(poly(a, r), poly(b, r));
	return(poly(multcoeffs(g, 1/lcoeff(g)), T));
    end_if;

    # univariate case? #
    if nops(X) = 1 then
	r:= gcdlib::udom_gcd(a,b)
    else
	# reduce to univariate case #
	x:= X[1];
	X[1]:= NIL;
	a:= poly(a, [x], Poly(X,T));
	b:= poly(b, [x], Poly(X,T));
	# try zero evaluation point first #
	# if the modular gcd is 1, the gcd equals the content #
	g:=map([a,b],fun(mapcoeffs(args(1),coeff,0)));
	# set g to list of modular images of a and b #
	if map(g,degree)=map([a,b],degree) then
		if degree(gcd(op(g)))=0 then
			g:=gcd(op(map([a,b],content)));
			userinfo(5,"Trailing coefficients are relatively prime");
			userinfo(5,"gcd cannot depend on ".expr2text(x));
		end_if;
	# try some random evaluation points #
	elif T::random <> FAIL then
		for i from 1 to 3 do  # three evaluation points #
			r:=[ T::random() $hold(j)=1..nops(X)];
			g:=map([a,b],fun(mapcoeffs(args(1),fun(evalp(args(1),	
			X[j]=r[j] $hold(j)=1..nops(X))))));
			if degree(gcd(op(g)))=0 and 
				map(g,degree)=map([a,b],degree) then 
				g:=gcd(op(map(g,content)));
				userinfo(5,"Modular images are relatively prime");
				userinfo(5,"gcd cannot depend on ".expr2text(x));
				break;
			end_if;			
		end_for;
	end_if;
	# if no success -> slow method #
	if domtype(g)=DOM_LIST then	  			
		g:= gcdlib::udom_gcd(a,b);
	end_if;
	# convert back to multivariate polynomial #
	X:= [x].X;
	r:= poly(T::zero,X,T);
	for i from 1 to nterms(g) do
	    x:= nthcoeff(g,i);
	    r:= r + poly(nthterm(g,i),X,T) *
		    _plus((poly(nthcoeff(x,j),X,T) * poly(nthterm(x,j),X,T))
			  $ j=1..nterms(x));
	end_for;
    end_if;

    # normalize result if possible #
    if T::unitNormalRep <> FAIL then
	x:= T::unitNormalRep(lcoeff(r));
	r:= multcoeffs(r, x[2]);
    end_if;
    r
end_proc:

# end of file #
