# $Date: 1998/01/20 12:32:04 $ $Author: stefanw $ $Revision: 1.5 $ #

# kg, 09/12/93 #

#--
gcdlib::int_gcd -- compute the gcd of two polynomials over the integers

gcdlib::int_gcd(p1, p2)

p1,p2 - non-zero polynomials of type 'Expr' over the integers

gcdlib::int_gcd computes the gcd of 2 non-zero polynomials over the integers.
The arguments are not checked.
--#

gcdlib::int_gcd:= proc(a, b)
    local ca, cb, g, inds, phia, phib, rndgen, rndvec, m, T, dependsOn;
begin
    userinfo(2, "Integer gcd called with input in the variables ".
		expr2text(op(a,2)) );
    # remove integer contents #
    ca:= icontent(a);
    cb:= icontent(b);
    a:= multcoeffs(a, 1/ca);
    b:= multcoeffs(b, 1/cb);
    ca:= igcd(ca, cb);
    
    inds:= op(a,2);

    # easy check: for polynomials with many indeterminates, 
      try some random homomorphism which does not map both
      lcf(a) and lcf(b) to zero - since lcf(gcd(a,b)) divides
      both of them, this homomorphism won't decrease 
      the degree of the gcd. Therefore test whether gcd of
      images is already 1 : then gcd(a,b) does only depend
      on those variables for which something was inserted .
   #
     
    if nops(inds)>=2 then
	dependsOn:=[];
	rndgen:=random(1000); # should suffice #
	for m from 1 to nops(inds) do
	    repeat
		rndvec:=[rndgen() $hold(g)=2..nops(inds)];
	 	phia:=evalp(a, op(zip([op(inds,1..m-1)
	           ,op(inds,m+1..nops(inds))], rndvec, _equal)));
		phib:=evalp(b, op(zip([op(inds,1..m-1)
		   ,op(inds,m+1..nops(inds))], rndvec, _equal))) 
	    until degree(a, op(inds,m)) = degree(phia)
	       or degree(b, op(inds,m)) = degree(phib) end_repeat;			

            if degree(gcdlib::int_gcd(phia,phib))>0 then
	       userinfo(4, "gcd probably depends on ".expr2text(inds[m]));
	       dependsOn:=append(dependsOn, inds[m])
	    else
	       userinfo(4, "gcd does not depend on ".expr2text(inds[m]))
	    end_if;	
	 end_for;
	 if dependsOn = [] then
	    return(poly(ca,inds))
	 end_if;
	 if dependsOn <> inds then # remove unnecessary variables #
	    T:=select(inds, fun(contains(dependsOn, args(1))=0)),
                Poly(dependsOn);
            gcd(coeff(poly(a,T)), coeff(poly(b,T)) );
            g:= poly(%, inds)
	 end_if;
     end_if;

    if g=hold(g) then
      if nops(inds) < 5 then
	# try heuristic and use sparse modular alg. if it fails #
	userinfo(2, "Trying heuristic gcd");
	g:= gcdlib::heu_gcd(a, b);
	if domtype(g) <> DOM_POLY then
	    userinfo(2, "Using sparse modular gcd-algorithm");
	    g:= gcdlib::int_mod_gcd(a, b)
	end_if;
      else
	userinfo(2, "Using sparse modular gcd-algorithm");
	g:= gcdlib::int_mod_gcd(a, b)
      end_if;
    end_if;
    
    # normalize result #
    if lcoeff(g) < 0 then
        multcoeffs(g, -ca/icontent(g))
    else
        multcoeffs(g, ca/icontent(g))
    end_if
end_proc:

# end of file #
