# $Date: 1995/11/21 09:38:49 $ $Author: kg $ $Revision: 1.10 $ #

# kg, 14/09/94 #

#--
Scale -- scale exponents with factors

Scale(p, S)

p - non-zero polynomial
S - list containing scale factor for each variable of p

Multiplies each exponent of variable xi of p with factor S[i]. The resulting
new exponents must be non-negative integers.
--#

alias(
Scale(p, S) =
    _plus(multcoeffs(poly(_mult(op(zip(X, zip(degreevec(nthterm(p,i)), S, _mult), _power))), 
    			  X, T),
		     nthcoeff(p,i))
    	  $ hold(i)=1..nterms(p))
):

#--
gcdlib::special_cases(a, b, gcdp, normp)

a, b        - polynomials
gcdp, normp - procedures

Computes the gcd of a and b. At first some special cases are handled.
Only if neccecary the 'real' gcd routine 'gcdp' is called. 'normp' is
used to normalize the result if a divides b or vice versa.
--#

gcdlib::special_cases:= proc(a, b, gcdp, normp)
    local n, da, db, ma, mb, i, j, x, g, T, X, Xn, S, f, doScale;
begin
    X:= op(a,2);
    n:= nops(X);
    T:= op(a,3);

    # ldegree(xi) > 0 ? #
    ma:= [ ldegree(a, X[i]) $ i=1..n ];
    if _plus(op(ma)) <> 0 then
        a:= divide(a, poly(_mult(op(zip(X, ma, _power))), X, T), Exact);
    end_if;
    mb:= [ ldegree(b, X[i]) $ i=1..n ];
    if _plus(op(mb)) <> 0 then
        b:= divide(b, poly(_mult(op(zip(X, mb, _power))), X, T), Exact);
    end_if;
    f:= zip(ma, mb, min);
    if _plus(op(f)) <> 0 then
        f:= poly(_mult(op(zip(X, f, _power))), X, T)
    else
        f:= a^0
    end_if;

    if divide(a, b, Exact) <> FAIL then
        return(f * normp(b, a))
    elif degree(b) < 2 then
        return(f * normp(poly(content(b), X, T), a))
    elif divide(b, a, Exact) <> FAIL then
        return(f * normp(a, b))
    elif degree(a) < 2 then
        return(f * normp(poly(content(a), X, T), b))
    end_if;

    # substitution x -> x^(1/g) possible? #
    if n = 1 then
	da:= [[ degree(nthterm(a,j)) $ j=1..nterms(a) ]];
	db:= [[ degree(nthterm(b,j)) $ j=1..nterms(b) ]];
    else
	da:= [ degreevec(nthterm(a,j)) $ j=1..nterms(a) ];
	db:= [ degreevec(nthterm(b,j)) $ j=1..nterms(b) ];
	da:= [ map(da, _index, j) $ j=1..n ];
	db:= [ map(db, _index, j) $ j=1..n ];
    end_if;

    S:= zip(da, db, fun(max(1, igcd(0, op(args(1)), op(args(2))))));
    doScale:= bool(select(S, _equal, 1) <> S);
    if doScale then
        g:= map(S, 1/id);
        a:= Scale(a, g);
        b:= Scale(b, g);
    end_if;

    # reduce to common indets with positive degree #
    x:= X;
    repeat
        g:= map([ min(degree(a, x[j]), degree(b, x[j])) $ j=1..n ], _equal, 0);
        Xn:= [ (if g[j] then null() else x[j] end_if) $ j=1..n ];
        if Xn = x or Xn = [] then break end_if;
        x:= [ (if g[j] then x[j] else null() end_if) $ j=1..n ];
        a:= gcd(coeff(poly(a, x, Poly(Xn, T))));
        b:= gcd(coeff(poly(b, x, Poly(Xn, T))));
        x:= Xn;
        n:= nops(x);
    until FALSE end_repeat;

    if nops(Xn) = 0 then
	g:= f^0
    else
	g:= gcdp(a, b);
	if Xn <> X then g:= poly(g, X, T) end_if
    end_if;

    if doScale then
        f * Scale(g, S);
    else
	f * g
    end_if;
end_proc:

unalias(Scale):

# end of file #
