# $Date: 1997/09/11 12:55:25 $  $Author: kg $  $Revision: 1.5 $ #

#--
C.mu - C-formatted output of expressions

Some of the functions implemented here look quite obscure. The reason is that
generate::C may call DOM_EXPR::C which in turn calls 'level'. In order to
avoid naming conflicts of actual parameters with formal parameters or local
variables due to this call of 'level' I use formal parameters or local
variables only if neccesary and name them obscure (like _0_s for expample).

This problem should become obselete in later versions of MuPAD either when
static scoping is implemented or when operators are represented by function
environments (rather than by identifiers which have to be evaluated to get
the corresponding function environments).
--#

proc() begin

#++
generate::C -- return C-formatted string for expression

generate::C(e)

e - expression, equation or list of equations

An equation represents an assignment of C-type 'double'.
++#

generate::C:= proc() option hold; local _0_s, _0_a, _0_t; begin
    if args(0) <> 1 then error("wrong no of args") end_if;
    if domtype( context(args(1)) ) = DOM_ARRAY then _0_t:= args(1) end_if;
    _0_a:= map([args()],context);
    _0_s:= "";
    if has(_0_a[1], PI) then
	_0_s:= "#ifndef PI\n#define PI 3.141592653589793\n#endif\n"
    end_if;
    if has(_0_a, E) then
	_0_s:= _0_s."#ifndef EXP\n#define EXP 2.718281828459045\n#endif\n"
    end_if;
    if has(_0_a, EULER) then
	_0_s:= _0_s."#ifndef EULER\n#define EULER 0.5772156649015328\n#endif\n"
    end_if;
    case type(_0_a[1])
    of DOM_LIST do
        _concat(_0_s,
        	op(zip(map(_0_a[1], generate::Cassignment, "double"),
		       ["\n" $ (nops(_0_a[1])-1)],
		       _concat, "")));
        break;
    of "_equal" do
        _0_s.generate::Cassignment(_0_a[1], "double");
        break;
    of DOM_ARRAY do
        _0_s.DOM_ARRAY::C(_0_t,_0_a[1], "double");
        break
    otherwise
        _0_s.generate::Cassignment(genident("t")=_0_a[1], "double");
    end_case
end_proc;

# ------------------------ internas ----------------------- #

#--
generate::Cassignment -- generate C-assignment from equation

generate::Cassignment(eq,t)

eq - equation
t  - expression type ("double" or "int")
--#

generate::Cassignment:= fun((
    if type(args(1)) <> "_equal" then error("not an assignment") end_if;
    "    ".generate::CC(op(args(1),1), 2, args(2))." = ".
    generate::CC(op(args(1),2), 2, args(2))." ;"
));

#--
generate::CC -- return C-formatted string for expression

generate::CC(e,p,t)

e - expression
p - priority of enclosing expression
t - expression type ("double" or "int")
--#

generate::CC:= fun((
    if contains(generate::Cdefs, args(1)) then
	generate::Cdefs[args(1)]
    else
	if domattr(domtype(args(1)), "C") = FAIL then
	    error("illegal type")
	end_if;
	domattr(domtype(args(1)), "C")(args())
    end_if
));

generate::Cdefs:= table(
    hold(PI)="PI", hold(E)="EXP", hold(EULER)="EULER"
);

#--
generate::Cop -- C-formatting of n-ary operator

generate::Cop(sym, tp, ep, t, op)

sym - operator symbol
tp  - priority of this expression
ep  - priority of enclosing expression
t   - expression type ("double" or "int")
op  - list of operands
--#

generate::Cop:= fun(
    _concat(
        (if args(3) >= args(2) then "(" else "" end_if),
        op(zip(map(args(5), generate::CC, args(2), args(4)),
               [ args(1) $ (nops(args(5))-1) ],
               _concat, "")),
        (if args(3) >= args(2) then ")" else "" end_if))
);

#--
generate::Cunop -- C-formatting of unary operator

generate::Cunop(sym, tp, ep, t, op)

sym - operator symbol
tp  - priority of this expression
ep  - priority of enclosing expression
t   - expression type ("double" or "int")
op  - operand
--#

generate::Cunop:= fun(
    _concat(
        (if args(3) >= args(2) then "(" else "" end_if),
        args(1), generate::CC(args(5), args(2), args(4)),
        (if args(3) >= args(2) then ")" else "" end_if))
);

#--
generate::Ccall -- C-formatting function call

generate::Ccall(o,t,e)

o - C string of operator
t - expression type ("double" or "int")
e - list of operand expressions
--#

generate::Ccall:= fun(
    _concat(args(1), "(",
    	    (if args(3) = [] then "" else
    	         op(zip(map(args(3), generate::CC, 1, args(2)),
    	                [ ", " $ (nops(args(3))-1) ],
    	                _concat, ""))
    	     end_if), ")")
);

#--
generate::Cisdiv -- test if expression is 1/x

generate::Cisdiv(e)

e - expression
--#

generate::Cisdiv:= fun((
    if type(args(1)) = "_power" then
        if op(args(1),2) = -1 then
            return(TRUE)
        end_if
    end_if;
    FALSE
));

#--
generate::Cisminus -- test if expression is -x

generate::Cisminus(e)

e - expression
--#

generate::Cisminus:= fun((
    if type(args(1)) = "_mult" then
        if testtype(op(args(1),nops(args(1))),NUMERIC) then
            return(bool(op(args(1),nops(args(1))) < 0))
        end_if
    elif testtype(args(1),NUMERIC) then
        return(bool(args(1) < 0))
    end_if;
    FALSE
));

# ------------- TeX formatting of domain elements ------------- #

DOM_INT::C:= proc(e, p, t) local DIGITS; begin
    DIGITS:= 16;
    if t = "int" then expr2text(e) 
    else DOM_FLOAT::C(float(e), p, t) 
    end_if
end_proc;

DOM_RAT::C:= proc(e, p, t) local DIGITS; begin
    if t = "int" then error("not an integer") end_if;
    generate::Cop("/", 13, p, t,
    	[float(op(e,1)), float(op(e,2))])
end_proc;

DOM_FLOAT::C:= proc(e, p, t) begin
    if t = "int" then error("not an integer") end_if;
    # remove trailing 0's #
    e:= text2list(expr2text(e), [".", "e"], Cyclic);
    p:= op(e,3);
    while substring(p, strlen(p)-1, 1) = "0" do
	if strlen(p) = 1 then break end_if;
	p:= substring(p, 0, strlen(p)-1)
    end_while;
    _concat(op(subsop(e, 3=p)))
end_proc;

DOM_POLY::C:= fun(generate::CC(expr(args(1)), args(2..3)));

DOM_STRING::C:= fun("\"".expr2text(args(1))."\"");

DOM_IDENT::C:= fun(expr2text(args(1)));

DOM_EXPR::C:= proc(_0_x) local _0_f; begin
    _0_f:= op(_0_x,0);
    if domtype(_0_f) = DOM_IDENT then
	_0_f:= level(_0_f);
    	if domtype(_0_f) = DOM_FUNC_ENV then
    	    if funcattr(_0_f, "C") = FAIL then
    	    	if domtype(op(_0_f,2)) = CAT_EXEC then
    	    	    generate::Ccall(op(_0_f,[2,4]), args(3), [op(_0_x)])
    	    	elif domtype(op(_0_f,1)) = CAT_EXEC then
    	    	    generate::Ccall(op(_0_f,[1,3]), args(3), [op(_0_x)])
    	    	else
    	    	    generate::Ccall(generate::CC(op(_0_x,0),15,args(3)), args(3), [op(_0_x)])
    	    	end_if
    	    else
    	    	funcattr(_0_f, "C")(_0_f, _0_x, args(2..3))
    	    end_if
	else
	    generate::Ccall(generate::CC(op(_0_x,0),15,args(3)), args(3), [op(_0_x)])
    	end_if
    else
    	generate::Ccall(generate::CC(_0_f,15,args(3)), args(3), [op(_0_x)])
    end_if
end_proc;

#--
    returns a C formatted string for arrays

    e  : name of identifier holding the array
    a  : the array
    typ: type of expressions of a
--#
DOM_ARRAY::C:= proc(e,a,typ)
    local S, X, b, f, n, s, i, p, carry;
begin
    s:= [op(a,[0,2..nops(op(a,0))])];
    if contains(map(s,domtype),DOM_LIST) <> 0 then 
        error("can't handle block structure of arrays") 
    elif contains({op(a)},NIL) then
        error("array contains undefined entries")
    end_if;

    b:= map(s,op,1); f:= map(s,op,2);
    n:= nops(b);
    s:= b;
    S:= "    " . generate::CC( _index(e,op(s)),2,typ ) . " = " . generate::CC(_index(a,op(s)),2,typ );

    for i from 2 to nops(op(a)) do
        carry:= TRUE;
        p:= n;
        while carry do
            s[p]:= s[p] + 1;
            if s[p] > f[p] then s[p]:= b[p]; p:= p - 1 else carry:= FALSE end_if
        end_while;
        S:= S . "\n" . "    " . generate::CC( _index(e,op(s)),2,typ ) . " = " . generate::CC(_index(a,op(s)),2,typ )
    end_for; 
    S
end_proc:

# ------------------- C formatting of func-env's ------------------- #
#--
Each C-formatting routine must have the form
	myformatfunc(fenv, expr, prio, type)
where fenv is the func-env of this routine, expr is the original expression
causing this call (the operator of expr was evaluated to fenv), prio is the
priority of the enclosing expression and type is a string giving the type of
the expression.

Be aware that one of the called functions generate::CC... may call
DOM_EXPR::C which in turn calls 'level'! In order to avoid a naming
conflict one should use formal parameters or local variables only
if neccesary and name them obscure (libe _0_s for expample).
--#

_mult:= funcattr(_mult, "C",
    proc() local _0_n, _0_d; begin
    	if testtype(op(args(2), nops(args(2))), NUMERIC) then
    	    if op(args(2), nops(args(2))) = -1 then
    	    	return(generate::Cunop("-", 14, args(3..4), 
    	    				subsop(args(2), nops(args(2)) = null())))
    	    end_if;
	    if select({op(args(2), 1..nops(args(2)-1))}, testtype, NUMERIC) 
	       = {}  then # expression is in MuPAD normal form #
    	      return(funcattr(_mult,"C")(
    	    	    args(1), hold(_mult)(op(args(2), nops(args(2))),
    	    	    		op(args(2), 1..(nops(args(2))-1))), args(3..4)))
	    end_if;			  
    	end_if;
    	_0_n:= select(args(2), not generate::Cisdiv);
    	_0_d:= select(args(2), generate::Cisdiv);
    	if _0_d = 1 then
	    generate::Cop("*", 13, args(3..4), [op(args(2))])
	else
	    generate::Cop("/", 13, args(3..4), [_0_n, 1/_0_d])
	end_if
    end_proc
);

_plus:= funcattr(_plus, "C", fun((
        _concat(
            (if args(3) >= 12 then "(" else "" end_if),
            generate::CC(op(args(2),1), 12, args(4)),
            ((if generate::Cisminus(op(args(2),i)) then
                  " - ".generate::CC(-op(args(2),i), 12, args(4))
              else
                  " + ".generate::CC( op(args(2),i), 12, args(4))
              end_if)
             $ hold(i)=2..nops(args(2))),
            (if args(3) >= 12 then ")" else "" end_if))
)));

_power:= funcattr(_power, "C", fun((
        if args(4) = "int" then error("illegal type") end_if;
        if domtype(op(args(2),2)) = DOM_INT then
            if op(args(2),2) = -1 then
                generate::Cop("/", 13, args(3..4), [1.0, op(args(2),1)])
	    elif op(args(2),2) = 2 then
	        if contains({DOM_IDENT, "_index"}, type(op(args(2),1))) then
		    generate::Cop("*", 13, args(3..4), [op(args(2),1), op(args(2),1)])
		else
		    generate::Ccall("pow", "double", [op(args(2))])
		end_if
	    # 'ipower' is not always available:
            elif op(args(2),2) > 0 and op(args(2),2) < 2^15-1 then
                "ipower(".generate::CC(op(args(2),1), 1, "double").
                ", ".generate::CC(op(args(2),2), 1, "int").")"
            #
            else
                generate::Ccall("pow", "double", [op(args(2))])
            end_if
        else
            generate::Ccall("pow", "double", [op(args(2))])
        end_if
    ))
);

_index:= funcattr(_index, "C", fun((
    _concat(
        generate::CC(op(args(2),1), 15, args(4)), "[",
        op(zip(map([op(args(2),2..nops(args(2)))], generate::CC, 1, "int"),
               [ ", " $ (nops(args(2))-2) ],
               _concat, "")), "]")
)));

ln:= funcattr(ln, "C", fun((
    if args(4) = "int" then error("illegal type") end_if;
    generate::Ccall("log", "double", [op(args(2))])
)));

atan:= funcattr(atan, "C", fun((
    if args(4) = "int" then error("illegal type") end_if;
    case nops(args(2))
    of 1 do
	generate::Ccall("atan", "double", [op(args(2))]); 
	break;
    of 2 do
	generate::Ccall("atan2", "double", [op(args(2),2), op(args(2),1)]); 
	break;
    end_case
)));

abs:= funcattr(abs, "C", fun((
    if args(4) = "int" then
	generate::Ccall("abs", "int", [op(args(2))])
    else
	generate::Ccall("fabs", "double", [op(args(2))])
    end_if
)));

_mod:= funcattr(_mod, "C", fun((
    if args(4) = "double" then error("illegal type") end_if;
    generate::Cop(" % ", #13# 15, #args(3)# 15, args(4), [op(args(2))])
)));

_div:= funcattr(_div, "C", fun((
    if args(4) = "double" then error("illegal type") end_if;
    generate::Cop("/", 13, args(3..4), [op(args(2))])
)));

end_proc():

# end of file #
