#--
fortran.mu - fortran-formatted output of expressions

Some of the functions implemented here look quite obscure. The reason is that
generate::fortran may call DOM_EXPR::fortran which in turn calls 'level'. In order to
avoid naming conflicts of actual parameters with formal parameters or local
variables due to this call of 'level' I use formal parameters or local
variables only if neccesary and name them obscure (like _0_s for expample).

This problem should become obselete in later versions of MuPAD either when
static scoping is implemented or when operators are represented by function
environments (rather than by identifiers which have to be evaluated to get
the corresponding function environments).

The user can define himself a method for a domain d. He only has to write a function
d::fortran(e,p,t) that takes as arguments the element e of domain d, the priority p
of the enclosing expression, and the wanted type t (either "int" for an integer, or
"double" for a double floating-point). The call d::fortran(e,p,t) returns a string
for the formatting of e, with parentheses if required.

The following priorities are defined:
	1	, (expression sequence)
	2	= (equality)
        12	+ (addition)
	13	*,/ (multiplication,division)
	14	-,^ (unary minus, power)
	15	operator,index
--#

proc() begin

#++
generate::fortran -- return fortran-formatted string for expression

generate::fortran(e)

e - expression, equation or list of equations

An equation represents an assignment of fortran-type 'double'.
++#

generate::fortran:= proc() local _0_s,PRETTY_PRINT,TEXTWIDTH,i; begin
    if args(0) <> 1 then error("wrong no of args") end_if;
    case type(args(1))
    of DOM_LIST do
        _0_s:=map(args(1), generate::fortranassignment, "double");
        break;
    of "_equal" do
        _0_s:=[generate::fortranassignment(args(1), "double")];
        break;
    otherwise
        _0_s:=[generate::fortranassignment(genident("t")=args(1), "double")];
    end_case;
    PRETTY_PRINT:=FALSE; TEXTWIDTH:=78;
    _0_s:=map(_0_s,generate::fortrancut,1);
    # print(Unquoted,_concat(_0_s[1],("\n"._0_s[i])$i=2..nops(_0_s)))
	-- don't print the string, otherwise we can't put it onto file etc. (kg) #
    _concat(_0_s[1],("\n"._0_s[i])$i=2..nops(_0_s))
end_proc;

# s is the string to print, n is the line number #
generate::fortrancut:=proc(s,n) local t;
begin
    if n=1 then t:="      " else t:="     ~" end_if;
    if strlen(s)<=66 then t.s
    else (t.substring(s,0,66)),generate::fortrancut(substring(s,66,strlen(s)-66),n+1)
    end_if
end_proc:

# ------------------------ internas ----------------------- #

#--
generate::fortranassignment -- generate fortran-assignment from equation

generate::fortranassignment(eq,t)

eq - equation
t  - expression type ("double" or "int")
--#

generate::fortranassignment:= fun((
    if type(args(1)) <> "_equal" then error("not an assignment") end_if;
    generate::Fortran(op(args(1),1), 2, args(2))." = ".
    generate::Fortran(op(args(1),2), 2, args(2))
));

#--
generate::Fortran -- return fortran-formatted string for expression

generate::Fortran(e,p,t)

e - expression
p - priority of enclosing expression
t - expression type ("double" or "int")
--#

generate::Fortran:= fun((
    if contains(generate::fortrandefs, args(1)) then
	generate::fortrandefs[args(1)]
    else
	if domattr(domtype(args(1)), "fortran") = FAIL then
	    error("illegal type")
	end_if;
	domattr(domtype(args(1)), "fortran")(args())
    end_if
));

generate::fortrandefs:= table(
    hold(PI)="3.141592653589793D0", hold(E)="2.718281828459045D0", 
    hold(EULER)="0.5772156649015329D0"
);

#--
generate::fortranop -- fortran-formatting of n-ary operator

generate::fortranop(sym, tp, ep, t, op)

sym - operator symbol
tp  - priority of this expression
ep  - priority of enclosing expression
t   - expression type ("double" or "int")
op  - list of operands
--#

generate::fortranop:= fun(
    _concat(
        (if args(3) > args(2) then "(" else "" end_if),
        op(zip(map(args(5), generate::Fortran, args(2), args(4)),
               [ args(1) $ (nops(args(5))-1) ],
               _concat, "")),
        (if args(3) > args(2) then ")" else "" end_if))
);

#--
generate::fortranunop -- fortran-formatting of unary operator

generate::fortranunop(sym, tp, ep, t, op)

sym - operator symbol
tp  - priority of this expression
ep  - priority of enclosing expression
t   - expression type ("double" or "int")
op  - operand
--#

generate::fortranunop:= fun(
    _concat(
        (if args(3) >= args(2) then "(" else "" end_if),
        args(1), generate::Fortran(args(5), args(2), args(4)),
        (if args(3) >= args(2) then ")" else "" end_if))
);

#--
generate::fortrancall -- fortran-formatting function call

generate::fortrancall(o,t,e)

o - fortran string of operator
t - expression type ("double" or "int")
e - list of operand expressions
--#

generate::fortrancall:= fun(
    _concat(args(1), "(",
    	    (if args(3) = [] then "" else
    	         op(zip(map(args(3), generate::Fortran, 1, args(2)),
    	                [ "," $ (nops(args(3))-1) ],
    	                _concat, ""))
    	     end_if), ")")
);

#--
generate::fortranisdiv -- test if expression is 1/x

generate::fortranisdiv(e)

e - expression
--#

generate::fortranisdiv:= fun((
    if type(args(1)) = "_power" then
        if op(args(1),2) = -1 then
            return(TRUE)
        end_if
    end_if;
    FALSE
));

#--
generate::fortranisminus -- test if expression is -x

generate::fortranisminus(e)

e - expression
--#

generate::fortranisminus:= fun((
    if type(args(1)) = "_mult" then
        if testtype(op(args(1),nops(args(1))),NUMERIC) then
            return(bool(op(args(1),nops(args(1))) < 0))
        end_if
    elif testtype(args(1),NUMERIC) then
        return(bool(args(1) < 0))
    end_if;
    FALSE
));

# ------------- TeX formatting of domain elements ------------- #

DOM_INT::fortran:= proc(e, p, t) local DIGITS; begin
    if t="int" then expr2text(e)
    else DIGITS:=16; DOM_FLOAT::fortran(float(e))
    end_if
end_proc;

DOM_RAT::fortran:= proc(e, p, t) local DIGITS; begin
    if t = "int" then error("not an integer") end_if;
    generate::fortranop("/", 13, p, t,
    	[float(op(e,1)), float(op(e,2))])
end_proc;

DOM_FLOAT::fortran:= proc(e, p, t) begin
    if t = "int" then error("not an integer") end_if;
    # remove trailing 0's #
    e:= text2list(expr2text(e), [".", "e"], Cyclic);
    p:= op(e,3);
    while substring(p, strlen(p)-1, 1) = "0" do
	if strlen(p) = 1 then break end_if;
	p:= substring(p, 0, strlen(p)-1)
    end_while;
    if nops(e)=3 then e:=append(e,"e","0") end_if;
    _concat(op(subs(subsop(e, 3=p),"e"="D")))
end_proc;

DOM_POLY::fortran:= fun(generate::Fortran(expr(args(1)), args(2..3)));

DOM_STRING::fortran:= fun("\"".expr2text(args(1))."\"");

DOM_IDENT::fortran:= fun(expr2text(args(1)));

DOM_EXPR::fortran:= proc(_0_x) local _0_f; begin
    _0_f:= op(_0_x,0);
    if domtype(_0_f) = DOM_IDENT then
	_0_f:= level(_0_f);
    	if domtype(_0_f) = DOM_FUNC_ENV then
    	    if funcattr(_0_f, "fortran") = FAIL then
    	    	if domtype(op(_0_f,2)) = CAT_EXEC then
    	    	    generate::fortrancall(op(_0_f,[2,4]), args(3), [op(_0_x)])
    	    	elif domtype(op(_0_f,1)) = CAT_EXEC then
    	    	    generate::fortrancall(op(_0_f,[1,3]), args(3), [op(_0_x)])
    	    	else
    	    	    generate::fortrancall(generate::Fortran(op(_0_x,0),15,args(3)), args(3), [op(_0_x)])
    	    	end_if
    	    else
    	    	funcattr(_0_f, "fortran")(_0_f, _0_x, args(2..3))
    	    end_if
	else
	    generate::fortrancall(generate::Fortran(op(_0_x,0),15,args(3)), args(3), [op(_0_x)])
    	end_if
    else
    	generate::fortrancall(generate::Fortran(_0_f,15,args(3)), args(3), [op(_0_x)])
    end_if
end_proc;

# ------------------- fortran formatting of func-env's ------------------- #
#--
Each fortran-formatting routine must have the form
	myformatfunc(fenv, expr, prio, type)
where fenv is the func-env of this routine, expr is the original expression
causing this call (the operator of expr was evaluated to fenv), prio is the
priority of the enclosing expression and type is a string giving the type of
the expression.

Be aware that one of the called functions generate::Fortran... may call
DOM_EXPR::fortran which in turn calls 'level'! In order to avoid a naming
conflict one should use formal parameters or local variables only
if neccesary and name them obscure (libe _0_s for expample).
--#

_mult:= funcattr(_mult, "fortran",
    proc() local _0_n, _0_d; begin
    	if testtype(op(args(2), nops(args(2))), NUMERIC) then
    	    if op(args(2), nops(args(2))) = -1 then
    	    	return(generate::fortranunop("-", 14, args(3..4),
    	    				subsop(args(2), nops(args(2)) = null())))
    	    end_if;
            if select({op(args(2), 1..nops(args(2)-1))}, testtype, NUMERIC) 
               = {}  then # expression is in MuPAD normal form #
                  return(funcattr(_mult,"fortran")(
    	    	    args(1), hold(_mult)(op(args(2), nops(args(2))),
    	    	    		op(args(2), 1..(nops(args(2))-1))), args(3..4)))
            end_if;
    	end_if;
    	_0_n:= select(args(2), not generate::fortranisdiv);
    	_0_d:= select(args(2), generate::fortranisdiv);
    	if _0_d = 1 then
	    generate::fortranop("*", 13, args(3..4), [op(args(2))])
	else
	    generate::fortranop("/", 13, args(3..4), [_0_n, 1/_0_d])
	end_if
    end_proc
);

_plus:= funcattr(_plus, "fortran", fun((
        _concat(
            (if args(3) >= 12 then "(" else "" end_if),
            generate::Fortran(op(args(2),1), 12, args(4)),
            ((if generate::fortranisminus(op(args(2),i)) then
                  "-".generate::Fortran(-op(args(2),i), 12, args(4))
              else
                  "+".generate::Fortran( op(args(2),i), 12, args(4))
              end_if)
             $ hold(i)=2..nops(args(2))),
            (if args(3) >= 12 then ")" else "" end_if))
)));

_power:= funcattr(_power, "fortran", fun((
        if domtype(op(args(2),2)) = DOM_INT then
            if op(args(2),2) = -1 then
                generate::fortranop("/", 13, args(3..4), [1.0, op(args(2),1)])
	    elif op(args(2),2) = 2 and type(op(args(2),1))=DOM_IDENT then
	        generate::fortranop("*", 13, args(3..4), [op(args(2),1), op(args(2),1)])
            else
                generate::Fortran(op(args(2),1),14,args(3))."**".
		generate::Fortran(op(args(2),2),14,"int")
                
            end_if
        elif op(args(2),2) = 1/2 then
            generate::fortrancall("dsqrt","double",[op(args(2),1)])
        else
            generate::Fortran(op(args(2),1),14,args(3))."**".
	    generate::Fortran(op(args(2),2),14,args(3))
        end_if
    ))
);

_index:= funcattr(_index, "fortran", fun((
    _concat(
        generate::Fortran(op(args(2),1), 15, args(4)), "(",
        op(zip(map([op(args(2),2..nops(args(2)))], generate::Fortran, 1, "int"),
               [ "," $ (nops(args(2))-2) ],
               _concat, "")), ")")
)));

ln:= funcattr(ln, "fortran", fun((
    generate::fortrancall("dlog", "double", [op(args(2))])
)));

atan:= funcattr(atan, "fortran", fun((
    case nops(args(2))
    of 1 do
	generate::fortrancall("atan", "double", [op(args(2))]); 
	break;
    of 2 do
	generate::fortrancall("atan", "double", [op(args(2),2)/op(args(2),1)]); 
	break;
    end_case
)));

end_proc():

# end of file #
