#--
generate::splitProduct  --  decomposes a product into a list
			    containing the sign, the numerator
			    and the denumerator

  generate::splitProduct( multExpr )

  multExpr --  an expression with type "_mult"

  splitProduct returns a list with the sign of the product
  (-1 or 1) and two lists with the numerator and the denum-
  erator. If the numerator is empty, a 1 is inserted.
  For the decision, if a factor has to be in the numerator 
  or not, exponents of powers are analysed. 
  Numerical factors (which are the last factors in the normal
  form) are splitted (rationals) and put in the first position.
  The result maybe used for output of "_mult" expressions.

  Examples:

>> generate::splitProduct( a / b);

                               [1, [a], [b]]

>> generate::splitProduct( - a / b);

                              [-1, [a], [b]]

>> generate::splitProduct( - a / b^(-x));

                                       x
                             [-1, [a, b ], []]

>> generate::splitProduct( - 2/3 * a / b^(x));

                                             x
                           [-1, [2, a], [3, b ]]

--#

generate::splitProduct :=
  proc( multExpr )
    local numList, denumList, signit, num, i, expo;
  begin
  #  ananyse a _mult-expr and return a list of a 1 number and 2 #
  #  lists; the number is the sign of the product 1 or -1, the  #
  #  1. list contains the numerator and the 2. the denominator. #

    if testargs() then
      if type( multExpr ) <> "_mult" then
        error("_mult-expr expected");
      end_if;
    end_if;

    signit := 1;
    numList := [];
    denumList := [];
    for i in op( multExpr ) do
      if testtype( i, Type::RealNum ) then
        if i < 0 then
	  i := -i;
          signit := signit * (-1);
        end_if;
        if testtype( i, DOM_RAT) then  # fraction #
          num := op(i,1);
          if num <> 1 then  # ignore 1 ( a/2 -> [[a],[2]] not [[1,a],[2]] ) #
            numList := [num].numList;
	  end_if;
	  denumList := [op(i,2)].denumList;
        elif i <> 1 then
              # real number, but no fraction #
          numList := [i].numList;
        end_if;
      elif testtype( i, DOM_COMPLEX ) then  # insert at the beginning #
        numList := [i].numList;
      elif testtype( i, "_power" ) then
	 		# no number, so insert at the end #
        # take a look at the exponent #
        expo := op(i ,2);
        if testtype( expo, "_mult") then
          if testtype( op(expo, nops(expo)), Type::RealNum ) then
            if  op(expo, nops(expo)) < 0 then
              denumList := denumList.[1/i];
            else
              numList := numList.[i];
            end_if; # product contains a real number #
          else
            # exponent is a product with no real number #
            numList := numList.[i];
          end_if;
         # exponent is no product #
         # if exponent is less zero, so insert 1/i in denumerator #
         elif testtype( expo, Type::RealNum ) then
            if op(i,2) < 0 then
              denumList := denumList.[1/i];
            else
              numList := numList.[i];
            end_if;
         else  # exponent is neither power nor real number #
           numList := numList.[i];
         end_if;
      else   # factor is no _power, just insert in numerator #
	numList := numList.[i];
      end_if;
    end_for;

    if numList = [] then   # dont't return empty numerator #
      numList := [ 1 ];
    end_if;

    return( [signit, numList, denumList] );
  end_proc:

