#
 groebner::factor(l) takes as input a list of polynomials (for example
	the output of groebner::gbasis) describing a variety, and factors
	the variety if possible.
 Input: a list of polynomials
 Output: a set of list of polynomials
 Example: 
>> l:=[x^2 + y + z - 3,x + y^2 + z - 3,x + y + z^2 - 3]:
>> g:=groebner::gbasis(l,LexOrder):
>> groebner::factor(g);

{[x - 1, y - 1, z - 1], [x + 3, y + 3, z + 3],

                          2       2
   [x + y - 1, - y + z + y  - 2, z  - 2],

                                   2
   [x + z - 1, y + z - 1, - 2 z + z  - 1]}
#
groebner::factor := proc(l) local s,p,n,g,i,t,q,unk,one,post,unk0,d,T;
begin
   if l=[] then return({[]}) end_if;
   n:=nops(l);
   s:={};
   p:=l[n]; # last polynomial #
   t:=genident("t");
   if type(p)=DOM_POLY then
      unk0:=op(p,2); unk:=[t,op(unk0)]; d:=op(p,3);
      T:=poly(t,unk,d);
      l:=map(l,poly,unk,d);
      one:=poly(1,unk,d);
      post:=fun((if has(op(args(1),1),t) then null() else 
	poly(args(1),unk0,d) end_if));
   else
   one:=1; T:=t; post:=fun((if has(args(1),t) then null() else args(1) end_if))
   end_if;
   p:=factor(p);
   if nops(p)=1 then return({args(1)}) end_if; # degree 0 case #
   p:=[p[2*i]$hold(i)=1..nops(p) div 2];
   if type(p[1])=DOM_POLY then p:=map(p,poly,unk,op(p[1],3)) end_if;
   for i from 1 to nops(p) do
      # use an auxiliary variable t so that q=0 ==> all other factors
	cannot vanish #
      q:=_mult(op(subsop(p,i=T)))-one;
      g:=groebner::gbasis(append(subsop(l,n=p[i]),q),LexOrder,Reorder);
      g:=map(g,post);
      if nops(g)<>1 or expr(g[1])<>1 then s:=s union {g} end_if
   end_for;
   s
end_proc:
