# $Date $  $Author $ $Revision $ #

# General convention: a monomial ordering is a function
  ord: N^k x N^k -> {TRUE,FALSE}

  where ord([m_1,..,m_k],[n_1,..,n_k])=TRUE means
  [m_1,..,m_k] <= [n_1,..,n_k]
#


# groebner::lterm(f,ord)
  
  f   - polynomial
  ord - monomial ordering

  computes the leading term with respect to ord
# 
groebner::lterm:= proc(f,ord)
local result,dv,nt,dvec,i;

begin
if args(0)=1 or type(ord)=DOM_IDENT then
	return(lterm(args()))
end_if; 
result:=nthterm(f,1);
dv:=degreevec(result);
for i from 2 to nterms(f) do
        nt:=nthterm(f,i);
	dvec:=degreevec(nt);
	if ord(dv,dvec) then
		dv:=dvec;
		result:=nt
	end_if
end_for;
result
end_proc:

# groebner::lmonomial(f,ord <,Rem>)
  
  f   - polynomial
  ord - monomial ordering 

  computes the leading monomial with respect to ord
  if the option Rem is given, a list 
  [lmonomial(f),f-lmonomial(f)] is returned
#

groebner::lmonomial:= proc(f,ord)
local result,dv,nt,dvec,i;

begin
if args(0)=1 or type(ord)=DOM_IDENT then
	return(lmonomial(args()))
end_if;
result:=nthmonomial(f,1);
dv:=degreevec(result);
for i from 2 to nterms(f) do
	if ord(dv,(dvec:=degreevec((nt:=nthmonomial(f,i))))) then
		dv:=dvec;
		result:=nt
	end_if
end_for;
if args(0)=2 then
	result
elif args(3)=Rem then
	[result,f-result]
else
        error("Illegal third argument")
end_if
end_proc:

# groebner::lcoeff(f,ord)
  
  f   - polynomial
  ord - monomial ordering 

  computes the leading coefficient with respect to ord
#


groebner::lcoeff:=fun((
	if args(0)=1 or type(args(2))=DOM_IDENT then
		lcoeff(args())
	else
		coeff(groebner::lmonomial(args()))
	end_if) ):

# groebner::tcoeff(f,ord)
  
  f   - polynomial
  ord - monomial ordering 

  computes the trailing coefficient with respect to ord
#

groebner::tcoeff:=fun((
	if args(0)=1 or type(args(2))=DOM_IDENT then
		tcoeff(args())
	else
		groebner::lcoeff(args(1),not args(2))
	end_if) ):
