# $Date: 1997/08/05 16:02:40 $ $Author: zimmerma $ $Revision: 1.8 $ #

# kg, 27/12/93 #

#--
groebner::reduce_basis -- auto-reduces a groebner basis from a list

groebner::reduce_basis(S, o, monic)

S   - groebner basis given as list of monic sugared polynomials
o   - term ordering
monic - normalizer

returns a list of monic polynomials representing the reduced basis.

This version does only work with update.mu where redundant basis elements
are already deleted!
--#

groebner::term_less: # force loading #

groebner::reduce_basis:= proc(S, o, monic)
    local R, i, h, p, lm;
begin
    userinfo(1,"auto reduces list of ".nops(S)." polynomials");
    case nops(S)
    of 0 do return([]);
    of 1 do return([S[1][1]]);
    end_case;
    # now S can't contain polynomials of degree 0 #

    S:= sort(S, fun(groebner::term_less(args(1),args(2),o)));
    
    R:= groebner::redset_insert([], S[1]);
    
    for i from 2 to nops(S) do
        # S[i] can't be reduced by S[j] with j > i because of the sorting of S,
          so we need to reduce S[i] only by R.
          The leading term of S[i] can't be reduced by R because it is > than any
    	  leading term in R, so we split it away before reduction.
    	  Such the leading term of the new, reduced polynomial stays the same and
    	  can't in turn reduce any element in R; i.e. R need not to be reduced
    	  with the new polynomial. #

    	p:= S[i][1];
    	lm:= groebner::lmonomial(p, o, Rem);
    	p:= lm[2];
    	lm:= lm[1];
    	h:= groebner::reduce([p, groebner::lterm(p, o), 0], R, o);
    	if h[1] = p then
    	    R:= groebner::redset_insert(R, S[i])
    	else
    	    R:= groebner::redset_insert(R, [monic(h[1] + lm, o), S[i][2], 0])
    	end_if
    end_for;

    # convert reducing set into polynomial list #
    map(_concat(op(R)), op, 1)
end_proc:

# end of file #
