#
intlib::byparts(f,u) performs integration by parts on f where u is the part
		     to be integrated.

First implementation from Francois Maltey improved by Paul Zimmermann.
Example:
>> intlib::byparts(hold(int)(cos(x)^n,x=0..PI/2),cos(x));

 n + -1         /              n + -2                     n + -2         PI
0       + -1 int| sin(x) cos(x)       + -1 n sin(x) cos(x)      , x = 0..--
                \                                                        2

\
| + -1
/

>> intlib::byparts(hold(int)(sqrt(1+x^2)*x^n,x),x^n);

         -1  n + 1   2     1/2                 -1  n + 2   2     -1/2
  (n + 1)   x      (x  + 1)    + -1 int((n + 1)   x      (x  + 1)    , x)
#

intlib::byparts := proc (intf, ud) local v, u, br, var, x, uv, udv;
  begin
    br := op (intf, 1) ;
    var := op (intf, 2) ;
    u := expand(int (ud, var));
    v := br / ud ;
    uv:=combine(u*v);
    x:=op(var,1);
    udv:=combine(u*diff(v,x));
    if type(var)="_equal" then # definite integration #
       # eval(subs(uv,x=op(var,[2,2])))-eval(subs(uv,x=op(var,[2,1]))) #
       subs(uv,x=op(var,[2,2]),Simplified)-subs(uv,x=op(var,[2,1]),Simplified)
	- hold(int)(udv, var)
    else
       uv - hold(int)(udv, var)
    end_if
end_proc:
