# $Date: 1999/08/04 09:08:15 $ $Author: fakler $ $Revision: 1.1.2.1 $ #

#++ ---------------- heaviInt.mu ---------------------

Author: Winfried Fakler,  University of Paderborn
Date Created:  7 July 1999
Last Update :  8 July 1999


Description:
This file contains functions for integrating (generalized) functions containing
the heaviside function.
							  
Functions: 

 - used Parameter:
    ++ f  :DOM_EXPR
    ++ x  :DOM_IDENT				  
    ++ f     = (complex) generalized function (distribution)
    ++ x     = indeterminate of f					  

 - intlib::heavisideInt(f,x)
    ++ returns FAIL or an (formal) antiderivative of the (generalized) function
    ++ f w.r.t. x, wherby the antiderivative F is defined as follows:
    ++
    ++          F:=int(g(x)*heaviside(p(x)),x)
    ++    and
    ++          F = int(g(x),x)*heaviside(p(x))-int(int(g(x),x)*dirac(p(x)),x)
    ++
    ++ When it is not possible to evaluate int(g(x),x) explicitly, FAIL is
    ++ is returned.
                    
Examples:
++#

intlib::heavisideInt:=
  proc(_f_,_x_) 
    local g,G,H,s;
  begin
    //print("entering heavisideInt with ",_f_,_x_);
    case type(_f_)
      of "_plus" do //print("case _plus");
        return( map(_f_, proc(e)
                       begin
                         if hastype(e,"heaviside") then
                            intlib::heavisideInt(e,_x_) 
                         else intlib::int(e,_x_) end_if
                       end_proc)); break;
      of "heaviside" do //print("case heaviside");
        g:=1; H:=_f_; break;
      of "_mult" do //print("case _mult");
        s:=split(_f_,hastype,"heaviside");
        if type(s[1])="heaviside" then H:=s[1]; g:=s[2]*s[3] else return(FAIL)
        end_if; break;
      otherwise //print("case otherwise");
        return(FAIL);
    end_case;

    // now _f_(_x_)=g(_x_)*heaviside(p(_x_))
    if degree(poly(op(H),[_x_]))=1 then 
      G:=int(g,_x_);
      if has(G,int) then FAIL
      elif has(G,heaviside) then //to avoid loops int(heaviside(.)*dirac(.)..)
        if _f_=G*dirac(op(H)) then G*H/2 else 0 end_if;
      else
        G*H-int(G*dirac(op(H)),_x_);
      end_if;
    else
      FAIL;
    end_if;
  end_proc:
