# $Date: 1999/09/14 14:16:41 $ $Author: fakler $ $Revision: 1.45.2.16 $ #
#--
intlib::int(f,x) - computes the integral of the function f(x) with respect to 
the variable x.
--#
intlib::int:=proc(_F_,_x_)
option remember;
local opt,n,k_sub,k_ident,k_diff,k_deg,k_alg,Q,Q_sub,i,integral,tosubs,hassincos,k,hasI,Float,_f_,_f_0,_x_0;
begin
# test of input and options #
  if testargs()
    then 
         if domtype(_F_)<>DOM_IDENT and domtype(_F_)<>DOM_EXPR
            and not testtype(_F_,NUMERIC)
           then error("Illegal integrand") end_if;
  end_if;
  _F_:=subs(_F_,_x_=_x_); # to put _F_ in normal form in case it is not #
# First, transform each floating point number to its rational approximation #
  _f_:= maprec( _F_, {DOM_FLOAT} = fun(
    (if domtype(args(1))=DOM_FLOAT then 
      sharelib::rational( args(1) )
    else args(1) 
    end_if)
  ));
  _f_0:=_f_; _x_0:=_x_; // keeping rationalized arguments
  Float:=(if _f_ <> _F_ then float else id end_if);
  hassincos:=has(_f_,{sin,cos,tan}); hasI:=has(_f_,I);
  // test if integrand contains special functions that can be treated
  integral:=intlib::specialFunctionInt(_f_,_x_);
  if integral<>FAIL then return( Float(integral) ) end_if;
  # lookup method #
  integral:=intlib::lookup(subs(_f_,_x_=_X));
  if integral<>FAIL then return( Float(subs(integral,_X=_x_)) ) end_if;
  opt:=hold(Exp);
  n:=1; k:=[_x_];
  k_sub:=[_x_]; k_ident:=[genident()]; k_diff:=[1]; k_deg:=[0]; k_alg:=[0];
  Q:=[]; Q_sub:=[];
# substitution and construction of the field extensions of the
  rational numbers #
  if hassincos then _f_:=combine(_f_,sincos) end_if; //produces a simpler tower
  _f_:=intlib::subst(_f_);
  if has(_f_,exp) then _f_:=combine(expand(_f_),exp) end_if;
  userinfo(2,"after substitution, integrand is",_f_);
  repeat i:=intlib::tower(_f_) until i<>FALSE end_repeat;
  if i=FAIL then
     userinfo(1,"construction of tower extension failed");
     //return(Float(intlib::lastChance(args())))
     integral:=FAIL;
  else
     userinfo(2,"field extension of rational numbers",Q,Q_sub);
     userinfo(2,"tower extension of functions",k,k_sub,k_alg);
     userinfo(2,"after field extensions, integrand is",i);
     integral:=intlib::intt(i,n);
  end_if;
  if integral=FAIL then
     # try to simplify the integrand #
     //assume(_x_,Type::RealNum);
     userinfo(4,"try to simplify integrand");
     i:=simplify(_F_);
     if i<>_f_ and not contains(intlib::tries,i) or has(i, sign) then
        i:=rationalize(i,fun(domtype(args(1))=DOM_EXPR and type(args(1))<>"sign"));
        if i[2]<>{} then # to avoid loops #
           sysassign(intlib::tries,intlib::tries union {subs(i[1],i[2])});
           n:=intlib::int(i[1],_x_);
           if type(n)<>"int" then 
		# return( Float(eval(subs(n,i[2]))) ) #
		return( Float(subs(n,i[2],Simplified)) )
	   end_if
        end_if;
     end_if;
     userinfo(4,"try to integrate term by term");
     return(Float(intlib::lastChance(_f_0,_x_0)))
  end_if;
  tosubs:=[k_sub[n-i]$hold(i)=0..n-2, Q_sub[nops(Q)-i]$hold(i)=0..nops(Q)-1];
  while has(integral,tosubs) do
     # take care with eval here, for example in int(1/(x^2+k^2),x) !!! #
     if traperror((integral:=#eval(#subs(integral,
                k_sub[n-i]=k[n-i]$hold(i)=0..n-2,
                Q_sub[nops(Q)-i]=Q[nops(Q)-i]$hold(i)=0..nops(Q)-1,
		hold(k)=hold(_intlib_k),hold(n)=hold(_intlib_n),
		hold(i)=hold(_intlib_i))#)#;))<>0 then
        userinfo(1,"something is wrong, probably there is an undetected ".
               "algebraic relation between some functions of the integrand.".
                "\nPerhaps rewriting the integrand e.g. with ".
                "'rewrite(integrand,tan)' will help.");
        return(hold(int)(_F_,_x_))
      end_if;
  end_while;
  integral:=subs(integral,hold(_intlib_k)=hold(k),hold(_intlib_n)=hold(n),
	hold(_intlib_i)=hold(i));
  userinfo(2,"after substitution, integral is",integral);
  if hasI=FALSE and has(integral,I) then # gets only the real part #
// original version:    
    // n:=sign(_x_);
    // sysassign(sign(_x_),1);
    // i:=rectform(integral,{_x_});
    // if op(i,3)=0 then # unresolved part #
    //	i:=op(i,1); # real part #
    //	if not has(i,{Re,Im,abs}) then integral:=i end_if;
    // end_if;
    // sysassign(sign(_x_),(if type(n)<>"sign" then n end_if)); # else NIL #
// end original version
// new version:
    if hastype(integral, "sum") then
      integral:=text2expr(expr2text(integral));
      integral:=proc(a) begin level(a) end_proc(integral);
    end_if;
    integral:=intlib::simplifyComplexIntegral(integral,_x_);
  end_if;
  if hassincos then
     intlib::lookupsin; # to load intlib::to_sincos #
     integral:=combine(expand(intlib::to_sincos(integral,_x_)),sincos)
  end_if;
  // simplifying subexpressions like exp(a*ln(x))
  if has({integral},exp) then
    integral:=(maprec(integral,{"exp"}=
                      proc() begin expand(exp(op(args()))) end_proc ))
  end_if;
  Float(integral)
end_proc:

# if both the lookup method and Risch algorithm failed,
  and if the function is a sum, it can be that some
  terms can be integrated by the lookup method and some
  other by Risch algorithm, like for example int(diff(g(x),x)+x+2,x) #
intlib::lastChance := proc(f,x)
local f0,x0;
begin
   f0:=f; x0:=x;
   if type(f)="_mult" and contains(map({op(f)},type),"_plus") then
      f:=expand(f)
   end_if;
   if type(f)="_plus" then
      f:=map(f,intlib::int,x);
      if not has(f,int) then return(f) end_if
   end_if;
   hold(int)(f0,x0)
end_proc:

intlib::tries:={}:


#++ ---------------- simpCInt.mu ---------------------

Author: Winfried Fakler,  University of Paderborn
Date Created: 13 December 1998
Last Update :  5 March    1999


Description:
This file contains functions to simplify complex elementary functions.
							  
Functions: 

 - used Parameter:
    ++ f,A,B :DOM_EXPR
    ++ x     :DOM_IDENT				  
    ++ f     = (complex) elementary function
    ++ A,B   = polynomial expressions in x over a field of characteristic zero
               not containing sqrt(-1)
    ++ x     = indeterminate of f					  

 - intlib::applyEulerIdentity(f,x <, Complete >)
    ++ applies Euler's indentity to all subexpression of f of type "_power"
    ++ which have complex exponents and containing x in the basis as well as
    ++ all subexpressions of type "exp" with complex argument.
    ++ If option Complete is choosen, subexpressions of f of type "_power" with
    ++ arbitrary basis are treated.
 - intlib::complexLnConversion(f,x)
    ++ tries to convert sums of complex logarithms to real functions using
    ++ Rioboo's conversion procedure intlib::lnToArctan and some special
    ++ simplifications.
 - intlib::lnToArctan(A,B,x)
    ++ returns a sum f of arctangents of polynomials over the same coefficient
    ++ field as A and B, such that
    ++                diff(f,x) = diff(I*ln((A+I*B)/(A-I*B)),x)
    ++ where is needed B<>0 and sqrt(-1) is not contained in the coefficient
    ++ field [1, p.63].
 - intlib::simplifyComplexIntegral(f,x)
    ++ tries to rewrite an elementary function which contains the complex I
    ++ into an equivalent real form.
    
See:
  [1] Bronstein, M. (1997). Symbolic Integration I: Transcendental Functions.
      Springer.

Examples:
>> i:=int(sin(ln(x-1)),x);
>> intlib::simplifyComplexIntegral(i,x);
>> intlib::applyEulerIdentity(i,x);
++#


intlib::simplifyComplexIntegral:=
  proc(integral,_x_) 
    local i,n;
  begin
    if has(integral, exp) then
      i:=rewrite(combine(integral,exp),sincos);
      if not has(i,I) then return(i) end_if;
    end_if;
    userinfo(6,"...apply Euler's identity");
     //this line is needed since otherwise int(1/(sin(x)^4+1),x) cannot treated
    if not hastype(integral, "sum") then integral:=expand(integral) end_if;
    integral:=intlib::applyEulerIdentity(integral,_x_);
    if has(integral, I) then
      if has(integral, ln) then
        //this is needed to prevent choosing a wrong branch in rectform
        userinfo(6,"...try to convert complex logarithms to real functions");
        integral:=intlib::complexLnConversion(integral,_x_)
      end_if;
      userinfo(6,"...apply rectform");
      // old version
      n:=sign(_x_);
      sysassign(sign(_x_),1);
      // traperror, since there might be problems within rectform
      if traperror((i:= rectform( integral,{_x_} );)#,20#)=0 then
      # try to get only the real part #
      //if op(i,3)=0 and iszero(simplify(op(i,2))) then # unresolved part? #
        if iszero(op(i,3)) /* and iszero(op(i,2)) */ then # unresolved part? #
	  i:=op(i,1); # real part #
	  if not has(i,{Re,Im,abs}) then integral:=i end_if;
        end_if;
      end_if;
      sysassign(sign(_x_),(if type(n)<>"sign" then n end_if)); # else NIL #
    end_if;
    integral
  end_proc:

sysassign(intlib::applyEulerIdentity,
  proc(_f_,_x_,o) local rationalizeFraction,subsExp,subsPower,J,total,f0;
  begin
    J:=genident("I");
    if o=hold(Complete) then total:=TRUE else total:=FALSE end_if;
    rationalizeFraction:=  //remove complex unit from denominator
    proc(e,J)
      local num,den,a,b;
    begin
      den:=subs(expand(eval(subs(denom(e),J=I))),I=J); //doing simplifications
      a:=coeff(den,[J],J,0);
      b:=coeff(den,[J],J,1);
      if a<>FAIL and b<>FAIL and b<>0 then
        num:=subs(expand(eval(subs(numer(e)*(a-b*J),J=I))),I=J);
        collect(num/(a^2+b^2),[J])
      else
        e
      end_if
    end_proc:
         
    subsPower:=
    proc()
      local basis,exponent,a,b;
    begin
      basis:=op(args(),1);
      exponent:=op(args(),2);
      if (has(basis,_x_) or total) and has(exponent,J) then
      //if has(basis,_x_) and has(exponent,J) then
        exponent:=rationalizeFraction(exponent,J);
	a:=coeff(exponent,[J],J,0);
	b:=coeff(exponent,[J],J,1);
        if a<>FAIL and b<>FAIL then
          _mult(basis^a,cos(b*ln(basis))+J*sin(b*ln(basis)))
        else
          _power(basis,exponent)
        end_if;
      else
	_power(basis,exponent)
      end_if;
    end_proc;

    subsExp:=
    proc()
      local exponent,a,b;
    begin
      exponent:=op(args(),1);
     //if has(exponent,J) and has(exponent,_x_) then
     if has(exponent,J) then
        exponent:=rationalizeFraction(exponent,J);
	a:=coeff(exponent,[J],J,0);
	b:=coeff(exponent,[J],J,1);
        if a<>FAIL and b<>FAIL then
	  _mult(exp(a),cos(b)+J*sin(b))
        else
          exp(exponent)
        end_if
      else
	exp(exponent)
      end_if;
    end_proc;

    _f_:=eval(subs(maprec(subs(_f_,I=J),{"_power"}=subsPower),J=I));
    if has(_f_, exp) then
      _f_:=eval(subs(maprec(subs(_f_,I=J),{"exp"}=subsExp),J=I));
    end_if;
// in the following lines problems occured in traperror with time bound        
    if has(_f_,I) then
//       // traperror, since there might be problems within simplification
//       if traperror((f0:=simplify(_f_))#,20#)=0 then f0 else _f_ end_if;
      if traperror((f0:=intlib::simpleRectform(_f_)))=0 then f0 
      else _f_ end_if;
      //_f_
    else _f_ end_if;
  end_proc):

sysassign(intlib::simpleRectform,
  proc(_f_srf)
    local J,num,den,fexpr,a,b;
  begin
    J:=genident("I");
    fexpr:=subs(_f_srf,I=J); 
    if testtype(fexpr,Type::RatExpr(J)) then
      den:=divide(denom(fexpr),J^2+1,[J],Rem);
      a:=coeff(den,[J],J,0);
      b:=coeff(den,[J],J,1);
      if a<>FAIL and b<>FAIL and b<>0 then
        num:=divide(numer(fexpr)*(a-b*J),J^2+1,[J],Rem);
        den:=a^2+b^2;
      else // denominator is already in real form
        num:=divide(numer(fexpr),J^2+1,[J],Rem);
      end_if;
      // this condition is used to prevent time expensive simplification
      if length(num)+length(den)<600 then //subs should be unnecessary
        subs(expr(mapcoeffs(poly(num,[J]),simplify)),J=I)/simplify(den); 
      else
        _f_srf // should return subs(collect(num,[J])/den,J=I); but this can be
            // too large
      end_if;      
    else
      _f_srf
    end_if;      
  end_proc):


sysassign(intlib::lnToArctan,
  proc(A,B,_x_)
    local g;
  begin
    if divide(A,B,[_x_],Rem)=0 then return(2*atan(A/B))
    elif degree(A,[_x_],_x_)<degree(B,[_x_],_x_) then
      return(intlib::lnToArctan(-B,A,_x_))
    else
      g:=gcdex(B,-A,_x_);
      2*atan(normal((A*g[2]+B*g[3])/g[1]))+intlib::lnToArctan(g[2],g[3],_x_)
    end_if
  end_proc):

sysassign(intlib::complexLnConversion,
  proc(s,_x_)
    local J,sl,splitLnParts,lnPart,pos;
  begin
    
    // s - expression
    // J - variable representing the complex unit I
    // splits s into parts consisting of ln's of the form c*ln(A+I*B) and
    // returns a list of 3 lists: the first list includes all such ln's where
    // c is imaginary, the third list where c is real part and the second list
    // includes all other subexpressions.
    // Note: The third list could removed, when the simplifier will improved!
    splitLnParts:=proc(s,J)
                    local r,sl;
                  begin
                    r:=[[],[],[]];
                    map([op(expand(s))],
                        proc(e)
                          local f,A,B,h,cRe,cIm;
                        begin
                          if testtype(e,"_mult") then 
                           sl:=split([op(e)], testtype, "ln")
                          else
                            r[2]:=append(r[2],e);
                            return(r)
                          end_if;
                           case nops(sl[1])
                             of 1 do 
                               f:=_mult(op(sl[2]));
                               cRe:=coeff(f,[J],J,0);
                               cIm:=coeff(f,[J],J,1);
                               h:=op(op(sl[1]));
                               A:=coeff(h,[J],J,0);
                               B:=coeff(h,[J],J,1); 
                               if /*A=0 or*/ B=0 //or cIm=0
                                 //Note: the following two conditions are
                                 //due to the fact that e.g. poly(1/x,[x])
                                 //returns FAIL and thus would cause an error
                                 //in intlib::lnToArctan
                                  or not(testtype(A,Type::PolyExpr(_x_)))
                                  or not(testtype(B,Type::PolyExpr(_x_)))
                                 then r[2]:=append(r[2],e); break;
                               else
                                 if cRe<>0 then
                                  r[3]:=append(r[3],table("A"=A,"B"=B,"c"=cRe))
                                 end_if;
                                 if cIm<>0 then 
                                  r[1]:=append(r[1],table("A"=A,"B"=B,"c"=cIm))
                                 end_if;
                               end_if; break;
                             otherwise r[2]:=append(r[2],e)
                           end_case
                        end_proc
                        );
                    r
                  end_proc:

    J:=genident("I");
    lnPart:=0;
    sl:=splitLnParts(subs(s,I=J),J);
    if sl[1]=[] and sl[3]=[] then return(s)
    else
      // handle first list
      while sl[1]<>[] do
        pos:=contains(sl[1],table("A"=sl[1][1]["A"],"B"=-sl[1][1]["B"],
                                  "c"=-sl[1][1]["c"]));
        if pos>0 then
          if sl[1][1]["c"]=abs(sl[1][1]["c"]) then
            lnPart:=lnPart+sl[1][1]["c"]*
                     intlib::lnToArctan(sl[1][1]["A"],sl[1][1]["B"],_x_);
          else
            lnPart:=lnPart+sl[1][pos]["c"]*
                     intlib::lnToArctan(sl[1][pos]["A"],sl[1][pos]["B"],_x_);
          end_if;
          unassign(sl[1][pos]);
          unassign(sl[1][1]);
        else
          sl[2]:=append(sl[2],
                        sl[1][1]["c"]*I*ln(sl[1][1]["A"]+sl[1][1]["B"]*I));
          unassign(sl[1][1]);
        end_if
      end_while;
      // handle third list
      while sl[3]<>[] do
        pos:=contains(sl[3],table("A"=sl[3][1]["A"],"B"=-sl[3][1]["B"],
                                  "c"=sl[3][1]["c"]));
        if pos>0 then
          lnPart:=lnPart+sl[3][1]["c"]*ln(sl[3][1]["A"]^2+sl[3][1]["B"]^2);
          unassign(sl[3][pos]);
          unassign(sl[3][1]);
        else
          sl[2]:=append(sl[2],
                        sl[3][1]["c"]*ln(sl[3][1]["A"]+sl[3][1]["B"]*I));
          unassign(sl[3][1]);
        end_if
      end_while;
      if sl[2]=[] then lnPart
      else subs(_plus(lnPart,op(sl[2])),J=I)
      end_if
    end_if
  end_proc):
