# intlib::lookup(e) returns FAIL or a primitive, the main variable is _X.

  It is possible to define an "int" attribute for a given function.
  It works as follows: int(F(g(x)),x) calls the "int" attribute of F
  with the arguments F(g(x)) and x. The "int" attribute should return
  either the integral of FAIL.
#

# _X is the integration variable #
intlib::lookup := fun((
   # remove independent factors #
   if type(args(1))="_mult" then
      select(args(1),fun(not has(args(1),_X)))*intlib::lookup2(select(args(1),has,_X))
   else intlib::lookup2(args(1))
   end_if)):

alias(indep(a,x)=_pattern(hold(a),not has,x)):

# functions 'xx' for which intlib::lookupxx exists #
intlib::Functions:={"asin","asinh","acos","acosh","atan","besselJ","cos","exp",
"ln","sin","tan"}:
hastype: # to load Indets #
_dir:=pathname("INTLIB","LOOKUP"):
intlib::lookupasin:=loadproc(intlib::lookupasin,_dir,"asin"):
intlib::lookupasinh:=loadproc(intlib::lookupasinh,_dir,"asinh"):
intlib::lookupacos:=loadproc(intlib::lookupacos,_dir,"acos"):
intlib::lookupacosh:=loadproc(intlib::lookupacosh,_dir,"acosh"):
intlib::lookupatan:=loadproc(intlib::lookupatan,_dir,"atan"):
intlib::lookupbesselJ:=loadproc(intlib::lookupbesselJ,_dir,"besselJ"):
intlib::lookupcos:=loadproc(intlib::lookupcos,_dir,"cos"):
intlib::lookupexp:=loadproc(intlib::lookupexp,_dir,"exp"):
intlib::lookupln:=loadproc(intlib::lookupln,_dir,"ln"):
intlib::lookupsin:=loadproc(intlib::lookupsin,_dir,"sin"):
intlib::lookuptan:=loadproc(intlib::lookuptan,_dir,"tan"):

# if f is a product, all factors depend on _X #
intlib::lookup2 := proc(_f_) local g,intg,a,xx,p,q,f,dg,i,b,c,d,m;
begin
   if not has(_f_,_X) then return(_f_*_X) end_if;
   for g in select(Indets(_f_,intlib::Functions),has,_X) do
      i:=domattr(intlib,"lookup".type(g))(_f_);
      if i<>FAIL then return(i) end_if
   end_for;
   _f_:=maprec(_f_,{"_power"}=fun((if type(op(args(1),1))="_power" and
	type(op(args(1),[1,2]))=DOM_INT and type(op(args(1),2))=DOM_RAT then
	op(args(1),[1,1])^(op(args(1),[1,2])*op(args(1),2)) 
	else args(1) end_if)));
   # first do pattern matching #
   case type(_f_)
   of "_power" do
      if op(_f_,2)=1/2 then return((
         if testtype(op(_f_,1),Type::PolyExpr(_X)) then
            intlib::looksqrt(1,op(_f_,1),_X)
         elif intlib::matchassign(op(_f_,1),_X/(indep(a,_X)-_X^3)) then
	    2/3*asin(_X^(3/2)/a^(1/2))
         elif intlib::matchassign(op(_f_,1),(indep(a,_X)+_X)/(indep(a,_X)-_X))
         then (_X-a)*_f_+a*atan(_X*_f_/(a+_X))
         else FAIL end_if))
      elif op(_f_,2)=-1/2 then return((
         if testtype(op(_f_,1),Type::PolyExpr(_X)) then
	    intlib::looksqrt2(1,op(_f_,1),_X)
         else FAIL end_if))
      elif op(_f_,2)=3/2 and testtype(op(_f_,1),Type::PolyExpr(_X)) then
         return(intlib::looksqrt(op(_f_,1),op(_f_,1),_X))
      elif op(_f_,2)=-1 and intlib::matchassign(op(_f_,1),indep(b,_X)*_X^2+
	indep(b,_X)*_X+indep(a,_X)) then 
		(4*a*b-b^2)^(1/2); return(2*atan(b*(2*_X+1)/%)/%)
      end_if; break
   of "_mult" do
      for g in _f_ do
         if g=_X then next # to avoid infinite loops #
         elif contains({"function","exp"},type(g)) and nops(g)=1 then
            # try to recognize int(a*F(f(x))*diff(f(x),x),x) = a*int(F(u),u) #
            if not has((a:=_f_/g/diff(op(g),_X)),_X) then
               intlib::int(subsop(g,1=_X),_X);
               return((if type(%)<>"int" then return(a*subs(%,_X=op(g))) 
		else FAIL end_if))
            end_if
         elif type(g)="diff" then
            # int a(g) D(g) dx = int a(t) dt #
               if nops(g)=2 then intg:=op(g,1) 
               else intg:=subsop(g,nops(g)=null())
               end_if;
               if op(g,nops(g))=_X and not has((a:=subs(_f_/g,intg=xx)),_X)
               then return(subs(intlib::int(a,xx),xx=intg))
               end_if
         elif iszero((dg:=diff(g,_X))) then
         elif not has((a:=_f_/dg),_X) then return(a*g)
         elif not has((a:=normal(subs(a,g=xx))),_X) then
	    return(subs(intlib::int(a,xx),xx=g))
         end_if
      end_for;
      # transform sqrt(a/b)*(c/d) into normal form sqrt(a*b)*(c/d/b) #
      if intlib::matchassign(_f_,_pattern(p)*_pattern(q)^(1/2)) then
         p*(expand(numer(q)*denom(q)))^(1/2)/denom(q);
         if _f_<>% then return(intlib::lookup2(%)) end_if
      end_if;
      # case g(x)*sqrt(a+c*x+b*x^2) #
      if intlib::matchassign(_f_,sqrt(indep(a,_X)+indep(c,_X)*_X+indep(b,_X)
	*_X^2)*_pattern(hold(g))) then
         p:=(a+c*_X+b*_X^2)^(1/2);
         return((if testtype(g,Type::PolyExpr(_X)) then
	    intlib::looksqrt(g,a+c*_X+b*_X^2,_X)
         # case x^m*sqrt(a+b*x) #
         elif b=0 and intlib::matchassign(g,_X^_pattern(hold(m),is,
		Type::NonNegInt)) then 2/c^(m+1)*p*hold(sum)((-a)^(m-i)*
			(a+c*_X)^(1+i)*binomial(m,i)/(3+2*i),i=0..m)
         # case x^m*sqrt(c*x-x^2) #
         elif a=0 and b=-1 and intlib::matchassign(g,_X^_pattern(hold(m),is,
		Type::NonNegInt)) then
	    a:=c/2; a^(m+2)*asin((_X-a)/a)*binomial(2*m+1,m)/(m+2)/2^m
		+p*(_X^(m+1)/(m+2)-sum(2^(i-m)*a^(m+1-i)*_X^i*binomial(2*m+1,m)
		/(m+2)/binomial(2*i+1,i)/(i+1),i=0..m))
         elif c<>0 then FAIL
         elif g=1/_X then p-a^(1/2)*atanh(a^(1/2)/p)
         elif g=1/_X^2 then -p^3/a/_X+b*_X*p/a+b^(1/2)*ln(b^(1/2)*_X+p)
         elif g=1/_X^3 then -p^3/a/_X^2/2+b/a/2*(p-a^(1/2)*atanh(a^(1/2)/p))
         elif not has((c:=_X^2-1/g),_X) then # g=1/(x^2-c) #
            if type(c)="_power" and op(c,2)=2 then c:=op(c,1) else c:=c^(1/2)
            end_if;
            p:=c+_X; q:=_X-c; g:=(b*c^2+a)^(1/2);
            d:=(b*p^2-2*b*c*p+b*c^2+a)^(1/2);
            m:=(b*q^2+2*b*c*q+b*c^2+a)^(1/2);
            i:=b^(1/2);
            m/2/c+i/2*ln(i*_X+m)-g/2/c*atanh((b*c^2+a+b*c*q)/g/m)
	    -d/2/c+i/2*ln(i*_X+d)+g/2/c*atanh((b*c^2+a-b*c*p)/g/d)
         else FAIL
         end_if))
      # case g(x)/sqrt(a+d*x+b*x^2) #
      elif intlib::matchassign(_f_,_pattern(hold(g))/(indep(b,_X)*_X^2
	+indep(d,_X)*_X+indep(a,_X))^(1/2)) then
         p:=(a+d*_X+b*_X^2)^(1/2);
         return((if testtype(g,Type::PolyExpr(_X)) then
	    intlib::looksqrt2(g,a+d*_X+b*_X^2,_X)
         elif g=1/_X then
            if a=0 then -2*p/d/_X 
            else -atanh((a+d*_X/2)/p/a^(1/2))/a^(1/2) end_if
         elif b=0 and intlib::matchassign(g,(indep(c,_X)
		+_X*indep(b,_X))^indep(m,_X)) then 
		if m=-1 then -2*atan((-b)^(1/2)*p/(-d*c+a*b)^(1/2))
			/(a*b-d*c)^(1/2)/(-b)^(1/2)
                elif is(m,Type::NonNegInt)=TRUE then
		   2*p*fact(m)^2/d/fact(1+2*m)*hold(sum)((4*c
			-4*a*b/d)^(m-i)*(c+b*_X)^i*binomial(2*i,i),i=0..m)
                else FAIL end_if
         elif d=0 and (not has((c:=1/g+_X^2),_X) or not has((c:=1/g-_X^2),_X))
	 then
            if type(c)="_power" and op(c,2)=2 then g:=op(c,1) else g:=c^(1/2)
	    end_if;
            if iszero(c) then -1/_f_/_X^3/a
            else
               d:=(b*c+a)^(1/2); p:=_X+g; q:=_X-g;
               atanh((b*c+a+b*g*q)/d/(b*q^2+2*b*g*q+b*c+a)^(1/2));
               (%-subs(%,q=-p))/2/d/g;
               if c=1/g+_X^2 then % else -% end_if
            end_if
         elif a=0 and b=-1 then
            if intlib::matchassign(g,_X^_pattern(hold(m),is,Type::NonNegInt))
            then a:=d/2; binomial(2*m,m)*(a/2)^m*(asin(_X/a-1)-p*sum((2/a)^i
		*_X^(i-1)/binomial(2*i,i)/i,i=1..m))
            elif intlib::matchassign(g,1/_X^_pattern(hold(m),is,Type::PosInt))
            then a:=d/2; -p*sum((a/2)^(i-m)*_X^(-1-i)*binomial(2*i,i)
		/binomial(2*m,m)/m,i=0..m-1)
            else FAIL end_if
         else FAIL
         end_if))
      # case 1/x/sqrt(c+a*x^m) #
      elif intlib::matchassign(_f_,(indep(a,_X)*_X^indep(m,_X)-indep(c,_X))
	^(-1/2)/_X) then return(2*atan((a*_X^m-c)^(1/2)/c^(1/2))/m/c^(1/2))
      # case x^2*(a+b*x^2)^(3/2) #
      elif intlib::matchassign(_f_,_X^2*(indep(a,_X)+indep(b,_X)
	*_X^2)^(3/2)) then
         p:=(a+b*_X^2)^(1/2);
         return(_X*p/b*(p^4/6-a*p^2/24-a^2/16)-a^3*ln(b^(1/2)*_X+p)/b^(3/2)/16)
      # case x^2/(a+b*x^2)^(3/2) #
      elif intlib::matchassign(_f_,_pattern(hold(g))
	/(indep(a,_X)+indep(b,_X)*_X^2)^(3/2)) then
         p:=(a+b*_X^2)^(1/2); 
         return((if g=_X^2 then -_X/b/p+ln(b^(1/2)*_X+p)/b^(3/2)
         elif g=1/_X then 1/a/p-atanh(a^(1/2)/p)/a^(3/2)
         elif g=1/_X^3 then
            -1/a/_X^2/p/2-3/2*b/a*(1/a/p-atanh(a^(1/2)/p)/a^(3/2))
         else FAIL end_if))
      # try Bertrand's algorithm for hyperelliptic functions #
      elif (g:=intlib::match(_f_,_pattern(p,Type::PolyExpr(_X,Type::Rational))/
	_pattern(q,Type::PolyExpr(_X,Type::Rational))/
	sqrt(_pattern(f,Type::PolyExpr(_X,Type::Rational)))))<>FAIL then
         userinfo(1,"recognized hyperelliptic integral");
         # check the conditions #
         p:=subs(p,g); q:=subs(q,g); f:=subs(f,g);
         if degree(f,_X) mod 2 = 1 and gcd(q,f,_X)=1 and gcd(p,q,_X)=1
         and gcd(f,diff(f,_X),_X)=1 and gcd(q,diff(q,_X),_X)=1 then
            return(intlib::hyperint(p,q,f,_X))
         end_if
      end_if;
      break
   of "diff" do
      if op(_f_,2)=_X then
         return((if nops(_f_)=2 then op(_f_,1) else subsop(_f_,2=null()) end_if))
      end_if
   otherwise # try to use "int" attributes #
      g:=op(_f_,0);
      if type(g)="D" then
         if nops(g)=1 then g:=D([1],op(g)) end_if;
         for a from 1 to nops(op(g,1)) do
            if op(_f_,op(g,[1,a]))=_X 
	    and not has(subsop([op(_f_)],op(g,[1,a])=NIL),_X)
	    then return(D(subsop(op(g,1),a=null()),op(g,2))(op(_f_)))
            end_if
         end_for
      elif type(level(g,2))=DOM_FUNC_ENV then
	 g:=funcattr(level(g,2),"int");
	 if g<>FAIL then
	    g:=g(_f_,_X);
	    if g<>FAIL then return(g) end_if
	 end_if
      end_if; break
   end_case;
   # tries to map on sums #
   # call intlib::lookup here to ensure that the factors
     independent of _X are removed #
   if type(_f_)="_plus" then map(_f_,intlib::lookup)
   else FAIL
   end_if
end_proc:

intlib::lookup2(hold(abs(_X))) := hold(_X*abs(_X)/2):
intlib::lookup2(1/sqrt(1-_X^2)) := hold(asin(_X)):
intlib::lookup2(1/sqrt(_X^2-1)) := hold(acosh(_X)):
intlib::lookup2(sign(_X)) := hold(_X*sign(_X)):
intlib::lookup2((1-_X^2)^(1/2)):=1/2*_X*(1-_X^2)^(1/2)+1/2*asin(_X):
intlib::lookup2(1/_X/sqrt(_X^2-1)):=hold(-atan(1/sqrt(_X^2-1))):
intlib::lookup2(1/sqrt(_X^2+1)):=asinh(_X):
intlib::lookup2(1/_X^3/sqrt(_X^2+1)):=-sqrt(_X^2+1)/2/_X^2+atanh(sqrt(_X^2+1))/2:
intlib::lookup2(1/_X^3/sqrt(_X^2-1)):=hold(sqrt(_X^2-1)/2/_X^2-atan(1/sqrt(_X^2-1))/2):
intlib::lookup2(1/_X^3/(1-_X^2)^(1/2)):=-(1-_X^2)^(1/2)/2/_X^2
	-atanh(1/(1-_X^2)^(1/2))/2:
intlib::lookup2((1+_X^2)/(1-_X^2)/(1+_X^4)^(1/2)):=
	ln((2^(1/2)*_X+(1+_X^4)^(1/2))/(1-_X^2))/2^(1/2):
intlib::lookup2((1-_X^2)/(1+_X^2)/(1+_X^4)^(1/2)):=
	hold(atan(2^(1/2)*_X/(1+_X^4)^(1/2))/2^(1/2)):
intlib::lookup2(1/(_X*(-1/2) + (_X^2-4)^(1/2)*1/2)):=
	_X^2*(-1/4) + _X*(_X^2-4)^(1/2)*(-1/4) + ln(_X + (_X^2-4)^(1/2)):
intlib::lookup2(1/(_X*(-1/2) + (_X^2-4)^(1/2)*(-1/2))):=
	-1/4*_X^2+1/4*_X*(_X^2-4)^(1/2)-ln(_X+(_X^2-4)^(1/2)):
intlib::lookup2((_X^2-1)^(1/2)/(_X^2*2-2)):=1/2*ln(_X+(_X^2-1)^(1/2)):
intlib::lookup2((_X^2-1)^(1/2)/(-_X^2*2+2)):=-1/2*ln(_X+(_X^2-1)^(1/2)):
	
intlib::looksqrt:=loadproc(intlib::looksqrt,pathname("INTLIB"),"looksqrt"):
intlib::looksqrt2:=loadproc(intlib::looksqrt2,pathname("INTLIB"),"looksqrt"):
