# $Date: 1999/08/04 09:07:43 $ $Author: fakler $ $Revision: 1.1.2.1 $ #

#++ ---------------- specfunc.mu ---------------------

Author: Winfried Fakler,  University of Paderborn
Date Created:  29 July 1999
Last Update :  29 July 1999


Description:
This file contains functions for integrating generalized functions as well as
special functions and "functions" like "big Oh".
							  
Functions: 

 - used Parameter:
    ++ f,a,b :DOM_EXPR
    ++ x     :DOM_IDENT				  
    ++ f     = (complex) generalized function 
    ++ x     = indeterminate of f					  
    ++ a,b   = constant expression w.r.t. x, representing lower and upper limit

 - intlib::specialFunctionInt(f,x)
    ++ returns FAIL or an (formal) antiderivative of the generalized function
    ++ f w.r.t. x.                               
 - intlib::specialFunctionDef(f,x,a,b) 
    ++ returns FAIL or the definite integral of the generalized function f 
    ++ w.r.t. x in the range of a to b.
 - intlib::bigOhInt(f,x)
    ++ returns the integral unevaluated or an formal antiderivative of f
    ++ w.r.t. x, if it contains an order term.

Examples:
++#

intlib::specialFunctionDef:=
  proc(_f_,_x_,a,b)
    local pv;
  begin
    userinfo(4,"check for special functions");
    if args(0)=5 then pv:=args(5) else pv:=FALSE end_if;
    
    // big Oh
    if hastype(_f_,O) then
      return(hold(int)(_f_,_x_=a..b,(if pv then PrincipalValue
                                 else null() end_if)))
    end_if;

    // generalized functions
    if has(_f_,dirac) then return(intlib::diracDef(_f_,_x_,a,b,pv));
      // heaviside case is treated by indefinite integration 
    end_if;

    // otherwise
    FAIL;
  end_proc:

intlib::specialFunctionInt:=
  proc(_f_,_x_)
  begin
    userinfo(4,"check for special functions");
    
    // big Oh
    if hastype(_f_,O) then return(intlib::bigOhInt(_f_,_x_)) end_if;

    // generalized functions
    if has(_f_,dirac) then return(intlib::diracInt(_f_,_x_));
    elif has(_f_,heaviside) then return(intlib::heavisideInt(_f_,_x_));
    end_if;

    // otherwise
    FAIL;
  end_proc:


intlib::bigOhInt:=
  proc(_f_,_x_) 
    local g;
  begin
    //print("entering bigOhInt with ",_f_,_x_);
    case type(_f_)
      of "_plus" do //print("case _plus");
        return( map(_f_, proc(e)
                       begin
                         if hastype(e,O) then intlib::bigOhInt(e,_x_) 
                         else intlib::int(e,_x_) end_if
                       end_proc)); break;
      of O do break; //print("case O");
      otherwise //print("case otherwise");
        return(hold(int)(_f_,_x_)); 
    end_case;

    // now _f_(_x_)=O(g(_x_))
    g:=op(_f_,1);
    O::create(int(g,_x_),_f_::points(_f_));
  end_proc:

