#-

io::readdata - reads data files

Calling sequence:

io::readdata(filename [, seperator] [, NonNested] )

Parameter:

filename  - non-empty string
seperator - optional string , strlen(sperator) = 1
NonNested - optional name

Summary:

io::readdata is used to read data files produced by external programs.
Any data must be a valid MuPAD expression. In contrast to finput
the data must not be ended by a colon or semicolon. The data is
seperated by seperator. The default seperator is a white space.
If NonNested is used, the result will be a list containing all
data. 

Examples:

File data:
1 2 3
4 5 6
7 8 9

>> io::readdata("data");

        [ [1, 2, 3], [4, 5, 6], [7, 8, 9] ]	

>> io::readdata("data", NonNested);

	[ 1, 2, 3, 4, 5, 6, 7, 8, 9 ]


File data:

1|2|3

4|5|6.65786

7|8|9| 5 | "ahfjd" | ab

>> io::readdata("data","|");

          [[1, 2, 3], [4, 5, 6.65786], [7, 8, 9, 5, "ahfjd", ab]]
		
See also:

read, finput, fread, ftextinput, text2expr, text2list

-#

io::readdata := proc(filename, seperator, opt)
   local err, fd, l, line, nested, r, t, i;
begin
   nested := TRUE;

   case args(0)
     of 3 do 
        if opt = NonNested then
           nested := FALSE
        else
           error("Illegeal 3rd argument")
        end_if;
     of 2 do 
        if seperator = NonNested then
           nested := FALSE
	elif not (testtype(seperator, DOM_STRING) and 
	     strlen(seperator) = 1) then
	     error("Illegal seperator")
	end_if
     of 1 do
        if not (testtype(filename, DOM_STRING) and 
	   strlen(filename) > 1 ) then
	   error("Illegal filename")
	end_if;
	break
     otherwise
	error("Wrong number of arguments")
   end_case;
   
   if (fd := fopen(filename)) = FAIL then
      error("Cant open file ".filename)
   end_if;
   if not testtype(seperator, DOM_STRING) then
      seperator := " "
   end_if;
   r := [];

   err := traperror((
     while type((line:=ftextinput(fd))) <> DOM_NULL do
       t := text2list(line, [seperator]); l := [];
       for i from 1 to nops(t) step 2 do
           if t[i] <> "" then
              l := append(l, eval(text2expr(t[i])))
           end_if
       end_for;
       if l <> [] then
          if nested then
             r := append(r, l)
          else
             r := r.l
          end_if
       end_if
     end_while
    ));

    fclose(fd);
    if err <> 0 then
       error("Cant convert data")
    end_if;
    r
end_proc:
