#
>> io::readlisp("(INTEGRATE (EXPT X -1) X)");

                                   / 1    \
                                int| -, X |
                                   \ X    /

>> io::readlisp("(BESSEL-J X 1)");

                               besselJ(1, X)
#
io::readlisp:=proc(s#string#) local n;
begin
   io::parseLisp([substring(s,n,1)$n=0..strlen(s)-1])[1]
end_proc:

# io::parseLisp(l) parses a Lisp expression and returns this expression
  together with the remaining stream.
Example:
>> io::parseLisp(["(","E","X","P","T"," ","X"," ","-","1",")"," ","X",")"]);

                         -- 1                  --
                         |  -, [" ", "X", ")"]  |
                         -- X                  --
#
io::LispBlank:={" ","\n","\t"}:

io::parseLisp:=proc(l)
begin
   while l<>[] and contains(io::LispBlank,l[1]) do l[1]:=NIL end_while;
   if l[1]="(" then io::parseLambda(l)
   else io::parseIdent(l)
   end_if
end_proc:

io::parseLambda:=proc(l) local f,Args,g;
begin
   while l<>[] and l[1]<>"(" do l[1]:=NIL end_while;
   if l[1]<>"(" then error("missing open parenthesis") end_if;
   l[1]:=NIL;
   ([f,l]):=io::parseIdent(l);
   while l<>[] and contains(io::LispBlank,l[1]) do l[1]:=NIL end_while;
   Args:=null();
   while l<>[] do
      while l<>[] and contains(io::LispBlank,l[1]) do l[1]:=NIL end_while;
      if l=[] then error("missing closing parenthesis")
      elif l[1]=")" then l[1]:=NIL;
         if domtype(io::lispFun2MuPAD(f))=DOM_EXEC then
            g:=io::lispFun2MuPAD(f)
         else g:=f
         end_if;
         return([g(Args),l])
      end_if;
      g:=io::parseLisp(l);
      Args:=Args,g[1];
      l:=g[2];
   end_while;
end_proc:

io::parseIdent:=proc(l) local f;
begin
   f:="";
   while l<>[] and not contains(io::LispBlank union {"(",")"},l[1]) 
   do f:=f.l[1]; l[1]:=NIL end_while;
   f:=io::Ident2MuPAD(f);
   [f,l]
end_proc:

io::Ident2MuPAD:=proc(s) begin text2expr(s) end_proc:
io::Ident2MuPAD("*"):=_mult:
io::Ident2MuPAD("+"):=_plus:
io::Ident2MuPAD("/"):=divide:
io::Ident2MuPAD("ARCCOS"):=acos:
io::Ident2MuPAD("ARCCOSECH"):=acsch:
io::Ident2MuPAD("ARCCOSH"):=acosh:
io::Ident2MuPAD("ARCCOT"):=acot:
io::Ident2MuPAD("ARCCOTH"):=acoth:
io::Ident2MuPAD("ARCCSC"):=acsc:
io::Ident2MuPAD("ARCSEC"):=asec:
io::Ident2MuPAD("ARCSECH"):=asech:
io::Ident2MuPAD("ARCSIN"):=asin:
io::Ident2MuPAD("ARCSINH"):=asinh:
io::Ident2MuPAD("ARCTAN"):=atan:
io::Ident2MuPAD("ARCTANH"):=atanh:
io::Ident2MuPAD("BESSEL-J"):=besselJ:
io::Ident2MuPAD("COS"):=cos:
io::Ident2MuPAD("COSEC"):=csc:
io::Ident2MuPAD("COSECH"):=csch:
io::Ident2MuPAD("COSH"):=cosh:
io::Ident2MuPAD("COT"):=cot:
io::Ident2MuPAD("COTH"):=coth:
io::Ident2MuPAD("CSC"):=csc:
io::Ident2MuPAD("CSCH"):=csch:
io::Ident2MuPAD("ERF"):=erf:
io::Ident2MuPAD("EXP"):=exp:
io::Ident2MuPAD("EXPT"):=_power:
io::Ident2MuPAD("GAMMA"):=gamma:
io::Ident2MuPAD("INF"):=infinity:
io::Ident2MuPAD("LOG"):=ln:
io::Ident2MuPAD("MINF"):=-infinity:
io::Ident2MuPAD("SEC"):=sec:
io::Ident2MuPAD("SECH"):=sech:
io::Ident2MuPAD("SIN"):=sin:
io::Ident2MuPAD("SINH"):=sinh:
io::Ident2MuPAD("SQRT"):=sqrt:
io::Ident2MuPAD("TAN"):=tan:
io::Ident2MuPAD("TANH"):=tanh:

io::lispFun2MuPAD(besselJ):=fun((besselJ(args(2),args(1)))):

io::lispFun2MuPAD(divide):=fun(args(1)/args(2)):

io::lispFun2MuPAD(hold(HYPERGEOMETRIC)):=fun((
   args(0); contains([args()],hold(OVER));
   hypergeom([args(1..%-1)],[args(%+1..%2-1)],args(%2))
)):

io::lispFun2MuPAD(hold(INTEGRATE)):=fun((
   if args(0)=2 then hold(int)(args())
   else hold(int)(args(1),args(2)=args(3)..args(4))
   end_if)):

