# #
# $Date: 1997/12/16 11:32:47 $  $Author: wseiler $   $Revision: 1.5 $ #
# #

#++
the library 'listtools' implements some auxiliary methods for lists. It comprises:

        removeDuplicates(l) -- removes duplicate elements from a list; only
                the last occurance is kept. The ordering of the remaining
                elements is not changed.

	removeDupSorted(l) -- removes duplicate elements from a sorted list;
		faster than removeDuplicates if there are not many duplicates
		and the list is rather long.

        sort_desc(l) -- brings l in descending order, i.e. the reverse of
                what sort does.

        merge(l1,l2) resp. merge(l1,l2,less) -- merges two sorted lists; the
                optional third argument is a boolean function according to
                which the lists must be sorted (this is not checked!).

	singleMerge(l1,l2) resp. singleMerge(l1,l2,less) -- merges two sorted
		lists; if elements occur in both lists, they are taken only
		once. Thus if l1 and l2 contain no duplicates, the results
		will also contain no duplicates. The third argument has the
		same effect as in merge.

        setDifference(l1,l2) -- returns all elements of l1 which are not
                elements of l2. Works also for sets. Return type is the
                type of l1.

Authors: Eduard Niess, Werner M. Seiler, Uni Karlsruhe
Date created: May 23rd, 1996
Last update: December 15th, 1997
++#

proc()
    option noDebug;
    local path;
begin
    listtools:= domain("listtools"):
    listtools::info:= "Library 'listtools': Utilities for list operations":
    listtools::interface:= { 
        hold(removeDuplicates), hold(merge), hold(setDifference)
    }:
    
    path := pathname("LIST"):

    listtools::removeDuplicates:= loadproc( listtools::removeDuplicates, path, "mvdup" ):
    listtools::removeDupSorted:= loadproc( listtools::removeDupSorted, path, "mvdups" ):
    listtools::merge:= loadproc( listtools::merge, path, "merge" ):
    listtools::singleMerge:= loadproc( listtools::singleMerge, path, "smerge" ):
    listtools::setDifference:= loadproc( listtools::setDifference, path, "setdiff" ):

    stdlib::LoadlibState["listtools"]:= TRUE:
    null()
end_proc():

# end of file #
