# $Date: 1997/10/16 10:52:57 $ $Author: olli $ $Revision: 1.3 $ #

proc()
    option noDebug;
begin

#--------------------------------------------------------------------#
# Interface fuer Semaphoren und Locks                                #
# Beide Domains als uebergeordnete Schicht der low-level Routinen    #
# init_sem, semV, und semP.                                          #
#--------------------------------------------------------------------#

Semaphore            := domain("Semaphore"):
Semaphore::info      := "Routines for using semaphores.":
Semaphore::interface := { hold(sinit), hold(V), hold(P), hold(N) }:

Semaphore::sinit     := proc( N )
begin
   if not testtype(N, Type::PosInt) then
      error("Positive integer expected")
   end_if;
   new(Semaphore, init_sem(N), N)
end_proc:

Semaphore::new   := Semaphore::sinit:

Semaphore::V     := fun(semV(extop(args(), 1))):
Semaphore::P     := fun(semP(extop(args(), 1))):
Semaphore::N     := fun(extop(args(), 2)):
Semaphore::print := fun(sem_.extop(args(), 1)):
Semaphore::testtype := fun(bool(type(args(1))=args(2))):
 
Lock             := domain("Lock"):
Lock::info       := "Routines for using locks.":
Lock::interface  := { hold(linit), hold(lock), hold(unlock) }:
Lock::linit      := fun(new(Lock, init_sem(1))):
Lock::new        := Lock::linit:
Lock::lock       := fun(semP(extop(args(1),1),args(2..args(0)))):
Lock::spinlock   := fun(semP(extop(args(1), 1),Spin,args(2..args(0)))):
Lock::unlock     := fun(semV(extop(args(), 1))):
Lock::print      := fun(lock_.extop(args(), 1)):
Lock::testtype   := fun(bool(type(args(1))=args(2))):

#--------------------------------------------------------------------#
# Interface for netwide semaphores                                   #
# Based on atomic global write statement, chh, Juli 97               #
#--------------------------------------------------------------------#

GlobalSemaphore            := domain("GlobalSemaphore"):
GlobalSemaphore::info      := "Routines for using net wide semaphores.":
GlobalSemaphore::interface := { hold(sinit), hold(V), hold(P), hold(N) }:

# Initialisiere Globales Semaphore #
GlobalSemaphore::sinit     := proc( N )
   local ident;
begin
   if not testtype(N, Type::PosInt) then
      error("Positive integer expected")
   end_if;
   ident:=genident("__gsem_");
   # eval(subsop(hold(global(xx,N)),1=level(ident))); #
   subsop(hold(global(xx,N)),1=ident,Simplified);
   new(GlobalSemaphore, ident, N)
end_proc:

GlobalSemaphore::new       := GlobalSemaphore::sinit:

# Belege eine Instanz. Blockiere, bis eine Instanz verfuegbar ist. #
GlobalSemaphore::P     := proc(gs)
local ok,wert,ident;
begin
  if not testtype(gs, GlobalSemaphore) then
      error("Global Semaphore expected");
  end_if;

  ident:=extop(gs,1);
  ok:=FALSE;

  while not ok do
    #
    eval(subs(hold(
      global(xx,
             (wert:=global(xx);
             if wert=0 then 0 else ok:=TRUE; wert-1; end_if) )
    ), xx=level(ident)));
    #
    subs(hold(
      global(xx,
             (wert:=global(xx);
             if wert=0 then 0 else ok:=TRUE; wert-1; end_if) )
    ), hold(xx)=ident, Simplified);
  end_while;

end_proc:

# Gebe eine Instanz frei #
GlobalSemaphore::V     := proc(gs)
local ok,wert;
begin
  if not testtype(gs, GlobalSemaphore) then
      error("Global Semaphore expected");
  end_if;
  #
  eval(subs(hold(
    global(xx,
           (wert:=global(xx);
           if wert=extop(gs,2) then ok:=FALSE; wert; else ok:=TRUE; wert+1; end_if) )
  ), xx=level(extop(gs,1))));
  #
  subs(hold(
    global(xx,
           (wert:=global(xx);
           if wert=extop(gs,2) then ok:=FALSE; wert; else ok:=TRUE; wert+1; end_if) )
  ), hold(xx)=extop(gs,1), Simplified);

  if not ok then error("Unbalanced call of GlobalSemaphore::V/GlobalSemaphore::P"); wert else wert+1 end_if;
end_proc:

# Anzahl der Ressourcen, Ausgabe und testtype #
GlobalSemaphore::N     := fun(extop(args(), 2)):
GlobalSemaphore::print := fun(extop(args(), 1)):
GlobalSemaphore::testtype := fun(bool(type(args(1))=args(2))):

end_proc():

# the end #
