# $Date: 1999/05/26 10:41:03 $  $Author: olli $  $Revision: 1.149.2.5 $ #
#
kg, 11.05.93

stdlib.mu -- lib-file for standard library STDLIB
#

proc()
    option noDebug;
    local path;
begin

#--
  If you need to use a local variable which is protected in the
  global context, you may use sysblock to push this variable on
  the stack, evaluate an expression and then pop the variable
  from the stack.
  
  Usage:
  
     stdlib::sysblock( var, expression )
  
--#
stdlib::sysblock := proc()
  option hold, noDebug;
  local oldProtect,varList;
begin
  if type(level(args(1),1)) <> DOM_LIST then
    varList := [level(args(1),1)];
  else
    varList := level(args(1),1);
  end_if;
  oldProtect := map(varList, protected, None);
  # eval( subsop(hold(block(dummy, eval(args(2..args(0))))), 
               [1,1]=level(args(1),1))); #
  subsop(hold(block(dummy, context(args(2..args(0))))), 
               [1,1]=level(args(1),1), Simplified);
  zip(varList, oldProtect, protected);
  %2    
end_proc:

#--
  If a library routine often uses protected variables, these
  statements may be executed by syseval. 
  This routine is used by loadlib and loadproc, so it has to
  be defined before the first execution of both. 
  
  Usage:
  
     stdlib::syseval( expr )      

  !!! stdlib::syseval is used inside of loadproc. Do not move !!!!
--#
stdlib::syseval := 
proc()
  option hold, noDebug;
  local oldProtect;
begin
  oldProtect := protected(hold(_assign), None);
  block(_assign, (
    sysassign(_assign, sysassign);
    eval(args());
  ));
  protected(hold(_assign), oldProtect);
  %2    
end_proc:

#--
  The following sets are used for the protect mechanism of the
  MuPAD library. The variables in stdlib::ENVIRONMENT_VARIABLES
  and stdlib::SYSTEM_CONSTANTS are not protected, the variables
  in stdlib::LIBRARY_CONSTANTS are protected.
  The protection is done at the end of .mupadsysinit.
--#
stdlib::ENVIRONMENT_VARIABLES:= {          
  hold(DIGITS), hold(ERRORLEVEL), hold(EVAL_STMT), hold(HISTORY), 
  hold(LEVEL), hold(LIB_PATH), hold(MAXDEPTH), hold(MAXLEVEL), 
  hold(ORDER), hold(PRETTY_PRINT), hold(PRINTLEVEL), hold(READ_PATH), 
  hold(ROUNDING), hold(SEED), hold(TEXTWIDTH), hold(WRITE_PATH),
  hold(procname), hold(TEST_PATH), hold(_mod)
}:


stdlib::SYSTEM_CONSTANTS := {
  hold(E), hold(EULER), hold(FALSE), hold(I), hold(PI), hold(TRUE), 
  hold(UNKNOWN)
}:   

stdlib::LIBRARY_CONSTANTS := { 
  hold(infinity), hold(MSDOS), hold(NO_CONVERGENCE), hold(NO_SOLUTION), 
  hold(toBeDefined), hold(UNBOUNDED), hold(undefined),
  hold(_dom_alias_element_), hold(complexInfinity)
}:

stdlib::OPTIONS := {
  hold(Above), hold(Adaptive), hold(All), hold(Alldata), hold(Always), 
  hold(AndMesh), hold(AndULine), hold(AndVLine), hold(Any), hold(Append), 
  hold(Approx), hold(Arrows), hold(Ascii), hold(Attached), hold(Automatic), 
  hold(Axes), hold(AxesOrigin), hold(AxesScaling),

  hold(BackGround), hold(BackSubstitution), hold(Beam), hold(Below), 
  hold(Bin), hold(Binary), hold(Bottom), hold(Box), hold(Banded),
  
  hold(CameraPoint), hold(Capacity), hold(Center), hold(Closed), hold(Coeffs), 
  hold(Color), hold(ColorPatches), hold(Colors),  hold(Column), hold(Concave), 
  hold(Constrained), hold(Contours), hold(Convex), hold(Corner), hold(Curve), 
  hold(Curves), hold(Cyclic),
  
  hold(DashedLines), hold(DebugOnTheFly), hold(DegInvLexOrder), hold(Degree), 
  hold(DegreeOrder), hold(Diagonal), hold(Dimension), hold(DomainsOnly), 
  
  hold(Error), hold(Eweight), hold(Exact), hold(Expr), hold(Extended), 
  
  hold(Filled), hold(FilledCircles), hold(FilledSquares), hold(First), 
  hold(Flat), hold(FloatFormat), hold(FocalPoint), hold(FontFamily),
  hold(FontSize), hold(FontStyle), hold(ForeGround), hold(Format), 
  hold(Frames), hold(Frobenius), 
  
  hold(Gauss), hold(GaussChebyshev), hold(GaussLegendre), 
  hold(GaussTschebyscheff), hold(Gif), hold(Grid), 
  
  hold(Height), hold(HiddenLine), 
  
  hold(Impulses), hold(Include), hold(Independent), hold(Index), 
  hold(IndexList), hold(Infinity), hold(IntMod), hold(Interactive), 
  
  hold(Labeling), hold(Labels), hold(Left), hold(Length), hold(LexOrder), 
  hold(Line), hold(Lines), hold(LineStyle), hold(LineWidth), hold(List), 
  hold(Log), 
  
  hold(MaxDegree),  hold(Mesh), hold(MinorExpansion), hold(Mode), 
  hold(ModuleTrace), hold(Multiple), 
  
  hold(Name), hold(Named), hold(NewtonCotes), hold(NoCheck), hold(NoErrors), 
  hold(NoLeftVectors), hold(NonNested), hold(NonNegative), hold(None),
  hold(NoRightVectors),
  
  hold(Off), hold(On), hold(Only), hold(Open), hold(Origin), 
  
  hold(Path), hold(Piechart), hold(Piechart3d), hold(Plain), hold(PlotDevice), 
  hold(Points), hold(PointStyle), hold(PointWidth), hold(PolyExpr), 
  hold(PostMap), hold(PreMap), hold(Pretty), hold(PrincipalValue),
  
  hold(Quiet), hold(Quo), 
  
  hold(Raster), hold(RatExpr), hold(Recursive), hold(Rem), hold(Reorder), 
  hold(Right), hold(Root), 

  hold(Sample), hold(Scales), hold(Scaling), hold(Simplified), hold(Smoothness), 
  hold(SolidLines), hold(Special), hold(Stepsize), hold(Style), hold(Surface),
  hold(Symbolic), hold(SymbolicAnalysis), hold(System),
  
  hold(Term), hold(Terms), hold(Test), hold(Text), hold(Ticks), hold(Title), 
  hold(TitlePosition), hold(Titles), hold(ToNearest), hold(ToInfinity), 
  hold(ToMinusInfinity), hold(ToZero), hold(TrailingZeroes), hold(Transparent),

  hold(ULine), hold(UnConstrained), hold(Unique), hold(Unquoted), 
  hold(Unrestricted), hold(User), hold(Unsimplified), hold(Up), 
  
  hold(Values), hold(VectorOf), hold(ViewingBox), hold(VLine), hold(Vweight), 

  hold(Laurent), hold(Taylor), hold(Puiseux),
  
  hold(Warning), hold(WireFrame), hold(Write), 
  
  hold(X11),hold(XMax), hold(XMin),

  hold(YMax), hold(YMin), hold(YRange),

  hold(ZMax), hold(ZMin),
  
  hold(sincos), hold(sinhcosh)
}:

stdlib::SYSTEM_TABLES := {
  hold(stdlib::Alias), hold(stdlib::LoadlibState)
}:

#--
  Used in plot2d and plot3d to return the plot
  data instead of plotting the data
--#
stdlib::ShowPlot := TRUE:

path:= pathname("STDLIB"):

D:= loadproc(D, path, "D"):
evalassign:= loadproc(evalassign, path, "evalass"):
export:= loadproc(export, path, "export"):
func:= loadproc(func, path, "func"):
fun:= loadproc(fun, path, "fun"):
info:= loadproc(info, path, "info"):
iquo:= loadproc(iquo, path, "iquo"):
irem:= loadproc(irem, path, "irem"):
plot2d:= loadproc(plot2d, path, "plot2d"):
plot3d:= loadproc(plot3d, path, "plot3d"):
profile:= loadproc(profile, path, "prof"):
random:= loadproc(random, path, "random"):
seq:= loadproc(seq, path, "seq"):
Poly:= loadproc(Poly, path, "Poly"):
O:= loadproc(O, path, "O"):
taylor:= loadproc(taylor, path, "taylor"):
gcd:= loadproc(gcd, path, "gcd"):
lcm:= loadproc(lcm, path, "lcm"):
content:= loadproc(content, path, "content"):
loadlib:= loadproc(loadlib, path, "loadlib"):
normal:= loadproc(normal, path, "normal"):
numer:= loadproc(numer, path, "numer"):
denom:= loadproc(denom, path, "denom"):
bernoulli:= loadproc(bernoulli, path, "bernoull"):
binomial:= loadproc(binomial, path, "binomial"):
fft:= loadproc(fft, path, "fft"):
ifft:= loadproc(ifft, path, "ifft"):
version:= loadproc(version, path, "version"):
factor:= loadproc(factor, path, "factor"):
sqrfree:= loadproc(sqrfree, path, "sqrfree"):
insert_ordered:= loadproc(insert_ordered, path, "ins_ord"):
resultant:= loadproc(resultant, path, "resultan"):
discrim:= loadproc(discrim, path, "discrim"):
linsert:= loadproc(linsert, path, "linsert"):
gcdex:= loadproc(gcdex, path, "gcdex"):
pdioe:= loadproc(pdioe, path, "pdioe"):
partfrac:= loadproc(partfrac, path, "partfrac"):
sum:= loadproc(sum, path, "sum"):
contfrac:= loadproc(contfrac, path, "contfrac"):
series:= loadproc(series, path, "series"):
limit:= loadproc(limit, path, "limit"):
powermod:= loadproc(powermod, path, "powermod"):
primpart:= loadproc(primpart, path, "primpart"):
infinity:= loadproc(infinity, path, "infinity"):
undefined:= loadproc(undefined, path, "undef"):
complexInfinity:= loadproc(complexInfinity, path, "cinfty"):
Factor:= loadproc(Factor, path, "factor"):
collect:= loadproc(collect, path, "collect"):
length:= loadproc(length, path, "length"):
stdlib::length:= loadproc(stdlib::length, path, "length"):
int:= loadproc(int, path, "int"):
maprat:= loadproc(maprat, path, "maprat"):
userinfo:= loadproc(userinfo, path, "userinfo"):
setuserinfo:= loadproc(setuserinfo, path, "setuinfo"):
expose:= loadproc(expose, path, "expose"):
radsimp:= loadproc(radsimp, path, "radsimp"):
combine:= loadproc(combine, path, "combine"):
decompose:= loadproc(decompose, path, "decomp"):
linsolve:= loadproc(linsolve, path, "linsolve"):
alias:= loadproc(alias, path, "alias"):
unalias:= loadproc(unalias, path, "unalias"):
RootOf := loadproc(RootOf, path, "RootOf"):
revert := loadproc(revert, path, "revert"):
rationalize := loadproc(rationalize, path, "maprat"):
randpoly := loadproc(randpoly, path, "randpoly"):
algsolve:= loadproc(algsolve,path,"algsolve"):
read:= loadproc(read, path, "read"):
maprec:= loadproc(maprec, path, "maprec"):
breakmap:= loadproc(breakmap, path, "maprec"):
asympt:= loadproc(asympt, path, "asympt"):
lazy_and:= loadproc(lazy_and, path, "lazy_and"):
_seqin:= loadproc(_seqin, path, "seqin"):
irreducible:= loadproc(irreducible, path, "irreduc"):
rewrite:= loadproc(rewrite, path, "rewrite"):
discont:= loadproc(discont, path, "discont"):
plotfunc:= loadproc(plotfunc, path, "plotfunc"):
plotfunc2d:= loadproc(plotfunc2d, path, "plotf2d"):
assign:= loadproc(assign, path, "assign"):
hastype:= loadproc(hastype, path, "hastype"):
optimize:= loadproc(optimize, path, "optimize"):
ithprime:=loadproc(ithprime, path, "ithprime"):
conjugate:= loadproc(conjugate, pathname("STDLIB","CONJ"), "conj"):
simplify:= loadproc(simplify, pathname("STDLIB","SIMPLIFY"), "simplify"):
rectform:= loadproc(rectform, pathname("STDLIB","RECTFORM"), "rectform"):
Re:= loadproc(Re, pathname("STDLIB","RE"), "Re"):
Im:= loadproc(Im, pathname("STDLIB","IM"), "Im"):
changevar:= loadproc(changevar, path, "changevr"):
product:= loadproc(product, path, "product"):
poly2list:= loadproc(poly2list, path, "poly2lst"):
lagrange:= loadproc(lagrange, path, "lagrange"):
spline:= loadproc(spline, path, "spline"):
max:= loadproc(max, path, "minmax"):
min:= loadproc(min, path, "minmax"):

#-- 
    New self-defined operators:

    @@ - repeated composition
    -> - functional operator

--#

bin_op("@@", repcom):
repcom := loadproc(repcom, path, "repcom"):

bin_op("->", arrow):
arrow := loadproc(arrow, path, "arrow"):


#--
   solve -- solve equations and algebraic equations
--#
path:= pathname("STDLIB","SOLVE"):

solve                  := loadproc(solve,                  path,"solve"):
stdlib::solve_algebraic:= loadproc(stdlib::solve_algebraic,path,"algebr"):
stdlib::solve_eq       := loadproc(stdlib::solve_eq,       path,"equation"):
stdlib::solve_isalgebraic:= loadproc(stdlib::solve_isalgebraic,path,"isalgeb"):
stdlib::solve_islinear := loadproc(stdlib::solve_islinear, path,"islin"):
stdlib::isolate        := loadproc(stdlib::isolate,        path,"isolate"):
stdlib::solve_poly_irred:= loadproc(stdlib::solve_poly_irred,path,"pirred"):
stdlib::solve_poly     := loadproc(stdlib::solve_poly,     path,"poly"):
stdlib::solve_poly3    := loadproc(stdlib::solve_poly3,    path,"poly3"):
stdlib::solve_poly4    := loadproc(stdlib::solve_poly4,    path,"poly4"):
stdlib::solve_sys      := loadproc(stdlib::solve_sys,      path,"system"):
stdlib::ineqs          := loadproc(stdlib::ineqs,          path,"ineqs"):

#--
	'export' some of the functions of property
--#
path:= pathname("PROP"):

is		:= loadproc(is,		path,"Propis"):
getprop		:= loadproc(getprop,	path, "Propget" ):
unassume	:= loadproc(unassume,	path, "Propdel" ):
assume		:= loadproc(assume, 	path, "Propset" ):


#--
        integration methods
--#
path:= pathname("INTLIB");

DOM_POLY::int:= loadproc( DOM_POLY::int, path, "DOM_POLY" ):

#--
        'warning' will be replaced by a kernel function in a future release
--#
warning := proc()
  local PRETTY_PRINT; 
begin 
  PRETTY_PRINT := FALSE;
  if type(args()) <> DOM_STRING then
    error("String expected"):
  end_if;
  print(Unquoted, "Warning: ".args()):
end_proc:

protect   := fun( (if args(0) = 2 then
                     protected(args(1), args(2))
                   else
                     protected(args(1), Warning)
                   end_if), hold ):
unprotect := fun( protected(args(1),    None), hold ):

stdlib::LoadlibState["stdlib"]:= TRUE:

end_proc():

# end of file #
