# #
# $Date: 1997/03/19 11:20:48 $  $Author: frankp $   $Revision: 1.16 $ #
# #
# frankp, 16/08/94 #

#++
  adjoint.mu

	linalg::adjoint(A) -- return the adjoint of the square matrix A

	Computes the adjoint of a matrix over a commutative ring.
	This is the matrix whose (i,j)-th entry is the (j,i)-th
	cofactor of A.

	The (j,i)-th cofactor of A is defined by 

	    A'(i,j) = (-1)^(i+j)*det(A(i,j))

	where A(i,j) is the submatrix of A obtained from A by
	deleting the i-th row and j-th column.
++#

linalg::adjoint := proc(A)
    name linalg::adjoint;
    local n, R, i, j, a, det, Ai, delRow, delCol, R_negate;
begin
    if testargs() then
        if args(0) <> 1 then error("expecting one argument") end_if;
	if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("first argument is not of 'Cat::MatrixCat'")
	end_if;
	if not (A::coeffRing)::hasProp( Cat::CommutativeRing ) then
	    error("expecting matrix over 'Cat::CommutativeRing'")
	end_if;
	n := A::dimen(A);
	if op(n,1) <> op(n,2) then error("expecting a square matrix") end_if
    end_if;

    det := linalg::det;
    delRow := A::delRow;
    delCol := A::delCol;
    
    R := A::coeffRing;
    R_negate := R::_negate;

    n := op( A::dimen(A),1 );
    a := array( 1..n,1..n );
    for i from 1 to n do
	Ai := delCol( A,i );
	for j from 1 to n do
	    a[i,j] := det(delRow( Ai,j ));
	    if i+j mod 2 = 1 then
		a[i,j] := R_negate( a[i,j] )
	    end_if
	end_for
    end_for;

    A::newThis( a ) 
end_proc:

# end of file #
