# #
# $Date: 1999/06/02 15:45:38 $  $Author: frankp $   $Revision: 1.25.2.1 $ #
# #
# frankp, 16/08/94 #

#++
  charmat.mu

	linalg::charMatrix  --  return the characteristic matrix

	linalg::charMatrix(A,x)
	
	A: square matrix over a commutative ring
	x: valid polynomial indeterminate
	 
	This function constructs the characteristic matrix, defined
	by x*E(n) - A, where E(n) is the identity matrix of 
	dimension n=dimen(A).
	The characteristic matrix will be defined over the domain 
	Dom::DistributedPolynomial([x],R), where R is the coefficient 
	domain of A. R must be a commutative ring.

	Note: Evaluations of the characteristic matrix C for a matrix
	      A of matrix domain M for special values u of x can be 
	      done by

		   M::map( C,evalp,x=u )
++#

linalg::charMatrix := proc(A,x)
    name linalg::charMatrix;
    local t, S, nam;
begin
    if testargs() then
        if args(0) <> 2 then error("expecting 2 arguments") end_if;
	if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("first argument is not of 'Cat::MatrixCat'")
	end_if;
	if not (A::coeffRing)::hasProp( Cat::CommutativeRing ) then
	    error("expecting matrix over 'Cat::CommutativeRing'")
	end_if;
	t := A::dimen(A);
	if op(t,1) <> op(t,2) then
	    error("expecting a square matrix")
	end_if:
	if poly(x) = FAIL then
	    error("illegal indeterminate")
	end_if
    end_if;

    S := Dom::DistributedPolynomial( [x],A::coeffRing );
    A := domattr(Dom::Matrix(S),"coerce")( A );
    nam:= (if S::name = FAIL then S::key else S::name end_if);
    if A = FAIL then
	error("unable to convert matrix to a matrix over ".expr2text(nam))
    end_if;

    A::_plus( 
        A::newThis(op(A::dimen(A)),subs(fun(hold(XX)),hold(XX)=S::new(x)),hold(Diagonal)),A::_negate(A)
    )
end_proc:

# end of file #
