# #
# $Date: 1997/03/19 11:20:52 $  $Author: frankp $   $Revision: 1.11 $ #
# #
# frankp, 22/11/94 #

#++
  curl.mu

        linalg::curl -- computes the curl of a vector over fields

        curl(x,m,[ogSystem])

        x       : list or vector of expressions of length 3
        m       : list or vector of three variables
	ogSystem: (optional) name or list of the scale factors of an 
                  orthogonally curvilinear coordinate system

	This function computes the curl of 'x'.
        If only two arguments are given 'linalg::curl' operates on
        the cartesian coordinate system.

        Otherwise 'linalg::curl' operates on the orthogonally 
	curvilinear coordinate system specified by a list of scale 
	factors or by a name which scale factors are defined in table 
	'linalg::ogCoordTab'.

	If 'x' is a vector over a Cat::Field R, then R has to provide a 
	method "diff". Otherwise an error message will be returned.
++#

linalg::curl := proc(x,m)
    name linalg::curl;
    local f, f1, f2, f3, g, i, R, rot, Rmult, Rplus, R_negate;
begin
    if testargs() then
	if args(0) <> 2 and args(0) <> 3 then
	    error("expecting 2 or 3 arguments")
	end_if;
	if testtype( x,linalg::VectorOf(Type::AnyType,3) ) then
	    if not testtype( m,linalg::VectorOf(x::coeffRing,3) )
	       and not testtype( m,Type::ListOf(x::coeffRing,3,3) )
	    then
		error("2nd argument has invalid number or type of entries")
	    end_if
	elif domtype(x) = DOM_LIST then
            if domtype(m) <> DOM_LIST then
                error("2nd argument must be a list")
            end_if;
	    if nops(m) <> 3 or nops(x) <> 3 then 
		error("expecting lists with 3 elements");
	    end_if;
	else
	    error("first argument must be a list or vector with 3 entries")
	end_if;

	if args(0) = 3 then
	    if domtype(args(3)) <> DOM_LIST and 
               type( linalg::ogCoordTab[args(3),hold(Scales)] ) = "_index"
            then
                error("illegal or undefined coordinate system")
            end_if
	end_if
    end_if;

    if args(0) = 2 then 
	rot := [diff(x[3],m[2])-diff(x[2],m[3]),
		diff(x[1],m[3])-diff(x[3],m[1]),
		diff(x[2],m[1])-diff(x[1],m[2])];
	if domtype(x) = DOM_LIST then
	    return( rot )
	else
	    return( x::newThis(op(x::dimen(x)),rot) )
	end_if
    end_if;
    g := args(3);
    if domtype(g) <> DOM_LIST then
        g := linalg::ogCoordTab[g,hold(Scales)](m[i] $ i=1..3)
    end_if;
    if domtype(x) <> DOM_LIST then
	R := x::coeffRing;
	if not R::hasProp( Cat::Field ) then
	    error("expecting vector over a Cat::Field")
	end_if;

	m:= map( m,R::coerce );
	if has( m,FAIL ) then
	    error("incompatible 2nd argument")
	end_if;
	g:= map( g,R::coerce );
 	if has(g,FAIL) then
	    error("unable to use given coordinate system over the coefficient domain")
	end_if;

	Rmult:= R::_mult;
	Rplus:= R::_plus;
	R_negate:= R::_negate;

	f := map( [g[1],g[2],g[3]],R::_invert );
	f1 := Rmult(f[2],f[3]);
	f2 := Rmult(f[1],f[3]);
	f3 := Rmult(f[1],f[2]);
	x::newThis(op(x::dimen(x)),[
	    Rmult( f1,Rplus(
		diff(Rmult(g[3],x[3]),m[2]),
                R_negate(diff(Rmult(g[2],x[2]),m[3])))
	    ),
            Rmult( f2,Rplus(
		diff(Rmult(g[1],x[1]),m[3]),
                R_negate(diff(Rmult(g[3],x[3]),m[1])))
            ),
            Rmult( f3,Rplus(
		diff(Rmult(g[2],x[2]),m[1]),
                R_negate(diff(Rmult(g[1],x[1]),m[2])))
            )
	])
    else
	f := [ 1/g[1], 1/g[2], 1/g[3] ];
        f1 := f[2]*f[3]; f2 := f[1]*f[3]; f3 := f[1]*f[2];
	f := [f1*(diff(g[3]*x[3],m[2])-diff(g[2]*x[2],m[3])),
                f2*(diff(g[1]*x[1],m[3])-diff(g[3]*x[3],m[1])),
                f3*(diff(g[2]*x[2],m[1])-diff(g[1]*x[1],m[2]))
	     ]
    end_if
end_proc:

# end of file #
