# #
# $Date: 1997/03/19 11:20:53 $  $Author: frankp $   $Revision: 1.20 $ #
# #
# frankp, 21/02/94 #

#++
  det.mu

	linalg::det -- computes the determinant of a matrix
		       over a commutative ring
++#

linalg::det := proc(A)
    name linalg::det;
    option remember;
    local Rmult, R_negate, R, n;
begin
    if testargs() then
	if args(0) <> 1 then error("expecting 1 argument") end_if;
        if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("argument is not of 'Cat::MatrixCat'")
        end_if;
	if not (A::coeffRing)::hasProp( Cat::CommutativeRing ) then
	    error("expecting matrix over 'Cat::CommutativeRing'")
	end_if;
	n := A::dimen(A);
	if op(n,1) <> op(n,2) then
	    error("expecting a square matrix")
	end_if
    end_if;

    n := op(A::dimen(A),1);
    if n < 4 then
	R := A::coeffRing;
	Rmult := R::_mult;
	R_negate := R::_negate
    end_if;
    case n
    of 1 do return( A[1,1] );
    of 2 do return( R::_plus( 
		    Rmult( A[1,1],A[2,2] ), 
 		    R_negate( Rmult( A[1,2],A[2,1] ) )
                    )
            )
    of 3 do return( R::_plus( 
		    Rmult( A[1,1],A[2,2],A[3,3] ),
		    R_negate( Rmult( A[1,1],A[2,3],A[3,2] ) ),
		    R_negate( Rmult( A[2,1],A[1,2],A[3,3] ) ),
		    Rmult( A[2,1],A[1,3],A[3,2] ),
		    Rmult( A[3,1],A[1,2],A[2,3] ),
		    R_negate( Rmult( A[3,1],A[1,3],A[2,2] ) )
                  )
             )
    otherwise if (A::coeffRing)::hasProp( Cat::IntegralDomain ) then
	          return( op(A::gaussElim(A),3) )
	      else
		  return( linalg::detOverCommutativeRing(A) )
	      end_if
    end_case
end_proc:

#--
        linalg::detOverCommutativeRing

        Computes the determinant of a square matrix over a 
	commutative ring using Laplace expansion formula.
--#

linalg::detOverCommutativeRing:= proc(x)
    local R;
begin
    R:= x::coeffRing;
    case op( x::dimen(x),1 )
    of 1 do x[1,1]
    of 2 do R::_plus( 
                R::_mult( x[1,1],x[2,2] ), 
                R::_negate( R::_mult( x[1,2],x[2,1] ) )
            );
	    break
    of 3 do R::_plus( 
                R::_mult( x[1,1],x[2,2],x[3,3] ),
                R::_negate( R::_mult( x[1,1],x[2,3],x[3,2] ) ),
                R::_negate( R::_mult( x[2,1],x[1,2],x[3,3] ) ),
                R::_mult( x[2,1],x[1,3],x[3,2] ),
                R::_mult( x[3,1],x[1,2],x[2,3] ),
                R::_negate( R::_mult( x[3,1],x[1,3],x[2,2] ) )
            );
	    break
    otherwise R::_plus( R::_mult(
		R::_power(R::_negate(R::one),i+1),x[i,1], 
		linalg::detOverCommutativeRing( 
		    x::delRow(x::delCol(x,1),i)
	      ) ) $ hold(i)=1..op(x::dimen(x),1) )
    end_case
end_proc:

# end of file #
