# #
# $Date: 1997/03/19 11:20:54 $  $Author: frankp $   $Revision: 1.11 $ #
# #
# frankp, 31/10/94 #

#++
  diverge.mu

        linalg::divergence -- computes the divergence of a vector function
			      over fields

        divergence(x,m,[ogSystem])

        x       : list or vector of expressions (the vector function)
        m       : list or vector of variables (same length as x)
	ogSystem: (optional) name or list of the scale factors of an 
                  orthogonally curvilinear coordinate system

	This function computes the divergence of 'x'.
        If only two arguments are given 'linalg::divergence' operates on
        the cartesian coordinate system. Hence the result is the sum
	diff(x[i],m[i]) as i ranges over the length of 'x'.

        Otherwise 'linalg::divergence' operates on the orthogonally 
	curvilinear coordinate system specified by a list of scale 
	factors or by a name which scale factors are defined in table 
	'linalg::ogCoordTab'.

	If 'x' is a vector over R then R has to provide a method "diff".
++#

linalg::divergence := proc(x,m)
    name linalg::divergence;
    local g, i, R, Rmult, R_invert, Rnormal;
begin
    if testargs() then
	if args(0) <> 2 and args(0) <> 3 then
	    error("expecting 2 or 3 arguments")
	end_if;
	if testtype( x,linalg::VectorOf(Type::AnyType) ) then
	    if not (x::coeffRing)::hasProp( Cat::Field ) then
                error("expecting vectors over 'Cat::Field'")
            end_if;

	    if not testtype(m,Type::ListOf( x::coeffRing,nops(x) $ 2)) and not testtype(m,linalg::VectorOf( x::coeffRing,nops(x))) then
		error("incompatible or invalid 2nd argument");
	    end_if
	elif domtype(x) = DOM_LIST then
	    if not testtype( m,Type::ListOfIdents(nops(x) $ 2) ) then
		error("expecting lists with same number of elements")
	    end_if
	else
	    error("1st argument must be a list or vector")
	end_if;
	if args(0) = 3 then
	    if nops(m) <> 3 then
	        error("expecting list or vector with 3 components");
	    elif domtype(args(3)) <> DOM_LIST and 
                 type(linalg::ogCoordTab[args(3),hold(Scales)]) = "_index"
            then
                error("illegal or undefined coordinate system")
            end_if
	end_if
    end_if;

    if args(0) = 2 then 
	return( _plus( diff(x[i],m[i]) $ i=1..nops(m) ) )
    end_if;
    g:= args(3);
    if domtype(g) <> DOM_LIST then
        g:= linalg::ogCoordTab[g,hold(Scales)](m[i] $ i=1..3)
    end_if;
    if domtype(x) <> DOM_LIST then
	R:= x::coeffRing;
	for i from 1 to 3 do
	    if (g[i]:= R::coerce(g[i])) = FAIL then
		error("unable to compute the divergence over ".expr2text(R))
	    end_if
	end_for;
	Rmult:= R::_mult; 
	R_invert:= R::_invert;
	Rnormal:= id
    else
	Rmult:= _mult;
	R_invert:= 1/id;
	Rnormal:= normal
    end_if;

    Rnormal(Rmult( R_invert(g[i]) $ hold(i)=1..3,
           _plus( diff(Rmult(g[2],g[3],x[1]),m[1]),
                  diff(Rmult(g[3],g[1],x[2]),m[2]),
                  diff(Rmult(g[1],g[2],x[3]),m[3]) )
    ))
end_proc:

# end of file #
