# #
# $Date: 1997/03/19 11:20:56 $  $Author: frankp $   $Revision: 1.29 $ #
# #
# frankp, 1/08/94 #

#++
  expr2mat.mu

    linalg::expr2Matrix  --  builds up a matrix from a set or list of 
			     equations

	expr2Matrix(eqns [,vars] [,coeffRing] [{,Include,Append}])

	eqns, vars - sets or lists
	Include    - ident

	Generates the coefficient matrix from the linear system of 
	equations eqns in the unkown vars. If vars is not given,
	'indets(eqns,PolyExpr)' is used to get the unknown vars of the
	equations. The coefficient matrix and a vector which represents
	the right sides of the equations are returned as a list.

	If the option 'Append' is given, then the right side of the
	equations will be included as the last column of the matrix,
	which is returned.

	If an argument 'Include' is present, the negative of the
	right side of the equations will be included as the last column
	of the matrix, which is returned.

	If coeffRing is missing then the default coeffcient domain for 
	matrices will be used (i.e. 'ExpressionField()').
	If coeffRing is given then it must be of category CommutativeRing.

	This function can be used for building up the coefficient matrix of
	expressions instead of equations in the same way. In this case
	expressions are treated as equations, where the right side is set
	to zero.
++#

linalg::expr2Matrix := proc(eqns)
    name linalg::expr2Matrix;
    local vars, R, flag, a, t, v, i, j, r, c, Rcoerce, R_negate, k;
begin
    if testargs() then
	case args(0)
	of 4 do if not contains( {hold(Include),hold(Append)},args(4) )
		and (args(4))::hasProp( Cat::CommutativeRing ) <> TRUE
		then
		    error("invalid 4th argument")
		end_if
	of 3 do if not contains( {hold(Include),hold(Append)},args(3) )
		and (args(3))::hasProp( Cat::CommutativeRing ) <> TRUE
		then
		    error("invalid 3rd argument")
		end_if
	of 2 do t := args(2); 
		if not contains( {DOM_LIST,DOM_SET},domtype(t) ) then
		    if not contains( {hold(Include),hold(Append)},t )
		    and t::hasProp( Cat::CommutativeRing ) <> TRUE 
		    then
			error("invalid 2nd argument")
		    end_if
		else
		    if nops(t) = 0 then
			error("missing indeterminates")
		    end_if;
		    for i from 1 to nops(t) do
			if poly(op(t,i)) = FAIL then
			    error("invalid indeterminate: ".expr2text(op(t,i)))
			end_if
		    end_for
		end_if
	of 1 do if not contains( {DOM_LIST,DOM_SET},domtype(eqns) ) then
		    error("expecting a list or set of expressions")
		end_if;
		if nops(eqns) = 0 then
		    error("missing equations")
		end_if;
		break
	otherwise
	    error("wrong no of args")
	end_case
    end_if;

    flag:= FALSE;
    case args(0)
    of 1 do R:= Dom::ExpressionField();
	    break;
    of 2 do t:= args(2);
	    if contains( {DOM_LIST,DOM_SET},domtype(t) ) then
		vars:= t;
	 	R:= Dom::ExpressionField();
		break
	    end_if;
	    if contains( {hold(Include),hold(Append)},t ) then
		R:= Dom::ExpressionField();
		flag:= t;
		break
	    end_if;
	    R:= t;
	    break
    of 3 do if contains( {hold(Include),hold(Append)},args(3) ) then
		flag:= args(3)
	    else
		flag:= FALSE
	    end_if;
	    if contains( {DOM_LIST,DOM_SET},domtype(args(2)) ) then
		vars:= args(2);
		if flag = FALSE then R:= args(3)
		else R:= Dom::ExpressionField(normal)
		end_if;
		break
	    end_if;
	    if flag = FALSE then R:= args(2) end_if;
	    break
    of 4 do vars:= args(2);
	    if contains( {hold(Include),hold(Append)},args(3) ) then 
		R:= args(4);
		flag:= args(3)
	    else 
		flag:= args(4);
		R:= args(3) 
	    end_if
    end_case;

    Rcoerce:= R::coerce;
    R_negate:= R::_negate;

    eqns := [ op(eqns) ];
    r := nops(eqns);

    userinfo(2,"change expressions to equations");

    t := array(1..r,1..1);
    for i from 1 to r do
	if type( eqns[i] ) <> "_equal" then 
	    t[i,1] := 0; 
	    next 
	end_if;
	# save the right side # 
	t[i,1] := op( eqns[i],2 );

	eqns := subsop( eqns, i=op( eqns[i],1 ) )
    end_for;

    if vars = hold(vars) then
	vars := _union( indets(eqns[j],hold(PolyExpr)) $ j=1..r )
    end_if;
 
    c := nops(vars);
    if flag <> FALSE then
        a := array(1..r,1..c+1,[ [R::zero $ c+1] $ r ]);
    else
	a := array(1..r,1..c,[ [R::zero $ c] $ r ])
    end_if;
 
    userinfo(2,"compute the right and left side of each eq.");

    for i from 1 to r do 
	# get constant coefficient j of i-th equation #
	j := poly( eqns[i],[op(vars)] );
        ( j := coeff(j,0) ) $ hold(k)=1..nops(vars);

	eqns := subsop( eqns,i=eqns[i]-j );
	j := t[i,1] - j;
	# get constant coefficient v of j #
	v := poly( j,[op(vars)] );
        ( v := coeff(v,0) ) $ hold(k)=1..nops(vars);

        if domtype(v) = R then
            t[i,1] := v;
	elif (t[i,1]:= Rcoerce( v )) = FAIL then
            error("unable to convert ".expr2text(v))
	end_if;

	eqns := subsop( eqns,i=eqns[i]-j+v ); 

        for j from 1 to c do
            v := poly( eqns[i],[op(vars)] );
            case degree(v,op(vars,j))
            of 0 do a[i,j] := R::zero;
                    break
            of 1 do v := coeff( v,op(vars,j),1 );
                    if domtype(v) = DOM_POLY then
                        if degree(v,[op(vars)]) <> 0 then
                            error("expecting linear equations")
                        end_if;
                        v := expr(v)
                    end_if;
                    if domtype(v) = R then a[i,j]:= v
                    elif (a[i,j]:= Rcoerce( v )) = FAIL then
                        error("unable to convert ".expr2text(v))
                    end_if;
                    break
            otherwise
                error("expecting linear equations")
            end_case;

        end_for;
	if flag = hold(Include) then
	    a[i,c+1]:= R_negate(t[i,1])
	elif flag <> FALSE then
	    a[i,c+1]:= t[i,1]
 	end_if
    end_for;

    a := domattr(Dom::Matrix(R),"newThis")(a);
    if flag = FALSE then
        [a,a::newThis( t )]
    else
	a
    end_if
end_proc:

# end of file #
