# #
# $Date: 1997/09/08 13:33:59 $  $Author: frankp $   $Revision: 1.1 $ #
# #

#++
	factorLU(A) -- computes LU decomposition of a matrix A.
                Returns 3-tuple: a Dom::Matrix containing
                L and U, a Dom::Vector with the row permutation used for
                partial pivoting and a list of the used pivots.

Authors: Eduard Niess, Werner M. Seiler, Uni Karlsruhe
Date created: May 23rd, 1996
Last update: August 10th, 1997
++#

linalg::factorLU:= proc(A)
    local maxR,maxC,PermV,Pivs,s,i,i0,j,k,d,dim;
begin
    if A::hasProp(Cat::MatrixCat) then
        if testargs() and not (A::coeffRing)::hasProp( Cat::Field ) then
            error("expecting matrix over Cat::Field")
        end_if;
        dim := A::dimen(A);
        maxR := dim[1];
        maxC := dim[2];
        if maxR<>maxC then error("not a square matrix") end_if;
    else 
        error("expecting argument of category Cat::MatrixCat")
    end_if;

    PermV := array(1..maxR);
    Pivs := [];

    for j from 1 to maxC do
	for i from 1 to j-1 do
	    s := A[i,j];
	    for k from 1 to i-1 do
	        s := s - A[i,k]*A[k,j];
	    end_for;
	    A[i,j] := s;
	end_for;
	i0 := -1;
	for i from j to maxR do
	    s := A[i,j];
	    for k from 1 to j-1 do
	        s := s - A[i,k]*A[k,j];
	    end_for;
	    A[i,j] := s;
	    if s<>0 then
		if i0<0 then i0 := i end_if;
	    end_if;
	end_for;
	if i0<0 then error("singular matrix") end_if;

	if j<>i0 then A := A::swapRow(A,j,i0) end_if;
	PermV[j] := i0;
	Pivs := [A[j,j]] . Pivs;

	if j<>maxC then
	    d := 1/A[j,j];
	    for k from j+1 to maxR do
		A[k,j] := d*A[k,j];
	    end_for;
	end_if;
    end_for;

    (A, PermV, Pivs);
end_proc:

# end of file #
