# #
# $Date: 1997/03/05 12:09:03 $  $Author: olli $   $Revision: 1.20 $ #
# #
# frankp, 04/09/94 #

#++
    linalg::freeSet  --  returns a set of linear independent vectors
                         over a field
++#

linalg::freeSet:= proc()
    name linalg::freeSet;
begin
    if args(0) = 0 then
	error("expecting at least one argument")
    elif args(0) = 1 then
	linalg::basis( TRUE,args(1) )
    elif args(args(0)) = hold(Intersect) then
	linalg::intBasis( TRUE,args(1..args(0)-1) )
    elif args(args(0)) = hold(Sum) then
	linalg::sumBasis( TRUE,args(1..args(0)-1) )
    else
	error("wrong no of arguments")
    end_if
end_proc:
	    
#--
    linalg::basis  --  return a set of linear independent vectors
                       over a field

    basis(S)

    S : list or set of vectors

    Removes such vectors in S which are linearly dependent of all
    the other vectors in S.

    The computed set is a basis for the vectorspace spanned by
    the vectors in S.
--#
linalg::basis:= proc(S)
    name linalg::basis;
    local i, j, p, k, kk, n, t, R, FS, rS, c, Miszero, Riszero, u, v, l;
begin
    if testargs() or S = TRUE then
	if args(0) = 2 then
	    if args(1) <> TRUE then
	        error("wrong no of args")
	    else
		S:= args(2)
	    end_if
	end_if;

	t := Type::ListOf;
	case domtype(S)
	of DOM_SET  do
	    t := Type::SetOf
	of DOM_LIST do
	    if nops(S) = 0 then break end_if;	
	    R := op(S,1);
	    if not testtype(R,linalg::VectorOf(Type::AnyType)) then
		error("expecting set or list of vectors")
	    end_if;
	    n := max( op(R::dimen(R)) ); R := R::coeffRing;
	    if not R::hasProp( Cat::Field ) then
		error("expecting vectors over a field")
	    end_if;
   	    if not testtype( S,t(linalg::VectorOf(R,n)) ) then
		error("vectors are not compatible")
	    end_if;
	    break
	otherwise
	    error("expecting a list or set of vectors")
	end_case
    elif domtype(S) = DOM_BOOL then
	S:= args(2)
    end_if;

    rS := [ op(S) ];
    if nops(rS) = 0 then return( S ) end_if;
    t := rS[1];
    R := t::coeffRing;
    Miszero := t::iszero;
    Riszero := R::iszero; 

    n := max( op(t::dimen(t)) );
    FS := [];
    t := [ NIL $ n ]; 
    p := 1;
    for i from 1 to nops(rS) do
	if Miszero( rS[i] ) then next end_if;
 	t[p] := rS[i];
	kk := 1;
	u := t[p];
	# test whether u is linearly dependent of one of the other vectors #
	for j from 1 to p-1 do
	    v := t[j];
	    for k from kk to n do
		if Riszero(u[k]) and not Riszero(v[k]) then
		    break  # u and v are linearly independent #
		end_if;
		if Riszero(u[k]) then
		    next # k #
		end_if;
		kk := k;
		if Riszero(v[k]) then
		    t[j] := u; u := v; t[p] := v;
		    break # next j #
		end_if;
		c := R::_negate( R::_divide(u[k],v[k]) );
		u[k] := R::zero;
		(u[l] := R::_plus(u[l],R::_mult(v[l],c))) $ hold(l)=k+1..n;
		t[p] := u;
		break # next j #
	    end_for
	end_for;

	if not Miszero(u) then
	    FS := append( FS,rS[i] );
	    if nops(FS) = n then break end_if;
	    p := p + 1
	end_if
    end_for;

    if domtype(S) = DOM_LIST then FS else {op(FS)} end_if
end_proc:

#--
    linalg::sumBasis  --  return a set of linear independent vectors
                          for the sum of vector spaces

    sumBasis(S1, S2 [,S3,...])

    Si : list or set of vectors

    sumBasis returns a basis for the sum of the vector spaces spanned
    by the vectors in Si.
--#
linalg::sumBasis:= proc()
    local argus, t, test;
begin
    argus:= [args()];
    if domtype(argus[1]) = DOM_BOOL then 
	test:= argus[1];
	unassign( argus[1] )
    else 
	test:= testargs()
    end_if;

    t:= map( {op(argus)},domtype );
    if t minus {DOM_LIST,DOM_SET} <> {} then
	error("expecting sets or lists of vectors")
    elif contains( t,DOM_SET ) then
	linalg::basis( test,{op( map(argus,op) )} )
    else
	linalg::basis( test,map(argus,op) )
    end_if
end_proc:


#--
    linalg::intBasis  --  return a set of linear independent vectors
                          for the intersection of vector spaces

    intBasis(S1, S2 [,S3,...])

    Si : list or set of vectors

    intBasis returns a basis for the intersection of the vector 
    spaces spanned by the vectors in Si is returned.
--#

linalg::intBasis:= proc()
    local argus, test, i, j, l, k, n, v, M, R, V, Vold,
          nV, nVold, K, s, T, R_negate, nullSpace;
begin
    argus:= [args()];
    if domtype(argus[1]) = DOM_BOOL then
	test:= argus[1];
	unassign( argus[1] )
    else
	test:= testargs()
    end_if;
	
    Vold:= linalg::basis( test,argus[1] );

    v:= op(argus[1],1);
    n:= nops( v ); # dimension of the vectors #
    M:= domtype( v ); # type of the vectors #

    R:= M::coeffRing;
    nullSpace:= linalg::nullSpace;
    R_negate:= R::_negate;

    for i from 2 to nops(argus) do
        nVold:= nops(Vold);
	V:= linalg::basis( test,argus[i] );
        nV:= nops(V);

        K:= array( 1..n, 1..nVold+nV );
	( (K[k,j]:= op(Vold,j)[k]) $ k=1..n ) $ j=1..nVold;
	( ( K[k,j]:= op(V,j-nVold)[k] ) $ k=1..n ) $ j=nVold+1..nVold+nV;

	K:= nullSpace( M::newThis( K ) );
        for l from 1 to nops(K) do
            s:= op(Vold,1) * K[l][1];
	    ( s:= s + K[l][k] * op(Vold,k) ) $ k=2..nVold;
	    T[l]:= s
        end_for;

        Vold:= map( {T[k] $ k=1..nops(K)}, R_negate )
    end_for;

    Vold
end_proc:

# end of file #
