# #
# $Date: 1997/03/19 11:20:59 $  $Author: frankp $   $Revision: 1.18 $ #
# #
# frankp, 2/03/94 #

#++
  gausjord.mu

	linalg::gaussJordan  --  perform the Gauss-Jordan elimination

	gaussJordan(A [,All])

	gaussJordan(A) returns A reduced to the reduced-row-echolon form.
	With option 'All', the rank, determinant (or FAIL) and a set
	of the characteristic column indices of A are returned additionally
	in form of a list.
++#

linalg::gaussJordan := proc(A)
    name linalg::gaussJordan;
    local p, R, T, t, c, i, j, d, ns, _ns, de, r,
	  Rmult, R_divide, Rplus, R_negate, Rzero, Rone, Riszero;
begin
    if testargs() then
	if args(0) < 1 or args(0) > 2 then
	    error("expecting 1 or 2 arguments")
	end_if;
	if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("first argument is not of 'Cat::MatrixCat'")
	end_if;
	if not (A::coeffRing)::hasProp( Cat::IntegralDomain ) then
	    error("expecting matrix over 'Cat::IntegralDomain'")
	end_if;
	if args(0) = 2 then
	    if args(2) <> hold(All) then
	        error("expecting 'All' as option")
	    end_if
	end_if
    end_if;
  
    userinfo(1,"compute Gauss-Jordan form");
    userinfo(2,"perform Gaussian elimination");
    T:= A::gaussElim(A);
    A:= op(T,1);
    ns:= op(T,4);

    R:= A::coeffRing;

    Rplus:= R::_plus;
    R_negate:= R::_negate;
    Rmult:= R::_mult;
    Rzero:= R::zero;

    d:= A::dimen(A);
    r:= d[1];
    c:= d[2];

    userinfo(2,"compute row reduced form");

    # ds -- list of [i,j] whereby A[i,j] <> 0 #
    t:= [];
    j:= 1;
    for i from 1 to c do
	if not contains( ns,i ) then
	    t:= append(t,[j,i]);
	    j:= j + 1;
	    if j > r then break end_if
 	end_if
    end_for;
    de:= []; # sort descending #
    for i from nops(t) downto 1 do
	de:= append( de,t[i])
    end_for;

    # Gauss-Jordan over fields #
    if R::hasProp(Cat::Field) then
	R_divide:= R::_divide;
	Rone:= R::one;

	for t in de do
	    # select kerVects belong to the right of A[.,t[2]] #
	    _ns:= select( ns,fun(bool(args(1)>t[2])) );

 	    # the pivot #
	    p:= A[t[1],t[2]];

	    for j in _ns do 
		A[t[1],j]:= R_divide(A[t[1],j],p)
	    end_for;
	    # now, the pivot become 1 #
	    A[t[1],t[2]]:= Rone;
	    for i from t[1]-1 downto 1 do
		p:= R_negate( A[i,t[2]] );
		for j in _ns do
		    A[i,j]:= Rplus(
			A[i,j], Rmult( p,A[t[1],j] )
		    )
	        end_for;
		A[i,t[2]]:= Rzero
	    end_for
	end_for

    # Gauss-Jordan over integral domains #
    else
        for t in de do
	    # select kerVects belong to the right of A[.,t[2]] #
	    _ns:= select( ns,fun(bool(args(1)>t[2])) );

	    # the pivot #
            p:= A[t[1],t[2]];

	    de[1]:= NIL;
	    for i from t[1]-1 downto 1 do
		# multiply each entry of row i by p #
		for j from i to c do
		    if j <> t[2] then A[i,j]:= Rmult( A[i,j],p ) end_if
		end_for;

		d:= R_negate( A[i,t[2]] );
		for j in _ns do
		    A[i,j]:= Rplus( Rmult(A[t[1],j],d), A[i,j] )
		end_for;

		A[i,t[2]]:= Rzero
	    end_for
        end_for;

        if (t:= R::gcd) <> FAIL then
	    R_divide:= R::_divide;
	    Riszero:= R::iszero;
	    Rone:= R::one;
	    for i from 1 to r do
		d:= t( op(A::row(A,i)) );
		if not Riszero(d) and d <> Rone then
		    ( A[i,j]:= R_divide(A[i,j],d) ) $ hold(j)=1..c
		end_if
	    end_for
	end_if
    end_if;

    if args(0) = 2 then [A,T[2],T[3],ns] else A end_if
end_proc:

# end of file #
