# #
# $Date: 1997/03/19 11:21:01 $  $Author: frankp $   $Revision: 1.4 $ #
# #
# frankp, 31/10/94 #

#++
  isOrthog.mu

        linalg::isOrthogonal -- tests for orthogonal matrices

        isOrthogonal(A)

        A: square matrix

        This function tries to find out if A is orthogonal.
        TRUE is returned if A is orthogonal.

        Matrix A is orthogonal iff A^(-1) = transpose(A). This is equivalent 
        to the fact that the columns of A build an orthonormal system (in
        respect to the scalar product defined by linalg::scalarProduct).
        
	Note that one can only be sure that A is not an orthogonal matrix
	if the axiom 'canonicalRep' is stated for the coefficient domain 
	of A. In respect to axiom 'canonicalRep' is stated or not, FALSE
	and FAIL is returned, respectivly.

	The coefficient domain of A has to be an integral domain.
++#

linalg::isOrthogonal := proc(A)
    name linalg::isOrthogonal;
    local i, j, n, R, ci, scalarProduct, ret, col, Riszero, Requal, Rone;
begin
    if testargs() then
	if args(0) <> 1 then
	    error("expecting 1 argument")
	end_if;
	if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("argument is not of 'Cat::MatrixCat'")
	end_if;
	if not (A::coeffRing)::hasProp( Cat::IntegralDomain ) then
	    error("expecting matrix over 'Cat::IntegralDomain'")
	end_if	
    end_if;
   
    n := A::dimen(A);
    if op(n,1) <> op(n,2) then return( FALSE ) end_if;
    R := A::coeffRing;
    if R::hasProp( Ax::canonicalRep ) = FALSE then
	ret := FAIL else ret := FALSE
    end_if;


    scalarProduct := linalg::scalarProduct;
    n := op(n,1);
    col := A::col;
    Requal:= R::equal;
    Rone:= R::one;
    Riszero:= R::iszero;

    for i from 1 to n do
	ci := col(A,i);
	if not Requal( scalarProduct( ci,ci ),Rone ) then
	    return( ret )
	end_if;
	for j from i+1 to n do
	    if not Riszero( scalarProduct(ci,col(A,j)) ) then
		return( ret )
	    end_if
	end_for
    end_for;

    TRUE
end_proc: 

# end of file #
