# #
# $Date: 1997/03/19 11:21:02 $  $Author: frankp $   $Revision: 1.6 $ #
# #
# frankp, 30/03/95 #

#++
  jacobian.mu

        linalg::jacobian -- computes the Jacobian matrix of a vector function

        jacobian(f, v)

        f : list or vector of expressions (the vector function)
        v : list or vector of variables

	jacobian(f,v) computes the Jacobian matrix of f with respect to v.
	Th (i,j)-th entry of matrix is defined to be diff(f[i],v[j]).

	'v' must be a list if 'f' is it, and then the jacobian matrix will
	be defined ober ExpressionField().
	If 'f' is a vector over a coefficient domain R then 'v' must be a 
	vector over R too. In this case the coefficient domain of 'f' has 
	to provide the method "diff".
++#

linalg::jacobian := proc(f,v)
    name linalg::jacobian;
    local i, j, Rdiff, nv, nf;
begin
    if testargs() then
	if args(0) <> 2 then error("expecting 2 arguments") end_if;
	if testtype( f,linalg::VectorOf(Type::AnyType) ) then
	    if not (f::coeffRing)::hasProp( Cat::Field ) then
                error("expecting vector function over 'Cat::Field'")
            end_if;
	    if not testtype( v,linalg::VectorOf(f::coeffRing) )
	    then
		error("expecting two vectors of same type")
	    end_if
	elif domtype(f) = DOM_LIST then
	    if not testtype( args(2),Type::ListOfIdents() ) then
		error("expecting a list of variables")
	    end_if
	else
	    error("expecting vector function in form of a list or vector")
	end_if
    end_if;

    if domtype(f) = DOM_LIST then
	Dom::Matrix()([[diff(f[i],v[j]) $ j=1..nops(v)] $ i=1..nops(f)])
    else
        if (Rdiff := (f::coeffRing)::diff) = FAIL then
	    error("Domain attribute \"diff\" missing")
   	else 
            nv := v::nops(v);
	    nf := f::nops(f);
	    f::newThis( nf,nv,[[Rdiff(f[i],v[j]) $ j=1..nv] $ i=1..nf] )
	end_if
    end_if
end_proc:

# end of file #
