# #
# $Date: 1997/09/08 13:33:58 $  $Author: frankp $   $Revision: 1.1 $ #
# #

#++
        linearSolveLU(LU,Perm,b) -- solves the linear system Ax=b after an LU
                decomposition of A has been computed with Decomp(A). The
                matrix LU and the vector Perm are the first two components
                of the result of Decomp.

Authors: Eduard Niess, Werner M. Seiler, Uni Karlsruhe
Date created: May 23rd, 1996
Last update: August 10th, 1997
++#

linalg::linearSolveLU:= proc(lu, perm, x)
    local dim,maxR,s,i,ii,ip,j;
begin
    if lu::hasProp(Cat::MatrixCat) then
        if not (lu::coeffRing)::hasProp( Cat::Field ) then
            error("expecting matrix over Cat::Field")
        end_if;
	dim := lu::dimen(lu);
	maxR := dim[1];
    else 
        error("expecting argument of category Cat::MatrixCat")
    end_if;

    if testargs() then
	if not(x::hasProp(Cat::MatrixCat)) then 
            error("expecting argument of category Cat::MatrixCat")
        elif not (x::coeffRing)::hasProp( Cat::Field ) then
            error("expecting matrix over Cat::Field")
        end_if;
	dim := x::dimen(x);
	if dim[1]<>maxR then error("incompatible dimensions") end_if;
    end_if;

    ii := -1;
    for i from 1 to maxR do
	ip := perm[i];
	s := x[ip,1];
	x[ip,1] := x[i,1];
	if ii>=0 then
	    for j from ii to i-1 do
		s := s - lu[i,j] * x[j,1];
	    end_for;
        elif not(iszero(s)) then ii := i end_if;
	x[i,1] := s;
    end_for;

    for i from maxR downto 1 do
	s := x[i,1];
	for j from i+1 to maxR do
	    s := s - lu[i,j]*x[j,1];
	end_for;
	x[i,1] := s/lu[i,i];
    end_for;

    x
end_proc:

# end of file #
