# #
# $Date: 1997/03/19 11:21:06 $  $Author: frankp $   $Revision: 1.25 $ #
# #
# frankp, 20.09.1994 #

#++
  nulspace.mu

	linalg::nullSpace  --  compute the kernel of a matrix

	nullSpace(A)

	A: matrix

	linalg::nullSpace(A) computes a basis for the kernel of A.
	A must belong to the category Cat::Field.
++#

linalg::nullSpace := proc(A)
    name linalg::nullSpace;
    local R, c, r, ns, rs, k, v, i, j, m, Rzero, Rone, R_negate, newThis;
begin
    if testargs() then
	if args(0) <> 1 then error("expecting 1 argument")
	elif A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("argument is not of 'Cat::MatrixCat'")
	end_if;
	if not (A::coeffRing)::hasProp( Cat::Field ) then
	    error("expecting matrix over 'Cat::Field'")
	end_if
    end_if;

    userinfo(1,"compute kernel of a matrix");

    if A::hasProp( Cat::SquareMatrixCat ) then
        newThis:= (A::getSuperDomain(A))::newThis
    else
	newThis:= A::newThis
    end_if;

    k:= A::dimen(A);
    r:= k[1]; c:= k[2];

    userinfo(2,"compute Gauss-Jordan form");
    A:= linalg::gaussJordan( A,hold(All) );

    ns:= A[4]; A:= A[1];

    if nops( ns ) = 0 then
	return( [] )
    end_if;

    userinfo(2,"solve homogeneous system");
    R:= A::coeffRing;
   
    R_negate:= R::_negate;
    Rone:= R::one;
    Rzero:= R::zero;

    rs:= [];
    ns:= sort( [op(ns)] );
    m:= min( r,c );
    for i from 1 to nops(ns) do
        k:= ns[i];
        v:= map( [op(A[1..m,k..k])], R_negate );
        for j from 1 to i do
            if j = i then
                v:= [op(v,1..k-1),Rone,op(v,k..m+j-1)];
            else
                R:= ns[j];
                v:= [op(v,1..R-1),Rzero,op(v,R..m+j-1)]
            end_if
        end_for;

        rs:= rs . [newThis( c,1,[op(v,1..min(c,m+i))] )]
    end_for;

    rs
end_proc:

# end of file #
