# #
# $Date: 1997/03/19 11:21:08 $  $Author: frankp $   $Revision: 1.18 $ #
# #
# frankp, 21/02/94 #

#++
  rank.mu

	linalg::rank - compute the rank of a matrix over an 
	               integral domain

	rank( A )
	rank( S )

	A - matrix over an integral domain
	S - set or list of vectors defined over an integral domain

	rank(A) computes the rank of A.
	rank(S) computes the rank of the matrix which has
	the vectors in S as its columns.
++#

linalg::rank := proc(x)
begin
    if testargs() then
	if args(0) <> 1 then error("expecting 1 argument") end_if;
        if (args(1))::hasProp( Cat::MatrixCat ) <> TRUE then
	    case domtype(x)
	    of DOM_SET do
		if nops(x) = 0 then break end_if;	
		op(x,1);
		if ((last(1))::coeffRing)::hasProp(Cat::IntegralDomain) <> TRUE then
                    error("expecting vectors over an integral domain")
		elif not testtype( x,Type::SetOf( linalg::VectorOf( (last(1))::coeffRing,nops(last(1)))) ) then
		    error("incompatible vectors")
		end_if;
		break
	    of DOM_LIST do
		if nops(x) = 0 then break end_if;
		op(x,1);
		if ((last(1))::coeffRing)::hasProp(Cat::IntegralDomain) <> TRUE then
		    error("expecting vectors over an integral domain")
		elif not testtype( x,Type::ListOf( linalg::VectorOf( (last(1))::coeffRing,nops(last(1)))) ) then
                    error("incompatible vectors")
                end_if;
		break
	    otherwise
	        error("expecting a matrix or a set or list of vectors")
	    end_case
	elif not (x::coeffRing)::hasProp(Cat::IntegralDomain) then
	    error("expecting matrix over 'Cat::IntegralDomain'")
	end_if
    end_if;

    if domtype(x) = DOM_LIST or domtype(x) = DOM_SET then
	if nops(x) = 0 then 0 else op( (op(x,1))::gaussElim(_concat(op(x))),2 ) end_if
    else
	op( x::gaussElim(x),2 )
    end_if
end_proc:

# end of file #
