# #
# $Date: 1997/03/19 11:21:10 $  $Author: frankp $   $Revision: 1.23 $ #
# #
# frankp, 29/07/94 #

#++
  scmult.mu

    	linalg::scalarProduct(u,v) -- returns the Euclidian scalar product
				      of u and v where u and v are
				      vectors over the same coeffRing. 

	The Euclidian scalar product of two vectors is defined by

	    u[1]*conjugate(v[1]) + u[2]*conjugate(v[2]) + ...

	If the coeffRing of u and v does not provide a method
	"conjugate" (which must return the complex conjugate
	of an expression) then the sum

	    u[1]*v[1] + u[2]*v[2] + ...

	is computed.
++#

linalg::scalarProduct := proc(u,v)
    name linalg::scalarProduct;
    local i, R, Rmult;
begin
    if testargs() then
	if args(0) <> 2 then error("expecting 2 arguments") end_if;
	if not testtype( u,linalg::VectorOf(Type::AnyType) ) then
	    error("expecting two vectors")
	end_if;
	if not testtype(v,linalg::VectorOf(u::coeffRing,u::nops(u))) then
	    error("vectors are not compatible")
	end_if
    end_if;

    R:= u::coeffRing;
    Rmult:= R::_mult;

    if R::conjugate <> FAIL then 
	R::_plus( 
	  Rmult(u[i],R::conjugate(v[i])) $ i=1..u::nops(u)
	)
    else 
	R::_plus( Rmult( u[i],v[i] ) $ i=1..u::nops(u) )
    end_if
end_proc:

# end of file #
