
linopt::Land_Doig := proc(f, cons, cond, int_cons)

   local all_add, add_cons, add_tasks, best_int, c, cmp, 
         cnt1, cnt2, cnt3, cnt4, create_task, eqn, i, j,
         non_int, pool, s, task, v, x;
begin

   # Lokale Prozeduren #

   cmp := proc(a, b) 
       begin bool( a[5] < b[5] ) end_proc:

   create_task := proc(f, cons, cond, add_cons, v, new_con)
       local eqn, op_new_con, x;
   begin
       x := op(new_con, 1); op_new_con := op(new_con, 0);
       for eqn in add_cons do
          if op(eqn, 1) = x and op(eqn, 0) = op_new_con then
             add_cons := add_cons minus {eqn};
             break;
          end_if
       end_for;
       [ f, cons, cond, add_cons union {new_con}, v ]
   end_proc:

   add_tasks := proc(all_add, add_cons, v)
   begin 
       if not contains(all_add, add_cons) then
          if best_int[1] <> {} then
             if v > best_int[2] then
                cnt4 := cnt4 + 1;
                return(FALSE)
             end_if
          end_if;
          TRUE
       else
          cnt4 := cnt4 + 1;
          FALSE
       end_if;
   end_proc:
   
   # Hauptprogramm #

   userinfo(1, "Searching for an integer solution.");
   userinfo(1, "Entering Land/Doig algorithm.");

   # Initialisiere den Aufgaben-Pool #
   
   pool := [ [ f, cons, cond, {} ] ];

   # Bislang beste ganzzahlige Loesung #

   best_int := [ {} ]; 

   # Menge aller bisherigen zusaetzlichen Restriktionen #

   all_add := {};

   # Task-Counter fuer statische Zwecke #

   cnt1 := 0; cnt2 := 1; cnt3 := 0; cnt4 := 0;

   while pool <> [] do
      
      cnt1 := cnt1 + 1;

      task := pool[1]; # Aufgabe mit kleinstem Zielfunktionswert #
      f    := task[1]; cons := task[2];
      cond := task[3]; add_cons := task[4];
      pool[1] := NIL; # Loesche die Aufgabe aus dem Pool #

      s := linopt::minimize_generic(f, cons union add_cons, cond);

      if s = {} then 
         next 
      elif s = hold(UNBOUNDED) or s = FAIL then
         return( s )
      end_if;

      # s <> {} #

      # Speichere nicht ganzzahlige Komponenten, die auch in #
      # der Menge 'int_cons' vorhanden sind, der Loesung 's' #
      # in der Liste 'non_int'.                              #

      non_int := [];
      for eqn in s do 
          # eqn: x = c #
          x := op(eqn, 1); c := op(eqn, 2);
          if type(c) <> DOM_INT and contains(int_cons, x) then
             non_int := append(non_int, [x,c])
          end_if
      end_for;

      v := subs(f, s); # Zielfunktionswert dieser Aufgabe #

      if non_int = [] then

         # Ganzzahlige Loesung bzgl. der Variablen #
         # 'int_cons' gefunden                     #

         userinfo(2, "Integer solution found: ".expr2text(s));

         if best_int[1] = {} then
            # Erste ganzzahlige Loesung gefunden #
            best_int := [ s, v ]
         elif v < best_int[2] then
            # Diese ganzzahlige Loesung ist die bisher Beste #
            best_int := [ s, v ]
         else
            # Ist v >= best_int[2], so ist der Zielfunktionswert v #
            # dieser ganzzahligen Loesung s groesser und somit     #
            # schlechter als der bislang beste, d.h. kleinste Ziel-#
            # funktionswert best_int[2] der ganzahligen Loesung    #
            # best_int[1]. Diese Aufgabe wird daher nicht weiter   #
            # aufgeteilt.                                          #
            next
         end_if;

         # Streichung aller Aufgaben im Pool, deren Zielfunk-   #
         # tionswert groesser ist als best_int[2]               #

         j := 0;
         for i from 1 to nops(pool) do
             task := pool[i-j]; 
             if task[5] > best_int[2] then

                pool[i-j] := NIL; j := j + 1;

                userinfo(4, "The following task with the additional constraints");
                userinfo(4, task[4]);
                userinfo(4, "is rejected, because the value of the objective");
                userinfo(4, "function is greater than the value of the integer");
                userinfo(4, "solution found so far.");
                cnt3 := cnt3 + 1;

             end_if
         end_for

     else # non_int <> [], d.h. Loesung s ist nicht-ganzzahlig. #

         for eqn in non_int do
 
             x := eqn[1]; c := floor(eqn[2]); 

             if add_tasks(all_add, add_cons union {x <= c}, v) then

                task := create_task( f, cons, cond, add_cons, v, x <= c);
                pool := insert_ordered(task, pool, cmp);

                task := create_task( f, cons, cond, add_cons, v, x >= c+1);
                pool := insert_ordered(task, pool, cmp);

                all_add := all_add union { add_cons union {x <= c  } };
                all_add := all_add union { add_cons union {x >= c+1} };

                userinfo(4, "Generating 2 tasks with additional constraints");
                userinfo(4, add_cons union {x <= c  }, add_cons union {x >= c+1});

                cnt2 := cnt2 + 2;

             else

                userinfo(4, "The following tasks with the additional constraints");
                userinfo(4, add_cons union {x <= c  }, add_cons union {x >= c+1});
                userinfo(4, "are rejected, because they are already computed");
                userinfo(4, "or the value of the objective function is greater");
                userinfo(4, "than value of the integer solution found so far.");

                cnt4 := cnt4 + 2;

             end_if

         end_for

     end_if

   end_while;

   userinfo(3, "Tasks generated: ".cnt2);
   userinfo(3, "Tasks executed : ".cnt1);
   userinfo(3, "Tasks rejected");
   userinfo(3, "- from the pool           : ".cnt3);
   userinfo(3, "- before entering the pool: ".cnt4);

   best_int[1]

end_proc:
