alias(verify2=misc::check_globals_aux):

misc::checkFunction:=proc(f)
  local rememberTable;
begin
   if type(f)=DOM_FUNC_ENV then 
     if type(op(f,3)) = DOM_TABLE then
       rememberTable := map({op(op(f,3))}, op, 2);
       if contains( map(rememberTable, type), stdlib::LoadProc) then
         rememberTable := map(rememberTable, level, 2 );
       end_if;
       map(select( rememberTable, testtype, DOM_PROC), 
           misc::checkFunction);
       map(select( rememberTable, testtype, DOM_FUNC_ENV), 
           misc::checkFunction);
     end_if;
     f:=op(f,1) 
   end_if;
   if type(f)=DOM_PROC then
      userinfo(1, "checking function ".expr2text(op(f,6))." ...");
      misc::checkGlobals(f);
      misc::checkLocals(f);
   else
     if type(f) = "fun" then
         userinfo(1, "checking function ".expr2text(op(f,3))." ...");
         misc::checkGlobals(f);
         misc::checkLocals(f);         
     else
       if type(f) <> DOM_EXEC then
         print(Unquoted, "Warning: unexpected argument in misc::checkFunction ".
                         "from type ".expr2text(type(f)).", not checked\n");
         userinfo(2, "the argument is: ".expr2text(expose(f))):
       end_if;
     end_if;
   end_if;
   null()
end_proc:

misc::checkIgnore := # these variables will be ignored in all check routines #
  stdlib::ENVIRONMENT_VARIABLES union stdlib::SYSTEM_CONSTANTS union 
    stdlib::LIBRARY_CONSTANTS union stdlib::OPTIONS
    union stdlib::PROTECTED:

misc::checkGlobals := proc(f) local procName, pars,locals,body,s;
  begin
    if type(f) = DOM_PROC then 
      if type(op(f,6)) = DOM_STRING then
        procName := op(f,6);
      elif op(f,6) <> NIL then
        procName := expr2text(op(f,6));
      else
        procName := "";
      end_if;
      pars:=(if op(f,1)=NIL then null() else op(f,1) end_if);
      locals := (if op(f,2)=NIL then null() else op(f,2) end_if);
      body := op(f,4);
    elif type(f) = "fun" then
      procName := op(f,3);
      pars := null();
      locals := null();
      body := op(f,5);
    else
      return(null())
    end_if;

    if body = hold(body) then
      return(null());
    end_if;
        
    s:=verify2(body,{pars,locals}) minus misc::checkIgnore;
    if s<>{} then 
      if procName = "" then 
        print(Unquoted,"Warning: global variable(s) used in ".
                       "unnamed procedure:\n\t".expr2text(op(s))."\n");
        userinfo(2, "checked function was :\n".expr2text(f)." ...");
      else
        print(Unquoted,"Warning: global variable(s) used in ".
                       procName.":\n\t".expr2text(op(s))."\n");
      end_if;
    end_if; 
  end_proc:

# returns the set of global variables used #
verify2 := proc() local s,pars_and_locals,i,t, tmp;
   begin
      s := args(1..args(0)-1);
      if s = hold(s) then return({}) end_if;
      
      pars_and_locals := args(args(0));
      if contains({"hold","args",DOM_DOMAIN,DOM_RAT,DOM_BOOL,DOM_INT,DOM_STRING,DOM_FLOAT,DOM_COMPLEX,"error"},type(s)) then {}
      elif type(s)=DOM_IDENT then
         if contains({DOM_FUNC_ENV,DomainConstructor,Category,CategoryConstructor,DOM_DOMAIN},type(level(s,2))) then
            return({})
         end_if;
	 if contains({"DIGITS","DOM_RAT","DOM_POLY","DOM_STRING","DOM_IDENT","DOM_EXPR","DOM_BOOL","DOM_INT","DOM_COMPLEX","DOM_PROC","DOM_FLOAT","DOM_SET","DOM_DOMAIN"},expr2text(s)) then return({}) end_if;
	 if not contains(pars_and_locals,s) then {s} else {} end_if;
      elif type(s)=DOM_PROC then
         if op(s,4)=NIL then {}
	 else verify2(op(s,4),pars_and_locals union {op(s,1),op(s,2)})
         end_if
      elif type(s)="fun" then
         if op(s,5)=NIL then {}
	 else verify2(op(s,5),pars_and_locals)
         end_if        
      elif type(s)="function" and op(s,0)=hold(func) then
         verify2(op(s,1),pars_and_locals union {op(s,2..nops(s))});
      elif type(s)="block" then
         verify2(op(s,2),pars_and_locals union {op(op(s,1))});
      elif type(s) = "_seqgen" then
        i := nops(s);          # the kernel does no flattening !! #
        if type(op(s,i)) = "_equal" then
	  tmp := op(s,[i,1]);
	  if type(tmp) = "hold" and type(op(tmp,1)) = DOM_IDENT then 
            pars_and_locals := pars_and_locals union {op(tmp,1)};
          end_if;
        end_if;
         op(_union(map({op(s,1..i-1)}, verify2, pars_and_locals)));
      elif type(s) = "_for_par" then 
	 verify2([op(s,1..4)], pars_and_locals) union
         verify2([op(s,6..nops(s))], pars_and_locals union {op(s,5)}) 
      elif type(s) = "_for_in_par" then
	 verify2([op(s,1..2)], pars_and_locals) union
	 verify2([op(s,4)], pars_and_locals union {op(s,3)})
      elif s<>op(s) then # to avoid infinite recursion #
         t:={};
	 for i from 1 to nops(s) do
	    if op(s,i)<>NIL then
	       if nops([op(s,i)])=1 then t:=t union verify2(op(s,i),
			pars_and_locals)
	       else t:=t union verify2([op(s,i)],pars_and_locals)
	       end_if
	    end_if;
	 end_for;
         t
      else {} # for basic objects #
      end_if
   end_proc:

misc::checkLocals:=proc(f) local procName, notused;
begin
   if type(f) = DOM_PROC then
      if type(op(f,6)) = DOM_STRING then
        procName := op(f,6);
      elif op(f,6) <> NIL then
        procName := expr2text(op(f,6));
      else
        procName := "";
      end_if;
   elif type(f) = "fun" then
      procName := op(f,3);
   end_if:
      notused:=misc::checkLocals2(f);
      if op(notused, 1)<>{} then
        if procName = "" then 
          print(Unquoted,"Warning: unused variable(s) in unnamed procedure".
                         ":\n\t".expr2text(op(op(notused,1)))."\n");
          userinfo(2, "checked function was :\n".expr2text(f)." ...");
        else
          print(Unquoted,"Warning: unused variable(s) in ".procName.
                         ":\n\t".expr2text(op(op(notused,1)))."\n");
        end_if;
      end_if;
      if op(notused, 2)<>{} then
        if procName = "" then 
          print(Unquoted,"Warning: protected variable(s) used ".
                         "as local variable(s) in unnamed procedure". 
			 ":\n\t".expr2text(op(op(notused, 2)))."\n");
        else
          print(Unquoted,"Warning: protected variable(s) used as".
                         " local variable(s) in ".procName.
                         ":\n\t".expr2text(op(op(notused, 2)))."\n");
        end_if;
      end_if;
      if op(notused, 3)<>{} then
        if procName = "" then 
          print(Unquoted,"Warning: illegal assignment(s) to protected".
	                 " variable(s) in unnamed procedure:\n\t". 
	                 expr2text(op(op(notused, 3)))."\n");
        else
          print(Unquoted,"Warning: illegal assignment(s) to protected".
	                 " variable(s) in ".
	  procName.":\n\t".expr2text(op(op(notused, 3)))."\n");
        end_if;
      end_if;
      if type(f) = DOM_PROC and {op(f, 1)} intersect stdlib::PROTECTED <> {} then
        if procName = "" then 
          print(Unquoted,"Warning: protected variable(s) used ".
                         "as parameter in unnamed procedure". 
			 ":\n\t".expr2text(op({op(f, 1)} intersect stdlib::PROTECTED))."\n");
        else
          print(Unquoted,"Warning: protected variable(s) used as".
                         " parameter in ".procName.
                         ":\n\t".expr2text(op({op(f, 1)} intersect stdlib::PROTECTED))."\n");
        end_if;
      end_if;
end_proc:

# adapted from purge_locals in MuPAD german manual page 78,
  returns a set of not used local variables 
  
  returns a sequence of 3 sets: 
  
  1.  the local variables that were not used in this statement
  2.  the protected variables that were used as locals
  3.  protected variables which got a value assigned
#
misc::checkLocals2 := proc(f)
local s, i, tmp;
begin
   if type(f)=DOM_PROC then
      if f = stdlib::sysblock then return({},{},{}) end_if;
      if op(f,2)=NIL then s:={}, {}, {}
      else 
        s := select({op(f,2)},fun(not has(op(f,4),args(1)))),
             {op(f,2)} intersect stdlib::PROTECTED, {};
      end_if;
      tmp := misc::checkLocals2(op(f,4));
      tmp := op(zip([s], [tmp], _union));
# this trick with select finds procedures in procedures, where a 
  local variable of the father is only used by the child-procedure       #
#    select(op(tmp, 1), fun(not has(op(f,4),args(1)))), op(tmp,2), op(tmp,3);#
     tmp := op(tmp, 1) minus verify2(op(f,4), {}), op(tmp,2), op(tmp,3);
   elif type(f)="fun" then
      misc::checkLocals2(op(f,5));
   elif type(f) = "function" and  type(op(f,0)) = "domattr" and
      contains({hold(stdlib::syseval), hold(stdlib::sysblock)}, op(f,0)) then
     if nops(f) > 1 then
       tmp := [misc::checkLocals2(op(f,i))] $ i=1..nops(f);
       _union(map(tmp, op, 1)), _union(map(tmp, op, 2)), {};
     else
       tmp := misc::checkLocals2(op(f,1));
       op(tmp,1), op(tmp,2), {}
     end_if;
   elif type(f) = "block" then 
      s := select({op(op(f,1))},fun(not has(op(f,2),args(1)))),
             {op(op(f,1))} intersect stdlib::PROTECTED, {};
      tmp := misc::checkLocals2(op(f,2));
      tmp := op(zip([s], [tmp], _union));
# this trick with select finds procedures in procedures, where a 
  local variable of the father is only used by the child-procedure       #
#      select(op(tmp, 1), fun(not has(op(f,2),args(1)))), 
        op(tmp,2) minus {hold(_assign)}, op(tmp,3);             #
     op(tmp, 1) minus verify2(op(f,2), {}), op(tmp,2) minus {hold(_assign)}, 
        op(tmp,3);
     
   elif type(f) = "_assign" then
     tmp := [misc::checkLocals2(op(f,i))] $ i=1..nops(f);
     s := _union(map(tmp, op, 1)), _union(map(tmp, op, 2)), 
            _union(map(tmp, op, 3)); 
	    
     if type(op(f,1)) = "_index" or     # don't check the indices   #
        type(op(f,1)) = "domattr" then  # or attributes             #
       tmp := op(f,[1,1]);
       if type(tmp) = "args" then
         tmp = null();
       end_if; 
     elif type(op(f,1)) = "function" then
       tmp := op(f, [1,0]);
     else
       tmp := op(f,1);
     end_if;
     for i in (stdlib::SYSTEM_TABLES union stdlib::PROTECTED )
                                minus {hold(_index), hold(domattr)} do
       if has(tmp, i) then
         s := op(s,1), op(s,2), op(s,3) union {i};
       end_if;
     end_for;   
     s;
   elif type(f)=DOM_DOMAIN then 
     {}, {}, {} # to avoid infinite loops with domains #
   elif nops(f)=0 then 
     {}, {}, {}
   elif f<>op(f) then
     if nops(f) > 1 then 
       tmp := [misc::checkLocals2(op(f,i))] $ i=1..nops(f);
       _union(map(tmp, op, 1)), _union(map(tmp, op, 2)), 
            _union(map(tmp, op, 3)); 
     else
       misc::checkLocals2(op(f,1));
     end_if;
   else {}, {}, {}
   end_if;
end_proc:
