#++
   misc::checkLib calls misc::checkLib on all proc's and fun's
                  in a domain (specifically a library domain)
                  and scans recursively all included domains.
++#


misc::checkLib := proc(d)
  local i, PRETTY_PRINT, dom, procName, libName;
begin
   PRETTY_PRINT := FALSE:   
   if type(d) <> DOM_DOMAIN then
     print(Unquoted, "Warning: Library domain expected, not checked"):
     return(null()):
   end_if;
   level(d,2);       # load the library file #
   for i in op(d) do # this loads all loadproc's #
     if type(op(i,2)) = stdlib::LoadProc then 
       level(op(i,2),2);
     end_if;
   end_for;
 
   libName:= (if d::name = FAIL then d::key else d::name end_if);
   case type(libName) 
     of DOM_STRING do
       procName := libName;
       break;
     of DOM_FAIL do
       procName := "";
       break;
     otherwise
       procName := expr2text(libName);
   end_case;

   userinfo(1,
     (if procName <> "" then
       "\nchecking domain ".procName." ...\n"
     else
       "\nchecking unnamed domain ...\n"
     end_if)):
   map( select(map({op(d)}, op, 2), testtype, DOM_PROC), 
               misc::checkFunction);
   map( select(map({op(d)}, op, 2), testtype, DOM_FUNC_ENV), 
               misc::checkFunction);  
   dom := select(map({op(d)}, op, 2), testtype, DOM_DOMAIN);
   map(select(dom, _unequal, d), misc::checkLib);
   
   null():
end_proc:

misc::checkAll :=
proc()
  local PRETTY_PRINT, i, MAXDEPTH;
begin
  PRETTY_PRINT:= FALSE: 
  MAXDEPTH := 5000:  # 500 is too small #

  for i in anames(3) do  # load loadproc nodes #
    level(i, 2);
  end_for:

# noisy output for debugging # 
# setuserinfo( misc::checkFunction, 2):
  setuserinfo( misc::checkLib, 2):      
  setuserinfo( misc::checkGlobals, 2):      
  setuserinfo( misc::checkLocals, 2):      #
    
  misc::checkFunction: 
  print( Unquoted, "\nUnchecked global variables: \n".
                     "---------------------------\n\n");
		     
  misc::tableForm( 	     
      sort([op(map(misc::checkIgnore, expr2text))]), 1, Unquoted);

  print( Unquoted, "\nChecking procedures ...\n".
                     "-----------------------\n" );
  map(select( map(anames(3), level, 1), testtype, DOM_PROC), 
      misc::checkFunction); 

  print( Unquoted, "\nChecking function environments ...\n".
                     "----------------------------------\n" );
  map(select( map(anames(3), level, 1), testtype, DOM_FUNC_ENV), 
      misc::checkFunction); 

  print( Unquoted, "\nChecking fun's and func's ...\n".
                     "-----------------------------\n" );
  map(select( map(anames(3), level, 1), testtype, "fun"), 
      misc::checkFunction); 

  print( Unquoted, "\nChecking domains ...\n".
                     "--------------------\n" );
  map(select( map(anames(3), level, 1), 
              testtype, DOM_DOMAIN), misc::checkLib);  

  print( Unquoted, "\nDone" );
end_proc:


