# $Date: 1996/06/28 09:28:52 $ $Author: frankp $ $Revision: 1.2 $ #
# kg, 05/10/95 #

misc::frozen:= table():

#++
misc::freeze -- make function inactive

misc::freeze(f)

f - function

misc::freeze(f) returns a function which is an 'inactive' version of f.
The function returned by misc::freeze:
- only evaluates its arguments, but does not compute anything else,
- prints like f and has the same expression type as f,
- has all the functional attributes of f.

misc::freeze is used for example to create inactive functions to be
returned instead of expensive functions like 'int'. The inactive 'int'
is returned if the expression can not be integrated further in order to
avoid re-evaluation of 'int'.

Use 'misc::unfreeze' to re-activate the function.
++#

misc::freeze:= proc(f)
    option hold;
    local v, fn;
begin
    fn:= expr2text(f);
    v:= context(f);
    v:= funcattr(v, "unfreeze", v);
    v:= subsop(v, 1=subsop(
    	    proc() begin 
    	        subsop(hold(F)(args()), 0=misc::frozen[fn]) 
    	    end_proc,
    	    [4,2,2,2]=fn));
    sysassign( misc::frozen[fn], v );
    v
end_proc:

#++
misc::unfreeze -- re-activate inactive funtion

misc::unfreeze(f)

f - inactive function created by 'misc::freeze'

misc::unfreeze(f) returns the active version of an inactive function
which has been created by 'misc::freeze'. Use 'subsop(e,
0=misc::unfreeze(op(e,0)))' to unfreeze an expression where the operator
has been frozen.
++#

misc::unfreeze:= proc(e) local f; begin
    if testargs() then
        if args(0) <> 1 then error("wrong no of args") end_if;
        if domtype(e) <> DOM_FUNC_ENV then error("no function") end_if;
    end_if;
    f:= funcattr(e, "unfreeze");
    if f = FAIL then e else f end_if
end_proc:

# end of file #
