# kg, 26/06/96 #

#++
misc::mapFun -- map function to fun

misc::mapFun(pf, f, a1,...)

pf    - pure function
f     - function
a1... - additional arguments for f

misc::mapFun maps f to the body of the pure function pf, using the
additional arguments ai. A new pure function is returned.

misc::mapFun returns FAIL if the body of pf can not be converted to an
expression because it contains args(x) or args(x..y) where x or y are
not positive integers.

Examples:

misc::mapFun(func(E^x*ln(E), x), simplify)  -->  func(exp(X1), X1)
++#

misc::mapFun:= proc(f, g)
begin
    if testargs() then
        if args(0) < 2 then error("wrong no of args") end_if;
	if type(f) <> "fun" then
	    error("not a pure function")
	end_if
    end_if;

    f:= misc::subsFunArgs(f);
    if f = FAIL then return(FAIL) end_if;
    g:= g(f[1], args(3..args(0)));
    f:= f[2];

    case nops(f)
    of 0 do
	newpurefunc(expr2text(hold(func)(g)), NIL, g);
	break;
    
    of 1 do
	newpurefunc(
	    expr2text(hold(func)(g,
	    			 (if contains(f,i) then f[i] else genident() end_if) 
	    			 	$ hold(i)=1..op(f,[1,1]))),
	    NIL,
	    misc::subsFreeVars(g, op(f,[1,2])=newfuncarg(op(f,[1,1])), 
	    		       Unsimplified));
        break;
            
    otherwise
	newpurefunc(
	    expr2text(hold(func)(g,
	    			 (if contains(f,i) then f[i] else genident() end_if) 
	    			 	$ hold(i)=1..max(map(op(f), op, 1)))),
	    NIL,
	    misc::subsFreeVars(g, 
	    		       (op(f,[i,2])=newfuncarg(op(f,[i,1]))) 
	    		       		$ hold(i)=1..nops(f),
	    		       Unsimplified))
    end_case
end_proc:

# end of file #
