# misc::minimalDomain(e,p) returns the minimal domain containing e with the
  property p.

  Output: either a domain or FAIL if not such domain found.

  Example: 
>> misc::minimalDomain(-3, Cat::Ring);

                               Dom::Integer

>> misc::minimalDomain(-3, Cat::Field);

                               Dom::Rational

>> misc::minimalDomain(3*x*y+1/2, Cat::Ring);

             Dom::DistributedPolynomial([x, y], Dom::Rational)

>> misc::minimalDomain(3*x*y+2, Cat::Ring);

             Dom::DistributedPolynomial([x, y], Dom::Integer)

>> misc::minimalDomain(3*x*y+2, Cat::Field);

      Dom::Fraction(Dom::DistributedPolynomial([x, y], Dom::Integer))

>> misc::minimalDomain(3*x*y+2/x, Cat::Field);

      Dom::Fraction(Dom::DistributedPolynomial([x, y], Dom::Integer))

#

misc::minimalDomain := proc(e,prop) local l,p,R;
begin
   case [domtype(e),prop]
   of [DOM_INT,Cat::Ring] do return(Dom::Integer)
   of [DOM_INT,Cat::Field] do # go through #
   of [DOM_RAT,Cat::Ring] do # go through #
   of [DOM_RAT,Cat::Field] do return(Dom::Rational)
   of [DOM_IDENT,Cat::Ring] do
	return(Dom::DistributedPolynomial([e],Dom::Integer))
   of [DOM_EXPR,Cat::Ring] do
      p:=poly(e);
      if p<>FAIL then
         R:=misc::unionDomain(op(map([coeff(p)],misc::minimalDomain,Cat::Ring)));
         return(Dom::DistributedPolynomial(op(p,2),R))
      else return(Dom::ExpressionField())
      end_if
   of [DOM_IDENT,Cat::Field] do
       return(Dom::Fraction(misc::minimalDomain(e,Cat::Ring)))
   of [DOM_EXPR,Cat::Field] do
      l:=misc::unionDomain(misc::minimalDomain(numer(e),Cat::Ring),
        		   misc::minimalDomain(denom(e),Cat::Ring));
      if l=FAIL then l:=misc::minimalDomain(e,Cat::Ring) end_if;
      if l<>FAIL then l:=Dom::Fraction(l) end_if;
      return(l)
   end_case;
   if prop=Cat::Field then return(Dom::ExpressionField()) end_if;
   FAIL
end_proc:

alias(Ind(x)=domattr(x,"constructor_args")[1]):
alias(ring(x)=domattr(x,"constructor_args")[2]):

misc::unionDomain:=proc(a,b) local R;
begin
   case args(0)
   of 0 do return(FAIL)
   of 1 do return(a)
   of 2 do
      if a=b then return(a) end_if;
      case [a,b]
      of [Dom::Integer,Dom::Rational] do return(Dom::Rational)
      otherwise
         if {a::constructor,b::constructor}={Dom::DistributedPolynomial} then
            if (R:=misc::unionDomain(ring(a),ring(b)))<>FAIL then
               return(Dom::DistributedPolynomial(
		[op({op(Ind(a))} union {op(Ind(b))})],R))
            end_if
         end_if;
         return(FAIL)
      end_case
   otherwise return(misc::unionDomain(a,b),args(3..args(0)))
   end_case
end_proc:
