# $Date: 1997/09/17 06:34:43 $ $Author: kg $ $Revision: 1.14 $ #

# kg, 13/01/95 #

#++
misc::subsFreeVars -- substitute only free variables

misc::subsFreeVars(ex [, o=n...] [,Unsimplified])

ex, o, n     - expressions
Unsimplified - option

misc::subsFreeVars(ex,...) works similar to subs(ex,...) but tries to
substitute free variables only, ie. formal parameters and local variables of
procedures are not substituted even if they are left hand sides of the
substitution equations.

Substitutions inside of function environments and domains are also not
made because most often the user don't want them to occure and often they
may cause infinite recursions if a domain is defined recursively. (subs
changes a domain D if it is the 0-operand of a domain element created by
new(D,...) and has no method "subs".) 
++#

misc::subsFreeVars:= proc(ex)
    option hold;
    local doSubs, unsimp, subsProc, subsEq, i;
begin
    # do not flat 1st op #
    ex:= context(ex);
    subsEq:= context([args(2..args(0))]);
    if nops(subsEq) = 0 then return(ex) end_if;
    if subsEq[nops(subsEq)] = hold(Unsimplified) then 
    	if nops(subsEq) = 1 then return(ex) end_if;
	subsEq[nops(subsEq)]:= NIL;
    	unsimp:= hold(Unsimplified)
    else
        unsimp:= null()
    end_if;
    
    if testargs() then
    	if args(0) = 0 then error("wrong no of args") end_if;
    	if {op(map(subsEq, type))} <> {"_equal"} then
    	    error("illegal substitution equation")
    	end_if
    end_if;

    doSubs:= fun((
	if args(0) = 1 then return(args(1)) end_if;
	case domtype(args(1))
        of DOM_FUNC_ENV do
	of DOM_DOMAIN do
	    args(1);
    	    break;
    	
	of DOM_EXPR do
    	    # special bits need special treatment... #
    	    case type(args(1))
    	    of "_mult" do
    	        if nops(args(1)) = 2 then
		    if op(args(1),1) = 1 then
			return(doSubs(op(args(1),2), args(2)))
		    elif op(args(1), 2) = -1 then
			return(- doSubs(op(args(1),1), args(2)))
		    end_if
    	        elif op(args(1), nops(args(1))) = -1 then
		    return(- hold(_mult)(
			doSubs(op(args(1),i), args(2))
			$ hold(i)=1..(nops(args(1))-1)
		    ))
    	        end_if;
    	        break;
    	    of "_power" do
    	        if op(args(1),2) = -1 then
    	            return(1 / doSubs(op(args(1),1), args(2)))
    	        end_if;
    	        break;
    	    of "_not" do
    	        return(not doSubs(op(args(1),1), args(2)));
    	    end_case;
    	    subsop(args(1), (i=doSubs(op(args(1),i), args(2)))
			    $ hold(i)=0..nops(args(1)), unsimp);
	    break;
	
	of DOM_PROC do
	    subsProc(args(1), args(2));
	    break;
	of DOM_RAT do
	of DOM_COMPLEX do
	    subs(args(1), op(args(2)), unsimp);
	    break;
	
	of DOM_LIST do
	of DOM_SET do
	of DOM_ARRAY do
	    map(args(1), doSubs, args(2));
	    break;
	    
	otherwise
    	    if nops(args(1)) = 1 then
    	        subs(args(1), op(args(2)), unsimp)
    	    else
	        subsop(args(1), (i=doSubs(op(args(1),i), args(2)))
    			        $ hold(i)=1..nops(args(1)), unsimp)
            end_if
	end_case
    ));

    subsProc:= 	proc(ex, a)
	local bound;
    begin
	bound:= {(if op(ex,1) = NIL then null() else
		      if type(op(ex,1)) = "_exprseq" then op(op(ex,1))
    		      else op(ex,1) end_if
    	          end_if),
		 (if op(ex,2) = NIL then null() else
    		      if type(op(ex,2)) = "_exprseq" then op(op(ex,2))
    		      else op(ex,2) end_if
    	          end_if)};
	a:= select(a, fun(not contains(bound, op(args(1),1))));
	if a = [] then
    	    ex
	else
    	    subsop(ex, 4=doSubs(op(ex,4), a), 5=doSubs(op(ex,5), a), unsimp)
	end_if
    end_proc;

    if type(ex) = "_exprseq" then
	op(doSubs([ex], subsEq))
    else
	doSubs(ex, subsEq)
    end_if
end_proc:

# end of file #
