
#++
misc::subsFunArgs -- replace args(i) by dummy variable in pure function body

misc::subsFunArgs(f)

f - pure function

misc::subsFunArgs(f) returns the body of f, where each expression args(i)
has been replaced by a dummy variable. Returns a list

	[ b, table(a_1=v_1,... ) ]

where b is the substituted body of f, such that args(a_i) has been replaced 
by v_i.

If the body contains expressions args(x) or args(x..y) where x or y are
not positive integers, FAIL is returned.
++#

# Oops - without a name entry the very first equality test fails... (why?) #
domattr(domtype(newfuncarg(1)), "name"):= FAIL:

misc::subsFunArgs:= proc(f)
    local body, _subsargs_vars, _subsargs_failed;
begin
    if testargs() then
        if args(0) <> 1 then error("wrong no of args") end_if;
	if type(f) <> "fun" then
	    error("not a pure function")
	end_if
    end_if;
    
    # replace args(i) and newfuncarg(i) by dummy formal parameters #
    _subsargs_vars:= table();
    _subsargs_failed:= FALSE;
    body:= maprec(op(f,5), 
    		{"args", domtype(newfuncarg(1))} = proc() local a, i; begin
    		    a:= op(args(1),1);
    		    if a = 0 then
    		       _subsargs_failed:= TRUE;
    		       breakmap();
    		       args(1)
    		    elif domtype(a) = DOM_INT then
    		        if contains(_subsargs_vars, a) then
    		            _subsargs_vars[a]
    		        else
    		            _subsargs_vars[a]:= genident()
    		        end_if
    		    else
    		        if type(a) = "_range" and 
    		           domtype(op(a,1)) = DOM_INT and op(a,1) > 0 and
    		           domtype(op(a,2)) = DOM_INT and op(a,2) > 0 then

			   ((if contains(_subsargs_vars, i) then
			         _subsargs_vars[i]
			     else
			         _subsargs_vars[i]:= genident()
			     end_if) $ hold(i)=a)
    		        else
			   _subsargs_failed:= TRUE;
			   breakmap();
			   args(1)
    		        end_if
    		    end_if
    		end_proc,
    		hold(PostMap));

    if _subsargs_failed then 
        FAIL
    else
        [body, _subsargs_vars]
    end_if
end_proc:

# end of file #
