# $Date: 1996/12/04 13:15:48 $ $Author: frankp $ $Revision: 1.6 $ #
# #
# frankp, 28/08/96 #

#++
   tabForm.mu

        misc::tableForm  -- printing a set or list in table form

	tableForm( tab [,seperator ] [, colwidth] [,Unquoted] )

	tab      : set or list of expressions	
	seperator: (optional) string
	colwidth : (optional) non-negative integer
 	Unquoted : (optional) identifier

	tableForm( tab ) prints the entries of tab in a table form. The width
 	of the table depends on the size of TEXTWIDTH. 
	If a seperator is given then it is appended to each entry instead of
	the empty string.
	colwidth states an additionaly space among two columns. Its default
	value is zero.
	Each entry in tab is converted to a string. Thus, strings in tab
	will be printed in quotation marks. This can be avoided with
	the option 'Unquoted'.

	Note: The entries of tab will not be sorted explicetely. The value of
	      PRETTY_PRINT will not be changed during the output of the entries.

	Examples:

	>> PRETTY_PRINT:= FALSE:
	>> misc::tableForm( 
	    ["fp::apply", "fp::bottom", "fp::curry", "fp::fixargs", "fp::fixedpt",
             "fp::fold", "fp::nest", "fp::nestvals", "fp::unapply"], " ", Unquoted );

        fp::apply fp::bottom   fp::curry   fp::fixargs fp::fixedpt fp::fold,
        fp::nest  fp::nestvals fp::unapply

	>> tableForm( [xx $ 35], ",", 1 );
	xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx,
	xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx, xx
++#

misc::tableForm:= proc( tab, seperator, colwidth )
    local width, lengths, maxlen, col, pos, l, collen, tabwidth,
	  n, i, k, s, tabspace;
begin
    if testargs() then
	if args(0) < 1 or args(0) > 4 then
	    error("wrong no of args")
	end_if;
	if not contains({DOM_SET,DOM_LIST},domtype(tab)) then
	    error("1st argument must be a list or set")
	end_if;
	case args(0)
	of 4 do
	    if args(4) <> hold(Unquoted) then
		error("invalid 4th argument")
	    end_if
	of 3 do 
	    if not testtype(colwidth,Type::NonNegInt) and colwidth <> hold(Unquoted) then
		error("invalid 3nd argument")
	    end_if
	of 2 do
	    if domtype(seperator) <> DOM_STRING and not testtype(seperator,Type::NonNegInt) 
               and seperator <> hold(Unquoted) 
            then
                error("invalid 2nd argument")
	    end_if
	end_case
    end_if;


    if contains( {op(args())},hold(Unquoted) ) then
	tab:= map(tab,
	    fun((if domtype(args(1))<>DOM_STRING then expr2text(args(1)) else args(1) end_if))
	)
    else
        tab:= map(tab,expr2text)
    end_if;

    n:= nops(tab);
    width:= TEXTWIDTH-1; # not possible to print first row with full width #

    case args(0)
    of 1 do 
	seperator:= ""; colwidth:= 0; break
    of 2 do
	if seperator = hold(Unquoted) then
	    colwidth:= 0;
	    seperator:= ""
	elif domtype(seperator) = DOM_STRING then
	    colwidth:= 0
	else	
	    colwidth:= seperator;
	    seperator:= ""
	end_if;
	break
    of 3 do 
	if colwidth = hold(Unquoted) then
	    if domtype(seperator) = DOM_STRING then
	        colwidth:= 0
	    else
		colwidth:= seperator;
	 	seperator:= ""
	    end_if
	end_if
    end_case;

    tabspace:= colwidth + strlen(seperator);

    # 1. Search for entries s1,s2,...,sn with l1>=l2>=...>=ln such that
	 l1+l2+...+ln<=width, whereby li is the string length of si.
	 Then n is the number of columns.
    #
    if domtype(tab) = DOM_SET then
	lengths:= map( [op(tab)],strlen )
    else
        lengths:= map( tab,strlen )
    end_if;

    col:= 0;
    tabwidth:= 0;
    maxlen:= 0;
    l:= lengths;

    repeat
        maxlen:= max(op(l));
	if tabwidth+maxlen+tabspace <= width then
	    tabwidth:= tabwidth + maxlen + tabspace;
	    col:= col + 1;
            pos:= contains( l, maxlen );
            l[pos]:= NIL
	else
	    l:= []
	end_if;
    until nops(l) = 0 end_repeat;

    # 2. Search for the maximum sizes of each column #
    collen:= [NIL $ col];
    ( collen[i]:= max((0; for k from i to n step col do %,lengths[k] end_for)) ) $ i=1..col;

    # 3. Print each column #
    for k from 1 to n step col do
	s:= "";
	for i from 0 to col-1 do
	    if k+i > n then break end_if;

	    if i = col-1 and k+i <> n then # end of row but not the last entry #
		s:= s . op(tab,k+i) . seperator
	    elif k+i = n then # last entry #
	 	s:= s . op(tab,k+i)
	    else
		l:= collen[i+1]-lengths[k+i] + colwidth;
	        s:= _concat( s, op(tab,k+i), seperator, " " $ l )
	    end_if
	end_for;
	print( Unquoted, s )
    end_for
end_proc:

# end of file #
