
misc::TestFile:= FAIL:

#++
misc::testinit -- init testing

misc::testinit([f])

f - file name (default is "test")

Opens the test result file 'f.res'. If the variable 'TEST_PATH' has a
value, it is used as a prefix for the path name.
++#

misc::testinit:= proc(f)
begin
    if args(0) = 0 then f:= "test"
    elif args(0) = 1 then
	if domtype(f) <> DOM_STRING then
	    error("illegal file name prefix")
	end_if
    else error("wrong no of args") end_if;
    if TEST_PATH <> hold(TEST_PATH) then
        f:= TEST_PATH.f
    end_if;
    sysassign( misc::TestFile, fopen(Text, f.".res", Write) );
    if misc::TestFile = FAIL then
	error("could not open results file")
    end_if;
    sysassign( misc::TestNo, 1 );
    sysassign( misc::TestFunc, FAIL );
    null()
end_proc:

#++
misc::testexit -- exit testing

misc::testexit()

Close test results file.
++#

misc::testexit:= proc()
begin
    fclose(misc::TestFile);
    sysassign( misc::TestNo, 0 );
    sysassign( misc::TestFunc, FAIL );
    null()
end_proc:

#++
misc::testfunc -- init testing of function

misc::testfunc(f)

f - expression

misc::testfunc(f) initializes the testing of function f. The argument
is not evaluated. If the test result file has not been opened it is
opened by misc::testfunc. If the variable 'TEST_PATH' has a
value, it is used as a prefix for the path name.
++#

misc::testfunc:= proc()
    option hold;
begin
    if args(0) <> 1 then error("wrong no of args") end_if;
    if domtype(misc::TestFile) <> DOM_INT then
	sysassign( misc::TestFile, fopen(Text, 
		(if TEST_PATH <> hold(TEST_PATH) then
		     TEST_PATH."test.res"
		 else
		     "test.res"
		 end_if),
		Write)
	);
	if misc::TestFile = FAIL then
	    error("could not open test.res")
	end_if;
    end_if;
    sysassign( misc::TestNo, 1 );
    sysassign( misc::TestFunc, args(1) );
    null()
end_proc:

#++
misc::test -- test function

misc::test()
misc::test(a)
misc::test(a, b)

a, b - expressions

misc::test prints - depending upon the parameters - the current function
and test number onto the test results file 'test.res'. This signals
an error in the current test.

misc::test() prints the current function and test number unconditional.

misc::test(a) prints the current function and test number if a is not
equal to TRUE or if an error occures during the evaluation of a.

misc::test(a, b) prints the current function and test number if a is not
equal to b (in the sense of _equal) of if an error occures during the
evaluation of a or b.
++#

misc::test:= proc(a, b)
    option hold;
    local r, PRETTY_PRINT, TEXTWIDTH, lprint;
begin
    lprint:=proc() local TEXTWIDTH, PRETTY_PRINT;
    begin
	PRETTY_PRINT:= FALSE; TEXTWIDTH:= NIL;
	print(args(i)$i=1..args(0));
	null()
    end_proc:

    if domtype(misc::TestFile) <> DOM_INT then
	error("results file has not been opened")
    end_if;
    if misc::TestFunc = FAIL then
	error("no function to test")
    end_if;
    PRETTY_PRINT:= FALSE;
    TEXTWIDTH:= 2048;
    
    userinfo(1,"testing ",misc::TestFunc);
    case args(0)
    of 0 do
	fprint(Unquoted, misc::TestFile, misc::TestFunc, misc::TestNo);
	lprint(Unquoted, "Error in test", misc::TestFunc, misc::TestNo);
	break;
    of 1 do
	userinfo(2,"  for ",a," = TRUE");
	if traperror((r:= context(a))) = 0 then
	    if r = TRUE then break end_if
	end_if;
	fprint(Unquoted, misc::TestFile, misc::TestFunc, misc::TestNo, a);
	lprint(Unquoted, "Error in test", misc::TestFunc, misc::TestNo, a);
	break;
    of 2 do
	userinfo(2,"  for ",a," = ",b);
	if traperror((r:= bool(context(a) = context(b)))) = 0 then
	    if r = TRUE then break end_if
	end_if;
	fprint(Unquoted, misc::TestFile, misc::TestFunc, misc::TestNo,
	       hold(_equal)(a, b));
	lprint(Unquoted, "Error in test", misc::TestFunc, misc::TestNo,
	       hold(_equal)(a, b));
	break;
    otherwise error("wrong no of args");
    end_case;
    sysassign( misc::TestNo, misc::TestNo + 1 );
    null()
end_proc:

# end of file #
