# numerical evaluation of f for k=a..infinity using Euler-Maclaurin formula.
   Author: Paul Zimmermann, Paul.Zimmermann@loria.fr, February 1996.
   Reference: Computational Recreations in Mathematica, Ilan Vardi, Addison Wesley,
	      1991, page 160, formula (8.16).

   Examples:
>> misc::freeze(sum)(1/k^2+1/k^3,k=1..infinity);

                         / 1    1                   \
                         | -- + --, k = 1..infinity |
                      sum|  2    3                  |
                         \ k    k                   /

>> float(%);

                                2.84699097

1.3>> float(sum(ln(n)/n^(1.5-14.1*I),n=1..infinity));

                     - 0.3541793679 + 0.01416426969 I

>> DIGITS:=40: numeric::maclaurin(ln(x)/x^2,x,2);

                0.9375482543158437537025740945678649778978
#
sysassign( sum, funcattr(sum,"float",proc(f,x) local k,a,b,i;
begin
   if type(x)<>"_equal" then error("range expected as second argument")
   elif type((k:=op(x,1)))<>DOM_IDENT then error("invalid argument")
   elif type((x:=op(x,2)))<>"_range" then error("range expexted")
   else
      a:=op(x,1); b:=op(x,2);
      if map({a,b},type) minus {DOM_INT,type(infinity)}<>{} then
         error("integers or +/-infinity expected as bounds")
      end_if;
      if b<a then b+1; b:=a-1; a:=%2 end_if;
      if type(a)=DOM_INT then
         if type(b)=DOM_INT then 
            # _plus(float(eval(subs(f,k=i)))$i=a..b) #
            _plus(float(subs(f,k=i,Simplified))$i=a..b)
         else # b=infinity #
            numeric::maclaurin(f,k,a)
         end_if
      else hold(float)(hold(sum)(f,x))
      end_if
   end_if
end_proc) ):

numeric::maclaurin:= proc(f,k,a) local F,n,m,S,L,t,oldS;
begin
   if type(a)<>DOM_INT then return(FAIL) end_if;
   F:=int(f,k);
   if type(F)="int" then return(FAIL) end_if;
   case (L:=limit(F,k=infinity))
   of infinity do 
   of -infinity do return(L)
   end_case;
   n:=a+DIGITS;
   t:=float(subs(f,k=n)); 
   if not contains({DOM_COMPLEX,DOM_FLOAT},type(t)) then return(FAIL) end_if;
   S:=float(_plus(subs(f,k=m)$m=a..n-1)+L-subs(F,k=n)+t/2);
   m:=1; f:=diff(f,k); 
   repeat
      t:=float(bernoulli(2*m)/fact(2*m)*subs(f,k=n));
      oldS:=S; S:=S-t; if S=oldS then break end_if;
      m:=m+1; f:=diff(f,k,k);
   until FALSE end_repeat;
   # we should check in addition that f is (2m+4)-times differentiable with
       diff(f,k$2*m+2) and diff(f,k$2*m+4) of one sign on 1..infinity #
   S
end_proc:

# end of file #
