# #
# $Date: 1997/10/14 14:48:11 $  $Author: stefanw $   $Revision: 1.12 $ #
# #

#++

	numeric::newton(f,x=a..b) tries to find a root of f=0 over x=a..b
	where a<=b, and a,b can be at +/-infinity, using Newton method.
	Output: either a root or null()
	Example:

	>> numeric::newton(x*tan(x)-1/2,x=-infinity..infinity);

                               0.6532711871
++#

numeric::newton := proc(f,x)
   local l,v,oldv,eps,df,i,v0,fv0,fv,die,dv,a,b,maxit,r;
begin
   if testargs() then
      if args(0)<1 or args(0)>2 then
	 error("expecting one or two arguments");
      end_if;
      if type(f)="_equal" then f:=op(f,1)-op(f,2) end_if;
      l:=indets(f);
      l:=l minus select(l,testtype,Type::Constant);
      if args(0) = 2 then
          if domtype(x) = DOM_IDENT then
	    a:= -infinity; b:= infinity
	  else
	    if type(x) <> "_equal" then
	      error("expecting indeterminate or range of the form 'x=a..b'");
	    elif type((r:= op(x,2))) <> "_range" then
	      error("expecting range of the form 'x=a..b'");
	    else
	      a:= float(op(r,1)); b:= float(op(r,2));
	      if (not testtype(a,NUMERIC)) and has(a,infinity)=FALSE or
	         (not testtype(b,NUMERIC)) and has(b,infinity)=FALSE 
	      then
		error("not a numeric range");
	      end_if;
	      x:= op(x,1)
	    end_if
	  end_if;
          if l<>{x} then error("no or more than one indeterminate") end_if
      elif nops(l) <> 1 then
	error("no or more than one indeterminate") 
      else
	x:= op(l,1)
      end_if
   else
	if domtype(x)=DOM_IDENT then
	    a:= -infinity; b:= infinity
	else
	    a:= float(op(x,[2,1])); b:= float(op(x,[2,2]));
	    x:= op(x,1)
	end_if
   end_if;

   df:=normal(diff(f,x));
   # choose a starting point at random #
   if has({a,b},infinity) then # infinite range #
      die:=fun(tan(float(atan(a)+(atan(b)-atan(a))*random()/10^12)))
   else die:=fun(float(a+(b-a)*random()/10^12))
   end_if;
   v0:=die();
   # fv0:=abs(float(eval(subs(f,x=v0)))); #
   fv0:=abs(float(subs(f,x=v0,Simplified)));
   for i from 1 to 9 do
      v:=die();
      # fv:=abs(float(eval(subs(f,x=v)))); #
      fv:=abs(float(subs(f,x=v,Simplified)));
      if fv<fv0 then v0:=v; fv0:=fv end_if;
   end_for;
   v:=v0;
   eps:=10^(-DIGITS);
   maxit:=round(3.4*ln(DIGITS)); # ln(10)/ln(2) ~ 3.4 #
   for i from 1 to maxit do
      oldv:=v;
      # a:=float(eval(subs(f,x=oldv))); #
      # b:=float(eval(subs(df,x=oldv)));#
      a:=float(subs(f,x=oldv,Simplified));
      b:=float(subs(df,x=oldv,Simplified));
      if abs(b)<eps then
      	   userinfo(2, "Zero derivative occurred, Newton iteration failed");
	   return(null())
      end_if;
      dv:=a/b;
      v:=oldv-dv;
      if abs(dv)<=abs(v)*eps then break end_if
   end_for;
   if i>maxit then null()
   # elif abs(float(eval(subs(f,x=v))))>eps then #
   elif abs(float(subs(f,x=v,Simplified)))>eps then
      null()
   else 
      numeric::complex_round(v,10.0^(-DIGITS))
   end_if
end_proc:

# end of file #
