
# $Date: 1997/11/25 10:16:46 $ # # $Author: stefanw $ # # $Revision: 1.4 $ #

#++    
numlib::mpqs - the multi-polynomial quadratic sieve for factoring integers

numlib::mpqs(n,options) 

n -  integer to be factored

The following options can be given in any order:

Factorbase=l          -  l: list of primes
		      use the members of l as a factorbase
MaxInFactorbase=maxfb     -  maxfb: positive integer
		      use all suitable primes below maxfb as a factorbase
		      (cannot be given together with Factorbase=l)
		      The default is maxfb=300.
NumberOfPolynomials=k - k: positive integer
		      Sieve the values of k polynomials (default:k=1)
SieveArrayLimit=M     -  M: positive integer
		      For every polynomial f to be considered, find
		      smooth values among the numbers f(-M)...f(M).
		      (Default: M=1000).
Tolerance=T           -  T: positive real number 
		      Allow numbers to pass the first step of the
 		      sieve if the non-smooth part of them does probably
		      not exceed maxfb^T, where maxfb is the maximum 
		      element of the factorbase. 
		      (Default: T=2).
LargeFactorBound=B    -   B: positive integer
		      After factoring the values that have passed the
		      first step of the sieve, rule out those which 
		      have an irreducible factor exceeding B.
		      (Default: B=maxfb^T).
InteractiveInput      
   		      Enter the above options interactively.
ExtendedInformation
		      Collect information about the course of the 
		      algorithm, and return them as a list.

++# 

alias(FBSIZE=300):  # default size of maximum prime in factorbase #
alias(PRECMULT=100): # use logarithms with precision only two decimal digits #
alias(ARRSIZE=1000): # default M for sieve array #
alias(NPOLYS=1):  # default number of polynomials used #


numlib::selectodd:=fun
# given a pair (a,b) , return a if b is odd, and "nothing" otherwise # 
(
	(
		if args(2) mod 2 = 1 then
			args(1)
		else
			null()
		end_if
	)
):




numlib::factorbase:=proc(n,size)
# given n, select those primes <> 2 for which n is a quadratic residue #
# 2 is handled separately by the mpqs-algorithm #
begin
select(select([$3..size],isprime),fun(numlib::jacobi(n,args(1))=1))
end_proc:

numlib::sievestep:=proc(n,M,npolys,fb,fbprod,bound,largefactorbound)
local alist,blist,clist,dlist,h1list,h2list,d,st;

begin
# create list of d's #
d:=4*(floor(sqrt(sqrt(n)/(M*sqrt(2)))) div 4) + 3;
userinfo(4,"Initial d chosen:".expr2text(d));
st:=time();
dlist:=[];
while nops(dlist) < npolys do
	if isprime(d) and numlib::jacobi(d,n)=1 then
		dlist:=append(dlist,d);
	end_if;
	d:=d+4;
end_while;	
userinfo(1,"Time for creating list of d's :".expr2text(time()-st));
st:=time();
alist:=map(dlist,fun(args(1)^2 mod n));
h1list:=map(dlist,fun(powermod(n,(args(1)+1) div 4,args(1))));
h2list:=zip(dlist,h1list,fun(((n-args(2)^2)/args(1))/(2*args(2)) mod args(1)));
blist:=zip(zip(h1list,zip(h2list,dlist,_mult),_plus),alist,irem);
blist:=zip(blist,alist,fun(
	(if args(1) mod 2=1 then args(1) else args(2)-args(1) end_if) ) );
clist:=zip(blist,alist,fun((args(1)^2-n)/(4*args(2))));
userinfo(1,"Time for creating the lists of a,b,c:".expr2text(time()-st));
_concat(op(_concat(map([$1..nops(dlist)],fun(numlib::sievepoly(
	n,M,fb,fbprod,bound,largefactorbound,
        alist[args(1)],blist[args(1)],clist[args(1)],dlist[args(1)]))))))
end_proc:

numlib::sievepoly:=proc(n,M,fb,fbprod,bound,largefactorbound,a,b,c,d)
local p,f,r,st,larray,result,
# private # logp,q, rr, mask, offset;

# sieves with f=ax^2+bx+c #
# returns a list of sieve reports, each of the form
  [f(r),u,smallf, largef,H] where H^2 \equiv f(r) mod n 
   and f(r)=u*smallf*largef #
# smallf is the product of primes in the factorbase #

begin
st:=time();
f:=poly(a*r^2+b*r+c,[hold(r)]);
# as c=f(0) indicates whether f(0) is even or odd, initialize array #
if c mod 2=1 then
        larray:=[(0,round(PRECMULT*float(ln(2)))) $ M].[0];
elif (a+b+c) mod 2 = 1 then # f(1) is odd #
        larray:=[(round(PRECMULT*float(ln(2))),0) $M].
                [round(PRECMULT*float(ln(2)))]
else
	larray:=[ round(PRECMULT*float(ln(2))) $2*M+1 ];
end_if;


# now, larray[i] contains the factorization of i-(M+1) #
# and the factorization of i can be found in larray[i+M+1] #
# since c is always even, zero is always a root of f for p=2 #
for p in fb do #parallel#
	# private logp,rr,q, mask, offset; #

        logp:=round(PRECMULT*float(ln(p)));
        # solve f(rr)=0 mod p #
        if a mod p = 0 then
        # now b mod p \neq 0, because p does not divide n #
                rr:=c/b mod p 
        else
                q:=op(numlib::msqrts(n,p),1);
                rr:=[(q-b)/(2*a) mod p,(-q-b)/(2*a) mod p];
               
        end_if;
	# create mask #
	# mask corresponds to [ 0 mod p ... p-1 mod p ] #
	mask := [0 $ p];
	offset:= M mod p; 
	# offset: the last element not covered by the mask #

	if domtype(rr)=DOM_LIST then
	    # polynomial has two roots modulo p #
	      mask[rr[1]+1]:=logp;
	      mask[rr[2]+1]:=logp;
	      # handle left part #
              if p-rr[1] <= offset  then
	        larray[offset-p+rr[1]+1]:=larray[offset-p+rr[1]+1] + logp
	      end_if;
	      if p-rr[2] <= offset then
		larray[offset-p+rr[2]+1]:=larray[offset-p+rr[2]+1] + logp 
	      end_if		      
	else
	    # only one root #
	      mask[rr+1]:=logp;
	      if p-rr <= offset then
		larray[offset-p+rr+1]:= larray[offset-p+rr+1] + logp
	      end_if;
	end_if;

	# add mask to array #

        larray:=zip(larray, [0 $offset]._concat(mask $((2*M+1) div p)+1),_plus);
end_for;
userinfo(1,"Time for sieving ".expr2text(f).":".expr2text(time()-st));
# find out which numbers -M..M have small factors #
st:=time();
result:=map(select([$1..2*M+1],fun(larray[args(1)]>= bound)),
	proc(x)
		local largef,ggcd,signum,smoothpart,H;
	
		begin
		H:=(2*a*(x-(M+1))+b)/(2*d) mod n;
		x:=evalp(f,r=x-(M+1));
		signum:=sign(x);
		largef:=abs(x);
		smoothpart:=1;
		while (ggcd:=igcd(largef,fbprod))<> 1 do
			largef:=largef/ggcd;
			smoothpart:=smoothpart*ggcd
		end_while;	
		 [x,signum,smoothpart,largef,H]
	 	end_proc
	 );
# now result is a list of entries of the form [x,signum,smoothpart,largef,H], 
where signum is the sign, smoothpart is the product of factors over 
the factorbase, largef
  is the non-smooth part of x, and H is a square root of x modulo n #
# then x=signum*smoothpart*value holds #
userinfo(1,"Time for trial divisions:".expr2text(time()-st));
userinfo(3,"The polynomial ".expr2text(f) ." generated ".
	 expr2text(nops(result)). " values conjectured to be smooth");
result:=select(result,fun(op(args(1),4) < largefactorbound ));
userinfo(3,"The polynomial ".expr2text(f) ." generated ".
	 expr2text(nops(result))." sieve reports");
userinfo(5,"Sieve reports:");
for r in result do 
	userinfo(5,expr2text(op(r,1))."=".expr2text(op(r,2))."*".
		expr2text(op(r,3))."*".expr2text(op(r,4))." ; H = ".
		expr2text(op(r,5)));
end_for;	
result
end_proc:

numlib::mpqs:=proc(n)
local i,r,optionalargs,fb,l,fbprod,npolys,M,bound,oldn,largefactorbound,T,
factorizations,H,multiplicities,j,change,globalchange,indx,HH,ll,st,
multiplier,extendedinformation, result;

begin
n:=abs(n);
extendedinformation:=FALSE; # unless otherwise set #
if type(n)<>DOM_INT then
	error("First argument must be integer (number to be factored)");
end_if;
if isprime(n) or n=1 or n=0 then
	return(n);
end_if;


# select multiplier #
oldn:=n;
multiplier:=1/n mod 8;
n:=n*multiplier; 
userinfo(2,"Select multiplier ".expr2text(multiplier).", factoring ".expr2text(n));
result:=["Multiplier :".expr2text(multiplier)];

if args(0)>1 then
	optionalargs:=[args()]; 
	unassign(optionalargs[1]);
	map(optionalargs,proc(x)
		begin
		if type(x) = "_equal" then
			case op(x,1)
			of hold(Factorbase) do							                           fb:=op(x,2); break
			of hold(MaxInFactorbase) do
	  		   if fb<>hold(fb) then
				error("Options Factorbase and MaxInFactorbase".
					"cannot be given together");
			   end_if;
			   fb:=numlib::factorbase(n,op(x,2));
			   userinfo(4,"Factorbase chosen:".expr2text(fb));
			   break
			of hold(LargeFactorBound) do
			   largefactorbound:= op(x,2); break
			of hold(NumberOfPolynomials) do
			   npolys:=op(x,2); break
			of hold(SieveArrayLimit) do
			   M:=op(x,2); break
			of hold(Tolerance) do
			   T:=op(x,2); break
			otherwise
			   error("Unknown option ".expr2text(op(x,1)))
			end_case;
		else
			case x
			   of hold(InteractiveInput) do 
				input("Enter limit for size of primes in \
factorbase",fb);
				fb:=numlib::factorbase(n,fb);
				userinfo(4,"Factorbase chosen:".expr2text(fb));				
				print(Unquoted,
			 "Sieving will be done on some interval [-M,M] .");
				input("Enter value for M", M);
				input("Enter number of polynomials to be used",
					npolys);
				input("Enter sieve tolerance", T);
				input("Enter bound for maximum size of really \
encountered large factor", largefactorbound);
				break;
			    of hold(ExtendedInformation) do
				extendedinformation:=TRUE;
				break;				
			otherwise return("Unknown option ".expr2text(x))
			end_case;
		end_if;		
		end_proc			   
	);  # end of map #
end_if;
			   


# use defaults as far as options were not given #
# generate factorbase #
if fb=hold(fb) then
	fb:=numlib::factorbase(n,FBSIZE);
	userinfo(4, "Factorbase chosen: ". expr2text(fb))
end_if;
fbprod:=_mult(2,multiplier,op(fb)); # in any case #

# select number of polynomials : default: 1 #

if npolys=hold(npolys) then
	npolys:=NPOLYS;
end_if;

if T=hold(T) then
	T:=2;
end_if;

if largefactorbound=hold(largefactorbound) then
	largefactorbound:=fb[nops(fb)]^T;
	userinfo(4, "Bound for large factors: ".expr2text(largefactorbound))
end_if;

if M=hold(M) then 
	M:= ARRSIZE;
	userinfo(4, "Size of sieve array: ".expr2text(M))
end_if;

bound:=round((float(ln(M*sqrt(n/2)))-float(T*ln(fb[nops(fb)])))*PRECMULT);
userinfo(4, "Bound for passing first step of the sieve: ".expr2text(bound));
userinfo(4, "Bound corresponds to a smooth part of ".
	    expr2text(float(exp(bound/PRECMULT))));


if extendedinformation then
	result:=result.["Size of factorbase:".expr2text(nops(fb))];
end_if;


st:=time();
l:=numlib::sievestep(n,M,npolys,fb,fbprod,bound,largefactorbound);
userinfo(1,"Total time for sieve-step: ".expr2text(time()-st));
userinfo(1,"Total number of sieve reports: ".expr2text( nops(l)));
if extendedinformation then 
	result:=append(result," Sieve Reports: ".expr2text(nops(l)));
end_if;

# now, eliminate sieve reports which are not usable because of their
  large factors # 

st:=time();

factorizations:=map(l,op,4);
# extract large factors #
factorizations:=map(factorizations,fun
		(
			(if args(1)=1 then
				[]
			 elif args(1)< op(fb,nops(fb))^2 then
				[args(1)]
			 elif isprime(args(1)) then
				[args(1)]
			 else
				ifactor(args(1));
				zip([op(%,2*j) $hold(j)=1..nops(%) div 2],
				    [op(%,2*j+1) $hold(j)=1..nops(%) div 2],
				    numlib::selectodd)
			 end_if
			 )
		) );

multiplicities:=Dom::Multiset(op(map(factorizations,op)));
userinfo(5,"Large factors detected:".expr2text(multiplicities));

repeat
	globalchange:=FALSE;
	i:=1;
	while i<=nops(factorizations) do
	   change:=FALSE;	
	   for j in factorizations[i] do
		indx:=contains(multiplicities,j);
		if change=FALSE then	
		   if op(multiplicities,[indx,2])=1 then
			# only one occurence #
			# throw away the sieve report #
			unassign(factorizations[i]);
			unassign(l[i]);	
			globalchange:=TRUE;			
			change:=TRUE;	
		    end_if;
		end_if;
	        
		if change then 
		    # correct the multiset #	
		    multiplicities:=subsop(multiplicities,[indx,2]=
					op(multiplicities,[indx,2])-1);	
		end_if;
	   end_for;
	   if change=FALSE then i:=i+1; end_if;	
	end_while
until not globalchange end_repeat;

multiplicities:=select(multiplicities,fun(not iszero(op(args(1),2))));

userinfo(1,"Time for handling large factors:".expr2text(time()-st));
userinfo(3,"Multiple large factors left after elimination: "
	.expr2text(multiplicities));

# there are two ways how one could proceed : 
   a) Cycle detection
   b) simply adding all large factors to the factorbase
   Here, the second alternative is chosen
#

if multiplier <> 1 then
	fb:=[2,multiplier].fb.sort(map([op(multiplicities)],op,1))
else
	fb:=[2].fb.sort(map([op(multiplicities)],op,1));
end_if;

userinfo(4, "Size of extended factorbase: ".expr2text(nops(fb)));
userinfo(4, "Number of useful sieve reports: ".expr2text(nops(l)));

if extendedinformation then
	result:=append(result,"Size of extended factorbase:".expr2text(nops(fb)));
	result:=append(result,"Number of useful sieve reports:".expr2text(nops(l)));
end_if;

userinfo(2,"Handling negative sieve reports");

H:=map(l,op,5);
r:=select([$1..nops(l)],fun(op(l,[args(1),2])=-1));
l:=map(l,op,1);
# get positive numbers #
if r<>[] then
	i:=r[1];
	HH:=H[i];
	ll:=l[i];
	map(r,fun((
			H[args(1)]:=H[args(1)]*HH mod n; 
			l[args(1)]:=l[args(1)]*ll
		)) );
	unassign(l[i]);
	unassign(H[i]);
end_if;

userinfo(2,"Creating exponent vectors");
factorizations:=map(l,ifactor);
ll:=[ [0 $nops(fb)] $nops(l) ];
# list of exponent vectors # 
# ll[i][j] is the exponent of fb[j] in the i-th sieve report #

for i from 1 to nops(ll) parallel
	private r,j;
	
	r:=factorizations[i];
	for j from 2 to nops(r) step 2 parallel
		if contains(fb,r[j])=0 then
			# should not happen #
			userinfo(1,"Warning : previously undetected factor");
			fb:=append(fb,r[j]);
		end_if;
		ll[i][contains(fb,r[j])]:=r[j+1];
	end_for;
end_for;

userinfo(2,"Solving linear system");
st:=time();		
for i from 1 to nops(fb) do
	userinfo(5,"Working on ".expr2text(i)."th column");
	r:=select([$1..nops(ll)],fun(ll[args(1)][i] mod 2 = 1));
	if r<>[] then
		indx:=r[1];
		HH:=ll[indx];
		j:=H[indx];
		map(r,fun((
			ll[args(1)]:=zip(ll[args(1)],HH,_plus);
			H[args(1)]:=H[args(1)]*j mod n;
		    )) );
		unassign(H[indx]);
		unassign(ll[indx]);
	end_if;
	# now, there are only even entries in the i-th column #
end_for;

#loop invariant: \prod_i fb[i]^ll[k][i] \equiv H[k]^2 mod n for any k # 

#now, there are only even entries in the exponent vectors #
		

userinfo(1,"Time for solving linear system:".expr2text(time()-st));
userinfo(2,"Number of solutions: ".expr2text(nops(ll)));
if extendedinformation then
	result:=append(result,"Number of Solutions:".expr2text(nops(ll)));
end_if;

if ll=[] then
	if extendedinformation then
		return(append(result,"No factor found"));
	else 
		return(FAIL) 
	end_if;
end_if;

for i from 1 to nops(ll) do
	l:=zip(ll[i],fb,fun(powermod(args(2),args(1) div 2,n)));
	# now l is a list of appropriate prime powers #
	HH:=1;
	for j from 1 to nops(l) do
		HH:=HH*l[j] mod n;
	end_for;
	# now l^2 \equiv H[i]^2 \bmod n #
	r:=igcd(HH-H[i],oldn);
	if r=1 or r=oldn then
		userinfo(2,"Solution number ".expr2text(i)." unusable")
	elif extendedinformation then		
		return(append(result," Solution: ".expr2text(r)));
	else
		return(r)	
	end_if;

end_for;
if extendedinformation then
        return(append(result,"No factor found"));
else
        return(FAIL)
end_if;

end_proc:


unalias(FBSIZE):
unalias(PRECMULT):
unalias(ARRSIZE):
unalias(NPOLYS):
