#  ode::autonomous(eq,y,x,n) solves the autonomous equation 
   eq=0 of order n in y(x),
   using the algorithm described in Zwillinger, section 36 page 153.

   Output: FAIL or a list of solutions for y(x).

   Example: 
   >> ode::autonomous(diff(y(x),x,x)-diff(y(x),x)-2*y(x)*diff(y(x),x),y,x,2);
#
ode::autonomous := proc(eq,_0_y,_0_x,n) local u,i,solu,soly,s;
begin
   userinfo(2,"trying to recognize an autonomous equation");
   u:=genident("u");
   eq:=subs(eq,diff(_0_y(_0_x),_0_x$i)=ode::auto_transform(_0_y,u,i)$i=1..n,_0_y(_0_x)=_0_y);
   if has(eq,_0_x) then FAIL 
   else 
      userinfo(1,"autonomous equation");
      sysassign(ode::depth,ode::depth+1);
      solu:=ode::solve_eq(eq,u,_0_y);
      sysassign(ode::depth,ode::depth-1);
      if type(solu)="solve" or has(solu,FAIL) then
         userinfo(1,"autonomous method failed");
         return(FAIL)
      end_if;
      userinfo(2,"solutions for u(y) are",solu);
      # now each solution corresponds to u that is y'(x) #
      s:=subs(eq,u(_0_y)=0);
      # if s=0, then u(y) is a factor of eq, therefore u=0 (y=C) is already in solu #
      if s=0 then soly:={} 
      else soly:=solve(s,_0_y,(if has(s,_0_x) then ode::maxdegree else 2 end_if),
		PrincipalValue) end_if;
      if type(solu)="_union" then solu:=map(solu,fun((if type(args(1))=DOM_SET
	then args(1) else {args(1)} end_if))) end_if;
      for u in solu do
         if u=0 then s:={genident("C")} # y'=0 #
         elif u=FAIL then s:={}
         else
            userinfo(2,"integrating",1/u);
            # level needed for u=(y^2 + 1)^(3/2) for instance #
            s:=int(1/u,_0_y); s:=level(s,2);
            s:=solve(s=_0_x+genident("C"),_0_y,ode::maxdegree,PrincipalValue);
            userinfo(2,"solution is",s);
         end_if;
         soly:=soly union s
      end_for;
      userinfo(1,"autonomous method worked");
      soly
   end_if
end_proc:

# gives the value of diff(y(x),x$n) in terms of u(y)=diff(y(x),x) #
ode::auto_transform:=proc(_0_y,u,n) local e;
begin
   e:=u(_0_y);
   while n>1 do e:=expand(diff(e,_0_y)*u(_0_y)); n:=n-1 end_while;
   e
end_proc:
