#  Input: eq is a first-order ODE in y(x).
   Output: a solution or FAIL.
   Example: ode::Clairaut((x*diff(y(x),x)-y(x))^2-diff(y(x),x)^2-1,y,x); 
   Reference: Zwillinger chap. 38 page 158
#
ode::Clairaut := proc(eq,y,x) local yp,Z,f;
begin
   eq:=expand(subs(eq,diff(y(x),x)=yp,y(x)=x*yp-Z));
   if not has(eq,{x,y}) and type(eq)="_plus" then
      f:=select(eq,has,Z);
      if not has(f,yp) then
         userinfo(1,"Clairaut's equation");
         return(ode::clairaut(subs(f,Z=x),subs(f-eq,yp=x),x))
      end_if
   end_if;
   FAIL
end_proc:

# solves f(x*y'-y) = g(y').
   A general solution is given by f(xC-y)=g(C) where C is a constant.
   Example: ode::clairaut(x^2,x^2-1,x);
#
ode::clairaut := proc(_f_,_g_,x) # f and g are expressions in x #
local C,gensol,y,eq,yp,partsol,lev;
begin
   userinfo(3,"f=",_f_,"g=",_g_);
   # search for a general solution #
   C:=genident("C");
   lev:=max(2-ode::depth,ode::maxdegree);
   gensol:=solve(subs(_f_,x=x*C-y)=subs(_g_,x=C),y,lev);
   userinfo(3,"general solution is",gensol);
   # search for a singular solution #
   eq:=subs(diff(_f_,x),x=x*yp-y)*x-subs(diff(_g_,x),x=yp);
   partsol:=solve(eq,yp,ode::maxdegree);
   userinfo(3,"singular solution for y' is",partsol);
   if type(partsol)="solve" then partsol:={} end_if;
   if partsol<>{} then
      partsol:=op(partsol,1);
      eq:=numer(subs(_f_,x=x*partsol-y)-subs(_g_,x=partsol));
      partsol:=solve(eq,y,lev);
      userinfo(3,"particular solution is",partsol);
   end_if;
   gensol union partsol
end_proc:
