# ode::equidimensional_in_y(eq,y,x,n) solves the nth order ODE eq=0 for y(x)
  if it is equidimensional in y (cf Zwillinger chap. 48 p. 191)
  Input: eq is an expression, y,x are variables, n is the order
  Output: FAIL or a list of solutions

  Example: ode::equidimensional_in_y((1-x)*(y(x)*diff(y(x),x,x)-diff(y(x),x)^2)+x^2*y(x)^2,y,x,2);
#

ode::equidimensional_in_y := proc(eq,_y,_x,n) local a,u;
begin
   # first substitute y by a*Y #
   # if has(normal(eval(subs(eq,_y(_x)=a*_y(_x)))/eq),{_y,_x}) then #
   if has(normal(subs(eq,_y(_x)=a*_y(_x),Simplified)/eq),{_y,_x}) then 
      FAIL
   else
      userinfo(1,"equidimensional in y equation");
      # eq:=eval(subs(eq,_y(_x)=exp(u(_x)))); #
      eq:=subs(eq,_y(_x)=exp(u(_x)),Simplified);
      userinfo(2,"new equation is",eq);
      sysassign(ode::depth,ode::depth+1);
      eq:=ode::solve_eq(expand(eq),u,_x,{});
      sysassign(ode::depth,ode::depth-1);
      if contains(eq,FAIL) then FAIL else map(eq,exp) end_if
   end_if;
end_proc:

