# ode::int_factors(eq,y,x) tries to solve the ODE eq with respect to y(x)
   using the method of integrating factors (see Zwillinger's book page 259).
   Currently this works only for first order equations.

   Output: either FAIL or a set of solutions.

   Examples:
   sysassign(ode::int_factors,
	loadproc(ode::int_factors,pathname("ODE"),"intfacts")):
   ode::int_factors(diff(y(x),x)+y(x)/x-x^2,y,x);

                               {         3 }
                               {   C3   x  }
                               { - -- + -- }
                               {   x    4  }

   ode::int_factors(y(x)*(y(x)^2-x)+x^2*diff(y(x),x),y,x);

             {  1/2   3       2 1/2     1/2   3       2 1/2 }
             { 2    (x  + C4 x )       2    (x  + C4 x )    }
             { --------------------, - -------------------- }
             {      2 C4 + 2 x              2 C4 + 2 x      }
#

ode::int_factors := proc(eq,y,x) local yp,N,M,f,g,u;
begin
   userinfo(2,"trying to find an integrating factor");
   eq:=subs(eq,diff(y(x),x)=yp,y(x)=y);
   if testtype(eq,Type::PolyExpr(yp)) then
      eq:=poly(eq,[yp]);
      if degree(eq)=1 then
         M:=coeff(eq,0); N:=coeff(eq,1); # M*dx+N*dy=0 #
         eq:=diff(M,y)-diff(N,x);
         if not has((f:=normal(eq/N)),y) then # case (A) #
            u:=int(f,x)
         elif not has((g:=normal(-eq/M)),x) then # case (B) #
            # there is a typo in Zwillinger for the sign of g #
            u:=subs(int(g,y),y=y(x))
         else u:=FAIL
         end_if;
         if u<>FAIL and type(u)<>"int" then
            u:=expand(exp(u));
            userinfo(1,"found integrating factor",u);
            u:=ode::exact_first_order(u*args(1),y,x);
            if not has(u,FAIL) then
               userinfo(1,"integrating factor method worked");
            end_if;
            return(u);
         end_if
      end_if;
   end_if;
   FAIL
end_proc:
