# ode::lagrange(eq,y,x) tries to solve the first-order ODE eq=0 wrt y(x)
   trying to recognize a Lagrange equation, i.e. an equation of the form

                          y = x*F(y')+G(y')

   If F=1, then we have a Clairaut's equation.

Examples: 
>> ode::lagrange:=loadproc(ode::lagrange,pathname("ODE"),"lagrange"):
>> ode::lagrange(y(x)-2*x*diff(y(x),x)+a*diff(y(x),x)^3,y,x);

              4          4       2   3           2  2  
{RootOf(27 a y  + 64 C1 x  + 64 a  C1  + 128 a C1  x  +

    2                     3      
   y  (- 144 a C1 x - 16 x ), y)}

>> ode::lagrange(y(x)-2*x*diff(y(x),x)+diff(y(x),x)^2,y,x);

                                 2      3          3      2  2     
       {RootOf(- 18 C1 x y + 9 C1  + 4 y  + 12 C1 x  - 3 x  y , y)}
#

ode::lagrange:=proc(eq,y,x) local p,F,G;
begin
   userinfo(2,"trying to recognize a Lagrange equation");
   p:=genident("p"); # to avoid name space pb #
   eq:=subs(eq,diff(y(x),x)=p,y(x)=y);
   if testtype(eq,Type::PolyExpr(y)) then
      if degree((eq:=poly(eq,[y])))=1 then
         eq:=-coeff(eq,0)/coeff(eq,1); # should match x*F(y')+G(y') #
         if testtype(eq,Type::PolyExpr(x)) then
            if degree((eq:=poly(eq,[x])))=1 then
               userinfo(1,"Lagrange equation");
               F:=coeff(eq,1); G:=coeff(eq,0);
               eq:=diff(x(p),p)-x(p)*diff(F,p)/(p-F)-diff(G,p)/(p-F);
               userinfo(2,"new equation is",eq);
               sysassign(ode::depth,ode::depth+1);
               eq:=ode::solve_eq(eq,x,p,{});
               sysassign(ode::depth,ode::depth-1);
               eq:=map(eq,fun(numer(args(1)-x)));
               eq:=map(eq,resultant,y-x*F-G,p);
               # p=y' should not appear any more now #
               if has(eq,{FAIL,p}) then
                  userinfo(1,"Lagrange method failed");
                  return(FAIL) 
               end_if;
               eq:=map(eq,fun(solve(expand(args(1)/content(args(1),[y])),y,
		ode::maxdegree)));
               userinfo(1,"Lagrange method worked");
               return(op(eq))
            end_if
         end_if
      end_if
   end_if;
   FAIL
end_proc:
