# solves a set of linear ode in z
  l is a list [u,v,...] of indeterminates for the functions u(z), v(z)
  A is a table of (ord+1) lists of n lists of n coefficients
  b is a list
  n=nops(l) is the number of unknown functions
  ord=nops(A)-1 is the maximal order of the equations
  nn is the number of wanted solutions (nn=n when ord=1)
  the coefficient of diff(l[j](z),z$k) in equation i is stored in A[k][i][j]
  and the inhomogeneous coefficient of equation i is stored in _b[i]

  Example: ode::linsys(table(0 = [[-1, -8], [-9, -2]], 1 = [[0, 1], [1, 0]]),
                       [-1, 0], {y1, y2}, t, 2, 1); solves the first-order system
           y2'(t)-y1(t)-8*y2(t)=1, y1'(t)-9*y1(t)-2*y2(t)=0 with 
#
ode::linsys := proc(AA,_b,l,z,n,ord,nn)
local k,lc,M,Id,d,invlc,C,eigenvals,lambda,eigenvects,sol,v,i,m,A,j,hsol,
	T,maxord;
begin
   if ord>=2 then
      if AA[0]=[[0$n]$n] then
         sol:=ode::linsys(table(i=AA[i+1]$i=0..ord-1),_b,l,z,n,ord-1,nn);
         sol:=map(sol,fun(op(args(1),1)=int(op(args(1),2),z)+genident("C")));
         return(sol)
      end_if;
      # reduce to a system of smaller order #
      # l[1..n] represent the original functions,
        l[n+1..2*n] their derivatives,
        l[2*n+1..3*n] their second derivatives, ... #
      A:=table(); T:=table(); maxord:=table();
      for j from 1 to n do
         for k from ord downto 1 do
            for i from 1 to n do if AA[k][i][j]<>0 then break end_if end_for;
            if i<=n then break end_if
         end_for;
         maxord[j]:=k; T[j,0]:=j;
         while k>1 do
            k:=k-1; l:=append(l,genident()); T[j,k]:=nops(l)
         end_while
      end_for;
      v:=[0$nops(l)]; 
      A[0]:=[v$nops(l)]; A[1]:=A[0];
      for i from 1 to n do
         # equations (1) to (n) #
         for j from 1 to n do
            A[0][i][j]:=AA[0][i][j]; A[1][i][j]:=AA[1][i][j];
            for k from 2 to maxord[j] do
               A[1][i][T[j,k-1]]:=AA[k][i][j]
            end_for
         end_for;
      end_for;
      i:=n; for j from 1 to n do
         for k from 1 to maxord[j]-1 do
            # diff(y[j],z$k)'=diff(y[j],z$k+1) #
            i:=i+1;
            A[0][i][T[j,k]]:=1; A[1][i][T[j,k-1]]:=-1;
         end_for
      end_for;
      return(ode::linsys(A,[op(_b),0$(nops(l)-n)],l,z,nops(l),1,n))
   end_if;
   # now ord=1 #
   if ord=1 then nn:=n; end_if;
   # first transform to matrices #
   M := (Dom::Matrix)(Dom::ExpressionField(normal));
   A[0]:=M(AA[0]); A[1]:=M(AA[1]);
   # normalize the leading coefficient #
   lc:=A[1];
   Id:=M(n,n,1,Diagonal);
   if lc<>Id then
      if (invlc:=lc^(-1))=FAIL then return(FAIL) end_if;
      A[0]:=invlc * A[0];
   end_if;
   if not has(AA[0],z) then # solve Y' + A[0] * Y = -_b #
      lc:=[]; # will contain the integration constants #
      # find the eigenvalues #
      eigenvects:=linalg::eigenVectors(A[0]);
      if type(eigenvects)<>DOM_LIST then return(FAIL) end_if;
      eigenvals:=map(eigenvects,op,1);
      userinfo(2,"eigenvalues are",eigenvals);
      sol:=M(n,1); k:=table();
      for lambda in eigenvects do
         # expr is needed if one uses ExpressionField(normal) #
         m:=lambda[2]; v:=lambda[3]; lambda:=expr(lambda[1]);
         userinfo(2,"for eigenvalue",lambda);
         userinfo(2,"dimension of eigenspace is",m);
         userinfo(2,"eigenvectors for value",lambda,"are",v);
         if v=[] then return(FAIL) else v:=v[1] end_if;
         k:=exp(-lambda*z);
         for i from 1 to m do
            C:=genident("C"); lc:=append(lc,C);
            sol:=sol+C*z^(i-1)*k*v;
            if i=m then break
            elif i=1 then d:=A[0]-lambda*Id end_if;
            v:=d*v; # see last formula on page 318 of [Zwillinger] #
         end_for;
      end_for;
      if domtype(l)=DOM_SET then l:=[op(l)] end_if;
      hsol:={l[i](z)=expr(sol[i]) $ hold(i)=1..nn};
      if {op(_b)}={0} then {hsol} # homogeneous system #
      else
         lc:=linsolve({expr(_plus(A[1][i,j]*sol[j] $ j=1..n))+_b[i] $ 
		hold(i)=1..n},{op(lc)});
         if nops(lc)=0 or lc=FAIL then
            userinfo(1,"unable to solve for integration constants");
            FAIL
         else
            lc:=map(lc,func(op(eq,1)=op(eq,1)+int(op(eq,2),z),eq));
            lc:=subs(hsol,lc);
	    {map(map(lc,expand),fun(op(args(1),1)=combine(op(args(1),2),exp)))}
         end_if
      end_if
   else FAIL
   end_if
end_proc:
