# ode::solvable_for_x(eq,y,x) tries to solve the first-order ODE eq=0 wrt y(x)
   using the method of "solvable in x" equations (cf Zwillinger pages 304-305).

   Output: either FAIL or a set of solutions.

   Example:
ode::solvable_for_x:=loadproc(ode::solvable_for_x,pathname("ODE"),"solvablx"):
ode::solvable_for_x(2*x*diff(y(x),x)+y(x)*diff(y(x),x)^2-y(x),y,x);

                                       2 1/2              2 1/2 
          {I x, (- I) x, - (2 C1 x + C1 )   , (2 C1 x + C1 )   }
#

ode::solvable_for_x := proc(eq,y,x) local p,xx,sol;
begin
   userinfo(2,"trying to recognize an equation solvable for x");
   p:=genident("p");
   eq:=subs(eq,diff(y(x),x)=p,y(x)=y);
   sol:=solve(eq,x,ode::maxdegree,PrincipalValue);
   if type(sol)="solve" then return(FAIL) end_if;
   sol:=select(sol,not hastype,RootOf);
   if nops(sol)=0 then return(FAIL) end_if;
   userinfo(1,"equation solvable for x");
   xx:=subs(op(sol,1),p=p(y));
   sol:=numer(diff(xx,y)-1/p(y));
   sysassign(ode::depth,ode::depth+1);
   sol:=ode::solve_eq(sol,p,y,{});
   sysassign(ode::depth,ode::depth-1);
   # don't allow nested implicit solutions #
   if type(sol)="solve" or has(sol,FAIL) then 
      userinfo(1,"solvable for x method failed");return(FAIL)
   elif type(sol)=DOM_SET then
      if map(sol,type) intersect {"solve",RootOf} <> {} then
         userinfo(1,"solvable for x method failed");return(FAIL) 
      end_if
   end_if;
   sol:=map(sol,fun(numer(subs(eq,p=args(1)))));
   sol:=map(sol,solve,y);
   userinfo(1,"solvable for x method worked");
   map(sol,fun((if type(args(1))="solve" then args(1) else op(args(1)) end_if)))
end_proc:
