# main function to solve a single differential equation eq=0 in y(z) 
  with initial conditions inits #
ode::solve_eq := proc(eq,_0_y,z,inits)
local _l,i,den;
# global ode::depth; #
begin
   if ode::depth>ode::maxDepth then return({FAIL}) end_if;
   if type(inits)<>DOM_SET then inits:={} end_if;
   if type(eq)="_equal" then eq:=op(eq,1)-op(eq,2) end_if;
   if has(eq,exp) then eq:=combine(eq,exp) end_if; # simplify exp(t)*exp(-t) #
   if iszero(eq) then return({_0_y(z)}) end_if;
   den:=denom(eq); # must not vanish #
   eq:=numer(eq);
   userinfo(2,"trying to factor");
   _l:=maprat(eq,factor); # to avoid problems with gcd #
   userinfo(2,"factorization ".
	(if nops(_l)=3 then "failed" else "worked" end_if));
   _union(ode::solve_eq_irred(_l[2*i],_0_y,z,inits,den) $ i=1..nops(_l) div 2)
end_proc:

if not testtype(ode::maxdegree,Type::PosInt) then
   ode::maxdegree:=1 # maximal degree of polynomial solutions solved #
end_if:

alias(READLIB(method,file)=
   (if domattr(ode,method)=FAIL then
      sysassign(domattr(ode,method), loadproc(domattr(ode,method),pathname("ODE"),file))
    end_if; 
    domattr(ode,method))):

# returns a list of solutions or [] if no solution found #
ode::solve_eq_irred := proc(eq,_0_y,z,inits,den)
local n,s,_l,sol,yp,lhs,_a,i,csts,islin;
begin
   userinfo(2,"solving ODE",eq,"with respect to",_0_y(z));
   if not testtype(eq,Type::ODE(_0_y,z)) then
      error("not an ordinary differential equation in the given variable")
   end_if;
   n:=ode::order(eq,{_0_y});
   if n=-1 then return({}) end_if; # no y(..) #
   userinfo(2,"ordinary differential equation of order",n);
   if n=0 then
      s:=subs(eq,_0_y(z)=yp);
      # solve up to quadratics here, don't use ode::maxdegree #
      sol:=solve(s,yp,2);
   elif (sol:=ode::separate(eq,_0_y,z,n))=FAIL then
      # first try to recognize linear equations #
      _l:=[diff(_0_y(z),z$i)$i=0..n];
      if (_l:=Type::Linear(eq,_l))<>FALSE then islin:=TRUE; sol:=ode::solve_linear(_l,z,n)
      else # non linear #
	 userinfo(2,"non linear ordinary differential equation");
	 sol:=ode::autonomous(eq,_0_y,z,n);
	 # exact n-th order equations #
	 if sol=FAIL then
	    if n=1 then
               sol:=ode::exact_first_order(eq,_0_y,z);
               if sol=FAIL then sol:=READLIB("homogeneous","homogen")(eq,_0_y,z) end_if;
               if sol=FAIL then sol:=READLIB("int_factors","intfacts")(eq,_0_y,z) end_if;
               if sol=FAIL then sol:=READLIB("lagrange","lagrange")(eq,_0_y,z) end_if;
               if sol=FAIL then sol:=READLIB("riccati","riccati")(eq,_0_y,z) end_if;
               if sol=FAIL then sol:=READLIB("solvable_for_x","solvablx")(eq,_0_y,z) end_if;
               # solvable_for_y currently not used because it can return only
                 parametric solutions, which are not yet dealt with #
               # if sol=FAIL then sol:=READLIB("solvable_for_y","solvably")(eq,_0_y,z) end_if; #
	    elif n=2 then sol:=ode::exact_second_order(eq,_0_y,z)
	    end_if
	 end_if;
	 if sol=FAIL then sol:=ode::Bernoulli(eq,_0_y,z) end_if;
	 if sol=FAIL then sol:=ode::Clairaut(eq,_0_y,z) end_if;
	 if sol=FAIL then sol:=ode::differentiation(eq,_0_y,z) end_if;
	 if sol=FAIL and n=1 then sol:=ode::contact(eq,_0_y,z) end_if;
      end_if;
      # currently factoring of diff. operator only implemented for linear eqs #
      if sol=FAIL and islin=TRUE then sol:=READLIB("factor","factor")(eq,_0_y,z,n) end_if;
      if sol=FAIL then sol:=ode::ymissing(eq,_0_y,z,n) end_if;
      if sol=FAIL then sol:=ode::equidimensional_in_y(eq,_0_y,z,n) end_if;
      if sol=FAIL then sol:=ode::equidimensional_in_x(eq,_0_y,z,n) end_if;
      # methods that call other methods #
         # scale_invariant calls equidimensional_in_x #
      if sol=FAIL then sol:=READLIB("scale_invariant","scaleinv")(eq,_0_y,z,n) end_if;
      # put at the end methods that call solve_eq recursively #
      if sol=FAIL then sol:=READLIB("interchange","interch")(eq,_0_y,z,n) end_if;
   end_if;
   userinfo(2,"solution is",sol);
   if sol=FAIL then return({FAIL}) # do not return FAIL because of _concat #
   elif inits={} then return(sol)
   end_if;
   userinfo(1,"trying to fulfill initial conditions");
   userinfo(2,"which are",inits);
   # do not deal with implicit solutions for BVPs #
   if type(sol)="solve" then
   elif type(sol)<>DOM_SET then return({FAIL}) end_if;
   s:={};
   csts:=indets(sol) minus (indets(eq) union {_0_y,z});
   for eq in inits do
      if has(op(eq,2),_0_y) then error("dependent variable not allowed in rhs of initial conditions") end_if;
      lhs:=op(eq,1); i:=0; _a:=op(lhs); 
      lhs:=op(lhs,0); # should be y or D(...D(y)...) #
      case type(lhs)
      of DOM_IDENT do break
      of "D" do repeat lhs:=op(lhs); i:=i+1 until type(lhs)<>"D" end_repeat; break
      otherwise error("invalid initial conditions")
      end_case;
      if lhs<>_0_y then error("invalid initial conditions") end_if;
      s := s union {[i,_a,op(eq,2)]}; # [i,a,b] means (D@@i)(f)(a)=b #
      # if eval(subs(den,z=_a,op(inits)))=0 then #
      if subs(den,z=_a,op(inits),Simplified)=0 then
         userinfo(1,"denominator ".expr2text(den)." vanishes for given initial conditions");
         return({})
      end_if;
   end_for;
   if type(sol)="solve" then
      if op(sol,2)=_0_y and nops(s)=1 and op(s,[1,1])=0 and nops(csts)=1 then
         eq:=op(sol,1); csts:=op(csts,1); s:=op(s,1);
         _a:=solve(subs(eq,z=op(s,2),_0_y=op(s,3)),csts);
         if type(_a)<>"solve" and nops(_a)=1 then
            return(solve(subs(eq,csts=op(_a,1)),_0_y))
         end_if
      end_if;
      return({FAIL})
   else map(sol,ode::solvinit,s,csts,z)
   end_if
end_proc:

# inits is a set of triples [i,a,b] #
ode::solvinit := proc(sol,inits,csts,z)
local s,t,i;
begin
   userinfo(3,"general solution is",sol);
   if type(sol)=RootOf then sol:=subs(sol,extop(sol,2)=extop(sol,2)(z)) end_if;
   userinfo(3,"initial conditions are",inits);
   s:={};
   if traperror((for t in inits do
     # s:=s union {eval(subs(diff(sol,z$t[1]),hold(int)=fun(0),z=t[2]))=t[3]}; #
     s:=s union {subs(diff(sol,z$t[1]),hold(int)=fun(0),z=t[2],Simplified)=t[3]};
   end_for))<>0 then return(FAIL) end_if;
   if hastype(s,RootOf) then return(FAIL) end_if;
   userinfo(3,"system to solve is",s,"with respect to",csts);
   s:=solve(s,csts);
   s:=select(s,fun(contains({DOM_LIST,DOM_SET},type(args(1)))));
   userinfo(3,"solution is",s);
   if type(s)<>DOM_SET then FAIL
   elif nops(s)=0 then null() # no solution #
   else # s is a set of solutions #
      sol:=[subs(sol,op(s,i)) $ hold(i)=1..nops(s)];
      if has(sol,{sin,cos}) then sol:=combine(expand(sol),sincos) end_if;
      op(sol)
   end_if
end_proc:

