# solves the equation l[1]*y(z)+l[2]*diff(y(z),z)+...+l[n+1]*diff(y(z),z$n)+l[n+2]=0 #

ode::secondOrder:=loadproc(ode::secondOrder,pathname("ODE","LIOUVILL"),
	"2ndOrder"):

ode::solve_linear := proc(l,z,n)
local inho,i,k,y,partsol,lc,gensol,yy,Y,eq;
# global ode::depth; #
begin
   if ode::depth>ode::maxDepth then return(FAIL) end_if;
   userinfo(1,"linear ordinary differential equation of order ".n);
   if n=0 then # l[1]*y(z)+l[2]=0 #
      if iszero(l[1]) then return({}) else return({-l[2]/l[1]}) end_if
   end_if;
   userinfo(2,_plus(l[i+1]*diff(Y(z),z$i)$i=0..n)+l[n+2]=0);
   # try first exact_nth_order which deals with the inhomogeneous coeff. #
   y:=ode::exact_nth_order(l,z,n);
   if y<>FAIL then return(y) end_if;
   inho:=l[n+2]; l[n+2]:=NIL; lc:=l[n+1]; # leading coefficient #
   y:=FAIL;
   if not has(l,z) then
      userinfo(1,"with constant coefficients");
      y:=ode::constant(l,z,n)
   else
      if n=1 then y:=[ode::firstord(l[1]*yy(z)+l[2]*diff(yy(z),z),yy,z)]
      else
         if map({op(l)},testtype,Type::PolyExpr(z,Type::Rational))={TRUE} then
            userinfo(1,"with polynomial coefficients");
            # strategy decided in Oberwolfach'97 #
            eq:=_plus(l[i+1]*diff(Y(z),z$i)$i=0..n);
            if inho=0 then
               if n=2 then
                  userinfo(1,"looking for Liouvillian solutions");
                  y:=ode::secondOrder(eq,Y,z,2)
               end_if;
               if y=FAIL or y={} or type(y)="solve" then
		  userinfo(1,"looking for rational solutions");
		  y:=ode::ratsols(eq,Y,z,n);
               end_if
            end_if;
            if nops(y)<n then
	       if inho=0 and y={} then # look for exponential solutions #
		  userinfo(1,"looking for exponential solutions");
		  y:=ode::liouvillianSolutions(eq,Y(z));
	       elif inho=0 then # use reduction method #
                  userinfo(1,"using reduction method");
                  ode::dAlembert;
		  return(ode::reduction(eq,Y(z),op(y,1))) 
	       end_if;
            end_if;
            if nops(y)<n then y:=ode::euler(l,z,n) end_if;
         else
            userinfo(1,"with non constant coefficients");
         end_if;
      end_if;
   end_if;
   userinfo(2,"solution of homogeneous part is",y);
   if y=FAIL then return(ode::adjoint(l,z,inho)) end_if;
   gensol:=_plus(genident("C")*op(y,i)$hold(i)=1..n);
   # a solution of the homogeneous part of the linear DE is C1*y[1]+...+Cn*y[n] #
   # we now find a particular solution of the full equation #
   # using the method described on page 314 of Zwillinger (Variation of parameters) #
   partsol:=ode::variationOfConstants(y, z, inho, n, lc);
   if partsol=FAIL then return(FAIL) end_if;
   partsol:=expand(partsol);
   if has(partsol,{sin,cos}) then
      k:=combine(partsol,sincos);
      if k<>partsol and has(k,hold(int)) then k:=level(k) end_if;
      partsol:=k
   end_if;
   {gensol+partsol}
end_proc:

#
Input: sys is a set of solutions for the homogeneous equation
       z is the independent variable
       inho is the inhomogeneous term
       n is the order of the equation
       lc is the leading coefficient of the equation
Output: FAIL or a particular solution of the inhomogeneous equation
#
ode::variationOfConstants :=
proc(sys, _z_, inho, n, lc)
  local _up, _k, _y, _yy;
begin
   # method described on page 314 of Zwillinger (Variation of parameters) #
   if inho=0 then 0
   else
      _y:=[op(sys)];
      sys:={_plus(diff(_y[_k],_z_$i)*_up[_k] $ _k=1..n) $ hold(i)=0..n-2,
		_plus(diff(_y[_k],_z_$(n-1))*_up[_k] $ _k=1..n)+inho/lc},
            {_up[_k] $ _k=1..n};
      userinfo(2,"system to solve is",sys);
      sys:=linsolve(sys);
      if sys=FAIL then return(FAIL) end_if;
      userinfo(2,"solution is",sys);
      for _k from 1 to n do
         userinfo(2,"integrating",up[_k]);
         _yy[_k]:=int(eval(subs(_up[_k],sys)),_z_);
         userinfo(2,"solution is",yy[_k]);
      end_for;
      _plus(_yy[_k]*_y[_k] $ hold(_k)=1..n);
   end_if;
end_proc:

ode::constant := proc(l,_z_c,n) local char_eq,i,X;
begin
   char_eq:=_plus(l[i+1]*X^i $ i=0..n);
   userinfo(2,"solving characteristic equation",char_eq);
   l:=solve(char_eq,X,hold(Multiple));
   l:=[expand(l)];
   userinfo(2,"solutions are",l);
   if nops(l)<>n then return(FAIL) end_if;
   ode::vspace(l,_z_c);
end_proc:
