#++ ---------------- wronsk.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 13 September 1996
Last Update :  8 November  1996


Functions: 

 - ode::wronskianMatrix([v_i], x, < R >)
    ++ returns the Wronskian matrix of the functions v_i  w.r.t. x.
    ++ When the option R is given, then the specified differential ring will
    ++ be choosed for reprensenting the matrix entries, otherwise 
    ++ Dom::ExpressionField(id, iszero@normal) will be used instead. 
 - ode::Wronskian([v_i], x, < R >)
    ++ returns the Wronskian (determinant) of the functions v_i  w.r.t. x.
 - used Parameter:
    ++ v_i, v     :DOM_EXPR
    ++ x          :DOM_IDENT
    ++ R          :DOM_DOMAIN

>> ode::Wronskian([sqrt(x),1/sqrt(x),y(x)],x);      

                   diff(y(x), x, x)   y(x)   diff(y(x), x)
                 - ---------------- + ---- - -------------
                          x              3         2
                                      4 x         x

>> ode::wronskianMatrix([sqrt(x),1/sqrt(x),y(x)],x);                    

                +-                                      -+
                |     1/2       1                        |
                |    x   ,     ----,         y(x)        |
                |               1/2                      |
                |              x                         |
                |                                        |
                |     1          1                       |
                |   ------,  - ------,   diff(y(x), x)   |
                |      1/2        3/2                    |
                |   2 x        2 x                       |
                |                                        |
                |      1        3                        |
                |  - ------,  ------,  diff(y(x), x, x)  |
                |       3/2      5/2                     |
                |    4 x      4 x                        |
                +-                                      -+


++#

ode::wronskianMatrix:= proc(vis, x, R) #option: R #
  local i, mat;
  begin
    if R::hasProp(Dom::BaseDomain) = FAIL 
      then R := Dom::ExpressionField(id, iszero@normal) 
     elif not(R::hasProp(Cat::CommutativeRing) and
              R::hasProp(Cat::DifferentialRing) and
              iszero( R::characteristic() ) or
              R::hasProp(Dom::ExpressionField(id, iszero@normal))) 
          then error("illegal coefficient ring") end_if;
    mat := [map(vis, R::new)];
    for i from 1 to nops(vis)-1 do
      mat := append(mat, map(mat[i], diff, x)) 
    end_for;
    (Dom::Matrix(R))::newThis(mat)
  end_proc:

ode::Wronskian:=fun( expr(linalg::det(ode::wronskianMatrix( args() ))) ):

