# Input: a n-th order ODE in y(x).
   Output: a solution or FAIL.
   Uses the dependent variable missing method, cf Zwillinger chap. 43 p. 175
   Example: ode::ymissing(diff(y(x),x,x)+diff(y(x),x)-x,y,x,2);
#
ode::ymissing := proc(eq,y,x,n) local yp,i,sol,C,lord;
begin
   eq:=subs(eq,diff(y(x),x$i)=yp[i] $ i=1..n);
   if has(eq,y(x)) then FAIL
   else
      userinfo(1,"using dependent variable missing method");
      lord:=1; while not has(eq,yp[lord]) do lord:=lord+1 end_while;
      # lord is the lowest derivative order in y(x) #
      eq:=subs(eq,yp[i]=diff(y(x),x$(i-lord)) $ i=lord..n);
      userinfo(2,"solving new equation",eq);
      sysassign(ode::depth,ode::depth+1);
      sol:=ode::solve_eq(eq,y,x);
      sysassign(ode::depth,ode::depth-1);
      if has(sol,FAIL) then return(FAIL) end_if;
      for i from 1 to lord do
         C:=genident("C");
         sol:=ode::mapsol(sol,proc(t) begin int(t,x)+C end_proc);
         sol:=level(sol,2);
      end_for
   end_if
end_proc:

