# $Date: 1996/06/27 08:33:10 $  $Author: snoo $  $Revision: 1.3 $ #

#++
chebyshev1(n, x) - generates the nth Chebyshev polynomial (first kind)
++#

#--
Chebyshev1(n, x) - recurrence relation for Chebyshev polynomials
--#

#--
computing Chebyshev polynomial T(n,x) by using the recurrence relation
T(0,x) = 1
T(1,x) = x
T(n,x) = 2 * x * T(n-1,x) - T(n-2,x) for n > 1
--#
orthpoly::chebyshev1 := proc(n,x)
begin
	if args(0) <> 2 
                then
                        error("wrong number of arguments!"); 
        end_if;

	if not (domtype(n) = DOM_INT)
		then 
			error("1st argument must be integer!");
	end_if;

	if n < 0 
		then
			error("1st argument must be positive!");
	end_if;
	
	if domtype(x) = DOM_IDENT then
			orthpoly::Chebyshev1(n,x)
		else
                        orthpoly::Chebyshev1Compact(n,x)
	end_if;
end_proc:

#--
Computing the Chebyshev polynomial by an iteration given in
Wolfram Koepf, "Efficient Computation of Orthogonal Polynomials in
Computer Algebra", Preprint SC 95-42, Dec. 1995, ZIB, page 10
--#
orthpoly::Chebyshev1 := 
proc(n, x)
   local k, tmp, result;
begin
   if n=0 then
      return(poly(1,[x]))
   elif n=1 then
      return(poly(x, [x]))
   end_if ;

   tmp := poly(2^(n-1)*x^n, [x]) ;
   result := tmp ;
   for k from 1 to n div 2 do
      tmp := mapcoeffs(tmp, _mult, -1/4/k*(n-2*k+2)*(n-2*k+1)/(n-k)) ;
      tmp := divide(tmp, poly(x^2, [x]), Quo) ;
      result := result+tmp ;
   end_for ;
   result
end_proc:

#--
Computing the Chebyshev polynomial, represented in a not distributed way,
by the recursion 2Tn(x)Tm(x) = Tn+m(x)+Tn-m(x) (n>m).
See Wolfram Koepf, "Efficient Computation of Orthogonal Polynomials in 
Computer Algebra", Preprint SC 95-42, Dec. 1995, ZIB, page 11
--#
orthpoly::Chebyshev1Compact :=
proc(n, x)
   option remember;
begin
   if n=0 then
      1  
   elif n=1 then
      x  
   elif n mod 2 = 0 then
      2*orthpoly::Chebyshev1Compact(n/2, x)^2-1
   else
      2*orthpoly::Chebyshev1Compact((n-1)/2,x)*orthpoly::Chebyshev1Compact((n+1)/2,x)-x
   end_if
end_proc:
