/*
Eingabe: Argumente gemaess plot2d
Ausgabe: Modifizierte Argumente (Clipping gemaess ViewingBox)
         konform zur Spezifikation von plot2d
*/

plotlib::clip2d := proc()
  local clipping, i, l, l1, n, na, new_args, 
        new_list, obj_nr, p, q, t, vbox;
begin

  new_args := [args()]; na := new_args; clipping := FALSE;

  t := select(new_args, has, ViewingBox);
  if t <> [] and 
     type(t[1]) = "_equal" and op(t[1], 2) <> Automatic then
     // t = [ViewingBox=lhs]
     t := float(op(t[1], 2)); // t = float(lhs)
     if testtype(t, Type::ListOf(Type::RealNum,4,4)) then
        // ViewingBox = [minx, maxx, miny, maxy]
        // vbox = [minx, miny, maxx, maxy]
        if t[1] >= t[2] or t[3] >= t[4] then
           error("ViewingBox must describe a rectangle.")
        end_if;
        vbox := [t[1], t[3], t[2], t[4]];
        clipping := TRUE; 
     else
       error("ViewingBox: List of 4 real numbers expected.")
     end_if
  end_if;
 
  if hastype(new_args, DOM_POLYGON) then

     // Es wird nach geschlossenen und/oder gefuellten
     // Polygonen gesucht, die aus mehr als 3 Punkten
     // bestehen. Diese koennen durch die grundlegende
     // plot-Routine nicht behandelt werden. Um dies
     // jedoch in plot2d zu ermoeglichen, werden solche
     // Polygonzuege mittels plotlib::polygonplot
     // triangularisiert, d.h. in mehrere Polygone mit
     // jeweils nur 3 Punkten aufgeteilt.
     
     na := [];

     for l in new_args do
         if type(l) = DOM_LIST and type(l[1]) = "_equal" and
            op(l[1], 1) = Mode and op(l[1], 2) = List then
            // [Mode=List, l2], l2 = [...]
            new_list := [];
            for p in l[2] do
                if domtype(p) = DOM_POLYGON and 
                   (n := nops(p)) > 6       and 
                   (op(p, n-1) or op(p, n)) then
                   // Gefuelltes oder geschlossenes Polygon 
                   // bestehend aus mehr als 3 Punkten
                   q := (plotlib::polygonplot)::PlotScene([p]);
                   // q = [Mode=List, [...]]
                   new_list := append(new_list, op(q[2]));
                else
                   new_list := append(new_list, p);
                end_if;
            end_for;
            na := append(na, [Mode=List, new_list, l[i]$i=3..nops(l)]);
         else
            na := append(na, l);
         end_if
     end_for
  end_if;

 if not clipping then return(op(na)) end_if;

 obj_nr := 0; new_args := [];

 for l in na do

     if type(l) = DOM_LIST and type(l[1]) = "_equal" and
        op(l[1], 1) = Mode then 

        case op(l[1], 2)

          of Curve do
             # [ Mode = Curve, [fx(t), fy(t)], t=[a,b], ... ] #
             obj_nr := obj_nr + 1;
             l1 := plotlib::clip2d_Curve(l, vbox, obj_nr);
             new_args := append(new_args, l1);
             break;
        
          of List do
             # [ Mode = List, [... ] ] #
             l1 := plotlib::clip2d_List(l, vbox);
             new_args := append(new_args, l1);
             break;

          otherwise
             error("Unknown Mode: Curve or List expected.");

         end_case;

     elif type(l) = "_equal" then

         case op(l, 1) 

          of ViewingBox do
             break;

          of Ticks do
             l1 := plotlib::check_ticks(op(l,2), vbox);
             new_args := append(new_args, l1);
             break;

          otherwise
             new_args := append(new_args, l)
        
         end_case 

     else

        new_args := append(new_args, l)

     end_if

   end_for;

   if select(new_args, has, Axes) = [] then
      // Default: Axes = Corner
      new_args := [Axes=Corner] . new_args
   end_if;

   if select(new_args, has, Ticks) = [] then
      // Default: Ticks = 10
      new_args := [plotlib::check_ticks(10, vbox)] . new_args
   end_if;

   return(op(new_args))

end_proc:


plotlib::clip2d_Curve := proc(l      : DOM_LIST, 
                              vbox   : DOM_LIST, 
                              obj_nr : DOM_INT )

  local a, b, clip_1, clip_2, color, convf, fx, fy, 
        grid, iv, j, k, l1, maxx, maxy, minx, miny, 
        r, smooth, t;
begin

  clip_1 := proc(f, xs, xe, grid, miny, maxy)
   local bisection, d, mx, my, r, x, xl, y;
  begin

   bisection := proc(f, x0, y0, x1, y1, bound)
      local c, eps;
   begin
      if y0 < bound then
         c := x0; x0 := x1; x1 := c;
      end_if; 
      eps := 10^(-DIGITS);
      // y0 = f(x0) > bound, y1 = f(x1) < bound
      while abs(x1-x0) > eps do
         c := (x0+x1)/2;
         if float(f(c)) > bound then
            x0 := c;
         else
            x1 := c;
         end_if
      end_while;
      x0;
   end_proc;

   my := miny; r := []; d := float((xe-xs)/grid);
   xs := float(xs); xe := float(xe);

   for x from xs to xe step d do
       y := float(f(x));
       if y > maxy then
          if type(xl) <> DOM_IDENT then
             r := append(r, [ xl, bisection(f, mx, my, x, y, maxy) ]);
             unassign(xl)
          end_if
       elif y < miny then
          if type(xl) <> DOM_IDENT then
             r := append(r, [ xl, bisection(f, mx, my, x, y, miny) ]);
             unassign(xl)
          end_if
       elif type(xl) <> DOM_IDENT then
          if my > maxy then
             r := append(r, [ xl, bisection(f, mx, my, x, y, maxy) ]);
             unassign(xl)
          elif my < miny then
             r := append(r, [ xl, bisection(f, mx, my, x, y, miny) ]);
             unassign(xl)
          end_if
       elif my > maxy then
          xl := bisection(f, mx, my, x, y, maxy)
       elif my < miny then
          xl := bisection(f, mx, my, x, y, miny)
       else
          xl := x
       end_if;
       mx := x; my := y;
  end_for;
  if type(xl) <> DOM_IDENT then
     r := append(r, [xl, mx])
  end_if;
  r 
 end_proc:


 clip_2 := proc(fx, fy, a, b, grid, minx, maxx, miny, maxy)
     local iv, l, r;
 begin
     l := clip_1(fx, a, b, grid, minx, maxx);
     if l = [] then return(l) end_if;
     r := [];
     for iv in l do
         r := r . clip_1(fy, iv[1], iv[2], grid, miny, maxy)
     end_for;
     return(r)
 end_proc:

 convf := proc(f, x) 
   local _f_ ;
 begin
   if not contains({DOM_PROC, DOM_EXEC, DOM_FUNC_ENV}, domtype(f)) then 
      if contains({DOM_EXPR,DOM_IDENT,DOM_INT,DOM_RAT,DOM_FLOAT}, domtype(f)) then
         f := subsop(proc(x) begin _f_ end_proc, [1,1] = x, 4 = f)
      else
         error("Illegal argument: ".expr2text(f))
      end_if 
    end_if; 
    f
 end_proc:

 minx := vbox[1]; maxx := vbox[3]; 
 miny := vbox[2]; maxy := vbox[4];

 # [ Mode = Curve, [fx(t), fy(t)], t=[a,b], ... ] #

 grid := 100; smooth := 0; color := FALSE;

 # 2nd argument must be of the form [fx(t), fy(t)] #

 k := l[2];

 if not testtype(k, Type::ListOf(Type::AnyType,2,2)) then
    error("List of 2 expressions expected")
 end_if;
 fx := k[1]; fy := k[2];

 # 3rd argument must be of the form t = [a, b] #

 r := l[3];
 if not (testtype(r, "_equal") and
    testtype(float(op(r,2)), Type::ListOf(Type::RealNum,2,2))) then
    error("Specification of the form t = [a,b] expected (a, b real numbers)");
 end_if;
 t := op(r, 1); a := op(r, [2,1]); b := op(r, [2,2]);

 for j from 4 to nops(l) do
     k := l[j];
     if not testtype(k, "_equal") then
        error("Specification of the form option = value expected");
     end_if;
     case op(k, 1)
       of Color do
          color := TRUE;
          break;
       of Grid do
          # Grid = [n] #
          grid := op(k, 2);
          if not testtype(grid, Type::ListOf(Type::PosInt,1,1)) then
             error("Grid: positive integer expected")
          end_if;
          grid := grid[1];
          break;
       of Smoothness do
          # Smoothness = [m] #
          smooth := op(k, 2);
          if not testtype(smooth, Type::ListOf(Type::PosInt,1,1)) then
             error("Smoothness: positive integer expected")
          end_if;
          smooth := smooth[1];
          break;
     end_case
  end_for;

  fx := convf(fx,t); fy := convf(fy,t);
  r := clip_2( fx, fy, a, b, grid*(smooth+1), minx, maxx, miny, maxy);

  if nops(r) = 0 then return(null()) end_if;

  if not color then
     l := append(l, Color = [Flat, RGB::ColorList[obj_nr]])
  end_if;

  l1 := null();

  for iv in r do
      l1 := l1, subsop(l, [3,2,1]=iv[1], [3,2,2]=iv[2]);
  end_for;

  return(l1);

end_proc: 
 

plotlib::clip2d_List := proc(l      : DOM_LIST,
                             vbox   : DOM_LIST)

  local col, i, n, new_list, opts, p, q, r, plist;

begin

  # [ Mode = List, [... ] ] #

  if domtype(l[2]) <> DOM_LIST then
     error("Mode = List: Second argument must be a list.")
  end_if;

  new_list := []; 
  
  for p in l[2] do

      opts := null(); col := null();

      if domtype(p) = DOM_POINT then
         if plotlib::in_vbox(p, vbox) then
            new_list := append(new_list, p)
         end_if

      elif domtype(p) = DOM_POLYGON then
         // polygon(p1,...,pn, color, closed, filled)
         // color  : list of 3 reals
         // closed : TRUE or FALSE
         // filled : TRUE or FALSE
         n := nops(p);
         plist := [op(p,q) $ q = 1..n-3];
         if op(p, n-2) <> NIL then
            // Color = ...
            col  := Color=op(p, n-2);
            opts := col;
         end_if;
         if op(p, n-1) then
            // Closed = TRUE  
            // Die Option Closed=TRUE wird umgesetzt,
            // indem der Startpunkt als zusaetzlicher
            // Endpunkt in das Polygon eingefuegt wird.
            plist := append(plist, op(p, 1));
            opts := opts, Closed=TRUE
         end_if:
         if op(p, n) then
             // Filled = TRUE
             if n > 6 then
                error("Only triangles can be filled.")
             end_if;

             if nops(plist) = n-3 then
                // Option Closed=TRUE nicht gesetzt,
                // d.h. Startpunkt wurde nicht ergaenzt.
                // Muss beim Clipping ergaenzt werden, da
                // sonst falsches Resultat erzielt wird.
                plist := append(plist, op(p, 1));
             end_if;

             r := plotlib::clip2d_filled_polygon(plist, vbox);

             if r <> [] then  
                p := [polygon(op(r), opts, Filled=TRUE)];
                p := (plotlib::polygonplot)::PlotScene(p); 
                // p = [Mode=List, [...]]              
                new_list := append(new_list, op(p[2])); 
             end_if
          else
             // Filled = FALSE
             r := plotlib::clip2d_polygon(plist, vbox);
             if r <> [] then
                // Hier nicht opts verwenden, da opts u.U.
                // den Eintrag Closed=TRUE enthaelt. Dies
                // fuehrt zu einem Fehler, da ein Polygon
                // hier mehr als 3 Punkte enthalten kann.
                // Die Option Closed=TRUE wurde umgesetzt,
                // indem der Startpunkt als zusaetzlicher
                // Endpunkt in das Polygon eingefuegt wurde.
                p := polygon(op(r[i]), col) $ i=1..nops(r);
                new_list := append(new_list, p);              
             end_if;
          end_if

     else

        error("point or polygon expected.")

     end_if

  end_for;

  if new_list = [] then
     return( null() )
  else
     return( [Mode=List, new_list] )
  end_if

end_proc:

//
// point_list - [ point(x_1,y_1), ... , point(x_n, y_n) ] , n >= 1
// vbox       - [ minx, miny, maxx, maxy ] 
//
// Rueckgabe: [] - Polygon liegt vollstaendig ausserhalb der ViewingBox
//            [plist_1, ..., plist_m] , m > 0, plist_i sind Listen der
//                 Form [point(x_i_1, y_i_1),...,point(x_i_ni, y_i_ni)]
//                 mit ni >= 1. Die einzelnen Listen plist_i bilden die
//                 Polygone, die innerhalb der ViewingBox liegen.
// 

plotlib::clip2d_polygon := proc(point_list : DOM_LIST, 
                                vbox       : DOM_LIST)

   local i, j, n, p, polygon_list, s; 

begin

   n := nops(point_list);

   if n = 1 then
      if plotlib::in_vbox(point_list[1], vbox) then
         return([point_list])
      else
         return([])
      end_if
   end_if;

   i := 1; polygon_list := null();

   while i <= n and not plotlib::in_vbox(point_list[i], vbox) do
      if i > 1 then
         p := [plotlib::intersect_vbox( point_list[i-1], point_list[i], vbox, TRUE)];
         if p <> [] then 
            polygon_list := polygon_list, p 
         end_if;
      end_if;
      i := i+1;
   end_while;

   if i > n then
      return( [polygon_list] );
   elif i = 1 then
      p := null(); s := 1;
   else // 1 < i <= n 
      p:=plotlib::intersect_vbox(point_list[i-1], point_list[i], vbox, FALSE);
      if p = point_list[i] then
         p := null()
      end_if;
      s := i
   end_if;

   i := i+1;
   while i <= n and plotlib::in_vbox(point_list[i], vbox) do
      i := i+1;
   end_while;

   if i > n then
      // Polygon point_list[s],...,point_list[n]
      // liegt innerhalb der ViewingBox
      return( [polygon_list, [p, point_list[j] $ j =s..n]] );
   else
      polygon_list := 
         polygon_list, 
          [ p, point_list[j] $ j=s..i-1,
            plotlib::intersect_vbox(point_list[i], point_list[i-1], vbox, FALSE) 
          ];
      if i = n then
         return( [polygon_list] );
      else
         return( [polygon_list] . 
                 plotlib::clip2d_polygon( [point_list[j] $ j=i..n], vbox)
         );
      end_if;
   end_if;
end_proc: 

//
// p    - point(x, y)
// vbox - [minx, miny, maxx, maxy]
//

plotlib::in_vbox := proc(p, vbox)
   local eps;
begin
   eps := 10^(-DIGITS+1);
   if op(p, 1) >= vbox[1]-eps and op(p, 1) <= vbox[3]+eps and
      op(p, 2) >= vbox[2]-eps and op(p, 2) <= vbox[4]+eps then
      return(TRUE)
   else
      return(FALSE)
   end_if
end_proc:

//
// p_out       - point(x1, y1) 
// p_in        - point(x2, y2) 
// vbox        - [minx, miny, maxx, maxy]
// both_points - TRUE : Ausgabe beider Punkte
//             - FALSE : Ausgabe des Schnittpunktes
//
// Ist both_points = FALSE, dann muss p_out ausserhalb 
// und p_in innerhalb der ViewingBox vbox liegen.
// Ist both_points = TRUE, dann liegen p_out und p_in
// beide ausserhalb der ViewingBox vbox
//
// Rueckgabe:
// 1) Ein Punkt point(x, y) oder 
// 2) Sequenz zweier Punkte point(x3, y3), point(x4,y 4)
//

plotlib::intersect_vbox := proc(p_out, p_in, vbox, both_points : DOM_BOOL)
   local eps, f, lb, m, p, p2, r, t, 
         ub, x, x1, x2, y1, y2;
begin 

   x1 := op(p_out, 1); y1 := op(p_out, 2);
   x2 := op(p_in, 1);  y2 := op(p_in, 2);
   eps := 10^(-DIGITS+1);

   if abs(x1-x2)<eps then
      // Beiden Punkte definieren eine Senkrechte
      if x1 >= vbox[1] and x1 <= vbox[3] then
      // Die Senkrechte liegt bzgl. der x-Koordinate in vbox
         if both_points and
            ((y1 < vbox[2] and y2 < vbox[2]) or 
             (y1 > vbox[4] and y2 > vbox[4])) then
            return( null() );
         end_if;
         p := point(x1, min(max(y1, vbox[2]), vbox[4]));
         if both_points then
           p := p, point(x2, min(max(y2, vbox[2]), vbox[4]))
         end_if;
         return( p )
      else
         return( null() )
      end_if
   elif abs(y1-y2)<eps then
      // Beiden Punkte definieren eine Waagerechte
      if y1 >= vbox[2] and y1 <= vbox[4] then
         // Die Waagerechte liegt bzgl. der y-Koordinate in vbox
         if both_points and
            ((x1 < vbox[1] and x2 < vbox[1]) or 
             (x1 > vbox[3] and x2 > vbox[3])) then
            return(null())
         end_if;
         p := point(min(max(x1, vbox[1]), vbox[3]), y1);
         if both_points then
            p := p, point(min(max(x2, vbox[1]), vbox[3]), y2)
         end_if;
         return( p )
      else
         return( null() )
      end_if
  end_if;

  m := (y2-y1)/(x2-x1); t := y1-m*x1; f := m*x+t;

  r := solve({f >= vbox[2], f <= vbox[4]},x);

  // r = {lb <= x , x <= ub} 

  if r = {} then return(null()) end_if;

  lb := max(op(r, [1,1,1]), vbox[1]); 
  ub := min(op(r, [1,2,2]), vbox[3]);

  if lb > vbox[3] then
     return(null())
  end_if;

  if x1 < lb then
     p := point(lb, subs(f, x=lb))
  elif x1 > ub then
     p := point(ub, subs(f, x=ub))
  else
     p := point(x1,y1)
  end_if;

  if both_points then
     if x2 < lb then
        p2 := point(lb, subs(f,x=lb))
     elif x2 > ub then
        p2 := point(ub, subs(f,x=ub))
     else
        p2 := point(x2,y2)
     end_if;
     if plotlib::in_vbox(p, vbox) and plotlib::in_vbox(p2, vbox) then
        if p <> p2 then
           p := p, p2
        end_if
     else
        p := null()
     end_if
  end_if;

  return( p ); 

end_proc:


//
// point_list - [ point(x_1,y_1), ... , point(x_4, y_4) ] 
// vbox       - [ minx, miny, maxx, maxy ] 
//
// 
// Es muss gelten: x_1 = x_4 und y_1 = y_4, d.h. es
// liegt ein geschlossener Polygonzug vor, der ein
// Dreieck bildet. Dreieck kann entartet sein, d.h.
// auch eine Strecke bilden.
//

plotlib::clip2d_filled_polygon := proc(point_list : DOM_LIST, 
                                       vbox       : DOM_LIST)

   local m, n, p, p1, p2, polygon_list;

begin

   polygon_list := plotlib::clip2d_polygon(point_list, vbox);
   
   n := nops(polygon_list);
   case n
     of 0 do
        return( null() ) 
     of 1 do
        p := [op(polygon_list[1])]; 
        break;
     of 2 do
     of 3 do
        m  := nops(polygon_list[1]);
        p1 := polygon_list[1][m];
        p2 := polygon_list[2][1];
        p  := [ op(polygon_list[1]), plotlib::add_edge(p1, p2, vbox), 
                op(polygon_list[2]) ];

        if n = 2 then break end_if;
        
        m  := nops(polygon_list[2]);
        p1 := polygon_list[2][m];
        p2 := polygon_list[3][1];
        p  := append(p, plotlib::add_edge(p1, p2, vbox), op(polygon_list[3]));

   end_case;

   m  := nops(polygon_list[n]);
   p1 := polygon_list[1][1];
   p2 := polygon_list[n][m];
   if p1 <> p2 then
      p := append(p, plotlib::add_edge(p1, p2, vbox), p1);
   end_if;

   return( p );

end_proc:  

plotlib::add_edge := proc(p1, p2, vbox)
   local eps, p, x1, x2, y1, y2;
begin

   x1 := op(p1, 1); y1 := op(p1, 2);
   x2 := op(p2, 1); y2 := op(p2, 2);
   eps := 10^(-DIGITS+1);

   if abs(x1-x2)>eps and abs(y1-y2)>eps then
      if x1 = vbox[1] then
         if y2 < y1 then
            p := point(vbox[1], vbox[2])
         else
            p := point(vbox[1], vbox[4])
         end_if
      elif x1 = vbox[3] then
         if y2 < y1 then
            p := point(vbox[3], vbox[2])
         else
            p := point(vbox[3], vbox[4])
         end_if
      else
         p := plotlib::add_edge(p2, p1, vbox);
      end_if
   else
      p := null()
   end_if;

   return( p );

end_proc:

plotlib::calc_ticks := proc(a : Type::RealNum, 
                            b : Type::RealNum, 
                            n : DOM_INT     )
    local d, eps, l, s;
begin
    eps := 0.005;
    a := float(a); b := float(b);
    if abs(b-a) < 10^(-DIGITS) then
       return( 1.0 )
    end_if;
    l := b - a;
    if n = 1 then
       return( l )
    end_if;
 
    d := l / (n-1);
    s := 1.0;
    if d > 1.0 + eps then
	   while d > 1.0 + eps do
	    d := d / 10.0;
	    s := s * 10.0;
       end_while
    elif d < 0.1 - eps then
	   while d < 0.1 - eps do
	    d := d * 10.0;
	    s := s / 10.0;
	   end_while
    end_if;
    // tick marks may be at 0.1, .125, .2, .25, .5 and 1.
    if d < 0.125 - eps then
	   d := 0.1 * s;
    elif d < 0.2 - eps then
	   d := 0.125 * s;
    elif d < 0.25 - eps then
	   d := 0.2 * s;
    elif d < 0.5 - eps then
	   d := 0.25 * s;
    elif d < 1.0 - eps then
	   d := 0.5 * s;
    else
	   d := s;
    end_if;
    return( d );
end_proc:

plotlib::check_ticks := proc(rhs, vbox : DOM_LIST)

   local dx, dy, l, maxx, maxy, minx, miny, sx, sy;

begin

   if type(rhs) = DOM_INT and rhs > 0 then
      // Ticks = n
      rhs := [rhs, rhs]
   elif type(rhs) <> DOM_LIST or nops(rhs) <> 2 then
      error("Illegal Ticks");
   end_if;

   l := [plotlib::check_tick(rhs[1], vbox[1], vbox[3])];
   l := l . [plotlib::check_tick(rhs[2], vbox[2], vbox[4])];     

   return( Ticks = l )

end_proc:

plotlib::check_tick := proc(l, a, b)
   local l1, k, t;
begin
   if type(l) = DOM_INT then
      return( [a..b, plotlib::calc_ticks(a, b, l), 1] )
   end_if;

   if type((t:=l[1])) = "_range" then
      // [ al..bl, d, s ]
      if nops(l) <> 3 then
         error("Illegal Ticks")
      end_if; 
      l1 := [min(a, op(t, 1))..max(b, op(t, 2)), l[2], l[3]]
   else
      // [ k_1, ..., k_n ]
      // k_i -> r_i oder k_i -> r_i = s_i
      l1 := l;
      if a < min(op(l)) then l1 := append(l1, a) end_if;
      if b > max(op(l)) then l1 := append(l1, b) end_if;
   end_if;

   return( l1 )
      
end_proc:
      
