/*

plotlib::combine(p_1, p_2, ...) 

p_i - Graphik-Struktur

Mittels plotlib::combine koennen Graphiken zusammengefasst
werden. Dies ist besonders dann interessant, wenn Graphiken
mit unterschiedlichen Befehlen, etwa plotlib::contourplot
und plotfunc, erzeugt wurden. Das folgende Beispiel zeigt
die Vorgehensweise.

Zunaechst wird das sofortige Zeichnen der Graphik unterbunden.

plotlib::show(FALSE):

Alle plot-Befehle (plot2d, plot3d, plotfunc, Befehle der
Bibliothek plotlib, usw.) liefern nun als Ergebnis eine
Struktur (Ausdruckssequenz oder Liste), die weiterverarbeitet
werden kann und spaeter dann gezeichnet wird.

a:=plotfunc(sin(sqrt(x^2+y^2)),x=-5..5,y=-5..5):

b:=plotlib::contourplot([[x,y,sin(sqrt(x^2+y^2))],
                         x=[-5,5],y=[-5,5],Grid=[21,21],
                         Colors=[Height]],Style=Bottom):

Zuruecksetzen, d.h. Graphiken werden wieder sofort gezeichnet.

plotlib::show(TRUE):

Der 2D-contourplot wird nun in ein 3D-Objekt konvertiert. 
Der z-Wert der Funktion ist konstant -2:

c:=plotlib::conv2d({-2}, b):

Kombination der beiden 3D-Plots:

d:=plotlib::combine(a,c): 

Zeichnen der Graphik:

plotlib::show(d):

*/

plotlib::combine := proc()
  local a, arg, b, objs, sobjs, t;
begin
  objs := null(); sobjs := null();
  for arg in args() do
      if type(arg) = "_equal" then
         a := op(arg,1); b := op(arg,2);
         if type(t[a]) = "_index" then
            t[a] := b; sobjs := sobjs, arg;
         elif a = Dimension then
            if t[a] <> b then
              t:="It's not possible to combine 2D and 3D plots.";
              t:=t."\n         Use plotlib::conv2D to convert ";
              t := t."2D into 3D plots.\n        ";
              error(t);
            end_if;
            t[a]:=3
         elif t[a] <> b then
            print(Unquoted, expr2text(a)." declared twice. ".expr2text(arg)." ignored.")
         end_if
      else
         objs := objs, arg
      end_if;
  end_for;
  return(sobjs, objs)
end_proc:
     