#-

plotlib::contourplot([[x,y,exp(x*y)],x=[-1,1],y=[-1,1]]):

plotlib::contourplot([[x,y,exp(x*y)],x=[-1,1],y=[-1,1],Colors=[Height]]):

plotlib::contourplot([[x,y,exp(x*y)],x=[-1,1],y=[-1,1]],Style=Attached):

Multiple contour plot: 

plotlib::contourplot([[x,y,sin(x*y)],x=[-PI,PI],y=[-PI,PI],Grid=[20,20]],
                     [[x,y,x + 2*y] ,x=[-PI,PI],y=[-PI,PI],Colors=[Flat,RGB::Blue]]):

Plot of the implicite function defined by (x^2+y^2)^3-(x^2-y^2)^2 = 0:

plotlib::contourplot([[x,y,(x^2+y^2)^3-(x^2-y^2)^2],x=[-1,1],y=[-1,1],
                     Contours=[0], Grid=[20,20]]):

-#

plotlib::contourplot := proc()
     local a, b, c, colors, contours, d, def, f1, f2, f3, i, l,
           m, n, objects, object_options, scene_options, style, xa, ya, za;
begin

     #------------------- Evaluate options --------------------------#

     objects := null();

     l := plotlib::get_objects(args());

     for i in l do

       plotlib::set_options(i);

       #--- Define matrix and initialize all elements with 1 ----------#

       def := map(array(0 .. m, 0 .. n), 1);

       #--- Evaluate the functions f1, f2, f3 on the m by n grid at ---#
       #--- equally spaced points in the ranges a..b and c..d       ---#
     
       xa := plotlib::gridpoints(f1,a,b,c,d,m,n);
       ya := plotlib::gridpoints(f2,a,b,c,d,m,n);
       za := plotlib::gridpoints(f3,a,b,c,d,m,n);

       objects := objects, 
                [Mode=List,[plotlib::levels(m,n,contours,colors)], object_options]

     end_for;

     #-- Plot the list of lines constructed by the function levels --#

     if style = Bottom then
        plot2d(scene_options, objects) 
     else
        plot3d(scene_options, objects)
     end_if

end_proc:


plotlib::gridpoints := proc(f, xmin, xmax, ymin, ymax, m, n)
     local a, i, j, xinc, yinc, x, y, z;
     // def is declared in plotlib::contourplot
begin
    a := array(0 .. m, 0 .. n);
    xinc := (xmax - xmin)/m;
    yinc := (ymax - ymin)/n;
    x := xmin;
    for i from 0 to m do
        y := ymin;
        for j from 0 to n do
            if traperror((z := float(f(x, y))))<>0 or type(z)<>DOM_FLOAT then
               def[i, j] := 0;
            else
               a[i, j] := z;
            end_if;
            y := y + yinc
        end_for;
        x := x + xinc
    end_for;
    a
end_proc:


plotlib::levels := proc(m, n, contours, colors)
    local ha, i, inc, j, Min, nc, nu, numpts, s, wa1, wa2, wa3, wa4;
     // style and za are declared in plotlib::contourplot
begin
    s := (za[i, j] $ i = 0 .. m) $ j = 0 .. n;
    Min := min(s);
    if testtype(contours, DOM_INT) then
        inc := (max(s) - Min)/(contours-1);
        ha := [Min + i*inc $ i = 0 .. contours - 1]
    else 
        ha := contours
    end_if;
    nu := nops(ha); nc := nops(colors);
    numpts := array(1 .. nu);
    wa1 := array(1 .. nu, 0 .. 6*m*n);
    wa2 := array(1 .. nu, 0 .. 6*m*n);
    wa3 := array(1 .. nu, 0 .. 6*m*n);
    wa4 := array(1 .. nu, 0 .. 6*m*n);

    plotlib::levelcurve(m, n, nu);

    if nu=1 and numpts[1]<0 then
       error("No contour lines at ".expr2text(ha[1]))
    end_if;

    if style = Bottom then

       (polygon(point(wa1[nu - j + 1, i], wa2[nu - j + 1, i]),
                point(wa3[nu - j + 1, i], wa4[nu - j + 1, i]),
                Color= colors[modp(j+nc-1, nc)+1]
               ) $ i = 0 .. numpts[nu - j + 1]
       ) $ j = 1 .. nu

    else

       (polygon(point(wa1[nu - j + 1, i], wa2[nu - j + 1, i], ha[j]),
                point(wa3[nu - j + 1, i], wa4[nu - j + 1, i], ha[j]),
                Color= colors[modp(j+nc-1, nc)+1]
               ) $ i = 0 .. numpts[nu - j + 1]
       ) $ j = 1 .. nu
 
    end_if

end_proc:


plotlib::levelcurve := proc(m, n, nu)
   local a, i, j, k, lev, eps, x1, x2, y1, y2, c, flag, u,
         d_1, d_2, d_3, h_1, h_2, h_3, x_1, x_2, x_3, y_1, y_2, y_3;
   // def, xa, ya, za are declared in plotlib::contourplot
   // ha, numpts, wa1, wa2, wa3, wa4 are declared in plotlib::levels
begin
    k  := 0;
    for i from 0 to m do 
        for j from 0 to n do
            if def[i, j] = 1 then
               if k = 0 then
                  x1 := za[i, j]; x2 := za[i, j]; k := 1
               elif x2 < za[i, j] then 
                  x2 := za[i, j]
               elif za[i, j] < x1 then
                  x1 := za[i, j]
               end_if
           end_if
        end_for
    end_for;
    if k = 0 then error("could not evaluate expression") end_if;
    eps := 10.0^(-DIGITS)*(x2 - x1);
    for u from 1 to nu do
        lev := ha[u];
        k := 0;
        for i from 0 to m - 1 do 
            for j from 0 to n - 1 do 
                for c from 1 to 2 do
                    if c = 1 then
                        d_1 := def[i, j];
                        d_2 := def[i + 1, j];
                        d_3 := def[i, j + 1];
                        x_1 := xa[i, j];
                        x_2 := xa[i + 1, j];
                        x_3 := xa[i, j + 1];
                        y_1 := ya[i, j];
                        y_2 := ya[i + 1, j];
                        y_3 := ya[i, j + 1];
                        h_1 := za[i, j] - lev;
                        h_2 := za[i + 1, j] - lev;
                        h_3 := za[i, j + 1] - lev
                    else
                        d_1 := def[i + 1, j + 1];
                        x_1 := xa[i + 1, j + 1];
                        y_1 := ya[i + 1, j + 1];
                        h_1 := za[i + 1, j + 1] - lev
                    end_if;
                    if d_1*d_2*d_3 = 0 then next end_if;
                    flag := 1;
                    if h_1 < -eps then
                        if h_2 < -eps then
                            if eps < h_3 then
                                a := - h_1/(h_3 - h_1);
                                x1 := x_3*a + (1.0 - a)*x_1;
                                y1 := y_3*a + (1.0 - a)*y_1;
                                a := - h_2/(h_3 - h_2);
                                x2 := x_3*a + (1.0 - a)*x_2;
                                y2 := y_3*a + (1.0 - a)*y_2
                            else 
                                flag := 0
                            end_if
                        elif eps < h_2 then
                            a := - h_1/(h_2 - h_1);
                            x1 := x_2*a + (1.0 - a)*x_1;
                            y1 := y_2*a + (1.0 - a)*y_1;
                            if h_3 < -eps then
                                a := - h_2/(h_3 - h_2);
                                x2 := x_3*a + (1.0 - a)*x_2;
                                y2 := y_3*a + (1.0 - a)*y_2
                            elif eps < h_3 then
                                a := - h_1/(h_3 - h_1);
                                x2 := x_3*a + (1.0 - a)*x_1;
                                y2 := y_3*a + (1.0 - a)*y_1
                            else 
                                x2 := x_3; y2 := y_3
                            end_if
                        else
                            x1 := x_2;
                            y1 := y_2;
                            if h_3 < -eps then flag := 0
                            elif eps < h_3 then
                                a := - h_1/(h_3 - h_1);
                                x2 := x_3*a + (1.0 - a)*x_1;
                                y2 := y_3*a + (1.0 - a)*y_1
                            else 
                                x2 := x_3; y2 := y_3
                            end_if
                        end_if
                    elif eps < h_1 then
                        if h_2 < -eps then
                            a := - h_1/(h_2 - h_1);
                            x1 := x_2*a + (1.0 - a)*x_1;
                            y1 := y_2*a + (1.0 - a)*y_1;
                            if h_3 < -eps then
                                a := - h_1/(h_3 - h_1);
                                x2 := x_3*a + (1.0 - a)*x_1;
                                y2 := y_3*a + (1.0 - a)*y_1
                            elif eps < h_3 then
                                a := - h_2/(h_3 - h_2);
                                x2 := x_3*a + (1.0 - a)*x_2;
                                y2 := y_3*a + (1.0 - a)*y_2
                            else 
                                x2 := x_3; y2 := y_3
                            end_if
                        elif eps < h_2 then
                            if h_3 < -eps then
                                a := - h_1/(h_3 - h_1);
                                x1 := x_3*a + (1.0 - a)*x_1;
                                y1 := y_3*a + (1.0 - a)*y_1;
                                a := - h_2/(h_3 - h_2);
                                x2 := x_3*a + (1.0 - a)*x_2;
                                y2 := y_3*a + (1.0 - a)*y_2
                            else 
                                flag := 0
                            end_if
                        else
                            x1 := x_2;
                            y1 := y_2;
                            if h_3 < -eps then
                                a := - h_1/(h_3 - h_1);
                                x2 := x_3*a + (1.0 - a)*x_1;
                                y2 := y_3*a + (1.0 - a)*y_1
                            elif eps < h_3 then 
                                flag := 0
                            else 
                                x2 := x_2; y2 := y_2
                            end_if
                        end_if
                    else
                        x1 := x_1;
                        y1 := y_1;
                        if h_2 < -eps then
                            if h_3 < -eps then flag := 0
                            elif eps < h_3 then
                                a := - h_2/(h_3 - h_2);
                                x2 := x_3*a + (1.0 - a)*x_2;
                                y2 := y_3*a + (1.0 - a)*y_2
                            else 
                                x2 := x_3; y2 := y_3
                            end_if
                        elif eps < h_2 then
                            if h_3 < -eps then
                                a := - h_2/(h_3 - h_2);
                                x2 := x_3*a + (1.0 - a)*x_2;
                                y2 := y_3*a + (1.0 - a)*y_2
                            elif eps < h_3 then 
                                flag := 0
                            else 
                                x2 := x_3; y2 := y_3
                            end_if
                        else
                            if h_3 < eps and -eps < h_3 then
                                wa1[u, k] := x_1;
                                wa2[u, k] := y_1;
                                wa3[u, k] := x_2;
                                wa4[u, k] := y_2;
                                k := k + 1;
                                wa1[u, k] := x_2;
                                wa2[u, k] := y_2;
                                wa3[u, k] := x_3;
                                wa4[u, k] := y_3;
                                k := k + 1;
                                wa1[u, k] := x_1;
                                wa2[u, k] := y_1;
                                wa3[u, k] := x_3;
                                wa4[u, k] := y_3;
                                k := k + 1;
                                flag := 0
                            else 
                                x2 := x_2; y2 := y_2
                            end_if
                        end_if
                    end_if;
                    if flag <> 0 then
                        wa1[u, k] := x1;
                        wa2[u, k] := y1;
                        wa3[u, k] := x2;
                        wa4[u, k] := y2;
                        k := k + 1
                    end_if
                end_for
            end_for
        end_for;
        numpts[u] := k - 1
    end_for;
end_proc:


plotlib::set_options := proc(l)
    local color, grid, i, j, k, nc, t, x, y, xr, yr,
          convf, From, To, inc1, inc2, inc3;
    // a, b, c, colors, contours, f1, f2, f3, m, n, object_options
    // are declared in plotlib::contourplot
begin

    convf := proc(f, x, y) local _f_; begin
       if not contains({DOM_PROC, DOM_EXEC, DOM_FUNC_ENV}, domtype(f)) then 
          if contains({DOM_EXPR,DOM_IDENT,DOM_INT,DOM_RAT,DOM_FLOAT}, domtype(f)) then
             f := subsop(proc(x,y) begin _f_ end_proc, 
                         [1,1] = x, [1,2] = y, 4 = f)
          else
             error("Illegal argument: ".expr2text(f))
          end_if 
       end_if; 
       f
    end_proc:
      

    object_options := null();
    color := [Flat];
    grid := [11, 11]; 
    contours := 8;

           # 1st argument must be of the form [f1(x,y), f2(x,y), f3(x,y)] #

           k := l[1];

           if not testtype(k, Type::ListOf(Type::AnyType,3,3)) then
              error("List of 3 expressions expected")
           end_if;
           f1 := k[1]; f2 := k[2]; f3 := k[3];

           # 2nd argument must be of the form x = [a, b] #

           xr := l[2];
           if not (testtype(xr, "_equal") and
                   testtype(float(op(xr,2)), Type::ListOf(Type::RealNum,2,2))) then
              error("Specification of the form x = [a,b] expected (a, b real numbers)");
           end_if;
           x := op(xr, 1); a := op(xr, [2,1]); b := op(xr, [2,2]);

           # 3rd argument must be of the form y = [c, d] #

           yr := l[3]; y := op(yr, 1);
           if not (testtype(yr, "_equal") and
                   testtype(float(op(yr,2)), Type::ListOf(Type::RealNum,2,2))) then
              error("Specification of the form x = [a,b] expected (a, b real numbers)");
           end_if;
           y := op(yr, 1); c := op(yr, [2,1]); d := op(yr, [2,2]);

           for j from 4 to nops(l) do
               k := l[j];
               if not testtype(k, "_equal") then
                  error("Specification of the form option = value expected");
               end_if;
                  case op(k, 1)
                       of Contours do
                          t := op(k, 2);
                          if (testtype(t, DOM_INT) and t >= 2) or 
                             testtype(t, Type::ListOf(Type::RealNum,1)) then
                             contours := t;
                             break;
                          end_if;
                          error("Contours = integer or Contours = [r1,...,rN] expected")
                       of Colors do
                          color := op(k,2);
                          if not testtype(color, DOM_LIST) then
                             error("Colors = [...] expected")
                          end_if;
                          break;
                       of Grid do
                          grid := op(k, 2);
                          if not testtype(grid, Type::ListOf(Type::PosInt,2,2)) then
                             error("Grid: positive integers expected")
                          end_if;
                          break;
                       otherwise
                          object_options := object_options, k;
                  end_case
           end_for;

    m := grid[1]; n := grid[2];

    # Convert f1, f2, f3 #
     
    f1 := convf(f1, x, y); f2 := convf(f2, x, y); f3 := convf(f3, x, y);

    # Set list of colors #

    case color[1]
      of Flat do
         if nops(color) = 1 then
            colors := [ RGB::Red ]
         elif nops(color) = 2 and 
              testtype(color[2], Type::ListOf(Type::RealNum,3,3)) then
            colors := [ color[2] ]
         else
            error("Illegal color specification")
         end_if;
         break;
      of Height do
         if nops(color) = 1 then 
            From := RGB::Red; To := RGB::Yellow;
         elif nops(color) = 3 and
            testtype(color[2], Type::ListOf(Type::RealNum,3,3)) and
            testtype(color[3], Type::ListOf(Type::RealNum,3,3)) then
            From := color[2]; To := color[3];
         else
            error("Illegal color specification") 
         end_if;
         if testtype(contours, DOM_INT) then 
            n := contours 
         else 
            n := nops(contours)
         end_if;
         if n = 1 then
            colors := [ From ]
         else
            inc1 := (To[1]-From[1])/(n-1);
            inc2 := (To[2]-From[2])/(n-1);
            inc3 := (To[3]-From[3])/(n-1);
            colors := [[From[1]+i*inc1,From[2]+i*inc2,From[3]+i*inc3]
                       $ hold(i)=0..n-1];
         end_if;
         break;
      of Curves do
         if nops(color) < 2 or
            not _and(testtype(color[i], 
                     Type::ListOf(Type::RealNum,3,3)) $ hold(i) = 2 .. nops(color)
                    ) then
            error("Illegal color specification")
         end_if;
         colors := [ color[i] $ hold(i) = 2 .. nops(color) ];
         break;
      otherwise
         error("Illegal color specification");
    end_case;

end_proc:

plotlib::get_objects := proc()
    local i, l, objects;
    // scene_options and style are declared in plotlib::contourplot
begin
    scene_options  := null();
    objects := [];
    style := Bottom;
    
    for i from 1 to args(0) do 
        l:=args(i); 
        if testtype(l, DOM_LIST) then
           objects := append(objects, l);
        else
           if testtype(l, "_equal") and op(l, 1) = Style then
              style := op(l, 2);
              if not contains({Bottom,Attached}, style) then
                 error("Style: Bottom or Attached expected")
              end_if;
           else
              scene_options := scene_options, l
           end_if
        end_if
    end_for;
    objects
end_proc:
              


