/*

plotlib::conv2d(p)
plotlib::conv2d({zv}, p)
plotlib::conv2d({zv, [x,y,z]}, p)


p -  2D-Graphik-Struktur
zv - Konstanter z-Wert

Mittels plotlib::conv2d koennen 2D-Graphiken in 3D-Graphiken
umgeformt werden. Das folgende Beispiel zeigt die 
Vorgehensweise.

Zunaechst wird das sofortige Zeichnen der Graphik unterbunden.

plotlib::show(FALSE):

Alle plot-Befehle (plot2d, plot3d, plotfunc, Befehle der
Bibliothek plotlib, usw.) liefern nun als Ergebnis eine
Struktur (Ausdruckssequenz oder Liste), die weiterverarbeitet
werden kann und spaeter dann gezeichnet wird.

a:=plotfunc(sin(sqrt(x^2+y^2)),x=-5..5,y=-5..5):

b:=plotlib::contourplot([[x,y,sin(sqrt(x^2+y^2))],
                         x=[-5,5],y=[-5,5],Grid=[21,21],
                         Colors=[Height]],Style=Bottom):

Zuruecksetzen, d.h. Graphiken werden wieder sofort gezeichnet.

plotlib::show(TRUE):

Der 2D-contourplot wird nun in ein 3D-Objekt konvertiert. 
Der z-Wert der Funktion ist konstant -2:

c:=plotlib::conv2d({-2}, b):

Kombination der beiden 3D-Plots:

d:=plotlib::combine(a,c): 

Zeichnen der Graphik:

plotlib::show(d):

*/

plotlib::conv2d := proc()
  local a, arg, b, i, l, objs, pl, sobjs, t, z_val;
begin
  objs := null(); sobjs := null();
  z_val := 0; t[1]:=1; t[2]:=2; t[3]:=3;
  for arg in args() do
      if type(arg) = DOM_SET then
         case nops(arg)
            of 1 do
               a := op(arg, 1);
               if type(a) = DOM_LIST then
                  t := a
               else
                  z_val := a
               end_if;
               break;
            of 2 do
               a := op(arg, 1);
               if type(a) = DOM_LIST then
                  t := a;
                  z_val := op(arg, 2)
               else
                  z_val := a; t := op(arg, 2)
               end_if;
               break;
            otherwise
               error("Illegal first argument.")
         end_case
      elif type(arg) = "_equal" then
         a := op(arg, 1); b := op(arg, 2);
         case a 
           of CameraPoint do
              # Ignore it #
              break;
           of Dimension do
              if b <> 2 then
                 error("2d plot expected!")
              end_if;
              sobjs := sobjs, Dimension=3;
              break;
           of Ticks      do
           of AxesOrigin do
           of FocalPoint do
              # [tx, ty] -> [tx, ty, 0] #
              if type(b) = DOM_LIST and nops(b) = 2 then
                 sobjs := sobjs, a = append(b, 0)
              end_if;
              break;
           otherwise
              sobjs := sobjs, arg
        end_case
      else
        case op(arg, 1) 
          of Mode=Curve do
             arg := subsop(arg, [2,3]=z_val);
             arg := subsop(arg, [2,1]=op(arg, [2,t[1]]), [2,2]=op(arg, [2,t[2]]), 
                                [2,3]=op(arg, [2,t[3]])); 
             break
          of Mode = List do
           a := arg[2]; l := [];
           for a in arg[2] do
               if type(a) = DOM_POINT then
                  pl := op(a,1..2), z_val;
                  pl := subsop(pl, 1=op(pl, t[1]), 2=op(pl, t[2]), 3=op(pl, t[3])); 
                  if type(op(a, 3)) = DOM_NIL then 
                    l := append(l, point(pl))
                  else
                    l := append(l, point(pl, Color=op(a,3)))                  
                  end_if
               elif type(a) = DOM_POLYGON then
                  pl := null();
                  for i from 1 to nops(a)-3 do
                      p := op(a, i);
                      b := op(p,1..2), z_val;
                      b := subsop(b, 1=op(b, t[1]), 2=op(b, t[2]), 3=op(b, t[3]));
                      if type(op(p, 3)) = DOM_NIL then 
                         pl := pl, point(b)
                      else
                        pl := pl, point(b, Color=op(p,3))                
                      end_if
                  end_for;
                  if op(a, i) = NIL then
                     l := append(l, polygon(pl, Closed=op(a,i+1), Filled=op(a,i+2)))
                  else
                     l := append(l, polygon(pl, Color=op(a,i),Closed=op(a,i+1), Filled=op(a,i+2)))
                  end_if
               end_if
           end_for;
           arg := [Mode=List, l];
           break
        end_case;
        objs := objs, arg
      end_if
  end_for;
  sobjs, objs
end_proc: