#

export(plotlib): export(RGB):

dataplot(Piechart, [5,12,38,14,25]):

dataplot(Piechart3d, [5,12,38], Colors=[RoyalBlue,VioletRed,GreenPale]):

dataplot(Line,   [[5,10,24,-3],[6,5,2,18],[19,45,12,-10]], Colors=[Red,Green,Blue]):

dataplot(Curve, [[5,10,24,-3],[6,5,2,18],[19,45,12,-10]], Colors=[Red,Green,Blue]):

dataplot(Column, [[5,10,24,-3],[6,5,2,18],[19,45,12,-10]], Colors=[Red,Green,Blue]):

dataplot(Beam, [[5,10,24,-3],[6,5,2,18],[19,45,12,-10]], Colors=[Red,Green,Blue]):

dataplot(Column, [[5,10,24,-3,6,5,2,18]]):

dataplot(Plain, [[5,10,24,-3],[6,5,2,18],[19,45,12,-10]], Colors=[Red,Red,Green]):

dataplot(Surface, [[5,10,24,-3],[6,5,2,18],[19,45,12,-10]]):

#

alias(Gray   = RGB::Gray):
alias(Black  = RGB::Black):

plotlib::dataplot := proc(dtyp, l)
  local c, i, t, set_option;
begin

  set_option := proc(eqn)
  begin
     if not testtype(eqn, "_equal") then
        error("Colors = [...] or Titles = [...] expected")
     end_if;
     case op(eqn,1) 
         of Titles do
            t := op(eqn,2); break;
         of Colors do
            c := op(eqn,2); break;
         otherwise
            error("Unknown option")
     end_case;
  end_proc:
  
  c := RGB::ColorList;
  t := [];

  case args(0) 
    of 4 do
       set_option(args(4));
    of 3 do
       set_option(args(3));
    of 2 do  
       if not testtype(c, DOM_LIST) or nops(c) < nops(l) then
          error("3rd argument must be a list with ".nops(l)." color names")
       end_if;
       case dtyp 
         of Piechart   do
         of Piechart3d do
            if not testtype(l, Type::ListOf(Type::RealNum,1)) or
               _or(l[i]<=0$i=1..nops(l)) then
               error("List of positive real numbers expected")
            end_if;
            next;
         of Piechart do
            plotlib::piechart(l, c, t, 2); break;
         of Piechart3d do
            plotlib::piechart(l, c, t, 3); break;
         of Line      do
         of Column    do
         of Beam      do
#
         of Histogram do
#
         of Plain     do
         of Curve     do
         of Surface   do
            if not testtype(l, Type::ListOf(Type::ListOf(Type::RealNum,1),1)) then
               error("2nd argument must be a list of lists")
            elif _or(nops(l[i])<>nops(l[i+1])$i=1..nops(l)-1) then
              error("Lists must have the same number of elements")
            end_if;
            next;
         of Line do
            plotlib::line(l, c, t); break;
         of Column do
            plotlib::column(l, c, t); break;
         of Beam do
            plotlib::beam(l, c, t); break;
#
         of Histogram do
            plotlib::histogram(l, c, t); break;
#
         of Plain do
            plotlib::solid_plain(l, c, t); break;
         of Surface do
            plotlib::surface(l, c, t); break;
         of Curve do
            plotlib::curve(l, c, t); break;
         otherwise
            error("Unknown diagram type")
       end_case;
       break;
   otherwise
      error("Wrong number of parameters")
 end_case
end_proc: 

plotlib::piechart := proc(l, c, t, d)
  local a, b, fsin, fcos, grid, i, obj, p, p2, rgb1, s, u, v;
begin

  rgb1 := proc(c)
  begin
      c[1] := max(0,c[1]-0.1);
      c[2] := max(0,c[2]-0.1);
      c[3] := max(0,c[3]-0.1);
      c
  end_proc:
     
  fsin := funcattr(sin, "float"): 
  fcos := funcattr(cos, "float"): 
 
  s := _plus(op(l)); obj := null(); grid := 30;
  p := float(PI); p2 := p/2; b := -p; 
  t := plotlib::set_titles(t, l, nops(l)*(d-1));

  for i from 1 to nops(l) do
      a := b; b := b+l[i]/s*2*p;
      obj := obj, [Mode=Surface,[v*fsin(u),v*fcos(u), 0],u=[a,b],v=[0,1],
                   Style=[ColorPatches,Only], Color=[Flat, c[i]],
                   Grid=[grid,grid], Title=t[(d-1)*(i-1)+1]];
      if d = 3 then
         obj := obj, [Mode=Surface,[fcos(u+p2),fsin(u+p2), v], u=[-b,-a], v=[-0.2,0],
                      Style=[ColorPatches,Only], Color=[Flat, rgb1(c[i])],
                      Grid=[grid,grid], Title=t[2*i]]
      end_if
  end_for;

  plot3d(Axes=None, obj)

end_proc:


#- 

Das gesamte Liniendiagramm koennte durch ein Objekt dargestellt werden. Man 
koennte dann aber nur zwei Beschriftungen eingeben, eine fuer die Szene und 
eine fuer das Objekt. Aus diesem Grund wird jede Linie zwischen zwei Punkten
als ein eigenes Objekt dargestellt. Zusaetzlich wird noch jeder Punkt als
eigenes Objekt gezeichnet. Demnach koennen m*(n-1)+m*n Beschriftungen 
eingefuegt werden.

-#

plotlib::line := proc(l, c, t)
  local i, i1, j, j1, m, n;
begin
  m := nops(l); n := nops(l[1]);
  t := plotlib::set_titles(t, l, m*n+m*(n-1));
  plot2d(
   Scaling=UnConstrained,Ticks=min(max(2,n-2),10),Labeling=TRUE,Labels=["",""],Axes=Origin,
   ([Mode=List, [point(i,l[j][i+1],Color=c[j])],
     Title=t[i*m+j],TitlePosition=[0,0]
    ]$i=0..n-1)$j=1..m
   ,
   ([Mode=List,
      [polygon(point(i1,l[j1][i1+1]),point(i1+1,l[j1][i1+2]),Color=c[j1])],
     Title=t[(n+i1-1)*m+j1]]$i1=0..n-2)$j1=1..m
  )
end_proc:

plotlib::curve := proc(l, c, t)
  local i, j, m, n, x;
begin
  m := nops(l); n := nops(l[1]);
  t := plotlib::set_titles(t, l, m*n+m*(n-1));
  plot2d(
   Scaling=UnConstrained,Ticks=min(max(2,n-2),10),Labeling=TRUE,Labels=["",""],Axes=Origin,
   ([Mode=List, [point(i,l[j][i+1],Color=c[j])],
     Title=t[i*m+j],TitlePosition=[0,0]
    ]$i=0..n-1)$j=1..m
   ,
   [Mode=Curve, [x, lagrange([j$j=0..n-1],l[i],x)(x)],
                 x=[0,n-1],Color=[Flat,c[i]]
   ]$i=1..m
  )
end_proc:

plotlib::surface := proc(l, c, t)
  local j, m, n, x, y;
begin    
  m := nops(l); n := nops(l[1]);
  t := plotlib::set_titles(t, l, 1);
  plot3d(
   Scaling=UnConstrained,Labeling=FALSE,Axes=Box,
   [Mode=Surface, [x, y, lagrange([[j$j=0..m-1],[j$j=0..n-1]],l,[x,y])(x,y)],
                  x=[0,m-1], y=[0,n-1],Style=[HiddenLine,Mesh]
   ]
  )
end_proc:



#-

Rechtecke muessen durch zwei Polygone, die jeweils ein Dreieck bilden,
dargestellt werden, da derzeit nur Dreiecke mit einer Farbe gefuellt
werden koennen. Ein weiteres Polygon bildet den Rahmen des Rechtecks.
Ein Rechteck besteht daher aus drei Polygonen.

Das gesamte Saeulen- oder Balkendiagramm koennte durch ein Objekt dar-
gestellt werden. Man knnte dann aber nur zwei Beschriftungen eingeben,
eine fuer die Szene und eine fuer das Objekt. Aus diesem Grund wird
jedes Polygon als ein eigenes Objekt dargestellt. Demnach koennen
3*m*n+1 Beschriftungen eingefuegt werden.

-#


plotlib::column := proc(l, c, t)
  local i, j, m, n, nc, offset;
begin
  m := nops(l); n := nops(l[1]); nc := nops(c);
  offset := (if m = 1 then 0.2 else 1 end_if);
  t := plotlib::set_titles(t, l, 3*m*n);
  plot2d(
    Scaling=UnConstrained,Ticks=min(n,10),Labeling=TRUE,Labels=["",""],Axes=Origin,
    (plotlib::rectangle(
       j+i*(m+offset),0,j+1+i*(m+offset),l[j+1][i+1],
       (if m=1 and nc=n then c[i+1] else c[j+1] end_if),t[i*m+j+1],
       (if m=1 then 0.1,0.3,TRUE else null() end_if)
    )$j=0..m-1)$i=0..n-1
 )
end_proc:

plotlib::beam := proc(l, c, t)
  local i, j, m, n, nc, offset;
begin
  m := nops(l); n := nops(l[1]); nc := nops(c);
  offset := (if m = 1 then 0.2 else 1 end_if);
  t := plotlib::set_titles(t, l, 3*m*n);
  plot2d(Scaling=UnConstrained,Axes=None,
         [Mode=List,[polygon(point(0,0),point(0,(m+offset)*n-offset),Color=Black)]],
         (plotlib::rectangle(
           0,j+i*(m+offset),l[j+1][i+1],j+1+i*(m+offset),
           (if m=1 and nc=n then c[i+1] else c[j+1] end_if),t[i*m+j+1],
           (if m=1 then 0.3,0.1,FALSE else null() end_if)
         )$j=0..m-1)$i=0..n-1
 )
end_proc:

plotlib::plain := proc(l,c,t)
  local i, j, m, n;
begin
  m := nops(l); n := nops(l[1]);
  plot3d(Axes=Corner, Scaling=UnConstrained,
         [Mode=List,[polygon(point(i,j,l[j+1][i+1])$j=0..m-1)$i=0..n-1,
                     polygon(point(i,j,l[j+1][i+1])$i=0..n-1)$j=0..m-1
                    ]]
  )
end_proc:

plotlib::solid_plain := proc(l,c,t)
  local i, j, m, n;
begin
  m := nops(l); n := nops(l[1]);
  plot3d(Axes=Box, Scaling=UnConstrained,Ticks=0,
         [Mode=List,
           [((polygon(point(i,j,l[j+1][i+1]), point(i,j+1,l[j+2][i+1]),
                      point(i+1,j+1,l[j+2][i+2]), Color=c[j+1],Filled=TRUE),
              polygon(point(i,j,l[j+1][i+1]),point(i+1,j,l[j+1][i+2]),
                      point(i+1,j+1,l[j+2][i+2]), Color=c[j+1],Filled=TRUE),
              polygon(point(i,j,l[j+1][i+1]), point(i,j+1,l[j+2][i+1]),
                      point(i+1,j+1,l[j+2][i+2]), point(i+1,j,l[j+1][i+2]),
                      point(i,j,l[j+1][i+1]),Color=c[m])
            )$i=0..n-2)$j=0..m-2]]
  )
end_proc:




plotlib::histogram := proc(l, c, t)
  local i, j, m, n, offset;
begin
  m := nops(l); n := nops(l[1]);
  offset := (if m = 1 then 0.2 else 0.2 end_if);
  t := plotlib::set_titles(t, l, m*n);
  plot3d(Scaling=UnConstrained,Axes=None,
    (plotlib::cube(i+i*offset,j+j*offset,0,
          i+1+i*offset,j+1+j*offset,l[j+1][i+1],
          c[j+1],t[i*m+j+1]
         )$i=0..n-1
    )$j=0..m-1
 )
end_proc:

#------------------------------ Utilities -------------------------------------#

plotlib::set_titles := proc(t, l, n)
  local i, j, k, m;
begin
  if (k:=contains(t, Values)) <> 0 then
     unassign(t[k]);
     if testtype(l[1], DOM_LIST) then
        t := t.[(expr2text(l[i][j])$i=1..nops(l))$j=1..nops(l[1])]
     else
        t := t.[expr2text(l[i])$i=1..nops(l)]
     end_if
  end_if;
  m := nops(t);
  if m < n then t := t.[""$hold(i)=1..n-m] end_if;
  t
end_proc:

plotlib::rectangle := proc(a,b,c,d,co,ti,sx,sy,vertical)
begin
  [Mode=List,[polygon(point(a,b), point(a,d), point(c,d), Filled=TRUE,Color=co)],
   Title=ti,TitlePosition=[0,0]],
  [Mode=List,[polygon(point(a,b), point(c,b), point(c,d), Filled=TRUE,Color=co)]],
  (if args(0) = 6 then
     # Draw frame #
     [Mode=List,[polygon(point(a,b), point(a,d), point(c,d), point(c,b), point(a,b),
                 Color=Black)]]
   else
     # Draw shadow frame #
     if vertical then
        if d < 0 then sy := -sy; end_if;
        [Mode=List,
         [polygon(point(a+sx,b),point(a+sx,d+sy),point(c+sx,d+sy),Filled=TRUE,Color=Gray),
          polygon(point(a+sx,b),point(c+sx,b),point(c+sx,d+sy),Filled=TRUE,Color=Gray)]
        ]
     else
       if c < 0 then sx := -sx; end_if;
       [Mode=List,
         [polygon(point(a,b-sy),point(a,d-sy),point(c+sx,d-sy),Filled=TRUE,Color=Gray),
             polygon(point(a,b-sy),point(c+sx,b-sy),point(c+sx,d-sy),Filled=TRUE,Color=Gray)]
       ]
       end_if
  end_if)
end_proc:


plotlib::rectangle3d := proc(a,b,c,d,e,f,co,ti,x)
  local p1, p2;
begin
  
  if x then 
     p1 := a,e,f; p2 := d,b,c;
  else
     p1 := a,b,f; p2 := a,e,c
  end_if;

  [ Mode=List, [ 
       polygon(point(a,b,c), point(p1), point(d,e,f), Filled=TRUE, Color=co),
       polygon(point(a,b,c), point(p2), point(d,e,f), Filled=TRUE, Color=co),
       polygon(point(a,b,c), point(p1), point(d,e,f), point(p2), point(a,b,c),
               Color=Black)
     ], Title=ti, TitlePosition=[0,0]
  ]
end_proc:

plotlib::cube := proc(a,b,c,d,e,f,co,ti)
begin
  plotlib::rectangle3d(a,b,c,d,e,c,co,ti,TRUE),  # Bottom #
  plotlib::rectangle3d(a,b,f,d,e,f,co,ti,TRUE),  # Top    #
  plotlib::rectangle3d(a,b,c,d,b,f,co,ti,TRUE),  # Front  #
  plotlib::rectangle3d(a,e,c,d,e,f,co,ti,TRUE),  # Back   #
  plotlib::rectangle3d(a,b,c,a,e,f,co,ti,FALSE), # Left   #
  plotlib::rectangle3d(d,b,c,d,e,f,co,ti,FALSE)  # Right  #
end_proc:
