
/*
 Vector (1-dim. array): Mode = {Points, Line, Curve}
   A:=array(2..5,[2,5,3,9]): 
   plotlib::arrayplot(A, Curve): 
 Matrix (2-dim. array): Mode = {Points, Surface}
   A:=array(2..4,3..5,[[2,5,1],[3,9,11],[-1,5,0]]): 
   plotlib::arrayplot(A, Surface):
*/

plotlib::arrayplot := proc(A:DOM_ARRAY, mode)
   local d, i, j, m, ma, mb, n, na, nb, obj, p, x, xl, y, yl, zl;
begin
   d := op(A, [0,1]); m := op(A, [0,2]);
   ma := op(m, 1); mb := op(m, 2);
   xl := [i $ i = ma..mb];
   
   if d = 1 then
      yl := [A[i] $ i = ma..mb];
      obj := [Mode=List, [point(i,A[i]) $ i = ma..mb]];
      case mode 
        of Points do break;
        of Line   do
           obj := obj, [Mode=List, 
                        [polygon(point(i, A[i]), point(i+1, A[i+1])) $ i = ma..mb-1]];
           break;
        of Curve  do 
           obj := obj, [Mode=Curve, [x, lagrange(xl, yl, x)(x)], x=[ma,mb]];
           break;
        otherwise
          error("Unknown mode")
      end_case;

      plot2d(Scaling=UnConstrained,Ticks=[xl, yl],Labeling=TRUE,Labels=["",""],
             Axes=Origin,#AxesOrigin=[0,0],# obj)

   elif d = 2 then

      n := op(A, [0,3]);
      na := op(n, 1); nb := op(n, 2);
      yl := [j $ j = na..nb];
      zl := [(A[i,j] $ i = ma..mb) $ j = na..nb];
      obj := [Mode=List, [(point(i,j, A[i,j]) $ i = ma..mb) $ j = na..nb]];
      case mode 
        of Points do break;
        of Line   do
           obj := obj, [Mode=List, 
                        [polygon((point(i, j, A[i,j]) $ j = na..nb) $ i = ma..mb)]];
           break;
        of Surface do 
           p := lagrange([xl,yl],[[A[i,j]$j=na..nb]$i=ma..mb],[x,y]);
           obj := obj, [Mode=Surface, [x, y, p(x,y)],
                        x=[ma,mb], y=[na,nb],Style=[HiddenLine,Mesh]];
           break;
        otherwise
            error("Unknown mode")
      end_case;

      plot3d(Scaling=UnConstrained,Ticks=[xl, yl, zl],
             Labeling=TRUE,Labels=["","",""], Axes=Box, # AxesOrigin=[0,0,0], #
             obj)
   end_if
end_proc:

