#
| plotlib::fieldplot     - displays vectorfield in two variables
|
| args                   - <scene_options ,> object_1 <, object_2, ...>
|
|                        Hereby the <scene_options> describes the options
|                        which can be set in order to influence the whole
|                        graphics (like Scaling). These options are optio-
|                        nal. All options listed in the reference manual
|                        can be used here.
|
|                        object_i is used to specify the i-th object of
|                        the current graphics. This is to be given in form
|                        of a list:
|
|                        [expr_xy, x = [xmin, xmax], y = [ymin, ymax], 
|                             options]
|
|                        expr_xy - describes the field to be plotted
|                        x       - the first independent variable
|                        y       - the second independent variable
|                        xmin    - minimum value for the range in which x
|                                  is evaluated
|                        xmax    - maximum value for the range in which x
|                                  is evaluated
|                        ymin    - minimum value for the range in which 
|                                  y is evaluated
|                        ymax    - maximum value for the range in which 
|                                  y is evaluated
|                        options - used to influence the graphical represen-
|                                  tation of the object. All options listed
|                                  for plot3d() in the reference manual can
|                                  be used here.
#
plotlib::fieldplot := proc()
    local i, j, list, calc_pointlist, convf;
begin

    calc_pointlist := proc(list)
       local f_x, f_y, points, i, grad_x, grad_y,
             act_x, xmin, xmax, xgrid, delta_x,
             act_y, ymin, ymax, ygrid, delta_y,
    	        delta_xy, ratio, rat_x, rat_y, result;
    begin
        #
        | Check the arguments of list, left out
        #
        
        xmin    := float(op(list, [2,2,1])):
        xmax    := float(op(list, [2,2,2])):
        ymin    := float(op(list, [3,2,1])):
        ymax    := float(op(list, [3,2,2])):
        xgrid   := op(list, [4,2,1]):
        ygrid   := op(list, [4,2,2]):
        delta_x := (xmax - xmin)/(xgrid-1):
        delta_y := (ymax - ymin)/(ygrid-1):
        delta_xy:= min(delta_x,delta_y); // Walter, 18.5.99

        points  := []:
    
        f_x := convf(op(list, [1,1]), op(list, [2,1]), op(list, [3,1]));
        f_y := convf(op(list, [1,2]), op(list, [2,1]), op(list, [3,1]));

        for act_x from xmin to xmax step delta_x do
            for act_y from ymin to ymax step delta_y do
	        #
	        | Evaluate the expression expr_xy at the point
	        | (act_x, act_y).
	        #
             grad_x := float(f_x(act_x, act_y)); grad_y := float(f_y(act_x, act_y));

             ratio  := 2.0/(2.5*sqrt(grad_x^2+grad_y^2)):
             rat_x  := delta_xy*ratio*grad_x:
             rat_y  := delta_xy*ratio*grad_y: // Walter, 18.5.99
                                                      
             points := append(points, plotlib::vector(
                                        point(act_x, act_y),
                                        point(act_x+rat_x, act_y+rat_y),
                                        PI/10, 0.5
                                      )
                              );
	    end_for:
        end_for:
        result := [Mode = List, points, (list[i] $ i = 5..nops(list))]:
        return(result);
    end_proc:

    convf := proc(f, x, y)
        local _dx_, _dy_, _f_ ;
    begin
        if not contains({DOM_PROC, DOM_EXEC, DOM_FUNC_ENV}, domtype(f)) then 
           if contains({DOM_EXPR,DOM_IDENT,DOM_INT,DOM_RAT,DOM_FLOAT}, domtype(f)) then
              f := subsop(proc(_dx_, _dy_) begin _f_ end_proc, 
                          [1,1] = x, [1,2] = y, 4 = f)
           else
              error("Illegal argument: ".expr2text(f))
           end_if 
        end_if;
        f
    end_proc:
    
    if args(0) = 0 then 
        plot(Dimension = 3)
    else 
        for i from 1 to args(0) do 
            list[i]:=args(i); 
            #
            | Check whether args(i) is an object or a scene-option.
            #
            if testtype(list[i], DOM_LIST) 
               and op(list[i][1],1)<>Mode  // allow other objects such as
               then                        // [Mode = Curve, ...] to be processed
                                           // as usual by plot2d
	        list[i] := calc_pointlist(list[i]):
            end_if; 
        end_for; 
	plot2d(list[j] $ j = 1..nops(list))
    end_if
end_proc:

