# #
# $Date: 1999/05/17 16:02:12 $  $Author: olli $   $Revision: 1.4.2.2 $ #
# #
# jusche, 04/07/96 #

#++
	plotlib::polygonplot - grafische Ausgabe zweidimensionaler Polygonen

	Calling sequence:
	polygonplot(<scene-option, ...>, [polygon_1, <polygon_2,...> ],
	            <option> <,specification> )

            scene-option  ->  identifier=expr
            option        ->  [r,g,b]

	Parameter:
	identifier -- Bezeichner
	expr        -- Ausdruck
	polygon_i   -- durch polygon erzeugte 2-dim. Polygonprimitive
	r, g, b     -- reell evaluierbare Ausdr\"ucke zwischen 0.0 und 1.0
	specification-- Convex oder Concave



Summary:
polygonplot dient zur grafischen Ausgabe von zweidimensionalen Polygonen.
Dabei k"onnen beliebig viele Polygone zu einer Szene zusammengefa\"st und
dargestellt werden.
Mit Hilfe der Funktion polygonplot k\"onnen alle Arten von zweidimensio-
nalen Polygonen mit beliebig vielen Eckpunkten grafisch dargestellt werden.
Optional kann den Begrenzungslinien der Polygone eine Farbe zugewiesen
werden, wobei r, g und b reelle Werte zwischen 0.0 und 1.0 sein m\"ussenm,
die den Rot-, Gr\"un- bzw. Blau-Anteilen der gew\"unschten Farbe entsprechen.
Der Default-Wert f\"ur die Polygonr\"ander ist [0.0,0.0,1.0] an.   F\"ur die
Spezifizierung der Polygone kann optional Convex und Concave angegeben
werden.   zur Verf\"ugung.   Wird keine Spezifikation festgelegt,  werden die
Objekte als beliebige, z.B. sich selbst \"uberlappende Polygone betrachtet.
Falls ein Polygon "einfach" ist, d.h.  die Polygonseiten schneiden sich nicht
untereinander, wird hier die Option Concave gesetzt.
Ein Polygon ist genau dann Convex, wenn jede Verbindungslinie zwischen
zwei beliebigen Punkten des Polygons ebenfalls vollst\"andig im Innern dieses
Polygons liegt.
Wenn ein Polygon eindeutig Convex oder Concave ist, so sollte der entspre-
chende Ausdruck gesetzt werden, da sich hierdurch die Laufzeit der Funktion
polygonplot verk\"urzt. In unsicheren F\"allen sollte auf diese Angabe 
verzichtet werden, um eine korrekte F\"ullung der Objekte zu erhalten.
Vor der Beschreibung der Objekte mu\"s zuerst eine Sequenz von Optionen
f\"ur die gesamte Szene festgelegt werden, wobei eine einzelne Option wie 
folgt aufgebaut ist:

    identifier = expr

Hierbei ist identifier ein Bezeichner f\"ur die Option und expr ein Ausdruck,
der den Wert der Option angibt. Als  M\"oglichkeit  stehen  bei  der  
Szene-Beschreibung  zur  Verf\"ugung: PlotDevice, Title, TitlePosition, 
Axes, Ticks, Arrows, Labeling, ViewingBox, Scaling, FontFamily, 
FontStyle, FontSize, LineStyle, LineWidth, ForeGround, BackGround, 
PointWidth, PointStyle, Labels, AxesOrigin.
Bis auf die letzten beiden Optionen handelt es sich dabei um Optionen,
die  unabh\"angig  von  der  Dimension  der  Szene  verwendet  werden  
k\"onnen.  F\"ur  die  genauere  Auflistung  dieser  samt  ihrer  
zul\"assigen  Parameter und Defaultwerte sei auf Tabelle 1 des 
"MuPAD - User's Manual" im Abschnitt "Plot Options" verwiesen.  
Zus\"atzlich zu den dort aufgelisteten Optionen k\"onnen f\"ur 
2D-Szenen noch die Optionen in Tabelle 2 desselben Abschnitts
verwendet werden.
++#

plotlib::polygonplot:= domain("plotlib::polygonplot"):
alias( polygonplot = plotlib::polygonplot ):

################################################################
#                                                              #
# Prozedur: DOUBLE              Suchen nach doppeltem Punkt    #
#                                                              #
# ------------------------------------------------------------ #
# INPUT:  Liste mit Punkten des Polygons                       #
# OUTPUT: Liste ohne doppelte Punkte                           #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::double:=proc(polynew)
local i;
begin;
  i:=1;

  if nops(polynew)<>nops({op(polynew)}) then

    # 
    | Untersuche die Polygoneckpunkte und ihre jeweiligen
    | Nachfolgeeckpunkte eines Polygons auf Gleichheit, d.h.
    | aus Polygon (point(A),point(B),point(B),point(C))
    | wird Polygon(point(A),point(B),point(C))
    #
    while i < nops(polynew) do
      if polynew[i]=polynew[i+1] then
        polynew[i]:=NIL;  # durchlaufe Schleife nochmal #
      else
        i:=i+1;
      end_if;
    end_while;

    #
    | Untersuche den ersten und letzten Polygoneckpunkt
    | eines Polygons auf Gleichheit, d.h.
    | aus Polygon(point(A),point(B),point(C),point(A))
    | wird Polygon(point(A),point(B),point(C))
    #
    if polynew[1]=polynew[nops(polynew)] then
      polynew[1]:=NIL;
    end_if;
  end_if;

polynew;

end_proc:


################################################################
#                                                              #
# Prozedur: XSMALL              Suchen des kleinsten x-Wertes  #
#                               (Liste)                        #
# ------------------------------------------------------------ #
# INPUT:  Liste mit Punkten                                    #
# OUTPUT: "Kleinster" Punkt, Stelle in der Liste               #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::xsmall:=proc(points)
local i,xmin,j,l,n;
begin;

  j:=1;
  
  #
  | Suchen des Punktes mit dem kleinsten x-Wert in einer
  | Liste mit Punkten,
  | xmin ist jeweils der x-Wert des Punktes,
  | pmin ist der kleinste Punkt der Liste,
  | j ist die Stelle an der pmin innerhalb der Liste steht 
  #
  xmin:=op(points,[1,1]);
  n:=nops(points); 
  for i from 2 to n do 
    if (l:=op(points,[i,1])) < xmin then 
      j:=i;
      xmin:=l;
    end_if;
  end_for; 

[op(points,j),j];

end_proc:


################################################################
#                                                              #
# Prozedur: TRIANG              Konstruktion des Dreiecks      #
#                                                              #
# ------------------------------------------------------------ #
# INPUT:  Liste mit Punkten des Polygons                       #
#         Stelle an der "kleinster" Punkt in Polygon liegt,    #
# OUTPUT: Eckpunkte des abgetrennten Dreiecks,                 #
#         Liste mit Punkten des Polygons ohne Dreick           #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::triang:=proc(polynew,j)
local n,d1,d3,polyinter;
begin;

  #
  | polyinter ist jeweils Polygoneckpunktliste ohne die drei
  | Dreieckseckpunkte
  #

  # 
  | Falls der Polygoneckpunkt mit dem kleinsten x-Wert 
  | der erste Eckpunkt der Polygoneckpunktliste ist, dann sind
  | die beiden anderen Dreieckseckpunkte (d1,d3) der letzte bzw.
  | der zweite Polygoneckpunkt
  #
  n:=nops(polynew);
  if j=1 then
    d1:=op(polynew,n);
    d3:=op(polynew,2);
    polyinter:=[op(polynew,3..n-1)];

  #
  | Falls dieses Minimum der letzte Polygoneckpunkt ist, 
  | sind die anderen Dreieckseckpunkte (d1,d3) der vorletzte
  | und der erste Polygoneckpunkt
  #
  elif j=n then
    d3:=op(polynew,1);
    d1:=op(polynew,j-1);
    polyinter:=[op(polynew,2..n-2)];

  #
  | Falls der Minimalpunkt an j-ter Stelle liegt, liegen 
  | die anderen Dreieeckseckpunkte an (j-1)-ter und 
  | (j+1)-ter Stelle
  #
  else
    d3:=op(polynew,j+1);
    d1:=op(polynew,j-1);
    polyinter:=[op(polynew,1..j-2),
               op(polynew,j+2..n)];
  end_if;

[d1,d3,polyinter];

end_proc:

################################################################
#                                                              #
# Prozedur: INTER               Prueft das Dreieck, ob weitere #
#                               Polygoneckpunkte im Innern     #
#                               liegen koennten                #
# ------------------------------------------------------------ #
# INPUT:  Eckpunkte des Dreiecks,                              #
#         Liste mit restlichen Punkten des Polygons            #
# OUTPUT: Liste mit moeglichen Polygoneckpunkten im Innern     #
#         des Dreiecks                                         #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::inter:=proc(d1,d2,d3,polyinter)
local i,tri,max1,max2,min1;
begin;
  tri:=[];
  max1:=max(op(d1,1),op(d3,1));
  max2:=max(op(d1,2),op(d2,2),op(d3,2));
  min1:=min(op(d1,2),op(d2,2),op(d3,2));

  #
  | Einschr"ankung des Bereichs, in dem die Polygoneckpunkte 
  | liegen, die im Innern des Dreiecks liegen k"onnten, d.h.
  | x-Wert des Polygoneckpunktes <= max (x-Werte des Dreiecks),
  | y-Wert des Polygoneckpunktes <= max (y-Werte des Dreiecks),
  | y-Wert des Polygoneckpunktes >= min (y-Werte des Dreiecks),
  | ausserdem muss gelten Polygoneckpunkt <> Dreieckseckpunkt,
  | (sonst Endlosschleife m"oglich)
  | tri ist Liste mit Kandidaten f"ur im Innern des Dreiecks 
  | liegende Polygoneckpunkte
  #
  for i in polyinter do
    if op(i,1) <= max1 and 
       op(i,2) <= max2 and
       op(i,2) >= min1 and
       i <> d1 and i <> d3 then
      tri:=tri . [i];
    end_if;
  end_for;

tri;

end_proc:


################################################################
#                                                              #
# Prozedur: INSIDE              Prueft, ob moegliche Innen-    #
#                               punkte tatsaechlich im Innern  #
#                               des Dreiecks liegen            #
# ------------------------------------------------------------ #
# INPUT:  Eckpunkte des Dreiecks,                              #
#         Liste mit moeglichen Innenpunkten                    #
# OUTPUT: Liste mit tatsaechlichen Dreiecksinnenpunkten        #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::inside:=proc(d1,d2,d3,tri)
local i,j,k,tri1,tri2,tri3,xlin1,xlin2,d11,d12,d21,d22,d31,
      d32,t1,t2;
begin;

  d11:=op(d1,1);
  d12:=op(d1,2);
  d21:=op(d2,1);
  d22:=op(d2,2);
  d31:=op(d3,1); 
  d32:=op(d3,2);

  i:=1;
  tri1:=[];
  tri2:=[];
  tri3:=[];
  while i <= nops(tri) do

    #
    | Falls der y-Wert des betreffenden Punktes zwischen denen
    | von d1 und d3 und denen von d1 und d2 liegt, dann berechne
    | den x-Wert auf der Linie zwischen d1 u. d3 und den x-Wert 
    | auf der Linie zwischen d1 u. d2 mit dem y-Wert des
    | betreffenden Punktes, wenn der x-Wert des betreffenden
    | Punktes zwischen oder auf den errechneten x-Werten liegt, 
    | dann ist der Punkt tats"achlich im Innern des Dreiecks und
    | wird in die Liste tri1 geschrieben
    #
    t1:=op(tri[i],1);
    t2:=op(tri[i],2);
    if t2 <= max(d32,d12) and
       t2 >= min(d32,d12) and
       t2 <= max(d22,d12) and
       t2 >= min(d22,d12) and
       d32 <> d12 and  # sonst division by zero # 
       d22 <> d12 then 
    xlin1:=[((d31-d11)*(t2-d12)/
          (d32-d12)) + d11];
    xlin2:=[((d21-d11)*(t2-d12)/
          (d22-d12)) + d11];

      if (round(t1*10^DIGITS) <=
            max(round(op(xlin1)*10^DIGITS),
                round(op(xlin2)*10^DIGITS)) and
          round(t1*10^DIGITS) >=
            min(round(op(xlin1)*10^DIGITS),
                round(op(xlin2)*10^DIGITS))) then
        tri1:=[tri[i]] . tri1;
        tri[i]:=NIL;

      end_if;
    end_if;
    i:=i+1;
  end_while;

  j:=1;

  while j <= nops(tri) do

    #
    | Falls der y-Wert des betreffenden Punktes zwischen denen
    | von d1 und d2 und denen von d2 und d3 liegt, dann berechne
    | den x-Wert auf der Linie zwischen d1 u. d2 und den x-Wert 
    | auf der Linie zwischen d2 u. d3 mit dem y-Wert des
    | betreffenden Punktes, wenn der x-Wert des betreffenden
    | Punktes zwischen oder auf den errechneten x-Werten liegt, 
    | dann ist der Punkt tats"achlich im Innern des Dreiecks und
    | wird in die Liste tri2 geschrieben
    #
    t1:=op(tri[j],1);
    t2:=op(tri[j],2);
    if t2 <= max(d22,d12) and
       t2 >= min(d22,d12) and
       t2 <= max(d22,d32) and
       t2 >= min(d22,d32) and
       d22 <> d12 and  # sonst division by zero #
       d22 <> d32 then
    xlin1:=[((d21-d11)*(t2-d12)/
          (d22-d12)) + d11];
    xlin2:=[((d21-d31)*(t2-d32)/
          (d22-d32)) + d31];
      if (round(t1*10^DIGITS) <=
            max(round(op(xlin1)*10^DIGITS),
                round(op(xlin2)*10^DIGITS)) and
          round(t1*10^DIGITS) >=
            min(round(op(xlin1)*10^DIGITS),
                round(op(xlin2)*10^DIGITS))) then
        tri2:=[tri[j]] . tri2;
        tri[j]:=NIL;
      end_if;
    end_if;
    j:=j+1;
  end_while;

  k:=1;

  while k <= nops(tri) do

    #
    | Falls der y-Wert des betreffenden Punktes zwischen denen
    | von d2 und d3 und denen von d1 und d3 liegt, dann berechne
    | den x-Wert auf der Linie zwischen d2 u. d3 und den x-Wert 
    | auf der Linie zwischen d1 u. d3 mit dem y-Wert des
    | betreffenden Punktes, wenn der x-Wert des betreffenden
    | Punktes zwischen oder auf den errechneten x-Werten liegt, 
    | dann ist der Punkt tats"achlich im Innern des Dreiecks und
    | wird in die Liste tri3 geschrieben
    #
    t1:=op(tri[k],1);
    t2:=op(tri[k],2);
    if t2 <= max(d32,d22) and
       t2 >= min(d32,d22) and
       t2 <= max(d32,d12) and
       t2 >= min(d32,d12) and
       d32 <> d22 and  # sonst division by zero #
       d32 <> d12 then
    xlin1:=[((d31-d21)*(t2-d22)/
          (d32-d22)) + d21];
    xlin2:=[((d31-d11)*(t2-d12)/
          (d32-d12)) + d11];
      if (round(t1*10^DIGITS) <=
            max(round(op(xlin1)*10^DIGITS),
                round(op(xlin2)*10^DIGITS)) and
          round(t1*10^DIGITS) >=
            min(round(op(xlin1)*10^DIGITS),
                round(op(xlin2)*10^DIGITS))) then
        tri3:=[tri[k]] . tri3;
       tri[k]:=NIL;
      end_if;
    end_if;
    k:=k+1;
  end_while;

#
| Liste mit den Polygoneckpunkten, die im 
| Innern des Dreiecks liegen
#
tri1 . tri2 . tri3 ;

end_proc:

################################################################
#                                                              #
# Prozedur: POLPOL              Zerlegung des Polygons in zwei #
#                               Polygone                       #
# ------------------------------------------------------------ #
# INPUT:  Polygon in Liste,                                    #
#         "kleinster" Dreiecksinnenpunkt,                      #
#         Stelle des "kleinsten" Polygoneckpunktes             #
# OUTPUT: 2 Polygone in Liste                                  #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::polpol:=proc(polynew,triin,polyj)
local k,l,n,p,poly1,poly2;
begin;

  #
  | "polyj" ist die Stelle, die der Minimalpunkt des Polygons
  | innerhalb der Polygoneckpunktliste hat
  #

  #
  | Festhalten der Stelle, die der kleinste Polygonpunkt, der 
  | im Innern des Dreiecks liegt, innerhalb des Polygons hat
  | (hier = l)
  #
  n:=nops(polynew);
  for k from 1 to n do
    if (p:=op(polynew,k))=triin then
      l:=k;
    end_if;
  end_for;

  # 
  | Falls polyj = 1, bildet sich das erste Polygon aus dem
  | ersten bis l-ten Punkt des Ausganpolygons und das zweite
  | Polygon aus dem l-ten bis letzten und dem ersten Punkt 
  | des Ausgangpolygons
  #
  if polyj = 1 then                      
    poly1:=[op(polynew,1..l)];
    poly2:=[op(polynew,l..n),op(polynew,1)];

  #
  | Falls polyj = letzter Punkt, bildet sich das erste Polygon 
  | aus dem letzten und dem ersten bis l-ten Punkt des
  | Ausganpolygons und das zweite aus dem l-ten bis letzten 
  | Punkt des Ausgangpolygons
  #
  elif polyj = n then 
    poly1:=[op(polynew,n),op(polynew,1..l)];
    poly2:=[op(polynew,l..n)]; 
  else

  #
  | Falls l < polyj, bildet sich das erste Polygon aus dem
  | ersten bis l-ten und dem polyj-ten bis letzten Punkt des
  | Ausgangpolygons und das zweite aus dem l-ten bis polyj-ten 
  | Punkt
  #
    if l < polyj then
      poly1:=[op(polynew,1..l),
              op(polynew,polyj..n)];
      poly2:=[op(polynew,l..polyj)];

  #
  | Falls polyj < l, bildet sich das erste Polygon aus dem 
  | ersten bis polyj-ten und dem l-ten bis letzten Punkt des
  | Ausgangpolygons und das zweite aus dem polyj-ten bis l-ten 
  | Punkt
  #
    else
      poly1:=[op(polynew,1..polyj),
              op(polynew,l..n)];
      poly2:=[op(polynew,polyj..l)];
    end_if;
  end_if; 

[poly1,poly2];

end_proc:


################################################################
#                                                              #
# Prozedur: MAINTRI             Triangulieren eines einfachen, #
#                               konkaven Polygons bzw.         #
#                               Bearbeitung dieses Polygons    #
#                               zur grafischen Darstellung     #
#                               (Plotten)                      #
# ------------------------------------------------------------ #
# INPUT:  Liste mit den Punkten des Polygons,                  #
#         Farbe des Polygons                                   #
# OUTPUT: Trianguliertes Polygon in angegebener Farbe          #
#         (default-Wert ist rot),                              #
#         evtl. Polygonrand in angegebener Farbe               #
#         (default-Wert ist blau)                              #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::maintri:=proc(polypnt,colorin)
local d1,d2,d3,triin,tri,j,polyj,poly1,poly2,polynow,polyinter,
      pmin;
begin;

  #
  | Prozedur: XSMALL
  #
  polygonplot::xsmall(polypnt);
  pmin:=op(%,1);
  j:=op(%2,2);

  #
  | Festhalten der Stelle des Minimalpunktes des Polygons 
  | innerhalb der Eckpunktliste
  #
  polyj:=j;

  #
  | Prozedur: TRIANG
  #
  polynow:=polygonplot::triang(polypnt,j);
  d1:=polynow[1];
  d2:=op(polypnt,j);
  d3:=polynow[2];
  polyinter:=polynow[3];

  #
  | Prozedur: INTER
  #
  tri:=polygonplot::inter(d1,d2,d3,polyinter);

  #
  | Falls es Kandidaten f"ur im Dreieck liegende Punkte gibt
  #
  if nops(tri) > 0 then

    #
    | Prozedur: INSIDE
    #
    triin:=polygonplot::inside(d1,d2,d3,tri);

    # 
    | Falls Dreiecksinnenpunkt existiert
    #
    if nops(triin) > 0 then

      #
      | Falls es mehrere Dreiecksinnenpunkte gibt
      #
      if nops(triin) > 1 then

        #
        | Prozedur: XSMALL
        #
        polygonplot::xsmall(triin);
        pmin:=op(%,1);
        j:=op(%2,2);

        # 
        | triin ist kleinster Dreiecksinnenpunkt
        #
        triin:=pmin;
      else
        triin:=op(triin);
      end_if;

      #
      | Prozedur: POLPOL
      #
      polygonplot::polpol(polypnt,triin,polyj);
      poly1:=op(%,1);
      poly2:=op(%2,2);

    #
    | Falls kein Dreiecksinnenpunkt gefunden wurde, 
    | ist poly1 abzutrennendes Dreieck, poly2 restliches Polygon
    #
    else
      poly1:=[d1,d2,d3];
      polypnt[polyj]:=NIL;
      poly2:=polypnt;
    end_if;

  #
  | Falls kein Kandidat f"ur Dreiecksinnenpunkt vorhanden,
  | ist poly1 abzutrennendes Dreieck, poly2 restliches Polygon
  #
  else
    poly1:=[d1,d2,d3];
    polypnt[polyj]:=NIL;
    poly2:=polypnt;
  end_if;

  #
  | Falls poly1 oder poly2 mehr als 3 Eckpunkte => Rekursion
  #
  if (nops(poly1) = 3) and (nops(poly2) = 3) then
    return([polygon(op(poly1),Filled=TRUE,Color=colorin),
            polygon(op(poly2),Filled=TRUE,Color=colorin)]);
  elif nops(poly1) = 3 then
    return([polygon(op(poly1),Filled=TRUE,Color=colorin),
            op(polygonplot::maintri(poly2,colorin))]);
  elif nops(poly2) = 3 then
    return([op(polygonplot::maintri(poly1,colorin)),
            polygon(op(poly2),Filled=TRUE,Color=colorin)]);
  else
    return([op(polygonplot::maintri(poly1,colorin)),
            op(polygonplot::maintri(poly2,colorin))]);
  end_if;

end_proc:


################################################################
#                                                              #
# Prozedur: TRICONC             Triangulieren von einfachen,   #
#                               konkaven Polygonen bzw.        #
#                               Bearbeitung dieser Polygone    #
#                               zur grafischen Darstellung     #
#                               (Plotten)                      #
# ------------------------------------------------------------ #
# INPUT:  2d-Polygone in Liste, Farbe der Polygonraender       #
# OUTPUT: Triangulierte Polygone jeweils in angegebener Farbe  #
#         (default-Wert ist rot),                              #
#         evtl. Polygonraender in einer angegebenen Farbe      #
#         (default-Wert ist blau)                              #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::triconc:=proc(poly_list,color)
local pnt_list,n,contouri,three,i,k,colorin,plotout,pntlist,Nops;
begin;
  if args(0) = 1 then color := null() end_if;

  plotout:=[];

  for i from 1 to nops(poly_list) do

    #
    | Pruefung der Eingabedaten
    #
    if testargs() then
      if not (testtype(poly_list,DOM_LIST)) then
        error("Polygons in list expected")
      elif not (testtype(poly_list[i],DOM_POLYGON)) then
        error("Illegal argument [polygon]")
      elif nops(op(poly_list[i],1))-1 <> 2 then
        error("Invalid dimension for polygon")
      elif testtype(color,DOM_LIST) then
        if nops(color) <> 3 then
          error("Illegal color specification");
        end_if;
      end_if;
    end_if;  
  end_for;

  pnt_list:=poly_list;
  n:=nops(pnt_list);

  for i from 1 to n do

    #
    | "pntlist" ist Liste mit Punkten des Ausgangpolygons
    #
    Nops := nops(op(pnt_list[i]));
    pntlist:=[op(pnt_list[i],1..Nops-3)];

    #
    | Festlegen der Polygonkontur f"ur die Option Closed=TRUE.
    | Wurde eine Farbe angegeben, so gilt diese fuer alle Polygone
    | und ueberschreibt damit die Farbe des Polygons. Die Farbe
    | des Polygons darf nur dann als Kontourfarbe verwendet werden,
    | wenn keine Kontourfarbe explizit angegeben wurde. Falls weder
    | eine Polygon, noch eine Kontourfarbe angegeben wurde, ist die
    | default-Farbe der Kontour blau. 
    #

    if testtype(color,DOM_LIST) then
      contouri:=[polygon(op(pntlist . [pntlist[1],
                 Color=color]))];
    elif op(pnt_list[i], Nops-2) <> NIL then
      contouri:=[polygon(op(pntlist . [pntlist[1],
                 Color=op(pnt_list[i], Nops-2)]))];
    else
      contouri:=[polygon(op(pntlist . [pntlist[1],
                 Color=[0,0,1]]))];
    end_if;

    #
    | Falls Option Filled=FALSE, dann ist keine Triangulierung
    | notwendig, also kann sofort "geplottet" werden.
    | Bei Option Closed=TRUE wird Polygonkontur "geplottet",
    | bei Option Closed=FALSE wird Ausgangpolygon "geplottet"
    #
    if op(poly_list[i],nops(op(poly_list[i])))=FALSE then
      if op(poly_list[i],nops(op(poly_list[i]))-1)=TRUE then
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
        plotout:=contouri . plotout;
      else
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
        plotout:=[poly_list[i]] . plotout;
      end_if;

    else
  
      #
      | Umwandeln aller Koordinaten in float-Punkte, da bei
      | Relationen f"ur die Operanden der gleiche numerische
      | Wert vorausgesetzt werden muss
      #
      for k from 1 to nops(pntlist) do
        pntlist[k]:=point(float(op(pntlist[k],1)),
                          float(op(pntlist[k],2)));
      end_for;

      #
      | Festlegen der Polygoninnenfarbe
      #
      if op(poly_list[i],nops(op(poly_list[i]))-2) = NIL then
        colorin:=[1,0,0];
      else 
        colorin:=op(poly_list[i],nops(op(poly_list[i]))-2);
      end_if;


      #
      | Falls das Polygon nur drei Eckpunkte hat, 
      | gehe "uber zum "Plotten"
      #
      if nops(pntlist) = 3 then

        #
        | "Plotte" das Polygon mit Kontur, 
        | falls Option Closed=TRUE, sonst ohne Kontur
        #
        if op(poly_list[i],nops(op(poly_list[i]))-1)=TRUE then
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
          plotout:=contouri . 
                   [polygon(op(pntlist),Filled=TRUE,
                   Color=colorin)] .  plotout;
        else
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
          plotout:=[polygon(op(pntlist),Filled=TRUE,
                   Color=colorin)] .  plotout;
        end_if;

    #
    | Falls das Polygon weniger als drei Eckpunkte hat, 
    | "plotte" die Kontur des Ausgangpolygons
    #
    elif nops(pntlist) <= 2 then
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
      plotout:=contouri . plotout;
     
    else

      #
      | Prozedur: MAINTRI
      #
      three:=polygonplot::maintri(pntlist,colorin);
  
      #
      | "Plotte" das Polygon in der default-Farbe mit Kontur,
      | falls Option Closed=TRUE, sonst ohne Kontur
      #
      if op(poly_list[i],nops(op(poly_list[i]))-1)=TRUE then
        plotout:=contouri . three . plotout;
      else
        plotout:=three  . plotout;
      end_if;

      end_if;
    end_if;
  end_for;

plotout;

end_proc:


################################################################
#                                                              #
# Prozedur: TRISELF             Triangulieren eines einfachen, #
#                               konkaven Polygons bzw.         #
#                               Bearbeitung dieses Polygons    #
#                               zur grafischen Darstellung     #
#                               (Plotten)                      #
# ------------------------------------------------------------ #
# INPUT:  Liste mit den Punkten des Polygons,                  #
#         Farbe des Polygons                                   #
# OUTPUT: Trianguliertes Polygon in angegebener Farbe          #
#         (default-Wert ist rot),                              #
#         evtl. Polygonrand in angegebener Farbe               #
#         (default-Wert ist blau)                              #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::triself:=proc(polypnt,colorin)
local d1,d2,d3,triin,tri,j,polyj,poly1,poly2,polynow,polyinter,
      pmin;
begin; 

  #
  | Prozedur: XSMALL
  #
  polygonplot::xsmall(polypnt);
  pmin:=op(%,1);
  j:=op(%2,2);

  #
  | Festhalten der Stelle des Minimalpunktes des Polygons 
  | innerhalb der Eckpunktliste
  #
  polyj:=j;

  #
  | Prozedur: TRIANG
  #
  polynow:=polygonplot::triang(polypnt,j);
  d1:=polynow[1];
  d2:=op(polypnt,j);
  d3:=polynow[2];
  polyinter:=polynow[3];

  #
  | Prozedur: INTER
  #
  tri:=polygonplot::inter(d1,d2,d3,polyinter);

  #
  | Falls es Kandidaten f"ur im Dreieck liegende Punkte gibt
  #
  if nops(tri) > 0 then

    #
    | Prozedur: INSIDE
    #
    triin:=polygonplot::inside(d1,d2,d3,tri);

    # 
    | Falls Dreiecksinnenpunkt existiert
    #
    if nops(triin) > 0 then

      #
      | Falls es mehrere Dreiecksinnenpunkte gibt
      #
      if nops(triin) > 1 then

        #
        | Prozedur: XSMALL
        #
        polygonplot::xsmall(triin);
        pmin:=op(%,1);
        j:=op(%2,2);

        # 
        | triin ist kleinster Dreiecksinnenpunkt
        #
        triin:=pmin;
      else
        triin:=op(triin);
      end_if;

      #
      | Prozedur: POLPOL
      #
      polygonplot::polpol(polypnt,triin,polyj);
      poly1:=op(%,1);
      poly2:=op(%2,2);

    #
    | Falls kein Dreiecksinnenpunkt gefunden wurde, 
    | ist poly1 abzutrennendes Dreieck, poly2 restliches Polygon
    #
    else
      poly1:=[d1,d2,d3];
      polypnt[polyj]:=NIL;
      poly2:=polypnt;
    end_if;

  #
  | Falls kein Kandidat f"ur Dreiecksinnenpunkt vorhanden,
  | ist poly1 abzutrennendes Dreieck, poly2 restliches Polygon
  #
  else
    poly1:=[d1,d2,d3];
    polypnt[polyj]:=NIL;
    poly2:=polypnt;
  end_if;

  #
  | Falls poly1 oder poly2 mehr als 3 Eckpunkte => Rekursion
  #
  if (nops(poly1) = 3) and (nops(poly2) = 3) then
    return([polygon(op(poly1),Filled=TRUE,Color=colorin),
            polygon(op(poly2),Filled=TRUE,Color=colorin)]);
  elif nops(poly1) = 3 then
    return([polygon(op(poly1),Filled=TRUE,Color=colorin),
            op(polygonplot::triself(poly2,colorin))]);
  elif nops(poly2) = 3 then
    return([op(polygonplot::triself(poly1,colorin)),
            polygon(op(poly2),Filled=TRUE,Color=colorin)]);
  else
    return([op(polygonplot::triself(poly1,colorin)),
            op(polygonplot::triself(poly2,colorin))]);
  end_if;

end_proc:


################################################################
#                                                              #
# Prozedur: SECT                Schnittpunkt der Polygonseiten #
#                                                              #
# ------------------------------------------------------------ #
# INPUT:  Stelle der Eckpunktliste in Liste,                   #
#         je zwei Eckpunkte von zwei Polygonkanten,            #
#         aktuelles Polygon                                    #
# OUTPUT: Schnittpunkt der Geraden,                            #
#         erneuerte Eckpunktliste                              #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::sect:=proc(j,k,s11,s12,s21,s22,polyList)
local s111,s112,s121,s122,s211,s212,s221,s222,m1,m2,b1,b2,
      xsect,ysect,sectp;
begin;

  s111:=op(s11,1);
  s112:=op(s11,2);
  s121:=op(s12,1);
  s122:=op(s12,2);
  s211:=op(s21,1);
  s212:=op(s21,2);
  s221:=op(s22,1);
  s222:=op(s22,2);

  #
  | Falls beide Geraden, die durch die Eckpunkte verlaufen,
  | vertikal sind, gibt es keinen Schnittpunkt
  #
  if s121=s111 then 
    if s221=s211 then
      return( FAIL );
    
    #
    | Falls nur die erste Gerade vertikal verl"auft,
    | ist der x-Wert des Schnittpunktes = x-Wert der Vertikalen
    | und der y-Wert wird berechnet
    #
    else
      xsect:=s111;
      m2:=(s222-s212)/(s221-s211);
      b2:=s212-m2*s211;
      ysect:=m2*s111+b2;
      ysect:=float(round((ysect)*10^DIGITS)/10^DIGITS);
    end_if;

  #
  | Falls nur die zweite Gerade vertikal verl"auft,
  | ist der x-Wert des Schnittpunktes = x-Wert der Vertikalen
  | und der y-Wert wird berechnet
  #
  elif s221=s211 then
    xsect:=s211;
    m1:=(s122-s112)/(s121-s111);
    b1:=s112-m1*s111;
    ysect:=m1*s211+b1;
    ysect:=float(round((ysect)*10^DIGITS)/10^DIGITS);
 
  #
  | Falls keine der beiden Geraden vertikal verlaufen,
  | gibt es keinen Schnittpunkt, wenn sie parallel sind,
  | sonst wird der Schnittpunkt berechnet
  #
  else
    m1:=(s122-s112)/(s121-s111);
    m2:=(s222-s212)/(s221-s211);
    if m1=m2 then 
      return( FAIL );
    else
      b1:=s112-m1*s111;
      b2:=s212-m2*s211;
      xsect:=(b2-b1)/(m1-m2);
      xsect:=float(round((xsect)*10^DIGITS)/10^DIGITS);
      ysect:=m1*xsect+b1;
      ysect:=float(round((ysect)*10^DIGITS)/10^DIGITS);
    end_if;
  end_if;

  #
  | Falls der Schnittpunkt der Geraden zugleich Schnittpunkt
  | der Polygonkanten ist, deklariere ihn jeweils als
  | zus"atzlichen Polygoneckpunkt auf beiden Polygonkanten,
  | sonst gibt es keinen Schnittpunkt
  #
  if float(trunc((ysect)*10^(DIGITS-1))/10^(DIGITS-1)) >= 
     float(trunc((max(min(s112,s122),
    min(s212,s222)))*10^(DIGITS-1))/10^(DIGITS-1)) and
     float(trunc((ysect)*10^(DIGITS-1))/10^(DIGITS-1)) <= 
     float(trunc((min(max(s112,s122),
    max(s212,s222)))*10^(DIGITS-1))/10^(DIGITS-1)) and
     float(trunc((xsect)*10^(DIGITS-1))/10^(DIGITS-1)) >=
     float(trunc((max(min(s111,s121),
    min(s211,s221)))*10^(DIGITS-1))/10^(DIGITS-1)) and
     float(trunc((xsect)*10^(DIGITS-1))/10^(DIGITS-1)) <= 
     float(trunc((min(max(s111,s121),
    max(s211,s221)))*10^(DIGITS-1))/10^(DIGITS-1))
  then
       sectp:=point(xsect,ysect);
       [[op(polyList,1..j),sectp,op(polyList,j+1..k),sectp,
                  op(polyList,k+1..nops(polyList))],
       sectp]
  else
       FAIL
  end_if;

end_proc:


################################################################
#                                                              #
# Prozedur: MAINTRISELF         Triangulieren von komplexen    #
#                               Polygonen bzw.                 #
#                               Bearbeitung dieser Polygone    #
#                               zur grafischen Darstellung     #
#                               (Plotten)                      #
# ------------------------------------------------------------ #
# INPUT:  2d-Polygone in Liste, Farbe der Polygonraender       #
# OUTPUT: Triangulierte Polygone jeweils in angegebener Farbe  #
#         (default-Wert ist rot),                              #
#         evtl. Polygonraender in einer angegebenen Farbe      #
#         (default-Wert ist blau)                              #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::maintriself:=proc(poly_list,color)
local pnt_list,sectp,contouri,i,j,k,three,newlist,
      polyi,plotout,pntlist,n,colorin,Nops;
begin;
  if args(0) = 1 then color := null() end_if;

  plotout:=[];

  for i from 1 to nops(poly_list) do

    #
    | Pruefung der Eingabedaten
    #
    if testargs() then
      if not (testtype(poly_list,DOM_LIST)) then
        error("Polygons in list expected")
      elif not (testtype(poly_list[i],DOM_POLYGON)) then
        error("Illegal argument [polygon]")
      elif nops(op(poly_list[i],1))-1 <> 2 then
        error("Invalid dimension for polygon")
      elif testtype(color,DOM_LIST) then
        if nops(color) <> 3 then
          error("Illegal color specification");
        end_if;
      end_if;
    end_if;  
  end_for;

  pnt_list:=poly_list;
  n:=nops(pnt_list);

  for i from 1 to n do

    #
    | "pntlist" ist Liste mit Punkten des Ausgangpolygons
    #
    Nops := nops(op(pnt_list[i]));
    pntlist:=[op(pnt_list[i],1..Nops-3)];

    #
    | Festlegen der Polygonkontur f"ur die Option Closed=TRUE.
    | Wurde eine Farbe angegeben, so gilt diese fuer alle Polygone
    | und ueberschreibt damit die Farbe des Polygons. Die Farbe
    | des Polygons darf nur dann als Kontourfarbe verwendet werden,
    | wenn keine Kontourfarbe explizit angegeben wurde. Falls weder
    | eine Polygon, noch eine Kontourfarbe angegeben wurde, ist die
    | default-Farbe der Kontour blau. 
    #

    if testtype(color,DOM_LIST) then
      contouri:=[polygon(op(pntlist . [pntlist[1],
                 Color=color]))];
    elif op(pnt_list[i], Nops-2) <> NIL then
      contouri:=[polygon(op(pntlist . [pntlist[1],
                 Color=op(pnt_list[i], Nops-2)]))];
    else
      contouri:=[polygon(op(pntlist . [pntlist[1],
                 Color=[0,0,1]]))];
    end_if;

    #
    | Falls Option Filled=FALSE, dann ist keine Triangulierung
    | notwendig, also kann sofort geplottet werden.
    | Bei Option Closed=TRUE wird Polygonkontur geplottet,
    | bei Option Closed=FALSE wird Ausgangpolygon geplottet
    #
    if op(poly_list[i],nops(op(poly_list[i])))=FALSE then
      if op(poly_list[i],nops(op(poly_list[i]))-1)=TRUE then
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
        plotout:=contouri . plotout;
      else
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
        plotout:=[poly_list[i]] . plotout;
      end_if;

    else

      #
      | Umwandeln aller Koordinaten in float-Punkte, da bei
      | Relationen f"ur die Operanden der gleiche numerische
      | Wert vorausgesetzt werden muss
      #
      for k from 1 to nops(pntlist) do
        pntlist[k]:=point(float(op(pntlist[k],1)),
                          float(op(pntlist[k],2)));
      end_for;

      #
      | Prozedur: DOUBLE
      #
      pntlist:=polygonplot::double(pntlist);

      #
      | Festlegen der Polygoninnenfarbe
      #
      if op(poly_list[i],nops(op(poly_list[i]))-2) = NIL then
        colorin:=[1,0,0];
      else 
        colorin:=op(poly_list[i],nops(op(poly_list[i]))-2);
      end_if;

      #
      | Falls das Polygon nur drei Eckpunkte hat, 
      | gehe "uber zum "Plotten"
      #
      if nops(pntlist) = 3 then

        #
        | "Plotte" das Polygon mit Kontur, 
        | falls Option Closed=TRUE, sonst ohne Kontur
        #
        if op(poly_list[i],nops(op(poly_list[i]))-1)=TRUE then
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
          plotout:=contouri . 
                   [polygon(op(pntlist),Filled=TRUE,
                   Color=colorin)] .  plotout;
        else
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
          plotout:=[polygon(op(pntlist),Filled=TRUE,
                   Color=colorin)] .  plotout;
        end_if;

      #
      | Falls das Polygon weniger als drei Eckpunkte hat, 
      | "plotte" die Kontur des Ausgangpolygons
      #
      elif nops(pntlist) <= 2 then
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
        plotout:=contouri . plotout;
     

      #
      | Falls das Polygon mehr als drei Eckpunkte hat, 
      | gehe "uber zum Suchen nach eventuellen Schnittpunkten
      #
      else
  
        #
        | Teste, ob die Polygonkante zwischen dem ersten und dem
        | zweiten Eckpunkt (|1,2|) die Polygonkante zwischen dem
        | dritten und dem vierten Eckpunkt (|3,4|) schneidet bis
        | zur Polygonkante |1,2| und der Polygonkante zwischen 
        | dem vorletzten und dem letzten Eckpunkt
        #
        j:=1;
        for k from 3 to nops(pntlist)-1 do

          if op(pntlist,j)<>op(pntlist,k) and 
             op(pntlist,j)<>op(pntlist,k+1) and
             op(pntlist,j+1)<>op(pntlist,k) and 
             op(pntlist,j+1)<>op(pntlist,k+1) then

            #
            | Prozedur: SECT
            #
            sectp:= polygonplot::sect(j,k,op(pntlist,j),op(pntlist,j+1),
                        op(pntlist,k),op(pntlist,k+1),pntlist);
            if sectp <> FAIL then
		pntlist:= sectp[1];
                sectp:= sectp[2];
            end_if
          end_if
        end_for;

        #
        | Teste, ob die Polygonkante |2,3| die Kante |4,5|
        | schneidet bis zur Kante zwischen |2,3| und
        | |letzter Punkt, 1|, weiter mit Kante |3,4| und |5,6|
        | bis |3,4| und |letzter Punkt,1|, usw. bis Kanten
        | |drittletzter Punkt, vorletzter Punkt| und
        | |letzter Punkt, 1| getestet sind
        #
        j:=2;
        newlist:=pntlist . [pntlist[1]];
        while j <= nops(newlist)-3 do
          for k from j+2 to nops(newlist)-1 do
           # if op(newlist,k+1) = FAIL then
              pntlist[k+1]:=op(pntlist,1)
            end_if;#
            if op(newlist,j)<>op(newlist,k) and
               op(newlist,j)<>op(newlist,k+1) and
               op(newlist,j+1)<>op(newlist,k) and
               op(newlist,j+1)<>op(newlist,k+1) then

              #
              | Prozedur: SECT
              #
              sectp:=polygonplot::sect(j,k,op(newlist,j),op(newlist,j+1),
                     op(newlist,k),op(newlist,k+1),newlist);
              if sectp <> FAIL then
                newlist:=sectp[1]:
                sectp:=sectp[2]:
              end_if;
            end_if;
          end_for;
          j:=j+1;
        end_while;
        pntlist:=[op(newlist,1..nops(newlist)-1)];

        #
        | Prozedur: TRISELF
        #
        three:=polygonplot::triself(pntlist,colorin);

      #
      | "Plotte" das Polygon in der default-Farbe mit Kontur,
      | falls Option Closed=TRUE, sonst ohne Kontur
      #
      if op(poly_list[i],nops(op(poly_list[i]))-1)=TRUE then
        plotout:=contouri . three . plotout;
      else
        plotout:=three . plotout;
      end_if;

      end_if;
    end_if;
  end_for;

  plotout
end_proc:


################################################################
#                                                              #
# Prozedur: TRICONV_NEU         Triangulieren von konvexen     #
#                               Polygonen bzw. Bearbeitung     #
#                               dieser Polygone zur grafischen #
#                               Darstellung (Plotten)          #
# ------------------------------------------------------------ #
# INPUT:  2d-Polygone in Liste, Farbe der Polygonraender       #
# OUTPUT: Triangulierte Polygone jeweils in angegebener Farbe  #
#         (default-Wert ist rot),                              #
#         evtl. Polygonraender in einer angegebenen Farbe      #
#         (default-Wert ist blau)                              #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::triconv_neu:=proc(poly_list,color)
local pnt_list,n,i,k,l,three,contouri,pntlist,plotout,colorin,Nops;
begin;
  if args(0) = 1 then color := null() end_if;

  plotout:=[];

  for i from 1 to nops(poly_list) do

    #
    | Pruefung der Eingabedaten
    #
    if testargs() then
      if not (testtype(poly_list,DOM_LIST)) then
        error("Polygons in list expected")
      elif not (testtype(poly_list[i],DOM_POLYGON)) then
        error("Illegal argument [polygon]")
      elif nops(op(poly_list[i],1))-1 <> 2 then
        error("Invalid dimension for polygon")
      elif testtype(color,DOM_LIST) then
        if nops(color) <> 3 then
          error("Illegal color specification");
        end_if;
      end_if;
    end_if;  
  end_for;

  pnt_list:=poly_list;
  n:=nops(pnt_list);

  for i from 1 to n do

    #
    | "pntlist" ist Liste mit Punkten des Ausgangpolygons
    #
    Nops := nops(op(pnt_list[i]));
    pntlist:=[op(pnt_list[i],1..Nops-3)];

    #
    | Festlegen der Polygonkontur f"ur die Option Closed=TRUE.
    | Wurde eine Farbe angegeben, so gilt diese fuer alle Polygone
    | und ueberschreibt damit die Farbe des Polygons. Die Farbe
    | des Polygons darf nur dann als Kontourfarbe verwendet werden,
    | wenn keine Kontourfarbe explizit angegeben wurde. Falls weder
    | eine Polygon, noch eine Kontourfarbe angegeben wurde, ist die
    | default-Farbe der Kontour blau. 
    #

    if testtype(color,DOM_LIST) then
      contouri:=[polygon(op(pntlist . [pntlist[1],
                 Color=color]))];
    elif op(pnt_list[i], Nops-2) <> NIL then
      contouri:=[polygon(op(pntlist . [pntlist[1],
                 Color=op(pnt_list[i], Nops-2)]))];
    else
      contouri:=[polygon(op(pntlist . [pntlist[1],
                 Color=[0,0,1]]))];
    end_if;

    #
    | Falls Option Filled=FALSE, dann ist keine Triangulierung
    | notwendig, also kann sofort "geplottet" werden,
    | bei Option Closed=TRUE wird Polygonkontur "geplottet",
    | bei Option Closed=FALSE wird Ausgangpolygon "geplottet"
    #
    if op(poly_list[i],nops(op(poly_list[i])))=FALSE then
      if op(poly_list[i],nops(op(poly_list[i]))-1)=TRUE then
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
        plotout:=contouri . plotout;
      else
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
        plotout:=[poly_list[i]] . plotout;
      end_if;

    else

      #
      | Umwandeln aller Koordinaten in float-Punkte, da bei
      | Relationen f"ur die Operanden der gleiche numerische
      | Wert vorausgesetzt wird
      #
      for k from 1 to nops(pntlist) do
        pntlist[k]:=point(float(op(pntlist[k],1)),
                          float(op(pntlist[k],2)));
      end_for;

      #
      | Festlegen der Polygoninnenfarbe
      #
      if op(poly_list[i],nops(op(poly_list[i]))-2) = NIL then
        colorin:=[1,0,0];
      else 
        colorin:=op(poly_list[i],nops(op(poly_list[i]))-2);
      end_if;

      #
      | Falls das Polygon nur drei Eckpunkte hat, 
      | gehe "uber zum "Plotten"
      #
      if nops(pntlist) = 3 then

        #
        | "Plotte" das Polygon mit Kontur, 
        | falls Option Closed=TRUE, sonst ohne Kontur
        #
        if op(poly_list[i],nops(op(poly_list[i]))-1)=TRUE then
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
          plotout:=contouri . 
                   [polygon(op(pntlist),Filled=TRUE,
                   Color=colorin)] .  plotout;
        else
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
          plotout:=[polygon(op(pntlist),Filled=TRUE, 
                   Color=colorin)] .  plotout;
        end_if;

      #
      | Falls das Polygon weniger als drei Eckpunkte hat, 
      | "plotte" die Kontur des Ausgangpolygons
      #
      elif nops(pntlist) <= 2 then
### WICHTIG !!! hier: zuerst das zuletzt zu plottende Objekt ###
        plotout:=contouri . plotout;
      else

        # 
        | Falls das bearbeitete Polygon mehr als drei Eckpunkte
        | hat, gehe "uber zum Triangulieren
        #
        three:=[];
        for l from 3 to nops(pntlist) do
          three:=three . [polygon(op(pntlist,1),
                 op(pntlist,l-1),
                 op(pntlist,l),Filled=TRUE,Color=colorin)];
        end_for;

        #
        | "Plotte" das Polygon in der default-Farbe mit Kontur,
        | falls Option Closed=TRUE, sonst ohne Kontur
        #
        if op(poly_list[i],nops(op(poly_list[i]))-1)=TRUE then
          plotout:=contouri . three . plotout;
        else
          plotout:=three . plotout;
        end_if;
      end_if;
    end_if;
  end_for;

plotout;

end_proc:


################################################################
#                                                              #
# HAUPT-PROZEDUR: POLYGONPLOT   Plotten von beliebigen,        #
#                               zweidimensionalen Polygonen    #
# ------------------------------------------------------------ #
# INPUT:  Scene-options,                                       #
#         2d-Polygone in Liste,                                #
#         Farbe der Polygonr"ander,                            #
#         Polygonspezifizierung                                #
# OUTPUT: Rueckgabewert ist NULL                               #
# ------------------------------------------------------------ #
#                                                              #
polygonplot::new:=proc(n)
local i,k,inputList,polygonlist,options,color;
begin;
inputList:=[args()];
polygonlist:=FALSE;
i:=1;

  #
  | Suchen innerhalb der Benutzer-Eingabe nach der Polygonliste
  #
  while i <= nops(inputList) do
    if testtype(op(inputList,i),DOM_LIST) then
      k:=i;
      polygonlist:=inputList[k];
      i:=nops(inputList)+1;
    else
      i:=i+1;
    end_if;
 end_while;
  if polygonlist=FALSE then
    error("List expected");
  elif not testtype(polygonlist[1],DOM_POLYGON) then
    error("Illegal argument [polygon]")
  end_if;

  #
  | Suchen innerhalb der Benutzer-Eingabe nach den 
  | Szene-Optionen
  #
  if k<>1 then
    options:=op(inputList,1..k-1);
  else
    options:=null();
  end_if;

  #
  | Suchen innerhalb der Benutzer-Eingabe nach der 
  | Polygonkontur-Farbe
  #
  if testtype(op(inputList,k+1),DOM_LIST) then
    color:=inputList[k+1];
  elif testtype(op(inputList,k+2),DOM_LIST) then
    color:=inputList[k+2];
  else
    color:=null();
  end_if;

  #
  | Suchen innerhalb der Benutzer-Eingabe nach der 
  | Polygonspezifikation
  #
  if not testtype(op(inputList,nops(inputList)),DOM_LIST) then
    if inputList[nops(inputList)]=Convex then
      plot2d(options,[Mode=List,
             polygonplot::triconv_neu(polygonlist,color)]);
    elif inputList[nops(inputList)]=Concave then
      plot2d(options,[Mode=List,polygonplot::triconc(polygonlist,color)]);
    else
      error("Illegal polygon specification")
    end_if;
  elif not testtype(op(inputList,nops(inputList)-1),DOM_LIST) and 
       nops(inputList) > 2 then
    if inputList[nops(inputList)-1]=Convex then
      plot2d(options,[Mode=List,
             polygonplot::triconv_neu(polygonlist,color)]);
    elif inputList[nops(inputList)-1]=Concave then
      plot2d(options,[Mode=List,polygonplot::triconc(polygonlist,color)]);
    else
      error("Illegal polygon specification")
    end_if;
  else
    plot2d(options,[Mode=List,polygonplot::maintriself(polygonlist,color)]);
  end_if;

end_proc:

# PlotScene  --  like 'new' but returns the scene without plotting it #
polygonplot::PlotScene:=proc(n)
local i,k,inputList,polygonlist,options,color;
begin;
inputList:=[args()];
polygonlist:=FALSE;
i:=1;

  #
  | Suchen innerhalb der Benutzer-Eingabe nach der Polygonliste
  #
  while i <= nops(inputList) do
    if testtype(op(inputList,i),DOM_LIST) then
      k:=i;
      polygonlist:=inputList[k];
      i:=nops(inputList)+1;
    else
      i:=i+1;
    end_if;
 end_while;
  if polygonlist=FALSE then
    error("List expected");
  elif not testtype(polygonlist[1],DOM_POLYGON) then
    error("Illegal argument [polygon]")
  end_if;

  #
  | Suchen innerhalb der Benutzer-Eingabe nach den 
  | Szene-Optionen
  #
  if k<>1 then
    options:=op(inputList,1..k-1);
  else
    options:=null();
  end_if;

  #
  | Suchen innerhalb der Benutzer-Eingabe nach der 
  | Polygonkontur-Farbe
  #
  if testtype(op(inputList,k+1),DOM_LIST) then
    color:=inputList[k+1];
  elif testtype(op(inputList,k+2),DOM_LIST) then
    color:=inputList[k+2];
  else
    color:=null();
  end_if;

  #
  | Suchen innerhalb der Benutzer-Eingabe nach der 
  | Polygonspezifikation
  #
  if not testtype(op(inputList,nops(inputList)),DOM_LIST) then
    if inputList[nops(inputList)]=Convex then
      return( options,[Mode=List,
             polygonplot::triconv_neu(polygonlist,color)]
      );
    elif inputList[nops(inputList)]=Concave then
      return( options,[Mode=List,polygonplot::triconc(polygonlist,color)] );
    else
      error("Illegal polygon specification")
    end_if;
  elif not testtype(op(inputList,nops(inputList)-1),DOM_LIST) and 
       nops(inputList) > 2 then
    if inputList[nops(inputList)-1]=Convex then
      return(options,[Mode=List,
             polygonplot::triconv_neu(polygonlist,color)]);
    elif inputList[nops(inputList)-1]=Concave then
      return(options,[Mode=List,polygonplot::triconc(polygonlist,color)]);
    else
      error("Illegal polygon specification")
    end_if;
  else
    return(options,[Mode=List,polygonplot::maintriself(polygonlist,color)]);
  end_if;

end_proc:

unalias( polygonplot ):

# end of file #
