/*

plotlib::show()         - Liefert Status (TRUE oder FALSE)
plotlib::show(TRUE)     - Grafiken werden gezeichnet
plotlib::show(FALSE)    - Grafikstruktur wird als Ergebnis
                          geliefert. Grafik wird nicht 
                          gezeichnet
plotlib::show(p)        - Grafikstruktur wird gezeichnet

p - Graphik-Struktur

Das folgende Beispiel zeigt
die Vorgehensweise.

Zunaechst wird das sofortige Zeichnen der Graphik unterbunden.

plotlib::show(FALSE):

Alle plot-Befehle (plot2d, plot3d, plotfunc, Befehle der
Bibliothek plotlib, usw.) liefern nun als Ergebnis eine
Struktur (Ausdruckssequenz oder Liste), die weiterverarbeitet
werden kann und spaeter dann gezeichnet wird.

a:=plotfunc(sin(sqrt(x^2+y^2)),x=-5..5,y=-5..5):

b:=plotlib::contourplot([[x,y,sin(sqrt(x^2+y^2))],
                         x=[-5,5],y=[-5,5],Grid=[21,21],
                         Colors=[Height]],Style=Bottom):

Zuruecksetzen, d.h. Graphiken werden wieder sofort gezeichnet.

plotlib::show(TRUE):

Der 2D-contourplot wird nun in ein 3D-Objekt konvertiert. 
Der z-Wert der Funktion ist konstant -2:

c:=plotlib::conv2d({-2}, b):

Kombination der beiden 3D-Plots:

d:=plotlib::combine(a,c): 

Zeichnen der Graphik:

plotlib::show(d):

*/

plotlib::show := proc()
  local r;
begin
  if args(0) = 0 then
     return(stdlib::ShowPlot)
  elif args(0) = 1 and domtype(args(1)) = DOM_BOOL then
     case args(1) 
       of TRUE  do
       of FALSE do
          r := stdlib::ShowPlot;
          /* Die geschuetzte Variable ShowPlot wird  */
          /* in plot2d und plot3d verwendet. Da alle */
          sysassign(stdlib::ShowPlot, args(1));
          return(r);
       otherwise
          error("Illegal argument.")
      end_case;
  else
     plot(args())
  end_if
end_proc:
        