
#( Prule - Domain of 'A Rule of Properties'
           


)#

alias(POp = property::POp):
alias(Prule = POp::Prule):
alias(Arg(x, y) = op(extop(x, 5), y)):

Prule:= domain("POp::Prule"):
Prule::info:= "A Rule of Properties":
Prule::type:= "Prule":
  
Prule::new:=
	proc()
	  local _op, _fir, _sec, _res;
	begin
	  if args() = null() then return(null()) end_if;
	  if contains({DOM_EXEC, DOM_PROC}, domtype(args(1)))
	     or contains({hold(Pfunc), "Pfunc", "P-func"}, args(args(0))) then
	    return(new(Prule, args(1),
		              _if(args(0) > 1,
		                  _if(type(args(2)) = DOM_STRING,
		                      args(2),
		                      expr2text(args(2))),
		                  "no name"),
		              _if(args(0) > 2,
		                  _if(type(args(3)) = DOM_STRING,
		                      args(3),
		                      expr2text(args(3))),
		                  "no information"),
	                      "Pfunc"))
	  end_if;
	  if type(args(1)) = "_equal" then
	    _op:= op(args(1), [1, 0]);
	    _fir:= op(args(1), [1, 1]);
	    _sec:= op(args(1), [1, 2]);
	    _res:= op(args(1), 2)
	  else
	    if domtype(args(1)) = DOM_EXPR and args(0) < 2 then
	      _op:= op(args(1), 0);
	      _fir:= op(args(1), 1);
	      _sec:= op(args(1), 2);
	      _res:= "??"
	    else
	      _op:= args(1);
	      _fir:= args(2);
	      _sec:= args(3);
	      _res:= _if(args(0) = 3, "??", args(4))
	    end_if
	  end_if;
	  new(Prule, level(_op, 2), level(_fir, 2),
		     level(_sec, 2), level(_res, 2))
	end_proc:

Prule::expr:=
	proc()
	begin
	  op(args(), 1)(op(args(), 2..3)) = op(args(), 4)
	end_proc:

Prule::dom:=
	proc()
	begin
	  return([domtype(op(args(), 2)), domtype(op(args(), 3))])
	end_proc:

Prule::opr:=
	proc()
	begin
	  op(args(), 1)
	end_proc:

Prule::first:=
	proc()
	begin
	  [op(args(), 1), op(args(), 2)]
	end_proc:

Prule::fir:=
	proc()
	begin
	  op(args(), 2)
	end_proc:

Prule::sec:=
	proc()
	begin
	  op(args(), 3)
	end_proc:

Prule::result:=
	proc()
	begin
	  op(args(), 4)
	end_proc:

Prule::_equiv:=
	proc()
	begin
	  if args(0) > 2 then
	    if args(3) = "Exact" then
	      return(bool(op(args(1), 1) = op(args(2), 1)) and
	             bool(op(args(1), 2) = op(args(2), 2)) and
	             bool(op(args(1), 3) = op(args(2), 3)))
	    end_if
	  end_if;
	  bool(op(args(1), 1) = op(args(2), 1)) and
	  bool(op(args(1), 2) = op(args(2), 2))
	end_proc:

Prule::_equif:=
	proc()
	begin
	  if args(0) > 2 then
	    if args(3) = "Exact" then
	      return(bool(op(args(1), 1) = op(args(2), 1)))
	    end_if
	  end_if;
	  bool(op(args(1), 2) = op(args(2), 2))
	end_proc:

Prule::_name:=
	proc()
	begin
	  if Prule::_func(args(1)) then
	    if args(0) > 1 then
		extsubsop(args(1),
			  _if(args(2) <> "", 2 = args(2), null()),
			  _if(args(0) > 2, 3 = args(3), null()))
	    else
	      op(args(1), 2) . " - " . op(args(1), 3)
	    end_if
	  else
	    FAIL
	  end_if
	end_proc:

#Prule::_info:=#
	#proc()#
	#begin#
	  #if Prule::_func(args()) then#
	    #op(args(), 4)#
	  #else#
	    #FAIL#
	  #end_if#
	#end_proc:#

Prule::_func:=
	proc()
	begin
	  #bool(contains({DOM_EXEC, DOM_PROC, DOM_FUNC_ENV}, type(op(args(1), 1))))#
	  bool(op(args(), 4) = "Pfunc")
	end_proc:

#( Argumente: (P-func, P-rule) oder (P-func, _op, a, b) )#
Prule::exec:=
	proc()
	begin
	  if not(Prule::_func(args(1))) then
	    userinfo(1, "Not a P-func!");
	    return(FAIL)
	  end_if;
	  if domtype(args(2)) = Prule then
	    op(args(1), 1)(op(args(2), 1..3))
	  else
	    op(args(1), 1)(args(2..args(0)))
	  end_if
	end_proc:

Prule::print:=
	proc()
	begin
	  if Prule::_func(args()) then
	    "P-func: " . Prule::fir(args()) . " - " . Prule::sec(args())
	  else
	    if Prule::sec(args()) <> FAIL then
	      expr2text(Prule::opr(args())
		        (POp::Pid(Prule::fir(args())),
	                 POp::Pid(Prule::sec(args())))
		         = Prule::result(args()))
	    else
	      expr2text(Prule::opr(args())
		        (POp::Pid(Prule::fir(args())))
		         = Prule::result(args()))
	    end_if
	  end_if
	end_proc:

Prule::fprint:= 
	proc(_DAT)
	begin
	  fprint(hold(Unquoted), _DAT, Prule::_fprint(args(2), "nice").":")
	end_proc:

Prule::_fprint:=
	proc()
	begin
	  if args(args(0)) = "nice" then
	    if Prule::_func(args(1)) then
	      expr2text(op(args(1), 1..3))
	    else
	      
	      expr2text(op(args(1), 1)(
	      POp::Pid(expr2text(domtype(op(args(1), 2))).
	               "(".expr2text(op(op(args(1), 2))).")"),
	      POp::Pid(expr2text(domtype(op(args(1), 3))).
	               "(".expr2text(op(op(args(1), 3))).")")) =
	      POp::Pid(expr2text(domtype(op(args(1), 4))).
	               "(".expr2text(op(op(args(1), 4))).")"))

	    end_if
	  else
	    if Prule::_func(args(1)) then
	      #expr2text(op(args(1), 1..3))#
	      expr2text(op(args(1), 1)(
	      POp::Pid(expr2text(domtype(op(args(1), 2))).
	               "(".expr2text(op(op(args(1), 2))).")"),
	      POp::Pid(expr2text(domtype(op(args(1), 3))).
	               "(".expr2text(op(op(args(1), 3))).")")) =
	      POp::Pid(expr2text(domtype(op(args(1), 4))).
	               "(".expr2text(op(op(args(1), 4))).")"))
	    else
	      expr2text(op(args(1), 1)).", ".
	      expr2text(domtype(op(args(1), 2))).
	      "(".expr2text(op(op(args(1), 2)))."), ".
	      expr2text(domtype(op(args(1), 3))).
	      "(".expr2text(op(op(args(1), 3)))."), ".
	      expr2text(domtype(op(args(1), 4))).
	      "(".expr2text(op(op(args(1), 4))).")"
	    end_if
	  end_if
	end_proc:

unalias(Arg):
unalias(Prule):
unalias(POp):
