# solves the recurrence u(n+1)=r*u(n) where r is rational in n #
rec::hypergeom := proc(r,inits,u,n)
local p,q,i0;
begin
   r:=normal(subs(r,n=n-1)); # r = u(n)/u(n-1) #
   if not has(r,n) then
      p:=map(inits,op,[1,1]);
      if p={} then p:=0 else p:=min(op(p)) end_if;
      return(subs(u(p),op(inits))*r^(n-p))
   elif testtype(r,Type::RatExpr(n)) then
      p:=numer(r);
      q:=denom(r);
      userinfo(2,"hypergeometric recurrence with ratio",p/q);
      i0:=max(0,op(sharelib::iroots(q,FALSE)));
      p:=rec::unratio(p,n,i0);
      q:=rec::unratio(q,n,i0);
      if p<>FAIL and q<>FAIL then return(p/q*subs(u(i0),op(inits))) end_if
   end_if;
   FAIL
end_proc:

# e is a polynom in n
  output f such that f(n)=e(n)*e(n-1)*...*e(i0+1) with i0>=0 #
rec::unratio := proc(e,n,i0)
local a,b,d;
begin
   if testtype(e,Type::PolyExpr(n)) then
      case (d:=degree(e,n))
      of 0 do return(e^(n-i0))
      of 1 do 
         a:=coeff(e,n,1); b:=coeff(e,n,0);
         # product(a*k+b,k=i0+1..n) = a^(n-i0)*product(k+b/a,k=i0+1..n) #
         b:=b/a; a:=a^(n-i0);
         if testtype(i0+b,Type::NonNegInt) then
            return(a*expand(fact(n+b))/fact(i0+b))
         elif testtype(i0+b,Type::NegInt) then return(0)
         else return(a*expand(fact(n+b))/gamma(i0+b+1))
         end_if
      otherwise # try to recognize a power #
                if nops((e:=sqrfree(e)))=3 and e[3]<>1 then
			return(e[1]^(n-i0)*rec::unratio(e[2],n,i0)^e[3])
                #if e=n^d then return(fact(n)^d/fact(i0)^d) #
                end_if
      end_case;
   end_if;
   FAIL
end_proc:


