# $Date: 1997/08/07 13:43:27 $ $Author: tonner $ $Revision: 1.4 $ #

#--
	besselJ/Series -- the function attribut "series" for besselJ
--#

# Hankel's symbol (cf table of symbols p. 1046) #
alias(Hankel(n,k)=gamma(1/2+n+k)/fact(k)/gamma(1/2+n-k)):

besselJ := funcattr(besselJ,"series",proc(v,f,x,n)
   local k,l,s,P,Q,xi;
begin
   if has(v,x) then FAIL
   else
      l:=limit(f,x,Right);
      if l=0 then # expansion in 0: use definition 9.1.10 #
         s:=series((x/2)^v*_plus((-x^2/4)^k/fact(k)/gamma(k+v+1)
	    $k=0..n div 2),x,n);
         if f=x then s else s @ series(f,x,n) end_if
      elif l=infinity then # formula 9.2.5 with 9.2.9 and 9.2.10 #
         # changed z into 1/x (z->infinity when x->0+) #
         P:=series(sqrt(x)*_plus((-1)^k*Hankel(v,2*k)*(x/2)^(2*k)
	    $ k=0..(n-1) div 2),x,n);
         Q:=series(sqrt(x)*_plus((-1)^k*Hankel(v,2*k+1)*(x/2)^(2*k+1)
	    $k=0..(n-3) div 2),x,n);
         if f<>1/x then s:=series(f,x,n); P:=P @ s; Q:=Q @ s end_if;
         xi:=f-(v/2+1/4)*PI;
         s:=P*cos(xi)-Q*sin(xi);
         s::_mult(s,sqrt(2/PI))
      else Series::unknown(besselJ(v,f),x,n)
      end_if
   end_if
end_proc):

# end of file #
