alias(BranchOrder(a)=extop(a,1)):
alias(valuation(a)=extop(a,2)): # valuation #
alias(err(a)=extop(a,3)):
alias(ListCoeffs(a)=extop(a,4)):
alias(variable(a)=op(extop(a,5),1)): # series variable #
alias(point(a)=op(extop(a,5),2)): # expansion point #

Puiseux::indet:=fun(variable(args(1))):
Puiseux::point:=fun(point(args(1))):

Puiseux := domain("Puiseux"):

Puiseux::match_x_power_k:=proc(e,x) # matches x^k with k a rational #
# returns k if match successful, FALSE otherwise #
begin
   if e=1 then 0
   elif e=x then 1
   elif type(e)="_power" and op(e,1)=x and contains({DOM_INT,DOM_RAT},type(op(e,2))) then op(e,2)
   else FALSE
   end_if
end_proc:

# these procedures are to be used from outside to
  create Puiseux expansions efficiently without knowing their
  internal structure #
Puiseux::create := proc(bo,valuation,ord,l,x,x0)
begin
   if args(0)<6 then x0:=0 end_if;
   new(Puiseux,bo,valuation,ord,l,x=x0)
end_proc:

Puiseux::ldegree:=fun(block((l,n),(
   l:=ListCoeffs(args(1));
   n:=valuation(args(1));
   while l<>[] do
      if Puiseux::iszero(l[1]) then l[1]:=NIL; n:=n+1
      else return(n/BranchOrder(args(1)))
      end_if
   end_while;
   FAIL))):

Puiseux::order := proc(a) begin err(a)/BranchOrder(a) end_proc:

# default test to zero #
Puiseux::iszero:=fun(iszero(expr(args(1))) or Puiseux::fiszero(args(1))):

# test to zero usinf float for constants #
Puiseux::fiszero:=fun((
   if testtype(args(1),Type::Constant) then
      float(args(1)); 
      if type(%)=DOM_FLOAT and abs(%)<10.0^(-DIGITS) then TRUE 
      else FALSE end_if
   else FALSE end_if)):

Puiseux::normal:=proc(a) # remove zero coefficients in head #
local l,v;
begin
   l:=ListCoeffs(a); # removes a0 #
   if nops(l)=0 then a
   elif not Puiseux::iszero(l[1]) then a
   else
      v:=valuation(a);
      while l<>[] do
         if Puiseux::iszero(l[1]) then l[1]:=NIL; v:=v+1 else break end_if
      end_while;
      extsubsop(a,2=v,4=l)
   end_if
end_proc:

# [branch order, valuation, ord, [coeffs]] #
Puiseux::new := proc(e,x,Ord) 
   local i,big_O,ord,k,d,bo,l;
   begin
      if type(1/x)=DOM_IDENT then # x=1/t for expansions at t=infinity #
         return(series(e,1/x=infinity,Ord))
      end_if;
      ord:=proc(t) begin if type(t)="_power" and op(t,1)=x and
	type(op(t,2))=DOM_RAT then denom(op(t,2)) else 1 end_if end_proc:
      l:=indets(e,PolyExpr);
      bo:=ilcm(ord(op(l,i))$i=1..nops(l));
      # find the O-term if any, and the branching order #
      if type(e)="_plus" then e:=[op(e)] else e:=[e] end_if;
      big_O:=0;
      for i from 1 to nops(e) do
         if type(op(e,i))=O then big_O:=op(e,i); e:=subsop(e,i=null()); break end_if
      end_for;
      if big_O<>0 then 
         if nops(big_O)<>1 or (ord:=Puiseux::match_x_power_k(op(big_O),x))=FALSE
         then error("invalid big-O") end_if;
         bo:=ilcm(bo,denom(ord));
         ord:=ord*bo;
      end_if;
      e:=_plus(op(e));
      e:=subs(e,x=x^bo); # should have integer exponents now #
      # now find the valuation and the degree #
      k:=ldegree(e,[x]);
      if k=FAIL then return(FAIL) end_if; # not a Puiseux #
      if type(ord)<>DOM_INT then
         if type(Ord)<>DOM_INT then Ord:=ORDER end_if;
         ord:=k+Ord
      end_if;
      d:=min(degree(e,[x]),ord-1);
      new(Puiseux,bo,k,ord,[coeff(e,[x],i)$hold(i)=k..d],x=0);
   end_proc:

Puiseux::set_var := proc(a,X,x,x0) # specify variable of expansion #
begin
   extsubsop(subs(a,X=(if type(x0)=stdlib::Infinity then
	sign(x0)/x else x-x0 end_if)),5=(x=x0))
end_proc:

Puiseux::subs:=fun(extsubsop(args(1),
	4=subs(ListCoeffs(args(1)),args(2..args(0))))):

Puiseux::print :=
   proc(a,x) local l,i,bo,k;
   begin
      bo:=BranchOrder(a);
      k:=valuation(a);
      l:=ListCoeffs(a);
      x:=extop(a,5); 
      if domtype(op(x,2))=stdlib::Infinity then x:=1/op(x,1)
      else x:=op(x,1)-op(x,2)
      end_if;
      l:=[l[i]*x^((k+i-1)/bo) $ i=1..nops(l)];
      l:=map(l,func((if x=0 then null() else x end_if),x));
      i:=hold(O)(x^(err(a)/bo));
      if l=[] then i else hold(_plus)(op(l),i) end_if
end_proc:

Puiseux::expr := proc(a)
local l,x,i,bo,k;
begin
   bo:=BranchOrder(a);
   k:=valuation(a);
   l:=ListCoeffs(a);
   if type(point(a))=stdlib::Infinity then x:=1/variable(a)
   else x:=variable(a)-point(a)
   end_if;
   _plus(l[i]*x^((k+i-1)/bo) $ i=1..nops(l))
end_proc:

Puiseux::multbo := proc(a,n) # multiply branching order of a by n (integer) #
local l,i,z;
begin
   l:=ListCoeffs(a);
   if l<>[] then 
      z:=0$(n-1);
      l:=[(l[i],z)$i=1..nops(l)-1,l[nops(l)]]
   end_if;
   extsubsop(a,1=BranchOrder(a)*n,2=valuation(a)*n,3=err(a)*n,4=l)
end_proc:

Puiseux::plus := proc(a,b) local ka,kb,qa,qb,la,lb,i,dif,da,db,lc,boa,bob;
begin
   if type(a)<>Puiseux then
      qb:=err(b);
      a:=Puiseux::new(a,variable(b),qb);
      qa:=err(a);
   elif type(b)<>Puiseux then
      # is b in a more general domain ? #
      b::convert(a); if %<>FAIL then return(%+b) end_if;
      b:=Puiseux::new(b,variable(a));
      qa:=err(a); qb:=qa;
   else
      qa:=err(a); qb:=err(b);
   end_if;
   boa:=BranchOrder(a); bob:=BranchOrder(b);
   if boa<>bob then
      i:=ilcm(boa,bob);
      Puiseux::plus(Puiseux::multbo(a,i/boa),Puiseux::multbo(b,i/bob))
   else
      ka:=valuation(a); kb:=valuation(b);
      if kb>=qa then a
      elif ka>=qb then b
      elif ka<=kb then
         dif:=kb-ka; # dif>=0 #
         la:=ListCoeffs(a); lb:=ListCoeffs(b);
         da:=nops(la); db:=nops(lb)+dif;
         if da<=db then
            if dif<=da then 
               lc:=[la[i]$i=1..dif,la[i]+lb[i-dif]$i=dif+1..da,lb[i-dif]$i=da+1..min(db,qa-ka)]
            else # no overlapping #
               lc:=[la[i]$i=1..da,0$(dif-da),lb[i-dif]$i=dif+1..db]
            end_if
         else # da>db thus da>dif ==> always overlapping #
            lc:=[la[i]$i=1..dif,la[i]+lb[i-dif]$i=dif+1..db,la[i]$i=db+1..da];
         end_if;
         while nops(lc)<>0 do
            if lc[1]=0 then
               lc:=subsop(lc,1=null()); ka:=ka+1; if lc=[] then break end_if
            else break
            end_if
         end_while;
         extsubsop(a,2=ka,3=min(qa,qb),4=lc)
      else Puiseux::plus(b,a)
      end_if
   end_if
end_proc:

Puiseux::_plus:=fun((case args(0)
    of 1 do return(args(1))
    of 2 do return(Puiseux::plus(args(1), args(2)))
    of 3 do return((if type(args(1))=Puiseux then
	Puiseux::plus(Puiseux::plus(args(1), args(2)),args(3))
        else Puiseux::plus(args(1),Puiseux::plus(args(2),args(3))) end_if))
    otherwise
      _plus(_plus(args(i) $ hold(i) = 1..args(0) div 2),
      _plus(args(i) $ hold(i) = (args(0) div 2) + 1..args(0)))
  end_case)):

Puiseux::_negate := proc(a) local l,i;
begin
   l:=ListCoeffs(a);
   extsubsop(a,4=[-l[i]$i=1..nops(l)])
end_proc:

Puiseux::mult := proc(a,b) local ka,kb,qa,qb,da,db,la,lb,i,j,i1,i2,l1,l2,l3,imax,boa,bob;
begin
   if type(a)<>Puiseux then
      if a=1 then return(b) else a:=Puiseux::new(a,variable(b),err(b)) end_if
   elif type(b)<>Puiseux then 
      if contains({DOM_EXPR,DOM_IDENT,DOM_INT,DOM_RAT,DOM_FLOAT},domtype(b)) then
         b:=Puiseux::new(b,variable(a),err(a));
      else return(b::_mult(b::convert(a),b))
      end_if
   end_if;
   boa:=BranchOrder(a); bob:=BranchOrder(b);
   if boa<>bob then
      i:=ilcm(boa,bob);
      Puiseux::mult(Puiseux::multbo(a,i/boa),Puiseux::multbo(b,i/bob))
   else
      ka:=valuation(a); kb:=valuation(b);
      qa:=err(a); qb:=err(b);
      la:=ListCoeffs(a); lb:=ListCoeffs(b);
      da:=nops(la); db:=nops(lb);
      imax:=min(qa-ka,qb-kb); # maximal number of terms in the product #
      if da<=db then i1:=da; i2:=db
      else i1:=db; i2:=da
      end_if;
      l1:=_plus(la[j]*lb[i+1-j]$hold(j)=1..i)$hold(i)=1..i1;
      if da<=db then    
	 l2:=_plus(la[j]*lb[i+1-j]$hold(j)=1..da)$hold(i)=da+1..min(db,imax);
      else
	 l2:=_plus(la[i+1-j]*lb[j]$hold(j)=1..db)$hold(i)=db+1..min(da,imax);
      end_if;
      l3:=_plus(la[j]*lb[i+1-j]$hold(j)=i+1-db..da)$hold(i)=i2+1..min(da+db-1,imax);
      extsubsop(a,2=ka+kb,3=min(ka+qb,kb+qa),4=map([l1,l2,l3],expand))
   end_if
end_proc:

Puiseux::_mult:=misc::genassop(_mult,args(1),Puiseux::mult(args(1),args(2))):

Puiseux::square := proc(a) local ka,qa,da,la,i,j,l1,l3,imax;
begin
   ka:=valuation(a);
   qa:=err(a);
   la:=ListCoeffs(a);
   da:=nops(la); 
   imax:=qa-ka; # maximal number of terms in the product #
   l1:=_plus(la[j]*la[i+1-j]$hold(j)=1..i)$hold(i)=1..da;
   l3:=_plus(la[j]*la[i+1-j]$hold(j)=i+1-da..da)$hold(i)=da+1..min(2*da-1,imax);
   extsubsop(a,2=2*ka,3=ka+qa,4=[l1,l3])
end_proc:

Puiseux::_power := proc(a,n)
begin
   if a=FAIL then a
   elif type(n)=DOM_RAT then Puiseux::power(a,n)
   elif n<0 then 
      a:=normal(a);
      if n=-1 then Puiseux::_invert(a)
      else Puiseux::_invert(Puiseux::_power(a,-n))
      end_if
   elif n=0 then Puiseux::one(n,variable(a))
   elif n=1 then a
   elif n mod 2 = 0 then Puiseux::_power(Puiseux::square(a),n div 2)
   else Puiseux::mult(a,Puiseux::_power(Puiseux::square(a),n div 2))
   end_if
end_proc:

# from Zippel, page 165 #
Puiseux::power := proc(a,s)
local v,p,ps,k,q,l,d,p0,C,bo;
begin
   # works only for valence 0 #
   v:=valuation(a);
   if v<>0 then 
      # a = x^(v/bo) * (1 + ...) thus a^s = B * C
        where B = x^(s*v/bo) and C = (1 + ...)^s #
      # B has branching order denom(s*v/bo) and C has branching order bo #
      bo:=BranchOrder(a);
      p:=s*v/bo;
      C:=Puiseux::power(extsubsop(a,2=0,3=err(a)-v),s);
      if bo mod denom(p) <> 0 then
         k:=ilcm(bo,denom(p));
         C:=Puiseux::multbo(C,k/bo);
      else k:=bo
      end_if;
      extsubsop(C,2=p*k,3=err(C)+p*k)
   else
      q:=err(a); # O(x^q) #
      p:=ListCoeffs(a);
      d:=nops(p);
      if d=0 then return(extsubsop(a,3=s*q)) end_if;
      p0:=p[1];
      ps[1]:=p0^s;
      for k from 1 to q-1 do
         ps[k+1]:=_plus((l*s-k+l)*p[l+1]*ps[k+1-l]$l=1..min(k,d-1))/p0/k
      end_for;
      extsubsop(a,4=[ps[k]$hold(k)=1..q])
   end_if
end_proc:

# from Zippel, page 167 #
Puiseux::_fconcat:=proc(P,Q)
local q,R,i,l,d,k,vQ,lR,boP,boQ;
begin
      boP:=BranchOrder(P);
      # P(x<-Q) = P(x<-x^boP)(x<-Q^(1/boP)) #
      if boP<>1 then P:=extsubsop(P,1=1); Q:=Puiseux::power(Q,1/boP) end_if; 
      boQ:=BranchOrder(Q);
      vQ:=valuation(Q);
      if point(P)=0 then
         if vQ<=0 then error("invalid composition") end_if
      else # point(P)=a<>0 therefore Q must tend to a #
         Q:=lmonomial(Q,Rem);
         if Q[1]<>point(P) then error("invalid composition") end_if;
         Q:=Q[2]; vQ:=valuation(Q);
      end_if;
      # P = a[k]*x^k + ... + a[p]*x^p + O(x^q) #
      k:=valuation(P);
      l:=ListCoeffs(P);
      d:=nops(l);
      q:=err(P);
      # initialize with a[p] + O(x^(vQ*(q-p))) #
      R:=extsubsop(Q,2=0,3=vQ*(q-k-d+1),4=[l[d]]);
      for i from d-1 downto 1 do
	 R:=Puiseux::mult(R,Q); # now R = b*x^vQ + ... #
	 lR:=[l[i],0$(vQ-1),op(ListCoeffs(R))];
         R:=extsubsop(R,2=0,4=lR);
      end_for;
      # now R = a[k] + a[k+1]*Q + ... #
      if k=0 then R else Puiseux::_mult(R,Puiseux::_power(Q,k)) end_if
end_proc:

Puiseux::scalmult:=proc(p,a,k) # a*x^k*p #
local l,i;
begin
   l:=ListCoeffs(p);
   if args(0)<3 then extsubsop(p,4=[a*l[i]$i=1..nops(l)])
   else
      extsubsop(p,4=[a*l[i]$i=1..nops(l)],2=valuation(p)+k*BranchOrder(p),
	3=err(p)+k*BranchOrder(p))
   end_if
end_proc:

Puiseux::one:=proc(n,x) begin new(Puiseux,1,0,n,[1],x=0) end_proc:

Puiseux::frompoly := proc(p,n,x)
local i,v;
begin
   # to avoid coefficients like 0.0 #
   p:=mapcoeffs(p,fun((if iszero(args(1)) then 0 else args(1) end_if)));
   if iszero(p) then new(Puiseux,1,n,n,[],x=0)
   else
      v:=ldegree(p);
      new(Puiseux,1,v,max(n,v),[coeff(p,i)$i=v..min(degree(p),n-1)],x=0)
   end_if
end_proc:

# the following is about twice faster than Puiseux::power(a,-1) #
Puiseux::_invert := proc(a) local k,q,d,l,i,a1,b,j;
begin
   l:=ListCoeffs(a);
   d:=nops(l);
   q:=err(a);
   if d=0 then # 1/O(x^q) -> division by zero #
      error("division by zero")
   else # 1/(a1*x^k+...+ad*x^(k+d-1)+O(x^q)) = x^(-k)/(a1+...+ad*x^(d-1)+O(x^(q-k))) #
      a1:=l[1];
      k:=valuation(a);
      b:=table(); b[1]:=1/a1;
      for i from 3 to d+1 do b[i-1]:=-_plus(l[j]*b[i-j]$j=2..i-1)/a1 end_for;
      for i from d+2 to q-k+1 do b[i-1]:=-_plus(l[j]*b[i-j]$j=2..d)/a1 end_for;
      extsubsop(a,2=-k,3=q-2*k,4=[b[i]$hold(i)=1..q-k])
   end_if;
end_proc:

Puiseux::equalx := proc(a) # is a x + O(...) #
begin
   if valuation(a)=1 and nops(ListCoeffs(a))>=1 and ListCoeffs(a)[1]=1 then TRUE else FALSE end_if
end_proc:

Puiseux::coeff := proc(a,n) # gives the coefficient of x^n #
local k,q,l,bo;
begin
   bo:=BranchOrder(a);
   k:=valuation(a);
   if n<k/bo then 0
   else
      q:=err(a);
      if n>=q/bo then FAIL
      else # x^k/bo corresponds to l[1], thus x^n corresponds to l[n*bo+1-k] #
         n:=n*bo+1-k;
         if type(n)<>DOM_INT then return(0) end_if;
         l:=ListCoeffs(a);
         if n<=nops(l) then l[n] else 0 end_if
      end_if
   end_if
end_proc:

Puiseux::nthterm := proc(a,n) local l;
begin
   l:=ListCoeffs(a);
   if n>nops(l) then FAIL else variable(a)^((valuation(a)+n-1)/BranchOrder(a)) end_if
end_proc:

Puiseux::lcoeff := proc(_a) local l;
begin
   l:=ListCoeffs(_a);
   while l<>[] do
      simplify(l[1]);
      if Puiseux::iszero(%) then l[1]:=NIL
      else return(%)
      end_if
   end_while;
   FAIL
end_proc:

# lmonomial(s) returns the leading term of s
  lmonomial(s,Rem) returns [u,r] where u is the leading non-zero term and r=s-u,
  both return FAIL if there is no non-zero leading term, i.e. s = O(...) #
Puiseux::lmonomial := proc(_a,opt) local l,n,u;
begin
   l:=ListCoeffs(_a);
   n:=0; while l<>[] do
      if Puiseux::iszero(l[1]) then l[1]:=NIL
      else 
         u:=l[1]*variable(_a)^((valuation(_a)+n)/BranchOrder(_a));
         if opt=hold(Rem) then
            _a:=Puiseux::normal(extsubsop(_a,2=valuation(_a)+n+1,4=subsop(l,1=null())));
            return([u,_a])
         else return(u) 
         end_if;
      end_if;
      n:=n+1;
   end_while;
   FAIL
end_proc:

Puiseux::lterm := proc(_a) local l,n;
begin
   l:=ListCoeffs(_a);
   n:=0; while l<>[] do
      if Puiseux::iszero(l[1]) then l[1]:=NIL
      else return(variable(_a)^((valuation(_a)+n)/BranchOrder(_a)))
      end_if;
      n:=n+1;
   end_while;
   FAIL
end_proc:

Puiseux::evaluate := proc(_a_)
begin
   ListCoeffs(_a_);
   # extsubsop(_a_,4=eval(%)) #
   extsubsop(_a_,4=subs(%,Simplified))
end_proc:

# reverse(a[k]*x^(k/b) + a[k+1]*x^((k+1)/b) + ... ) = 
	reverse(a[k]*x^k + a[k+1]*x^(k+1) + ...)^b #
Puiseux::revert := proc(a) # using poly is faster #
local k,c,l,n,m,i,p,y,q,eq;
begin
   if BranchOrder(a)<>1 then
      Puiseux::revert(extsubsop(a,1=1))^BranchOrder(a)
   else # branching order is 1 #
      k:=valuation(a); # y = x^k + ... ==> x = y^(1/k) + y^(2/k) + ... #
      if k<0 then # y = 1/x^j + ... ==> 1/y = x^j + x^(j+1) + ... #
         1/Puiseux::revert(1/a)
      elif k=0 then # y = c + x^j + ... ==> (y-c) = x^j + ... #
         c:=lcoeff(a);
         a:=Puiseux::lmonomial(a,Rem)[2];
         extsubsop(Puiseux::revert(a),5=(variable(a)=c))
      else # k>0 #
         n:=err(a)-k; # number of knowns terms of y #
         m:=ListCoeffs(a);
         if nops(m)<n then m:=m . [0$(n-nops(m))] end_if;
         l[1]:=1/m[1]^(1/k); # coefficient of y^(1/k) #
         p:=l[1]*y; q:=poly(m[1]*y^k,[y]);
         for i from 2 to n do
            # compute the coefficient of y^(i/k) #
            # we need only the coefficients m[1]...m[i] #
            p:=p+c*y^i;
            q:=q+poly(m[i]*y^(i+k-1),[y]);
            eq:=coeff(evalp(q,y=p),[y],i+k-1); # this is the expensive step #
            l[i]:=-coeff(eq,[c],0)/coeff(eq,[c],1);
            p:=subs(p,c=l[i]);
         end_for;
         extsubsop(a,1=k,2=1,3=1+n,4=[l[i]$hold(i)=1..n])
      end_if
   end_if
end_proc:

# suggestion from Frank Postel #
Puiseux::map := fun(	
     extsubsop( args(1),4=map(ListCoeffs(args(1)),args(i) $ hold(i)=2..args(0)) )
   ):

Puiseux::float:=proc(x) begin
   map(x,fun((if args(1)=0 then 0 else float(args(1)) end_if)))
end_proc:

Puiseux::contfrac := proc(a)
local l,bo,n,j,v,k,a1,b1,z,q;
begin
   z:=variable(a);
   a:=Puiseux::normal(a);
   bo:=BranchOrder(a);
   v:=valuation(a);
   q:=err(a);
   l:=ListCoeffs(a);
   n:=nops(l);
   if n=0 then return(O(z^(q/bo)))
   elif n=1 then return(l[1]*z^(v/bo)+O(z^(q/bo)))
   end_if;
   # a is normalized ==> l[1]<>0 #
   # find second non zero term #
   for j from 2 to n do if l[j]<>0 then break end_if end_for;
   if j=n then return(l[1]*z^(v/bo)+l[j]*z^((v+j-1)/bo)+O(z^(q/bo))) end_if;
   # is there a third non zero term ? #
   for k from j+1 to n do if l[k]<>0 then break end_if end_for;
   if l[k]=0 then # there are only two non zero terms #
      l[1]*z^(v/bo)+l[j]*z^((v+j-1)/bo)+O(z^(q/bo))
   else
      a1:=l[1]*z^(v/bo);
      b1:=z^((v+j-1)/bo);
      a1+b1/Puiseux::contfrac(b1/(a-a1))
   end_if
end_proc:

Puiseux::series:=proc()
begin
  series(Puiseux::print(args(1)),args(2..args(0)))
end_proc:

Puiseux::diff := proc(a,x)
local l,i,k,bo,ord;
begin
   case args(0)
   of 0 do error("wrong number of arguments")
   of 1 do return(a)
   of 2 do break # normal case #
   otherwise return(Puiseux::diff(Puiseux::diff(a,x),args(3..args(0))))
   end_case;
   bo:=BranchOrder(a);
   k:=valuation(a);
   ord:=err(a);
   l:=ListCoeffs(a);
   if not has(variable(a)-x,x) then
      # diff(c*x^((k+i-1)/bo),x)=x*(k+i-1)/bo*x^((k+i-1)/bo-1) #
      extsubsop(a,2=k-bo,3=ord-bo,4=[l[i]*(k+i-1)/bo$i=1..nops(l)])
   elif has(variable(a),x) then FAIL
   else # only differentiate coefficients #
      map(a,diff,x)
   end_if
end_proc:

Puiseux::laplace := proc(a,t,s) local x,l,i,bo,ord,k;
begin
   x:=variable(a);
   if x=t then
      bo:=BranchOrder(a); k:=valuation(a); ord:=err(a); l:=ListCoeffs(a);
      extsubsop(a,2=k+bo,3=ord+bo,4=[l[i]*gamma((k+i-1)/bo+1)$i=1..nops(l)],5=1/s)
   else map(a,transform::laplace,t,s)
   end_if
end_proc:

Puiseux::ilaplace := proc(a,s,t) local x,l,i,bo,ord,k;
begin
   x:=variable(a);
   if x=1/s then
      bo:=BranchOrder(a); k:=valuation(a); ord:=err(a); l:=ListCoeffs(a);
      extsubsop(a,2=k-bo,3=ord-bo,4=[l[i]/gamma((k+i-1)/bo)$i=1..nops(l)],5=t)
   else map(a,transform::ilaplace,s,t)
   end_if
end_proc:

Puiseux::has:=fun(has([extop(args(1),4..5)],args(2))):
Puiseux::int:=proc(a,x)
begin
   if x<>variable(a) then map(a,int,x)
   else series(int(expr(a),x),extop(a,5),err(a)+1);
        //extsubsop(%,3=err(%)+1)
   end_if
end_proc:

unalias(BranchOrder, valuation, err, ListCoeffs, variable):

