Series::series := fun(Series::Puiseux(args())):

Series::Puiseux := proc(_f,__x,_n)
local tf,p;
begin
   userinfo(2,"computing expansion of",_f);
   if testtype(_f,Type::PolyExpr(__x,Type::AnyType)) then Puiseux::frompoly(poly(_f,[__x]),_n,__x)
   else
      tf:=type(_f); # "exp" or "tan" or ... #
      # first catch functions with variable number of arguments #
      if tf="_plus" then Series::plus(args())
      elif tf="_mult" then Series::mult(args())
      elif tf="_power" then Series::power(op(_f),args(2..args(0)))
      else
         p := op(_f,0);
         if domtype(level(p,2)) = DOM_FUNC_ENV then 
            p:=funcattr(level(p,2),"series");
	    if contains({DOM_PROC,DOM_EXEC},domtype(p))
            then return(p(op(_f),args(2..args(0)))) end_if;
	 end_if;
         if traperror((p:=Series::unknown(args())))=0 then p else FAIL end_if
      end_if
   end_if
end_proc:

Series::unknown := proc(_f_,_x,_n)
local g,fa,i,g0,l;
begin
   fa:=1;
   g:=normal(_f_);
   if bool(type(g)="int") and has(op(g,2),_x) then g0:=0 
   # else g0:=eval(subs(g,_x=0)) end_if; #
   else g0:=subs(g,_x=0,Simplified) end_if;
   l:=[g0];
   for i from 1 to _n-1 do
      g:=diff(g,_x);
      g:=Series::diff2D(g);
      fa:=fa*i;
      # l:=append(l,eval(subs(g,_x=0))/fa); #
      l:=append(l,subs(g,_x=0,Simplified)/fa);
   end_for;
   normal(Puiseux::create(1,0,_n,l,_x))
end_proc:

Series::diff2D := proc(_f) 
begin
   case type(_f)
   of "_plus" do
   of "_mult" do
   of "_power" do return(map(_f,Series::diff2D))
   of "diff" do return(_fconcat(D$nops(_f)-1)(op(_f,[1,0]))(op(_f,[1,1]))*
				Series::diff2D(diff(op(_f,[1,1]),op(_f,2))))
   end_case;
   _f
end_proc:

Series::plus:=proc(_f,_x,n) # f is a sum #
local s,i;
begin
   s:=Series::Puiseux(op(_f,1),_x,n);
   for i from 2 to nops(_f) do s:=s+Series::Puiseux(op(_f,i),_x,n) end_for;
end_proc:

Series::mult:=proc(__f,x,_n) # __f is a product #
local s,i,k,j;
begin
   ([__f,k,s]):=split(__f,has,x); j:=0;
   if __f=1 then Puiseux::create(1,0,_n,[k],x) # does not depend on x #
   else
      if type(__f)<>"_mult" then s:=Series::Puiseux(__f,args(2..args(0)))
      else
         ([j,__f,s]):=split(__f,fun(args(1)=x or type(args(1))="_power"
		and op(args(1),1)=x and type(op(args(1),2))=DOM_INT));
         if j=1 then j:=0 elif j=x then j:=1 else j:=op(j,2) end_if;
         if type(__f)<>"_mult" then s:=Series::Puiseux(__f,args(2..args(0)))
         else
	    s:=Series::Puiseux(op(__f,1),args(2..args(0)));
	    for i from 2 to nops(__f) do
	       s:=s*Series::Puiseux(op(__f,i),args(2..args(0)))
	    end_for;
         end_if
      end_if;
      if type(s)=Puiseux then s::scalmult(s,k,j)
      else s::scalmult(s,k,x^j) end_if
   end_if
end_proc:

Series::power:=proc(a,b,x,_n)
begin
   case type(b)
   of DOM_INT do
   of DOM_RAT do return(Series::Puiseux(a,args(3..args(0)))^b)
   otherwise # to avoid infinite loops #
       Series::Puiseux(hold(exp)(b*ln(a)),args(3..args(0)))
   end_case
end_proc:

# end of file #
