#++

plotsetup(dev)

dev -- device ( X11 or Term )

plotsetup is used for plotting with mupad (terminal version of MuPAD).
If mupad is running in an X11 environment, plotsetup(X11) modify the
functions plot2d and plot3d to display the graphic in a separate
window on the workstation display. 
plotdevice(Term) resets plot2d and plot3d.
plotdevice is only available on Unix based operating systems.
 
++#

alias(DumbTerminal=1201):

sharelib::plot2d_ext := proc()
  name plot2d;
begin
    sharelib::show_plot(traperror(sharelib::plot2d_sys(args())))
end_proc:

sharelib::plot3d_ext := proc()
  name plot3d;
begin
    sharelib::show_plot(traperror(sharelib::plot3d_sys(args())))
end_proc:

sharelib::show_plot := proc(errnr)
begin
    if errnr = DumbTerminal then
       if sharelib::DEVICE = hold(X11) then
          if system("vcam save.mp") <> 0 then
             print("Can't spawn external previewer.")
          else
             return( null() )
          end_if;
          print("Plotdata saved in binary file save.mp")
       end_if
    else
       print("Illegal argument(s) in plot command.")
    end_if;
    null()
end_proc:

sharelib::plotsetup := proc(dev)
begin
  if sysname() <> "UNIX" then
     error("Not an Unix OS.");
  end_if;

  case dev

   of hold(Term) do

     sysassign( sharelib::DEVICE, dev );
     if sharelib::plot2d_sys <> FAIL then
        sysassing( plot2d, sharelib::plot2d_sys );
        sysassign( plot3d, sharelib::plot3d_sys );
     end_if;
     break;

  of hold(X11) do

    sysassign( sharelib::DEVICE, dev);
    if sharelib::plot2d_sys = FAIL then
       sysassign( sharelib::plot2d_sys, plot2d );
       sysassign( sharelib::plot3d_sys, plot3d );
    end_if;
    sysassign( plot2d, sharelib::plot2d_ext );
    sysassign( plot3d, sharelib::plot3d_ext );
    break;
 
  otherwise

    error("Unknown device '".dev."'.");
 
  end_case;
  
  null()
  
end_proc:

unalias(DumbTerminal):

# end of file #
