# after sharelib::trace(f) where f is a procedure or a function environment, 
  all the calls of f will print their input and output.

  Examples:

>> sharelib::trace(has): has(x,y);
trace, enter has with args=, x, y
trace, exit has with result=, FALSE

                                   FALSE

>> sharelib::untrace(has): has(x,y);

                                   FALSE

>> fib := proc(n) begin if n<2 then n else fib(n-1)+fib(n-2) end_if end_proc:
>> sharelib::trace(fib): fib(2);
trace, enter fib with args=, 2
trace, enter fib with args=, 1
trace, exit fib with result=, 1
trace, enter fib with args=, 0
trace, exit fib with result=, 0
trace, exit fib with result=, 1

                                     1

>> sharelib::untrace(fib): fib(2);

                                     1

>> g:=fun(1+args(1)): sharelib::trace(g): g(3);
trace, enter g with args=, 3
trace, exit g with result=, 4

                                     4

>> sharelib::trace(domtype(infinity)):
>> infinity+3;
trace, enter (stdlib::Infinity)::evaluate with args=, infinity
trace, exit (stdlib::Infinity)::evaluate with result=, infinity
trace, enter (stdlib::Infinity)::_plus with args=, infinity, 3
trace, enter (stdlib::Infinity)::thisDomain with args=, infinity
trace, exit (stdlib::Infinity)::thisDomain with result=, TRUE
trace, enter (stdlib::Infinity)::thisDomain with args=, 3
trace, exit (stdlib::Infinity)::thisDomain with result=, FALSE
trace, enter (stdlib::Infinity)::isInverted with args=, infinity
trace, exit (stdlib::Infinity)::isInverted with result=, FALSE = TRUE
trace, enter (stdlib::Infinity)::isUndefined with args=, infinity
trace, exit (stdlib::Infinity)::isUndefined with result=, FALSE = UNKNOWN
trace, enter (stdlib::Infinity)::thisDomain with args=, infinity
trace, exit (stdlib::Infinity)::thisDomain with result=, TRUE
trace, enter (stdlib::Infinity)::thisDomain with args=, 3
trace, exit (stdlib::Infinity)::thisDomain with result=, FALSE
trace, exit (stdlib::Infinity)::_plus with result=, infinity

                                 infinity

>> f := proc(x) begin NIL end_proc:
>> sharelib::trace(f):
>> f();
trace, enter f with args=
trace, exit f with result=, NIL

                                    NIL
#

sharelib::trace:=proc(_f_) local ff,p;
option hold;
begin
   ff:=context(_f_);
   case domtype(ff)
   of DOM_FUNC_ENV do
      stdlib::syseval(
        evalassign(_f_,subsop(ff,1=sharelib::trace2(op(ff,1),
                              expr2text(_f_))),1)); 
      break
   of DOM_PROC do of DOM_EXEC do
      stdlib::syseval(
        evalassign(_f_,sharelib::trace2(ff,expr2text(_f_)),1)); 
      break
   of DOM_DOMAIN do
      for p in ff do
         # do not trace the method "print" !!! #
         if contains({DOM_FUNC_ENV,DOM_PROC,DOM_EXEC},domtype(op(p,2)))
            and not contains({"print","printElem"},op(p,1)) then
           stdlib::syseval((
            domattr(ff,op(p,1)):=sharelib::trace2(op(p,2),
	       expr2text(hold(domattr)(ff,op(p,1))))))
         end_if
      end_for; break
   end_case;
   setuserinfo(trace,1);
   null()
end_proc:

sharelib::trace2:=proc(p,n) local x,g,EVAL_STMT,res;
begin
   x:=genident();
   evalassign(x,p,1);
   EVAL_STMT:=FALSE;
   g:=_stmtseq(userinfo(1,"xxx",args()),
                           _assign(res,[xxx(args())]),
                           userinfo(1,"xxx",op(res)),op(res));
   g:=subsop(g,[1,2]=("enter ".n." with args="));
   g:=subsop(g,[2,2,1,0]=x);
   g:=subsop(g,[3,2]=("exit ".n." with result="));
   subsop(_procdef(NIL,res,NIL,NIL,NIL,trace),4=g)
end_proc:

sharelib::untrace:=proc(_f_) local p,ff;
option hold;
begin
   ff:=context(_f_);
   if type(ff)=DOM_FUNC_ENV then p:=op(ff,1)
   elif type(ff)=DOM_PROC then p:=ff
   else error("not a procedure nor a function environment")
   end_if;
   if op(p,6)=hold(trace) then
      p:=op(p,[4,2,2,1,0]);
      p:=level(p,2);
      stdlib::syseval(
        evalassign(_f_,(if type(ff)=DOM_FUNC_ENV then 
                          subsop(ff,1=p) 
                        else p end_if),1))
   end_if;
end_proc:
