# #
# $Date: 1999/05/19 09:41:31 $ $Author: stefanw $ $Revision: 1.14.2.1 $ #
# #
# frankp, 13.12.1994 #

#++
sign.mu

	sign -- sign function for real and complex expressions

	sign(x)

	The sign function returns the sign of a real or complex
	number, defined by x/abs(x) for x<>0.

	'sign(0)' is undefined in the way that 'sign(0)' can be evaluated
	to anything. One can define the value of 'sign(0)' by setting
	    'sign(0)' := a 

	'sign(0)' := 0 is predefined.
	
	For user defined functions, 'sign' tests for the
	function attribute "sign" and call the procedure
	if it exists.
++#

sign := func_env(
proc(_x_)
    name sign;
    local t, s, ss, f;
begin
    if (t:= _x_::sign) <> FAIL then return( t(args()) ) end_if;
    if args(0) <> 1 then 
	error("wrong no of args")
    end_if;

    case (t:= domtype(_x_))
    of DOM_INT   do
    of DOM_RAT   do
    of DOM_FLOAT do return( stdlib::sign(_x_) )
    end_case:
    
    if testtype( _x_,Type::Constant ) then
	# is _x_ real ? #
        s := float(_x_);
        if domtype(s) = DOM_FLOAT then
            if iszero(s) then return( sign(0) )
            elif s > 0 then return( 1 )
            else return( -1 )
            end_if
        end_if
    end_if;

    case t
    of DOM_COMPLEX do
                 if iszero(_x_) then
                   return(0)
                 else
                 return( _x_/abs(_x_) )
                 end_if
    of DOM_IDENT   do return( procname(_x_) )
    of DOM_EXPR    do
	t := op(_x_,0);
	if domtype(level(t,2)) = DOM_FUNC_ENV and
	  (s:= funcattr(level(t,2),"sign")) <> FAIL then
	    return( s(_x_) )
	elif domtype(level(t,2)) = DOM_IDENT then
	# f(x) #
	    return( procname(args()) )
	end_if;

	if t = hold(_mult) then
	    s := 1; ss := 1;
	    for f in _x_ do 
		t := sign(f);
		if has(t,hold(sign)) then ss := ss*f else s := s*t end_if
	    end_for;
	    if ss = _x_ then
		return( procname(_x_) )
	    else
	        return( s*sign(ss) )
	    end_if
	elif t = hold(_plus) then
	    s := {};
	    for f in _x_ do
		t := sign(f);
		if not iszero(t) then 
		    s := (s union {t});	
		    if nops(s) > 1 then break end_if
		end_if
	    end_for;
	    if nops(s) = 1 then return( op(s,1) ) end_if
	elif t = hold(_power) then
	    if testtype(op(_x_,2),Type::Constant) then
		return( sign(op(_x_,1))^op(_x_,2) )
	    end_if
	end_if
    end_case;

    procname(_x_)
end_proc,
NIL,
table( "type"="sign", "print"="sign", "info"="sign(x) -- the sign of x [try ?sign for details]" )
):

sign:=funcattr(sign, "sign", id):

sign:=funcattr(sign,"series",proc() 
  begin error("no series expansion at the origin")
end_proc):

sign := funcattr(sign,"rectform",
    loadproc(funcattr(sign,"rectform"),pathname("STDLIB","RECTFORM"),
        "sign")
):

sign := funcattr(sign,"diff",
    loadproc(funcattr(sign,"diff"),pathname("SPECFUNC","DIFF"),
        "sign")
):

sign := funcattr( sign,"expand",
proc(_x_)
    local s, ss, t;
begin
    s := Re(_x_);
    if not has(s,{Re,Im}) and s <> _x_ then
        ss := Im(_x_);
        if not has(ss,{Re,Im}) then
            t := sqrt( s^2*sign(s)^2+ss^2*sign(ss)^2 );
            if not iszero(t) then return( (s+I*ss)/t ) end_if
        end_if
    end_if;
 
    hold(sign)(_x_)
end_proc ):

# end of file #
