# $Date: 1999/04/27 08:32:44 $ $Author: stefanw $ $Revision: 1.14.2.1 $ #

# frankp, 01/02/95 #

#++
	abs -- returns the absolute value of an expression

	abs(x)

	x - expression
++#

abs := func_env(
proc(x)
    name abs;
    local t, aa, a, f;
begin
    # test for overloading #
    if x::abs <> FAIL then return( x::abs(args()) ) end_if;

    if args(0) <> 1 then
	error("wrong no of args")
    end_if;

    case domtype(x)
    of DOM_INT     do
    of DOM_RAT     do
    of DOM_FLOAT   do
    of DOM_COMPLEX do return( stdlib::abs(x) )
    of DOM_EXPR    do
	t := op(x,0);
        if domtype(level(t,2)) = DOM_FUNC_ENV then
            if funcattr(level(t,2),"abs") <> FAIL then
                return( funcattr(level(t,2),"abs")(op(x)) )
            end_if
        end_if;

        if t = hold(_mult) then
	    aa := 1; a := 1;
            for f in x do
		t := abs(f);
		if has(t,hold(abs)) then aa := aa*f else a := a*t end_if
	    end_for;
	    if aa <> x then
		if aa <> 1 then return( a*hold(abs)(aa) )
		else return( a )
		end_if
	    end_if
	elif t = hold(_plus) then
            a := Re(x);
            if not has(a,{hold(Re),hold(Im)}) then
                aa := Im(x);
                if not has(aa,{hold(Re),hold(Im)}) then
                   return( expand(sqrt(a^2+aa^2)) )
                end_if
	    end_if
	elif t = hold(_power) then
	    if testtype( op(x,2),Type::RealNum ) then
	        return( abs(op(x,1))^op(x,2) )
	   end_if
        elif t = hold(abs) then
           return(x)
	end_if
    end_case;

    t:= sign(x);
    if t = 0 or t = 1 then x
    elif t = -1 then -x
    else procname(x)
    end_if
end_proc,
NIL,
table( "type"="abs", "print"="abs",
       "info"="abs(x) -- absolute value of x" )
):

abs(I):= 1: abs(PI):=PI: abs(E):=E: abs(EULER):=EULER:

abs:= funcattr(abs, "diff",
    fun(sign(op(args(1),1)) *
        diff(op(args(1),1), args(i) $ hold(i)=2..args(0)))):

abs:= funcattr(abs,"sign",proc(X) begin
    case sign(op(X)) 
    of 0 do return( 0 )
    of -1 do
    of 1 do return( 1 )
    otherwise return( hold(sign)(X) )
    end_case
end_proc):


abs:= funcattr(abs,"rectform",
    loadproc(funcattr(abs,"rectform"),pathname("STDLIB","RECTFORM"),"abs")
):

abs := funcattr( abs, "Re", hold(abs) ):
abs := funcattr( abs, "Im", 0 ):

abs := funcattr( abs, "series", proc(f,x,n,dir) begin
   sign(f/x);
   if contains({1,-1},%) then
      f:=(if dir=Right then %*f elif dir=Left then -%*f else FAIL end_if)
   else
      limit(f,x,dir);
      f:=(if is(%>0)=TRUE then f elif is(%<0) then -f else FAIL end_if)
   end_if;
   if f=FAIL then f else Series::series(f,x,n,dir) end_if
end_proc):

abs := funcattr( abs, "expand", proc() begin
   if type(args(1))="_mult" then map(args(1),abs)
   else abs(args(1)) end_if
end_proc):

# end of file #
