# $Date: 1997/08/07 13:42:28 $ $Author: tonner $ $Revision: 1.20 $ #

# kg, 13/01/94 #

#++
asin -- the inverse sine

asin(x)

x - expression
++#

asin:= proc(x)
    local f;
    name asin;
begin
    if x::asin <> FAIL then return(x::asin(args())) end_if;
    if args(0) <> 1 then error("wrong no of args") end_if;

    case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-asin(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(funcattr(asin,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(funcattr(asin,"float")(x))
	end_if;
	if op(x,1) = 0 then return(asinh(-x*I)*I) end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::RealNum) then
	    if f < 0 then return(-asin(-x)) end_if;
	end_if;
	break;

    of "sin" do # asin : [-1..1] --> [-PI/2..PI/2] #
        if (is(op(x)>=-PI/2) and is(op(x)<=PI/2))=TRUE then return(op(x))
        elif type((f:=round(Re(op(x))/PI)))=DOM_INT then
           return((-1)^f*(op(x)-f*PI))
        end_if; break
    end_case;

    procname(x)
end_proc:

asin(0):= 0:
asin(1/2):= PI/6:
asin((2^(1/2))/2):= PI/4:
asin((3^(1/2))/2):= PI/3:
asin(1):= PI/2:
asin(infinity):= -I*infinity:
asin(-infinity):= I*infinity:
asin(I*infinity):= I*infinity:
asin(-I*infinity):= -I*infinity:


asin:= funcattr(asin,"print","asin"):
asin:= funcattr(asin,"info","asin -- the inverse sine" ):
asin:= funcattr(asin,"type","asin"):

asin:= funcattr(asin,"diff",fun((
    op(args(1),1);
    diff(%, args(2..args(0))) / sqrt(1-%^2)
))):

asin:= funcattr(asin, "series", proc(f,x,n)
   local s,_l,k;
begin
   _l:=limit(f,x);
   if is(abs(_l)<1)=TRUE then
      # formula 4.4.40 in Abramowitz & Stegun #
      if n<2 then s:=[]
      else s:=1; s:=[s,(0,(s:=s*(2*k-1)/2/k)/(2*k+1))$k=1..(n div 2)-1]
      end_if;
      s:=Puiseux::create(1,1,n,s,x);
      if f=x then s
      else Puiseux::_fconcat(s,Series::Puiseux(f,x,n))
      end_if
   elif indets(_l)<>{} or is(abs(1-_l)<2)=TRUE then # formula 4.4.41 #
      s:=1; _l:=1; f:=1-f; for k from 1 to n-2 do
         _l:=_l*f*(2*k-1)/4/k;
         s:=s+_l/(2*k+1)
      end_for;
      Series::series(PI/2-sqrt(2*f)*s,x,n)
   else FAIL
   end_if
end_proc):

asin:= funcattr(asin,"rectform",
    loadproc(funcattr(asin,"rectform"),pathname("STDLIB","RECTFORM"),"asin")
):

# end of file #
