# $Date: 1999/03/08 13:15:06 $ $Author: frankp $ $Revision: 1.26.2.1 $ #

# kg, 13/01/94 #

#++
atan -- the inverse tangens

atan(x)
atan(u,v)

x   - expression
u,v - real quantifiers, representing the complex number u+I*v

atan(x) computes the inverse tangens of an expression x.

atan(u,v) computes the principal value of the argument of the complex
number u+I*v, so -PI < atan(u,v) <= PI.  u and v represents real 
quantifiers. The argument of 0 is defined to be 0, but can be
redefined by 'atan(0,0):= phi' through the user.
++#

atan:= proc(x)
    local f, y, sx, sy;
    name atan;
begin
    if x::atan <> FAIL then return(x::atan(args())) end_if;
    if args(0) < 1 or args(0) > 2 then error("wrong no of args") end_if;
    if args(0) = 2 then
    # the two-argument atan #
	y := args(2);
	if y::atan <> FAIL then return(y::atan(args())) end_if;
	sx := sign(x); 
	sy := sign(y);
	if iszero(sx) and iszero(sy) then
	    # each value between -PI < phi <= +PI is possible here #
	    userinfo(1,"indeterminate function value (return unevaluated)");
	    return( procname(args()) )
	elif domtype(sx) = DOM_COMPLEX or domtype(sy) = DOM_COMPLEX then
	    error("expecting real quantifiers")
	elif iszero(y) then
	    f:= 1/2*(1-sx)*PI
	elif testtype(sx,NUMERIC) then
	    if has(x,stdlib::Infinity ) then
		if testtype(sy,NUMERIC) and not has(y,stdlib::Infinity) then
		    f:= 1/2*sy*(1-sx)*PI
		else
		    return( procname(x,y) )
		end_if
	    elif has(y,stdlib::Infinity) then
		if testtype(sy,NUMERIC) then 
		    f:= 1/2*sy*PI
		else
		    return( procname(x,y) )
		end_if
	    elif iszero(sx) then f:= 1/2*sy*PI
	    elif sx = 1 then f:= atan(y/x)
	    else f:= atan(y/x)+PI*sy
	    end_if
	else
	    return( procname(x,y) )
	end_if;

        if domtype(x) = DOM_FLOAT and domtype(y) = DOM_FLOAT then
            return( float(f) )
        else
            return( f )
        end_if
    end_if;

    case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-atan(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(stdlib::atan(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(stdlib::atan(x))
	end_if;
	if x = I then error("singularity") end_if;
	if op(x,1) = 0 then return(I*atanh(-I*x)) end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::RealNum) then
	    if f < 0 then return(-atan(-x)) end_if;
	end_if;
	break;

    of "tan" do # the real part of atan is in ]-PI/2,PI/2] #
        if (is(op(x)>-PI/2) and is(op(x)<=PI/2))=TRUE then return(op(x))
        elif type((f:=round(-Re(op(x))/PI)))=DOM_INT then return(op(x)+f*PI)
        end_if;
        break;
    end_case;

    procname(x)
end_proc:

atan:=funcattr(atan,"float",proc(x,y)
begin
   if args(0)=1 then stdlib::atan(float(x))
   elif args(0)=2 then atan(float(x),float(y))
   else error("Wrong number of arguments")
   end_if
end_proc):

atan(0):= 0:

atan((3^(1/2))/3):= PI/6:
atan(1):= PI/4:
atan(3^(1/2)):= PI/3:
atan(infinity):= 1/2*PI:
atan(-infinity):= -1/2*PI:
atan(I*infinity):= 1/2*PI:
atan(-I*infinity):= -1/2*PI:
atan(2^(1/2)-1):=PI/8:
atan(-2^(1/2)-1):=-3*PI/8:


atan:= funcattr(atan,"print","atan"):
atan:= funcattr(atan,"info",
"atan -- the inverse tangens (including the two-argument function atan) [try ?atan for help]"):
atan:= funcattr(atan,"type","atan"):

atan:= funcattr(atan,"diff",fun((
    if nops(args(1)) = 1 then op(args(1),1)
    else op(args(1),2)/op(args(1),1)
    end_if;

    diff(%, args(2..args(0))) / (1+%^2)
))):

atan:= funcattr(atan, "series", proc(f,x,n)
   local s,p,l,opt;
begin
   s:= [args()];
   if (p:= contains( s,Right )) = 0 then p:= contains( s,Left ) end_if;
   if p <> 0 then opt:= s[p]; unassign( s[p] ) end_if;

   if nops(s) = 4 then
       // The two-argument arcus tangens
       f:= x/f; x:= n; n:= s[4]
   end_if;

   if n<2 then s:=[]
   else s:=1;s:=[s,(0,(s:=-s)/(2*l+1))$l=1..(n div 2)-1]
   end_if;
   s:=Puiseux::create(1,1,n,s,x);

   if f=x then s
   elif (l:=limit(f,x,Right))=infinity then
      # atan(x)=PI/2-atan(1/x) #
      if traperror( (f:= Puiseux::_fconcat(s,Series::Puiseux(1/f,x,n))) ) <> 0 then
          error("unable to compute series")
      else
          return( PI/2-f )
      end_if
   elif traperror( (f:= Puiseux::_fconcat(s,Series::Puiseux(f,x,n))) ) <> 0 then
      error("unable to compute series")
   else
      return( f )
   end_if
end_proc):

atan:= funcattr(atan,"rectform",
    loadproc(funcattr(atan,"rectform"),pathname("STDLIB","RECTFORM"),"atan")
):

# end of file #
