# #
# $Date: 1996/07/03 13:45:49 $ $Author: frankp $ $Revision: 1.13 $ #

# zimmerma, 16/02/95 #

#++
	besselJ(v,z) is the Bessel function of the first kind:

	besselJ(v,z) = 1/gamma(v+1)*(z/2)^v*0F1(v+1; -z^2/4)
 
            = (z/2)^v * sum( (-z^2/4)^k/k!/gamma(v+1+k), k=0..infinity)
++#

besselJ:=proc(v,z)
begin
   if z::besselJ <> FAIL then return( z::besselJ(args()) ) end_if;

   if type(z)=DOM_FLOAT then return(funcattr(besselJ,"float")(v,z))
   elif type(z)=DOM_COMPLEX then
      if type(op(z,1))=DOM_FLOAT or type(op(z,2))=DOM_FLOAT then
         return(funcattr(besselJ,"float")(v,z))
      end_if
   end_if;
   if v=1/2 then 2^(1/2)/PI^(1/2)/z^(1/2)*sin(z)
   else procname(v,z)
   end_if
end_proc:

besselJ:= funcattr(besselJ,"type","besselJ"):
besselJ:= funcattr(besselJ,"print","besselJ"):
besselJ:= funcattr(besselJ,"info",
  "besselJ(v,z) -- Bessel function of first kind, order v and argument z"
):

besselJ := funcattr(besselJ,"float",
    loadproc(funcattr(besselJ,"float"),pathname("SPECFUNC","FLOAT"),"besselJ")
):

besselJ := funcattr(besselJ,"diff",
    loadproc(funcattr(besselJ,"diff"),pathname("SPECFUNC","DIFF"),"besselJ")
):

besselJ := funcattr(besselJ,"series",
    loadproc(funcattr(besselJ,"series"),pathname("SERIES"),"besselJ")
):

besselJ:= funcattr(besselJ,"expand",
    loadproc(funcattr(besselJ,"expand"),pathname("SPECFUNC","EXPAND"),
        "besselJ"
    )
):


# end of file #
