# $Date: 1996/07/03 13:45:51 $ $Author: frankp $ $Revision: 1.16 $ #

# kg, 13/01/94 #

#++
cosh -- the hyperbolic cosine

cosh(x)

x - expression
++#

cosh:= proc(x)
    local f;
    name cosh;
begin
    if x::cosh <> FAIL then return(x::cosh(args())) end_if;
    if args(0) <> 1 then error("wrong no of args") end_if;

    case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(cosh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(funcattr(cosh,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(funcattr(cosh,"float")(x))
	end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::RealNum) then
	    if f < 0 then return(cosh(-x)) end_if;
	end_if;
	break;

    # cf Table 4.5.60 p. 85 in Abramowitz and Stegun #
    of "acosh" do return(op(x))
    of "asinh" do return(sqrt(1+op(x)^2))
    of "atanh" do return(1/sqrt(1-op(x)^2))
    # hold(_power) is needed because sqrt(1/z) evaluates to 1/sqrt(z) #
    of "acoth" do return(sqrt(hold(_power)(1-1/op(x)^2,-1)))

    end_case;

    procname(x)
end_proc:

cosh(0):= 1:

cosh:= funcattr(cosh, "print", "cosh"):
cosh:= funcattr(cosh, "type", "cosh"):
cosh:= funcattr( cosh, "info", "cosh -- the hyperbolic cosine"):

cosh:= funcattr(cosh, "diff", fun((
    op(args(1),1);
    diff(%, args(2..args(0))) * sinh(%)
))):

cosh := funcattr(cosh, "expand",
    loadproc(funcattr(cosh,"expand"),pathname("SPECFUNC","EXPAND"),"cosh")
):

cosh:= funcattr(cosh, "conjugate",
    loadproc(funcattr(cosh,"conjugate"),pathname("STDLIB","CONJ"),"cosh")
):

cosh:= funcattr(cosh,"rectform",
    loadproc(funcattr(cosh,"rectform"),pathname("STDLIB","RECTFORM"),"cosh")
):

cosh:= funcattr(cosh, "series",
    loadproc(funcattr(cosh,"series"),pathname("SERIES"),"cosh")
):

cosh:= funcattr(cosh,"Re",
    loadproc(funcattr(cosh,"Re"),pathname("STDLIB","RE"),"cosh")
):

cosh:= funcattr(cosh,"Im",
    loadproc(funcattr(cosh,"Im"),pathname("STDLIB","IM"),"cosh")
):

# end of file #
