
# ok, 27/01/94 #

#++
coth -- the hyperbolic cotangens

coth(x)

x - expression
++#

coth:= proc(x)
    local f;
    name coth;
begin
    if x::coth <> FAIL then return(x::coth(args())) end_if;
    if args(0) <> 1 then error("wrong no of args") end_if;

    case type(x)
    of DOM_INT do
    of DOM_RAT do
        if x = 0 then
           error("singularity")
	elif x < 0 then 
           return(-coth(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(funcattr(coth,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(funcattr(coth,"float")(x))
	elif op(x, 1) = 0 then
            return(-I*cot(-I*x))
	elif op(x, 1) < 0 and op(x, 2) < 0 then
            return(-coth(-x))
	end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::RealNum) then
	    if f < 0 then return(-coth(-x)) end_if;
	end_if;
	break;

    of "acoth" do return(op(x));
    of "asinh" do return(sqrt(op(x)^2+1)/op(x));
    of "acsch" do return(sqrt(op(x)^2+1));
    of "acosh" do return(op(x)/sqrt(op(x)^2-1));
    of "asech" do return(1/sqrt(1-op(x)^2));
    of "atanh" do return(1/op(x));

    end_case;

    procname(x)
end_proc:

coth:= funcattr(coth, "type", "coth"):
coth:= funcattr(coth, "print", "coth"):
coth:= funcattr( coth, "info", "coth -- the hyperbolic cotangent"):

coth:= funcattr(coth, "diff", fun(
    (op(args(1),1);
    diff(%, args(2..args(0))) * (1-coth(%)^2))
)):

coth:= funcattr(coth, "float",fun((
    # coth(e) = 1/tanh(e) #
    if args(1) = 0 or args(1) = 0.0 then
       error("singularity")
    else
       1/funcattr(tanh, "float")(args(1))
    end_if
))):

coth:= funcattr(coth, "expand", fun(expand(cosh(args(1))/sinh(args(1)))) ):

coth:= funcattr( coth, "conjugate",
    loadproc(funcattr(coth,"conjugate"),pathname("STDLIB","CONJ"),"coth")
):

coth:= funcattr(coth,"rectform",
    loadproc(funcattr(coth,"rectform"),pathname("STDLIB","RECTFORM"),"coth")
):

coth:=funcattr(coth,"series",proc(f,x,n)
begin
  f:=Series::Puiseux(tanh(f),x,n); Puiseux::_invert(f)
end_proc):

coth:= funcattr(coth,"Re",
    loadproc(funcattr(coth,"Re"),pathname("STDLIB","RE"),"coth")
):

coth:= funcattr(coth,"Im",
    loadproc(funcattr(coth,"Im"),pathname("STDLIB","IM"),"coth")
):

# end of file #
