# The dilogarithm function: dilog(x) = int(ln(t)/(1-t),t=1..x)

  Reference: Abramowitz & Stegun, Handbook of Mathematical Functions,
	section 27.7

To do: change to Li2(x) = int(-log(1-t)/t,t=0..x) like Mma and Macsyma.
Ref: L. Lewin, Dilogarithms and associated functions, Macdonald, London, 1958
     L. Lewin, Polylogarithms and associated functions, 1981
From Harald Anlauf <anlauf@crunch.ikp.physik.th-darmstadt.de>.
     Other notation ? polylog(n,x) == Li_n(x)
                      polylog(1,x) = -ln(1-x)
                      int( polylog(n,t)/t, t=0..x) = polylog(n+1,x)
                      int(ln(1-x)/x) = -polylog(2,x)
See PR 485.

  Functional relations:
  
[27.7.3]	dilog(x)+dilog(1-x) = -ln(x)*ln(1-x)+PI^2/6 for 0<=x<=1

[27.7.4]	dilog(1-x)+dilog(1+x) = 1/2*dilog(1-x^2) for 0<=x<=1

#

dilog := proc(x)
begin
   if x::dilog<>FAIL then return(x::dilog(args())) end_if;

   if args(0)<>1 then error("wrong number of arguments") end_if;

   case type(x)
      of DOM_FLOAT do return(funcattr(dilog,"float")(x))
   end_case;

   procname(x)
end_proc:
dilog(0):=PI^2/6:
dilog(1/2):=PI^2/12-ln(2)^2/2: # from 27.7.3 with x=1/2 #
dilog(1):=0:
dilog(2):=-PI^2/12:
dilog(infinity):=-infinity:

dilog:=funcattr(dilog,"print","dilog"):
dilog:=funcattr(dilog,"info","dilog -- the dilogarithm function dilog(x) = int(ln(t)/(1-t),t=1..x)"):
dilog:=funcattr(dilog,"type","dilog"):

dilog:=funcattr(dilog,"diff",fun((
   op(args(1)); ln(%)*diff(%,args(2))/(1-%)
))):

dilog:=funcattr(dilog,"discont",fun({})):

dilog:=funcattr(dilog,"float",
	fun(float(hold(int)(ln(hold(t))/(1-hold(t)),
		hold(t)=1..args(1))))):
