# ok, 29/10/94 #

#++
The Digamma and Polygamma functions

psi(x)
psi(x, n)

x - an expression
n - an non negative integer
++#

psi:=proc(x, n)
  name psi;
local i;
begin
  if x::psi <> FAIL then return(x::psi(args())) end_if;

  if testtype(x, NUMERIC) then
          Re(float(x));
          if % <= 0.0 and Im(x) = 0 and frac(%) = 0.0 then
             error("singularity encountered");
          end_if
       end_if;

  case args(0) 

    of 1 do

       case type(x) 

         of DOM_INT do
            if x < 100 and 1 < x then # See apendix #
               return( 1/(x-1)+psi(x-1) );
            end_if;
            break;

         of DOM_FLOAT do 
            return( stdlib::psi(x) ); 

         of DOM_COMPLEX do
            if type(op(x,1)) = DOM_FLOAT or type(op(x,2)) = DOM_FLOAT then
               return( stdlib::psi(x) ); 
            end_if;
            break;

         of DOM_RAT do
            if op(x,2)=2 and op(x,1)>=3 then
            # use psi(1/2) known and psi(x+1)=psi(x)+1/x #
               return(_plus(psi(1/2),2/(2*i-1)$i=1..op(x,1) div 2))
            end_if
       end_case;
 
       break;

    of 2 do 

       if testtype(n, NUMERIC) then
          if type(n) <> DOM_INT or n < 0 then
             error("second argument must be a non negative integer");
          elif n = 0 then 
             return( psi(x) );
          end_if;
             
          case type(x) 

            of DOM_INT do
            of DOM_RAT do
               if x = 1 then
                  return( (-1)^(n+1)*fact(n)*zeta(n+1) );
               elif x = 1/2 then
                  return( (-1)^(n+1)*fact(n)*(2^(n+1)-1)*zeta(n+1) );
	       elif type(2*x) = DOM_INT and abs(x) < 100-n then # See apendix #
                  if x < 0 then
                     return( psi(x+1, n)-(-1)^n*fact(n)/x^(n+1) );
                  else
                     return( (-1)^n*fact(n)/(x-1)^(n+1)+psi(x-1, n) );
                  end_if
                end_if;
                break;

            of DOM_FLOAT do
               return( stdlib::floatpsi(x, n) );

            of DOM_COMPLEX do
               if type(op(x,1)) = DOM_FLOAT or type(op(x,2)) = DOM_FLOAT then
                  return( stdlib::floatpsi(x, n) );
               end_if;
               break;
               
          end_case;
       end_if;

       break;

     otherwise

       error("wrong no of args");

  end_case;

  procname(args())

end_proc:

psi(1) := -EULER:
psi(1/2) := -EULER-2*ln(2):


psi:= funcattr(psi,"type","psi"):
psi:= funcattr(psi,"print","psi"):
psi:= funcattr(psi,"info","psi -- the Digamma and Polygamma functions [try ?psi for options]"):

psi:= funcattr(psi,"diff",fun((
    op(args(1),1);
    if nops(args(1)) > 1 then
	diff(%, args(2..args(0))) * psi(%, op(args(1),2)+1)
    else
        diff(%, args(2..args(0))) * psi(%, 1)
    end_if
))):

# Appendix:                                        #
# Due to the complexity to compute psi(x), where x #
# is an integer, compute psi(x) only if x < 100    #
# Also restrict the computation of psi(x,n), where #
# 2*x is an integer. Compute psi(x,n) only if      #
# abs(x) < 100-n. This restrict the number of      #
# recursive calls of psi and the fractional part   #
# of the result.                                   #

psi := funcattr(psi, "float", proc(x, n) begin
    case args(0) 
      of 1 do return( psi(float(x)) );
      of 2 do return( stdlib::floatpsi(x,n) );
    end_case;
    error("wrong no of arguments");
end_proc):

stdlib::floatpsi:= loadproc(
    stdlib::floatpsi,pathname("SPECFUNC","FLOAT"),"psi"
):

psi:= funcattr(psi, "expand",
    loadproc(funcattr(psi,"expand"),pathname("SPECFUNC","EXPAND"),"psi")
):

psi:=funcattr(psi,"series",
    loadproc(funcattr(psi,"series"),pathname("SERIES"),"psi")
):

# end of file #
