# $Date: 1996/07/03 13:46:04 $ $Author: frankp $ $Revision: 1.14 $ #

# kg, 13/01/94 #

#++
sinh -- the hyperbolic sine

sinh(x)

x - expression
++#

sinh:= proc(x)
    local f;
    name sinh;
begin
    if x::sinh <> FAIL then return(x::sinh(args())) end_if;
    if args(0) <> 1 then error("wrong no of args") end_if;

    case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-sinh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(funcattr(sinh,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(funcattr(sinh,"float")(x))
	end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::RealNum) then
	    if f < 0 then return(-sinh(-x)) end_if;
        elif type((f:=x/I/PI))=DOM_INT then return(0) # sinh(n*I*PI)=0 #
        elif type(2*f)=DOM_INT then return((-1)^(f-1/2)*I)
	end_if;
	break;
    
    # cf Table 4.5.60 p. 85 of Abramowitz and Stegun #
    of "asinh" do return(op(x))
    of "acosh" do return(sqrt(op(x)^2-1))
    of "atanh" do return(op(x)/sqrt(1-op(x)^2))
    of "acoth" do return(1/sqrt(op(x)^2-1))

    end_case;

    procname(x)
end_proc:

sinh(0):= 0:
sinh(infinity):=infinity:

sinh:= funcattr(sinh,"info","sinh -- the hyperbolic sine"):
sinh:= funcattr(sinh, "print", "sinh"):
sinh:= funcattr(sinh, "type", "sinh"):

sinh:= funcattr(sinh, "diff", fun((
    op(args(1),1);
    diff(%, args(2..args(0))) * cosh(%)
))):

sinh := funcattr(sinh, "expand",
    loadproc(funcattr(sinh,"expand"),pathname("SPECFUNC","EXPAND"),"sinh")
):

sinh:= funcattr(sinh,"conjugate",
    loadproc(funcattr(sinh,"conjugate"),pathname("STDLIB","CONJ"),"sinh")
):

sinh:= funcattr(sinh,"rectform",
    loadproc(funcattr(sinh,"rectform"),pathname("STDLIB","RECTFORM"),"sinh")
):

sinh:= funcattr(sinh, "series",
    loadproc(funcattr(sinh,"series"),pathname("SERIES"),"sinh")
):

sinh:= funcattr(sinh,"Re",
    loadproc(funcattr(sinh,"Re"),pathname("STDLIB","RE"),"sinh")
):

sinh:= funcattr(sinh,"Im",
    loadproc(funcattr(sinh,"Im"),pathname("STDLIB","IM"),"sinh")
):

# end of file #
