# $Date: 1997/08/07 13:42:32 $ $Author: tonner $ $Revision: 1.14 $ #

# kg, 13/01/94 #

#++
tanh -- the hyperbolic tangens

tanh(x)

x - expression
++#

tanh:= proc(x)
    local f;
    name tanh;
begin
    if x::tanh <> FAIL then return(x::tanh(args())) end_if;
    if args(0) <> 1 then error("wrong no of args") end_if;

    case type(x)
    of DOM_INT do
    of DOM_RAT do
	if x < 0 then return(-tanh(-x)) end_if;
	break;
    
    of DOM_FLOAT do
	return(funcattr(tanh,"float")(x));

    of DOM_COMPLEX do
	if domtype(op(x,1)) = DOM_FLOAT or
	   domtype(op(x,2)) = DOM_FLOAT then
	    return(funcattr(tanh,"float")(x))
	end_if;
	break;

    of "_mult" do
	f:= op(x, nops(x));
	if testtype(f, Type::RealNum) then
	    if f < 0 then return(-tanh(-x)) end_if;
        elif type((f:=x/I/PI))=DOM_INT then return(0) # tanh(k*I*PI)=0 #
        elif type(2*f)=DOM_INT then return((-1)^(f-1/2)*I*infinity)
	end_if;
	break;

    # Table 4.5.60 p. 85 of Abramowitz and Stegun #
    of "atanh" do return(op(x))
    of "asinh" do return(op(x)/sqrt(1+op(x)^2))
    of "acosh" do return(sqrt(op(x)^2-1)/op(x))
    of "acoth" do return(1/op(x))

    end_case;

    procname(x)
end_proc:

tanh(0):= 0:
tanh(infinity):=1:

tanh:= funcattr(tanh,"info","tanh -- the hyperbolic tangent"):
tanh:= funcattr(tanh,"print","tanh"):
tanh:= funcattr(tanh,"type","tanh"):

tanh:= funcattr(tanh,"diff",fun((
    op(args(1),1);
    diff(%, args(2..args(0))) / cosh(%)^2
))):

tanh:= funcattr(tanh, "expand", fun(expand(sinh(args(1))/cosh(args(1)))) ):

tanh:= funcattr(tanh, "conjugate",
    loadproc(funcattr(tanh,"conjugate"),pathname("STDLIB","CONJ"),"tanh")
):

tanh:= funcattr(tanh,"rectform",
    loadproc(funcattr(tanh,"rectform"),pathname("STDLIB","RECTFORM"),"tanh")
):

tanh:= funcattr(tanh,"series",proc(f,x,n)
begin
   series(sinh(f),x,n)/series(cosh(f),x,n)
end_proc):

tanh:= funcattr(tanh,"Re",
    loadproc(funcattr(tanh,"Re"),pathname("STDLIB","RE"),"tanh")
):

tanh:= funcattr(tanh,"Im",
    loadproc(funcattr(tanh,"Im"),pathname("STDLIB","IM"),"tanh")
):

# end of file #
