
# ok, 21/01/94 #

#++
The Riemann Zeta function

zeta(x)

x - an expression
++#

zeta := proc(x)

begin
    if x::zeta <> FAIL then return(x::zeta(args())) end_if;

    if args(0) <> 1 then
       error("1 argument expected.");
    end_if;

    case domtype(x) 

      of DOM_FLOAT do
         
         # first try Apery-like formulas for x=2*n+1 #
         if expr2text(frac(x))="0.0" and round(x) mod 2=1 then
            funcattr(zeta, "float".round(x));
            if %<>FAIL then return(%()) end_if
         end_if;
	 return( funcattr(zeta, "float")(x) );

      of DOM_INT do

        if x < 0 then
           if x mod 2 = 0 then 
              return ( 0 )
           elif -x < 100 then # See apendix #
	      return( -bernoulli(1-x) / (1-x) )
           end_if
        elif x > 0 then
	     if x mod 2 = 0 and x <= 100 then # See apendix #
                return( 1/2*(2*PI)^x*abs(bernoulli(x))/fact(x) )
	     elif x = 1 then
                error("singularity encountered.")
	     end_if
	else
	     return( -1/2 )
	end_if;

	break;

      of DOM_COMPLEX do

        if domtype(op(x, 1)) = DOM_FLOAT or domtype(op(x, 2)) = DOM_FLOAT then
           return( funcattr(zeta, "float")(x) );
	end_if;
        break;

     end_case;

     procname(x)
end_proc:

zeta:= funcattr(zeta,"print","zeta"):
zeta:= funcattr(zeta,"info",
   "zeta -- the Riemann zeta function [try ?zeta for details]"
):
zeta:= funcattr(zeta,"type","zeta"):
zeta:= funcattr(zeta,"series",proc(f,x,n) begin
   if limit(f,x)=1 then
      Puiseux::create(1,-1,1,[1,EULER],x,1) @ Series::Puiseux(f,x,n)
   else Series::unknown(zeta(f),x,n)
   end_if
end_proc):

# these procedures give a faster numerical evaluation at x=3, 5, 7
  using Apery-like formulas. Reference: Empirically Determined Ape'ry-Like
  Formulae for zeta(4*n+3), by Jonathan Borwein and David Bradley,
  http://www.cecm.sfu.ca 
  For DIGITS=300, zeta::float3 is about 8 times faster than the standard
  routine, zeta::float5 is about 9 times faster, zeta::float7 is about 7
  times faster.
#
zeta:=funcattr(zeta,"float3",
proc() local c,s,n;
begin
   c:=2.0; s:=0; 
   for n from 1 to 1+iquo(5*DIGITS,3) do
      s:=s+1/n^3/c; c:=-c*(4*n+2)/(n+1);
   end_for;
   5*s/2
end_proc):

zeta:=funcattr(zeta,"float5",
proc() local a,c,s,n,g;
begin
   a:=0; c:=2.0; s:=0;
   for n from 1 to 1+iquo(5*DIGITS,3) do
      g:=1.0/n^2; s:=s+(g-1.25*a)/n^3/c; c:=-c*(4*n+2)/(n+1); a:=a+g;
   end_for;
   2*s
end_proc):

zeta:=funcattr(zeta,"float7",
proc() local a,c,s,n,g;
begin
   a:=0; c:=2.0; s:=0;
   for n from 1 to 1+iquo(5*DIGITS,3) do
      g:=1.0/n^4; s:=s+(5*a+g)/n^3/c; c:=-c*(4*n+2)/(n+1); a:=a+g;
   end_for;
   5*s/2   
end_proc):

# Due to the complexity to compute the nth Bernoulli #
# number, compute bernoulli(n) only if n-1 <= 100    #

# end of file #
