
/*

Usage:
linReg(l)
linReg(c1,c2)

linear regression,
returns least square estimators a and b of function f(x)=a+b*x

Parameters:
l - list of rows with 2 elements each
c1, c2 - columns (lists of equal length)

Example:

lR := stats::linReg([[1,2],[3,5]]); 

	[1/2, 3/2]

a := lR[1]; b := lR[2]; 


*/

stats::linReg := proc(ll:DOM_LIST)
  local z, l, l_, a, b;
begin

if testargs() then
  if args(0)=2 then
    if type(args(2))<> DOM_LIST then
      error("Both arguments must be lists")
    end_if;
    if nops(args(1))<> nops(args(2)) then
      error("Lists must have the same length")
    end_if
  elif args(0)=1 then
    if map({op(ll)},nops)<>{2} then
      error("Rows must consist of exactly two entries each")
    end_if
  else
    error("Wrong number of arguments")
  end_if
end_if;

if args(0)=2 then
  l:=args(1), args(2)
else
  l:=map(ll, op, 1), map(ll, op, 2)
end_if;

  /* calculation */
  if op(l, 1) = [] then
    FAIL /* division by zero */
  else
    l_:=map(l, proc() begin _plus(op(args()))/nops(args()) end_proc ); /* mean */
    l:=zip(l, proc() begin [args(1) - op(l_, 1), args(2) - op(l_, 2)] end_proc );
    z:=_plus(op(map(l, proc() begin op(args(), 1)^2 end_proc )));
    if z = 0 then return(FAIL) end_if;
    b:=_plus(op(map(l, proc() begin op(args(), 1)*op(args(), 2) end_proc )))/z;
    a:=op(l_, 2)-b*op(l_, 1);
    return([a, b])
  end_if
end_proc:


