# $Date: 1997/03/05 12:09:24 $  $Author: olli $  $Revision: 1.10 $ #

# kg, 29/06/93 #

#++
Poly -- creates a domain of kernel polynomials

Poly(X [, R])

X - list of indeterminates
R - (optional) coefficient ring 'Expr', 'IntMod(n)' or domain,
    default is 'Expr'

Poly creates a domain of MuPAD kernel polynomials (CAT_POLY) with
indeterminates 'X' over the coefficient ring 'R'. If no ring is
given, 'Expr' is choosen as ring.

The main application of a domain created by Poly is the recursive
representation of polynomials: A domain created by Poly may be used
as a coefficient ring for kernel polynomials. Expressions like

	poly(x*y, [x], Poly([y]))

may be used to define polynomials which have kernel polynomials as
coefficients, giving rise to a recursive representation of polynomials.
(The internal representation of kernel polynomials is a distributive
one.)
++#

Poly:= proc(X, R)
    local D1;
    option remember;
begin
    if args(0) = 1 then
	R:= hold(Expr)
    elif testargs() then
	if args(0) < 1 or args(0) > 2 then
	    error("wrong no of args")
	end_if;
    end_if;

    D1:= domain(hold(Poly)(X,R));
    if D1::zero <> FAIL then return(D1) end_if;

    if domtype(R) = DOM_DOMAIN then
	D1::zero:= poly(R::zero, X, R);
	D1::one:= poly(R::one, X, R)
    else
	D1::zero:= poly(0, X, R);
	D1::one:= poly(1, X, R)
    end_if;

    D1::indets:= X;
    D1::coeffRing:= R;
    D1::_plus:= _plus;
    D1::_mult:= _mult;
    D1::_power:= _power;
#
    Eine Version mit Special-Bits benoetigt den folgenden Eintrag:
    D1::invert:= - id;
#
    D1::_negate:= _negate;
    D1::norm:= norm;
    D1::_divide:= fun(divide(args(1), args(2), hold(Exact)));
    D1::diff:= diff;
    D1::intmult:= multcoeffs;
    D1::expr:= expr;
    D1::gcd:= gcd;
    D1::convert:= newpurefunc("fun(...)", NIL,
			      subsop(hold(poly(args(1),hold(X), R)),
				     [2,1]=X, 3=R),
			      FALSE);
    D1::new:= D1::convert;
    D1
end_proc:

