RootOf:=domain("RootOf"):

RootOf::new := proc(eq,z)
begin
   if args(0)=1 then
      z:=indets(eq);
      if nops(z)<>1 then error("you should specify the RootOf variable") end_if;
      z:=op(z)
   end_if;
   if type(eq)=DOM_POLY then eq:=expr(eq) end_if;
   eq:=numer(eq);
   if testtype(eq,Type::PolyExpr(z,Type::Rational)) then eq:=primpart(eq,[z]) end_if;
   new(RootOf,eq,z)
end_proc:

RootOf::print := proc(a)
begin
   if indets(extop(a,1),PolyExpr)={extop(a,2)} then hold(RootOf)(extop(a,1))
   else hold(RootOf)(extop(a))
   end_if
end_proc:

RootOf::expr := proc(a) begin hold(RootOf)(extop(a)) end_proc:

# normalizes the fraction f in x, where x=RootOf(p) #
RootOf::evala := proc(f,p,_x)
local l;
begin
   userinfo(1,"f=",f,"p=",p,"x=",_x);
   l:=gcdex(denom(f),p,_x); 
   # l[1]=denom(f)*l[2]+p*l[3] thus 1/denom(f) = l[2]/l[1] mod p #
   if has(l[1],_x) then error("singular denominator") end_if;
   numer(f)*l[2]/l[1]
end_proc:

# computes the sum of the terms in f, where x goes through the roots of p #
RootOf::polysum := proc(f,p,x)
local d,y;
begin
   f:=divide(f,p,[x],Rem);
   d:=degree(p,[x]);
   if not has(f,x) then d*f
   else
      f:=resultant(p,numer(y-f),x);
      normal(-coeff(f,[y],d-1)/coeff(f,[y],d))
   end_if
end_proc:

RootOf::diff := proc(f,x) local P,_y,yy,yp;
begin
   if args(0)=1 then f
   elif args(0)=2 then
      _y:=extop(f,2);
      if x=_y then error("cannot differentiate with respect to RootOf variable")
      end_if;
      P:=extop(f,1);
      if testtype(P,Type::PolyExpr(_y)) then
         yp:=(if type(_y)=DOM_IDENT then genident(expr2text(_y))
	 elif type(_y)="function" then genident(expr2text(op(_y,0)))(op(_y))
	 else genident() end_if);
         P:=subs(P,_y=yy); # to avoid problems if y is not a name #
         RootOf(resultant(P,diff(P,x)+yp*diff(P,yy),yy),yp)
      else error("can differentiate only algebraic equations") 
      end_if
   else RootOf::diff(RootOf::diff(f,x),args(3..args(0)))
   end_if
end_proc:

RootOf::has:=proc(x,s) begin has(extop(x,1),s) end_proc:

RootOf::int:=proc() begin hold(int)(args()) end_proc:

# 
>> float(RootOf(x^4+x+1));

0.7271360845 + 0.9340992894 I, 0.7271360845 - 0.9340992894 I,

- 0.7271360845 - 0.4300142883 I, - 0.7271360845 + 0.4300142883 I

>> float(RootOf(-y^2+RootOf(0.7271360845+0.9340992894*I+y^2+1,y),y));

- 0.9340992894 + 0.7271360845 I, 0.9340992894 - 0.7271360845 I,

- 0.7271360845 - 0.9340992894 I, 0.7271360845 + 0.9340992894 I
#
RootOf::float:=proc(f) local l,x,v,i;
begin
   hastype; l:=Indets(op(f,1),RootOf);
   if nops(l)=0 then op(float(hold(solve)(op(f))))
   else
      x:=op(l,1); for i from 2 to nops(l) do
         if length(op(l,i))<length(x) then x:=op(l,i) end_if
      end_for;
      v:=[float(x)];
      float(subs(f,x=v[i]))$hold(i)=1..nops(v)
   end_if
end_proc:
