# #
# $Date: 1999/01/29 10:36:38 $ $Author: frankp $ $Revision: 1.9.2.1 $ #
# #
# Sergei O. Ivanov, June 23, 1995 #
# #
# Translated from Maple to MuPAD by P. Zimmermann, August 4, 1995 #
# Modifications: June 24-28 -- irrational inequalities #

#++
	INEQS - Inequalities solver

	Note: this solver can deal with rational and irrational 
	      inequalities, as well as systems of inequalities.
++#

alias(ineqs=stdlib::ineqs):

ineqs:=domain("stdlib::ineqs"):

alias(num_order=fun(float(args(1))<float(args(2)))):

alias(hastype(x,t)=has(map([op(x)],type),t)):

alias(iseven(x)=(if type(x)=DOM_INT then bool(modp(x,2)=0) else FALSE end_if)):

# indets(x,radical) #
alias(radindets(x)=select(select(indets(x),testtype,"_power"),
    fun(type(op(args(1),2))=DOM_RAT))):

ineqs::intersect := proc(a)
local i, res;
begin
 if type(a)<>DOM_SET then error("1st argument has to be a set") end_if;
 if a={} then return(a) end_if;
 res:=op(a,1);
 for i from 2 to nops(a) do res:=ineqs::intrsct(res,op(a,i)) end_for;
 res
end_proc:

ineqs::intrsct := proc(A,B)
local a, b, ex, L, lh, rh;
begin
 a := A; b := B;
 if type(a)=DOM_SET and hastype(a,"_unequal") then
  a := map(a,
    proc(elem) begin
       if hastype(elem,"_unequal") and type(elem)<>DOM_LIST then
          [elem]
       else
          elem
       end_if
    end_proc
   )
 end_if;
 if type(b)=DOM_SET and hastype(b,"_unequal") then
  b := map(b,
     proc(elem) begin
        if hastype(elem,"_unequal") and type(elem)<>DOM_LIST then
           [elem]
        else
           elem
        end_if
     end_proc)
 end_if;
 if A = [] then return(A) elif B = [] then return(B) end_if;

 if hastype({op(a)}, DOM_LIST) and not hastype({op(b)}, DOM_LIST) then
  return(subs(map(a,ineqs::intrsct,b),[]=null())) # eval not needed ? #
 elif hastype({op(b)}, DOM_LIST) and not hastype({op(a)}, DOM_LIST) then
  return(subs(map(b,ineqs::intrsct,a),[]=null()))
 elif hastype({op(a)}, DOM_LIST) and hastype({op(b)}, DOM_LIST) then
  return(map(subs(map(a,ineqs::intrsct,b),[]=null()),op))
 end_if;

 if op(a,[1,1]) = op(a,[2,2]) and type(op(a,1))="_less" and
    type(op(a,2))="_less" then
  return(b,a)
 end_if;
 if op(b,[1,1]) = op(b,[2,2]) and type(op(b,1))="_less" and
    type(op(b,2))="_less" then
  return(a,b)
 end_if;

 if type(a)="_equal" then a := [op(a,1)>=op(a,2), op(a,1)<=op(a,2)] end_if;
 if type(b)="_equal" then b := [op(b,1)>=op(b,2), op(b,1)<=op(b,2)] end_if;

 L := b;
 for ex in [a, b] do
  if float(op(ex,[1,1])) >= float(op(L,[1,1])) and
     float(op(ex,[1,1])) <= float(op(L,[2,2])) then
   lh := op(ex,1); break end_if;
  L := a
 end_for;
 L := b;
 for ex in [a, b] do
  if float(op(ex,[2,2])) <= float(op(L,[2,2])) then
   rh := op(ex,2); break end_if;
  L := a
 end_for;
 if type(lh)=DOM_IDENT or type(rh)=DOM_IDENT then return([]) end_if;
 if op(lh,1) = op(rh,2) then
  if type(lh)="_leequal" and type(rh)="_leequal" then return([lh, rh])
   else return([]) end_if
 end_if;
 [lh, rh]
end_proc:

ineqs::in_interval := proc(interval,pt)
local i;
begin
    for i in interval do
        if op(i,[1,1])=-infinity then # always greater #
           if op(i,[2,2])=infinity then return(TRUE)
           elif bool(map(subsop(op(i,2),1=pt),float)) then return(TRUE)
           end_if
        elif bool(map(subsop(op(i,1),2=pt),float)) then
           if op(i,[2,2])=infinity then return(TRUE)
           elif bool(map(subsop(op(i,2),1=pt),float)) then return(TRUE)
           end_if
        end_if
    end_for;
    FALSE
end_proc:

ineqs::left := proc(interv)
local link, ev_list;
begin
 link := [op(map(interv,op,[1,1]))];
 ev_list := map(link,fun(float(args(1))=args(1)));
 link := subs(sort(float(link), num_order),op(ev_list));
 link[1];
end_proc:

ineqs::right := proc(interv)
local recht, ev_list;
begin
 recht := [op(map(interv,op,[2,2]))];
 ev_list := map(recht,fun(float(args(1))=args(1)));
 recht := subs(sort(float(recht), num_order),op(ev_list));
 recht[nops(recht)];
end_proc:

ineqs::simplify := proc(ineq)
local expR, res;
begin
 res := null();
 for expR in ineq do
  if op(expR,[1,1]) = op(expR,[2,2]) then
   if type(op(expR,1))="_leequal" and type(op(expR,2))="_leequal" and
      (not ineqs::in_interval(ineq minus {expR}, op(expR,[1,1])) or nops(ineq) = 1) then
    res := res, op(expR,[1,2])=op(expR,[1,1])
   elif (ineqs::in_interval(ineq minus {expR}, op(expR,[1,1])) and
   not (type(op(expR,1))="_leequal" and type(op(expR,2))="_leequal")) 
   or nops(ineq) = 1 then
    res := res, op(expR,[1,2])<>op(expR,[1,1])
   end_if
  else
   res := res, expR
  end_if
 end_for;
 {res}
end_proc:

ineqs::test := proc(eqn, ineq, def)
local eps, ex, l, r;
begin
   if has(eqn,infinity) then return(def) end_if;
   if type(ineq)=DOM_SET or type(ineq)=DOM_LIST then
      if traperror((ex := map(float(subs(ineq,eqn)),bool)))<>0 then FALSE
      elif has(ex, TRUE) and not has(ex, FALSE) then TRUE
      else FALSE
      end_if
   else
      # PZ: added expand, and eval is required for abs(infinity) for example #
      # OK: check result of floating point evaluation:                       #
      #     1) complex numbers with imaginary part 0.0                       #
      #     2) cut off more than DIGITS digits (needed for comparison)       #
      if traperror(
              # (ex:=map(expand(eval(subs(ineq,eqn))),float)) #
              (ex:=map(expand(subs(ineq,eqn,Simplified)),float))
         ) <> 0 then FALSE
      else
         l := op(ex, 1); r := op(ex, 2); eps := 10^DIGITS;
         if testtype(r, DOM_COMPLEX) and Im(r) = 0.0 then
            r := Re(r);
         end_if;
         if testtype(l, DOM_COMPLEX) and Im(l) = 0.0 then     
            l := Re(l); 
         end_if;
         r := trunc(r*eps)/eps; l:= trunc(l*eps)/eps;
         ex := op(ex,0)(l, r);
         if traperror((ex:=bool(ex))) <> 0 then FALSE else ex end_if
      end_if
   end_if
end_proc:

# returns either FAIL or a set of solutions #
ineqs::ineqs := proc(ineq,var)
local ex, crit_pts, eq, neq, sls, solint, sol, interval, interv,
      DPV, ev_list, i, check_point, rdcs;
option remember;
begin
    if type(ineq)=DOM_SET or type(ineq)=DOM_LIST then
     sls:=map(ineq,ineqs::ineqs,var);
     if has(sls,FAIL) then return(FAIL) end_if;
     sls := ineqs::intersect({op(sls)});
     return(sls);
    end_if;
    ex := ineq; # bool(ineq) #
    if type(ineq)="_equal" or type(ineq)="_unequal" then
     sol := solve(op(ineq,1)-op(ineq,2)=0,var);
     if type(sol) = "solve" then sol := {}
     else
      if type(ineq)="_equal" then
	sol:=map(sol,fun([var>=args(1),var<=args(1)]))
      else
	sol := map(sol,fun([var>args(1),var<args(1)]))
      end_if
     end_if;
     return(sol);
    end_if;

    eq := normal(op(ex,1)-op(ex,2));
    crit_pts := {}; neq := 0; # PZ: replaced null() by {} #
    if type(eq)= "_mult" then
     neq:=select(eq,testtype,Type::RealNum);
     neq := eq/neq;
     if denom(neq) <> 1 then
      # PZ: replaced a by denom(neq) #
      crit_pts := solve(denom(neq)=0, var);
      if type(crit_pts)=DOM_SET then
         crit_pts:=select(map(crit_pts,normal),fun(Im(args(1))=0))
      end_if;
      eq := eq*denom(eq)^2;
     end_if
    end_if;
    DPV := {[var>-infinity, var<infinity]};
    rdcs:=radindets(ineq);
    if rdcs<>{} then # hastype(ineq, radical) #
     DPV := null();
     if denom(neq) <> 1 and has(neq, var) then
	DPV := DPV, denom(neq)<>0
     end_if;
     rdcs := [op(select(rdcs,has,var))];
     rdcs := select(rdcs,fun(modp(denom(op(args(1),2)),2)=0));
     if rdcs <> [] then DPV := DPV, op(map(rdcs,fun(op(args(1),1)>=0))) end_if;
     if DPV = null() then DPV := {[var>-infinity, var<infinity]}
     else
        DPV := ineqs::ineqs({DPV}, var);
        if DPV=FAIL then return(FAIL) end_if;
     end_if;
    end_if;
    sls := solve(eq = 0, var);
    if type(sls)="solve" then
       userinfo(1,"solve not powerful enough"); return(FAIL)
    end_if;
    if map(sls,type) intersect {"solve",DOM_SET}<>{} then return(FAIL) end_if;
    sls := select(sls,not testtype,"solve");
    if has(sls, RootOf) then sls := float(sls); end_if;
    # pick off complex roots #
    sls := sls minus select(sls,has,I);
    if sls = {} then
     if ineqs::test(var=1, ineq, FALSE) then
      # PZ: replaced x by var #
      return(ineqs::intersect({DPV, {[var>-infinity, var<infinity]}})) 
     else
      # rdcs = indets(ineq, radical) #
      rdcs:=radindets(ineq);
      if rdcs<>{} then return(DPV) end_if; # hastype(ineq, radical) #
      return({})
     end_if
    else sls:=map(sls,normal)
    end_if;
    sls := select(sls,fun(ineqs::in_interval(DPV,args(1))));
    sls := [ineqs::left(DPV), op(sls), ineqs::right(DPV)];
    ev_list := select(sls,not has,infinity); # not needed #

    # save unevaluated roots #
    ev_list := map(ev_list,fun(float(args(1))=args(1)));

    # evaluate roots using floating-point arithmetic #
    sls := float(sls);

    # sort roots in numerical order #
    sls := sort(sls, num_order);

    # restore to unevaluated state #
    sls := subs(sls,op(ev_list));
    solint := null();
    for i from 1 to nops(sls)-1 do
     solint := solint, [var>=sls[i], var<=sls[i+1]]
    end_for;
    solint := {solint};
    solint := ineqs::intersect({DPV, solint});
    sol := null();
    for i from 1 to nops(solint) do
        interval := [op(solint,[i,1,1]),op(solint,[i,2,2])];
        # PZ: modified because has(infinity,-infinity) gives TRUE in MuPAD #
        if has(op(interval,1),infinity) then check_point := interval[2]-1
        elif has(op(interval,2),infinity) then check_point := interval[1]+1
        else check_point := 1/2*interval[1]+1/2*interval[2]
        end_if;
        if ineqs::test(var=check_point, ineq, FALSE) then
         interv := null();
         if ineqs::test(var=interval[1], ineq, TRUE) then
          interv := var>=interval[1] else interv := var>interval[1] end_if;
         if ineqs::test(var=interval[2], ineq, TRUE) then
          interv := interv, var<=interval[2]
         else
          interv := interv, var<interval[2]
         end_if;
         sol := sol, [interv];
        else
         if ineqs::test(var=interval[1], ineq, FALSE) then
          sol := sol, [var>=interval[1], var<=interval[1]] end_if;
         if ineqs::test(var=interval[2], ineq, FALSE) then
          sol := sol, [var>=interval[2], var<=interval[2]] end_if;
        end_if
    end_for;
    sol := {sol}; sls := sol;
    for i in crit_pts do
     if ineqs::in_interval(sol, i) then
      sls := {op(sls), [var<i,i<var]}
     end_if
    end_for;
    sls;
end_proc:

#--
	ineqs::new(inequality, var)

        inequality - inequality to solve
        var        - variable to solve for
--#

ineqs::new := proc(ineq,var)
    local res, sol, i, check_point, interval, interv, solint;
begin
    i:= indets(ineq);
    case args(0)
    of 1 do
        var:= i; break
    of 2 do
        var:= {var}; break
    otherwise
        error("expecting two arguments")
    end_case;

    var:= i union var;
    if nops(var) <> 1 then 
        error("sorry, parameters can not be handled")
    else
        var:=op(var,1);
    end_if;

 res := ineqs::ineqs(ineq,var);
 if res=FAIL then return(hold(solve)(args())) end_if;
 sol := null(); solint := res;
 if radindets(ineq)<>{} then # hastype(ineq, radical) #
     for i from 1 to nops(res) do
        interval := [op(solint,[i,1,1]),op(solint,[i,2,2])];
        if has(interval,-infinity) then check_point := interval[2]-1
        elif has(interval,infinity) then check_point := interval[1]+1
        else check_point := 1/2*interval[1]+1/2*interval[2]
        end_if;
        if ineqs::test(var=check_point, ineq, FALSE) then
         interv := null();
         if ineqs::test(var=interval[1], ineq, TRUE) then
          interv := var>=interval[1] else interv := var>interval[1] end_if;
         if ineqs::test(var=interval[2], ineq, TRUE) then
          interv := interv, var<=interval[2]
         else
          interv := interv, var<interval[2]
         end_if;
         sol := sol, [interv];
        else
         if ineqs::test(var=interval[1], ineq, FALSE) then
          sol := sol, [var>=interval[1], var<=interval[1]] end_if;
         if ineqs::test(var=interval[2], ineq, FALSE) then
          sol := sol, [var>=interval[2], var<=interval[2]] end_if;
        end_if
    end_for;
    res := {sol}
 end_if;
 ineqs::simplify(res);
end_proc:

# end of file #
