# #
# $Date: 1999/06/24 13:04:10 $ $Author: stefanw $ $Revision: 1.37.2.1 $ #
# #
# zimmerma, 14/12/95 #

#--
        stdlib::isolate  --  isolate an identifier to the lhs
--#

alias(Z=(if principal=TRUE then 0 else Dom::DiscreteSet(k,k=-infinity..infinity) end_if)):

stdlib::isolate := proc(lhs,rhs,x,lev, reclevel)
local l,i,a,s,k,arg;
# global principal (local variable from solve) #
begin
   if reclevel=hold(reclevel) then 
	reclevel:=1
   elif reclevel>=30 then
	userinfo(2, "Maximum recursion level exceeded in stdlib::isolate");
	return(hold(solve)(lhs=rhs,x))
   end_if;
   userinfo(3,"trying to isolate",x,"in",lhs=rhs);
   userinfo(5,"Recursion level is ".reclevel);
   if lhs=x then return(rhs)
   elif not has(lhs,x) then
      return((if lhs=rhs then x else null() end_if))
   end_if;
   case type(lhs)
   of "_plus" do
      # Note: x-op(x,1) <> subsop(x,1=null()) #
      for i from 1 to nops(lhs) do
          if not has(op(lhs,i),x) then
              return(stdlib::isolate(
                  subsop(lhs,i=null()),rhs-op(lhs,i),x,lev, reclevel+1
	       ))
          end_if;
      end_for;
      if iszero(rhs) then
          # try to recognize equations of the form f(a)-f(b)=0
            where f has an inverse function #
          if nops(lhs)=2 then
             if type(op(lhs,1))="_mult" then a:=2 else a:=1 end_if;
             i:=-op(lhs,3-a); a:=op(lhs,a); # equation is a=i #
             if type(a)=type(i) and (l:=stdlib::inverse(type(a)))<>FAIL then
                return(stdlib::isolate(l(op(a),i),0,x,lev, reclevel+1))
             end_if;
          end_if;
          # try to factor the left hand side #
          l:=factor(numer(lhs));
          if nops(l)>=5 or l[2]<>lhs then
             return(stdlib::isolate(op(l,2*i),0,x,lev, reclevel+1)$hold(i)=1..nops(l) div 2)
          end_if
      end_if;
      # try to recognize a polynomial #
      if testtype(lhs,Type::PolyExpr(x)) then
         return(stdlib::solve_poly(poly(lhs-rhs,[x]),x,lev))
      end_if;
      if has(lhs,ln) and principal=TRUE then
         combine(lhs,ln);
         if %<>lhs then return(stdlib::isolate(%,rhs,x,lev, reclevel+1)) end_if
      end_if;
      break;
   of "_mult" do
      for i from 1 to nops(lhs) do
          if has(op(lhs,i),x) then
              l:= op(lhs,i);
              lhs:= subsop(lhs,i=null());
              if has(lhs,x) then lhs:= lhs*l; break end_if;
              return(stdlib::isolate(l,rhs/lhs,x,lev, reclevel+1))
          end_if;
      end_for;
      if iszero(rhs) then # one factor must be zero #
         if (a:=denom(lhs))=1 then
            return(stdlib::isolate(op(lhs,i),0,x,lev, reclevel+1) $ hold(i)=1..nops(lhs))
         else
            a:={stdlib::isolate(a,0,x,lev, reclevel+1)}; # forbidden values #
            s:=[stdlib::isolate(numer(lhs),0,x,lev, reclevel+1)];
            return(op(select(s,not has,a)))
         end_if
      end_if;
      break;
   of "_power" do
      if not has((s:=op(lhs,2)),x) then
         if testtype(s,Type::PosInt) then
            a:=[stdlib::solve_poly(poly(x^s-rhs,[x]),x,lev)];
            if op(lhs,1)=x then return(op(a))
            else
	       return(stdlib::isolate(op(lhs,1),a[i],x,lev, reclevel+1)$i=1..nops(a))
            end_if
         elif iszero(rhs) and is(op(lhs,2) < 0)=TRUE then
           return(null())
         elif principal=TRUE then
            return(stdlib::isolate(op(lhs,1),rhs^(1/op(lhs,2)),x,lev, reclevel+1))
         elif op(lhs,2)=-1 then
           return(stdlib::isolate(op(lhs,1),1/rhs,x,lev, reclevel+1))
         elif type(op(lhs,2))=DOM_RAT then
            s:=null(); l:=op(lhs,[2,1]); k:=op(lhs,[2,2]);
            # x^(l/k)=y <==> x^(1/k)=y^(1/l)*exp(2*I*PI*i/l) #
            for i from 0 to abs(l)-1 do
               a:=rhs^(1/l)*exp(2*I*PI*i/l);
               # now we must solve x^(1/k)=a, therefore abs(arg(a))<=PI/k #
               arg:=(if a=0 then 0 else k*atan(Re(a),Im(a)) end_if);
               if (is(-PI<arg) and is(arg<=PI))<>FALSE then
                  s:=s,stdlib::isolate(op(lhs,1),a^k,x,lev, reclevel+1)
               end_if
            end_for;
            return(s)
         elif type(rhs)<>Dom::DiscreteSet and is(rhs>=0)=TRUE then
           if rhs=0 and is(op(lhs,2)<0)=TRUE then
             return(null())
           else
             return(stdlib::isolate(op(lhs,1),rhs^(1/op(lhs,2)),
                                    x,lev, reclevel+1))
           end_if
         end_if
      else
         if iszero(rhs) then
            s:=[]; for i in [stdlib::isolate(op(lhs,1),0,x,lev, reclevel+1)] do
               # remove solutions with exponent 0, because 0^0=1 #
               # if traperror((a:=eval(subs(op(lhs,2),x=i))))=0 then #
               if traperror((a:=subs(op(lhs,2),x=i,Simplified)))=0 then
                  if a<>0 then s:=append(s,i) end_if
               end_if
            end_for;
            return(op(s))
         else
	    return(stdlib::isolate(
	       exp(op(lhs,2)*ln(op(lhs,1))),rhs,x,lev, reclevel+1
	    ))
         end_if
      end_if; break
   of "atan" do return(stdlib::isolate(op(lhs),tan(rhs),x,lev, reclevel+1))
   of "ln" do # the imaginary part of a log must be in ]-PI,PI] #
      l:=Im(rhs);
      if is(l<=-PI)=TRUE then return(null())
      elif is(l>PI)=TRUE then return(null())
      else return(stdlib::isolate(op(lhs),exp(rhs),x,lev, reclevel+1))
      end_if
   of "exp" do # exp(a)=b <==> a = ln(b) + 2*I*k*PI #
      if iszero(rhs) then return(null())
      else return(stdlib::isolate(stdlib::inverse("exp")(op(lhs),rhs),0,x,lev, reclevel+1))
      end_if
   of "sinh" do return(stdlib::isolate(op(lhs),asinh(rhs)+2*I*PI*Z,x,lev, reclevel+1))
   of "cosh" do
      l:=acosh(rhs);
      if l=0 then return(stdlib::isolate(op(lhs),2*I*PI*Z,x,lev, reclevel+1))
      else return(stdlib::isolate(op(lhs),l+2*I*PI*Z,x,lev, reclevel+1),
        stdlib::isolate(op(lhs),-l+2*I*PI*Z,x,lev, reclevel+1))
      end_if
   of "asinh" do return(stdlib::isolate(op(lhs),sinh(rhs),x,lev, reclevel+1))
   of "acosh" do return(stdlib::isolate(op(lhs),cosh(rhs),x,lev, reclevel+1))
   of "cos" do # cos(a)=b <==> a = +-acos(b) + 2*k*PI #
      l:=acos(rhs);
      if l=0 then return(stdlib::isolate(op(lhs),2*PI*Z,x,lev, reclevel+1))
      elif l=PI/2 then return(stdlib::isolate(op(lhs),l+PI*Z,x,lev, reclevel+1))
      elif l=PI then return(stdlib::isolate(op(lhs),l+2*PI*Z,x,lev, reclevel+1))
      else return(stdlib::isolate(op(lhs),l+2*PI*Z,x,lev, reclevel+1),
        stdlib::isolate(op(lhs),-l+2*PI*Z,x,lev, reclevel+1))
      end_if
   of "sin" do # sin(a)=b <==> a = {asin(b),PI-asin(b)} + 2*k*PI #
      if rhs=0 then return(stdlib::isolate(op(lhs),PI*Z,x,lev, reclevel+1))
      elif rhs=1 then return(stdlib::isolate(op(lhs),asin(rhs)+2*PI*Z,x,lev, reclevel+1))
      elif rhs=-1 then return(stdlib::isolate(op(lhs),-PI/2+2*PI*Z,x,lev, reclevel+1))
      else return(stdlib::isolate(op(lhs),asin(rhs)+2*PI*Z,x,lev, reclevel+1),
        stdlib::isolate(op(lhs),PI-asin(rhs)+2*PI*Z,x,lev, reclevel+1))
      end_if
   of "tan" do # tan(a)=b <==> a=atan(b)+k*PI #
      return(stdlib::isolate(op(lhs),atan(rhs)+PI*Z,x,lev, reclevel+1))
   of "abs" do # abs(a)=b <=> (a=b and a>=0) or (-a=b and a<0) #
      a:=op(lhs);
      l:=[stdlib::isolate(a,rhs,x,lev, reclevel+1)];
      if contains(map(l,type),"solve")<>0 then return(hold(solve)(lhs=rhs,x))
      end_if;
      i:=[stdlib::isolate(-a,rhs,x,lev, reclevel+1)];
      if contains(map(i,type),"solve")<>0 then return(hold(solve)(lhs=rhs,x))
      end_if;
      l:=map(l,fun((case is(subs(a,x=args(1))>=0) of TRUE do return(args(1))
	of FALSE do return(null()) of UNKNOWN do FAIL end_case)));
      i:=map(i,fun((case is(subs(a,x=args(1))<0) of TRUE do return(args(1))
        of FALSE do return(null()) of UNKNOWN do FAIL end_case)));
      if has([l,i],FAIL) then return(hold(solve)(lhs=rhs,x)) end_if;
      return(op(l),op(i))
   of "fact" do of "gamma" do return(null())
   end_case;
   # try to expand the left hand side if it contains tan, cot, ... #
   l:=expand(lhs);
   if indets(l,RatExpr)<>indets(lhs,RatExpr) then
      return(stdlib::isolate(l,rhs,x,lev, reclevel+1))
   end_if;
   if testtype(lhs,Type::RatExpr(x)) then
      return(stdlib::solve_poly(poly(numer(lhs-rhs),[x]),x,lev))
   end_if;
   # try to decompose #
   l:=rationalize(lhs-rhs,NIL,NIL,{"sin","exp"});
   userinfo(3,"after rationalize, equation is",l[1],"with",l[2]);
   if nops(l[2])=1 and not has(l[1],x) then
      i:=op(l[2],[1,1]);
      a:={stdlib::solve_eq(l[1],i,lev)};
      userinfo(3,"solutions of",l[1],"for",i,"are",op(a));
      if not has(a,"solve")
      and a<>{i-l[1]} # to avoid inf. loops, i.e. in solve(x^(1/2)=-1,x) #
      then
         if a={i} then return(x) # any x is solution #
         else return(stdlib::isolate(op(l[2],[1,2]),
	    op(a,i),x,lev, reclevel+1)$hold(i)=1..nops(a))
         end_if
      end_if
   end_if;
   # try to solve for square roots #
   l:=select(select(indets(lhs,PolyExpr),testtype,"_power"),
	fun(op(args(1),2)=1/2)
   );
   l:=select(l,has,x);
   if nops(l)=1 then
      a:=NIL; l:=op(l); # l = (...)^(1/2) #
      userinfo(3,"substituting a for the square root",l);
      userinfo(3,"having to solve",subs(lhs,l=a)=rhs);
      i:=[stdlib::isolate(subs(lhs,l=a),rhs,a,lev, reclevel+1)];
      userinfo(3,"solutions for a are",op(i));
      if not has(i,hold(solve)) then
         s:={};
         for a in i do
            a:=simplify(a);
            userinfo(3,"now solving",op(l,1)-a^2,"with respect to",x);
            i:={stdlib::solve_eq(op(l,1)-a^2,x,lev)};
            userinfo(3,"solutions are",op(i));
            if has(i,hold(solve)) then return(hold(solve)(lhs=rhs,x)) end_if;
            s := s union map(i,fun(stdlib::check_sqrt(subs(a,x=args(1)),args(1))));
         end_for;
         if contains(s,FAIL) then return(hold(solve)(lhs=rhs,x)) end_if;
         return(op(s))
      end_if
   end_if;

   l:= indets( lhs-rhs );
   if hastype(lhs-rhs,DOM_FLOAT) and l minus select(l,testtype,Type::Constant) = {x} then 
       op(funcattr(solve,"float")(lhs-rhs,x))
   else 
       hold(solve)(lhs=rhs,x)
   end_if
end_proc:

# stdlib::inverse("foo")(a,b) returns c that foo(a)=b <==> c=0 #
stdlib::inverse:=fun(FAIL):
stdlib::inverse("exp"):=proc(a,b) # exp(a)=b <==> a=ln(b)+2*I*k*PI #
begin
   if type(b)="exp" then b:=op(b) else b:=ln(b) end_if;
   if is(a-b,Type::RealNum)=TRUE then a-b else a-b+2*I*PI*Z end_if
end_proc:


# end of file #
