# $Date: 1998/03/27 16:24:17 $ $Author: stefanw $ $Revision: 1.17.2.1 $ #

#--
	stdlib::solve_poly --  solve a polynomial

	solve_poly( p,x,lev )

	p  : polynomial
	x  : indeteminate
	lev: positive integer (only for polynomials over Expr)
	     or option Multiple (only for polynomials over a domain)

	Solve a polynomial in explicit form, or returns RootOf(...) 
	for factors of degree>solve::maxdeg.
        Returns
	1) either x=1
	2) either x=1,x=2,...
	3) either x=x
	4) or null() if there is no solution

        Solve equations up to degree lev exactly.
--#

stdlib::solve_poly := proc(p,x,lev)
# p is supposed to be of type DOM_POLY, x is the variable #
local d,l,s,i;
begin
   # handle special cases w.r.t to the coefficient domain of p #
   if op((s:=op(p,3)),0)=hold(IntMod) then return(faclib::roots(p))
   elif s::constructor=Dom::IntegerMod and lev<>hold(Multiple) then
      return(op(map([faclib::roots(subsop(p,3=IntMod(s::constructor_args[1])))],		s::convert)))
   end_if;
   if s=Dom::Integer then
	if lev=hold(Multiple) then
		return(solve(expr(p), x=Dom::Integer, lev))
	else
		return(op(solve(expr(p), x=Dom::Integer, lev)))
	end_if
   end_if;
   if s::constructor=Dom::ExpressionField then
	if lev=hold(Multiple) then
		return(solve(expr(p), x, lev))
	else
		return(op(solve(expr(p), x, lev)))
	end_if
   end_if;
   # handle other domains #
   if not contains({Expr, Dom::Complex, Dom::Real}, s) then
	if iszero(p) then
		return(x)
	elif degree(p)=0 then
		return(null())
	end_if;
	if traperror((p:=factor(p)))<>0 then
		error("Cannot solve over that domain")
	end_if;
	if lev <> hold(Multiple) then
		p:=select({op(p,2*i) $hold(i)=1..nops(p) div 2}, 
			fun(degree(args(1))=1));
		return(op(map(p, _negate@tcoeff)))
	else
		p:=select({[op(p,2*i),op(p,2*i+1)] $hold(i)=1..nops(p) div 2},
			fun(degree(op(args(1),1))=1));
		return(map(p, fun(subsop(args(1),
			1=_negate(tcoeff(args(1)[1]))))))
	end_if
   end_if;

   # the general case #
   p:=poly(numer(op(p,1)),op(p,2..3));
   if hastype(op(p,1),DOM_FLOAT) then # remove 0.0 terms #
      userinfo(4, "Removing floating-point zeroes");
      p:=mapcoeffs(p,fun((if iszero(args(1)) then 0 else args(1) end_if)));
   elif not has(p,I) and not iszero(p) then
      p:=mapcoeffs(p,divide,content(p),Exact)
   end_if;
   d:=degree(p,x);
   if d=0 then if iszero(p) then x else null() end_if
   elif d=1 then
      if op(p,2)=[x] then -coeff(p,x,0)/coeff(p,x,1)
      else -expr(coeff(p,x,0))/expr(coeff(p,x,1))
      end_if
   else # d >=2 #
      userinfo(1,"trying to factor polynomial of degree ".d);
      userinfo(2,"polynomial is",p);
      if traperror((l:=factor(p)))<>0 then
         l:=[1,p,1] # can happen in solve(1/(x^2-1)*(-x^2-y^2+1),y) #
      end_if;
      s:=null();
      for i from 1 to nops(l) div 2 do
	 userinfo(4, "Processing ".expr2text(i)."th irreducible factor");
         s:=s,stdlib::solve_poly_irred(l[2*i],x,lev)$l[2*i+1]
      end_for;
      s
   end_if
end_proc:

# end of file #
