# #
# $Date: 1999/09/23 11:30:19 $ $Author: fakler $ $Revision: 1.26.2.1 $ #
# #
# zimmerma, 14/12/95 #

#++
        solve  --  solve equations and inequations

        solve(eq,x)			returns a set of solutions
        solve(eq,x,Multiple)		returns a multiset
        solve(eq,x,PrincipalValue)	returns only principal values
        solve(eq,x,i)			solves polynomials up to degree i
					only (default 4)
        solve(eq,x=t)			returns only roots in domain t
					(for ex. Dom::Integer)

        solve(sys,unk,BackSubstitution=TRUE)    enables back substitution
        solve(sys,unk,BackSubstitution=FALSE)   disables back substitution

        solve(sys,unk,MaxDegree=n)      solves explicitely polynomials
                                        up to degree n
++#

solve := proc(eqs,vars,lev)
local sol,principal,tv,p,i;
begin
   # first try overloading #
   if eqs::solve<>FAIL then return(eqs::solve(args())) end_if;
   principal:=contains({args(3..args(0))},hold(PrincipalValue));
   case type(eqs) 
   of DOM_SET do
      if args(0)<2 then vars:=indets(eqs) end_if;
      sol:=map(eqs,type);
      if nops({"_less","_leequal"} union sol)=2 then # inequalities #
         return(stdlib::ineqs(args()))
      end_if;
      if not contains({DOM_SET,DOM_LIST},type(vars))
      then error("set or list of unknowns expected") end_if;
      return(stdlib::solve_sys(eqs,vars,args(3..args(0))))
   of "_equal" do eqs:=op(eqs,1)-op(eqs,2); break
   of "_less" do
   of "_leequal" do
      return(stdlib::ineqs(args()))
   end_case;
   # only one equation #
   if args(0)<2 then
	if type(args(1))=DOM_POLY 
	 and not has({Dom::Complex, Dom::Real, Expr}, op(args(1),3)) then
		return({stdlib::solve_poly(args(1), op(op(args(1),2)))})
	end_if;
 	vars:=indets(expr(eqs)) 
   end_if;
   if (tv:= type( vars )) = DOM_SET then
      case nops(vars)
      of 0 do if iszero(eqs) then return({0=0}) else return({}) end_if
      of 1 do vars:=op(vars); break
      otherwise return(stdlib::solve_sys({eqs},vars,args(3..args(0))))
      end_case
   elif contains({DOM_IDENT,"_index","function"},tv) then # ok #
   elif tv = "_equal" then
	if type(op(vars,2)) <> DOM_DOMAIN then
	    error("Right hand side of second argument must be a domain")
        elif contains( {Dom::Integer,DOM_INT},op(vars,2) ) then
  	    p:= poly( eqs,[op(vars,1)] );
   	    if testtype( p,Type::PolyOf( Dom::Rational,1 ) ) then	
           	 userinfo(1,"use sharelib::iroots");
	   	 if contains({args(3..args(0))},hold(Multiple)) then
                	tv:= sharelib::iroots( p,hold(Multiple) );
			return( Dom::Multiset( 
 		   	 ( op(tv,[i,1]) $ op(tv,[i,2]) ) $ i=1..nops(tv)
			) )
	   	 else
			return( sharelib::iroots( p ) )
	    	 end_if
            end_if
	elif contains( {Dom::Rational, DOM_RAT, Dom::Real, Dom::Complex,
			 DOM_COMPLEX}, op(vars,2) ) then
		# do nothing: apply the general solver, selection
		  of 'good' solutions is done later on #
	else 
		# work over general domain # 
		p:=poly(eqs, [op(vars,1)], op(vars,2));
		if p=FAIL then
			error("Cannot convert equation to polynomial")
		end_if;
		if contains({args(3..args(0))},hold(Multiple)) then
			return(stdlib::solve_poly(p, op(vars,1),
				args(3..args(0)),hold(Multiple)))
		else 
			return({stdlib::solve_poly(p, op(vars,1),
				args(3..args(0)))}) 
		end_if
	end_if
   else error("invalid unknown")
   end_if;

   sol:=stdlib::solve_eq(eqs,(if tv="_equal" then op(vars,1) else vars end_if),lev);
   if type(sol)="solve" then procname(args(1..min(2,args(0))))
   else
      if tv="_equal" then
         sol:=op(select([sol],fun((op(vars,2))::convert(args(1))<>FAIL)))
      end_if;
      if contains({args(3..args(0))},hold(Multiple)) then Dom::Multiset(sol)
      else {sol}
      end_if
   end_if
end_proc:

# stdlib::check_sqrt(x,y) returns y if x can be a square root,
                                null() if x cannot be a square root,
                                FAIL otherwise #
stdlib::check_sqrt:=fun(
    (case is(Re(args(1))>0) or (is(Re(args(1))=0) and is(Im(args(1))>=0))
    of TRUE do return(args(2))
    of FALSE do return(null())
    of UNKNOWN do return(FAIL)
    end_case)
):

solve:= funcattr(solve, "type", "solve"):
solve:= funcattr(solve, "print", "solve"):
solve:= funcattr(solve, "info",
    "solve -- solve equations and inequalities [try ?solve for options]"
):

solve:= funcattr(solve, "float",
    loadproc(funcattr(solve,"float"),pathname("STDLIB","SOLVE"),"float")
):

# end of file #
