# $Date: 1997/10/10 15:56:26 $ $Author: tonner $ $Revision: 1.11 $ #

# kg, 13/01/95 #

#++
alias -- define alias name

alias(abbr=expr,...)
alias([abbr, func],...)

abbr - identifier or function call where the operands must be identifiers
expr - expression

An alias is an expression which is changed by the parser into another
one before evaluating the parsed expression. An alias may be defined
either for identifiers or for function calls:

- With an alias of the form alias(a=b) every a parsed is changed into b.

- With an alias of the form alias(f(x...)=z) every call f(a...) parsed is 
  replaced by subs(z, [x=a...]), ie. f(a...) is replaced by z, whereby the
  'formal parameters' x... of f are replaced by the 'actual parameters' a...

- With an alias of the form alias([f,g]) every call f(a...) parsed is
  replaced by the result of the evaluation of g(a...).
  In contrast to f g is evaluated when calling alias([f, g]).

alias doesn't evaluate its arguments.
++#

alias:= proc()
    local _0_arg;
    option hold, noDebug;
begin
    if testargs() and args(0) > 0 then
	for _0_arg in [ args() ] do
	    if domtype(op(_0_arg, 1)) <> DOM_IDENT then
		if domtype(op(_0_arg, 1)) = DOM_EXPR then
		    if nops(op(_0_arg, 1)) <> 0 then
			if {map(op(op(_0_arg, 1)), domtype)} <> {DOM_IDENT} then
			    error("illegal parameter(s)")
			end_if
		    end_if;
		    if type(op(_0_arg, 2)) = "_exprseq" then
			error("illegal alias value")
		    end_if
                else
                    error("illegal alias name")
                end_if;
	    end_if
	end_for
    end_if;

    # create tables for alias entries
    
       - stdlib::Alias[1][a] is the alias value of identifier a.
       - stdlib::Alias[2][v] is the identifier associated to value v, needed
            to unalias a (when unalias(a) is called a is replaced by v before
            evaluation, so one must use v to get back a again).
       - stdlib::Alias[3][a] is the alias value of operator a, consisting of
            the associated expression and the formal parameters.
       - stdlib::Alias[4][a] is the alias value of operator a, consisting of
            the function to be called with the parameters of the call.
    #

    if args(0) = 0 then
	# print alias entries #
	if stdlib::Alias = FAIL then
	    print( Unquoted, "No alias entries defined." );
	    return( null() )
	else
	    misc::tableForm( 
		map( [stdlib::Alias[1],stdlib::Alias[3],stdlib::Alias[4]],op )
	    , ",\n" );
	    return( null() )
	end_if
    end_if;


    if stdlib::Alias = FAIL then
        sysassign(stdlib::Alias, [ table(), table(), table(), table() ]);
    end_if;
   
    # assign alias function to parser #
    if domtype(_dom_alias_element_) = DOM_IDENT then
     	sysassign(_dom_alias_element_, 
	  proc() 
	    option noDebug;
	  begin
	    case domtype(args(1))
	    of DOM_IDENT do
		if contains(stdlib::Alias[1], args(1)) then
		    index_val(stdlib::Alias[1], args(1))
		else
		    args(1)
		end_if;
		break;
	    of DOM_EXPR do
		if contains(stdlib::Alias[3], op(args(1),0)) then
		    index_val(stdlib::Alias[3], op(args(1),0));
		    subs(%[1], zip(%[2], [op(args(1))], _equal), Unsimplified)
		else
                   if contains(stdlib::Alias[4], op(args(1),0)) then
                       level(stdlib::Alias[4][op(args(1),0)](op(args(1)))) ;
                   else
		       args(1)
                   end_if;
		end_if;
		break;
	    otherwise
		args(1)
	    end_case
	  end_proc)
    end_if;
    
    # insert new alias #
    for _0_arg in [ args() ] do
        case domtype(_0_arg)
        of DOM_EXPR do
	    case domtype(op(_0_arg, 1))
	    of DOM_IDENT do
	        sysassign(stdlib::Alias[1][op(_0_arg, 1)], op(_0_arg, 2));
	        sysassign(stdlib::Alias[2][op(_0_arg, 2)], op(_0_arg, 1));
	        break;
	    of DOM_EXPR do
	        sysassign(stdlib::Alias[3][op(op(_0_arg, 1), 0)], [op(_0_arg, 2), [op(op(_0_arg, 1))]]);
	        break;
	    otherwise error("illegal alias name")
	    end_case;
            break ;
        of DOM_LIST do
            sysassign(stdlib::Alias[4][op(_0_arg, 1)], context(op(_0_arg, 2)));
            break ;
        otherwise error("illegal argument") ;
        end_case ;
    end_for;
    null()
end_proc:


# end of file #
