
# assign.mu -- Andreas Sorgatz (andi@uni-paderborn.de), 15.08.1995 #

#++
   assign.mu

   assign -- perform assignments given as equations 

   assign( egns )
   assign( eqns, subset )

   egns   - a list, set or table of equations 'egn'
   eqn    - equation x=e with x evaluates to a free variable
   subset - set of variables

   For every equation  x=e  of 'eqns' the assignment x:=e is made.  If a
   'subset'  is specified,  an assignment x:=e  is made only  if x is an
   element of 'subset'.  Note that e is any valid expression,  expect an 
   expression sequence containing more than one element. If x is already 
   assigned the identifier z then assign({x=7}) would assign z the value
   7. If x does not evaluates to a free identifier,an error occures. The
   function returns its first argument.

   This function can be applied to a set of equations returned by 'solve'
   when it is desired to assign the solution values to their variables.

   Examples:

   >> solve({a+b=42,2*a+b=55}, {a,b});

                            [{b = 29, a = 13}]
                            
   >> assign(op(%)):  a,b, a*b;
                                13, 29, 377

   >> z:=NIL: x:=z: assign(table(x=42)):  x, level(x,1);

                                   42, z

   >> assign([d=hold((1,2,3))]): d;

                                  1, 2, 3
++#

assign := proc()
begin
    if args(0) = 0 then return() end_if;

    if testargs() then
       if args(0) > 2 then error("wrong no of args") end_if;
       if not contains({DOM_LIST, DOM_SET, DOM_TABLE}, domtype(args(1))) 
          then error( "illegal first argument" )
       end_if;
    end_if;

    if args(0) = 1 then              # assign all equations given in args(1) #
       # eval( map([op(args(1))], subsop, 0=_assign) ); #
       map([op(args(1))], subsop, 0=_assign, Simplified);
       return(args(1));
    end_if;
            # make assignments only for variables that are listed in args(2) #

    if domtype(args(2)) <> DOM_SET then error( "illegal second argument" )
    end_if;
    #
    eval( map([op( 
          select( args(1), fun(contains(args(2),op(args(1),1))), args(2) ) 
          )], subsop, 0=_assign ) );
    #
    map([op( 
          select( args(1), fun(contains(args(2),op(args(1),1))), args(2) ) 
          )], subsop, 0=_assign, Simplified );
    return(args(1));
end_proc:

# end of file #
