
#++
complexInfinity -- representing the 'complex infinity' in the compactification of C

There exists only one element 'complexInfinity' of this domain (stdlib::CInfinity)
which has no operands.
++#

proc()
    option noDebug;
    local cinf; 
begin

cinf:= domain("stdlib::CInfinity");
cinf::info:= "domain 'stdlib::CInfinity': representing the complex infinity";
cinf::interface:= {};

cinf::expr:= fun(hold(complexInfinity));
cinf::print:= cinf::expr;
cinf::TeX:= fun("\\infty");

cinf::_negate:= id;
cinf::_invert:= fun(0);
cinf::not:= fun(error("invalid operand"));

cinf::_power:= fun((
    case domtype(args(2))
    of stdlib::CInfinity do
        if domtype(args(1)) = stdlib::CInfinity or
           domtype(args(1)) = domtype(infinity) or
           args(1) = undefined then 
            return(undefined)
        end_if;
        if is(abs(args(1))>1) = TRUE then complexInfinity
        elif is(abs(args(1))<1) = TRUE then 0
        else hold(_power)(args())
        end_if;
        break;
    of DOM_FLOAT do
        if iszero(args(2)) then return(1.0) end_if;
    of DOM_INT do
        if args(2) = 0 then return(1) end_if;
    of DOM_RAT do
	if args(2) < 0 then 0 else complexInfinity end_if;
	break;
    otherwise
        if is(args(2)<0) = TRUE then return(0) end_if;
	hold(_power)(args())
    end_case)):

cinf::_plus:= fun((
    if map({args()}, domtype) intersect
	{stdlib::Undefined, stdlib::CInfinity, stdlib::Infinity} <> {}
    then
        undefined
    else 
        complexInfinity
    end_if));

cinf::_mult:= fun((
    _mult(op(select([args()], not (stdlib::CInfinity)::thisDomain)));
    if % = undefined or iszero(%) then undefined else complexInfinity end_if));

cinf::intmult:= fun((
    if iszero(args(2)) then undefined else complexInfinity end_if));

cinf::subs:= fun((
    if op(args(2),1) = complexInfinity then
        op(args(2),2)
    else
        complexInfinity
    end_if));

cinf::subsop:= fun((
    if args(2) = 1 then args(3) else FAIL end_if));

cinf::op:= fun((
    if args(0) = 2 then
        if args(2) = 1 then complexInfinity else FAIL end_if
    else 
        complexInfinity
    end_if));

cinf::has:= fun(bool(args(2) = complexInfinity));

cinf::O:= fun(undefined);

cinf::thisDomain:= fun(bool(domtype(args(1)) = stdlib::CInfinity));

complexInfinity:= new(cinf);
sysassign(stdlib::CInfinity, cinf);
null();
end_proc():

# end of file #
