# #
# $Date: 1999/09/14 14:17:04 $ $Author: fakler $ $Revision: 1.41.2.1 $ #

# frankp, 15.02.1995 #

#++
   combine.mu

	combine  --  combine terms of the same algebraic structure

	combine(expr) combines terms of the same algebraic
	structure.

	For expressions of type DOM_FUNC_ENV the function attribute
	"combine" will be called with the operands of expr.
	If such a function attribute does not exist the expression will
	be returned without changes.
++#

combine := proc(e,p)
    local _t, _f_comb, l, i, _n_, _b, _e;
begin
    # is overloaded ? #
    if e::combine <> FAIL then return( e::combine(e,args(2..args(0))) ) end_if;

    case args(0)
    of 1 do break
    of 2 do
        if type(p)=DOM_LIST then
           for _t in p do e:=combine(e,_t) end_for; return(e)
        elif traperror((p:=funcattr(combine,expr2text(args(2)))))=0 then return(p(e))
	else error("invalid second argument")
        end_if
    otherwise error("wrong no of args")
    end_case;

    case domtype(e) 
    # default is to return e, thus we do not need to consider these cases #
    of DOM_POLY	   do return( mapcoeffs(e,combine) )
    of Puiseux	   do return( map(e,combine) )
    of DOM_LIST    do 
    of DOM_ARRAY   do
    of DOM_SET     do return( map(e,combine) )
    of DOM_EXPR    do
        _t := op(e,0);
        if domtype(level(_t,2)) = DOM_FUNC_ENV and
	   (p:= funcattr(level(_t,2),"combine")) <> FAIL 
	then
            return( p(op(e)) )
	end_if;
	case _t
	of hold(_mult) do
            e:=map(e,combine);
            if type(e)<>"_mult" then return(e) end_if;
	    _f_comb := fun((if type(args(1))="_power" then 
			 if op(args(1),1) = args(2) then TRUE else FALSE end_if
		      else FALSE
		      end_if)
		 );
	    p := 1;
	    _e := {op(e)};
	    i := 1;
	    repeat
		_t := op(_e,i);
		if type(_t) = "_power" then
		    _b := op(_t,1); _n_ := op(_t,2)
		else 
		    _b := _t; _n_ := 1
		end_if;
		_e := _e minus {_t};
		l := select(_e, _f_comb, _b);
		_e := _e minus l;
		for _t in l do _n_ := _n_ + op(_t,2) end_for;
		p := p*_b^normal(_n_);
	    until nops(_e) = 0 end_repeat;
	    if hastype( p,"_power" ) then
		return( funcattr(combine,"_power")(p) )
	    elif p <> e then
		return( combine(p) )
	    else
	        return( p )
	    end_if
	of hold(_power) do 
   	    l:=combine(op(e,1)); _f_comb:=combine(op(e,2));
	    if type(l)="_power" and type(_f_comb)<>DOM_RAT then 
		return( op(l,1)^(op(l,2)*_f_comb) )
   	    end_if;

            if (p:= funcattr(combine,type(l))) <> FAIL then
                return( p(e) )
            end_if
	end_case;

	return( map(e,combine) )
    end_case;
    e
end_proc:

# sqrt(a)*sqrt(b) ==> sqrt(a*b) #
combine:=funcattr(combine,"sqrt",
proc(e) 
local t,a,f,g,matchsqrt,n;
begin
   matchsqrt := proc(e) # matches sqrt(f) and returns f, or FAIL #
   begin
      if type(e)="_power" then if op(e,2)=1/2 then return(op(e,1)) end_if
      elif type(e)="sqrt" then return(op(e))
      end_if;
      FAIL
   end_proc;

   case type(e)
   of "_mult" do
      f:=1; g:=1; n:= 0;
      for t in e do
         if (a:=matchsqrt(t))<>FAIL then 
	    if is(a>0) = TRUE then n:= n+1 end_if;
	    f:=f*a 
	 else 
	    g:=g*t 
   	 end_if
      end_for;
      if n+1 >= nops(e) then return(f^(1/2)*g)
      else return( e )
      end_if
   of "_plus" do return(map(e,funcattr(combine,"sqrt")))
   otherwise e
   end_case;
end_proc):

# cos(x)^2 -> cos(2*x)/2+1/2 #
combine:=funcattr(combine,"sincos",proc(e)
local l,k,f;
begin
   case domtype(e) # default is to return e #
   of DOM_POLY	  do return( mapcoeffs(e,funcattr(combine,"sincos")) )
   of Puiseux	  do
   of DOM_LIST    do 
   of DOM_ARRAY   do
   of DOM_SET     do return( map(e,funcattr(combine,"sincos")) )
   of DOM_EXPR    do
      case type(e)
      of "_mult" do e:=[op(e)]; break
      of "_power" do e:=[e]; break
      otherwise return(map(e,funcattr(combine,"sincos")))
      end_case;
      # now deal with _mult and _power case #
      k:=1; l:=[];
      for f in e do
         case type(f)
         of "sin" do
         of "cos" do l:=append(l,f); break
         of "_power" do
            if contains({"sin","cos"},type(op(f,1))) and testtype(op(f,2),Type::PosInt) then
               l:=append(l,op(f,1)$op(f,2)); break
            end_if
         otherwise k:=k*map(f,funcattr(combine,"sincos"))
         end_case;
      end_for;
      return(stdlib::combine_sin_cos(l,k))
   end_case;
   e
end_proc):

# k is a scalar, l is a list of sin or cos expressions #
stdlib::combine_sin_cos := proc(l,k)
begin
   if l=[] then return(k) end_if;
   while nops(l)>=2 do
      if type(l[1])="_plus" then
         l[1]:=map(l[1],stdlib::combine_sincos,l[2])
      else l[1]:=stdlib::combine_sincos(l[1],l[2])
      end_if;
      l[2]:=NIL;
   end_while;
   if type(l[1])="_plus" then map(l[1],_mult,k) else k*l[1] end_if
end_proc:

# for x*y where x and y can be cos(..) or a*cos(..) or a where a is a rational #
stdlib::combine_sincos := proc(x,y) local a,b,m;
begin
   if type(x)="_mult" then x:=split(x,testtype,Type::Union("cos","sin"));
	m:=x[2]; x:=x[1]; else m:=1 end_if;
   if type(y)="_mult" then y:=split(y,testtype,Type::Union("cos","sin"));
	m:=m*y[2]; y:=y[1] end_if;
   a:=op(x); b:=op(y);
   case type(x),type(y)
   of "sin","sin" do m/2*cos(a-b)-m/2*cos(a+b); break
   of "sin","cos" do m/2*sin(a+b)+m/2*sin(a-b); break
   of "cos","sin" do m/2*sin(a+b)-m/2*sin(a-b); break
   of "cos","cos" do m/2*cos(a-b)+m/2*cos(a+b); break
   otherwise m*x*y # it can happen that x or y is a constant, 
			for example cos(0) -> 1 for cos(t)^3 #
   end_case
end_proc:

# atan(x)+atan(y) = atan((x+y)/(1-x*y)) #
combine:=funcattr(combine,"atan",proc(e)
local s,f,x,y,i;
begin
   case domtype(e) # default is to return e #
   of DOM_POLY	  do return( mapcoeffs(e,funcattr(combine,"atan")) )
   of Puiseux	  do
   of DOM_LIST    do 
   of DOM_ARRAY   do
   of DOM_SET     do return( map(e,funcattr(combine,"atan")) )
   of DOM_EXPR    do
      case type(e)
      of "_plus" do
         s:=0; x:=0;
         for f in e do
            if type(f)="atan" then
               y:=op(f);
               x:=(x+y)/(1-x*y)
            elif type(f)="_mult" then # n*atan(y) #
               if nops(f)=2 and type(op(f,1))="atan" and type((i:=op(f,2)))=DOM_INT then
                  if i>0 then y:=op(op(f,1)) else y:=-op(op(f,1)); i:=-i end_if;
                  while i>0 do x:=(x+y)/(1-x*y); i:=i-1 end_while
               else s:=s+f
               end_if
            else s:=s+f
            end_if
         end_for;
         return(s+atan(x))
      otherwise return(map(e,funcattr(combine,"atan")))
      end_case
   end_case;
   e
end_proc):

# exp(a)*exp(b) ==> exp(a+b)
  exp(a)^b      ==> exp(a*b) #
combine:=funcattr(combine,"exp",proc(e) local _xl;
begin
   case type(e)
   of "_plus" do return(map(e,funcattr(combine,"exp")))
   of "_mult" do
      e:=map(e,funcattr(combine,"exp"));
      if type(e)="_mult" then
         _xl:=select(e,testtype,"exp");
         if type(_xl)="_mult" then
            return(e/_xl*exp(_plus(map(op(_xl),op))))
         end_if
      end_if; break
   of "_power" do 
      if type((_xl:=op(e,1)))="exp" then return(exp(op(_xl)*op(e,2))) end_if;
      break
   end_case;
   e
end_proc):

# ln(a)+ln(b) ==> ln(a*b)
  k*ln(a) ==> ln(a^k) #
combine:=funcattr(combine,"ln",proc(e) local f;
begin
   case type(e)
   of "_plus" do
      e:=map(e,funcattr(combine,"ln"));
      f:=select(e,testtype,"ln");
      if type(f)="_plus" then
         return((e-f)+ln(map(subsop(f,0=_mult),op)))
      end_if; break
   of "_mult" do
      e:=map(e,funcattr(combine,"ln"));
      f:=select(e,testtype,"ln");
      if type(f)="ln" then # only one factor ln(..) #
         return(ln(op(f)^(e/f)))
      end_if; break
   end_case;
   e
end_proc):

# 5^x*6^x/10^x/3^x ==> (5*6/10/3)^x ==> 1 // tonner #
combine:=funcattr(combine,"_power",proc(X) 
    local a,f,s,p,x,t;
begin
   if type(X)<>"_mult" then X
   else
      a:=1; # product of non-powers #
      s:={}; # set of exponents #
      p:=table(); # p[x] is the list of all factors being to power x #
      for f in X do
         if type(f)="_power" then
            x:=op(f,2);
            if not contains(s,x) then
               if not contains(s,-x) then p[x]:=[op(f,1)]; s:=s union {x}
               else p[-x]:=p[-x] . [1/op(f,1)]
               end_if
            else p[x]:=p[x] . [op(f,1)]
            end_if
         else a:=a*f
         end_if
      end_for;
      for f in s do
	 # x>0 or y>0  =>  x^b * y^b = (x*y)^b #
	 # Re(x)>0 or (Re(x)=0 and Im(y)>0)  =>  x^b * y^b = (x*y)^b #
	 t:= map( p[f], fun((
	     if testtype(args(1),NUMERIC) then bool(sign(args(1))=1)
	     else is(args(1)>0)
	     end_if
 	 )) );
	 # x>0 or y>0  =>  x^b * y^b = (x*y)^b #
	 if nops(select(t,_equal,TRUE)) + 1 >= nops(t) then
	     t:= _mult( op(p[f]) );
             if length(t^f)<=length((1/t)^(-f)) then 
		a:=a*t^f
             else 
		a:=a*(1/t)^(-f)
             end_if
	 else
            # Re(x)>0 or (Re(x)=0 and Im(x)>0)  AND
	      Re(y)>0 or (Re(y)=0 and Im(y)>0)      =>  x^b * y^b = (x*y)^b 
	    #
            t:= select( p[f], fun((
                (testtype(args(1),NUMERIC) and (sign(Re(args(1)))=1
                   or sign(Re(args(1)))=0 and sign(Im(args(1)))=1))
                or (is(Re(args(1))>0)=TRUE or
                     (is(Re(args(1))=0) and is(Im(args(1))>0)) = TRUE)
            )) );
            if t = p[f] then
		t:= _mult( op(p[f]) );
                if length(t^f)<=length((1/t)^(-f)) then 
		   a:=a*t^f
                else 
		   a:=a*(1/t)^(-f)
                end_if
	    else
	        a:= _mult(a,op(map(p[f],hold(_power),f)))
	    end_if
	end_if
      end_for;
      a
   end_if
end_proc):

#
|    The following routines are used to combine expressions including
|    sinh and cosh. The algorithms are the same as in case of combining
|    expressions involving sin and cos. Therefore the above procedures
|    combine:=funcattr(combine,"sincos",...), sdtlib::combine_sin_cos()
|    and stdlib::combine_sincos() have been renamed to
|    combine:=funcattr(combine,"sincos",...), stdlib::combine_sinh_cosh
|    and stdlib::combine_sinhcosh. 
|    T.S. 03.05.1996
#

combine:=funcattr(combine,"sinhcosh",proc(e)
local l,k,f;
begin
   case domtype(e) # default is to return e #
   of DOM_POLY    do return( mapcoeffs(e,funcattr(combine,"sinhcosh")) )
   of Puiseux     do
   of DOM_LIST    do
   of DOM_ARRAY   do
   of DOM_SET     do return( map(e,funcattr(combine,"sinhcosh")) )
   of DOM_EXPR    do
      case type(e)
      of "_mult" do e:=[op(e)]; break
      of "_power" do e:=[e]; break
      otherwise return(map(e,funcattr(combine,"sinhcosh")))
      end_case;
      # now deal with _mult and _power case #
      k:=1; l:=[];
      for f in e do
         case type(f)
         of "sinh" do
         of "cosh" do l:=append(l,f); break
         of "_power" do
            if contains({"sinh","cosh"},type(op(f,1))) and testtype(op(f,2),Type
::PosInt) then
               l:=append(l,op(f,1)$op(f,2)); break
            end_if
         otherwise k:=k*map(f,funcattr(combine,"sinhcosh"))
         end_case;
      end_for;
      return(stdlib::combine_sinh_cosh(l,k))
   end_case;
   e
end_proc):

# k is a scalar, l is a list of sinh or cosh expressions #
stdlib::combine_sinh_cosh := proc(l,k)
begin
   if l=[] then return(k) end_if;
   while nops(l)>=2 do
      if type(l[1])="_plus" then
         l[1]:=map(l[1],stdlib::combine_sinhcosh,l[2])
      else l[1]:=stdlib::combine_sinhcosh(l[1],l[2])
      end_if;
      l[2]:=NIL;
   end_while;
   if type(l[1])="_plus" then map(l[1],_mult,k) else k*l[1] end_if
end_proc:

# for x*y where x and y can be cos(..) or a*cos(..) or a where a is a rational #
stdlib::combine_sinhcosh := proc(x,y) local a,b,m,str1, str2;
begin
   if type(x)="_mult" then
        m:=op(x,2);
        x:=op(x,1):
   else
        m:=1
   end_if;
   if type(y)="_mult" then
       m:=m*op(y,2);
       y:=op(y,1):
   end_if;
   a:=op(x); b:=op(y);
   case type(x),type(y)
   of "sinh","sinh" do -m/2*cosh(a-b)+m/2*cosh(a+b); break
   of "sinh","cosh" do m/2*sinh(a+b)+m/2*sinh(a-b); break
   of "cosh","sinh" do
        str1 := expr2text(a):
        str2 := expr2text(b):
        if str1 <= str2 then
            m/2*sinh(a+b)-m/2*sinh(b-a);
        else
            m/2*sinh(a+b)+m/2*sinh(a-b);
        end_if:
   break
   of "cosh","cosh" do m/2*cosh(a-b)+m/2*cosh(a+b); break
   otherwise m*x*y # it can happen that x or y is a constant,
                        for example cos(0) -> 1 for cos(t)^3 #
   end_case:
end_proc:

_plus:=funcattr(_plus,"combine",
	loadproc(funcattr(_plus,"combine"),pathname("STDLIB","COMBINE"),
	"_plus")):

# end of file #

