# $Date: 1999/10/11 18:23:09 $ $Author: stefanw $ $Revision: 1.24.2.4 $ #

# zimmerma, 01/08/95 #

#++

	discont  --  determines the discontinuities of a function


	discont(f,x) returns a set of (possible) real discontinuities 
	of f.

	discont(f,x=a..b) returns the discontinuities in a..b, where a<=b.

	discont looks for an attribute "discont" to determine the 
	discontinuities of a function, for example to say that the only 
	discontinuity of ln is in 0, you write 
	ln:=funcattr(ln,"discont",{0})
++#

alias(Solve=stdlib::Solve):

discont := proc(_f,x) local i,s,r,sol;
begin
   if testargs() then
      if args(0)<>2 then error("two arguments expected") end_if;
      case type(x)
      of DOM_IDENT do break; # ok #
      of "_equal" do
         if map(x,type)<>(DOM_IDENT="_range") then
            error("2nd argument: range 'x=a..b' expected") 
         end_if;
         break
      otherwise 
         error("2nd argument: range 'x=a..b' expected") 
      end_case
   end_if;

   if type(x)="_equal" then
      # Enable overloading of function environments for discont( f,x=a..b )! #
      # The function attribute "discont" should return FAIL if it can not
        handle this case!
      #
      s:=op(_f,0); s:=level(s,2);
      if domtype(s)=DOM_FUNC_ENV then
          s:=funcattr(s,"discont")(_f,x);
          if s<>FAIL then return(s) end_if
      end_if;

      r:=op(x,2);
      case type((s:=discont(_f,op(x,1))))
      of DOM_SET do
         # if the interval is reduced to a point which is a singularity,
           then return it directly
         #
         i:= op(r,1); if i=op(r,2) and contains(s,i) then return({i}) end_if;

         sol:={};
         for i in s do
            if type(i)=Dom::DiscreteSet then
               i:=select(i,_leequal,op(r,2));
               if type(i)="select" then return(procname(args()))
               elif nops({i})<>1 or type(op({i},1))<>Dom::DiscreteSet then i:={i}
               end_if;
               i:=select(i,not _less,op(r,1));
               if type(i)=DOM_SET then i:=op(i) else i:=expand(i) end_if;
            elif hastype(i,Dom::DiscreteSet) then # cannot select #
               return(procname(args()))
            else case is(op(r,1)<=i) and is(i<=op(r,2))
		   of TRUE do break of FALSE do i:=null(); break 
		   of UNKNOWN do
		      case is(i<op(r,1)) or is(op(r,2)<i)
		      of TRUE do i:=null(); break
		      otherwise i:=FAIL
		      end_case
		   end_case
            end_if;
            sol:=sol union {i};
         end_for;
         if not has(sol,FAIL) then return(sol) end_if;
      otherwise return(procname(args()))
      end_case
   end_if;
   case type(_f)
   of DOM_INT do of DOM_RAT do of DOM_IDENT do of DOM_COMPLEX do 
   of DOM_FLOAT do return({})
   of "_index" do return((if not has(_f,x) then return({}) else FAIL end_if))
   of "_plus" do of "igamma" do
   of "_mult" do return(_union(discont(op(_f,i),x)$i=1..nops(_f)))
   of "_power" do
      loadlib("intlib");
      if is(op(_f,2)>0)=TRUE then s:={} 
      else s:=Solve(numer(op(_f,1)),x) union discont(op(_f,2),x)
      end_if;
      return(s union discont(op(_f,1),x))
   # functions without singularities #
   of "besselJ" do _f:=[op(_f,2)]
   of "sin" do of "cos" do of "abs" do of "exp" do
   of "acos" do of "acosh" do of "acot" do of "asin" do of "asinh" do
   of "sinh" do of "cosh" do of "erf" do of "erfc" do of "tanh" do
      return(discont(op(_f),x))
   of "atan" do if nops(_f)=1 then return(discont(op(_f),x))
	        else return(discont(op(_f,1),x) union discont(op(_f,2),x)
			union Solve(op(_f,2)=0,x)) end_if
   # functions with a discontinuity in 0 #
   of "besselY" do _f:=[op(_f,2)]
   of "coth" do of "eint" do of "dirac" do of "heaviside" do of "sign" do
   of "ln" do return(discont(op(_f),x) union Solve(op(_f),x))
   # functions with a discontinuity in 1 #
   of "zeta" do return(discont(op(_f),x) union Solve(op(_f)=1,x))
   # functions with discontinuities in 1 and -1 #
   of "acoth" do
   of "atanh" do return(discont(op(_f),x) union Solve(op(_f)=1,x) union Solve(op(_f)=-1,x))
   # functions with a discontinuity in -n #
   of "psi" do
   of "gamma" do
      return(discont(op(_f),x) union Solve(op(_f)=Dom::DiscreteSet(i,i=-infinity..0),x))
   of "fact" do return(discont(rewrite(_f,gamma),x))
   # functions with a discontinuity in k*PI #
   of "cot" do
      return(discont(op(_f),x) union Solve(op(_f)=Dom::DiscreteSet(i*PI,i=-infinity..infinity),x))
   # functions with a discontinuity in PI/2+k*PI #
   of "tan" do
      return(discont(op(_f),x) union Solve(op(_f)=Dom::DiscreteSet(PI/2+i*PI,i=-infinity..infinity),x))
   of "int" do
      return(discont(op(_f,1),x) union
         (if type(op(_f,2))="_equal" then discont(op(_f,[2,2,1]),x)
		union discont(op(_f,[2,2,2]),x) else {} end_if))
   end_case;
   s:=op(_f,0); s:=level(s,2);
   if domtype(s)=DOM_FUNC_ENV then
      if (s:= funcattr(s,"discont")(_f,x)) <> FAIL then
	 return(_union(discont(op(_f),x),Solve(op(_f)=op(s,i),x)$i=1..nops(s)))
      end_if
   end_if;   
   procname(args())
end_proc:

Solve:=
proc()
  local s;
begin
  s:=solve(args());
  if type(s)="solve" then
    s
  else # type(s)=DOM_SET #
    # remove complex roots #
    select(s,fun((if testtype(args(1),Type::Constant) and
                    not iszero(Im(args(1)))
                    then
                    FALSE
                  else
                    TRUE
                  end_if)))
   end_if;
end_proc:

# end of file #
