# $Date: 1997/03/13 13:02:35 $  $Author: zimmerma $  $Revision: 1.47 $ #

#++
factor-- return the factorization of multivariate polynomial in Z or
         factorization of univariate polynomial in Zp, output is given in form
         [u, f1, e1, ..., fn, en], (p=u*f1^e1*...*fn^en, u is the constant
         content), fi has the same form of input p. 

factor(p)
p - an expression or a polynomial
++#

factor:=proc(p)
local cc, f, factors, ff, fn, fu, i, j, k, n, pform, pp, q, t, xx;
begin
    if args(0) <> 1 then
       error("one argument expected");
    end_if;
    # test the type of the input expression, and #
    # change the suitable one to polynomial form # 
    case type(p)
    of "_plus" do p:=poly(p); 
		  if p = FAIL then error("not a polynomial") end_if;
                  pform:=FALSE;
                  # fall through #;
    of DOM_POLY do if degree(p) = 0 then return([coeff(p)]) end_if;
		   # remove indets which have 0 degree #
		   xx:= op(p,2);
		   f:= map(xx, fun((
		     if degree(p,args(1))<>0 then args(1) else null() end_if)));
		   if f <> xx then
		      p:= factor(poly(p,f));
		      if pform = FALSE then
			 return([p[1], (expr(p[2*i]), p[2*i+1])
				       $ hold(i)=1..(nops(p) div 2) ]);
		      end_if;
		      return([p[1], (poly(p[2*i],xx), p[2*i+1])
				    $ hold(i)=1..(nops(p) div 2) ]);
		   end_if;
                   # replace indets that are not in DOM_IDENT #
                   f:= select(xx,fun(not testtype(args(1),DOM_IDENT)));
                   if f<>[] then
                      f:=[f[i]=genident()$i=1..nops(f)];
                      p:=poly(subs(op(p,1..3),f));
                      p:=factor(p);
                      f:=[op(f[i],2)=op(f[i],1)$i=1..nops(f)];
                      p:=subs(p,f);
                      if pform<>FALSE then return(p)
                      else
                         return([p[1],(expr(p[2*i]),p[2*i+1])$i=1..nops(p) div 2])
                      end_if
                   end_if;
                   if nterms(p)=1 then
		      if pform = FALSE then
			 return([lcoeff(p), (xx[i], degree(p,xx[i]))
					    $ hold(i)=1..(nops(p) div 2) ]);
		      end_if;
                      return([lcoeff(p),(poly(xx[i],op(p,2..3)),
                              degree(p,xx[i]))$hold(i)=1..nops(xx)]);
                   end_if;
                   if pform <> FALSE then pform:=TRUE end_if;
                   break;
    of "_power" do f:=factor(op(p,1));
                   n:=op(p,2);
                   if testtype(n,DOM_INT) then
                      return(subsop(f,1=f[1]^n,
		         ((2*i+1)=n*f[2*i+1])$hold(i)=1..(nops(f)-1)/2))
                   else # n=1/2 or n symbolic #
                      return(subsop(f,1=f[1]^n,
		         ((2*i)=f[2*i]^n)$hold(i)=1..(nops(f)-1)/2))
                   end_if
    of "_mult" do n:=nops(p);
                  (fu[i]:=factor(op(p,i)))$hold(i)=1..n;
                  fn:=fu[1];
                  for i from 2 to n do
                      fn[1]:=fn[1]*op(fu[i],1); # collect scalars #
                      for j from 1 to (nops(fu[i])-1)/2 do
                          cc:=0;  
                          ff:=op(fu[i],2*j); 
                          for k from 1 to nops(fn) div 2 do
                              if ff=fn[2*k] then 
                                 cc:=1;
                                 break;
                              end_if;
                          end_for;
                          if cc=1 then 
			     fn[2*k+1]:=fn[2*k+1]+op(fu[i],2*j+1);
                          else fn:=append(fn,ff,op(fu[i],2*j+1));
                          end_if;
                      end_for;
                  end_for;
                  # delete zero exponents #
                  for i from nops(fn) div 2 downto 1 do
                     if fn[2*i+1]=0 then fn[2*i+1]:=NIL; fn[2*i]:=NIL end_if
                  end_for;
                  return(fn);
    of DOM_INT do
    of DOM_RAT do
	      return([p]);
    of DOM_IDENT do
    of "_index" do
	      return([1,p,1]);
    otherwise
	      if p::factor <> FAIL then return(p::factor(p))
              else return([1,p,1]) end_if;
    end_case;
    if (q:=op(p,3))<>Expr then
       if op(q,0)<>IntMod then return(faclib::domfact(p)) end_if;
       # factorization in Zp for univariate polynomial #
       if isprime((q:=op(p,[3,1]))) then 
          if nops((xx:=op(p,2)))=1 then
             ff:=[lcoeff(p)];
             p:=multcoeffs(p,1/op(ff));
             if degree((t:=nthterm(p,nterms(p))))<>0 then 
                ff:=append(ff,poly(op(xx),xx,IntMod(q)),degree(t));
                p:=divide(p,t,Exact);
             end_if;
             if degree(p)=1 then 
                return(append(ff,p,1));
             else return(ff.faclib::factor_mod(p,op(xx),q));
             end_if;
         # elif nops(xx)=2 then
             p:=faclib::bifactor(subsop(p,3=Dom::IntegerMod(q)));
             p:=subsop(p,1=expr(p[1]));
             return(subs(p,Dom::IntegerMod(q)=IntMod(q))); #
          else 
	     p:=faclib::multifactor(subsop(p,3=Dom::IntegerMod(q)));
	     p:=subsop(p,1=expr(p[1]));
	     return(subs(p,Dom::IntegerMod(q)=IntMod(q)));
          end_if;
       elif nops(op(p,2)) > 1 then
            error("Multivariate factorization modulo composite numbers "
		   ."not yet implemented")
	else 
	     return([1].faclib::factor_mod_composite(p,op(p,[2,1]),q))
       end_if;
    else # op(p,3)=Expr #
         if not testtype(p, Type::PolyOf(Type::Rational)) then 
           # try with maprat #
           q:=rationalize(expr(p),DOM_POLY);
           if pform or q[2]<>{} then # to avoid infinite loops #
              p:=subs(factor(q[1]),op(q[2]));
              if pform then return([p[1], (poly(p[2*i],xx), p[2*i+1])
                                       $ hold(i)=1..(nops(p) div 2) ]);
              else return(p)
              end_if
           end_if;
           error("Only for polynomial with rational coefficients")
         end_if;
         # factorization in Z #
         factors:=[icontent(p)*sign(lcoeff(p))];
         p:=multcoeffs(p,1/op(factors)); 
         if op((t:=faclib::monomial(p)),1)<>[] then
            factors:=factors.op(t,1);
	    p:= divide(p, poly(op(t,2), op(p,2..3)), Exact);
            # PZ : added the following line because divide does not
              reduce the number of indets, which causes a loop
              for example with factor(a^2*k-2*a*k+k) #
            p:=poly(expr(p));
         end_if;
         pp:=faclib::pfactor(p,0);
         # check sign #
         if expr(poly(_mult(_power(expr(op(pp,2*i-1)),op(pp,2*i))\
                         $hold(i)=1..nops(pp)/2)))<>expr(p) then 
            factors[1]:= -factors[1];
         end_if; 
    end_if;
    if pform then 
       factors:=factors.pp;
       return(subsop(factors,
           ((2*i)=subsop(op(factors,2*i),2=xx)
               $ hold(i)=1..(nops(factors)-1)/2)
       ))
    else
       # change to expression form #
       factors:=factors.pp;
       return(subsop(factors,((2*i)=expr(op(factors,2*i)))\
                                    $hold(i)=1..(nops(factors)-1)/2)); 
    end_if;        
end_proc:

alias(POWER(a,b)=(if b=1 then a else hold(_power)(a,b) end_if)):

DOM_INT::Factor := proc(p) # "pretty" output of ifactor #
  local l, i;
begin
  l:=ifactor(p);
  case nops(l)
  of 1 do return( l[1] )
  of 3 do
    if l[3] = 1 then
      l[2]
    else
      hold(_power)(l[2],l[3]);
    end_if;
    break
  otherwise
    hold(_mult)((if l[2*i + 1] = 1 then 
                   l[2*i]
 	         else
                   hold(_power)(l[2*i],l[2*i + 1]) 
                 end_if)  $ i = 1..nops(l) div 2)
  end_case ;
  if l[1] = -1 then 
    -% 
  else 
    % 
  end_if;
end_proc:

Factor := proc(p) local l,i;
begin
   if p::Factor <> FAIL then return( p::Factor(p) ) end_if;
   normal; l:=stdlib::normal(p); # [num,den] #
   if l[2]=1 then # no denominator #
      if map(indets(l[1],PolyExpr),type)<>{DOM_IDENT} then
         l:=maprat(l[1],factor)
      else l:=factor(l[1]) end_if;
      if nops(l)=1 then l[1] # no variable #
      elif iszero(l[1]-1) then
         if nops(l)=3 then l[2]^l[3]
         else _mult(l[2*i]^l[2*i+1]$i=1..nops(l) div 2)
         end_if
      else _mult(l[1],l[2*i]^l[2*i+1]$i=1..nops(l) div 2)
      end_if
   else # rational case #
      l:=map(l,Factor);
      l[1]/l[2]
   end_if
end_proc:

# special case for polynomials which automatically expand #
DOM_POLY::Factor:=proc(p) local l,i;
begin
      l:=factor(p);
      if nops(l)=1 then l[1] # no variable #
      elif iszero(l[1]-1) then
         if nops(l)=3 then POWER(l[2],l[3])
         else hold(_mult)(POWER(l[2*i],l[2*i+1])$i=1..nops(l) div 2)
         end_if
      else hold(_mult)(l[1],POWER(l[2*i],l[2*i+1])$i=1..nops(l) div 2)
      end_if
end_proc:

# returns a set of all factors of p #
factors := proc(p) local l,r,j,m,a;
begin
   if type(p)=DOM_LIST then l:=p;p:=l[2] else l:=factor(p) end_if;
   r:=0;
   for j from 1 to nops(l)/2 do
      for m from 1 to l[2*j+1] do a[r+m]:=l[2*j] end_for;
      r:=r+l[2*j+1];
   end_for;
   stdlib::multiples_of(a,r,p^0);
end_proc:

# find all the products of a[j], 1<=j<=r #
# p is an auxiliary product, initially 1 #
stdlib::multiples_of := proc(a,r,p)
begin
   if r=0 then {p}
   else
      stdlib::multiples_of(a,r-1,p) union 
      stdlib::multiples_of(a,r-1,expand(p*a[r]))
   end_if
end_proc:

# end of file #
