# $Date: 1997/10/10 15:56:28 $  $Author: tonner $  $Revision: 1.7 $ #
# kg, 15/07/94 #

#++
fun -- create a pure function

fun(e [,Named][,Name=String][,hold])

e      - expression, a function of args(1),...args(n)
Named  - option
Name   - option
String - a character string, the name of the fun
hold   - option

fun views the expression e as a function of the indeterminates args(i)
and returns either a 'pure function' computing that function.

A pure function is a special form of procedure which evaluates in the
context of the caller and is faster than procedures.

If the option 'Named' is used the function name is set to a default
value. With 'Name=String' this name can be given explicitly. Other-
wise it is anonymous, printed as 'fun(...)'.

If the option 'hold' is used the function created does not evaluates
its arguments.

The first argument 'e' of fun are not evaluated!

Examples:

f:= fun(sin(args(1)^2+args(2)^2));		-> fun(...)
f(x,1);						-> sin(x^2+1)

f:= fun(sin(args(1)^2+args(2)^2), Named);	-> fun(sin(args(1)^2+args(2)^2)
f(x,1);						-> sin(x^2+1)

f:= fun(args(1)+1, Name="Inc");                 -> Inc->fun(...)
f(42);                                          -> 43

f:= fun(args(1), hold);                         -> fun(...,hold)
f:= fun(args(1), hold, Named);                  -> fun(args(1),hold)
f:= fun(args(1), hold, Name="f");               -> f->fun(...,hold)
f(42+13);                                       -> 42 + 13
++#

# In most cases no options or only 'hold' are given, handle this fast... #

fun:= proc()
  option hold, noDebug;
begin
  if type(args(1)) = "_exprseq" then
    hold(op)([args(1)])
  else
    args(1)
  end_if;
  if args(0) = 1 then 
    newpurefunc("fun(...)", NIL, %, FALSE)
  elif args(0) = 2 and args(2) = hold(hold) then
    newpurefunc("fun(...,hold)", NIL, %, TRUE)
  else 
    stdlib::full_fun(args())
  end_if
end_proc:

# Handle general case if naming options are given... # 

stdlib::full_fun:= proc()
    option noDebug;
    local  OptionList, arg, NameString, HoldOption;
begin
    if args(0) = 0 then error("wrong no of args") end_if;
    NameString:="fun(...";
    HoldOption:=FALSE;
    OptionList:= context([args(2..args(0))]);

    for arg in OptionList do
       if arg = hold then 
	   HoldOption:= TRUE;
	   
       elif arg = hold(Named) then  
           NameString:= "fun(".expr2text(args(1));

       elif type(arg) <> "_equal" then
           error( "unknown option" );

       elif op(arg,1) <> hold(Name) or type(op(arg,2)) <> DOM_STRING then
           error("Name = string expected");
           
       else
           NameString:=op(arg,2)."->fun(...";
       end_if;
    end_for;

    if( HoldOption ) then NameString:= NameString.",hold)"
                     else NameString:= NameString.")"
    end_if;
    newpurefunc(NameString, NIL, args(1), HoldOption);
end_proc:

# end of file #
