# $Date: 1999/04/28 09:41:42 $ $Author: fakler $ $Revision: 1.30.2.4 $ #

# kg, 09/12/93 #

#++
gcd -- compute the gcd of polynomials or expressions

gcd(poly,...)
gcd(expr,...)

poly - polynomials
expr - polynomial expressions

gcd computes the gcd of polynomials or expressions.
++#

gcd:= proc()
    local a, p, q, n, c, T, i, g, f, conv, normp, gcdp;
begin
    if args(0) = 0 then return(0) end_if;
    a:= { args() }; //a:=level(a);  // to evaluate arguments
    
    # only integer or rational args? #
    T:= map(a, domtype);
    if (T minus {DOM_INT,DOM_RAT,DOM_FLOAT,DOM_COMPLEX}) = {} then
	if T = {DOM_INT} then return(igcd(op(a))) end_if;
	return(1)
    end_if;
    
    p:= op(a,1);
    n:= nops(a);
    # handle cases n=1 and n>2 #
    case n
    of 2 do break;
    of 1 do
	T:= op(T,1);
	if T::zero <> FAIL then return(gcd(p, T::zero)) end_if;
	if T = DOM_POLY then
	    if iszero(p) then return(p) # avoid infinite recursion #
	    else return(gcd(p, poly(0,op(p,2..3))))
	    end_if
	end_if;

	return(gcd(p, 0));

    otherwise
	if T minus {DOM_EXPR, DOM_POLY} <> {} and 
	   map(T minus {DOM_EXPR, DOM_POLY}, 
		func(g::hasProp(Cat::PolynomialCat),g)) <> {TRUE} then
		
	    g:= op(a,1);
            for i from 2 to n do
                g:= gcd(g, op(a,i))
            end_for;
	    return(g)
	else
        # arguments are polynomials or expressions #
	# remove contents #
	    a:= [op(a)];
	    c:=select(a,not iszero);
	    if nops(%)=0 then
		return(gcd(a[1],a[2]))
	    elif nops(c)<=2 then
		 return(gcd(op(c)))
	    else 
		 n:=nops(c);
		 a:=c
            end_if;
	    c:= map(a, content);
	    if contains(c, FAIL) <> 0 then return(FAIL) end_if;
	    a:= zip(a, c, fun(mapcoeffs(args(1), _divide, args(2))));
	    c:= gcd(op(c));
	    # quick exit if there is a constant polynomial among
	      the arguments #
	    if contains(map(a,degree),0)>0 then
		return(poly(c,op(p,2..3)))
	    end_if;
            T:= op(p, 3);
                    # sort according to ascending degrees, then iterate gcd #
            a:= sort(a, fun(bool(degree(args(1)) < degree(args(2)))));
            g:= a[1];
            for i from 2 to n do
                g:= gcd(g, a[i])
            end_for;
            return(multcoeffs(g, c))
        end_if;
    end_case;

    # now n=2 #
    q:= op(a,2);
    # try to overload #
    if p::gcd <> FAIL then return(p::gcd(p,q)) end_if;
    if q::gcd <> FAIL then return(q::gcd(p,q)) end_if;

    # convert args to polys #
    if T = {DOM_POLY} then
	T:= op(p, 3);
	conv:= id;
	# test args #
	if testargs() then
	    if op(p, 2..3) <> op(q, 2..3) then
		error("polynomial types differ")
	    end_if;
	    if domtype(T) = DOM_DOMAIN then
		# need gcd method for coeffs #
		if T::gcd = FAIL then error("method 'gcd' missing") end_if;
		if T::_divide = FAIL then error("method '_divide' missing") end_if
	    elif T = hold(Expr) then
		# can't handle real or complex coeffs #
		case gcdlib::coeff_type([p,q])
		of DOM_INT do
		of DOM_RAT do
		    break;
		otherwise
		    error("illegal coefficients")
		end_case
	    else # T = IntMod(p) #
		# polynomial with coeffs from Zp must have prime modulus #
		if not isprime(op(T,1)) then
		    error("modulus must be prime")
		end_if
	    end_if;
	end_if;
    else
	# convert expr to polys #
	T:= hold(Expr);
	conv:= expr;
	i:= indets(a, hold(PolyExpr));
	if (i = {}) or (i minus indets(i, hold(RatExpr)) <> {}) then
	    a:= op(a);
	    i:= map(a, normal);
	    if i = a then return(FAIL) end_if;
	    return(gcd(i))
	end_if;
	i:= [op(i)];
	p:= poly(p, i);
	q:= poly(q, i);
	if p = FAIL or q = FAIL then
	    a:= op(a);
	    i:= map(a, normal);
	    if i = a then return(FAIL) end_if;
	    return(gcd(i))
	end_if;
    end_if;
    
    if iszero(p) then return(conv(q)) end_if;
    if iszero(q) then return(conv(p)) end_if;

    f:= [];
    if T = hold(Expr) then
	# polynomial with expressions as coeffs #
	# change varibles which are expressions into indets,
	  otherwise scaling may cause problems later on #
	if select(map(op(p,2), domtype), _unequal, DOM_IDENT) <> [] then
	    f:= op(p,2);
	    g:= [ (if domtype(f[i]) = DOM_IDENT then f[i]
		   else genident() end_if) $ hold(i)=1..nops(f) ];
	    p:= poly(poly2list(p), g, T);
	    q:= poly(poly2list(q), g, T);
	end_if;
	T:= map({coeff(p), coeff(q)}, domtype);
	if T = {DOM_INT} then
	    gcdp:= gcdlib::int_gcd;
	    normp:= gcdlib::norm_intp;
	elif T minus {DOM_INT,DOM_RAT, DOM_COMPLEX} = {} then
	    if contains(T, DOM_COMPLEX) then
		# regard I as an indeterminate #
		i:=genident();
		p:=subs(p,I=i);
		q:=subs(q,I=i);
		//return(subs(gcd(expr(p),expr(q)),i=I))
                if conv=expr then
                  return(subs(gcd(expr(p),expr(q)),i=I))
                elif f <> [] then
                  return(poly(subs(gcd(expr(p),expr(q)),i=I),f,op(p,3)))
                else
                  return(poly(subs(gcd(expr(p),expr(q)),i=I),op(p,2..3)))
                end_if;
	    end_if;		
	    gcdp:= gcdlib::int_gcd;
	    normp:= fun(args(1));
	    conv:= conv @ fun(multcoeffs(args(1), 1/lcoeff(args(1))));
	    p:= multcoeffs(p, 1/icontent(p));
	    q:= multcoeffs(q, 1/icontent(q));
	else
	    # we try our best... #
	    if conv = expr then
		i:= [ mapcoeffs(p, normal), mapcoeffs(q, normal) ];
		if i = [p, q] then error("illegal coefficients") end_if;
		return(expr(gcd(op(i))))
	    end_if;
	    g:= gcd(expr(p), expr(q));
	    if g = FAIL then
		error("illegal coefficients")
	    end_if;
	    g:= poly(g, op(p, 2..3));
	    if f <> [] then g:= poly(poly2list(g), f, op(p,3)) end_if;
	    return(g);
	end_if
    elif domtype(T) = DOM_DOMAIN then
	gcdp:= gcdlib::dom_gcd;
	normp:= gcdlib::norm_domp;
    else # T = IntMod(p) #
	gcdp:= gcdlib::mod_gcd;
	normp:= gcdlib::norm_modp;
	conv:= fun(multcoeffs(args(1), 1/lcoeff(args(1))));
    end_if;
    
    g:= gcdlib::special_cases(p, q, gcdp, normp);
    if f <> [] then g:= poly(poly2list(g), f, op(p,3)) end_if;
    conv(g);
end_proc:

# end of file #
