# $Date: 1998/01/15 12:58:32 $ $Author: stefanw $ $Revision: 1.35 $ #

#++
infinity -- representing the real positive infinity

An infinite number is represented as an element of the domain
stdlib::Infinity. An element of this domain has one operand:

- The sign, which is an non-zero expression.

- Given the sign s the elements represent limes(s*a, a = infinity).

The domain element 'infinity' (which represents the positive real
infinity) is defined here. Other domain elements are only created
via the operations of this domain and NOT via 'new'.
++#

proc()
    option noDebug;
    local inf;
begin

inf:= domain("stdlib::Infinity"):

infinity:= new(inf, 1):

inf::info:= "Domain 'stdlib::Infinity': representing infinity":
inf::interface:= {}:

inf::expr:= fun(expr(extop(args(1),1)) * hold(infinity)):

inf::print:= inf::expr:

inf::evaluate:= fun((
    extop(args(1),1);
    # eval(%); #
    subs(%, Simplified);
    if iszero(%) then % else extsubsop(args(1), 1 = %) end_if)):

inf::TeX:= fun((
    case extop(args(1),1)
    of 1 do "\\infty"; break;
    of -1 do "- \\infty"; break;
    otherwise generate::TeX(extop(args(1),1))." \\infty";
    end_case)):

inf::_negate:= fun(extsubsop(args(1), 1 = -extop(args(1),1))):

inf::_invert:= fun(0):

inf::_subtract:= fun(args(1)+(-args(2))):

inf::_divide:= fun(args(1)*(1/args(2))):

inf::not:= fun(error("invalid operand")):

inf::_power:= fun((
    case domtype(args(2))
    of stdlib::Infinity do
        if domtype(args(1)) = stdlib::Infinity or 
           args(1) = complexInfinity or
           args(1) = undefined then
            return(undefined) 
        end_if;
        if args(2) = infinity then
           if is(args(1)>1) = TRUE then return(args(2))
           elif (is(-1<args(1)) and is(args(1)<1)) = TRUE then return(0)
           end_if
        else
            return((args(1)^extop(args(2),1))^infinity)
        end_if;
	return(hold(_power)(args()));
    of DOM_FLOAT do
        if iszero(args(2)) then return(1.0) end_if;
    of DOM_INT do
        if args(2) = 0 then return(1) end_if;
    of DOM_RAT do
	if args(2) < 0 then return(0) end_if;
	(extop(args(1),1)^args(2)) * infinity;
	break;
    otherwise
        if is(args(2)<0) = TRUE then return(0) end_if;
	hold(_power)(args())
    end_case)):

inf::_plus:= proc() local a, b, c; begin
    # select infinities and others #
    a:= select([args()], not (stdlib::Infinity)::thisDomain);
    a:= _plus(op(a));
    if a = undefined then return(undefined) end_if;
    if a = complexInfinity then return(complexInfinity) end_if;
    b:= select({args()}, (stdlib::Infinity)::thisDomain);
    c:= select(map(b, extop, 1), fun(bool(indets(args(1))={})));
    if nops(c) > 1 then return(undefined) end_if;
    if nops(c) = 1 then
        if domtype(op(c,1)) = DOM_COMPLEX and op(c,[1,1]) = 0 then
            a:= (stdlib::Infinity)::removeIm(a)
	elif testtype(op(c,1), Type::RealNum) then
            a:= (stdlib::Infinity)::removeRe(a)
        end_if
    end_if;
    
    if a = 0 then
	if nops(b) = 1 then
	    op(b)
	else
	    hold(_plus)(op(b))
	end_if
    elif type(a) = "_plus" then  # flat operands of a #
        hold(_plus)(op(a), op(b))
    else
        hold(_plus)(a, op(b))
    end_if
end_proc:

inf::_mult:= proc() local a, b, c, f; begin
    b:= [args()];
    if contains(b, undefined) <> 0 then return(undefined) end_if;
    if contains(b, complexInfinity) <> 0 then return(complexInfinity) end_if;
    # multiply in b the infinities and in c and a the other arguments #
    a:= select(b, not (stdlib::Infinity)::thisDomain);
    c:= select(a, not has, infinity);
    if nops(c) = nops(a) then
	a:= 1
    else
	a:= _mult(op(select(a, has, infinity)))
    end_if;
    # select infinities #
    b:= select(b, (stdlib::Infinity)::thisDomain);
    f:= b[1];
    b:= expand(_mult(op(c), op(map(b, extop, 1))));
    if iszero(b) then
       # 0*infinity is undefined #
       return(undefined)
    end_if;
    case type(b)
    of "_plus" do
        return(_plus(map(op(b), _mult, infinity, a)));
    of DOM_INT do
    of DOM_RAT do
    of DOM_FLOAT do
        b:= (if b > 0 then 1 else -1 end_if);
        break;
    # otherwise -- too expensive?
        b:= sign(b); #
    end_case;
    
    b:= extsubsop(f, 1=b);

    if a = 1 then
	b
    elif type(a) = "_mult" then  # flat operands of a #
        hold(_mult)(op(a), b)
    else
        hold(_mult)(a, b)
    end_if
end_proc:

inf::_less:= proc(x, y) begin
    if domtype(x) = stdlib::Infinity then
        if domtype(extop(x,1)) <> DOM_INT then
            return(hold(_less)(x, y))
        end_if;
        if domtype(y) = stdlib::Infinity then
	    if x = y then return(FALSE) end_if;
	    if domtype(extop(y,1)) <> DOM_INT then
		return(hold(_less)(x, y))
	    end_if;
	    return(bool(x <> infinity));
	else
	    if not testtype(y, Type::RealNum) then
		return(hold(_less)(x, y))
	    end_if;
	    return(bool(x <> infinity));
	end_if
    elif domtype(y) = stdlib::Infinity then
        if domtype(extop(y,1)) <> DOM_INT then
            return(hold(_less)(x, y))
        end_if;
        if not testtype(x, Type::RealNum) then
            return(hold(_less)(x, y))
        end_if;
        not (stdlib::Infinity)::_less(y, x)
    else
        hold(_less)(x, y)
    end_if
end_proc:

inf::_leequal:= proc(x,y)
    local c;
begin
    if x = y then
        TRUE 
    else
        c:= (stdlib::Infinity)::_less(x, y);
        if domtype(c) = DOM_BOOL then
            c
        else
	    hold(_leequal)(x, y)
        end_if
    end_if
end_proc:

inf::min:= proc() local a, aa, b, c, f; begin
    b:= { args() };
    # collect in b infinities, in c real numbers and in a the other arguments #
    a:= select(b, not (stdlib::Infinity)::thisDomain);
    f:= fun(testtype(args(1), Type::RealNum));
    aa := select(a, f);
    if nops(aa) > 0 then
  	c := min(op(aa));	
    else
	c := infinity;
    end_if;
    a:= select(a, not f);

    # collect in b infinities #
    b:= select(b,(stdlib::Infinity)::thisDomain);
    f:= fun(contains({-1,1}, extop(args(1),1)));
    a:= a union select(b, not f);
    b:= select(b, f);

    if b = {} then
    	if c = infinity then
    	    b:= null()
    	else
    	    b:= c
    	end_if
    else
	# get minimal real infinity #
	if contains(b, -infinity) then b:= -infinity
	else b:= infinity
	end_if;
	if (stdlib::Infinity)::_less(c, b) then
	    b:= c
	end_if
    end_if;

    if a = {} then
        b
    else
        hold(min)(op(a), b)
    end_if
end_proc:

inf::max:= proc() local a, aa, b, c, f; begin
    b:= { args() };
    # collect in b infinities, in c real numbers and in a the other arguments #
    a:= select(b, not (stdlib::Infinity)::thisDomain);
    f:= fun(testtype(args(1), Type::RealNum));
    aa := select(a, f);
    if nops(aa) > 0 then
  	c := max(op(aa));	
    else
	c := -infinity;
    end_if;
    a:= select(a, not f);

    # collect in b infinities #
    b:= select(b,(stdlib::Infinity)::thisDomain);
    f:= fun(contains({-1,1}, extop(args(1),1)));
    a:= a union select(b, not f);
    b:= select(b, f);
    
    if b = {} then
    	if c = -infinity then
    	    b:= null()
    	else
    	    b:= c
    	end_if
    else
	# get maximal real infinity #
	if contains(b, infinity) then b:= infinity
	else b:= -infinity
	end_if;
	if (stdlib::Infinity)::_less(b, c) then
	    b:= c
	end_if
    end_if;

    if a = {} then
        b
    else
        hold(max)(op(a), b)
    end_if
end_proc:

inf::intmult:= fun((
    if iszero(args(2)) then undefined
    elif args(2) > 0 then args(1)
    else extsubsop(args(1), 1 = -extop(args(1),1))
    end_if)):

inf::thisDomain:= fun(bool(domtype(args(1)) = stdlib::Infinity)):

inf::simplify:= fun(
    extsubsop(args(1), 1 = simplify(extop(args(1),1)))):

inf::sign:= fun(sign(extop(args(1),1))):

inf::subs:= fun((
    if domtype(op(args(2),1)) = stdlib::Infinity then
	# eval(subs(expr(args(1)), expr(op(args(2),1)) = op(args(2),2))) #
	subs(expr(args(1)), expr(op(args(2),1)) = op(args(2),2), Simplified)
    else
	# eval(subs(expr(args(1)), args(2))) #
	subs(expr(args(1)), args(2), Simplified)
    end_if)):

inf::subsop:= fun(
    # eval(subsop(expr(args(1)), args(2)=args(3)))): #
    subsop(expr(args(1)), args(2)=args(3), Simplified)):

inf::op:= fun((
    op(expr(args(1)), (if args(0) = 2 then args(2) else null() end_if));
    # eval(%))): #
    subs(%, Simplified))):

inf::has:= fun((
    if args(2) = infinity then
	TRUE
    elif domtype(args(2)) = stdlib::Infinity then
	has(expr(args(1)), expr(args(2)))
    else
	has(extop(args(1),1), args(2))
    end_if)):

inf::O:= fun(undefined):

inf::removeRe:= fun((
    case type(args(1))
    of "_plus" do
        map(args(1), (stdlib::Infinity)::removeRe); break;
    of DOM_INT do
    of DOM_RAT do
        0; break;
    of DOM_FLOAT do
        0.0; break;
    of DOM_COMPLEX do
        I * op(args(1),2); break;
    otherwise
        I * Im(args(1));
    end_case)):

inf::removeIm:= fun((
    case type(args(1))
    of "_plus" do
        map(args(1), (stdlib::Infinity)::removeIm); break;
    of DOM_COMPLEX do
        op(args(1),1); break;
    otherwise
        Re(args(1));
    end_case)):

inf::Re := id:

inf::Im := 0:

sysassign(stdlib::Infinity, inf):

end_proc():

# end of file #
