
#++
int -- integration

int(f,x)
int(p)
int(f,x=a..b)
int(f,x=a..b,PrincipalValue)

f - expression
x - identifier
p - procedure with one argument (piecewise defined)

This is merely a front end for indefinite integration via intlib::int,
and definite via intlib::intdef.
++#

proc() begin

int:= funcattr( proc() begin
   if args(0)=0 then error("wrong no of args in int(...)") end_if;
   if (args(1))::int <> FAIL then return((args(1))::int(args())) end_if;
   if args(0)=1 then error("wrong no of args in int(...)") end_if;
   if type(args(2))=DOM_IDENT then # indefinite integration #
      if args(0)<>2 then error("wrong no of args in int(...)") end_if;
      intlib::int(args())
   else
      if testargs() then
	 if type(args(2)) <> "_equal" then
	    error("illegal second argument in int(...)")
	 end_if;
	 if domtype(op(args(2),1)) <> DOM_IDENT then
	    error("integration variable expected, but got ".expr2text(op(args(2),1)))
	 end_if;
	 if type(op(args(2),2)) <> "_range" then
	    error("no range of integration in int(...)")
	 end_if;
	 if has(op(args(2),2), op(args(2),1)) then
	    error("range depends upon integration variable in int(...)")
	 end_if;
         if args(0)>3 then error("wrong no of args in int(...)") end_if;
         if args(0)=3 then
            if args(3)<>PrincipalValue then error("invalid third argument in int(...)") end_if
         end_if
      end_if;
      intlib::definite(args(1),op(args(2),1),op(op(args(2),2)),bool(args(0)=3))
   end_if
end_proc, "type", "int" ):

int:= funcattr( int,"info","definite and indefinite integration [Try ?int for details]" ):

intlib::definite := proc(_f_,_x_,_a_,_b_,pv) # int(f,x=a..b), pv=TRUE for principal value #
local i,_g_,_s,y,fac,eps,lr,ia,ib; //gleft,gright;
begin
   if testtype(_f_,Type::PolyExpr(_x_)) and not pv then
      # no singularity possible #
      _f_:=poly(_f_,[_x_]);
      _f_:=_plus(coeff(_f_,i)*_x_^(i+1)/(i+1) $ i=0..degree(_f_));
      stdlib::int_eval(_f_,_x_=_b_)-stdlib::int_eval(_f_,_x_=_a_)
   elif is(_b_<_a_)=TRUE then return(-intlib::definite(_f_,_x_,_b_,_a_,pv))
   else 
     // test if integrand contains special functions that can be treated
     integral:=intlib::specialFunctionDef(f,x,a,b,pv);
     if integral<>FAIL then return( integral )
       //else return( hold(int)(f,x=a..b) )
     //end_if;
     end_if;
      # first try the lookup method #
      # remove constant factors before lookup method #
      if type(_f_)="_mult" then
         _g_:=select(_f_,has,_x_); fac:=select(_f_,not has,_x_)
      else _g_:=_f_; fac:=1
      end_if;
      i:=intlib::intdef(_g_,_x_,_a_.._b_,pv);
      if i<>FAIL then return(fac*i) end_if;
      userinfo(1,"lookup method failed, try to use an antiderivative");
      # then try using an antiderivative #
      _g_:=int(_f_,_x_); # antiderivative #
      userinfo(2,"antiderivative is",_g_);
      if type(_g_)<>"int" then
         _s:=(if not has(_f_,dirac) then
                if has(_g_,I) then _g_:=simplify(_g_); end_if;
		discont(_g_,_x_=_a_.._b_); # discontinuities in a..b #
	      else {} end_if);
         userinfo(2,"discontinuities of the antiderivative are",_s);
         if type(_s)=DOM_SET then
            if contains(_s,_a_) then
              lr:=intlib::boundaryTest(_g_,_x_=_a_,Right);
              if lr[1] then ia:=lr[2] else return(hold(int)(_f_,_x_=_a_.._b_))
              end_if;
            else
              ia:=eval(subs(_g_,_x_=_a_));
            end_if;
            if contains(_s,_b_) then
              lr:=intlib::boundaryTest(_g_,_x_=_b_,Left);
              if lr[1] then ib:=lr[2] else return(hold(int)(_f_,_x_=_a_.._b_))
              end_if;
            else
              ib:=eval(subs(_g_,_x_=_b_));
            end_if;
            i:=ib-ia;
           // i:=-stdlib::int_eval(_g_,_x_=_a_,Right,contains(_s,_a_))
           //    +stdlib::int_eval(_g_,_x_=_b_,Left,contains(_s,_b_));
            _s:=_s minus {_a_,_b_};
            for y in _s do
               if pv=TRUE then # compute principal value #
                  i:=i+limit(subs(_g_,_x_=y-eps)-
                             subs(_g_,_x_=y+eps),eps=0,Right);
               else
                 // gleft:=limit(_g_,_x_=y,Left);
		 // gright:=limit(_g_,_x_=y,Right);
		 // i:=i+gleft-gright
                 lr:=intlib::poleLRTest(_g_,_x_=y);
                 if has(lr,{infinity,-infinity,undefinded}) then
                   return(undefined)
                 elif lr[1] then i:=i+lr[2]-lr[3];
                 else return(hold(int)(_f_,_x_=_a_.._b_))
                 end_if;                 
               end_if
            end_for;
            return(simplify(i))
         end_if
      end_if;
      if _a_=_b_ then 0 else hold(int)(_f_,_x_=_a_.._b_) end_if
   end_if
end_proc:

intlib::boundaryTest:= // test, if lower or upper bound of given range is
  proc(_f,_x,dir)        // bounded
    local t,r;
  begin
    userinfo(6,"test, if antiderivative is bounded at ".expr2text(x));
    r:=limit(_f,_x,dir);
    t:=(if has(r,{infinity,-infinity,undefined}) then FALSE else TRUE end_if);
    [t,r]
  end_proc:

intlib::poleLRTest:=  // test, if limit,s result is correct most probably 
  proc(_f,_x)
    local r;
  begin
   if type(op(_x,2))=stdlib::Infinity then 
     r:=limit(_f,_x);
     r:=[r,r];
   else
     r:=[limit(_f,_x,Left),limit(_f,_x,Right)]
   end_if;
   [(if has(r,{infinity,-infinity,undefined}) or r[1]<>r[2] then TRUE
     else FALSE
     end_if), op(r)]
  end_proc:

# stdlib::int_eval(f,x=a,dir,typ) evaluates f in x=a with direction dir (Left or Right)
  and typ=TRUE iff a is a singular point #
stdlib::int_eval := fun((
   if type(op(args(2),2))=stdlib::Infinity then limit(args(1),args(2))
   elif args(0)>=4 and args(4) then limit(args(1),args(2),args(3))
   # else eval(subs(args(1),args(2))) #
   else subs(args(1),args(2),Simplified)
   end_if
)):

int:= funcattr(int, "type", "int"):
int:= funcattr(int, "print", "int"):

# diff-attribute of int #

int:= funcattr(int, "diff",
    proc(f)
	local i, a, u, t, x;
    begin
	if args(0) = 1 then return(f) end_if;

	a:= []; # variables that appear in the integration variable or the bounds #
	u:= []; # other variables #
	for i from 2 to args(0) do
	    if has(op(f,2), args(i)) then
		a:= append(a, args(i))
	    else
		u:= append(u, args(i))
	    end_if
	end_for;

	# exchange diff and int #
	if nops(u) <> 0 then
	    return(diff(int(diff(op(f,1), op(u)), op(f,2)), op(a)))
	end_if;

	i:= contains(a, op(f,2));
	if i = 0 then
	    # diff(int(g,t=a..b),y) #
	    if contains(a, (t:=op(f,[2,1]))) = 0 then
		# diff(int(g,t=a(x)..b(x)),x) #
		# hold(diff)(args()) #
                x:=a[1]; a[1]:=NIL;
                diff(subs(op(f,1),t=op(f,[2,2,2]))*diff(op(f,[2,2,2]),x)
                -subs(op(f,1),t=op(f,[2,2,1]))*diff(op(f,[2,2,1]),x)
                +int(diff(op(f,1),x),op(f,2)),op(a))
	    else # diff(int(g,x=a..b),x) #
		0
	    end_if
	else # diff(int(g,x),x) #
	    a[i]:= NIL;
	    diff(op(f,1), op(a))
	end_if
    end_proc):

# float-attribute of int (numerical integration) #

int:= funcattr(int, "float", proc(f, x) local r; begin
   r := intlib::numint(f,x);
   if r = FAIL then hold(int)(f, x) else r end_if
end_proc):

DOM_PROC::int := fun((if args(0)=1 then 
	D; stdlib::mapproc(args(1),int,op(args(1),1))
	else error("illegal arguments") end_if)):

end_proc():

# end of file #
