# $Date: 1999/09/27 21:51:08 $ $Author: frankp $ $Revision: 1.44.2.7 $ #
# #
# frankp, 11.01.1995 #

#++
   limit.mu

	limit  -- computing the limit of expressions

	limit(expr,x[=a] [,dir])

	expr -- expression in x
	x    -- limit variable x (an identifier)
	a    -- (optional) limit point (possible infinity or -infinity)
		default is 0
	dir  -- (optional) direction: one of 'Left' or 'Right'

	The limit function attempts to compute the limiting value of expr
	as x goes to a.
   
	If dir is not specified, the limit is the real bidirectional limit,
	except in the case where the limit point is infinity or -infinity.
	In this case the limit is from the left to infinity and from the 
	right to -infinity, respectivly.
   
	limit only consider real valued functions, and any assigned 
	variables in the expression are assumed to be real.

	limit returns 'undefined' iff the limit is not defined, or 
	FAIL if ORDER is not big enough for series.
	If the limit could not be computed then limit returns
	unevaluated, i.e. the result is of type "limit".

	The Algorithm based on the paper of Gaston H. Gonnet and
	Dominik Gruntz: Limit Computation in Computer Algebra,
	Nov. 1992.
++#

limit := proc(ex,pointt)
    name limit;
    local __LIMITx, s, t, lp, nex, _x, ____OMEGA,____w____;
begin
    if testargs() then
        if args(0) < 2 or args(0) > 3 then
            error("wrong no of args")
        end_if;
        if type(pointt) = "_equal" then
            if domtype(op(pointt,1)) <> DOM_IDENT then
        	error("invalid limit variable")
            end_if;
            if has( op(pointt,2), op(pointt,1) ) then
                error("expansion variable appears in limit point")
            elif contains({"_equal","_range","_less","_leequal","_unequal",DOM_BOOL,DOM_TABLE,DOM_SET,
                         DOM_LIST,DOM_PROC,DOM_ARRAY,DOM_FAIL,DOM_NULL,DOM_NIL},type(op(pointt,2))) 
            then
                error("invalid limit point")
            end_if
	elif domtype(pointt) <> DOM_IDENT then
	    error("invalid limit variable")
	end_if;
	if args(0) = 3 then
            if not contains({Right,Left},args(3)) then
                error("unknown direction")
            elif op(pointt,2) = infinity and args(3) = Right or
                 op(pointt,2) = -infinity and args(3) = Left
            then
                error("inconsistent direction")
            end_if
	end_if
    end_if;

    case type( ex ) 
    of "limit" do
        _x:= op(ex,1);
        t:= genident();
        if type(pointt) = "_equal" then
	    lp:= op(pointt,2);
	    if lp = infinity then
	        assume( t > 0,_and );
	        assume( ln(t) > 0,_and );
        	sysassign(sign(t), 1);
        	sysassign(sign(ln(t)),1);
	    elif lp = -infinity then
                assume( t < 0,_and );
                sysassign(sign(t), -1);
	    elif testtype(lp,Type::RealNum) then
		case sign(lp)
		of 1 do
		    assume( t>0,_and ); 
		    if lp-1 > 0 then assume(ln(t)>0,_and)
		    elif lp-1 < 0 then assume(ln(t)<0,_and)
		    end_if;
		    sysassign( sign(t), 1 );
		    break
		of -1 do
		    assume( t<0,_and ); 
		    sysassign( sign(t), -1 );
		    break
		of 0 do
		    if args(0) = 3 then
		 	case args(3) 
			of hold(Left) do 
			    assume(t<0,_and);
			    break
			of hold(Right) do
			    assume(t>0,_and);
			    break
			end_case
		    end_if
                otherwise
                    assume( t,Type::RealNum,_and );
		end_case
	    else
		return( procname(args()) )
	    end_if;

            _x:= subs( _x,op(pointt,1)=t );
            s:= limit( _x,op(ex,2..nops(ex)) );
        else
            if args(0) = 3 then
                case args(3) 
                of hold(Left) do 
                    assume(t<0,_and);
                    break
                of hold(Right) do
                    assume(t>0,_and);
                    break
                end_case
            else
                assume( t,Type::RealNum,_and );
            end_if;

            _x:= subs( _x,op(pointt,1)=t );
            s:= limit( _x,op(ex,2..nops(ex)) );
        end_if;

        if s = ex then
            unassume( t );
            return( procname(args()) )
        elif type(s) = "limit" then
            unassume( t );
            return( procname(subs(s,t=op(pointt,1)),pointt) )
        else
            s:= limit( subs(s,t=op(pointt,1)),pointt );
            unassume( t );
            return( s )
        end_if
    of DOM_PROC do
    of DOM_EXEC do
    of "fun" do
    of Puiseux do
    of gseries do
    of "diff" do
    of "int" do
    of DOM_SET do
    of DOM_LIST do
	return( procname(args()) )
    end_case;
	
    if type(pointt) = "_equal" then
        _x := op(pointt,1);
        lp := op(pointt,2)
    else
	_x := pointt;
	lp := 0
    end_if;

    if not has( ex,_x ) then return( ex ) end_if;


    # reset limit computation #
    sysassign( domattr(stdlib::limit,"result"), FAIL );
    sysassign( Puiseux::iszero, fun((
	subs( normal( args(1) ), ____OMEGA=____w____);
        Puiseux::fiszero(%) or
        iszero(simplify(%,exp)) or iszero(simplify(%,ln))
    )) );

    assume( __LIMITx > 0 );
    assume( ln(__LIMITx) > 0,_and );
    sysassign(sign(__LIMITx), 1);
    sysassign(sign(ln(__LIMITx)),1);

    if args(0) = 2 then
        if lp <> -infinity and lp <> infinity then
            nex := subs(ex,_x=lp+1/__LIMITx);
            nex := (stdlib::limit)::preProcess(
		(stdlib::limit)::simp2explog(nex,__LIMITx),__LIMITx
	    );
	    nex := simplify( nex,exp );
            s := (stdlib::limit)::limit(nex,__LIMITx);
            if s <> FAIL then
                nex := subs(ex,_x=lp-1/__LIMITx);
                nex := (stdlib::limit)::preProcess(
		    (stdlib::limit)::simp2explog(nex,__LIMITx),__LIMITx
		);
	        nex := simplify( nex,exp );
                t := (stdlib::limit)::limit(nex,__LIMITx);
                if t <> FAIL then
		    sysassign( Puiseux::iszero, (stdlib::limit)::Piszero );
		    (stdlib::limit)::reset();
                    if t <> s then return( undefined )
                    else return( s )
                    end_if
                end_if
	    end_if;
	    (stdlib::limit)::reset();
	    s := (stdlib::limit)::result;
	    if domtype(s) = DOM_STRING then return( text2expr(s) )
            else return( hold(limit)(args()) )
	    end_if
        end_if;
        if lp = -infinity then
            ex := subs(ex,_x=-__LIMITx)
	else
	    ex := subs(ex,_x=__LIMITx)
        end_if;
        ex := (stdlib::limit)::preProcess(
	    (stdlib::limit)::simp2explog(ex,__LIMITx),__LIMITx
	);
	ex := simplify( ex,exp );
        s := (stdlib::limit)::limit(ex,__LIMITx);
        if s = FAIL then
	    (stdlib::limit)::reset();
	    s := (stdlib::limit)::result;
	    if domtype(s) = DOM_STRING then return( text2expr(s) )
	    else return( hold(limit)(args()) )
	    end_if
        end_if
    else
        if args(3) = hold(Right) then
	    ex := subs(ex,_x=lp+1/__LIMITx)
        else
	    ex := subs(ex,_x=lp-1/__LIMITx)
        end_if;
        ex := (stdlib::limit)::preProcess(
	    (stdlib::limit)::simp2explog(ex,__LIMITx),__LIMITx
	);
	ex := simplify( ex,exp );
        s := (stdlib::limit)::limit(ex,__LIMITx);
        if s = FAIL then
	    (stdlib::limit)::reset();
	    s := (stdlib::limit)::result;
            if domtype(s) = DOM_STRING then return( text2expr(s) )
            else return( hold(limit)(args()) )
            end_if
        end_if
    end_if;

    sysassign( Puiseux::iszero, (stdlib::limit)::Piszero );
    (stdlib::limit)::reset();
    s
end_proc:

limit := func_env(limit,NIL,table(
    "type"="limit",
    "info"="stdlib::limit -- calculate limits [try ?limit for options]",
    "print"="limit"
)):

proc() local LIMIT, DpreProcess, erfs;
begin

LIMIT := domain("stdlib::limit"):

LIMIT::result := FAIL:  
# This string (if defined) will be returned instead of an
  unevaluated result if (stdlib::limit)::limit returns FAIL.
  A possible message is "oszillating" or "FAIL".
  The string have to be converted to a mupad expression.
#

LIMIT::setuserinfo:= fun((
setuserinfo((stdlib::limit)::limit,3):
setuserinfo((stdlib::limit)::lterm,3):
setuserinfo((stdlib::limit)::getMRV,3):
setuserinfo((stdlib::limit)::maxMRV,3):
setuserinfo((stdlib::limit)::rewrite,3):
)):

LIMIT::reset := fun( stdlib::syseval((
    # clear remember tables #
    domattr(stdlib::limit,"limit") := subsop(
        domattr(stdlib::limit,"limit"),5=NIL
    );
    domattr(stdlib::limit,"lterm") := subsop(
        domattr(stdlib::limit,"lterm"),5=NIL
    );
    domattr(stdlib::limit,"getMRV") := subsop(
        domattr(stdlib::limit,"getMRV"),5=NIL
    );
    domattr(stdlib::limit,"rewrite") := subsop(
        domattr(stdlib::limit,"rewrite"),5=NIL
    );
    domattr(stdlib::limit,"maxMRV") := subsop(
        domattr(stdlib::limit,"maxMRV"),5=NIL
    );
    domattr(stdlib::limit,"sign") := subsop(
        domattr(stdlib::limit,"sign"),5=NIL
    );
    Puiseux::iszero := (stdlib::limit)::Piszero;
)) ):

LIMIT::simp2explog := proc(ex,__LIMITx)
    name (stdlib::limit)::simp2explog;
    local t;
    option remember;
begin
    if domtype(ex) = DOM_EXPR then
        case type(ex)
	of "_index" do return( ex )
        of "_power" do
            if testtype(op(ex,2),NUMERIC) then
        	return( (stdlib::limit)::simp2explog(
		    op(ex,1),__LIMITx)^op(ex,2)
		)
            else
                t := (stdlib::limit)::simp2explog( op(ex,1),__LIMITx );
                if iszero(t) then
                    return( 0 )
                else
                    return( exp((stdlib::limit)::simp2explog(op(ex,2),__LIMITx)*ln(t)) )
                end_if
            end_if;
        of "_mult"   do
        of "_plus"   do
            # return( eval(map(ex,(stdlib::limit)::simp2explog,__LIMITx)) ) #
            return( subs(map(ex,(stdlib::limit)::simp2explog,__LIMITx), Simplified) )
        end_case;
	t := op(ex,0);
	if domtype(level(t,2)) = DOM_FUNC_ENV then
            if nops(ex) = 1 then
		return( level(t,2)(
		    (stdlib::limit)::simp2explog(op(ex,1),__LIMITx)
		))
            else
		return( level(t,2)(op(
		    map([op(ex)],(stdlib::limit)::simp2explog,__LIMITx)
                )))
            end_if
	end_if
    end_if;

    ex
end_proc:

#--
	(stdlib::limit)::limit(ex,x)

	ex: expression
	x : limit variable

	Calls (stdlib::limit)::lterm and analyzes the leading
	coefficient of the asymptotic expansion (in w), i.e.
	returns the limit of ex for x -> infinity, if possible.
	Otherwise FAIL is returned.
--#

LIMIT::limit := proc(ex,__LIMITx)
    name (stdlib::limit)::limit;
    option remember;
    local ld, lc, s, S;
begin
    userinfo(3,"compute limit of ",ex);
    lc := (stdlib::limit)::lterm( ex,__LIMITx );
    if lc = FAIL then
	userinfo(3,"Try series direct");
	s := series( ex,__LIMITx=infinity );
	if has(s,hold(series)) then
	    userinfo(2,"** series computation fails");
	    return( FAIL )
	end_if;
	S:= ldegree(s); # do not use Puiseux::ldegree !
                          Ex.: series(exp(-n*ln(-x)),x=infinity)
                        #
	if S = FAIL then
	    if s::order(s) > 0 then return( 0 ) 
	    else
   	        userinfo(1,"** ORDER seems to be not big enough for series");
                return( FAIL )
	    end_if
        end_if;

	S:= -S; # ldegree returns the degree in terms of the infinitesimal
                  at the expansion point, which is x at x=0 and 1/x at 
                  x=infinity like here.
                  Therefore ldegree(s)=-degree(s) when s is an expansion 
                  at infinity. Example:

             >> series( 1/x,x=infinity);

					/ 1  \
				   1    | -- |
				   - + O|  6 |
				   x    \ x  /

             >> ldegree(%);

					1

	          Therefore: use -ldegree(s) instead of ldegree(s).
	        #
	lc := sign( S );
	case lc
	of  0 do
	    ld := lmonomial(s);
	    if ld = expr(s) then # avoid infinite loop #
		userinfo(2,"** lcoeff(s) = s !? ");
		(stdlib::limit)::reset();
		return( FAIL )
	    elif has(ld,__LIMITx) then
		return( (stdlib::limit)::limit(ld,__LIMITx) )
	    else
		return( ld )
	    end_if
	of -1 do return( 0 )
	of  1 do
	    S := lcoeff(s);
	    lc := sign( S );
	    if type(lc) = "sign" then
		(stdlib::limit)::reset();
		userinfo(1,"** can't determine sign of ",S);
                return( FAIL )
	    else
		return( lc * infinity )
	    end_if
	otherwise
	    (stdlib::limit)::reset();
	    userinfo(1,"** can't determine sign of ",S);
	    return( FAIL )
	end_case
    end_if;

# (d) Analyze the leading degree #

    s := sign(lc[3]);
    if s = 0 then lc[1]
    elif s = 1 then 0
    elif s = -1 then
        s:= (stdlib::limit)::sign(lc[1],__LIMITx);
	if s = FAIL then
	    (stdlib::limit)::reset();
	    FAIL
	else
	    s * infinity
	end_if
    else
        (stdlib::limit)::reset();
        userinfo(1,"** can't determine sign of ",lc[3]);
	FAIL
    end_if;
end_proc:


#--
	(stdlib::limit)::upmove(ex,x)
	(stdlib::limit)::downmove(ex,x)

	Subroutines for (stdlib::limit)::lterm.
--#

LIMIT::upmove := fun(
    # eval(subs(args(1),[ln(args(2))=args(2),args(2)=exp(args(2))])) #
    subs(args(1),[ln(args(2))=args(2),args(2)=exp(args(2))], Simplified)
):
LIMIT::downmove := fun(
    # eval(subs(args(1),[exp(args(2))=args(2),args(2)=ln(args(2))])) #
    subs(args(1),[exp(args(2))=args(2),args(2)=ln(args(2))], Simplified)
):

#--
        (stdlib::limit)::lterm(ex,x)

        ex: expression
        x : limit variable
	S : (optional) mrv set

        Compute the list [c0,w,e0], whereby c0*w^e0 is the leading
	term of an asymptotic expansion of ex for x -> infinity.
	Returns FAIL if this can not be computed.
--#

LIMIT::lterm := proc(ex,__LIMITx)
    name (stdlib::limit)::lterm;
    local ____S____, ____OMEGA, ____s____, ____w____, lc, le, ____t____, ex0;
    option remember;
begin
    ex0:=ex; # used to avoid infinite loops #

    if not has(ex,__LIMITx) then 
	return( [ex,1,0] )
    end_if;

# (a) Find the most rapidly varying subexpressions #

    if args(0) = 2 then
        userinfo(2,"(1)  Find MRV set");
        userinfo(3,"     of ",ex);
        ____S____ := (stdlib::limit)::getMRV(ex,__LIMITx);
        if ____S____ = FAIL then
            userinfo(2,"** determining MRV set fails");
	    return( FAIL )
        elif nops(____S____) = 0 then
	    return( [ex,1,0] )
        elif contains(____S____,undefined) then
            return( [undefined,1,0] )
        end_if;
    else
	____S____ := args(3);
	____t____ := select( ____S____, fun( has(args(2), args(1)) ), ex );
	if ____t____ = {} then 
	    ____S____ := (stdlib::limit)::getMRV(ex,__LIMITx);
	    if ____S____ = FAIL or contains(____S____,undefined) then
		userinfo(2,"** determining MRV set fails");
                return( FAIL )
	    end_if
	end_if
    end_if;

    if ____S____ intersect {__LIMITx} = {__LIMITx} then
    # Upward Movement #
	____w____ := (stdlib::limit)::lterm(
	        (stdlib::limit)::upmove(ex,__LIMITx),__LIMITx,
		(stdlib::limit)::upmove(____S____,__LIMITx)
	);
	if ____w____ = FAIL then return( FAIL )
	else return( (stdlib::limit)::downmove(____w____,__LIMITx) )
	end_if
    end_if;

# (b) Choose one expression in this set and call it w. Rewrite the    #
#     other expressions in that set as g(x)=w^p h(x) and consider all #
#     expressions  independent of w as constants.                     #

    userinfo(2,"(2)  Rewrite MRV set ");
    userinfo(3,"     i.e. rewrite ", ____S____);

    ____s____ := (stdlib::limit)::rewrite( ____S____,__LIMITx );
    if ____s____ = FAIL then
	userinfo(2,"** rewrite process fails");
	return( FAIL )
    end_if;
    userinfo(3,"result is ",____s____);
    ____w____ := op(____s____,1); ____S____ := op(____s____,2);
    # ex := combine(eval(subs(ex,op(____S____,i)=op(____s____,[3,i])
	$ hold(i)=1..nops(____S____),____w____=____OMEGA))); #
    ex := combine(subs(ex,op(____S____,i)=op(____s____,[3,i])
	$ hold(i)=1..nops(____S____),____w____=____OMEGA, Simplified));

# (c) Compute the leading term of the Puiseux series in w around w = 0+ #

    userinfo(2,"(3)  Compute series");
    userinfo(3,"     in ____OMEGA=0+ of ",ex);

    if not has(ex,____OMEGA) then 
	lc := ex; le := 0;
    else
        ____s____ := series(ex,____OMEGA,hold(Right));
        if has(____s____,hold(series)) then
        # does not have a series expansion #
            userinfo(2,"** series computation fails");
	    userinfo(3,"   on expression ",ex );
            return( FAIL )
        end_if;

        userinfo(3,"series is ",____s____);

        if domtype(____s____) = gseries then
	    userinfo(3,"## type is gseries ##");
	    ____t____ := gseries::lterm(____s____);
	    if not has(____t____,____OMEGA) then
		le:= 0;
		lc:= gseries::lmonomial(____s____)
	    elif type(____t____) = "ln" then
	        userinfo(4,"expression has a log-singularity");
	        le := 0;
	        lc := gseries::lmonomial(____s____)
	    elif type(____t____) = "_power" then
	        if op(____t____,1) = ____OMEGA then
		    userinfo(4,"series has real exponents");
		    le := sign(Re(op(____t____,2)));
		    if not contains({1,0,-1},le) then
		        userinfo(1,"** can't determine sign of ",op(____t____,2));
		        return( FAIL )
		    end_if;
		    lc := gseries::lcoeff(____s____)
		else
		    userinfo(2,"can't handle given case")
	        end_if
	    else
		lc:= gseries::lmonomial(____s____);
		le:= 0
	    end_if
        else
	    le := Puiseux::ldegree(____s____);
            lc := Puiseux::lcoeff(____s____)
        end_if;

        userinfo(3,"lcoeff is",lc, "; ldegree is ",le);
        if lc = FAIL then
	    if ____s____::order(____s____) > 0 then
		# the limit is zero #
	        return( [0,____w____,1] )
	    else
	        userinfo(1,"** ORDER seems to be not big enough for series");
	        sysassign( domattr(stdlib::limit,"result"), "FAIL" );
	        return( FAIL )
	    end_if
        end_if;

	if has( lc,____OMEGA ) then
	    #
            lc := eval(subs(lc,
	        [ln(____OMEGA)=simplify(ln(____w____),ln),____OMEGA=____w____]
            ))
	    #
            lc := subs(lc,
	        [ln(____OMEGA)=simplify(ln(____w____),ln),____OMEGA=____w____]
		, Simplified)
	end_if
    end_if;

    if le = 0 then
	if lc = ex0 then
	    userinfo(2,"entered in infinite loop!");
            FAIL # to avoid infinite loops #
        else
	    (stdlib::limit)::lterm(simplify(lc),__LIMITx)
	end_if
    else [lc,____w____,le]
    end_if
end_proc:

#--
        (stdlib::limit)::getMRV(f,x)

        f: expression
        x: limit variable

        Find the most rapidly varying subexpressions at x=infinity
        (section 3.1).
--#

LIMIT::getMRV := proc(f,__LIMITx)
    name (stdlib::limit)::getMRV;
    local s, t, g;
    option remember;
begin
    if not has(f,__LIMITx) then
        return( {} )
    elif f = __LIMITx then
        return( {__LIMITx} )
    else
        case type(f)
	of "_mult" do
        of "_plus" do
	    t := [];
	    for g in f do
		s := (stdlib::limit)::getMRV(g,__LIMITx);
		if s = FAIL or contains(s,undefined) then return( FAIL )
		else t := t.[s]
		end_if
	    end_for;
            return( (stdlib::limit)::maxMRV({op(t)},__LIMITx) )
        of "_power" do
	    if not has(op(f,2),__LIMITx) then
                return( (stdlib::limit)::getMRV(op(f,1),__LIMITx) )
 	    else		
		return( (stdlib::limit)::getMRV(
		    exp(op(f,2)*ln(op(f,1))),__LIMITx)
		)
            end_if
 	of "ln" do
	    if f = ln(__LIMITx) then return( {__LIMITx} ) 
	    else 
                s:= (stdlib::limit)::getMRV(op(f,1),__LIMITx);
                if s = FAIL or contains(s,undefined) then
                    return( FAIL )
                else
                    return( s )
                end_if
	    end_if
	of "exp" do
            if f = exp(__LIMITx) then return( {f} ) end_if;
            t := op(f,1);
            s := (stdlib::limit)::limit(t,__LIMITx);
	    if s = FAIL or s = undefined then return( FAIL )
            elif s = infinity or s = -infinity then
		# if the limit of t is real, then exp(t)>=0 #
		if is(s,Type::RealNum)=TRUE then assume(f>0,_and) end_if;
                s := (stdlib::limit)::getMRV(t,__LIMITx);
	 	if s = FAIL then return( FAIL )
                elif contains( s,FAIL ) then return( FAIL )
                else
		    return((stdlib::limit)::maxMRV({{exp(t)},s},__LIMITx))
                end_if
            elif not has(s,infinity) then
		if is(s,Type::RealNum)=TRUE then assume(f>0,_and) end_if;
                return( (stdlib::limit)::getMRV(t,__LIMITx) )
	    else
		return( FAIL )
            end_if
	of "sin"  do
	of "cos"  do
	    t := (stdlib::limit)::limit(op(f,1),__LIMITx);
	    if t = FAIL or t = undefined then return( FAIL )
            elif t <> infinity and t <> -infinity then
                return( (stdlib::limit)::getMRV(op(f,1),__LIMITx) )
            else # oscillating function #
                return( {undefined} )
            end_if
	of "tan"	do
	of "asin"	do
	of "acos"	do
	of "cot"	do
	of "fact"	do
	of "gamma"	do
	    t := (stdlib::limit)::limit(op(f,1),__LIMITx);
	    if t = FAIL or t = undefined then return( FAIL )
	    elif t <> infinity and t <> -infinity then
		return( (stdlib::limit)::getMRV(op(f,1),__LIMITx) )
	    else # essential singularity #
		return( FAIL )
	    end_if
        of "atan" do
            if has( {op(f)},__LIMITx ) then
                return( FAIL )
            else
                return( {} )
            end_if
        of "asinh" do
        of "psi" do
	of "acosh" do
	    t := (stdlib::limit)::limit(op(f,1),__LIMITx);
	    if t = FAIL or t = undefined then 
		return( FAIL )
	    else
		s := sign(t);
	        if s = 1 or s = 0 then
                    return( (stdlib::limit)::getMRV(op(f,1),__LIMITx) )
		else
		    userinfo( 1,"** ".type(f).": can't handle point ",t);
		    return( FAIL )
		end_if
	    end_if
	of "binomial" do
	    return( (stdlib::limit)::getMRV(expand(f),__LIMITx) )
	of "eint" do
	    return( (stdlib::limit)::getMRV(op(f,1),__LIMITx) )
	of "besselJ" do # besselJ(n,x) #
	    t := (stdlib::limit)::limit(op(f,2),__LIMITx); 
	    if t = -infinity or t = FAIL or t = undefined then # essential singularity #
		return( FAIL )
	    elif t = infinity or not has(t,infinity) then
		return( (stdlib::limit)::getMRV(op(f,2),__LIMITx) )
	    else # complex*infinity or sign(..)*infinity #
		return( FAIL )
	    end_if
	of "Si" do
	    t := (stdlib::limit)::limit(op(f,1),__LIMITx);
            if t = -infinity or t = FAIL or t = undefined then # essential singularity #
                return( FAIL )
            elif t = infinity or not has(t,infinity) then
                return( (stdlib::limit)::getMRV(op(f,1),__LIMITx) )
	    else # complex*infinity or sign(..)*infinity #
                return( FAIL )
            end_if
	# functions which were treated by the preprocessing step #
	of "erf" do
	of "limit::erfs" do
	of "gamma" do
	of "limit::gammas" do
	    return( (stdlib::limit)::getMRV(op(f,1),__LIMITx) )
        end_case
    end_if;

    FAIL
end_proc:

#--
        (stdlib::limit)::rewrite(S,x)

        S: mrv set of f in respect to x
        x: limit variable

        Returns [w,S,l] where w represents ____OMEGA and l is 
        the list of elements of S rewritten in ____OMEGA.
        The i.th element of l is the i.th element of S rewritten
        in ____OMEGA (i=1..|S|).

	Returns FAIL if the rewrite process can not be performed.
--#

LIMIT::rewrite := proc(S,__LIMITx)
    name (stdlib::limit)::rewrite;
    local _S_, t, s, i, w, INVw, f, c, l, ____OMEGA;
    option remember;
begin
#
  extract the set _S_ of expressions which contains subexpressions
  of the same comparability class
#
    _S_ := {};
    for i from 1 to nops(S) do
        s := op(S,i);
        w := select( S minus {s}, has, s );
        if w <> {} then
            _S_ := _S_ union w
        end_if
    end_for;
    S := S minus _S_;  # the complement of _S_ #

    w := op(sort( [op(S)],fun(
           (if length(args(1)) < length(args(2)) then TRUE
           else FALSE end_if)
         )),1);
    if type(w) <> "exp" then
        error("illegal element in mrv set: ".expr2text(w))
    end_if;

    S := (sort([op(_S_)],fun(
          (if has(args(1),args(2)) then TRUE else FALSE end_if)
         ))).[op(S minus {w})];

    s := (stdlib::limit)::sign(op(w),__LIMITx);
    if s = 1 then t := -op(w); w := 1/w
    elif s = -1 then t := op(w);
    else
        error("w = ".expr2text(w)." has illegal sign: ".expr2text(s))
    end_if;
    INVw := exp(-t);

    userinfo(3,"Selected ____OMEGA of ",S, "is ",w);

    l := [];
    for f in S do
	s := simplify(ln(f),ln);
	c := (stdlib::limit)::lterm( s/t,__LIMITx );
	if c = FAIL then
	    userinfo(2,"** unable to compute the leading term");
	    return( FAIL )
	elif c[3] <> 0 then
	    userinfo(2,"** elements are not in the same comp. class !");
	    return( FAIL )
	else
            l := append( l,exp(s-c[1]*t)*____OMEGA^c[1] );
	end_if
    end_for;

    [w,S.[INVw],l.[1/____OMEGA]]  # append 1/w to S #
end_proc:

#-
        (stdlib::limit)::maxMRV(s,x)

        s: set containing sets of rapidly varying terms in x
        x: limit variable

        Find the more rapidly varying set (section 3.2).
--#

LIMIT::maxMRV := proc(l,__LIMITx)
    name (stdlib::limit)::maxMRV;
    local f, g, s;
    option remember;
begin
    l := l minus {{}};
    if nops(l) < 2 then return( op(l) )
    elif nops(l) > 2 then
        f := op(l,1);
        return( (stdlib::limit)::maxMRV(
           {f,(stdlib::limit)::maxMRV(l minus {f},__LIMITx)}
        ,__LIMITx) )
    else
        f := op(op(l,1),1); g := op(op(l,2),1); # choose representatives #
        if type(f)="exp" then assume(op(f),Type::RealNum,_and) end_if;
        if type(g)="exp" then assume(op(g),Type::RealNum,_and) end_if;
        s := (stdlib::limit)::limit( simplify(ln(f)/ln(g),ln),__LIMITx );
        if s = FAIL or s = undefined then return( FAIL )
        elif s = infinity or s = -infinity then return( op(l,1) )
        elif iszero(s) then return( op(l,2) )
        else return( op(l,1) union op(l,2) )
        end_if
    end_if
end_proc:

#--
        (stdlib::limit)::sign(ex,x)

        ex: expression in x
        x : ident

        Compute the sign of ex, assuming that x goes to infinity.
	Returns FAIL if this can not be determined.
--#

LIMIT::sign := proc(ex,__LIMITx)
    name (stdlib::limit)::sign;
    local s, t, S;
    option remember;
begin
    userinfo(3,"compute the sign of ",ex);

    if ex = __LIMITx then return( 1 ) end_if;
    if not has(ex,__LIMITx) then
	s := sign(ex);
	if not has(s,sign) then
	    return( s )
	else
	    userinfo(1,"** can't determine sign of ",ex);
	    return( FAIL )
	end_if
    end_if;

    case type(ex)
    of "_mult"  do
	S := 1;
	for t in ex do
	    s:= (stdlib::limit)::sign(t,__LIMITx);
	    if s = FAIL then 
	        userinfo(1,"** can't determine sign of ",t);
		return( FAIL )
	    else
		S:= S * s;
	    end_if
	end_for;
	return( S )
    of "_power" do
	if domtype(op(ex,2)) = DOM_INT and modp(op(ex,2),2) = 0 then 
	    return( 1 )
	elif op(ex,1) = __LIMITx then
	    return( 1 )
        elif (stdlib::limit)::sign(op(ex,1),__LIMITx) = 1 then
	    return( 1 )
	else
	    t := (stdlib::limit)::lterm( ex, __LIMITx );
	    if t = FAIL then
                userinfo(1,"** can't determine sign of ",ex);
                return( FAIL )
	    else
		return( (stdlib::limit)::sign( t[1], __LIMITx ) )
	    end_if
	end_if
    of "_plus"do
	t := (stdlib::limit)::lterm(ex,__LIMITx);
	if t = FAIL then
	    userinfo(1,"** can't determine sign of ",ex);
	    return( FAIL )
	else
	    return( (stdlib::limit)::sign(t[1],__LIMITx) )
	end_if
    of "exp" do 
	return( 1 )
    otherwise
	t := (stdlib::limit)::lterm( ex, __LIMITx );
	if t = FAIL then
	    userinfo(1,"** can't determine sign of ",ex);
	    return( FAIL )
	else
	    (stdlib::limit)::sign( t[1],__LIMITx )
	end_if
    end_case
end_proc:

#--
        (stdlib::limit)::subsFUNC(ex,f1,f2)

        ex: expression
        f1: string (type of function)
	f2: function

	Substitutes all function calls f(x) of type f1 to 
	function calls f2(x).
--#

LIMIT::subsFUNC := proc(ex,f1,f2) 
    name (stdlib::limit)::subsFUNC;
    option remember;
begin
    case domtype(ex) 
    of Puiseux do
	# eval(map(ex,(stdlib::limit)::subsFUNC,f1,f2)); #
	subs(map(ex,(stdlib::limit)::subsFUNC,f1,f2), Simplified);
	break
    of DOM_EXPR do
	if type(ex) = f1 then ex := f2(op(ex)) end_if;
	# eval(map(ex,(stdlib::limit)::subsFUNC,f1,f2)); #
	subs(map(ex,(stdlib::limit)::subsFUNC,f1,f2), Simplified);
	break
    otherwise
	ex
    end_case
end_proc:
	
#--
        (stdlib::limit)::preProcess(ex,x)

        ex : expression
        x  : limit variable

--#

# note: 'hold' is requiered here; otherwise the pre-processing step
        only works for the *object* and not for its name. Thus, if
        for instance, the remember table of the object was changed
        afterwards, the process never works again!
#
LIMIT::setPreProcessFunc:= [
    hold(abs),hold(sign),hold(erf),hold(gamma),hold(atan),
    hold(sinh),hold(cosh),hold(tanh),hold(coth)
]:

LIMIT::preProcess := proc(ex,__LIMITx)
    name (stdlib::limit)::preProcess;
    local p;
begin
    if not has(ex,(stdlib::limit)::setPreProcessFunc) then
	ex
    elif domtype(ex) = DOM_EXPR then
	if (p:= contains((stdlib::limit)::setPreProcessFunc,op(ex,0))) > 0 and 
	    has(ex,__LIMITx) 
	then
	    domattr((stdlib::limit)::DpreProcess,expr2text( (stdlib::limit)::setPreProcessFunc[p] ))(
		op(map([op(ex)],(stdlib::limit)::preProcess,__LIMITx)), __LIMITx
	    )
	else
	    map(ex,(stdlib::limit)::preProcess,__LIMITx )
	end_if
    else
	ex
    end_if
end_proc:

#--
        Define the pre-processing algorithms for the
	smooth functions:

	(-) abs(x)  = x/sign(x)

                       /  1  if sign(x) -> 1
	(-) sign(x) = <
                       \  -1 if sign(x) -> -1
	
        (-) erf(x) = 1-exp(-x^2)*limit::erfs(x)  for x->infty
            erf(x) = -1+exp(-x^2)*limit::erfs(-x)  for x->-infty

	(-) gamma(x) = exp( gammas(x) )  for x->infty

	The algorithms are defined in the domain

	    (stdlib::limit)::preProcess

	The methods are called by the type of the regarded function.
	Two pre-pocessing procedures must be defined, one named by the 
	type of the original function, the second by the type of the 
	corresponding smooth function (for example "erf" and 
	"limit::erfs").
--#

LIMIT::DpreProcess := domain("LIMIT::DpreProcess"):
DpreProcess := LIMIT::DpreProcess:

DpreProcess::erf := proc(ex,x)
    local s;
begin
    s := limit(ex,x=infinity);
    if s = FAIL then FAIL
    elif s = infinity then
	1-exp(-ex^2)*domattr(stdlib::limit,"erfs")(ex)
    elif s = -infinity then
        -1+exp(-ex^2)*domattr(stdlib::limit,"erfs")(-ex)
    else 
	hold(erf)(ex)
    end_if
end_proc:
domattr(DpreProcess,"limit::erfs"):= proc(ex,x)
    local s;
begin
    s := limit(ex,x=infinity);
    if s = FAIL then FAIL
    elif s <> infinity and s <> -infinity then
	s := sign(s);
	if s = 1 then (1-erf(ex))*exp(ex^2)
	elif s = -1 then (1+erf(-ex))*exp(ex^2)
	else FAIL
	end_if
    else
	subsop((stdlib::limit,"erfs")(hold(X)),1=ex)
    end_if
end_proc:

DpreProcess::gamma:= proc(ex,x)
    local s;
begin
    s := limit(ex,x=infinity);
    if s = FAIL then FAIL
    elif s = infinity then
        exp(domattr(stdlib::limit,"gammas")(ex))
    else
        hold(gamma)(ex)
    end_if
end_proc:
domattr(DpreProcess,"limit::gammas"):= proc(ex,x)
    local s;
begin
    s := limit(ex,x=infinity);
    if s = FAIL then 
	FAIL
    elif s <> infinity and s <> -infinity then
        if sign(s) = 1 then ln(gamma(ex))
        else FAIL
        end_if
    else
        subsop((stdlib::limit,"gammas")(hold(X)),1=ex)
    end_if
end_proc:

DpreProcess::sign:= proc(ex,x)
    local t;
begin
    t:= (stdlib::limit)::sign(ex,x);
    if t = 1 then 1 
    elif t = -1 then -1
    else hold(sign)(ex)
    end_if
end_proc:

DpreProcess::abs:= fun( 
    args(1)/domattr((stdlib::limit)::DpreProcess,"sign")(args(1),args(2))
):

DpreProcess::atan:= proc(ex,x)
    local t;
begin
    if args(0) = 3 then
         # two-argument arcus tangens #
         ex:= x/ex; x:= args(3)
    end_if;
    t:= limit(ex,x=infinity);
    case (t:= limit(ex,x=infinity)) 
    of -infinity do
        return( -PI/2 )
    of infinity do
        return( PI/2 )
    of FAIL do
        return( FAIL )
    otherwise
        return( atan(t) )
    end_case
end_proc:

DpreProcess::sinh:= proc(x) begin (exp(x)-exp(-x))/2 end_proc:
DpreProcess::cosh:= proc(x) begin (exp(x)+exp(-x))/2 end_proc:
DpreProcess::tanh:= proc(x) begin (exp(x)-exp(-x))/(exp(x)+exp(-x)) end_proc:
DpreProcess::coth:= proc(x) begin (exp(x)+exp(-x))/(exp(x)-exp(-x)) end_proc:

stdlib::limit := LIMIT:
loadlib("Series"):
Puiseux::create:
domattr(stdlib::limit,"Piszero"):= Puiseux::iszero:


#--

        The function limit::erfs(x).

--#
domattr(stdlib::limit,"erfs"):=
    proc() begin subsop(hold(x)(hold(x)),0=level(procname,2),1=args()) end_proc:

domattr(stdlib::limit,"erfs"):= funcattr(domattr(stdlib::limit,"erfs"), "print", "(stdlib::limit)::erfs"):

domattr(stdlib::limit,"erfs"):= funcattr(domattr(stdlib::limit,"erfs"), "series", 
proc(f,x,n)
    local t, s, k;
begin
    t := Series::series(f,x,n);
    if not testtype(t,Type::Series(Puiseux)) then
	error("unable to compute series expansion")
    end_if;
    k := ldegree(t);
    if k = 0 then
	t := expr(t);
        s := Series::series((1-erf(t))*exp(t^2),x,n);
	(stdlib::limit)::subsFUNC(s,"erf",
	    fun(1-exp(-args(1)^2)*domattr(stdlib::limit,"erfs")(args(1))
	))
    elif k < 0 then # f goes to +/- infinity #
	if sign(lcoeff(t)) = 1 then # f goes to +infinity #
            s:= Puiseux::create(1,1,n,
              [(1/sqrt(PI)*(-1/4)^s*fact(2*s)/fact(s),0) $ s=0..iquo(n-1,2)]
            );
            Puiseux::_fconcat(Series::series(1/f,x,n),s)
	end_if
   else
        Series::unknown(domattr(stdlib::limit,"erfs")(f),x,n)
   end_if
end_proc):

domattr(stdlib::limit,"erfs"):= 
   funcattr(domattr(stdlib::limit,"erfs"), "type", "limit::erfs"):

domattr(stdlib::limit,"erfs"):= funcattr(domattr(stdlib::limit,"erfs"), "diff", 
proc(ex,x)
begin
    case args(0)
    of 1 do ex; break
    of 2 do
        ex:= op(ex);
        if has(ex,x) then
            (-2/sqrt(PI)+2*domattr(stdlib::limit,"erfs")(ex)*ex)*diff(ex,x)
        else 
	    0
        end_if;
	break
    otherwise
        diff( diff(ex,x),args(3..args(0)) )
    end_case
end_proc):

domattr(stdlib::limit,"erfs"):= funcattr(domattr(stdlib::limit,"erfs"), "(stdlib::limit)::sign", 
proc(ex,x)
    local s;
begin
    s:= (stdlib::limit)::limit(ex,x);
    if s = FAIL or s = undefined then FAIL
    else
	s:= sign(s);
	if s = 0 or s = 1 then 1 else FAIL end_if
    end_if
end_proc):

#--

        The function limit::gammas(x).

--#

domattr(stdlib::limit,"gammas"):=
    proc() begin subsop(hold(x)(hold(x)),0=level(procname,2),1=args()) end_proc:

domattr(stdlib::limit,"gammas"):= funcattr(domattr(stdlib::limit,"gammas"), "print", "(stdlib::limit)::gammas"):

domattr(stdlib::limit,"gammas"):= funcattr(domattr(stdlib::limit,"gammas"), "series",
proc(f,x,n)
    local t, s, k;
begin
    t:= Series::series(f,x,n);
    if not testtype(t,Type::Series(Puiseux)) then
        error("unable to compute series expansion")
    end_if;
    k := ldegree(t);
    if k = 0 then
        t := expr(t);
        s := Series::series(ln(gamma(t)),x,n);
        (stdlib::limit)::subsFUNC(s,"gamma",
            fun(exp((stdlib::limit)::gammas(args(1))))
        )
    elif k < 0 then # f goes to +/- infinity #
	s:= Puiseux::create(1,-1,13,
	    [(-ln(x)-1),1/2*ln(x)+1/2*ln(2*PI),1/12,0,-1/360,0,1/1260,0,-1/1680, 
	    0,1/1188,0,-691/360360],x
	);
	Series::series( subs(expr(s),x=1/expr(t)),x,n )
   else
        Series::unknown(domattr(stdlib::limit,"gammas")(f),x,n)
   end_if
end_proc):

domattr(stdlib::limit,"gammas"):=
   funcattr(domattr(stdlib::limit,"gammas"), "type", "limit::gammas"):

domattr(stdlib::limit,"gammas"):= funcattr(domattr(stdlib::limit,"gammas"), "diff",
proc(ex,x)
begin
    case args(0)
    of 1 do ex; break
    of 2 do
        ex:= op(ex);
        if has(ex,x) then
	    phi(ex)*diff(ex,x)
        else
            0
        end_if;
        break
    otherwise
        diff( diff(ex,x),args(3..args(0)) )
    end_case
end_proc):

domattr(stdlib::limit,"gammas"):= funcattr(domattr(stdlib::limit,"gammas"), "(stdlib::limit)::sign"
,
proc(ex,x)
    local s;
begin
    s:= (stdlib::limit)::limit(ex,x);
    if s = FAIL or s = undefined then FAIL
    elif sign(s) = 1 then 1 
    else FAIL 
    end_if
end_proc):

end_proc():

# end of file #
