# #
# $Date: 1999/09/06 16:35:46 $ $Author: frankp $ $Revision: 1.20.2.3 $ #
# #
# frankp, zimmerma #

#++
        linsolve  --  solve linear equations

	linsolve(sys,unk) solves the set sys of linear equations with 
	respect to the unknowns in unk. The output is either FAIL if 
	there is no solution, or a list of (one) solution, which may 
	contains free unknowns (possible empty, meaning that every
	unknown is free).

	Note: This function uses linalg::linearSolve.

	Examples:

	linsolve({x+y=1,2*x+2*y=3},{x,y})  ->  FAIL
	linsolve({x+y=1,2*x+y=3},{x,y})    ->  {y = -1, x = 2}
	linsolve({x+y+z=1,2*x+y=3},{x,y})  ->  {x = z + 2, y = - 2 z - 1}
	linsolve({0},{x,y})		   ->  {}
        linsolve({x=x+1},{x})		   ->  FAIL
        linsolve({y=0},{x,y});		   ->  {y = 0}
++#

linsolve := proc(sys,unk)
   local sol, i, normalForm, R, o_mult, o_divide, o_iszero;
begin
   if testargs() then
	if args(0) < 1 or args(0) > 3 then
	    error("wrong no of args")
	end_if;
	if not contains( {DOM_SET,DOM_LIST},domtype(sys) ) then
	    error("expecting set or list of equations")
	end_if;
	if args(0) > 1 then
	    if not contains( {DOM_SET,DOM_LIST},domtype(unk) ) then
		error("expecting set of unknowns")
	    end_if;
	    if args(0) = 3 then
		if (args(3))::hasProp( Cat::Field ) <> TRUE then
		    error("coefficient domain must be of Cat::Field")
		end_if
	    end_if
	end_if
   end_if;

   if args(0)=1 then 
       unk:=map( sys,proc(s) begin
           if type(s)="_equal" then indets(op(s,1)-op(s,2),PolyExpr) else indets(s,PolyExpr) end_if
       end_proc );
       unk:= _union( op(unk) );
   else # remove indets not appearing in the system #
       for i in unk do if not has(sys,i) then unk:=unk minus {i} end_if end_for
   end_if;

   if unk={} then if sys={0} then return({}) else return(FAIL) end_if end_if;

	normalForm:= proc( sys,vars )
	    local d;
	begin
	    if type(sys) = "_equal" then
		return( map(sys,normalForm,vars) )
	    end_if;
	    sys:= normal( sys );
	    if has((d:= denom(sys)),vars) then numer(sys)*d
	    else sys
	    end_if
	end_proc:

   userinfo(1,"create the coefficient matrix");
   if args(0) = 3 then
       sol:= linalg::expr2Matrix( sys,unk,args(3),hold(Append));
       userinfo(1,"call linalg::linearSolve");
       sol:= linalg::linearSolve( sol,unk )
   else
       # use a "special" instance of ExpressionField but keep the original version, 
         because changes of the domain influence _each_ instance! 
       #
       R:= Dom::ExpressionField();
       o_mult:= R::_mult; o_divide:= R::_divide; o_iszero:= R::iszero:

       R::_mult:= fun(normal(_mult(args())));
       R::_divide:= fun(normal(args(1)/args(2)));
       R::iszero:= fun(iszero(normal(numer(args(1)))));

       userinfo(1,"call linalg::linearSolve");
       if traperror((sol:= linalg::expr2Matrix( sys,unk,R,hold(Append)))) <> 0 
          or traperror((sol:= linalg::linearSolve( sol,unk ))) <> 0
       then
           sol:= "error"
       end_if;

       R::_mult:= o_mult;
       R::_divide:= o_divide;
       R::iszero:= o_iszero;
       if sol = "error" then error("internal error") end_if
   end_if;

   if sol = [] then return( FAIL ) end_if; # no solution #
   sys:= {};
   for i from 1 to nops(unk) do
      if expr(sol[i]) <> op(unk,i) then
         sys:= sys union {op(unk,i)=expr(sol[i])}
      end_if
   end_for;

   sys
end_proc:

# end of file #
