# $Date: 1996/09/03 15:26:12 $ $Author: kg $ $Revision: 1.7 $ #

# kg, 12/04/95 #

#++
maprec -- recursively maps operands of expression

maprec(e [, q=f...] [, PreMap] [, PostMap])

e       - expression to map
q       - qualifier (set of types or predicate function)
f       - matching function
PreMap  - option
PostMap - option

maprec recursively maps matching subexpressions of e. For each subexpression
s of e it is tested wether one of the matching equations apply. The first
matching equation defines the new value f(s) of the subexpression, which is
inserted into e instead of s. If no matching equation applies then s is not
altered.

Qualifiers used in matching equations may have one of two forms:

- {t1,...,tn} matches any subexpression s of type t1,...,tn.

- A predicate function p matches any subexpression s for which p(s) returns
  TRUE (p must return a boolean value).

If the option PreMap is given, then for each subexpression s of e first the
operands of s are mapped recursively; only after mapping the operands it ist
tested wether a matching equations applies for the result.

If the option PostMap is given, then for each subexpression s of e it is
tested wether any of the matching equations applies. If no match
applies then the operands of s are mapped recursively.

Both options may be given. If no option is given then only PreMap is used.

Important notes:

- Only operands of expressions (i.e. DOM_EXPRs), sets, lists, arrays and
  tables are mapped recusively. Operands of other types are not mapped in
  order to avoid unwanted substitutions. If one wants to map the operands of
  other types of subexpressions one has to do that explicitly by adding
  matching equations, like for example {DOM_PROC}=handle_procs.

- The operators of expressions (op 0) are also mapped recursively like the
  other operands.

One may overload maprec: If the domain of a subexpression has the method
"maprec" then that method is called with the subexpression and the other
arguments of the call. The result replaces the subexpression (no pre- or
post-mapping of operands is applied).

If one wants to stop the mapping then one should call breakmap(). After
calling this no further mappings are applied.
++#

maprec:= proc()
    local _0_bm, _0_pre, _0_post, _0_mapeq;
begin
    case args(0)
    of 0 do error("no args");
    of 1 do return(args(1));
    end_case;
    
    _0_mapeq:= [args(2..args(0))];
    _0_pre:= contains(_0_mapeq, hold(PreMap));
    if _0_pre = 0 then
	_0_pre:= FALSE
    else
	_0_mapeq[_0_pre]:= NIL; _0_pre:= TRUE
    end_if;
    _0_post:= contains(_0_mapeq, hold(PostMap));
    if _0_post = 0 then
	_0_post:= FALSE
    else
	_0_mapeq[_0_post]:= NIL; _0_post:= TRUE
    end_if;
    if not _0_pre and not _0_post then _0_pre:= TRUE end_if;

    if _0_mapeq = [] then return(args(1)) end_if;

    if testargs() then
	if {op(map(_0_mapeq, type))} <> {"_equal"} then
	    error("illegal equation")
	end_if
    end_if;
    
    _0_bm:= stdlib::do_breakmap;
    sysassign(stdlib::do_breakmap, FALSE);
    stdlib::maprec(args(1), _0_mapeq, _0_pre, _0_post);
    sysassign(stdlib::do_breakmap, _0_bm);
    %2
end_proc:

# stdlib::maprec does maprec's work #
stdlib::maprec:= proc(_0_ex)
    local _0_t, _0_eq, _0_f;
begin
    if stdlib::do_breakmap then return(_0_ex) end_if;
    _0_t:= domtype(_0_ex);
    
    if _0_t::maprec <> FAIL then 
        return(_0_t::maprec(_0_ex, op(args(2)),
        		(if args(3) then hold(PreMap) else null() end_if),
        		(if args(4) then hold(PostMap) else null() end_if)))
    end_if;

    if args(3) then
        if contains(stdlib::DomainsMapOp, _0_t) then
	    if _0_t = DOM_EXPR then
		_0_f:= stdlib::maprec(op(_0_ex,0), args(2..4));
		if _0_f = op(_0_ex,0) then
		    _0_ex:= map(_0_ex, stdlib::maprec, args(2..4))
		else
		    _0_ex:= _0_f(op(map([op(_0_ex)], stdlib::maprec, args(2..4))))
		end_if
            else
		_0_ex:= map(_0_ex, stdlib::maprec, args(2..4))
            end_if;
            if stdlib::do_breakmap then return(_0_ex) end_if
        end_if
    end_if;

    for _0_eq in args(2) do
        if domtype(op(_0_eq,1)) = DOM_SET then
            if contains(op(_0_eq,1), type(_0_ex)) then
                return(op(_0_eq,2)(_0_ex));
            end_if
        elif op(_0_eq,1)(_0_ex) = TRUE then
            return(op(_0_eq,2)(_0_ex))
        end_if;
    end_for;
    
    if args(4) then
        _0_t:= domtype(_0_ex);
        if contains(stdlib::DomainsMapOp, _0_t) then
	    if _0_t = DOM_EXPR then
		_0_f:= stdlib::maprec(op(_0_ex,0), args(2..4));
		if _0_f = op(_0_ex,0) then
		    _0_ex:= map(_0_ex, stdlib::maprec, args(2..4))
		else
		    _0_ex:= _0_f(op(map([op(_0_ex)], stdlib::maprec, args(2..4))))
		end_if
            else
		_0_ex:= map(_0_ex, stdlib::maprec, args(2..4))
            end_if
        end_if
    end_if;
    _0_ex
end_proc:

# domain types which map their operands #
stdlib::DomainsMapOp:= {
    DOM_ARRAY, DOM_SET, DOM_LIST, DOM_TABLE, DOM_EXPR
}:


#++
breakmap -- stop mapping with maprec

breakmap()

-no args-
++#

breakmap:= fun((
    if args(0) <> 0 then error("no args allowed") end_if;
    sysassign(stdlib::do_breakmap, TRUE)
)):

# end of file #
