# $Date: 1999/05/26 10:19:40 $  $Author: olli $  $Revision: 1.4.2.6 $ #

#
|                                                             
| Library-Routine plot2d().
|                                                             
|
|                                                             
| Diese Datei enth"alt die Library-Routinen plot2d(),
| die notwendig sind, um die verschiedenen im Hand- 
| buch aufgelisteten Grafikbefehle zu vereinbaren.            
| Die Argumente dieser Befehle werden so ge"andert,
| dass die Kernroutine plot() die richtigen Werte an
| das Grafiktool liefert.                                                    
|                                                             
|
|                                                             
| Autor: Thorsten Schulze                                     
|                                                             
| letzte "Anderung : 22.5.99                                  
|                                                             
#





plot2d := proc()
#  
|  Dieses ist die Library-Routine, die einen plot2d-Befehl in
|  einen plot-Befehl umwandelt.
|  Bei der Umwandlung ist darauf zu achten, dass einige der
|  2D-Optionen sich von den 3D-Optionen unterscheiden. In die-
|  sen F"allen muss die 2D-Option in eine 3D-Option umgewandelt
|  werden. Dabei werden aus dieser Library-Routine keine Fehler-
|  meldungen ausgegeben. Diese werden sowieso im plot-Befehl 
|  erzeugt. Ferner wird nicht "uberpr"uft, ob Optionen richtig 
|  eingegeben wurden. Weiterhin wird auch nicht getestet, ob die
|  Optionen mehrfach spezifiziert werden.
|  Im einzelnen sind das die zu "andernden Optionen die folgenden :
|      
|    ViewingBox : Hier wird der Benutzer im 2D-Fall vier Werte
|                 eingeben. Der plot-Befehl hingegen verlangt
|                 sechs Werte. Die fehlenden zwei Werte werden
|                 erg"anzt, indem f"ur die z-Werte der ViewingBox 
|                 die Sequenz 0,0 an die y-Werte der ViewingBox 
|                 angeh"angt werden. Dieses passiert auch, wenn 
|                 der Benutzer mehr als vier Werte eingegeben hat.
|
|    Labels     : Im 2D-Fall gibt der Benutzer im allgemeinen
|                 zwei Zeichenketten als Beschriftungen f"ur die
|                 x- und y-Achse ein. Im Gegensatz dazu verlangt
|                 die zugeh"orige 3D-Option drei Zeichenketten.
|                 Hat der Benutzer also die Option Labels verwen-
|                 det, so wird eine leere Zeichenkette an die
|                 zweite Zeichenkette dieser Option angeh"angt.
|                 Dieses geschieht auch, wenn der Benutzer mehr
|                 als zwei Zeichenketten vereinbart hat. Die
|                 Fehlermeldung wird dann im plot-Befehl erzeugt.
|
|    Funktionen : Im 2D-Fall werden im allgemeinen zwei Funktionen
|                 angegeben. Im 3D-Fall hingegen ben"otigen wir
|                 drei Funktionen (Parametrisierungen). Dieses 
|                 wird erreicht, indem an die zweite Funktion die
|                 Funktion 0 angeh"angt wird.
|                 Dieses geschieht auch, wenn mehr als zwei Funkti-
|                 onen spezifiziert wurden. Die Fehlermeldung wird
|                 dann im plot-Befehl ausgegeben.
|
|  Zus"atzlich m"ussen wir daf"ur sorgen, dass die richtige Per-
|  spektive erzeugt wird. Dazu wird im plot-Befehl die folgende
|  Option aufgerufen:
|
|     CameraPoint = [0, -0.00001, 1]
|
|  Ferner muss dann noch die Dimension der Szene mit "ubergeben 
|  werden. Dieses erreichen wir durch die Option Dimension = 1,
|  welche im plot-Befehl die Dimension der Szene setzt.
|
#


local i, arg, new_args, _s_, DIGITS; # In arg werden die Argumente des Aufrufes
                                     | gespeichert und die zu "andernden Optionen
                                     | des plot-Befehls werden ver"andert. Das 
                                     | steht dann in der Sequenz s. DIGITS dient
                                     | dazu, das globale DIGITS umzusetzen.
                                     #

begin
  DIGITS := 10:
  #
  | Falls keine Argumente "ubergeben wurden, so wird
  | auch plot3d ohne Argumente aufgerufen.
  #

  if args(0) = 0 
  then
    plot(Dimension=2):
  else
    #
    | Hier muss geprueft werden, ob args eine leere Liste ist.
    #
    if type(args(1)) = DOM_LIST and nops(args(1)) = 0 then
        error("Illegal argument"):
    end_if:

    new_args := plotlib::clip2d(args());

    #
    | Festlegen der Perspektive und der Dimension
    #

    new_args := Dimension = 2, CameraPoint = [0, -0.0001, 1.0], new_args:
    _s_ := null():
    
    #
    | Abarbeiten der Argumentenliste
    #

    for i from 1 to nops(new_args) do
      arg := op(new_args, i):

      #
      | Test, um welche Option es sich handelt.
      | Hier werden nur die Optionen Labels,
      | ViewingBox und Mode=Curve getestet.
      | Alle anderen Optionen werden nicht ge-
      | pr"uft, denn das passiert sowieso im
      | anschliessenden plot-Aufruf.
      #

      case op(arg,[1,1]) 
           of (Labels) do

              #
              | Die Option Labels wurde verwendet.
              | Falls die Anzahl der Operanden von arg gleich 2 ist,
              | "uberpr"ufe die Anzahl der Operanden des 2. Operanden von
              | arg. Ist diese gr"osser gleich 2, so h"ange
              | an den zweiten den String "" an.
              #

              if nops(arg) = 2 
              then
                if nops(op(arg,2)) >= 2
                then
                  arg := subsop(arg,[2,2]=(op(arg,[2,2]),"")):
                end_if:
              end_if:
              break;
           of (ViewingBox) do

              #
              | Die Option ViewingBox wurde gew"ahlt.
              | Falls die Anzahl der Operanden von arg gleich 2 ist,
              | "uberpr"ufe die Anzahl der Operanden des 2. Operanden von
              | arg. Ist diese gr"osser gleich 4, so h"ange an den
              | vierten Operanden die Sequenz ,0,0 an.
              #

              if  nops(arg) = 2
              then
                if nops(op(arg,2)) >= 4
                then
                  arg := subsop(arg,[2,4]=(op(op(arg,2),4),0,0)):
                end_if:
              end_if:
              break;
           of (Mode) do 

              #
              | Die Option Mode wurde verwendet.
              | Wir testen zun"achst, ob die Anzahl der Operanden in arg
              | gr"osser als 3 ist (Mode,Funktionen,Range,...). Danach pr"u-
              | fen wir, ob der zweite Operand des 1.Operanden von list[1]
              | gerade gleich Curve ist. Falls dem so ist, so testen wir, ob 
              | die Anzahl der Operanden des zweiten Operanden in arg
              | (das sind normalerweise die Funktionen) gr"osser gleich 2 ist.
              | Falls ja, so h"angen wir an den zweiten Operanden die Sequenz
              | ,0 an (das ist dann die z-Funktion).
              #

              if nops(arg) >= 3 
              then
                case (op(arg,[1,2])) 
                     of (Curve) do
                        if nops(op(arg,2)) >= 2
                        then
                          arg := subsop(arg,[2,2]=(op(arg,[2,2]),0)):
                        end_if:
                        break:
                end_case:
              end_if:
              break:
      end_case:
      _s_ := _s_ , arg;
    end_for:

    if stdlib::ShowPlot = FALSE then _s_ else plot(_s_) end_if

  end_if:
end_proc:
