# $Date: 1999/08/31 13:52:25 $ $Author: frankp $ $Revision: 1.14.2.7 $ #

# kg, 28/07/95 #

#++
plotfunc -- plot function graphs

plotfunc(f1,..., fn, x=xl..xu [, y=yl..yu])
plotfunc(f1,..., fn, x=[xl,xu] [, y=[yl,yu]])

f1,...,fn      - expression
x, y           - identifiers
xl, xu, yl, yu - real numbers

If only one equation x=xl..xu is given, then the fi are interpreted as
functions in x from the interval [xl,xu] into the real numbers.

If two equations x=xl..xu and y=yl..yu are given, the fi are interpreted as
functions in x and y from the range [xl,xu] x [yl,yu] into the real
numbers.

The graphs of these functions are then plotted.

Note that the fi are not evaluated, but the equations only.

Examples:
plotfunc(sin(x), cos(x), x=-[PI,PI]);	-- draws sin and cos on [-PI,PI]
plotfunc(sin(x*y)^2, x=0..1, y=0..1);	-- draws x,y -> sin(x*y)^2 on [0,1]x[0,1]
++#

stdlib::RATIO_YX := 10:

#
The variables plotfunc_2DSceneOptions and plotfunc_3DSceneOptions are
defined in the initialization file bin\sysinit.mu or lib/.mupadsysinit
                
plotfunc_2DSceneOptions := {Scaling=UnConstrained, Axes=Origin, Labeling=TRUE}:
plotfunc_3DSceneOptions := {Scaling=UnConstrained, Labeling=TRUE}:
#


plotfunc:= proc()
    // option hold;
    // Die Option hold verursacht auf Anwenderseite nur Probleme.
    // Auch ist z.B. f:=sin(x), cos(x): plotfunc(f, x=-PI..PI):
    // nicht moeglich. 
    // Die context-Aufrufe wurden ebenfalls auskommentiert. 
    local a, b, i, m, p, pf, xr, yr, DIGITS, getrange, t, __X_domain,
          ac, l, x, c, d, grid, iv, color, SceneOptions, first_func,
          LogXYZ, max_y, min_y;
begin
    DIGITS:= 10;
    if args(0) < 2 then error("wrong no of args") end_if;
    
    getrange:= proc(r) begin
        if domtype(op(r,1)) <> DOM_IDENT then error("no variable") end_if;
        if type(op(r,2)) = "_range" then
	    r:= (op(r,1) = [op(op(r,2))])
	elif domtype(op(r,2)) <> DOM_LIST then
	    error("no plot range")
	elif nops(op(r,2)) <> 2 then
	    error("no plot range")
	end_if;
	if domtype(float(op(r,[2,1]))) <> DOM_FLOAT then 
	    error("illegal plot range") 
	end_if;
	if domtype(float(op(r,[2,2]))) <> DOM_FLOAT then 
	    error("illegal plot range") 
	end_if;
        op(r,1) = op(r,2)
    end_proc;
    
    # get scene options #
    first_func := 1; SceneOptions := {};
    LogXYZ := [FALSE, FALSE, FALSE];
    t := args(first_func); 
    // t:=context(t);
    if type(t) = DOM_SET then
      first_func := first_func + 1;
      for a in t do
        if type(a) = "_equal" then
         case op(a, 1) 
          of AxesScaling do
            SceneOptions := SceneOptions union {a};
            a := op(a,2); m := nops(a);
            if domtype(a) = DOM_LIST and (m = 2 or m = 3) then
               if a[1] = Log then LogXYZ[1] := TRUE end_if;
               if a[2] = Log then LogXYZ[2] := TRUE end_if;
               if m = 3 and a[1] = Log then LogXYZ[3] := TRUE end_if;
            end_if; 
            break;
          otherwise
            SceneOptions := SceneOptions union {a};
         end_case
      else
         error("Illegal scene option ".expr2text(a));
      end_if
     end_for
    end_if;
             
    # get ranges #
    i:= args(0);
    a:= args(i);
    if type(a) <> "_equal" then error("no plot range") end_if;
    // a:= context(a);
    yr:= getrange(a);
    i:= i-1;
    a:= args(i);
    if type(a) = "_equal" then
        // a:=context(a);
        xr:= getrange(a);
        if op(yr, 1) = YRange then
           m:= hold(Mode=Curve);
           min_y := op(yr, [2,1]);
           max_y := op(yr, [2,2]);
           SceneOptions := SceneOptions union
                {ViewingBox = [op(xr, [2,1]), op(xr, [2,2]), min_y, max_y]};
           yr:= FAIL;
        else
           m:= hold(Mode=Surface);
        end_if;
        i:= i-1;
        if i = 0 then error("no function to plot") end_if;
    else
        m:= hold(Mode=Curve);
        xr:= yr;
        yr:= FAIL;
    end_if;
    
    # prepare creating of titles: objects of DOM_FLOAT should'nt be converted
      to strings via expr2text because of ending zeroes
    #
    __X_domain:= domain("__X_domain"):
    __X_domain::expr2text:= proc(x)
        local s,i;
    begin
        s:= expr2text( extop(args(1),1) );
        i:= strlen(s)-1;
        while i > 0 and s[i] = "0" and s[i-1] <> "." do i:= i - 1 end_while;
        substring( s,0..i )
    end_proc:

    # get functions #
    pf:= null();
    t:= "";
    while i >= first_func do
        # allow for plotting functions #
        a:=args(i);
        // a:= context(a); 
        ac := a;
        if contains({DOM_PROC, DOM_EXEC, DOM_FUNC_ENV}, domtype(a)) then
            if yr = FAIL then
		a:= subsop(hold(f(x)), 0=a, 1=op(xr,1))
	    else
	        a:= subsop(hold(f(x,y)), 0=a, 1=op(xr,1), 2=op(yr,1))
	    end_if
#- Do not evaluate the functions, if the next 2 lines are active
        else
            a:= args(i)
-#
        end_if;
	# add graph definition #
	if yr = FAIL then

        #------------ check for singularities ---------------#

        grid := 100; color := RGB::ColorList[i];
        x := op(xr,1); c := op(xr, [2,1]); d := op(xr, [2,2]);
        l := stdlib::get_intervals(ac, x, c, d, stdlib::RATIO_YX, grid, LogXYZ, min_y, max_y);    
        for iv in l do
	        pf:= pf, [m, [ x, a ], x=iv, Grid=[grid], Color=[Flat,color]]
        end_for
	else
	    pf:= pf, [m, [ op(xr,1), op(yr,1), a ], xr, yr]
	end_if;

        a:= expr2text( 
            maprec( args(i), fun( bool(domtype(args(1))=DOM_FLOAT) ) = fun( new(__X_domain,args(1)) ) )
        );
	if t = "" then
	    t:= a
	else
	    t:= a.", ".t
	end_if;
	i:= i-1;
    end_while;

    # define scene attributes #
    if yr = FAIL then 
	p:= plot2d;
	xr:= expr2text(op(xr,1));
     b:= {op(plotfunc_2DSceneOptions union {Labels=[xr, ""], Title=t})}
    else
	p:= plot3d;
	xr:= expr2text(op(xr,1));
	yr:= expr2text(op(yr,1));
	b:= {op(plotfunc_3DSceneOptions union {Labels=[xr, yr, ""], Title=t})}
    end_if;
    a := b;
    for t in b do
        if has(SceneOptions, op(t, 1)) then
           a := a minus {t}
        end_if
    end_for;
    a := op(a), op(SceneOptions);

    # plot the stuff #
    p(a, pf);
end_proc:

# end of 'plotfunc' #


#--

Diese Prozedur untersucht, ob die Funktion f(x) im endlichen
Intervall [a,b] Singularitaeten besitzt und teilt ggf. die Intervalle
auf.

Enthaelt f an der Stelle d eine Singularitaet, so wird ein
maximaler bzw. minimaler Wert f"ur die Funktion gesetzt, ein sogenannter
threshold. 

Ergebnis hat folgende Form: [[a_1,b_1], ... , [a_n, b_n]], n >= 1.

Bsp: 

>> get_intervals(1/cos(x),x,0,PI,10,100);

             [[0.0, 1.507091268], [1.634501384, 3.141592653]]

>> get_intervals(10*x/(x^2-2*x-8),x,-5,5,10,100);

  [[-5.0, -2.072061831], [-1.938005503, 3.860888646], [4.127955252, 5.0]]

Ohne Verfeinerung mittels Bisektion w"urden die folgenden Intervalle geliefert:

                [[-5.0, -2.09], [-1.88, 3.82], [4.13, 5.0]]

--#


stdlib::get_intervals := proc(f, x, a, b, ratio, grid, LogXYZ, thres_min_y, thres_max_y) 
   local eps, ds, f1, i, l, n, t, tl, bl, br,
         max_y, min_y, nb, new_bound, retry, bisection;
begin

   bisection := proc(a, b, bound)
      local c, eps;
   begin
      eps := 10^(-DIGITS);
      while abs(b-a) > eps do
         c := (a+b)/2;
         if float(f(c)) <= bound then
            a := c;
         else
            b := c;
         end_if
      end_while;
      a;
   end_proc;

   # Berechne a, sodass thres_min_y <= f(a) <= thres_max_y #

   new_bound := proc(a, eps)
      local min_eps, y;
   begin

      min_eps := 10^(-DIGITS+1);
     
      if traperror((
           while abs(eps) > min_eps and thres_min_y <= (y := float(f(a+eps))) and y <= thres_max_y do
             eps := eps/10;
           end_while;
         )) <> 0 or abs(eps) <= min_eps then return(a+eps*10) end_if;

      repeat
        a := a + eps; y := float(f(a));
      until thres_min_y <= y and y <= thres_max_y end_repeat;

      # Der folgende Abschnitt dient nur zur Verfeinerung des Wertes 'a' #
      # Diese verfeinert insbes. die graphische Ausgabe.                 #
      # Vor Ausf"uhrung des Abschnitts gilt entweder                     #
      # 1) f(a) <= thres_max_y und f(a-eps) > thres_max_y , oder         #
      # 2) f(a) >= thres_min_y und f(a-eps) < thres_min_y                #
      # Vor Ausf"uhrung des Abschnitts gilt entweder                     #
      # 1) f(a) = thres_max_y oder 2) f(a) = thres_min_y                 #

      y := float(f(a-eps));
      if y > thres_max_y then
         a := bisection(a, a-eps, thres_max_y)
      else
         a := bisection(a-eps, a, thres_min_y)
      end_if;

      a

   end_proc;

   #------------- Hauptprogramm 'get_intervals' ----------------------#

   if LogXYZ[1] then
      # a <= b #
      if float(b) <= 0.0 then
         error("Nothing to plot")
      elif float(a) <= 0.0 then
         a := 10^(-DIGITS)
      end_if
   end_if;

   case domtype(f)

     of DOM_EXEC do # f = func(..., x) #
        t := misc::subsFunArgs(f);
        if t = FAIL then
           return([[a,b]])
        end_if;
        f1 := t[1]; x := t[2][1];
        break;

     of DOM_EXPR do
        f1 := f;
        f :=  newpurefunc("fun(...)", NIL, 
                  misc::subsFreeVars(f, x=hold(args(1))),
                  FALSE);
        break;

     of DOM_IDENT do
        if LogXYZ[2] then
           if b <= 0 then
              error("Nothing to plot")
           else
              a := 10^(-DIGITS/2)
           end_if
        end_if;
   
     otherwise           # Prozeduren koennen z.B. #
        return([[a,b]])  # nicht behandelt werden. #

   end_case;

   if LogXYZ[2] then
      f1 := ln(f1); f := ln@f;
   end_if;

   ds := discont(f1, x=a..b);

   # Enthaelt f keine Singularitaeten oder konnten keine #
   # Singularitaeten ermittelt werden, so gebe Funktion  #
   # f und das Intervall [a, b] unveraendert zurueck.    #

   if type(ds) <> DOM_SET or ds = {} then 
      return([[a,b]])
   end_if:

   ds := sort(map([op(ds)], float));
   a := float(a); b := float(b);
   n := nops(ds); 
   l := []; tl := []; bl :=[]; br := [];

   if ds[1] <> a then
      t := float(f(a));
      if not LogXYZ[2] or type(t) <> DOM_COMPLEX then
         tl := append(tl, t);
         bl := [a, ds[1]];
      end_if;
   end_if;

   if ds[n] <> b then                                         
      t := float(f(b));
      if not LogXYZ[2] or type(t) <> DOM_COMPLEX then 
         tl := append(tl, t); 
         br := [ds[n], b]; 
      end_if;
   end_if; 
   
   for i from 2 to n do
      t := float(f(ds[i-1]+(ds[i]-ds[i-1])/2));
      if not LogXYZ[2] or type(t) <> DOM_COMPLEX then  
         tl := append(tl, t);  
         l  := append(l, [ds[i-1], ds[i]]);  
      end_if;
   end_for;

   min_y := min(op(tl)); max_y := max(op(tl)); 

   nb := 1; tl := l;

   repeat 

   retry := FALSE;

   t := max(0, ( ratio * (b-a) - (max_y - min_y) ) / 2);
   if type(thres_max_y) = DOM_IDENT then
      thres_max_y := max_y + t; thres_min_y := min_y - t;
   else
      if max_y + t > thres_max_y then
         thres_max_y := max_y + t;
      end_if;
      if min_y - t < thres_min_y then
         thres_min_y := min_y - t;
      end_if;     
   end_if;
#
   if LogXYZ[2] then 
      thres_min_y := max(thres_min_y, -5);
      thres_max_y := max(thres_max_y,  5);
   end_if;
#
   eps := (b-a)/grid;

   for i from 1 to nops(l) do                                       
       l[i][1] := new_bound(l[i][1], +eps);
       l[i][2] := new_bound(l[i][2], -eps); 
   end_for;                                                   
   if bl <> [] then                                          
      l := [[bl[1], new_bound(bl[2], -eps)]].l;     
   end_if;
   if br <> [] then                                         
      l := append(l, [new_bound(br[1], +eps), br[2]]);               
   end_if;

   for i in l do
       if abs(i[1]-i[2]) < 10^(-DIGITS) then
          // Linke und rechte Intervallgrenze
          // (x-Bereich) sind (fast) identisch.
          // Erhoehe das Verhaeltnis zwischen
          // x- und y-Bereich um einen groesseren
          // Bildausschnitt zu bekommen. Damit
          // wird implizit der y-Bereich festgelegt.
          ratio := ratio*2; l := tl;
          retry := TRUE; nb := nb + 1;
          break; 
       end_if
   end_for;
 
   // Nach 2 fehlgeschlagenen Versuchen den Bildbereich, 
   // d.h. automatisch den y-Bereich festzulegen wird abgebrochen.

   until not retry or nb > 3 end_repeat;

   if retry then
      error("Vertical range detection failed. Please use option YRange.")
   end_if;

   l

end_proc:

# end of file #
