# from Decreasing the Nesting Depth of Expressions Involving Square Roots
   by Borodin, Fagin, Hopcroft and Tompa, JSC 1, 1985, pages 169-188 

  Other reference to consider:
  Landau,-Susan, How to tangle with a nested radical.  1994
  [The-Mathematical-Intelligencer] 16 (1994), no.  2, 49--55.
#

alias(_radsimp_match2sqrt=stdlib::radsimp_match2sqrt,
	_radsimp_matchsqrt=stdlib::radsimp_matchsqrt,
	radnormal=stdlib::radnormal,
	_radsimp_matchmultsqrt=stdlib::radsimp_matchmultsqrt,
	_radsimp_sqrtdepth=stdlib::radsimp_sqrtdepth):

radsimp := proc(e)
local q,abr,a,b,r,l,ee;
begin
   if e::radsimp <> FAIL then return( e::radsimp(args()) ) end_if;
   if domtype(e)=DOM_EXPR then
      if testtype(e,Type::AlgebraicConstant) then e:=radnormal(e)
      elif type(e)="_plus" then e:=map(e,radsimp)
      elif contains({"_mult","_power"},type((e:=map(e,radsimp)))) then
         l:=[e]; ee:=0;
         repeat
            e:=l[1]; l[1]:=NIL;
	    if type((q:=denom(e)))="_plus" and
	       (abr:=_radsimp_match2sqrt(q))<>FAIL then # q = a+b*sqrt(r) #
		  a:=abr[1]; b:=abr[2]; r:=abr[3];
		  e:=expand(numer(e)*(a-b*sqrt(r)))/(a^2-b^2*r);
	          if testtype(e,Type::Constant) then e:=radnormal(e) end_if;
                  l:=append(l,(if type(e)="_plus" then op(e) else e end_if));
            else
               ee:=ee+e
	    end_if;
         until l=[] end_repeat;
         e:=ee;
        if testtype(e, Type::Constant) then
          e:=radnormal(e)
        end_if;
      end_if;
   end_if;
   if testtype(e, Type::AlgebraicConstant) then
     radnormal(e)
   else
     e
   end_if
end_proc:

DOM_SET::radsimp:=fun(map(args(1),radsimp)):
DOM_LIST::radsimp:=fun(map(args(1),radsimp)):

_radsimp_match2sqrt := proc(f) # match a+b*sqrt(r) and returns a,b,r #
local a,b,i,j,br;
begin
    if type(f)="_plus" then
      b:=op(f,1); j:=1;
      for i from 2 to nops(f) do
         if _radsimp_sqrtdepth(op(f,i))>_radsimp_sqrtdepth(b) then b:=op(f,i); j:=i end_if
      end_for;
      # a:=f-b; #
      a:=subsop(f, j=null());
      f:=b;
      # f should be b*sqrt(r) #
      if (br:=_radsimp_matchmultsqrt(f))<>FAIL then return(a,br) end_if
    end_if;
    FAIL
end_proc:

_radsimp_matchsqrt := proc(e) # matches sqrt(f) and returns f, or FAIL #
begin
   if type(e)="_power" then
      if op(e,2)=1/2 then return(op(e,1)) end_if
   elif type(e)="sqrt" then return(op(e))
   end_if;
   FAIL
end_proc:

# matches b*sqrt(r) and returns b,r (be can be 1 or a product of several expressions) #
_radsimp_matchmultsqrt := proc(e) 
local r,t,f,b;
begin
   if type(e)="_mult" then
      b:=1; r:=1;
      for t in e do
         if (f:=_radsimp_matchsqrt(t))<>FAIL then r:=r*f else b:=b*t end_if
      end_for;
      if r<>1 then return(b,r) end_if
   elif (r:=_radsimp_matchsqrt(e))<>FAIL then return(1,r)
   elif type(e)="_power" then
      if type(op(e,2))=DOM_RAT and type(2*op(e,2))=DOM_INT then
         return(op(e,1)^(op(e,2)-1/2),op(e,1))
      end_if
   end_if;
   FAIL
end_proc:

_radsimp_sqrtdepth := proc(e)
local i;
option remember;
begin
   if _radsimp_matchsqrt(e)<>FAIL then 1+_radsimp_sqrtdepth(op(e,1))
   elif type(e)="_power" then
      if type(op(e,2))=DOM_RAT and type(2*op(e,2))=DOM_INT then 1+_radsimp_sqrtdepth(op(e,1))
      else 0
      end_if
   elif contains({"_plus","_mult"},type(e)) then max(_radsimp_sqrtdepth(op(e,i))$i=1..nops(e))
   else 0
   end_if
end_proc:

radnormal:=
proc(e)
  local l,s,Q,a,b,p,i,j,f,X,y,eq;
begin
   normal; a:=stdlib::normal(e);
   if a[2]<>1 then
      a:=radnormal(a[1]); if a=0 then return(0) end_if
   end_if;           
   # simplify converts sqrt(6) to sqrt(2)*sqrt(3) #
   l:=rationalize(simplify(e),NIL,NIL,{"_power"},Recursive);
   s:=l[2]; y:=null();
   Q:=Dom::Rational;
   hastype; # to load Indets #
   for j from nops(s) downto 1 do
      eq:=s[j];
      X:=op(eq,1); a:=op(eq,2); 
      if a=I then b:=1/2; a:=-1
      elif type(a)<>"_power" then return(FAIL)
      else b:=op(a,2); a:=subs(op(a,1),y)
      end_if;
      f:=poly(X^denom(b)-(if b<0 then expr(Q(a)) else a end_if)^abs(numer(b)),
	[X],Q);
      p:=f; f:=factor(f);
      if nops(f)>3 then # find good factor #
         a:=1;
         for i from 2 to nops(f) step 2 do
            b:=abs(float(subs(expr(f[i]),op(s))));
            if b<a then a:=b; p:=f[i] end_if
         end_for;
         userinfo(1,subs(op(eq,2),op(l[2])),"simplifies to",
		subs(expr(p),X=hold(X),op(l[2])));
      else p:=f[2]
      end_if;
      if degree(p)>1 then Q:=Dom::AlgebraicExtension(Q,p)
      else
         p:=expr(p); y:=y,X=-coeff(p,[X],0)/coeff(p,[X],1); l[1]:=subs(l[1],y);
      end_if;
   end_for;
   expand(subs(expr(Q(numer(l[1]))*Q(denom(l[1]))^(-1)),op(l[2])))
end_proc:
