
#--

spline -- Spline interpolation 

Call:

spline([a,b], l)

Parameter:

a, b -- real numbers and a < b
l    -- a non-empty list of real numbers

Synopsis:

spline([a,b], l) computes the spline interpolation in the interval
[a, b] of the data l. l[1],...,l[n] are the values at the evaluation
points. The evaluation points are x[i] = a+(i-1)*h, i = 1..n , 
h = (b-a)/(n-1).

Examples:

>> f:=spline([0,1], [5,10,24,-3]): 
>> plot2d(Axes=Origin,Scaling=UnConstrained,[Mode=Curve,[x,hold(f(x))],x=[0,1]]):

--#

spline := proc(xl,y)
   local a, b, h, i, n, q, X, body, x;
begin

   if testargs() then
      if args(0) <> 2 then
         error("Wrong number of arguments")
      elif not testtype(xl, Type::ListOf(Type::RealNum,2,2)) or xl[1] >= xl[2] then
         error("Interval [a,b] (a, b real numbers, a<b) expected")
      elif not testtype(y, Type::ListOf(Type::RealNum,1)) then
         error("List must contain numerical elements")
      end_if
   end_if;

   a := xl[1]; b := xl[2]; n := nops(y);
   h := (b-a)/(n-1);
   X := array(0..n+1, [a+h*(i-1)$i=0..n+1]);
   q := array(0..n+1);

   assign( linsolve({
             q[i-1]+4*q[i]+q[i+1] = 6*y[i] $ i = 1..n,
             q[0]-2*q[1]+q[2]     = 0, 
             q[n-1]-2*q[n]+q[n+1] = 0
            },
            {q[i] $ i = 0..n+1}
          )
   );

   subs(

   proc(x)
     local p;
   begin
     p := proc(x) begin
       if -2 <= x and x <= -1 then
          (x^3+6*x^2+12*x+8)/6
       elif -1 <= x and x <= 0 then
          (-3*x^3-6*x^2+4)/6
       elif 0 <= x and x <= 1 then
          (3*x^3-6*x^2+4)/6
       elif 1 <= x and x <= 2 then
          (-x^3+6*x^2-12*x+8)/6
       else
          0
       end_if
     end_proc:
          
     body

   end_proc ,

   body = _plus(q[i]*p((x-X[i])/h)$i=0..n+1)

   )
 
end_proc:
   
