#-

string::contains -- tests if a substring exists

Calling sequence:

string::contains(s, t)
string::contains(s, t, Index)
string::contains(s, t, IndexList)

Parameter:

s         - non-empty string
t         - non-empty string
Index     - an optional name
IndexList - an optional name

Summary:

string::contains(s, t) returns TRUE, if the string s contains the
string t. Otherwise FALSE is returned.

string::contains(s, t, Index) returns a list [n, m, i]. n is the
length of s and m is the length of t. The length of a string is
the number of characters. If i = FAIL, then the string s doesnt 
contain the string t. Otherwise i is an integer and 
s[i..i+m-1] = substring(s, i, m) = t holds.

string::contains(s, t, IndexList) returns a list [n, m, l]. l is
a list. If l is the empty list, i.e. l = [], then the string s doesnt 
contain the string t. Otherwise s[l[i]..l[i]+m-1] = t for all
i = 1..nops(l) holds.

Examples:

>> string::contains("abcdabe", "ab");

           TRUE
		   
>> string::contains("abcdabe", "ab", Index);

           [7, 2, 0]
		   
>> string::contains("abcdabe", "ab", IndexList);

           [7, 2, [0, 4]]
		   
>> string::contains("abcdabe", "abd");

           FALSE
		   
>> string::contains("abcdabe", "abd", Index);

           [7, 3, FAIL]
		   
>> string::contains("abcdabe", "abd", IndexList);

           [7, 3, []]
		   
See also:

contains, substring, string::match

-#

string::contains := proc( s, t, opt )
   local i, k, l, m, n, r;
begin

   if testargs() then
   
      case args(0) 
        of 3 do 
	       if opt <> Index and opt <> IndexList then
		      error("Illegal 3rd argument")
		   end_if;
	    of 2 do
	       if not (testtype(s, DOM_STRING) and s <> "" and 
		           testtype(t, DOM_STRING) and t <> "") then
		      error("Non-empty string expected")
		   end_if;
		   break;
	   otherwise
	       error("Wrong number of arguments")
      end_case
	  
   end_if;
   
   case opt 
     of Index do
	 of IndexList do
	    m := strlen(t); n := strlen(s); 
		k := string::FirstIndex;
		next;
	 of Index do
		return( [ n, m, string::match(s, t, n, m, k) ] );
	 of IndexList do
	    r := [ n, m ]; l := []; 
		while ( i := string::match(s, t, n, m, k) ) <> FAIL do
		   l := append(l, i); k := i + m
		end_while;
	    return( append(r, l) );
	 otherwise
	    return( strmatch(s, "\*".t."\*") )
   end_case;
 
end_proc:

