#-

string::delete -- deletes substrings

Calling sequence:

string::delete(s, t)
string::delete(s, t, First)
string::delete(s, a)
string::delete(s, a..b)

Parameter:

s, t  - strings
a, b  - non-negative integers
First - name

Examples:

>> string::delete("abcdab", "ab", First);

                                  "cdab"

>> string::delete("abcdab", "ab");       

                                   "cd"

>> string::delete("abcdab", 3);

                                  "abcab"

>> string::delete("abcdab", 1..3);
 
                                   "aab"
-#

string::delete := proc(s, r, opt)
   local a, b, len;
begin

   if testargs() then

      if type(s) <> DOM_STRING then
         error("String expected");
      end_if;
      len := strlen(s);
      if type(r) = "_range" then
         a := op(r, 1); b := op(r, 2);
         if not (testtype(a, Type::NonNegInt) and testtype(b, Type::NonNegInt)) 
            or b < a or b >= len then
            error("Illegal range")
         end_if;
         string::subsop(s, r="")
      elif testtype(r, Type::NonNegInt) then
           if r >= len then
              error("Illegal index")
           end_if;
           string::subsop(s, r="") 
      elif type(r) = DOM_STRING then
           if args(0) = 3 then
              if opt <> First then
                 error("Illegal option")
              end_if;
              string::subs(s, r="", First)
           else
              string::subs(s, r="")
           end_if
      else
           error("Illegal argument")
      end_if

   else

     if type(r) = DOM_STRING then
        string::subs(args())
     else
        string::subsop(s, r="")
     end_if

   end_if
        
end_proc:
