#-

string::format -- formats strings

Calling sequence:

string::format(s, len, align, pat)

string::format(s, len)		-> string::format(s, len, Left, " ")
string::format(s, len, pat)     -> string::format(s, len, Left, pat)
string::format(s, len, align)   -> string::format(s, len, align, " ")

Parameter:

s     - string
pat   - character (strlen(pat) = 1)
len   - positive integer
align - Left, Right or Center


Examples:

>> string::format("abc", 10);

                               "abc       "

>> string::format("abc", 10, Center, "_");

                               "___abc____"

-#


string::DEFAULT_ALIGN := Left:
string::DEFAULT_PAT   := " ":

string::format := proc(s, len, align, pat)
   local l, d1, d2;
begin
   case args(0)
     of 2 do
        align := string::DEFAULT_ALIGN; 
        pat := string::DEFAULT_PAT;
        break;
     of 3 do
        if type(align) = DOM_STRING then
           pat := align; align := string::DEFAULT_ALIGN;
        else
           pat := string::DEFAULT_PAT
        end_if;
        break;
     of 4 do 
        break;
     otherwise
        error("Wrong number of arguments")
   end_case;

   if testargs() then

      if type(s) <> DOM_STRING then
         error("String expected")
      end_if;
      if not testtype(len, Type::PosInt) then
         error("2nd argument must be a positive integer")
      end_if;
      if not contains({Left, Right, Center}, align) then
         error("Illegal alignment")
      end_if;
      if type(pat) <> DOM_STRING or strlen(pat) <> 1 then
         error("Illegal pattern")
      end_if;

   end_if;
        
   l := strlen(s);
   if len < l then
      substring(s, 0, len)
   elif len = l then
      s
   else
      case align
        of Right do
           (pat $ len-l).s; break
        of Left do
           s.(pat $ len-l); break
        of Center do
           d1 := len-l; d2 := d1 div 2;
           (pat $ d2).s.(pat $ d1-d2);
           break
        otherwise
           error("Illegal alignment")
      end_case
   end_if

end_proc:
