#- 

string::subs -- substitute strings

Calling sequence:

string::subs(s, t = r)
string::subs(s, t = r, First)

Parameter:

s     - non-empty string
t     - non-empty string
r     - string
First - an optional name

Summary:

string::subs(s, t = r) replaces all substrings t in the string s by
the string r. This new string is returned. string::subs(s, t = r, First)
replaces only the first occurrence of t in s by r.

Examples:

>> string::subs("abcdabe", "ab" = "X");

           "XcdXe"
		   
>> string::subs("abcdabe", "ab" = "X", First);

           "Xcdabe"
		   
>> string::subs("abcdabe", "ab" = "");

           "cde"
		   
>> string::subs("abcdabe", "abd" = "X");

           "abcdabe"
		   
See also:

subs, string::contains

-#

string::subs := proc(s, eq, opt)
   local m, n, i, k, l, r, sn, t;
begin

   if testargs() then
	  case args(0)
	    of 3 do
		   if opt <> First then
		       error("Illegal 3rd argument")
		   end_if;
		of 2 do
		   if not (testtype(s, DOM_STRING) and s <> "") then
		      error("Non-empty string expected")
		   end_if;
		   if type(eq) <> "_equal" then
		      error("Equation expected")
		   end_if;
		   t := op(eq, 1); r := op(eq, 2);
		   if not (testtype(t, DOM_STRING) and t <> "") then
		      error("Non-empty string expected")
		   end_if;
		   if not testtype(r, DOM_STRING) then
		      error("String expected")
		   end_if;	
		   break;
		otherwise
		   error("Wrong number of arguments")
	  end_case	   
   else	   
      t := op(eq, 1); r := op(eq, 2);
   end_if;
   
   if opt = First then opt := Index else opt := IndexList end_if;
   l := string::contains(s, t, opt);
   n := l[1]; m := l[2]; l := l[3];
   if l = FAIL or l = [] then
      return( s )
   end_if;
   k := string::FirstIndex; sn := "";
   for i in l do
       if i > k then
	      sn := sn.substring(s, k, i-k)
	   end_if;
	   sn := sn.r; k := i + m ;
   end_for;
   if n > k then
      sn := sn.substring(s, k, n-k)
   end_if;
   sn
end_proc:

