#-

string::subsop -- substitutes substrings

Calling sequence:

string::subsop(s, a=r)
string::subsop(s, a..b=r)

Parameter:

s, r  - strings
a, b  - non-negative integers

Examples:

>> string::subsop("abcdef", 3 = "XX");

                                 "abcXXef"


>> string::subsop("abcdef", 3 = "");
 
                                  "abcef"

>> string::subsop("abcdef", 2..4 = "X");

                                  "abXf"

-#

string::subsop := proc(s, eqn)
   local a, b, l, len, r, t;
begin

   if testargs() then

      if type(s) <> DOM_STRING then
         error("String expected");
      end_if;
      len := strlen(s);
      l := op(eqn, 1); r := op(eqn, 2);
      if type(r) <> DOM_STRING then 
         error("String expected"); 
      end_if;
      if type(l) = "_range" then
         a := op(l, 1); b := op(l, 2);
         if not (testtype(a, Type::NonNegInt) and testtype(b, Type::NonNegInt)) 
            or b < a or b >= len then
            error("Illegal range")
         end_if
      elif not testtype(l, Type::NonNegInt) or l >= len then
         error("Illegal index")
      else
         a := l; b := a;
      end_if

   else

      len := strlen(s);
      l := op(eqn, 1); r := op(eqn, 2);
      if type(l) = "_range" then
         a := op(l, 1); b := op(l, 2);
      else
         a := l; b := a;
      end_if

    end_if;

   t := "";
   if a > 0 then
      t := substring(s, 0, a)
   end_if;
   t := t.r;
   if b < len-1 then
      t := t.substring(s, b+1, len-b-1)
   end_if;
   t
end_proc:
