# #
# frankp, 7.09.95 #
# #

#++
	student::Kn  --  the vectorspace of n-tupels over K

	Kn( [n,] F )

	n: (optional) positive integer
	F: a Field

	This contructor creates a domain representing the vectorspaces
	of n-tupels over K.
	The operator + is defined to add two n-tupels component-wise.
	Kn together with this operation forms an Abelian group.

	This domain constructor is simply build up by the domain
	constructor 'Dom::MatrixGroup'.

	The scalar multiplication (by default realized by
	Product["_mult"]) can be overloaded by the method
	"scalarMult".
++#

student::Kn:= DomainConstructor::new(

# Name = # student::Kn,

# Args = # [ n,F ],

# Local Paramameters = # [ MF ],

# Init = # (
    if args(0) <> 1 and args(0) <> 2 then
	error("wrong no of args")
    end_if;
    if args(0) = 1 then F:= n; n:= 1 end_if;
    if F::hasProp( Cat::Field ) <> TRUE then
        error("coefficient domain must be a field")
    end_if;
    if not testtype( n,Type::PosInt ) then
	error("number of components must be a positive integer")
    end_if;
    MF:= Dom::MatrixGroup( n,1,F )
),

# Super Domain = # Dom::MatrixGroup( n,1,F ),

# Categories = # [ Cat::VectorSpace( F ), Cat::MatrixCat( F ) ],

# Axiome = # [
    (if F::hasProp( Ax::canonicalRep ) then
	Ax::canonicalRep
    end_if)
],

# Methods = #
"new" = proc()
    local A;
begin
    if traperror( (A:=MF::new(n,1,args())) ) <> 0 then
        error("wrong argument(s)")
    end_if;

    extsubsop(A,0=this)
end_proc,

"_mult" = proc(x,y)
    local T;
begin
    if args(0) > 2 then
        return( this::_mult(x,
            this::_mult( y,args(i) $ hold(i)=3..args(0) )
        ) )
    end_if;
    if domtype(x) = this and domtype(y) = this then
	FAIL
    elif domtype(x) = this then
	if this::scalarMult <> FAIL then
	    this::scalarMult( y,x )
	else
	    T:= x::getSuperDomain(x);
	    T:= T::_mult( extsubsop(x,0=T),y );
	    if T <> FAIL then extsubsop(T,0=this) 
            else FAIL
            end_if
	end_if
    elif domtype(y) = this then
	if this::scalarMult <> FAIL then
	    this::scalarMult( x,y )
	else
            T:= y::getSuperDomain(y);
            T:= T::_mult( x,extsubsop(y,0=T) );
	    if T <> FAIL then extsubsop(T,0=this)
	    else FAIL
	    end_if
	end_if
    else
	FAIL
    end_if
end_proc,

"convert" = domattr(MF,"convert"),

"delRow" = domattr(MF,"delRow"),

"delCol" = domattr(MF,"delCol"),

"stackMatrix" = domattr(MF,"stackMatrix"),

"concatMatrix" = domattr(MF,"concatMatrix"),

"row" = domattr(MF,"row"),

"col" = domattr(MF,"col")

):

# end of file #
