#++
  frankp, 7/09/95

        student::equateMatrix  --  return a matrix equation

	equateMatrix( A,vars )

	A   : augmented coefficient matrix
	vars: set or list of unknowns

	equateMatrix( A,vars ) returns the equation
	    A[1..r,1..c-1] * X = A[1..r,c..c]

	where r and c are the row and column number of A and
	X is the vector formed by the unknowns in vars.
++#

student::equateMatrix:= proc(eA,vars)
    local M, MeA, X, b;
begin
    if testargs() then
        if args(0) <> 2 then
	    error("wrong no of args")
        end_if;
        if eA::hasProp( Cat::MatrixCat ) <> TRUE then
 	    error("expecting a matrix")
        end_if;
        if not contains({DOM_LIST,DOM_SET},domtype(vars)) then
	    error("expecting a list or set of unknowns") 
        end_if
    end_if;

    # Convert eA to a matrix over ExpressionField() #
    M:= Dom::Matrix();
    MeA:= M( eA );

    # Create X = [ x_i ]_i, x_i in vars #
    X:= M( nops(vars),1,[op(vars)] );

    d:= MeA::dimen(MeA);

    # Extract the right-side vector of eA #
    b:= MeA::col( MeA, d[2] );

    # Return the matrix equation #
    MeA[1..d[1], 1..d[2]-1] * X = b
end_proc:

# end of file #
