#++
  frankp, 28/01/96

        student::gaussElim  -- perform ordinary Gaussian elimination

        gaussElim( A )

        A : Matrix (domain of category Cat::MatrixCat(F))

        gaussElim( A ) performs ordinary Gaussian elimination
	on A over a field F and reduces A to an upper triangular 
	matrix T.

	The list [T,cl,det] is returned, whereby cl is
	a set of characteristic indices of A and det
	the determant of A. The last entry of that list
	is the value FAIL if A is not square.
++#

student::gaussElim := proc(A)
    local m, n, i, j, l, k, cl, t, R, det;
begin
    if testargs() then
	if args(0) <> 1 then
	    error("wrong no of arguments")
	end_if;
	if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("invalid argument")
	end_if;
	R:= A::coeffRing;
	if not R::hasProp( Cat::Field ) then
	    error("expecting matrix over a Cat::Field")
	end_if
    end_if;


    R:= A::coeffRing;
    k:= A::dimen(A);
    m:= k[1]; n:= k[2]; 

    det:= 1;

    userinfo(1,"perform Gaussian elimination");
    
    cl:= {};
    det:= R::one;
    j:= 1;
    for i from 1 to n do
	if j = m then break #i# end_if;

        userinfo(2,"search a pivot for column ",i);
        for l from j to m do 
            if A[l,i] <> 0 then break end_if;
        end_for;
        if l > m then
            userinfo(2,"no pivot found");
            cl:= cl union {i};
	    det:= R::zero;
	    next # i #
	end_if;

	userinfo(3,"found pivot ",A[l,i]," in row,col = ",l,i);
        
        if l <> j then
            userinfo(2,"swap rows ",l," and ",j);
	    A:= A::swapRow( A,l,j );
	    det:= -det
        end_if;

        t:= A[j,i];
	det:= det * t;
	userinfo(2,"reduce column ",i);
        for k from j+1 to m do
            for l from i+1 to n do
                A[k,l] := A[k,l] - A[j,l]*A[k,i]/t
            end_for;
            A[k,i]:= R::zero
        end_for;

	j:= j + 1
    end_for;
    # the case for m < n #
    while i < n+1 and A[j,i] = 0 do
	userinfo( 2,"no pivot found for column ",i);
        cl:= cl union {i}; i:= i + 1
    end_while;
    cl:= cl union {k $ hold(k)=i+1..n};

    if m = n then [A, cl, det*A[n,n]]
    else [A, cl, FAIL]
    end_if
end_proc:
 
# end of file #
