#++
  frankp, 7/09/95

 	student::genExample  --  create randomly unimodular matrices over rings

	genExample( m,n,b [,R] )

	m,n: positive integers (dimension of the matrix)
	b  : positive integer (max. norm of the entries)
	R  : (optional) coefficient ring

	genExample( m,n,b ) creates randomly an unimodular matrix over
	Z, i.e. a matrix A with det(A) = +- 1.

	If a coefficient ring R is given, then the matrix A will
	be defined over R and det(A) = R* (i.e. an unit in R).

	Reference: Juergen Hansen
	           'Generating Problems in Linear Algebra'
	           MapleTech, Volume 1, No.2   1994
++# 

student::genExample:= proc( m,n,b,R )
    local p, r, c, A, i, j, k, v, toBig, f, l, Rnorm;
begin
    if args(0) < 3 then
	error("wrong no of args")
    elif args(0) <> 4 then R:= Dom::Integer
    end_if;

    Rnorm:= R::norm;
    if Rnorm = FAIL then Rnorm:= length end_if;


    p:= random(1..3):
    v:= [R::_negate(R::one),R::zero,R::one];

    r:= random(1..m):
    c:= random(1..n):

    userinfo(1,"(1) Build up an upper triangular matrix");
    A:= Dom::Matrix(R)( m,n,1,hold(Diagonal) );
    for i from 1 to m do
	for j from i+1 to n do
	    A[i,j]:= v[p()]
	end_for
    end_for;

    userinfo(1,"(2) Interchange rows");
    for i from 1 to m do
	p:= random(i..m)();
	if p <> i then A:= A::swapRow( A,i,p ) end_if
    end_for;

    userinfo(1,"(3) perform row / column addition");
    l:= n*(trunc(ln(b)/ln(1.2))+2);
    for p from 1 to l do
	i:= r();
	j:= r();
	while i = j do j:= r() end_while;
	f:= (-1)^random(0..1)();
	toBig:= FALSE;
	v:= [ NIL $ n ];
	for k from 1 to n do
	    if toBig then break end_if;
	    v[k]:= A[j,k] + f*A[i,k];
	    if Rnorm(v[k]) > b then toBig:= TRUE end_if
	end_for;
	if not toBig then A:= linalg::setRow( A,j,v ) end_if;

	i:= c();
        j:= c();
        while i = j do j:= c() end_while;
        f:= (-1)^random(0..1)();
        toBig:= FALSE;
	v:= [ NIL $ m ];
        for k from 1 to m do
            if toBig then break end_if;
            v[k]:= A[k,j] + f*A[k,i];
            if Rnorm(v[k]) > b then toBig:= TRUE end_if
        end_for;
        if not toBig then A:= linalg::setCol( A,j,v ) end_if
    end_for;

    A
end_proc:

# end of file #
