#++
v1.0, 28.10.1996, Dirk Mueller (olorien)

   IntTrans -- a library for doing integral transformation
++#

alias( IntTrans = transform::IntTrans ):

IntTrans::mult_split:=proc(_f, var)
local _rat, _nonrat, _g, _h, P_list, _p, _paus, i, typ;
begin

  typ:=type(_f);
  if typ<>"_mult" then 
    if testtype(_f, Type::RatExpr(var)) then
      return(_f,1);
    else
      if typ="_power" then
        if testtype(op(_f, 2), Type::RealNum) and
           testtype(op(_f, 1), Type::RatExpr(var)) then
          return(_f, 1);
        else
          return(1, _f);
        end_if;
      else
        return(1,_f);
      end_if;
    end_if;
  end_if;

  _rat:=select(_f, testtype, Type::RatExpr(var));

  _nonrat:=normal(_f/_rat); 

  if _nonrat<>1 then
    if type(_nonrat)<>"_power" then
      _g:=select(_nonrat, testtype, "_power");
    else
      _g:=_nonrat;
    end_if;

    if _g<>1 and _g<>0 then
      if type(_g)="_power" then
        P_list:=[testtype(op(_g,1), Type::RatExpr(var))];
        if P_list[1] then
          _rat:=_rat*_g;
          _nonrat:=normal(_nonrat/_g);
        end_if;
      else
        P_list:=[map(op(_g), op, 1)];
        P_list:=map(P_list, testtype, Type::RatExpr(var));
        _p:=[contains(P_list, TRUE, i) $ hold(i)=1..nops(P_list)];
        _paus:=nops({op(_p)});
        if _p[1]<>0 then
          _h:=_mult(op(_g, _p[i]) $ hold(i)=1.._paus);
          _rat:=_rat*_h;
          _nonrat:=normal(_nonrat/_h);
        end_if;
      end_if;
    end_if;
  end_if;

  return(_rat, _nonrat);
end_proc:


IntTrans::nonrat_ana
:=proc(_fu, var, depth)
local _f, typ, types, expos, funcops, consts, const, i,
          this_op, rat_part, nonrat_part, op_typ, back;
begin
  _f:=_fu;
  if has(_f, exp) then 
    _f:=combine(_f, exp);
  end_if;

  typ:=type(_f);
  back:=[];
  expos:=[];
  funcops:=[];
  types:=[];
  consts:=[];

  case typ
  of "_mult" do

    const:=select(_f, not has, var);
    _f:=normal(_f/const);

    if type(_f)<>"_mult" then
      if type(_f)="_power" then
        if nops(op(_f,1))=1 then
          return(["_mult", [type(op(_f,1))], [op(_f,2)], [op(op(_f,1))], [const]]); 
        end_if;
        return(FAIL);
      end_if;
      

      if nops(_f)=1 then
        return(["_mult", [type(_f)], [1], [op(_f)], [const]]); 
      end_if;
      return(FAIL);
    end_if;

    for i from 1 to nops(_f) do

      this_op:=op(_f, i); op_typ:=type(this_op);
     case op_typ


      of "_power" do
         if nops(op(this_op,1))=1 then
           types:=types.[type(op(this_op,1))];
           expos:=expos.[op(this_op,2)];
           funcops:=funcops.[op(op(this_op,1))];

         end_if;
         break;

      otherwise 
        if nops(this_op)=1 then
          types:=types.[type(this_op)];
          expos:=expos.[1];
          funcops:=funcops.[op(this_op)];
          consts:=consts.[const];

        end_if; 
      end_case; # op_typ #
    end_for;
    return(["_mult", types, expos, funcops, const]);

  of "_plus" do
    for i from 1 to nops(_f) do
      this_op:=op(_f, i); op_typ:=type(this_op);
      case op_typ
      of "_mult" do
        rat_part:=select(this_op, testtype, Type::RatExpr(var));
        nonrat_part:=normal(this_op/rat_part);
        if not has(rat_part, var) then
          case type(nonrat_part)
          of "_power" do 
            if nops(op(nonrat_part,1))=1 then
              types:=types.[type(op(nonrat_part,1))];
              expos:=expos.[op(nonrat_part,2)];
              funcops:=funcops.[op(op(nonrat_part,1))];
              consts:=consts.[rat_part];

            end_if;

            break;

          of "_mult" do
            return(FAIL);

          otherwise
            types:=types.[type(nonrat_part)];
            expos:=expos.[1];
            funcops:=funcops.[ op(nonrat_part)];
            consts:=consts.[rat_part];

          end_case; # type(nonrat_part) #
        else
          return(FAIL);
        end_if;
        break;

      of "_power" do
         if nops(op(this_op,1))=1 then
           types:=types.[ type(op(this_op,1))];
           expos:=expos.[op(this_op,2)];
           funcops:=funcops.[op(op(this_op,1))];
           consts:=consts.[1];

         end_if;
         break;

      otherwise

        if testtype(this_op, Type::RatExpr(var) ) then
          types:=types.["RAT"];
          expos:=expos.[1];
          consts:=consts.[1];
          funcops:=funcops.[this_op];
        else

          if nops(this_op)=1 then
            types:=types.[type(this_op)];
            expos:=expos.[1];
            funcops:=funcops.[op(this_op)];
            consts:=consts.[1];
          end_if;
        end_if; 
      end_case; # op_typ #
      end_for;
      return(["_plus", types, expos, funcops, consts]);

  of "_power" do
    if nops(op(_f,1))=1 then
      return(["_mult", type(op(_f,1)), op(_f,2), op(op(_f, 1)), [1]]);
    end_if;

  otherwise
    if nops(_f)=1 then
      return(["_mult", [type(_f)], [1], [op(_f)], [1]]);
    end_if;
  end_case; # typ #
end_proc:


IntTrans::anamult:=proc(_f, _var)
local _L, code, OPS, i;
begin
  _L:=[]; OPS:=[]; code:=[];
  for i from 1 to nops(_f) do 
    if nops(op(_f, i))=1 then
      _L:=IntTrans::anapol(op(op(_f, i)), _var);
      if _L=FAIL then
        return(FAIL);
      end_if;
    else
      _L:=IntTrans::anapol(op(_f, i), _var);
      if _L=FAIL then
        return(FAIL);
      end_if;
    end_if;
    code:=code._L[1];
    OPS:=OPS._L[2];
  end_for;
  return(code, OPS);
end_proc:


IntTrans::anapol:=proc(_f, _var)
local i, _g, _coeff, Deg, expo, expos, code, OPS;
begin
  code:=[]; OPS:=[]; expos:=[];

  if type(_f)="_mult" then 
    Deg:=degree(_f, [_var]);
    if Deg=FAIL then
      return([[[0,0]], [[0,0]], [[0]]] )
    end_if;

    if not has(normal(_f/_var^Deg), _var) then
      expo:=1; expos:=expos.[[expo]];
      _coeff:=[coeff(poly(_f, [_var]),_var,j) $ hold(j)=0..Deg ];
      code:=code.[[expo,_plus(2^(j-1)*
             IntTrans::zero_one(_coeff[j]) $ hold(j)=1..Deg+1)]];
      OPS:=OPS.[_coeff];
    else
      for i from 1 to nops(_f) do 
        _g:=op(_f, i);
        if type(_g)="_power" then
          expo:=op(_g,2);

          if not testtype(expo, NUMERIC) then
            expos:=expos.[[expo]]; expo:=0; 
          else
            expos:=expos.[[expo]];
          end_if;
          _g:=op(_g,1);
        else
          expo:=1; expos:=expos.[[expo]];
        end_if;

        Deg:=degree(_g, [_var]); 
        _coeff:=[coeff(poly(_g, [_var]),_var,j) $ hold(j)=0..Deg ];
        if has(_coeff[1], _var) then
          return( [[[0,0]], [[0,0]], [[0]]]   );
        end_if;

        code:=code.[[expo,_plus(2^(j-1)*IntTrans::zero_one(_coeff[j]) $ hold(j)=1..Deg+1)]];
        OPS:=OPS.[_coeff];
      end_for;
    end_if;


  else

      _g:=_f;

      if type(_g)="_power" then
        expo:=op(_g,2);
        _g:=op(_g,1);

        if not testtype(expo, NUMERIC) then
          expos:=expos.[[expo]]; expo:=0;
        else
          expos:=expos.[[expo]];
        end_if;

      else
        expo:=1; expos:=expos.[[expo]];
      end_if;
      #
      if expo<0 then
        return([[[0,0]], [[0,0]], [[0]]]);
      end_if;
#
      Deg:=degree(_g, [_var]); 

      if Deg=FAIL then
        return([[[0,0]], [[0,0]], [[0]]])
      end_if;
      _coeff:=[coeff(poly(_g, [_var]),_var,j) $ hold(j)=0..Deg];
      if has(_coeff[1], _var) then
        return([[[0,0]], [[0,0]], [[0]]]);
      end_if;

      code:=code.[[expo, _plus(2^(j-1)*
            IntTrans::zero_one(_coeff[j]) $ hold(j)=1..Deg+1)]];
      OPS:=OPS.[_coeff];
    
  end_if;
  return([code, OPS, expos]);

end_proc:


IntTrans::ratfunc_test:=proc(_f, pat)
local Num, Den;
begin
  if testtype(_f,Type::RatExpr(pat)) then
    Num:=numer(_f); Den:=denom(_f);
    return(Num, Den);
  end_if;
  FAIL;
end_proc:


IntTrans::Minus:=proc(_z)
begin
  if testtype(_z, Type::RealNum) then
    if _z<0 then
      return(TRUE);
    end_if;
  elif type(_z)="_mult" then
    if testtype( op(_z, nops(_z)), Type::RealNum) then
      if op(_z, nops(_z))<0 then
        return(TRUE)
      end_if;
    end_if;
  end_if;
  FALSE;
end_proc:


#--
  computes the integral-transform of built in functions  
  if the computation is impossible, FAIL will be returned 
--#
IntTrans::built_in:=proc(T) local _f, var1, var2, tmp;
begin
  _f:=extop(T,1); var1:=extop(T,2); var2:=extop(T,3); tmp:=op(_f, 0);
  if type(level(tmp,2))=DOM_FUNC_ENV then
    return( funcattr(level(tmp,2), type(T))(_f, var1, var2) )
  end_if;  
  FAIL;
end_proc:


#--
   linearity of integral transforms 
--#
IntTrans::linear:=proc(T) 
  local _f, var1, var2, typ, nps, arg, _c, _s;
begin
  _f:=extop(T, 1); var1:=extop(T, 2); var2:=extop(T, 3);
  typ:=type(_f); arg:=op(_f); nps:=nops(_f);
  if typ="_plus" then 
    userinfo(3,"linear and additive");
    return( _plus(map(op(_f), domtype(T), var1, var2)))
  elif typ="_mult" then
    _s:=select(_f, has, var1);
    _c:=normal(_f/_s); # noetig,  da t^n/t^n nicht gekuerzt wird #
        

    if not (_c=1) then
      userinfo(3,"linear and multiplicative");
      return(_c*T::new(_s, var1, var2))
    else
      return(FAIL);
    end_if;
  else
    return(FAIL);
  end_if;
end_proc:

#--
   transforms the D-Operator into diff 
--#
IntTrans::D2diff:=proc(_f) local dop, var;
begin
  dop:=op(_f,0); var:=op(_f,1);
  if nops(dop)=1 then
    if domtype(op(dop))=DOM_IDENT then
      return( diff(op(dop)(var),var))
    elif type(op(_f,0))="D" then
      return( diff(IntTrans::D2diff( op(dop)(var)),var) )
    end_if;
  else
    var:=op(op(_f,0),1);
    diff( op(dop, 2)(op(_f, i) $ hold(i)=1..nops(_f)), op(_f, op(var[j])) $ hold(j)=1..nops(var));
  end_if;
end_proc:

IntTrans::funcpos:= fun((contains(map([op(args(1))],type),args(2)))):
IntTrans::sign:=fun(op( sign(args(1)),nops(sign(args(1))))) :
IntTrans::zero_one:=fun((if args(1)<>0 then return(1) else return(0) end_if; )):
IntTrans::anatype:=fun((if nops(args(1))=1 then [type(args(1))] else [map(op(args(1)), type)] end_if)):
IntTrans::anapols:=fun(map(args(1), IntTrans::anapol, args(2))):
IntTrans::split2:=fun( (if testtype(args(1), Type::RatExpr(args(2) ) ) then [args(1), 1] end_if; select(args(1), testtype, Type::RatExpr(args(2))); if %=1 then return([1, args(1)]) end_if;  [last(2), args(1)/last(2)])):

unalias( IntTrans ):

# end of file #
