# Dirk Mueller, olorien, 28/10/96, v1.0 #

alias( IntTrans = transform::IntTrans ):

transform::laplace:= proc()
    local laplace;
begin

laplace:=domain("transform::laplace"):
laplace::type:="transform::laplace":

laplace::has:=proc() local T, v;
begin
  T:=args(1); v:=args(2);
  bool( v=extop(T,2) or v=extop(T,3) or 
        has(extop(T,1), v) or v=hold(laplace)
  )
end_proc:

laplace::new:=proc(_f, var1, var2)
   local back;
begin
  if testargs() then
    if args(0)<>3 then error("wrong number of arguments") end_if;
 
    if domtype(_f)<>DOM_EXPR and domtype(_f)<>DOM_IDENT and
       not testtype(_f,NUMERIC) and domtype(_f)<>transform::ilaplace then
         error("first argument of wrong type");
    end_if;
    if domtype(var1)<>DOM_IDENT then
      error("second and third argument must be from type DOM_IDENT");
    end_if;
  end_if;

  if domtype(_f)=transform::ilaplace then
    return(subs(extop(_f,1), extop(_f,2)=var2))
  end_if;
  back:=(transform::laplace)::compute(_f, var1, var2);
 
  if back=FAIL then
    return(new(transform::laplace, _f, var1, var2))
  else
    return(back)
  end_if;
end_proc:


laplace::print:=proc(T)
begin
  hold(laplace)(extop(T,1), extop(T,2), extop(T,3));
end_proc:

#this part computes the laplacian transform#
laplace::compute:=proc(_g, var1, var2) local _f, back, typ, i;
begin 

  _f:=_g;
  if not has(_f, var1) then
    return(_f/var2);
  elif _f=var1 then
    return(1/var2^2);
  end_if;

  typ:=type(_f);

  if typ="_equal" then
    return(_equal((transform::laplace)(op(_f,1), var1, var2),
            (transform::laplace)(op(_f,2), var1, var2)))
  end_if;

  if typ="_mult" or typ="_plus" then
    back:=IntTrans::linear(new((transform::laplace), _f, var1, var2));
    if not (back=FAIL) then
      return(back);
    end_if;
  end_if;

#
  if typ="_mult" and type(op(_f,1))="_power" and type(op(_f,2))="_plus" then
    if op(op(_f,1),2)<0 then
      back:=_plus(
          (transform::laplace)( op(op(_f,2), i)*op(_f, 1), var1, var2)
             $ hold(i)=1..nops(op(_f,2)));
    end_if;
    if back<>FAIL then return(back) end_if;
  end_if;
#
  if typ="function" and type(op(_f,0))="D" then
    _f:=IntTrans::D2diff(_f);
  end_if;

  back:=IntTrans::built_in(new((transform::laplace), _f, var1, var2));
  if not (back=FAIL) then
    return(back);
  end_if;

  if typ="_mult" and type(op(_f,1))="_power" and type(op(_f,2))="_plus" then
    if op(op(_f,1),2)<0 then
      back:=_plus(
          (transform::laplace)( op(op(_f,2), i)*op(_f, 1), var1, var2)
             $ hold(i)=1..nops(op(_f,2)));
    end_if;
    if back<>FAIL then
      return(back)
    end_if;
  end_if;

 return(FAIL);
end_proc:


laplace::diff:=proc() local _f, _t, nps;
begin
  nps:=args(0);
  if nps<2 then
    error("wrong number of arguments")
  end_if;
  _f:=args(1); _t:=args(2);
  if _and(_equal(_t, args(i)) $ hold(i)=2..nps,_t=extop(_f,3)) then
    return((transform::laplace)((-1*_t)^(nps-1)*extop(_f, 1),extop(_f,2),extop(_f,3) ))
  elif _and(has(_f, args(i)) $ hold(i)=2..nps) then
    return( (transform::laplace)(diff(extop(_f, 1), args(i) $ hold(i)=2..nps), 
      extop(_f, 2), extop(_f, 3)))
  else
    return(0);
  end_if;
end_proc:


laplace::int:=proc() local _F, _f, _a, _b, _t, var1, var2;
begin
  _F:=args(1); _t:=op(args(2),1); _a:=op(op(args(2),2),1); _b:=op(op(args(2),2),2);
  _f:=extop(_F,1); var1:=extop(_F,2); var2:=extop(_F,3);
  if _t=var2 and _a=0 and _b=infinity then
    return(_F/var1)
  end_if;
  hold(int)(_F, _t=_a.._b);
end_proc:

laplace
end_proc():

atan:= funcattr( atan, "transform::ilaplace",
    loadproc(funcattr( atan, "transform::ilaplace"),
        pathname("TRANS","LAPLACE"), "IL_atan"
    )
):

_mult:= funcattr( _mult, "transform::laplace",
    loadproc(funcattr( _mult, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_mult"
    )
):

_mult:= funcattr( _mult, "transform::ilaplace",
    loadproc(funcattr( _mult, "transform::ilaplace"),
        pathname("TRANS","LAPLACE"), "IL_mult"
    )
):

_power:= funcattr( _power, "transform::laplace",
    loadproc(funcattr( _power, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_power"
    )
):
_power:= funcattr( _power, "transform::ilaplace",
    loadproc(funcattr( _power, "transform::ilaplace"),
        pathname("TRANS","LAPLACE"), "IL_power"
    )
):

sin:= funcattr( sin, "transform::laplace",
    loadproc(funcattr( sin, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_sin"
    )
):

sinh:= funcattr( sinh, "transform::laplace",
    loadproc(funcattr( sinh, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_sinh"
    )
):

cos:= funcattr( cos, "transform::laplace",
    loadproc(funcattr( cos, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_cos"
    )
):

cosh:= funcattr( cosh, "transform::laplace",
    loadproc(funcattr( cosh, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_cosh"
    )
):

diff:= funcattr( diff, "transform::laplace",
    loadproc(funcattr( diff, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_diff"
    )
):

ln:= funcattr( ln, "transform::laplace",
    loadproc(funcattr( ln, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_ln"
    )
):

ln:= funcattr( ln, "transform::ilaplace",
    loadproc(funcattr( ln, "transform::ilaplace"),
        pathname("TRANS","LAPLACE"), "IL_ln"
    )
):

heaviside:= funcattr( heaviside, "transform::laplace",
    loadproc(funcattr( heaviside, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_heavis"
    )
):

exp:= funcattr( exp, "transform::laplace",
    loadproc(funcattr( exp, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_exp"
    )
):

int:= funcattr( int, "transform::laplace",
    loadproc(funcattr( int, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_int"
    )
):

dirac:= funcattr( dirac, "transform::laplace",
    loadproc(funcattr( dirac, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_dirac"
    )
):

besselJ:= funcattr( besselJ, "transform::laplace",
    loadproc(funcattr( besselJ, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_bessJ"
    )
):

abs:= funcattr( abs, "transform::laplace",
    loadproc(funcattr( abs, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_abs"
    )
):

eint:= funcattr( eint, "transform::laplace",
    loadproc(funcattr( eint, "transform::laplace"),
        pathname("TRANS","LAPLACE"), "L_eint"
    )
):
unalias( IntTrans ):

# end of file #
