#++
Series - test for series expansions

Series(Puiseux)
Series(Laurent)
Series(Taylor)	or Series(Taylor, x=x0)

Note: Series gives only a correct result, if the expression
      is generated by series!

Examples:
 
>> s:=series(sin(sqrt(x)),x): testtype(s, Type::Series(Puiseux));
 
                TRUE
 
>> s:=x+x^3*(-1/6)+x^5*1/120+O(x^6): testtype(s, Type::Series(Puiseux));
 
                FALSE

>> s:=series(1/sin(x),x): testtype(s, Type::Series(Laurent));

                TRUE

>> s:=series(sin(sqrt(x)),x): testtype(s, Type::Series(Laurent));

                FALSE

>> s:=series(sin(x),x): testtype(s, Type::Series(Taylor));
 
                TRUE
 
>> testtype(s, Type::Series(Taylor,y));

                                   FALSE

>> s:=series(1/sin(x),x): testtype(s, Type::Series(Taylor));
 
                FALSE

>> s:=series(exp(1/z),z=infinity): testtype(s, Type::Series(Taylor,z));

		FALSE

>> s:=series(exp(1/z),z=infinity): testtype(s,Type::Series(Taylor,z=infinity));

		TRUE
++#

Type::Series := proc(T) begin

    if args(0) < 1 or args(0) > 2 or
       (args(0) = 2 and args(1) <> Taylor) then
       error("wrong no of args")
    elif not contains({Puiseux, Laurent, Taylor}, T) then
       error("Unknown series expansion")
    end_if;

    new(Type, hold(Series), proc(T,x) begin
            case T[1]
              of Puiseux do Type::Puiseux(x); break;
              of Laurent do Type::Laurent(x); break;
              of Taylor  do if nops(T) = 1 then
                               Type::Taylor(x)
                            else
                               Type::Taylor(x, T[2])
                            end_if;
                            break;
            end_case
          end_proc,
        [args()], FALSE)

end_proc:

Type::Puiseux := proc(x) begin 
  bool(type(x) = Puiseux)
end_proc:

Type::Laurent := proc(x) begin
  if not Type::Puiseux(x) then
     FALSE
   else
     bool(extop(x,1)=1 and not has(extop(x,4),x::indet(x)))
   end_if
end_proc:

Type::Taylor := proc(x) begin
  if not Type::Laurent(x) then
     FALSE
  elif args(0)=2 then bool(extop(x,2) >= 0 and 
	(x::indet(x)=x::point(x))=(if type(args(2))="_equal" then args(2)
		else args(2)=0 end_if))
  else bool(extop(x,2)>=0)
  end_if
end_proc:
