
#-

References:

[1] Fractals for Windows by Tim Wegner, Bert Typler, Mark Peterson 
    and Pieter Branderhorst, The Waite Group.
[2] Voyager - Maple V and Fractals
    http://www-users.informatik.rwth-aachen.de/~afw/mplfract.html

-#

#-

The Mandelbrot set 

Originally written by Esther Gelle and modified by John Oprea, 
oprea@math.csuohio.edu

-#


mandelbrot:=proc(x, y, it)
    # it is the number of iterations #
    local z, m;
begin
    z:=float(x+y*I);
    m:=0;
    while abs(z)<2 and m < it do
       z:=z^2+(x+y*I);
       m:=m+1;
    end_while;
    m
end_proc:

#-

Julia sets

written by John Oprea, oprea@math.csuohio.edu

-#

julia_set :=proc(c, x, y, it)
   local z, m;
begin
   z:=float(x+y*I);
   m:=0;
   while abs(z)<3 and m < it do
      z:=z^2+c;
      m:=m+1;
   end_while;
   m;
end_proc:

#-

Color functions 

-#

rainbow := proc(n, m) 
  local c;
begin  
  c := n / m; # 0 <= c <= 1 #
  if c > 4/5 then
        [1, 5*(1-c), 0]
    elif c > 3/5 then
        [1-5*(4/5-c), 1, 0]
    elif c > 2/5 then
        [0, 1-5*(3/5-c), 5*(3/5-c)]
    elif c > 1/5 then
        [5*(2/5-c), 0, 1]
    else
        [1, 0, 1-5*(1/5-c)]
    end_if
end_proc:

COLORS     := RGB::ColorNames():
MAX_COLORS := nops(COLORS):
 
wild := proc(n, m)
    local c;
begin
    c := n * (MAX_COLORS div m);
    domattr(RGB, expr2text(op(COLORS, c)))
end_proc:


#- 

Choose smaller values for the number of iterations
to decrease computation time.

-#

MAX_ITER_MANDEL    := 30:
MAX_ITER_JULIA     := 30:
MAX_ITER_MANDEL_3D := 25:

#- 

Choose wild instead of rainbow to get 'wild' colors

-#

mandel := proc(x, y) 
begin 
    rainbow( mandelbrot( x, y, MAX_ITER_MANDEL) , MAX_ITER_MANDEL ) 
end_proc:

#-

Modify PHONYVAR to choose different julia sets

-#

PHONYVAR := -1.25:

julia := proc(x, y) 
begin 
    rainbow( julia_set( PHONYVAR, x, y, MAX_ITER_JULIA) , MAX_ITER_JULIA ) 
end_proc:

#- 

Plot functions

-#

plot_julia := proc(xr, yr, gr)
begin
    plot_fractal("julia", julia, xr, yr, gr)
end_proc:

plot_mandel := proc(xr, yr, gr)
begin
    plot_fractal("mandel", mandel, xr, yr, gr)
end_proc:

plot_julia1 := proc(xr, yr, gr)
begin
    plot_fractal1("julia", julia, xr, yr, gr)
end_proc:
 
plot_mandel1 := proc(xr, yr, gr) 
begin 
    plot_fractal1("mandel", mandel, xr, yr, gr) 
end_proc:
  

plot_fractal := proc(n, f, xr, yr, gr)
  local x, y;
begin
      plot3d( Title = "plot_".n."(".convert(xr, yr, gr).")",
              Axes=None, FocalPoint = [-0.65, 0, 0],
              CameraPoint = [-0.65, -0.15, 9],
              [Mode = Surface,
                   [x, y, 0], x = xr, y = yr, Grid = gr,
                   Color = [Function, f],
                   Style = [ColorPatches, Only]
                   # Style = [Points] #
              ]
      )
end_proc:

plot_fractal1 := proc(n, f, xr, yr, gr)
    local l, sx, sy, x, y;
begin
    l := [];
    sx := (xr[2]-xr[1])/gr[1];
    sy := (yr[2]-yr[1])/gr[2];
    for x from xr[1] to xr[2] step sx do
        for y from yr[1] to yr[2] step sy do
            l := append(l, point(x, y, Color = f(x,y)))
        end_for
    end_for;

    plot2d(Title = "plot_".n."1(".convert(xr, yr, gr).")",
           Axes=None, [ Mode = List, l] )
end_proc:

plot3d_mandel := proc(xr, yr, gr)
    local x, y, z;
begin
    z := func( mandelbrot(x, y, 25), x, y );

    plot3d( Title = "plot3d_mandel(".convert(xr, yr, gr).")",
            Axes=None, Scaling=UnConstrained,
            CameraPoint=[1.5,-4.0,97.0], FocalPoint=[-0.65,0.0,12.5],
            [Mode = Surface,
               [x, y, hold(z)(x,y)], x = xr, y = yr, Grid = gr,
               Style = [HiddenLine, ULine],
               Color = [Height]
            ]
    )
end_proc:

#-

Utilities

-#

convert := proc()
  local DIGITS;
begin DIGITS:=3; expr2text(args()) end_proc:
