
# SparseMatrixF2 - the domain of sparse matrices over Dom::IntegerMod(2)
  A sparse matrix is represented as a list of rows; each row is a list
  of indices representing the nonzero entries.  #

# used by numlib::mpqs #

SparseMatrixF2:=DomainConstructor(
SparseMatrixF2,
[],
# local # [F2, sparsemultloaded],
( F2:=Dom::IntegerMod(2);
  if type(sparsemult)=DOM_DOMAIN then
	sparsemultloaded:=TRUE
  else
	sparsemultloaded:=FALSE
  end_if ),
# inherits # Dom::BaseDomain,
[Cat::MatrixCat(Dom::IntegerMod(2))],
# axiom # [Ax::canonicalRep],


"coeffRing"=F2,

"isSparse"=TRUE,
  
"new"=
proc(m:DOM_INT, n:DOM_INT, body)
  local seqn,i;
begin
  case type(body)
  of DOM_LIST do
    if nops(body)<>m then
      error("Wrong number of rows")
    end_if;
    case map({op(body)}, domtype)
    of {DOM_LIST} do
      return(new(this, m, n, body))
    of {DOM_SET} do
      return(new(this, m, n, map(body, proc(x) begin [op(x)] end_proc)))
    otherwise
      error("Illegal type of operands")
    end_case
    
  otherwise # regard third argument as proc #
    seqn:=[$1..n];
    return(new(this, m, n, [select(seqn, proc(u) begin                               
                                  F2(body(i,u))= F2::one end_proc)  
                           $i=1..m]))  
  end_case;
end_proc,

"convert_to"=
proc(A:this, T:DOM_DOMAIN)
begin
  case T
  of this do
    return(A)
  of Dom::Matrix(Dom::IntegerMod(2)) do
    return(T(A::nrows(A), A::ncols(A), func(A[i,j], i, j)))
  otherwise
    FAIL
    end_case
end_proc,
  
"nrows"= func( extop(A,1),A),


"ncols"= func( extop(A,2), A),

"body"= func(extop(A,3), A),

"op"= proc(A)
   begin
   if args(0)=1 then
      op(A::body(A)) # return sequence of columns #
   else
      A::_index(A,args(2..args(0)))
   end_if
   end_proc,

"dimen"= func( [extop(A,1), extop(A,2)], A),

"row"= func(new(this, 1, extop(A,2),[(extop(A,3))[n]]),A,n),

"col"= func( new(this, extop(A,1), 1,
[ (if has((A::body(A))[i], n) then [1] else [] end_if) $i=1..A::nrows(A)]),
A, n),

"iszero"= func( bool(map({op(A::body(A))}, nops)={0}),A),

"_index"=
proc(A:this,i:DOM_INT,j)
begin
  if args(0)<2 then
    error("Wrong number of arguments")
  end_if;
  if args(0)=2 then
    if A::nrows(A)=1 then
      return(A[1,i])
    elif A::ncols(A)=1 then
      return(A[i,1])
    else
      return(A::row(A,i))
    end_if
  end_if;
  
  if has(A::body(A)[i], j) then
    F2::one
  else
    F2::zero
  end_if
end_proc,

"set_index"=
proc(A:this, i:DOM_INT, j:DOM_INT, a)
begin
  if i<1 or i>A::nrows(A) or j<1 or j>A::ncols(A) then
    error("Illegal index")
  end_if;
  case F2::convert(a)
  of FAIL do
    error("Argument could not be converted to element of coefficient ring")
  of F2::one do
    return(extsubsop(A, 3=subsop(A::body(A), i=op(A::body(A),i).[j])))
  of F2::zero do
    return(extsubsop(A, 3=subsop(A::body(A), i=op(A::body(A),i).[j])))
  otherwise
    error("Cannot handle right hand side") # should not happen #
  end_case
end_proc,
       
  
"zeroMatrix"= func( new(this, m, n, [ ([] $m)]),m,n),

"randmatrix"=
proc(m:DOM_INT, n:DOM_INT, s)
  local rndgen,i,j;
begin
  if args(0)<3 then s:=6 end_if;
  rndgen:=random(n);
  new(this, m, n, [[ rndgen()+1 $j=1..s ] $i=1..m])
end_proc,
  
"concatMatrix"=
proc(A,B)

begin
if A::nrows(A)<>B::nrows(B) then
  error("Numbers of rows differ")
end_if;
  new(this, A::nrows(A), A::ncols(A)+B::ncols(B),
      zip(A::body(A), map(B::body(B), map, _plus, A::ncols(A)), _concat))
end_proc,

"_concat"=
proc()
begin
  if args(0)=1 then args(1)
  else
    this::concatMatrix(_concat(args(1..args(0)-1)), args(args(0)))
  end_if
end_proc,
  

"stackMatrix"=
proc(A,B)

begin
  if A::ncols(A) <> B::ncols(B) then
    error("Numbers of columns differs")
  end_if;
  
  new(this, A::nrows(A)+B::nrows(B), A::ncols(A),A::body(A).B::body(B))
end_proc,

"_plus"=
proc(A,B)

begin
  if args(0)=1 then
    args(1)
  elif args(0)>2 then
    this::_plus(this::_plus(args(1..args(0) div 2)),
               this::_plus(args((args(0) div 2)+1..args(0))))
  else
    if A::nrows(A)<>B::nrows(B) or A::ncols(A)<>B::ncols(B) then
      error("Dimensions do not match")
    end_if;
    
    new(this, A::nrows(A), A::ncols(A),
        zip(A::body(A), B::body(B),  
        proc(x,y) begin [op(({op(x)} union {op(y)}) minus ({op(x)} intersect {op(y)}))] end_proc))
  end_if
end_proc,

"_negate"= id,
  
"_mult"=
proc(A,B)
begin
  case type(A)
  of this do break;
  of Dom::IntegerMod(2) do
    if iszero(A) then
      return(this::zeroMatrix(B::nrows(B), B::ncols(B)))
    else
      return(B)
    end_if
  otherwise
    return(FAIL)
  end_case;

  case type(B)
  of this do break;
  of Dom::IntegerMod(2) do
    if iszero(B) then
       return(this::zeroMatrix(A::nrows(A), A::ncols(A)))
    else
      return(A)
    end_if
  otherwise
    return(FAIL)
  end_case;
  
  if A::ncols(A)<>B::nrows(B) then
    error("Dimensions do not match")
  end_if;

  if B::ncols(B)=1 then
    this::matrixvectorproduct(A,B)
  else
    error("Not implemented")
  end_if
end_proc,



"matrixvectorproduct"=
(if sparsemultloaded then
proc(A,v)

begin
new(this, A::nrows(A), 1, sparsemult::sparsemult(A::body(A), v::body(v)))
end_proc

 else
proc(A,v)
  local bd,Abd,f;
begin
  bd:=v::body(v);
  Abd:=A::body(A);
  f:= fun(op(bd[args(1)]));
new(this, A::nrows(A), 1,
    [  (if _plus(op(map(Abd[i], f))) mod 2 = 1 then
          [1] else [] end_if) $i=1..A::nrows(A) ])  
end_proc
  end_if),



"expr"= func(A::body(A),A),


"map"=proc()
     begin
       userinfo(5, "map called with ".expr2text(args()));
       error("Not yet implemented");
     end_proc
)():  
                     
# end_domain: #

