/*  LAST EDIT: Mon Aug 19 12:59:08 1996 by Holger Naundorf (snoo)  */

/**************************************************************************
**                           MBC_bytecode.h                              **
** In diesem File werden Funktionen definiert, mit deren Hilfe eine      **
** Bytecode-Maschine von MuPAD aus angesprochen werden kann.             **
** Autor:            :      Holger Naundorf                              **
** Erstellungsdatum  :      19/08/1996                                   **
** Letzte "Anderung  :      19/08/1996                                   **
**************************************************************************/

#ifndef NO_BYTECODE


#ifndef __MBC_bytecode__
#define __MBC_bytecode__

void MBC_init () ;
void MBC_exit () ;
S_Pointer MBC_eval_new_bytecode ( S_Pointer s,
                                  long      prev_func,
                                  long      eval_type,
                                  S_Pointer exec ) ;
S_Pointer MBC_eval_apply_bytecode1 ( S_Pointer s,
                                     long      prev_func,
                                     long      eval_type,
                                     S_Pointer exec ) ;
/**************************************************************************
** class MBCTstate                                                       **
** Eine Instanz dieser Klasse wird benutzt, um den Zustand einer Byte-   **
** codemaschine zu speichern. Diese Instanz enth"alt dabei sowohl den    **
** Instructionz"ahler als auch die Stacks.                               **
**************************************************************************/
class MBCTstate
{
private:
   long InstrCounter ; // Instruction Counter

   S_Pointer ObjStack ; // Stack f"ur Objekte
   long      ObjStackP ; // Zeiger auf Top des Stacks f"ur Objekte

   S_Pointer LongStack ; // Stack f"ur longs
   long      LongStackP ; // Zeiger auf Top des Stacks f"ur longs

   S_Pointer FloatStack ; // Stacks f"ur Maschinen-Floats
   long      FloatStackP ; // Zeiger auf Top des Stacls f"ur Floats

public:
   MBCTstate() ;
   ~MBCTstate() ;

   void setInstrCounter(long ic) ; // Setzt den Instruction Counter
                                   // auf den Wert ic.
   void moveInstrCounter(long ric) ; // Ver"andert den momentanen
                           // Wert des Instruction Counters um ric,
                           // es handelt sich praktisch um einen
                           // relativen Sprung.
   long getInstrCounter() ; // Auslesen des Instruction Counters


   void pushObject(S_Pointer obj) ; // Legt das Objekt obj auf den
                          // Top des Objekt-Stacks
   S_Pointer popObject() ; // L"oscht das oberste Objekt vom Objekt-Stack
                           // und gibt es zur"uck.
   S_Pointer getObject(long i=0) ; // Gibt das i-te Objekt von der obersten
                           // Schachtel aus gez"ahlt zur"uck. Auf die
                           // oberste Schachtel wird mit i=0 zugegriffen.
                           // Das Objekt wird nicht gel"oscht.
   void rotObjects(long i) ;// Rotiert die obersten i Eintr"age des
                            // Objekt-Stacks. Das oberste Element wird das
                            // i-te Element.

 
   void pushLong(long l) ; // Legt den Long l auf den 
                          // Top des Long-Stacks 
   long popLong() ; // L"oscht den oberste Long vom Long-Stack 
                           // und gibt es zur"uck. 
   long getLong(long i =0) ; // Gibt den i-ten Long von der obersten
                           // Schachtel aus gez"ahlt zur"uck. Auf die  
                           // oberste Schachtel wird mit i=0 zugegriffen. 
                           // Der Long wird nicht gel"oscht.
   void rotLongs(long i) ;// Rotiert die obersten i Eintr"age des
                            // Long-Stacks. Das oberste Element wird das 
                            // i-te Element.
   

   void pushFloat(double f) ; // Legt den Float l auf den  
                          // Top des Float-Stacks  
   double popFloat() ; // L"oscht den oberste Float vom Float-Stack  
                           // und gibt es zur"uck.  
   double getFloat(long i =0) ; // Gibt den i-ten Float von der obersten
                           // Schachtel aus gez"ahlt zur"uck. Auf die   
                           // oberste Schachtel wird mit i=0 zugegriffen.  
                           // Der Float wird nicht gel"oscht.
   void rotFloats(long i) ;// Rotiert die obersten i Eintr"age des
                            // Float-Stacks. Das oberste Element wird das 
                            // i-te Element.
} ;





/**************************************************************************
** MBCTcode                                                              **
** Eine Instanz dieser Klasse speichert ein Bytecode-Programm und        **
** Konstanten in Form von Objekten.                                      **
**************************************************************************/ 
class MBCTcode
{
private:
   S_Pointer program ;

public:
   MBCTcode(S_Pointer code) ;
   ~MBCTcode() ;

   S_Pointer getConstObj(long i) ; // Die konstanten Objekte, die zu einem
                        // Bytecode-Programm geh"oren, sind durchnumeriert.
                        // Mit dieser Methode kann man auf die i-te
                        // Konstante zugreifen.
   unsigned char getCode(long ic) ; // Diese Methode liefert den Code, der unter
             // der Adresse ic gespeichert ist. 
   char getSignedChar(long ic) ; // Diese Methode liefert den Wert, der
             // unter der Adresse ic gespeichert ist.
   long getSignedShort(long ic) ; // Diese Methode liefert den Wert, der
             // unter der Adresse ic und ic+1 gespeichert ist. Dabei ist
             // der Wert, der unter ic gespeichert ist, der h"oherwertige.
   long getSignedLong(long ic) ; // Diese Methode liefert den Wert, der
             // unter den Adressen ic bis ic+3 gespeichert ist. Dabei ist 
             // der Wert, der unter ic gespeichert ist, der h"ohstwertige.
} ;




/**************************************************************************
** MBCTbyte                                                              **
**************************************************************************/
typedef unsigned char MBCTbyte ;


/**************************************************************************
** MBCTbytecodeFunction                                                  **
**************************************************************************/
typedef void (MBCTbytecodeFunction)(MBCTcode& code, MBCTstate *state) ;




/**************************************************************************
** MBCTbytecode                                                          **
** Eine Instanz dieser Klasse repr"asentiert und Bytecode-Wert.          **
**************************************************************************/
class MBCTbytecode
{
private:
   unsigned char bytecode ;

public:
   MBCTbytecode(unsigned char bc) ;
   MBCTbytecode() ;

   void operator()(MBCTcode& code, MBCTstate *state) const ;
                  // Der Bytecode, den *this repr"asentiert, wird ausgef"uhrt.
                  // Es wird angenommen, da"s der Instruction Counter
                  // aus *state auf diesen Bytecode in code zeigt.
   int operator!=(unsigned char bc) const ; // Liefert 1, falls bc einen anderen
                 // anderen Bytecode repr"asentiert als *this, sonst 0.

   static MBCTbytecodeFunction *CodeTable[256] ;

   ///////////////////////////////////////////////////////////////////////
   // Bytecode Memonics
   ///////////////////////////////////////////////////////////////////////
   static const MBCTbyte Quit;//= 0  Beendet des Programms
   static const MBCTbyte TransFunc1;//= 1  Beschreibt Befehl, der eine Float-Zahl vom Stack
                        // nimmt, und eine Float-Zahl erzeugt. Der Bytecode
                        // nach diesem Bytecode vervollst"andigt den Befehl.
   static const MBCTbyte DupObj;//= 2  Nimmt ein Long i vom Long-Stack.
                        // Legt eine Kopie des Objekts, das durch diesen
                        // Wert auf dem Stack indiziert wird, oben auf
                        // den Stack.
                        // Ist i positiv, so wird relativ vom Top gez"ahlt,
                        // wobei das Topelement die Nummer 0 hat.
   static const MBCTbyte DupLong;//= 3  Funktioniert wie DupObj, arbeitet allerdings auf
                        // dem Long-Stack.
   static const MBCTbyte DupFloat;//= 4  Funktioniert wie DupObj, arbeitet allerdings auf 
                        // dem Float-Stack.
   static const MBCTbyte Obj2Float;//= 5  Konvertiert das oberste Objekt des
                        // Objekt-Stacks in eine double und legt diese
                        // auf den Float-Stack.
   static const MBCTbyte Float2Obj;//= 6  Konvertiert das oberste Objekt des Float-Stacks
                        // in ein Objekt und legt es auf den Objekt-Stack.
   static const MBCTbyte Obj2Long;//= 7  Konvertiert das oberste Objekt des Objekt-Stacks
                        // in ein long und legt diesen auf den Long-Stack.
   static const MBCTbyte Long2Obj;//= 8 ;// Kontevertiert das oberste Long des Long-Stacks
                        // in ein Objekt und legt dieses auf den Objekt-Stack.
   static const MBCTbyte SetLongQ;//= 9  Legt den vorzeichenbehafteten Bytewert, der
                        // hinter dem Bytecode liegt, als long auf den
                        // Long-Stack.
   static const MBCTbyte SetLong;//= 10 Legt die n"achten 4 Byte, die hinter dem Bytecode
                        // liegen, als long auf den Long-Stack.
   static const MBCTbyte GetConst;//= 11 Legt die i-te Konstante des Programms auf den
                        // Objekt-Stack, wobei i das oberste Element des
                        // Long-Stacks ist.
   static const MBCTbyte RotObj;//= 12 Rotiert die obersten i Elemente des Objekt-Stacks,
                        // wobei i der oberste Wert des Long-Stacks ist.
                        // Danach ist das ehemals oberste Element das i-te
                        // Element des Objekt-Stacks, die anderen Objekte
                        // sind um 1 nach oben gerutscht.
   static const MBCTbyte RotLong;//= 13 Rotiert die obersten i Elemente des Long-Stacks,
                        // wobei i der oberste Wert des Long-Stacks ist,
                        // bei der Rotation allerdings nicht mehr mitgez"ahlt
                        // wird.
                        // Danach ist das ehemals oberste Element das i-te
                        // Element des Long-Stacks, die anderen longs
                        // sind um 1 nach oben gerutscht.
   static const MBCTbyte RotFloat;//= 14 Rotiert die obersten i Elemente des Float-Stacks,
                        // wobei i der oberste Wert des Long-Stacks ist. 
                        // Danach ist das ehemals oberste Element das i-te 
                        // Element des Float-Stacks, die anderen Floats
                        // sind um 1 nach oben gerutscht.
   static const MBCTbyte BranchTrue;//= 15 Falls der oberste Wert des Long-Stacks ungleich
                        // 0 ist, springt der Befehl entsprechend den beiden
                        // nachfolgenden Bytes relativ zur Adresse des
                        // Bytecodes + 1.
                        // Sonst wird bei der Bytecodeadresse + 3 mit der
                        // Ausf"uhrung fortgefahren.
   static const MBCTbyte Negate;//= 16Legt 0 auf den Long-Stapel, falls das oberste
                        // Element auf dem Long-Stapel ungleich 0 und legt
                        // 1 auf den Long-Stapel, falls das oberste
                        // Element auf dem Long-Stapel gleich 0.
   static const MBCTbyte EqLong;//= 17 Legt 1 auf den Long-Stack, wenn die beiden obersten
                        // Elemente des Long-Stacks gleich sind und legt
                        // 0 auf den Long-Stack, wenn die beiden obersten
                        // Elemente des Long-Stacks ungleich sind.
   static const MBCTbyte AddLong;//= 18 Addiert die beiden obersten Elemente des Long-Stacks
                        // und legt das Ergebnis auf den Long-Stack.
   static const MBCTbyte CheckType ;//= 19 Pr"uft, ob das Topelement des Obj-Stacks den Typ hat,
                        // der durch das zweite Element des Objekt-Stacks
                        // beschrieben wird. Legt 1 auf den Long-Stack, falls
                        // dies der Fall ist, sonst 0.
   static const MBCTbyte Eval;//= 20 Evaluiert das Topelement des Objekt-Stacks, wobei
                        // das Topelement des Long-Stacks den eval-type der
                        // Evaluation bestimmt. Das Ergebnis wird auf den
                        // Objekt-Stack gelegt.
                        // Au"serdem wird der Wert der Variablen MEVV_STATUS
                        // nach der Evaluierung auf den Long-Stack gelegt.
   static const MBCTbyte NewQ;//= 21 Das Topelement Objekt-Stacks mu"s ein Domain sein,
                        // da"s keine new_extelement-Methode enth"alt.
                        // Die n"achsten i unter dem Topelement liegenden
                        // Objekte werden benutzt, um ein neues
                        // Domain-Element zu erzeugen. Das Ergebnis wird
                        // auf den Objekt-Stack gelegt. i ist das Topelement
                        // des Long-Stacks.
   static const MBCTbyte NewCat;//= 22 Es wird ein neues CAT-Objekt erzeugt. Das
                        // Topelement des Long-Stacks bestimmt die Kategorie.
                        // Das zweite Element des Long-Stacks bestimmt die
                        // special-Bits und das dritte Element des Long-Stacks
                        // bestimmt, wieviele Elemente vom Objekt-Stack
                        // genommen und in das neue Objekt geh"angt werden.
                        // Das Ergebnis wird auf den Objekt-Stack gelegt.
   static const MBCTbyte TransFunc2;//=23 Das Byte nach diesem Bytecode
                        // definiert, welche Funktion wirklich aufgerufen
                        // wird. Die aufgerufene Funktion nimmt zwei
                        // Floats vom Float-Stack und legt einen Float
                        // auf den Float-Stack.
   static const MBCTbyte Call;//=24 Das Topelement des Longstacks bestimmt
                        // die absolute Adresse, bei der weitergearbeitet
                        // werden soll. Wird auf ein Quit-Befehl getroffen,
                        // so wird nach dem Call-Befehl weitergearbeitet.
   static const MBCTbyte CallFar;//=25 Das Topelement des Objekt-Stacks
                        // mu"s ein Bytecode-Objekt sein und das Topelement
                        // des Long-Stacks gibt eine absolute Adresse in
                        // diesem Bytecode-Objekt dar. Das Programm wird
                        // an dieser Stelle fortgef"uhrt. Bei einem Quit
                        // wird nach dem CallFar-Befehl weitergemacht.
   static const MBCTbyte Nops;// Legt die Anzahl der Operanden des Topelement
                        // Objekt-Stacks auf den Long-Stack. Eventuelle
                        // Special-Bits werden nicht ber"ucksichtigt.
  static const MBCTbyte Op;// Legt den i-ten Sohn (Z"ahlweise bei 0
                        // beginnend) des Objekt-Stacks auf den Objekt-Stack.
                        // Special-Bits werden dabei nicht ber"ucksichtigt.
                        // i ist dabei der Wert des Topelement des
                        // Long-Stacks.
   static const MBCTbyte GetSpec; // Fa"st das Topelement des Objekt-Stacks
                        // als MMM_CATEGORY-Element auf und legt den
                        // Special-Wert auf den Long-Stack.
   static const MBCTbyte GetCat;// Fa"st das Topelement des Objekt-Stacks 
                        // als MMM_CATEGORY-Element auf und legt den 
                        // CAT-Wert auf den Long-Stack.
   static const MBCTbyte PopObj; // Nimmt oberstes Element vom O-Stack.
   static const MBCTbyte PopFloat ; // Nimmt oberstes Element vom F-Stack
   static const MBCTbyte PopLong ; // Nimmt oberstes Element vom L-Stack
} ;




/**************************************************************************
***************************************************************************
**                         Implementationsteil                           **
***************************************************************************
**************************************************************************/

/**************************************************************************
**                             MBCTstate                                 **
**************************************************************************/
inline MBCTstate::MBCTstate()
{
   InstrCounter = 0 ;

   ObjStack = MMMpalloc( MMMCglobal, MMM_ARRAY, 0L, 5L ) ;
   ObjStackP = 0 ;

   LongStack = MMMmalloc( MMMCglobal, MMM_ARRAY, 5L*sizeof(long), 0L ) ;
   LongStackP = 0 ;

   FloatStack = MMMmalloc( MMMCglobal, MMM_ARRAY, 5*sizeof(double), 0L ) ;
   FloatStackP = 0 ;
}


inline MBCTstate::~MBCTstate()
{
   MMMfree ( FloatStack ) ;
   MMMfree ( LongStack ) ;
   MMMfree ( ObjStack ) ;
}


inline void MBCTstate::setInstrCounter(long ic)
{
   InstrCounter = ic ;
}


inline void MBCTstate::moveInstrCounter(long ric)
{
   InstrCounter += ric ;
}



inline long MBCTstate::getInstrCounter()
{
   return InstrCounter ;
}


inline void MBCTstate::pushObject(S_Pointer obj)
{
   if ( MMMcounter(ObjStack) <= (unsigned long) ObjStackP )
      MMMnewcounter( &ObjStack, ObjStackP + 5 ) ;

   MMMset( MMMP( &ObjStack, ObjStackP, MMMwrite ), obj ) ;
   MMMinactiv ( &ObjStack ) ;
   ObjStackP++ ;
}


inline S_Pointer MBCTstate::popObject()
{
   ObjStackP-- ;
   S_Pointer s = *MMMP( &ObjStack, ObjStackP, MMMread ) ;
   *MMMP( &ObjStack, ObjStackP, MMMwrite ) = MMMNULL ;
   MMMinactiv ( &ObjStack ) ;
   return s ;
}


inline S_Pointer MBCTstate::getObject(long i)
{
   S_Pointer s = *MMMP( &ObjStack, ObjStackP-i-1, MMMread ) ;
   MMMinactiv ( &ObjStack ) ; 
   return s ; 
}


inline void MBCTstate::rotObjects(long i)
{
   long last = ObjStackP - i - 1 ;
   S_Pointer top = *MMMP( &ObjStack, ObjStackP-1, MMMread ) ;
   long j=ObjStackP-1 ;
   for ( ; j > last ; j-- )
      {
         *MMMP( &ObjStack, j, MMMwrite ) = *MMMP( &ObjStack, j-1, MMMread ) ;
      }
   *MMMP( &ObjStack, j, MMMwrite ) = top ;
   MMMinactiv ( &ObjStack ) ;
}


inline void MBCTstate::pushLong(long l)
{
   if ( MMMsize(LongStack) <= LongStackP*sizeof(long) )
      MMMresize( &LongStack, (LongStackP+5)*sizeof(long) ) ;

   *MMMarr( &LongStack, LongStackP, long, MMMwrite ) = l ;
   MMMinactiv( &LongStack ) ;
   LongStackP++ ;
}


inline long MBCTstate::popLong()
{
   LongStackP-- ;
   long r = *MMMarr( &LongStack, LongStackP, long, MMMread ) ;
   MMMinactiv( &LongStack ) ;
   return r ;
}


inline long MBCTstate::getLong(long i)
{
   long r = *MMMarr( &LongStack, LongStackP-i-1, long, MMMread ) ;
   MMMinactiv( &LongStack ) ; 
   return r ; 
}


inline void MBCTstate::rotLongs(long i)
{
   long last = LongStackP - i - 1 ; 
   long top = *MMMarr( &LongStack, LongStackP-1, long, MMMread ) ; 
   long j=LongStackP-1 ;
   for ( ; j > last ; j-- )
      { 
         *MMMarr( &LongStack, j, long, MMMwrite ) = *MMMarr( &LongStack, j-1, long, MMMread ) ; 
      } 
   *MMMarr( &LongStack, j, long, MMMwrite ) = top ; 
   MMMinactiv ( &LongStack ) ; 
}


inline void MBCTstate::pushFloat(double l)
{
   if ( MMMsize(FloatStack) <= FloatStackP*sizeof(double) )
      MMMresize( &FloatStack, (FloatStackP+5)*sizeof(double) ) ;
    
   *MMMarr( &FloatStack, FloatStackP, double, MMMwrite ) = l ; 
   MMMinactiv( &FloatStack ) ;
   FloatStackP++ ; 
} 
 
 
inline double MBCTstate::popFloat() 
{ 
   FloatStackP-- ; 
   double r = *MMMarr( &FloatStack, FloatStackP, double, MMMread ) ; 
   MMMinactiv( &FloatStack ) ; 
   return r ; 
} 
 
 
inline double MBCTstate::getFloat(long i) 
{ 
   double r = *MMMarr( &FloatStack, FloatStackP-i-1, double, MMMread ) ; 
   MMMinactiv( &FloatStack ) ;  
   return r ;  
}


inline void MBCTstate::rotFloats(long i) 
{
   long last = FloatStackP - i - 1 ;  
   double top = *MMMarr( &FloatStack, FloatStackP-1, double, MMMread ) ;  
   long j=FloatStackP-1 ;
   for ( ; j > last ; j-- ) 
      {  
         *MMMarr( &FloatStack, j, double, MMMwrite ) = *MMMarr( &FloatStack, j-1, double, MMMread ) ; 
      } 
   *MMMarr( &FloatStack, j, double, MMMwrite ) = top ;  
   MMMinactiv ( &FloatStack ) ;  
}




/**************************************************************************
**                             MBCTcode                                  **
**************************************************************************/

inline MBCTcode::MBCTcode(S_Pointer code)
{
   MMMset( &program, code ) ;
}

inline MBCTcode::~MBCTcode()
{
   MMMfree ( program ) ;
}


inline S_Pointer MBCTcode::getConstObj(long i)
{
   S_Pointer s = *MMMP( &program, i, MMMread ) ;
   MMMinactiv ( &program ) ;
   return s ;
}


inline unsigned char MBCTcode::getCode(long ic)
{
   unsigned char c = *MMMarr( &program, ic, unsigned char, MMMread ) ;
   MMMinactiv ( &program ) ; 
   return c ; 
}


inline char MBCTcode::getSignedChar(long ic)
{
   char c = *MMMarr( &program, ic, char, MMMread ) ;
   MMMinactiv ( &program ) ; 
   return c ; 
}

inline long MBCTcode::getSignedShort(long ic)
{
   long c = *MMMarr( &program, ic, char, MMMread )*256
          + *MMMarr( &program, ic+1, unsigned char, MMMread ) ;
   MMMinactiv ( &program ) ;
   return c ;  
}

inline long MBCTcode::getSignedLong(long ic)
{ 
   long c = *MMMarr( &program, ic, char, MMMread ) * 167772162 // 2^24
          + *MMMarr( &program, ic + 1, unsigned char, MMMread ) * 65536 // 2^16
          + *MMMarr( &program, ic + 2, unsigned char, MMMread ) * 256 // 2^8
          + *MMMarr( &program, ic + 3, unsigned char, MMMread ) ;
   MMMinactiv ( &program ) ; 
   return c ;   
}

/**************************************************************************
**                              MBCTbytecode                             **
**************************************************************************/

inline MBCTbytecode::MBCTbytecode()
{
}


inline MBCTbytecode::MBCTbytecode(unsigned char bc)
{
   bytecode = bc ;
}


inline void MBCTbytecode::operator()(MBCTcode& code, MBCTstate *state) const
{
   (*CodeTable[bytecode])(code, state) ;
}


inline int MBCTbytecode::operator!=(unsigned char bc) const
{
   return (bytecode != bc) ;
}

#endif /* __MBC_bytecode__ */

#endif /* NO_BYTECODE */
