/* $Date: 1997/09/02 07:48:52 $  $Author: andi $  $Revision: 1.42 $ */

/**************************************************************************/
/* FILE:             MCA_calc.h                                           */
/* Autor:            Oliver Kluge                                         */
/* Erstellt:         06/04/91                                             */ 
/* Letzte Aenderung: 03/03/97                                             */ 
/*                                                                        */
/* Ab Revision 1.28 (24.10.96) gibt es das Preprocessor-Flag NO_SPEC_BITS */
/* Hiermit werden die Abfragen auf die Special-Bits MINUS_BIT, DIV_BIT    */
/* und nicht mehr durchgef"uhrt, da davon ausgegangen wird, das keine     */
/* Funktion, insbesondere der Parser und der Simplifier, mehr solche Bits */
/* generieren.  (Oliver Kluge)                                            */
/* Ab Revision 1.38 (3.3.97) gibt es nur noch die Version ohne Special-   */
/* Bits. Das Flag NO_SPEC_BITS wurde entfernt.                            */
/**************************************************************************/

#ifndef __MCA_calc__
#define __MCA_calc__

#define MuPAD                    /* Fuer korrekte PARI-Adaption */
#include "genpari.h"             /* Diese Header - Datei immer zuerst ! */

#undef MIOPprint_string            /*** Werden in PARI definiert ***/
#undef MIOPprint_char
#undef MEVPset_error

#ifndef BYTES_IN_LONG              /*** PARI 1.37 <--> PARI 1.39 ***/
#  define OLD_PARI
#  define LONG_IS_32BIT
#endif

/* Folgende Definitionen gehoeren zur Environment-Variable ROUNDING */

#define MCAC_ToNearest       "ToNearest"
#define MCAC_ToZero          "ToZero"
#define MCAC_ToInfinity      "ToInfinity"
#define MCAC_ToMinusInfinity "ToMinusInfinity"
 
#define MCAC_ROUNDING_DEFAULT MCAC_ToNearest


/*      Signifikante Stellenanzahl in Gleitkommazahlen         */
/*      ----------------------------------------------         */
/* MCAV_DIGITS     ist die Stellenanzahl bzgl. Zahlen, die zur */
/*                 Basis 2^32 dargestellt werden.              */
/* MCAV_DEC_DIGITS ist die Anzahl der Dezimalstellen, also     */
/*                 die Stellenanzahl bzgl. Zahlen, die zur     */
/*                 Basis 10 dargestellt werden.                */

#define MCAV_DIGITS 		prec
#define MCAV_DEC_DIGITS_DEFAULT 10L

#ifdef LONG_IS_32BIT
#  define MCAC_MAX_PRIMELIMIT          2147483390L
#elif LONG_IS_64BIT
#  define MCAC_MAX_PRIMELIMIT 9223372036854775550L
#endif

#define MCAC_MINUS	 0
#define MCAC_DIV  	 1
#define MCAC_GCD         2
#define MCAC_LCM         3
#define MCAC_BINOMIAL    4
#define MCAC_PLUS       _PLUS	/* Diese m"ussen identisch mit 		*/
#define MCAC_MULT       _MULT	/* MPAC_PLUS, MPAC_MULT, MPAC_POWER	*/
#define MCAC_POWER      _POWER_	/* sein !				*/
#define MCAC_MOD	_MOD
#define MCAC_MODS	_MODS
#define MCAC_DIVG	_DIV

extern long MCAV_STACKSIZE;
extern long MCAV_PRIMELIMIT;
extern char MCAV_string[];
 
void MCA_init(VOID);
void MCA_exit(VOID);

void MCA_set_lock(long stackpointer);

GEN MCA_cat_to_pari(S_Pointer num);

GEN MCA_set_prec(GEN g);

S_Pointer MCA_calc_gen(S_Pointer num1, S_Pointer num2, S_Pointer *num3, short op);
 
S_Pointer MCA_init_cat_pari(GEN s, S_Pointer tmp, long reset_avma);

S_Pointer MCA_plus_minus(S_Pointer *num1, long c);

#ifdef OLD_PARI
S_Pointer MCA_pari_vector_to_cat_list(GEN p, long count, long sign);
#else
S_Pointer MCA_pari_vector_to_cat_list(GEN p, long sign);
#endif

S_Pointer MCA_pari_matrix_to_cat_list(GEN m);

S_Pointer MCA_recompute_pi(S_Pointer value);

S_Pointer MCA_recompute_e(S_Pointer value);

S_Pointer MCA_recompute_euler(S_Pointer value);

S_Pointer MCA_abs(S_Pointer arg);

S_Pointer MCA_issqr(S_Pointer arg);

MUPbool MCA_is_real_extension(S_Pointer arg);

MUPbool MCA_normal_rational(GEN b, GEN e, GEN *n, GEN *c);

void MCA_set_digits(long value);

S_Pointer MCA_float(S_Pointer number);

long MCA_is_abs_half(S_Pointer s);

void MCA_shift_adresses( GEN g, long typ, long offset );

#define MCA_validate_adresses(gen)                          \
{                                                           \
	long t;                                             \
        if ((t=typ(gen)) == CAT_RAT || t == CAT_COMPLEX) {  \
           long offset;                                     \
           if ( (offset = (long) &gen[3] - gen[1]) )        \
              MCA_shift_adresses(gen, t, offset);           \
         }                                                  \
}




/*****************************************************************/
/* Setzen des positiven Vorzeichen im PARI - Objekt 'z'          */
/* Parameter: 'z' -- PARI Obkekt (integer,rational,real,complex) */
/*****************************************************************/

#define MCA_set_no_sign_pari(z) \
{ GEN t; if (typ(z) == 6) { t = (GEN) z[2]; if (signe(t)) setsigne(t, 1L);} t = ((typ(z) == 4 || typ(z) == 6) ? (GEN) z[1] : z); if (signe(t)) setsigne(t, 1L);}

#define MCA_calc(num1, num2, op) \
	MCA_calc_gen(num1, num2, (S_Pointer *) MMMNULL, op)

#define MCA_calc_1(num1, num2, op) \
	MCA_calc_gen(num1, num2, &num1, op)

#define MCA_calc_2(num1, num2, op) \
	MCA_calc_gen(num1, num2, &num2, op)

/* L"oscht den PARI-Stack nachdem der S_Pointer erzeugt wurde */
#define MCA_new_cat_pari(type, s) \
        MCA_init_cat_pari(s, MTR_new_category(type,1L), 1L)

#define MCA_pari_to_cat(s) \
        MCA_init_cat_pari(s, MTR_new_category(VALUE_READ(MCAV_STORAGE),1L), 1L)

/* PARI-Stack bleibt nach der Erzeugung des S_Pointers unver"andert */
#define MCA_new_cat_pari0(type, s) \
        MCA_init_cat_pari(s, MTR_new_category(type,1L), 0L)

#define MCA_pari_to_cat0(s) \
        MCA_init_cat_pari(s, MTR_new_category(VALUE_READ(MCAV_STORAGE),1L), 0L)

#define MCA_set_long(c) \
	MCA_new_cat_pari(VALUE_READ(MCAV_STORAGE), stoi(c))

#define MCA_cat_to_int(c) \
	gtolong(MCA_cat_to_pari(c))

#define MCA_cat_to_double(c) \
	gtodouble(MCA_cat_to_pari(c))

#define MCA_int_to_string(c) \
	(sprintf(MCAV_string, "%ld", MCA_cat_to_int(c)), MCAV_string)
	/* Folgendes geht auf dem Mac und unter System V nicht:
	(char *) sprintf(MCAV_string, "%lu", MCA_cat_to_int(c)) */

#define MCA_double_to_pari(c) \
	dbltor(c)

#define MCA_eq(a, b) \
	(long) gegal(MCA_cat_to_pari(a), MCA_cat_to_pari(b))

#define MCA_neq(a, b) \
	(long) (!gegal(MCA_cat_to_pari(a), MCA_cat_to_pari(b)))

#define MCA_eq0(a)   \
	(long) gcmp0(MCA_cat_to_pari(a))

#define MCA_eq1(a) \
	(long) gcmp1(MCA_cat_to_pari(a))

#define MCA_eq_1(a) \
	(long) gcmp_1(MCA_cat_to_pari(a))

#define MCA_lt(a, b) \
	(long) (gcmp(MCA_cat_to_pari(a), MCA_cat_to_pari(b)) < 0)

#define MCA_le(a, b) \
	(long) (gcmp(MCA_cat_to_pari(a), MCA_cat_to_pari(b)) <= 0)

#define MCA_ge(a, b) \
	(long) (gcmp(MCA_cat_to_pari(a), MCA_cat_to_pari(b)) >= 0)

#define MCA_gt(a, b) \
	(long) (gcmp(MCA_cat_to_pari(a), MCA_cat_to_pari(b)) > 0)

#define MCA_pp(a) \
	MCA_plus_minus(&a, 1L)

#define MCA_mm(a) \
	MCA_plus_minus(&a, -1L)

#define MCA_negate(a) \
	MCA_new_cat_pari(VALUE_READ(MCAV_STORAGE), gneg(MCA_cat_to_pari(a)))

#define MCA_reciprocal(a) \
	MCA_new_cat_pari(VALUE_READ(MCAV_STORAGE), ginv(MCA_set_prec(MCA_cat_to_pari((a)))))

#define MCA_ident_is_E(ident) \
        *MMMP(&ident, 0L, MMMread) == VALUE_READ(MSTR_E)

/* Liefert -1L, 0L bzw. 1L, falls die PARI-Zahl g < 0,     */
/* g == 0 bzw. g > 0 ist. g darf keine komplexe Zahl sein. */

#define MCA_get_sign(g) \
        ((typ(g) == CAT_RAT) ? signe((GEN)g[1]) : signe(g))

/* Liefert _true_, falls S_Pointer a negativ ist. a muss   */
/* vom Typ CAT_INT, CAT_APM, CAT_RAT oder  CAT_FLOAT sein. */

MUPbool MCA_is_neg( S_Pointer a );

/*************************************************************************
Vorsicht: MMMP(&ident, 0, MMMread) ist wesentlich schneller, als die
========= eigentlich zu verwendende Funktion MNS_ident_name(ident). Bei
          "Anderungen des Zugriffs auf Bezeichner, verliert dieses Makro
          seine G"ultigkeit. Desweiteren wird davon ausgegangen, dass
          Bezeichner vom Typ MMM_STRING der "unique data representation"
          unterliegen, und daher ein Stringvergleich nicht notwendig ist,
          sondern ein Pointer-Vergleich ausreicht. Die "sichere" Funktion
          lautet:

MUPbool  
MCA_ident_is_E(S_Pointer ident) 
{
        S_Pointer str = MNS_ident_name(ident);
        int r = strcmp(MMMmvarr(&str, sizeof(STRSIZE), 0L, char, MMMread), "E");        
        MMMfree(str);
        return(r == 0 ? _true_ : _false_);
}

***************************************************************************/

/************************************************************************/
/* NAME:        MCA_complex_re                                          */
/* PARAMETER:   s = Komplexe Zahl (CAT_COMPLEX)                         */
/* FUNKTION:    Liefert die Kopie des Realteils eines CAT_COMPLEX.      */
/************************************************************************/
#define MCA_complex_re(_s_) \
	MCA_new_cat_pari(VALUE_READ(MCAV_STORAGE), ((GEN)gmul((GEN) MCA_cat_to_pari(_s_)[1],gun)))
        /** Ich multipliziere mit '1',  weil ich nicht weiss wie ich das **/
        /** PARI-Objekt physikalisch kopieren kann. Wird bald geaendert. **/
        
/************************************************************************/
/* NAME:        MCA_complex_im                                          */
/* PARAMETER:   s = Komplexe Zahl (CAT_COMPLEX)                         */
/* FUNKTION:    Liefert die Kopie des Imaginaerteils eines CAT_COMPLEX. */
/************************************************************************/
#define MCA_complex_im(_s_) \
        MCA_new_cat_pari(VALUE_READ(MCAV_STORAGE), ((GEN)gmul((GEN) MCA_cat_to_pari(_s_)[2],gun)))
        /** Ich multipliziere mit '1',  weil ich nicht weiss wie ich das **/
        /** PARI-Objekt physikalisch kopieren kann. Wird bald geaendert. **/

#define MCA_print_statistics() \
        etatpile(0)


/*** HIGHBIT wird in PARI-1.39 in 'gencom.h' in Abhaengigkeit von der Laenge ***/
/*** eines 'long' definiert.  Im Falle von OLD_PARI werden nur 32bit Systeme ***/
/*** unterstuetzt.                                                           ***/

#ifdef OLD_PARI
#  define HIGHBIT     0x80000000
#endif

#define MCA_is_int(g) \
	(typ(g) == CAT_APM && \
	    ((lgef(g) == 3L && (unsigned long)g[2] < HIGHBIT) || \
	     (lgef(g) == 2L)))

/******************  Signaturberechnung f"ur PARI - Zahlen ************************/

/*** MCA_pari_header: Blendet stoerende Daten wie Referenzzaehler aus damit ***/
/***                  der Header zur Signaturberechnung genutzt wreden kann ***/
/***                  Achtung dies mu unabhangig von der Laenge eines long ***/
/***                  funktionieren (32bit, 64bit, ...) !!!                 ***/


#if   defined LONG_IS_32BIT || defined OLD_PARI
#     define MCA_pari_header(g) (((unsigned long)g[0]) & 0x3f000000)
#elif defined LONG_IS_64BIT
#     define MCA_pari_header(g) (((unsigned long)g[0]) & 0x3fff000000000000)
#else
    /*** Bleibt frei zur schnellen Fehlererkennung !!! ***/
#endif

#define MCA_sig_int(g) \
((((signe(g) ? (g[2] + MCA_pari_header(g)) : MCA_pari_header(g))) + lgef(g)))

#define MCA_sig_rat(g) \
((((MCA_sig_int(((GEN) g[1])) + MCA_sig_int(((GEN) g[2])))) + MCA_pari_header(g)))


#define MCA_sig_float(g) \
        MCA_pari_header(g)

/* #define MCA_sig_float(g) \
	MCA_sig_int(g) */

#define MCA_sig_complex(g) \
((((MCA_sig_float(((GEN) g[1])) + MCA_sig_float(((GEN) g[2])))) + MCA_pari_header(g)))

/******************** Signaturhauptroutine ****************************************/

   /*** Teuer falls der Compiler ALLE Signaturen berechnet. Dies                ***/
   /*** ist nach K&R-C durchaus mglich! (andi, 14.08.95)                       ***/

#define MCA_sigp(g)                                        \
	( (typ(g) == CAT_APM) ? MCA_sig_int(g) :           \
	    ( (typ(g) == CAT_RAT) ? MCA_sig_rat(g) :       \
	      ( (typ(g) == CAT_FLOAT) ? MCA_sig_float(g) : \
		MCA_sig_complex(g)                         \
	      )                                            \
	    )                                              \
        )

/***********************************************************************

Werden alle PARI-Objekte, die mittels MMMforce_data in MuPAD-Objekte
umgewandelt werden mit gclone kopiert, so muss diese Version von
MCA_free_pari() genommen werden

#define MCA_free_pari(s)

#define MCA_free_pari(s) \
	{ (void) killbloc(MMMZ(&s, long, MMMread)); MMMinactiv(&s); }

*************************************************************************/

#define MCA_free_pari(s)	MMMfree(s)

#undef un       /* PARI-Definition r"uckg"angig machen, */
                /* um Konflikte zu vermeiden.           */


/************************************************************************

	WICHTIG:      

	Es muss immer gelten:
	CAT_APM      = PARI - Integer - Typ                 (derzeit 1)
	CAT_RAT      = PARI - Rational (irreducible) - Typ  (   "    4)
	CAT_FLOAT    = PARI - Real - Typ                    (   "    2)
	CAT_COMPLEXT = PARI - Complex - Typ                 (   "    6)

**************************************************************************/

/************************************************************************

S_Pointer MCA_calc(S_Pointer num1, num2; short op)

	Berechnet num1 op num2, wobei 

	* num1, num2 Kategorien vom Typ (*) und
	* op vom Typ (+) seien muessen.

	Ergebnis ist wiederum eine Kategorie vom Typ (*)


S_Pointer MCA_new_cat_pari(long typ; GEN z)

	Erstellt zu dem PARI - Objekt z die zugehoerige Kategory.
	Ergebnis ist eine Kategorie vom Typ (*).

S_Pointer MCAC_ci

	Kategorie vom Typ CAT_COMPLEX.
	Enthaelt die Konstante I = sqrt(-1).

long MCA_cmp(S_Pointer num1, num2)

	num1, num2 muessen Kategorien vom Typ (*) sein.
	Liefert 1, falls num1 == num2, 0 sonst.

long MCA_cmp0(S_Pointer num1)

	num1 muss Kategorie vom Typ (*) sein. 
	Liefert 1, falls num1 == 0, 0 sonst.

long MCA_cmp1(S_Pointer num1)
 
        num1 muss Kategorie vom Typ (*) sein. 
        Liefert 1, falls num1 == 1, 0 sonst.

long MCA_cmp_1(S_Pointer num1)
 
        num1 muss Kategorie vom Typ (*) sein. 
        Liefert 1, falls num1 == -1, 0 sonst.

void MCA_free_pari(S_Pointer num1)

	num1 muss Kategorie vom Typ MMM_PARI sein.
	Loescht num1.

(*) CAT_INT, CAT_APM, CAT_RAT, CAT_FLOAT, CAT_COMPLEX

(+) MCAC_PLUS, MCAC_MINUS, MCAC_MULT, MCAC_DIV, MCAC_DIVG,
    MCAC_MOD, MCAC_POWER, MCAC_GCD, MCAC_LCM, MCAC_BINOMIAL

====================================================================

(1) Signatur eines PARI-Objektes setzt sich zunaechst aus dem 1. Codeword
    des Objektes zusammen. Dies gewaehrleistet:
 
    * Objekte unterschiedlichen Typs haben verschiedene Signaturen.
    * Objekte gleichen Typs, aber unterschiedlicher Laenge haben
      verschiedene Signaturen.
    * Gleiche Objekte mit unterschiedlichem Vorzeichen haben
      dieselbe Signatur.

    Ist das Objekt ungleich 0, so wird in Byte 2 des 1. Codewords, das
    4. Byte der eigentlichen Zahl geschrieben. Dies gewaehrleistet:

    * Objekte gleichen Typs und gleicher Laenger, deren numerischen
      Werte sich jedoch unterscheiden, haben unterschiedliche Signaturen.

    F"ur die Signatur eines PARI-Objekt x gilt: 
			0 < sig(x) < 2 ^ 30

    BEACHTE:
    Voraussetzung an die Funktion MCA_sigp, die die Signatur berechnet,
    ist, dass der Datentyp long eine 32 - Bit Zahl speichern kann.


(2) Funktionen, die ausserhalb des Moduls MCA_calc.c definiert werden:

    #define MPA_string_to_number(s) \
       	    MCA_new_cat_pari(MPAC_STORAGE, gclone(lisexpr(s)))


    void
    MSG_sig_pari(s)
    **************************************************************************
    * Diese Routine berechnet die Signatur eines Objektes vom Typ MMM_PARI   *
    * Pararmeter:   s  --  S_Pointer auf MMM_PARI                            *
    **************************************************************************
    S_Pointer s;
    {
            GEN g = MMMZ(&s, long, MMMread);
    
            MMMnewsignature(s, MCA_sigp(g));

            MMMinactiv(&s);
    }

    long
    MCM_PARI_equal(s, t)
    **************************************************************************
    * Diese Routine vergleicht zwei Objekte vom Typ MMM_PARI. Liefert 1 bei  *
    * Gleichheit, sonst 0.                                                   *
    * Pararmeter:   s, t  --  S_Pointer auf MMM_PARI                         *
    **************************************************************************
    S_Pointer s, t;
    {
	return((long) gegal(MMMZ(&s, long, MMMread), MMMZ(&t, long, MMMread)));
    }

************************************************************************/

#endif /* __MCA_calc__ */

