#ifndef __MCT_iostream__
#define __MCT_iostream__


#include <stdio.h>
#ifndef MSDOS
#include <unistd.h>
#endif

#ifdef NEWCOMMUNICATION
#  ifdef DOS_FILENAMES
#    include "MPR_prot.h"
#  else
#    include "MPR_protocol.h"
#  endif
#endif

#include "MCT_tarfile.h"

/*
 * MCLT_bytestream -- abstrakte Basisklasse fuer Streams, die zur Zeit
 *	nur fuer MSV_save verwendet wird, damit MSV_save mit Windows-
 *	Bytestreams arbeiten kann. Windows-Bytestreams werden von
 *	MCLT_bytestream abgeleitet und intern mit CArchive implementiert.
 */
#define MCLC_undefined  0


class MCLT_bytestream
{
protected:
	long		itsID;
	short		itsType;
public:
	MCLT_bytestream();
	virtual ~MCLT_bytestream() { itsType = MCLC_undefined; };
	virtual int	io_fgetc() = 0;
	virtual int	io_fputc(int c) = 0;
	long	        get_id() { return itsID; }

	short	GetType()		{ return(itsType); }
	void    SetType(short aType)		{ itsType = aType; }
};



#if (! defined MSDOS) && (defined NEWCOMMUNICATION)
/* 
 * MCLT_MPR_stream -- diese Implementation von MCLT_bytestream
 *      setzt auf dem MPR-Protokoll auf.
 */
class MCLT_MPR_stream : public MCLT_bytestream
{
public:
        MPRTchannel     itsChannel;
        MCLT_MPR_stream(MPRTchannel  ch) ;
        MCLT_MPR_stream() ;
        int             io_fflush();
	int		io_fgetc();
	int		io_fputc(int c);
};

#endif // (! defined MSDOS) && (defined NEWCOMMUNICATION)


enum
{		
		MCLC_io_file = 1,
		MCLC_io_tar
};


class MCLT_iostream : public MCLT_bytestream
{
public:
	FILE			*itsFile;
	MCLT_TarFile	*itsTarFile;
	MCLT_FP			itsVFile;

		// construction / destruction
	MCLT_iostream();
	MCLT_iostream(FILE *fp);
	virtual ~MCLT_iostream();

		// member access
	void	SetFile(FILE *fp)	{ itsFile = fp; SetType(MCLC_io_file); }

		// open/close
	int		io_fopen( char *URL, char *permission);
	int		io_fclose();

		// status information
	int		io_fseek( long pos, int fromWhere );
	void	io_rewind();
	int		io_fflush();
	int		io_isopen();
	void	io_reset();

	long 	io_flength();

		// read/write
	char	*io_fgets( char *buffer, long numChars );
	int		io_fputs( char *buffer );
	int		io_fgetc();
	int		io_fputc(int c);
	size_t  io_fread(void *ptr, size_t size, size_t nitems);
	size_t  io_fwrite(const void *ptr, size_t size, size_t nitems);
	int		io_fprintf( const char *formatString, ...);
	int		io_fscanf( const char *formatString, void *a1);
	int		io_fscanf( const char *formatString, void *a1, void *a2);
	int		io_fscanf( const char *formatString, void *a1, void *a2, void *a3);
	int		io_fscanf( const char *formatString, void *a1, void *a2, void *a3, void *a4);
	int		io_fscanf( const char *formatString, void *a1, void *a2, void *a3, void *a4, void *a5);

		// type conversion
	operator	FILE*()												{ return( GetType() == MCLC_io_file? itsFile : (FILE *) NULL ); }
	int	operator==( MCLT_iostream &s );	
	int	operator!=( MCLT_iostream &s );

	friend int	operator==( FILE *fp , MCLT_iostream s) { 
	    return( s.GetType() == MCLC_io_file ? fp == s.itsFile : 0 );  
	}	
	friend int	operator==( MCLT_iostream s, FILE *fp ) { 
	    return( s.GetType() == MCLC_io_file ? fp == s.itsFile : 0 );  
	}	
	friend int	operator!=( FILE *fp , MCLT_iostream s) { 
	    return( s.GetType() == MCLC_io_file ? fp != s.itsFile : 1 );
	}	
	friend int	operator!=( MCLT_iostream s, FILE *fp ) { 
	    return( s.GetType() == MCLC_io_file ? fp != s.itsFile : 1 );
	}	
	
		// diagnostics and debugging
	int			ScanURL(const char *URL );
	int			ScanTarURL(const char *URL, int *prefixLen, char **postFix );

};

#endif
