/* $Date: 1997/10/20 12:25:17 $  $Author: olli $  $Revision: 1.27 $ */

/**************************************************************************/ 
/* FILE:             MDB_debug.h                                          */
/* Autor:            Oliver Kluge                                         */
/* Erstellt:         19/12/91                                             */ 
/* Letzte Aenderung: 21/10/93                                             */  
/**************************************************************************/

#ifndef __MDB_debug__
#define __MDB_debug__


#undef DEBUG_ON
#ifdef DBX
#define DEBUG_ON
#else
#ifdef MDX
#define DEBUG_ON
#endif
#endif

/*********************************************************************/
/* Ist der Trace, Debug oder Verbose-Modus eingeschaltet, so ist auf */
/* jeden Fall Bit 1 gesetzt. Ist keiner der Modi aktiviert, so ist   */
/* MDBV_debug_activ gleich 0.                                        */
/*********************************************************************/

extern long MDBV_debug_activ;

#ifdef DEBUG_ON

#ifdef DOS_FILENAMES
#  include "MDI_face.h"
#else
#  include "MDI_debug_interface.h"
#endif


#define SCAN_ERROR	'0'
#define IDENT		'1'
#define NUMBER		'2'
#define STRING		'3'
#define END		'4'

#define OFF		 0
#define ON 		 1


/****************************************************************/
/* Z.Z. wird davon ausgegangen, dass ein long 32 Bit ist, d.h.  */
/* BITS_IN_LONG = 32                                            */
/****************************************************************/

#define DBX_TYPE unsigned long

#define MDX_VAR 	"__MDX__"



/* Es muss gelten:                                              */
/*                                                              */ 
/* (1) MAX_FILES = 2 ^ n , 0 < n < BITS_IN_LONG - 1             */
/*                                                              */
/* (2) SHIFT     = n + 1                                        */ 
/*                                                              */ 
/* (3) MASK      = hex(MAX_FILES - 1)                           */


/*****************************************************************/
/* STOP_LIST_SIZE, PROC_LIST_SIZE, DISPLAY_LIST_SIZE, STACK_SIZE */
/* m"ussen > 0 sein.                                             */
/*****************************************************************/

#define STOP_LIST_SIZE	  50	/* Max. Anzahl von Stop-Marken	*/

#define PROC_LIST_SIZE   512	/* Max. Anzahl von Prozeduren	*/

#define DISPLAY_LIST_SIZE 50	/* Max. Anzahl von Display-Var.	*/

#define STACK_SIZE	 256	/* Max. Rekursionstiefe		*/

#define MAX_FILES	1024	/* Max. Anzahl von Dateinamen	*/

#define SHIFT             11 
 
#define MASK            0x000003ff

#define MAX_INFO	  10    /* Max. Anzahl der rekursiven   */
                                /* Aufrufen von add_debug_info  */

#ifdef USE_PRAGMA
#pragma GLOBAL_PRAGMA( MCLT_iostream , MDBV_covfile )
#endif
GLOBAL_DECL( MCLT_iostream , MDBV_covfile );



#define MDB_make_debug_info(line, name, valid) \
	(line << SHIFT | (MDB_get_key(name) << 1) | valid)


#define MDB_get_line_num(dbx) \
	(dbx >> SHIFT)

#define MDB_get_filekey(dbx) \
	((dbx >> 1) & MASK)

#define MDB_valid(dbx) \
	((dbx & 1) && (dbx >> SHIFT))

/* Debug-Modus eingeschaltet ? */

#define MDB_debug() \
	((MDBV_debug_activ & 17) == 17)

/* Trace-Modus eingeschaltet ? */

#define MDB_trace() \
	(MDBV_debug_activ & 2)

/* Verbose-Modus eingeschaltet ? */

#define MDB_verbose() \
	(MDBV_debug_activ & 4)

/* Wird Debugger "uber das Frontend gesteuert ? */

#define MDB_connected() \
        (MDBV_debug_activ & 8)

#define MDB_gcov() \
	(MDBV_covfile.io_isopen())

#define MDB_create_info() \
MDB_create_debug_info(MDB_make_debug_info(MPAV_LINE_NR, MPAV_FILE_NAME, 1))

#define MDB_set_last_info(db_info) \
	{ if (db_info != MMMNULL) last_dbx = MDB_get_info(db_info); }

#ifdef MMMDLEAN

/* Die im Debugger-Stack gehaltenen Zeiger sind keine S-Pointer,
 * sondern 'nur' C-Pointer (es wird nicht kopiert). Das macht Probleme
 * mit der Garbage-Collection in der 'lean' Speicherverwaltung.
 * Daher werden fuer die 'lean' Speicherverwaltung die Zeiger im Mem-
 * Teil des Stacks gespeichert und nicht im Point-Teil. (kg) */

#define MDB_get_dbx_stack(pos) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMread)->dbx)

#define MDB_put_dbx_stack(pos, dbx_info) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMwrite)->dbx) = dbx_info

#define MDB_get_procname_stack(pos) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMread)->procname)

#define MDB_put_procname_stack(pos, procname) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMread)->procname) = procname

#define MDB_get_formal_args_stack(pos) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMread)->formals)

#define MDB_put_formal_args_stack(pos, args) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMread)->formals) = args

#define MDB_get_proc_stack(pos) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMread)->proc)

#define MDB_put_proc_stack(pos, proc) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMread)->proc) = proc

#define MDB_get_local_args_stack(pos) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMread)->locals)

#define MDB_put_local_args_stack(pos, args) \
	(MMMarr(&stack, pos, STACK_TYPE, MMMread)->locals) = args

#else /* ! MMMDLEAN */

#define MDB_get_dbx_stack(pos) \
	*MMMarr(&stack, pos, DBX_TYPE, MMMread)

#define MDB_put_dbx_stack(pos, dbx) \
	*MMMarr(&stack, pos, DBX_TYPE, MMMwrite) = dbx

#define MDB_get_procname_stack(pos) \
	*MMMP(&stack, (pos)*4L, MMMread)

#define MDB_put_procname_stack(pos, procname) \
	*MMMP(&stack, (pos)*4L, MMMwrite) = procname

#define MDB_get_formal_args_stack(pos) \
        *MMMP(&stack, (pos)*4L+1L, MMMread)

#define MDB_put_formal_args_stack(pos, args) \
        *MMMP(&stack, (pos)*4L+1L, MMMwrite) = args

#define MDB_get_proc_stack(pos) \
        *MMMP(&stack, (pos)*4L+2L, MMMread)

#define MDB_put_proc_stack(pos, proc) \
        *MMMP(&stack, (pos)*4L+2L, MMMwrite) = proc

#define MDB_get_local_args_stack(pos) \
        *MMMP(&stack, (pos)*4L+3L, MMMread)

#define MDB_put_local_args_stack(pos, args) \
        *MMMP(&stack, (pos)*4L+3L, MMMwrite) = args

#endif /* ! MMMDLEAN */

#define MDB_get_dbx_proc_list(pos) \
        *MMMarr(&proc_list, pos, DBX_TYPE, MMMread)

#define MDB_put_dbx_proc_list(pos, dbx) \
        *MMMarr(&proc_list, pos, DBX_TYPE, MMMwrite) = dbx

#define MDB_get_procname_proc_list(pos) \
        *MMMP(&proc_list, pos, MMMread)

#define MDB_put_procname_proc_list(pos, procname) \
        *MMMP(&proc_list, pos, MMMwrite) = procname

#define MDB_get_dbx_stop_list(pos) \
        *MMMarr(&stop_list, pos, DBX_TYPE, MMMread)

#define MDB_put_dbx_stop_list(pos, dbx) \
        *MMMarr(&stop_list, pos, DBX_TYPE, MMMwrite) = dbx

#define MDB_get_condition_stop_list(pos) \
        *MMMP(&stop_list, pos, MMMread)

#define MDB_put_condition_stop_list(pos, condition) \
        *MMMP(&stop_list, pos, MMMwrite) = condition

#define MDB_free_entry_stop_list(pos) \
	MMMfree(*MMMP(&stop_list, pos, MMMread))

#define MDB_get_varname_display_list(pos) \
	*MMMP(&display_list, pos, MMMread)

#define MDB_put_varname_display_list(pos, varname) \
        *MMMP(&display_list, pos, MMMwrite) = varname

#define MDB_free_entry_display_list(pos) \
	MMMfree(*MMMP(&display_list, pos, MMMread))


void MDB_init( VOID );
void MDB_init_2( VOID );
void MDB_exit( VOID );
void MDB_close_tmp_file( VOID );
void MDB_reset_stop_list( VOID );
void MDB_display_variables( VOID );
void MDB_reset_last_info( VOID );
void MDB_init_stack( VOID );
void MDI_signal_handler( VOID );
void MDI_connect(char* receiver, long pid);
void MDB_interact_with_user(S_Pointer s, long cmd_mode);
void MDB_cmd_dispatcher( VOID );
void MDB_pop_procname(S_Pointer result, MUPbool remember);
void MDB_push_procname(S_Pointer procname, S_Pointer args, S_Pointer proc);
void MDB_execute_cmd(char* cmd_string);
void MDB_display_file(long status);
void MDB_show_buttons(char activate);
void MDB_new_entry_in_proc_list(S_Pointer proc, DBX_TYPE dbx);
void MDB_get_stack_pointers(long *theSP, long *theUpDownPtr );
void MDB_set_level(long *upDownPtr, long newLev );
void MDB_print_args(S_Pointer args);
void MDB_add_debug_info(S_Pointer *root);

MUPbool MDB_has_debug_infos(MTcell s);
MUPbool MDB_no_debug(MTcell proc);

MUPbool MDB_is_finished( VOID );

S_Pointer MDB_create_debug_info(DBX_TYPE mark_dbx);
S_Pointer MDB_eval_trace(S_Pointer s, long prev_func, long eval_type, S_Pointer exec);
S_Pointer MDB_eval_debug(S_Pointer root, long prev_func,long eval_type, S_Pointer exec);
S_Pointer MDB_eval_gcov(S_Pointer root, long prev_func,long eval_type, S_Pointer exec);
S_Pointer MDB_eval_predecessor(S_Pointer root, long prev_func,long eval_type, S_Pointer exec);
S_Pointer MDB_append_info(S_Pointer stmt, S_Pointer info);
S_Pointer MDB_prepand_info(S_Pointer stmt, S_Pointer info);
S_Pointer MDB_execute(S_Pointer cmd);
S_Pointer MDB_get_name_domattr(S_Pointer proc);

DBX_TYPE MDB_get_info(S_Pointer dbx);
DBX_TYPE MDB_is_in_proc_list(char* procname, long *pos);

long MDB_get_key(char* name);
 
char* MDB_get_direct_filename(long index);

inline char* MDB_get_filename(DBX_TYPE dbx) 
{ return(MDB_get_direct_filename(MDB_get_filekey(dbx))); }

MUPbool MDB_is_in_trace_list(S_Pointer procident);

#ifdef MACINTOSH
Handle	MDB_get_stop_marks(char *fileName);
Handle	MDB_build_stack_info(long maxLevel);
#endif

#endif /* DEBUG_ON */

#endif /* __MDB_debug__ */
