/* $Date: 1997/12/01 12:54:40 $  $Author: andi $  $Revision: 1.7 $ */

/******************************************************************************/
/* DATEI:       MDM_kern.h                                                    */
/* AUTOR:       Andreas Sorgatz                                               */
/* ERSTELLT:    23/01/93                                                      */
/* GEAENDERT:   01/12/97                                                      */
/* INHALT:      Definitionen zur Verwaltung dynamischer Module in MuPAD       */
/*                                                                            */
/* Hier (incl.`MDM_base.h') werden alle globale MDM-Definitionen vorgenommen. */
/*                                                                            */
/******************************************************************************/

#ifndef __MDM_kern__
#define __MDM_kern__


#include "MDM_base.h"                            /* Basis der Modulverwaltung */


/*** Die MuPAD-Bezeichner der MDM-Systemfunktionen ****************************/

#define MDMC_LOADMOD     "loadmod"
#define MDMC_UNLOADMOD   "unloadmod"
#define MDMC_EXTERNAL    "external"


/*** Die MuPAD-Bezeichner fuer Optionen von MDM-Systemfunktionen **************/

#define MDMC_FORCE       "Force"


/*** Suffixe fuer die verschiedenen MDM-Datei-Typen ***************************/

#define MDMC_SUFFIX_MDM  ".mdm"                       /* Ein MuPAD-Modul mit  */
#define MDMC_SUFFIX_GEN  ".mdg"                       /* Generischen Objekten */
#define MDMC_SUFFIX_HLP  ".mdh"                       /* Modul-Helpdatei      */


/*** Name der Modul-Initialisierungsfunktion **********************************/

#define MDMC_INIT_MODFUNC "initmod"
#define MDMC_EXIT_MODFUNC "exitmod"
#define MDMC_INFO_MODFUNC "infomod"


/*** Name des Tabelleneintrags fuer generische Methoden  **********************/

#define MDMC_GENERIC_METHODS "generic_methods"
#define MDMC_INCLUDE_METHODS "include"


/*** Im EXEC-Knoten einer Modulfunktion wird hinter der remember-table ein ****/
/*** direkter Verweis auf das zugehoerige Moduldomain eingetragen.         ****/

#define MDMC_EXEC_DOMAIN 4L

/*** Die Counter-Anzahl eines "normalen" Modulfunktion-EXEC enspricht der *****/
/*** Startposition der generischen Objekte in einem erweiterten EXEC      *****/

#define MDMC_EXEC_COUNT  (MDMC_EXEC_DOMAIN+1L)      /* STD: Counter-Anzahl    */
#define MDMC_GENERICS    MDMC_EXEC_COUNT            /* ERW: Generics-Position */


/*** Makros *******************************************************************/

#define MDM_modul_domain(ex)   ( *MMMP(&ex,MDMC_EXEC_DOMAIN,MMMread) )
#define MDM_numof_generics(ex) ( MMMcounter((ex)) - MDMC_EXEC_COUNT )


/*** Globale Variablen ********************************************************/

extern S_Pointer	MDMV_EVAL_EXEC;               /* mcode: MDM_eval_exec */

	
/*** Prototypen der MDM-System-Funktionen *************************************/

S_Pointer MDM_eval_loadmod	( S_Pointer	 s,         
				  long		 prev_func,
				  long		 eval_type,
				  S_Pointer	 exec
				);
S_Pointer MDM_eval_unloadmod	( S_Pointer	 s,         
				  long		 prev_func,
				  long		 eval_type,
				  S_Pointer	 exec
				);
S_Pointer MDM_eval_external	( S_Pointer	 s,         
				  long		 prev_func,
				  long		 eval_type,
				  S_Pointer	 exec
				);


/*** Prototypen weiterer Funktionen *******************************************/

long      MDM_invalid_name	( CONST char	*name );

S_Pointer MDM_create_func_env	( S_Pointer	 func,         
				  S_Pointer	 modul,
				  S_Pointer	 domain,
				  S_Pointer	 tab_of_generics 
				);
S_Pointer MDM_eval_exec		( S_Pointer	 s,         
				  long		 prev_func,
				  long		 eval_type,
				  S_Pointer	 exec
				);
				
long      MDM_exist		( CONST char	*name ); 
long      MDM_which		( CONST char	*name, 
				  CONST char	*suffix,
				  char		*fullname
				);
void      MDM_set_error		( long		 error,
				  char		*text,
				  char		*func
				);

S_Pointer MDM_get_generic	( S_Pointer	 exec, 
				  long		 idx,
				  long		 copy
				);
long      MDM_init_generics	( S_Pointer	*exec
				);
long      MDM_read_generics	( char		*gen_name, 
				  S_Pointer	*tab_of_generics 
				);
long      MDM_append_generics	( S_Pointer	*exec, 
				  S_Pointer	 tab_of_generics 
				);
				
void      MDM_prefix		( S_Pointer	 args,
				  S_Pointer	 pv,
				  S_Pointer	 lv,
				  S_Pointer	*pvc,
				  S_Pointer	*lvc
				);
S_Pointer MDM_postfix		( S_Pointer	 args,
				  S_Pointer	*result,
				  S_Pointer	 exec,
				  long		 remember,
				  long		 prev_func,
				  S_Pointer	 pv,
				  S_Pointer	 lv
				);

#endif /* __MDM_kern__ */
