/* $Date: 1998/03/30 18:29:29 $  $Author: andi $  $Revision: 1.61.2.1 $ */

/******************************************************************************/
/* DATEI:       MDM_mod.h                                                     */
/* AUTOR:       Andreas Sorgatz                                               */
/* ERSTELLT:    26/01/93                                                      */
/* GEAENDERT:   30/03/98                                                      */
/* INHALT:      Definitionen zum Zugriff auf Kern-Funktionen und -Variablen.  */
/*              Definitionen zur einfacheren Modulprogrammierung.             */
/*                                                                            */
/*                        *** UNBEDINGT LESEN !!! ***                         */
/*                                                                            */
/* In dieser Datei werden Makros fuer den Zugriff auf die  MuPAD-Kern-Objekte */
/* definiert.Dazu wird zu jedem Objekt ein gleichnamiges Definement erstellt, */
/* dass ueber das Makro  `MDM_K'  die Objekt-Adresse bestimmt und ein casting */
/* vornimmt. Bei Variablen wird hier zugleich der Inhalt rereferenziert,waeh- */
/* rend bei Funktionen nur ihr Ergebnis typisiert wird  und der Funktionszei- */
/* ger so beliebig parametrisiert werden kann. Der Modulprogrammierer hat so- */
/* mit die gleiche Programmierschnittstelle  wie der Kernprogrammierer.  All- */
/* kann der C-Compiler bei Modulen  nicht die Korrektheit der Parameteranzahl */
/* und -typen feststellen.                                                    */
/*                                                                            */
/* Wird ein dynamischer linker verwendet, so lassen sich die Makros ueber das */
/* Definement MDMD_USE_DYNLINK ausblenden. In diesem Fall werden die Kernfun- */
/* ktionen direkt (ohne Sprungtabelle) angesprungen. Hierbei wird auch das in */
/* den Kern-headerfiles definierte prototyping aktiv.  Mit dieser Option kann */
/* ein Modul in jedem Fall probeweise uebersetzt  (nicht gelinkt)  werden, um */
/* das aktive prototyping zur erweiterten Ueberpruefung der statischen Seman- */
/* tik der Modul-Source zu nutzen.                                            */
/*                                                                            */
/* Das Makro  `MDM_K(idx)' ist systemabhaengig und realisiert den Zugriff auf */
/* die Tabelle der Kernobjekte  `MDM_object_tab'.    Dieser Zugriff geschieht */
/* entweder ueber die Modulverwaltungsfunktion `MDM_object' oder aber direkt. */
/* Letzteres steigert die Performance um ca. 5-10%,   umgeht aber die Fehler- */
/* kontrolle und erschwert das 'Debug'-en. Der Parameter  idx  entspricht dem */
/* Objekt-Index in der Tabelle und setzt sich aus einem  `Paket-Index'   plus */
/* dem  `Objekt-Offset' zusammen (siehe unten).                               */
/*                                                                            */
/* Um die Erweiterung der Tabelle  und der Definements  moeglichst einfach zu */
/* halten, werden die Funktionen und Variablen in Pakete zusammengefasst. Da- */
/* bei enthaelt das Paket  MDM_XYZ  die "interessanten" Funktionen/Variablen, */
/* die in den Dateien  XYZ_*.h  deklariert sind  bzw. mit dem Praefix  `XYZ_' */
/* beginnen. Der Bereich  TOP  ist reserviert und  TMP  steht fuer temporaere */
/* Eintrage zur Verfuegung. In jedem Paket stehen die Funktionen am Anfang.   */
/*                                                                            */
/* Die Objektanzahl eines Paketes wird stets auf ein Vielfaches von  5 aufge- */
/* rundet.  Einerseits um Fehleintraege schneller zu finden,  und desweiteren */
/* um in jedem Paket Leereintraege zu haben,  die fuer Erweiterungen zur Ver- */
/* fuegung stehen. Diese Erweiterungen koennen jederzeit ohne Neuuebersetzung */
/* der Module vorgenommen werden.  Erst wenn sich ein Paket-Index verschiebt, */
/* oder bestehende Funktionen manipuliert werden,  muessen die Module neu ue- */
/* bersetzt werden. Werden neue Pakete definiert, oder ihre Objektanzahl ver- */
/* aendert, so sollte dies UNBEDINGT auch in den Kommentaren (auch in der Da- */
/* tei `MDM_obj.c') geschehen.  Ansonsten wird eine spaetere Fehlersuche  un- */
/* noetig erschwert !                                                         */
/*                                                                            */
/* Alle  Definements muessen  konsistent  zu den Eintragungen in der  Tabelle */
/* `MDMV_object_tab' in der Datei `MDM_obj.c' sein,  ansonsten hat der Aufruf */
/* von Kernfunktionen ggf. unabsehbare Folgen !!!                             */
/*                                                                            */
/* Diese Datei wird von jedem MuPAD-Modul eingebunden.                        */
/*                                                                            */
/******************************************************************************/

#ifndef __MDM_mod__
#define __MDM_mod__

/******************************************************************************/
/*** Alle Kernel header files einbinden                                     ***/
/******************************************************************************/

#include "MCA_calc.h"               /* Die Arithmetik ist sensibel, vorsicht! */


#ifndef MDMD_USE_DYNLINK /* Wenn ein dynam. Linker genutzt werden soll,  dann */
                         /* muessen die Makrodefinitionen ausgeblendet werden */
                         /* Das Makro wird bei Bedarf  vom Modulgenerator ge- */
                         /* gesetzt.                                          */


/*** Zur Zeit werden ANSI-C und C++ Compiler unterstuetzt. ********************/
#ifdef C_PLUSPLUS        /* C++    */
#  define MDMD_ARGS ...                                /* beliebige Argumente */
#else                    /* ANSI-C */
#  define MDMD_ARGS                                    /* beliebige Argumente */
#endif


/***                                                                        ***/
/*** Hier werden die Paket-Indizes aus dem Paket-Index des Vorgaengers plus ***/
/*** dessen Objektanzahl bestimmt.Auf die einzelnen Objekte wird dann unten ***/
/*** stets relativ zu dem entsprechenden Paket-Index zugegriffen.           ***/
/***                                                                        ***/

#define MDM_TOP	   (0       +  0)                   /* 5 Eintraege reserviert */
#define MDM_MAM    (MDM_TOP +  5) /*    5 */
#define MDM_MAR    (MDM_MAM + 30) 
#define MDM_MCA    (MDM_MAR + 10)
#define MDM_MCM    (MDM_MCA + 50)
#define MDM_MDF    (MDM_MCM + 15)
#define MDM_MDM    (MDM_MDF + 15)
#define MDM_MEV    (MDM_MDM + 55) /*  180 */
#define MDM_MFO    (MDM_MEV +190)
#define MDM_MGR    (MDM_MFO + 30)
#define MDM_MGS    (MDM_MGR + 90)
#define MDM_MHI    (MDM_MGS + 20)
#define MDM_MIO    (MDM_MHI + 25)
#define MDM_MMM    (MDM_MIO + 80) /*  615 */
#define MDM_MPA    (MDM_MMM + 60)
#define MDM_MPH    (MDM_MPA + 20)
#define MDM_MPL    (MDM_MPH + 30)
#define MDM_MPO    (MDM_MPL + 25)
#define MDM_MSE    (MDM_MPO + 60)
#define MDM_MSG    (MDM_MSE + 15)
#define MDM_MSR    (MDM_MSG + 10)
#define MDM_MST    (MDM_MSR + 25) /*  860 */
#define MDM_MSU    (MDM_MST + 70)
#define MDM_MSV    (MDM_MSU + 45)
#define MDM_MSY    (MDM_MSV + 40)
#define MDM_MTA    (MDM_MSY + 90)
#define MDM_MTR    (MDM_MTA + 10)
#define MDM_MUP    (MDM_MTR + 60)
#define MDM_MUT    (MDM_MUP + 50)
#define MDM_MXP    (MDM_MUT + 45)
#define MDM_PAR    (MDM_MXP + 05)
#define MDM_MDB    (MDM_PAR +260) /* 1335 */
/*** Hier duerfen neue Gruppen ergaenzt werden ***/
#define MDM_MKT    (MDM_MDB + 10)
#define MDM_END    (MDM_MKT +100) /* 1655 */


/***                                                                        ***/
/*** Hier werden die Makros zum Aufruf von MuPAD-Kernfunktionen und zum Zu- ***/
/*** auf MuPAD-Kernvariablen definiert.Die Namen der Makros entsprechen da- ***/
/**  bei denen der Funktionen bzw. Variablen im Kern von MuPAD.             ***/
/*** werden ueber ihren Tabellen-Index identifiziert,  der sich aus dem Pa- ***/
/***                                                                        ***/

/************ Globale Funktionen des Kerns ************************************/

	/*** #005 = TOP *******************************************************/

	/*** #030 = MAM *******************************************************/

#define MAMF_create_Mem1               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  0))
/*#define MAMF_create_Mem2               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  1))*/
#define MAMF_create_Mem1_size          (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  2))
#define MAMF_free_Mem1                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  3))
#define MAMF_content_Mem1              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  4))
#define MAMF_in_Mem1                   (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  5))
/*#define MAMF_in_Mem2                   (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  6))*/
#define MAMF_s_in_Mem1                 (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  7))
/*#define MAMF_s_in_Mem2                 (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  8))*/
#define MAMF_delete_Mem1               (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MAM+  9))
#define MAMF_insert_Mem1               (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MAM+ 10))
/*#define MAMF_insert_Mem2               (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MAM+ 11))*/
#define MAMFcond_insert                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MAM+ 12))
#define MAMF_r_insert_Mem1             (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MAM+ 13))
/*#define MAMF_r_insert_Mem2             (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MAM+ 14))*/

	/*** #010 = MAR *******************************************************/

#define MAR_eval_array                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MAR+  0))
#define MAR_read_array                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MAR+  1))
#define MAR_write_array                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MAR+  2))
#define MAR_eval_index                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MAR+  3))
#define MAR_get_status_info            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MAR+  4))

	/*** #050 = MCA *******************************************************/

#define MCA_cat_to_pari                (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  0))
#define MCA_calc_gen                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  1))
#define MCA_init_cat_pari              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  2))
#define MCA_plus_minus                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  3))
#define MCA_pari_vector_to_cat_list    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  4))
#define MCA_recompute_pi               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  5))
#ifdef NO_TRANS_E_TO_EXP
#define MCA_recompute_e                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  6))
#endif
#define MCA_abs                        (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  7))
#define MCA_issqr                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  8))
#define MCA_is_neg                     (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MCA+  9))
#define MCA_recompute_euler            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+ 10))
#define MCA_set_prec                   (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_MCA+ 11))
#define MCA_is_real_extension          (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MCA+ 12))
#define MCA_normal_rational            (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MCA+ 13))
#define MCA_set_digits                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MCA+ 14))
#define MCA_float                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MCA+ 15))

#define MCAC_ci                        (*(S_Pointer      *) MDM_K(MDM_MCA+ 25))
#define MCAC_one                       (*(S_Pointer      *) MDM_K(MDM_MCA+ 26))
#define MCAC_one_                      (*(S_Pointer      *) MDM_K(MDM_MCA+ 27))
#define MCAC_two                       (*(S_Pointer      *) MDM_K(MDM_MCA+ 28))
#define MCAC_two_                      (*(S_Pointer      *) MDM_K(MDM_MCA+ 29))
#define MCAC_zero                      (*(S_Pointer      *) MDM_K(MDM_MCA+ 30))
#define MCAC_half                      (*(S_Pointer      *) MDM_K(MDM_MCA+ 31))

#define MCAV_STORAGE                   (*(long           *) MDM_K(MDM_MCA+ 33))
#define MCAV_avglob                    (*(long           *) MDM_K(MDM_MCA+ 34))
#define MCAV_string                    ( (char           *) MDM_K(MDM_MCA+ 35))
#define MCAV_DEC_DIGITS                (*(long           *) MDM_K(MDM_MCA+ 36))

	/*** #015 = MCM *******************************************************/

#define MCM_equal_specless             (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MCM+  0))
#define MCM_compare_specless           (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MCM+  1))
#define MCM_compare_string_ident       (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MCM+  2))
#define MCM_equal_intern               (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MCM+  3))
#define MCM_compare_intern             (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MCM+  4))

	/*** #015 = MDF *******************************************************/

#define MDF_eval_diff                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDF+  0))
#define MDF_diff                       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDF+  1))
#define MDF_power_minus_one            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDF+  2))

#define MDF_new_diff_call              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDF+  5))
#define MDF_partial                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDF+  6))

#define MDFV_ASS_MEM_MAIN              (*(S_Pointer      *) MDM_K(MDM_MDF+ 10))

        /*** #055 = MDM *******************************************************/

#define MDM_eval_loadmod               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  0))
#define MDM_eval_unloadmod             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  1))
#define MDM_eval_external              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  2))
#define MDM_eval_exec                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  3))
#define MDM_create_func_env            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  4))
#define MDM_exist                      (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  5))
#define MDM_which                      (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  6))
#define MDM_set_error                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  7))
#define MDM_get_entry                  (*(MDM_mod_t* (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  8))
#define MDM_init_generics              (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+  9))
#define MDM_get_generic                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 10))
#define MDM_read_generics              (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 11))
#define MDM_append_generics            (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 12))

#define MDM_prefix                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 14))
#define MDM_postfix                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 15))

#define MDM_reset                      (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 20))
#define MDM_kick_out                   (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 21))
#define MDM_aging_intr                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 22))
#define MDM_aging                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 23))

#define MDM_load                       (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 26))
#define MDM_unload                     (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 27))
#define MDM_call                       (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 28))
#define MDM_object                     (*(MDM_jump_t (*)(MDMD_ARGS)) MDM_K(MDM_MDM+ 29))

#define MDMV_EVAL_EXEC                 (*(S_Pointer      *) MDM_K(MDM_MDM+ 35))

#define MDMV_pmod_num                  (*(long           *) MDM_K(MDM_MDM+ 37))
#define MDMV_pmod                      (*(MDM_pmod_t    **) MDM_K(MDM_MDM+ 38))
#define MDMV_object_tab                ( (MDM_jump_t     *) MDM_K(MDM_MDM+ 39))
#define MDMV_object_tab_len            (*(long           *) MDM_K(MDM_MDM+ 40))
#define MDMV_list                      (*(MDM_list_t    **) MDM_K(MDM_MDM+ 41))
#define MDMV_support                   (*(long           *) MDM_K(MDM_MDM+ 42))
#define MDMV_unload_support            (*(long           *) MDM_K(MDM_MDM+ 43))
#define MDMV_age_max                   (*(unsigned long  *) MDM_K(MDM_MDM+ 44))
#define MDMV_aging                     (*(unsigned long  *) MDM_K(MDM_MDM+ 45))

	/*** #190 = MEV *******************************************************/

#define MES_eval_return                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  0))
#define MES_eval                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  1))
#define MEV_execute                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  2))
#define MEV_eval                       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  3))
#define MEV_eval_args                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  4))
#define MEV_eval_args_index            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  5))
#define MEV_eval_index                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  6))
#define MEV_eval_not                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  7))
#define MEV_eval_bool                  (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  8))
#define MEV_set_error                  (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+  9))
#define MEV_refresh_ass_mem            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 10))
#define MEV_clear_ass_mem              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 11))
#define MEV_init_ass_mem_ident         (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 12))
#define MEV_init_ass_mem_expr          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 13))
#define MEV_init_ass_mem_simplifier    (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 14))
#define MEV_clear_ass_mem_simplifier   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 15))
#define MEV_delete_ass_mem_simplifier  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 16))
#define MEV_check_options              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 17))
#define MEV_normalize_options          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 18))
#define MEV_eval_sys_func_args         (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 19))
#define MEV_eval_gprof                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 20))
#define MEE_find_number_for_code       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 21))
#define MEE_find_code_for_number       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 22))
#define MEE_eval_func_env              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 23))
#define MEE_eval_built_in              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 24))
#define MEE_eval_exprseq               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 25))
#define MEE_eval_or                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 26))
#define MEE_eval_and                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 27))
#define MEE_eval_seqgen                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 28))
#define MEE_eval_less                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 29))
#define MEE_eval_leequal               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 30))
#define MEE_eval_unequal               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 31))
#define MEE_eval_range                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 32))
#define MEE_eval_mod                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 33))
#define MEE_eval_div                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 34))
#define MEE_eval_union                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 35))
#define MEE_eval_minus                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 36))
#define MEE_eval_intersect             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 37))
#define MEE_eval_plus                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 38))
#define MEE_eval_mult                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 39))
#define MEE_eval_power                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 40))
#define MEE_eval_concat                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 41))
#define MEE_domain_set_spec            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 42))
#define MEE_eval_fconcat               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 43))
#define MEV_eval_procdef               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 44))
#define MEV_eval_procd                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 45))
#define MES_eval_assign                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 46))
#define MES_eval_if                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 47))
#define MES_eval_case                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 48))
#define MES_eval_for                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 49))
#define MES_eval_for_par               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 50))
#define MES_eval_for_down              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 51))
#define MES_eval_for_in                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 52))
#define MES_eval_for_in_par            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 53))
#define MES_eval_while                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 54))
#define MES_eval_repeat                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 55))
#define MES_eval_next                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 56))
#define MES_eval_break                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 57))
#define MES_eval_quit                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 58))
#define MES_eval_parbegin              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 59))
#define MES_eval_stmtseq               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 60))
#define MEV_header_func                (*(short      (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 61))
#define MEV_remember_func              (*(short      (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 62))
#define MEV_domain_func                (*(short      (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 63))
#define MES_is_proc                    (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 64))
#define MES_eval_domattr_lhs           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 65))
#define MES_new_formal_rem_proc        (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 66))
#define MES_eval_lhs                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 67))
#define MES_assign_lhs_list            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 68))
#define MES_build_name                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 69))
#define MEV_interrupt                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 70))
#define MEV_eval_interpreter           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 71))
#define MEV_eval_float_gen             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 72))
#define MEV_eval_index_val             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 73))
#define MEV_eval_op_plusmult           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 74))
#define MEV_eval_op_power              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 75))
#define MEV_eval_op_fconcat            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 76))

#ifdef MDX
#define MEV_eval_debug                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 77))
#endif

#define MEE_eval_modp                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 78))
#define MEE_eval_mods                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 79))
#define MEE_eval_domattr               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 80))
#define MEE_eval_new_extelement        (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 81))
#define MEE_eval_new_domain            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 82))
#define MEE_eval_equal                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 83))
#define MEE_eval_funcattr              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 84))
#define MEE_eval_context               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 85))
#define MEV_eval_any                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 86))
#define MEV_sign_error                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 87))
#define MEV_eval_expr                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 88))
#define MEV_index                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 89))
#define MEV_eval_ident                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 90))
#define MEV_eval_set                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 91))
#define MEV_eval_stat_list             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 92))
#define MEV_eval_table                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 93))

#define MEV_eval_array                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 95))
#define MEV_eval_func_env              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 96))
#define MEV_eval_exec                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 97))
#define MEV_eval_ext                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 98))
#define MEV_eval_poly                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+ 99))

#define MEVV_sessionbegin_sec          (*(unsigned long  *) MDM_K(MDM_MEV+100))
#define MEVV_sessionbegin_msec         (*(unsigned long  *) MDM_K(MDM_MEV+101))
#define MEVV_EVAL_STMT                 (*(long           *) MDM_K(MDM_MEV+102))
#define MEVV_interrupt                 (*(unsigned long  *) MDM_K(MDM_MEV+103))
#define MEVV_LEVEL                     (*(long           *) MDM_K(MDM_MEV+104))
#define MEVV_MAXLEVEL                  (*(long           *) MDM_K(MDM_MEV+105))
#define MEVV_TEXTWIDTH                 (*(long           *) MDM_K(MDM_MEV+106))

#ifdef ERRORLEVEL
#define MEVV_ERRORLEVEL                (*(long           *) MDM_K(MDM_MEV+107))
#endif

#define MEVV_PRINTLEVEL                (*(long           *) MDM_K(MDM_MEV+108))
#define MEVV_ACT_PRINTLEVEL            (*(long           *) MDM_K(MDM_MEV+109))
#define MEVV_ASS_MEM_EXPR              (*(S_Pointer      *) MDM_K(MDM_MEV+110))
#define MEVV_ASS_MEM_IDENT             (*(S_Pointer      *) MDM_K(MDM_MEV+111))
#define MEVV_ASS_MEM_FUNC_ENV          (*(S_Pointer      *) MDM_K(MDM_MEV+112))
#define MEVV_ERROR                     (*(short          *) MDM_K(MDM_MEV+113))
#define MEVV_STATUS                    (*(short          *) MDM_K(MDM_MEV+114))
#define MEVV_ERROR_STR                 (*(char*          *) MDM_K(MDM_MEV+115))
#define MEVV_ASSIGNED                  (*(long           *) MDM_K(MDM_MEV+116))
#define MEVV_EVAL_LAST                 (*(S_Pointer      *) MDM_K(MDM_MEV+117))
#define MEVV_INTERPRETER               (*(S_Pointer      *) MDM_K(MDM_MEV+118))
#define MEVV_actual_stamp              (*(MEVTstamp      *) MDM_K(MDM_MEV+119))

typedef short	MDM_MEVV_LegalOpTab_t[MEVC_MAX_OPERATORS][MEVC_MAX_OPERANDS];
#define MEVV_LegalOpTab         (*(MDM_MEVV_LegalOpTab_t *) MDM_K(MDM_MEV+120))

#define MEVV_MAM_STACK                 (*(S_Pointer      *) MDM_K(MDM_MEV+121))
#define MEVV_MAM_CONTROL               (*(long           *) MDM_K(MDM_MEV+122))
#define MEVV_ACT_DEPTH                 (*(unsigned long  *) MDM_K(MDM_MEV+123))
#define MEVV_DEPTH_STACK               (*(S_Pointer      *) MDM_K(MDM_MEV+124))
#define MEVV_ARGS                      (*(S_Pointer      *) MDM_K(MDM_MEV+125))
#define MEVV_ARGS_OLD                  (*(S_Pointer      *) MDM_K(MDM_MEV+126))
#define MEVV_MAXDEPTH                  (*(long           *) MDM_K(MDM_MEV+127))
#define MEVV_ASS_MEM_DOMAINS           (*(S_Pointer      *) MDM_K(MDM_MEV+128))

#define MEVV_pref                      (*(MEVT_pref      *) MDM_K(MDM_MEV+132))

#define MEV_check_local_formals        (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+140))
#define MEV_gen_prof_data              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+141))
#define MEV_func_env_in                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+142))
#define MEV_func_env_insert            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+143))
#define MEV_header_func_1              (*(short      (*)(MDMD_ARGS)) MDM_K(MDM_MEV+144))
#define MEV_header_func_2              (*(short      (*)(MDMD_ARGS)) MDM_K(MDM_MEV+145))
#define MEV_first_ext                  (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+146))
#define MEV_push_mam_stack             (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+147))
#define MEV_pop_mam_stack              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+148))
#define MEV_push_depth_stack           (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+149))
#define MEV_pop_depth_stack            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+150))

#define MEV_eval_domainfunccall        (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+152))
#define MEE_get_domain                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+153))
#define MEE_read_domain                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+154))
#define MEE_write_domain               (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MEV+155))
#define MEE_domain_inquick             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+156))
#define MEE_domain_in                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+157))
#define MEE_domain_insert              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+158))

#define MEE_domain_initialize_quickreference (*(void (*)(MDMD_ARGS)) MDM_K(MDM_MEV+159))
#define MEE_domain_actualize_quickreference  (*(void (*)(MDMD_ARGS)) MDM_K(MDM_MEV+160))
#define MEE_domain_ass_call1           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+161))
#define MEE_domain_ass_call1quick      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+162))
#define MEE_domain_ass_callm           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+163))
#define MEE_domain_ass_callmquick      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+164))

#define MEE_func_env_ass_callm         (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+166))
#define MEE_copy                       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MEV+167))

	/*** #030 = MFO *******************************************************/

#define MFO_eval_fclose                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  0))
#define MFO_eval_fopen                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  1))
#define MFO_eval_fread                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  2))
#define MFO_eval_write                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  3))
#define MFO_eval_fprint                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  4))
#define MFO_eval_finput                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  5))
#define MFO_eval_ftextinput            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  6))
#define MFO_eval_protocol              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  7))
#define MFO_open_read                  (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  8))
#define MFO_open_write                 (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MFO+  9))
#define MFO_read                       (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MFO+ 10))

#define MFOV_PROT_FILE                 (*(MCLT_iostream           *) MDM_K(MDM_MFO+ 20))

	/*** #090 = MGR *******************************************************/

#define MGR_eval_plot                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+  0))
#define MGR_eval_point                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+  1))
#define MGR_eval_polygon               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+  2))

#define MGR_color_alloc_write_sp_color (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+  9))
#define MGR_color_eval_color           (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 10))
#define MGR_read_default_file          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 11))
#define MGR_get_default_scene          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 12))
#define MGR_get_default_object         (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 13))
#define MGR_calc_minmax_values_3d      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 14))
#define MGR_default_perspective        (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 15))
#define MGR_resize_scene_spointer      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 16))
#define MGR_get_scene_adr              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 17))
#define MGR_set_scene_options          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 18))
#define MGR_get_obj_adr                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 19))
#define MGR_set_object_options         (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 20))
#define MGR_read_scene                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 21))
#define MGR_read_object                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 22))
#define MGR_get_obj_pointlist          (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 23))
#define MGR_init_spointer              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 24))
#define MGR_set_spointer_options       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 25))
#define MGR_get_default_spointer       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 26))
#define MGR_set_default_options        (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 27))
#define MGR_read_defaults              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 28))
#define MGR_calc_points                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 29))
#define MGR_write_to_binary_file       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 30))
#define MGR_write_to_ascii_file        (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 31))
#define MGR_check_scene                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 32))
#define MGR_check_object               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 33))
#define MGR_check_file                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 34))
#define MGR_check_polygon              (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 35))
#define MGR_check_parametric           (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 36))
#define MGR_check_ident                (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 37))
#define MGR_check_bin_equation         (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 38))
#define MGR_check_n_real_list          (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 39))
#define MGR_check_string               (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 40))
#define MGR_convert_double_color       (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 41))
#define MGR_convert_long_color         (*(double     (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 42))
#define MGR_calc_sp_command            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 43))
#define MGR_send_sp_command            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 44))
#define MGR_set_objectstrings          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 45))
#define MGR_swap                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 46))
#define MGR_string_to_c                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 47))

#if ((defined OLDCOMMUNICATION) && (! defined MACINTOSH)) || (defined MSDOS)
#define MGR_output_descr               (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGR+ 75))
#define MGRV_output_file               (*(MCLT_iostream                    *) MDM_K(MDM_MGR+ 76))
#define MGRV_old_file                  (*(MCLT_iostream                    *) MDM_K(MDM_MGR+ 77))
#define MGRV_text_width                (*(long                    *) MDM_K(MDM_MGR+ 78))
#endif

#define MGRV_SP_DEFAULTS               (*(S_Pointer               *) MDM_K(MDM_MGR+ 80))
#define MGRV_NO_DEFAULT_FILE           (*(long                    *) MDM_K(MDM_MGR+ 81))
#define MGRV_MACHINE_FLOAT             (*(MUPbool                 *) MDM_K(MDM_MGR+ 82))

	/*** #020 = MGS *******************************************************/

#define MGS_quicksort                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGS+  0))
#define MGS_quicksort_generic          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGS+  1))
#define MGS_straightinsertion          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGS+  2))
#define MGS_straightinsertion_generic  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGS+  3))
#define MGS_flat                       (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MGS+  4))
#define MGS_set_spec                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MGS+  5))

#define MGSV_flat_table                (*(MUPbool*       *) MDM_K(MDM_MGS+ 15))
#define MGSV_redo                      (*(MUPbool        *) MDM_K(MDM_MGS+ 16))

	/*** #025 = MHI *******************************************************/

#define MHI_reset_interactiv           (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MHI+  0))
#define MHI_free                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MHI+  1))
#define MHI_write_                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MHI+  2))
#define MHI_read_                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MHI+  3))

#define MHIV_TAB_SIZE                  (*(long           *) MDM_K(MDM_MHI+ 10))
#define MHIV_ACT_POS                   (*(long           *) MDM_K(MDM_MHI+ 11))
#define MHIV_HISTORY                   (*(long           *) MDM_K(MDM_MHI+ 12))
#define MHIV_TOP_HISTORY               (*(long           *) MDM_K(MDM_MHI+ 13))
#define MHIV_PROC_HISTORY              (*(long           *) MDM_K(MDM_MHI+ 14))
#define MHIV_WRAP_AROUND               (*(long           *) MDM_K(MDM_MHI+ 15))
#define MHIV_TOP                       (*(long           *) MDM_K(MDM_MHI+ 16))
#define MHIC_name                      (*(S_Pointer      *) MDM_K(MDM_MHI+ 17))
#define MHIV_TOP_OLD                   (*(long           *) MDM_K(MDM_MHI+ 18))
#define MHIV_ACT_POS_OLD               (*(long           *) MDM_K(MDM_MHI+ 19))
#define MHIV_HISTORY_OLD               (*(long           *) MDM_K(MDM_MHI+ 20))

	/*** #080 = MIO *******************************************************/

#define MIO_out                        (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MIO+  0))
#define MIO_out_seqgen                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+  1))
#define MIO_out_stand_op               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+  2))

#define MIO_out_stand_sys_func         (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+  4))
#define MIO_out_index                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+  5))
#define MIO_out_assign                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+  6))
#define MIO_out_if                     (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+  7))
#define MIO_out_case                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+  8))
#define MIO_out_for                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+  9))
#define MIO_out_for_par                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 10))
#define MIO_out_for_down               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 11))
#define MIO_out_for_in                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 12))
#define MIO_out_for_in_par             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 13))
#define MIO_out_while                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 14))
#define MIO_out_repeat                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 15))
#define MIO_out_next                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 16))
#define MIO_out_break                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 17))
#define MIO_out_quit                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 18))
#define MIO_out_parbegin               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 19))
#define MIO_out_seqbegin               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 20))

#define MIO_print_string               (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 23))
#define MIO_print_char                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 24))

#define MIO_out_domattr                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 26))

#define MIO_expr2text                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 48))
#define MIO_create_debug               (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MIO+ 50))

#define MIOV_S_PLUS                    (*(S_Pointer      *) MDM_K(MDM_MIO+ 60))
#define MIOV_S_MULT                    (*(S_Pointer      *) MDM_K(MDM_MIO+ 61))
#define MIOV_S_POWER                   (*(S_Pointer      *) MDM_K(MDM_MIO+ 62))
#define MIOV_S_EXPRSEQ                 (*(S_Pointer      *) MDM_K(MDM_MIO+ 63))
#define MIOV_S_STMTSEQ                 (*(S_Pointer      *) MDM_K(MDM_MIO+ 64))
#define MIOV_OUTPUT                    (*(MCLT_iostream           *) MDM_K(MDM_MIO+ 65))
#define MIOV_PRETTY_PRINT              (*(long           *) MDM_K(MDM_MIO+ 66))
#define MIOV_BUFFER                    ( (char           *) MDM_K(MDM_MIO+ 67))
#define MIOV_STRING_UNQUOTED           (*(long           *) MDM_K(MDM_MIO+ 68))
#define MIOV_DOMAIN_ASS_OUTPUT         (*(S_Pointer      *) MDM_K(MDM_MIO+ 69))
#define MIOV_DOMAIN_ASS_EXPR2TEXT      (*(S_Pointer      *) MDM_K(MDM_MIO+ 70))

	/*** #060 = MMM *******************************************************/

#ifdef MMMD_GEN		/*** BEGIN: generic     Speicherverwaltung ***/

#define MMMcounter                     (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  0)) 
#define MMMfreeh                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  1))
#define MMMlfreeh                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  2))
#define MMMsize                        (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  3))
#define MMMtype                        (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  4))
#define MMMmalloc                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  5))
#define MMMpalloc                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  6))
#define MMMcalloc                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  7))
#define MMMresize                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  8))
#define MMMnewcounter                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  9))
#define MMMnewtype                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 10))
#define MMMrealloc                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 11))
#define MMMunmoveable                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 12))
#define MMMgarbage                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 13))
#define MMMforce_data                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 14))
#define MMMchange                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 15))
#define MMMseth                        (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 16))
#define MMMupper_anz                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 17))

#  ifdef MMMDACTIV        
#define MMMinactiv                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 18))
#  else
#  endif

#  ifdef MMMDSPECIAL
#define MMMPh                          (*(S_Pointer* (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 19))
#define MMMZh                          (*(char*      (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 20))
#define MMMarrh                        (*(char*      (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 21))
#define MMMmvarrh                      (*(char*      (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 22))
#define MMMmvh                         (*(char*      (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 23))
#  else
#  endif

#  ifdef MMMMOVE
#define MMMpoolinserth                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 24))
#define MMMpooldeleteh                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 25))
#  else
#  endif

#  ifdef MACINTOSH
#define MMMEmalloc                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 26))
#define MMMEfree                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 27))
#  else
#  endif

#define MMMVnetnode                    (*(long           *) MDM_K(MDM_MMM+ 30))
#define MMMVnode                       (*(long           *) MDM_K(MDM_MMM+ 31))
#define MMMVtopologie                  (*(S_Pointer      *) MDM_K(MDM_MMM+ 32))
#define MMMVson                        (*(long           *) MDM_K(MDM_MMM+ 33))
#define MMMVlocal_son                  (*(long           *) MDM_K(MDM_MMM+ 34))
#define MMMVnumber_of_processes        (*(long           *) MDM_K(MDM_MMM+ 35))
#define MMMVsystemfree                 (*(void (**)()     ) MDM_K(MDM_MMM+ 36))
#define MMMVsystemchange               (*(void (**)()     ) MDM_K(MDM_MMM+ 37))
#define MMMVno_memory                  (*(char*(**)(MUPbool)) MDM_K(MDM_MMM+ 38))

#define MMMVused_blocks                (*(long           *) MDM_K(MDM_MMM+ 41))
#define MMMVused_memory                (*(long           *) MDM_K(MDM_MMM+ 42))
#define MMMVreserved_blocks            (*(long           *) MDM_K(MDM_MMM+ 43))
#define MMMVreserved_memory            (*(long           *) MDM_K(MDM_MMM+ 44))
#define MMMVfrees                      (*(long           *) MDM_K(MDM_MMM+ 45))
#define MMMVrealfrees                  (*(long           *) MDM_K(MDM_MMM+ 46))
#define MMMVmallocs                    (*(long           *) MDM_K(MDM_MMM+ 47))

#define MMMVsharedh                    (*(S_Pointer      *) MDM_K(MDM_MMM+ 50))
#define MMMVlocal_sharedh              (*(S_Pointer      *) MDM_K(MDM_MMM+ 51))

#define MMMVcheck                      (*(MMMclock_t     *) MDM_K(MDM_MMM+ 55))
#define MMMMV_new_header               (*(long           *) MDM_K(MDM_MMM+ 56))
#define MMMV_new_lheader               (*(long           *) MDM_K(MDM_MMM+ 57))
#define MMMV_mem                       (*(long*          *) MDM_K(MDM_MMM+ 58))
#define MMMVBelegung                   ( (MMMTBelegung   *) MDM_K(MDM_MMM+ 59))


#elif defined MMMDAUTO	 	/*** END  : generic      Speicherverwaltung ***/
		                /*** BEGIN: automatische Speicherverwaltung ***/

#elif defined MMMDLEAN		/*** END  : automatische Speicherverwaltung ***/
                                /*** BEGIN: lean         Speicherverwaltung ***/

#define MMMmalloc                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  0))
#define MMMread                        (*(MMMulong       *)          MDM_K(MDM_MMM+  1))
#define MMMwrite                       (*(MMMulong       *)          MDM_K(MDM_MMM+  2))
#define MMMCglobal                     (*(MMMulong       *)          MDM_K(MDM_MMM+  3))
#define MMMNULL                        (*(S_Pointer      *)          MDM_K(MDM_MMM+  4))
#define MMMVno_memory                  (*(MMMTno_memory  *)          MDM_K(MDM_MMM+  5))
#define MMMresize                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  6))
#define MMMnewcounter                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  7))
#define MMMrealloc                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  8))
#define MMMchange                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  9))
#define MMMpalloc                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  0))
#define MMMcalloc                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 11))
#define MMMinsert_global               (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 12))
#define MMMmark_globals                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 13))
#define MMMmark_tree                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 14))
#define MMMfreeUnmarked                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 15))
#define MMMfree_unused                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 16))
#define MMMbytes                       (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 17))
#define MMMrecFree                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 18))
#define MMMfreeBlock                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 19))

#else                           /*** END  : lean        Speicherverwaltung ***/
                                /*** BEGIN: ftp         Speicherverwaltung ***/

#define MMMFpalloc                     (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  0))
#define MMMFcalloc                     (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  1))
#define MMMFfree                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  2))
#define MMMFlfree                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  3))
#define MMMFensure                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  4))
#define MMMFenlarge_freelist           (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  5))
#define MMMFenlarge_headerlist         (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  6))
#define MMMFresize                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  7))
#define MMMFnewcounter                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  8))
#define MMMFrealloc                    (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+  9))
#define MMMFforce_data                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 10))
#define MMMHmmalloc                    (*(char*      (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 11))
#define MMMHffree                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 12))
#define MMMFinit                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 13))
#define MMMFexit                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 14))
#define MMMHread_ulong                 (*(MMMulong      (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 15))
#define MMMHtransfer_content           (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 16))
#define MMMFtype                       (*(MMMulong      (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 17))
#define MMMFprepare_sleep              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 18))
#define MMMFsleep                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 19))
#define MMMFwake_up                    (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 20))
#define MMMFmalloc                     (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 21))

#  ifdef MACINTOSH
#define MMMEmalloc                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 26))
#define MMMEfree                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MMM+ 27))
#  else
#  endif

#define MMMVnetnode                    (*(long           *) MDM_K(MDM_MMM+ 30))
#define MMMVnode                       (*(long           *) MDM_K(MDM_MMM+ 31))
#define MMMVtopologie                  (*(S_Pointer      *) MDM_K(MDM_MMM+ 32))
#define MMMVson                        (*(unsigned long  *) MDM_K(MDM_MMM+ 33))
#define MMMVlocal_son                  (*(unsigned long  *) MDM_K(MDM_MMM+ 34))
#define MMMVnumber_of_processes        (*(unsigned long  *) MDM_K(MDM_MMM+ 35))
#define MMMVsystemfree                 (*(void (**)()     ) MDM_K(MDM_MMM+ 36))
#define MMMVsystemchange               (*(void (**)()     ) MDM_K(MDM_MMM+ 37))
#define MMMVno_memory                  (*(char*(**)(MUPbool)) MDM_K(MDM_MMM+ 38))
#define MMMVmalloc_help                (*(char*          *) MDM_K(MDM_MMM+ 39))
#define MMMVNULLtype                   (*(unsigned long  *) MDM_K(MDM_MMM+ 40))
#define MMMVused_blocks                (*(unsigned long  *) MDM_K(MDM_MMM+ 41))
#define MMMVused_memory                (*(unsigned long  *) MDM_K(MDM_MMM+ 42))
#define MMMVreserved_blocks            (*(unsigned long  *) MDM_K(MDM_MMM+ 43))
#define MMMVreserved_memory            (*(unsigned long  *) MDM_K(MDM_MMM+ 44))
#define MMMVfrees                      (*(unsigned long  *) MDM_K(MDM_MMM+ 45))
#define MMMVrealfrees                  (*(unsigned long  *) MDM_K(MDM_MMM+ 46))
#define MMMVmallocs                    (*(unsigned long  *) MDM_K(MDM_MMM+ 47))
#define MMMVlockcount                  (*(unsigned long  *) MDM_K(MDM_MMM+ 48))
#define MMMVlocktable                  (*(MMMTlock*      *) MDM_K(MDM_MMM+ 49))
#define MMMVsharedh                    (*(S_Pointer      *) MDM_K(MDM_MMM+ 50))
#define MMMVlocal_sharedh              (*(S_Pointer      *) MDM_K(MDM_MMM+ 51))

#endif		/*** END:   sequentiell Speicherverwaltung ***/


	/*** #020 = MPA *******************************************************/

#define MPA_exec_read                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPA+  0))
#define MPA_parse_input                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPA+  1))
#define MPA_parse_textinput            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPA+  2))
#define MPA_parse_finput               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPA+  3))
#define MPA_parse_ftextinput           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPA+  4))
#define MPA_parse_string               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPA+  5))

#define MPAV_INPUT_FILTER              (*(S_Pointer      *) MDM_K(MDM_MPA+ 15))
#define MPAV_JumpLabel                 (*(jmp_buf        *) MDM_K(MDM_MPA+ 16))

	/*** #030 = MPH *******************************************************/

#define MPH_delete_subtasks            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MPH+  0))
#define MPH_build_task                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPH+  1))
#define MPH_prepare_stack              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MPH+  2))
#define MPH_get                        (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MPH+  3))
#define MPH_put                        (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MPH+  4))

#ifdef PARALLEL
#define MPH_get_sleep                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MPH+  5))
#else
#define MPH_count_problemheap          (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MPH+  5))
#define MPH_get_random                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPH+  6))
#define MPH_get_task                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPH+  7))
#endif

#define MPHV_START                     (*(S_Pointer      *) MDM_K(MDM_MPH+ 20))

	/*** #025 = MPL *******************************************************/

#define MPL_create_queue               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+  0))
#define MPL_read_queue                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+  1))
#define MPL_read_queue_sleep           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+  2))
#define MPL_write_queue                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MPL+  3))
#define MPL_read_of_queue              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+  4))
#define MPL_read_of_queue_sleep        (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+  5))
#define MPL_write_to_queue             (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MPL+  6))
#define MPL_eval_readqueue             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+  7))

#define MPL_eval_writequeue            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+  9))
#define MPL_eval_readpipe              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+ 10))

#define MPL_eval_writepipe             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+ 12))
#define MPL_eval_global                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+ 13))
#define MPL_eval_topology              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPL+ 14))

	/*** #060 = MPO *******************************************************/

#define MPO_eval_poly                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+  0))

#define MPO_eval_coeff                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+  2))
#define MPO_eval_lcoeff                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+  3))
#define MPO_eval_nthcoeff              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+  4))
#define MPO_eval_tcoeff                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+  5))
#define MPO_eval_mapcoeffs             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+  6))

#define MPO_eval_degree                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+  8))
#define MPO_eval_degreevec             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+  9))

#define MPO_eval_lterm                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 11))
#define MPO_eval_nterms                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 12))
#define MPO_eval_nthterm               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 13))

#define MPO_eval_evalp                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 15))
#define MPO_eval_expr                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 16))

#define MPO_eval_lmonomial             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 18))
#define MPO_eval_nthmonomial           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 19))

#define MPO_eval_pdivide               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 21))
#define MPO_eval_divide                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 22))
#define MPO_eval_genpoly               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 23))
#define MPO_eval_iszero                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 24))
#define MPO_eval_multcoeffs            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 25))
#define MPO_eval_norm                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 26))
#define MPO_eval_icontent              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 27))
#define MPO_eval_plus                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 28))
#define MPO_eval_mult                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 29))
#define MPO_eval_power                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 30))
#define MPO_diff_poly                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 31))
#define MPO_cmp_types                  (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 32))
#define MPO_equal_types                (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 33))
#define MPO_negate                     (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 34))
#define MPO_expr                       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 35))
#define MPO_poly_expr                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 36))
#ifdef MPO_DO_SIMPLIFY
#define MPO_simplify                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 37))
#endif
#define MPO_eval_Dpoly                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 38))
#define MPO_eval_ldegree               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MPO+ 39))

	/*** #015 = MSE *******************************************************/

#define MSE_new_set                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSE+  0))
#define MSE_insert                     (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSE+  1))
#define MSE_insert_set                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSE+  2))
#define MSE_union                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSE+  3))
#define MSE_minus                      (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSE+  4))
#define MSE_intersect                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSE+  5))

	/*** #010 = MSG *******************************************************/

#define MSGV_JumpTab                   ( ( void (**)(MDMD_ARGS)    ) MDM_K(MDM_MSG+  9))

	/*** #025 = MSR *******************************************************/

#define MSR_eval_strlen                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  0))
#define MSR_eval_strmatch              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  1))
#define MSR_eval_substring             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  2))
#define MSR_eval_tbl2text              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  3))
#define MSR_eval_text2tbl              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  4))
#define MSR_eval_text2list             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  5))
#define MSR_nchar2string               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  6))
#define MSR_concat_catstring           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  7))
#define MSR_concat_cstring             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  8))
#define MSR_concat_mmmstring           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSR+  9))

#define MSR_empty_string               (*(int        (*)(MDMD_ARGS)) MDM_K(MDM_MSR+ 11))
#define MSR_search_string              (*(int        (*)(MDMD_ARGS)) MDM_K(MDM_MSR+ 12))
#define MSR_strcmp                     (*(int        (*)(MDMD_ARGS)) MDM_K(MDM_MSR+ 13))
#define MSR_strcmp_skip                (*(int        (*)(MDMD_ARGS)) MDM_K(MDM_MSR+ 14))
#define MSR_get_next_string_from_list  (*(char*      (*)(MDMD_ARGS)) MDM_K(MDM_MSR+ 15))

	/*** #070 = MST *******************************************************/

#define MST_ins_stack                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MST+  0))
#define MST_new_stack                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MST+  1))
#define MST_del_item                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MST+  2))
#define MST_write_ident                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MST+  3))
#define MST_read_ident                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MST+  4))

#define MST_reset_stack                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MST+  7))
#define MST_read_netglobal             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MST+  8))
#define MST_set_netglobal              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MST+  9))
#define MST_lock_netglobal             (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MST+ 10))

#define MST_set_environment_variables  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MST+ 13))

#define MSTV_STACK                     (*(S_Pointer      *) MDM_K(MDM_MST+ 30))
#define MSTR_remember                  (*(S_Pointer      *) MDM_K(MDM_MST+ 31))
#define MSTR_LEVEL                     (*(S_Pointer      *) MDM_K(MDM_MST+ 32))
#define MSTR_hold                      (*(S_Pointer      *) MDM_K(MDM_MST+ 33))
#define MSTR_procname                  (*(S_Pointer      *) MDM_K(MDM_MST+ 34))
#define MSTR_args                      (*(S_Pointer      *) MDM_K(MDM_MST+ 35))
#define MSTR_Bin                       (*(S_Pointer      *) MDM_K(MDM_MST+ 36))
#define MSTR_Write                     (*(S_Pointer      *) MDM_K(MDM_MST+ 37))
#define MSTR_Append                    (*(S_Pointer      *) MDM_K(MDM_MST+ 38))
#define MSTR_IntMod                    (*(S_Pointer      *) MDM_K(MDM_MST+ 39))
#define MSTR_Expr                      (*(S_Pointer      *) MDM_K(MDM_MST+ 40))
#define MSTR_EULER                     (*(S_Pointer      *) MDM_K(MDM_MST+ 41))
#define MSTR_PI                        (*(S_Pointer      *) MDM_K(MDM_MST+ 42))
#define MSTR_E                         (*(S_Pointer      *) MDM_K(MDM_MST+ 43))
#define MSTR_simplify                  (*(S_Pointer      *) MDM_K(MDM_MST+ 44))
#define MSTR_eval                      (*(S_Pointer      *) MDM_K(MDM_MST+ 45))
#define MSTR_off                       (*(S_Pointer      *) MDM_K(MDM_MST+ 46))
#define MSTR_address                   (*(S_Pointer      *) MDM_K(MDM_MST+ 47))
#define MSTR_content                   (*(S_Pointer      *) MDM_K(MDM_MST+ 48))
#define MSTR_cyclic                    (*(S_Pointer      *) MDM_K(MDM_MST+ 49))
#define MSTR_Unquoted                  (*(S_Pointer      *) MDM_K(MDM_MST+ 50))
#define MSTR_DegreeOrder               (*(S_Pointer      *) MDM_K(MDM_MST+ 51))
#define MSTR_Rem                       (*(S_Pointer      *) MDM_K(MDM_MST+ 52))
#define MSTR_Quo                       (*(S_Pointer      *) MDM_K(MDM_MST+ 53))
#define MSTR_Exact                     (*(S_Pointer      *) MDM_K(MDM_MST+ 54))
#define MSTR_Unsimplified              (*(S_Pointer      *) MDM_K(MDM_MST+ 55))
#define MSTR_type                      (*(S_Pointer      *) MDM_K(MDM_MST+ 56))
#define MSTVparallel                   (*(long           *) MDM_K(MDM_MST+ 57))
#define MSTR_PRINTLEVEL                (*(S_Pointer      *) MDM_K(MDM_MST+ 58))
#define MSTR_Non                       (*(S_Pointer      *) MDM_K(MDM_MST+ 59))
#define MSTR_Warning                   (*(S_Pointer      *) MDM_K(MDM_MST+ 60))
#define MSTR_Error                     (*(S_Pointer      *) MDM_K(MDM_MST+ 61))
#define MSTR_User                      (*(S_Pointer      *) MDM_K(MDM_MST+ 62))
#define MSTR_protecterror              (*(S_Pointer      *) MDM_K(MDM_MST+ 63))
#define MSTR_PRETTY_PRINT              (*(S_Pointer      *) MDM_K(MDM_MST+ 64))

	/*** #045 = MSU *******************************************************/

#define MSU_eval_nops                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+  0))
#define MSU_eval_nops_intern           (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+  1))
#define MSU_eval_op                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+  2))
#define MSU_eval_op_intern             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+  3))
#define MSU_eval_op_list               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+  4))

#define MSU_eval_subs                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+  6))
#define MSU_eval_subs_intern           (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+  7))
#define MSU_eval_subsex                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+  8))
#define MSU_eval_subsex_intern         (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+  9))

#define MSU_eval_subsop                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 11))
#define MSU_eval_subsop_list           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 12))
#define MSU_eval_subsop_int            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 13))
#define MSU_eval_testtype              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 14))
#define MSU_eval_type                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 15))

#define MSU_eval_domtype               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 17))

#define MSU_get_operand                (*(S_Pointer* (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 19))
#define MSU_build_range                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 20))
#define MSU_search_and_subs            (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 21))
#define MSU_specialMINUS2expr          (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 22))
#define MSU_specialDIV2expr            (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 23))
#define MSU_specialNOT2expr            (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 24))
#define MSU_check_range                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 25))
#define MSU_check_equation             (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 26))

#define MSU_eval_extnops               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 28))
#define MSU_eval_extop                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 29))
#define MSU_eval_extsubsop             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSU+ 30))

#define MSUV_GET_OPERAND_INFO          (*(long           *) MDM_K(MDM_MSU+ 35))

	/*** #040 = MSV *******************************************************/

#define MSV_open                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  0))
#define MSV_close                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  1))
#define MSV_read                       (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  2))
#define MSV_write                      (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  3))
#define MSV_put_char                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  4))
#define MSV_put_long                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  5))
#define MSV_put_long1                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  6))
#define MSV_put_type                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  7))
#define MSV_put_header                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  8))
#define MSV_put_Mem_generic            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+  9))
#define MSV_put_Point                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 10))
#define MSV_put_S_Pointer              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 11))
#define MSV_get_char                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 12))
#define MSV_get_long                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 13))
#define MSV_get_long1                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 14))
#define MSV_get_type                   (*(int        (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 15))
#define MSV_get_header                 (*(int        (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 16))
#define MSV_get_Mem_generic            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 17))
#define MSV_get_Point                  (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 18))
#define MSV_get_S_Pointer              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSV+ 19))


	/*** #090 = MSY *******************************************************/

#define MSY_eval_last                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  0)) 
#define MSY_eval_history               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  1))
#define MSY_eval_nohistory             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  2))
#define MSY_eval_hold                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  3))
#define MSY_eval_help                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  4))
#define MSY_eval_system                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  5))
#define MSY_eval_ptime                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  6))
#define MSY_eval_rtime                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  7))
#define MSY_eval_time                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  8))
#define MSY_eval_trans_func            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+  9))
#define MSY_eval_arith_func            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 10))
#define MSY_eval_print                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 11))
#define MSY_eval_input                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 12))
#define MSY_eval_textinput             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 13))
#define MSY_eval_text2expr             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 14))
#define MSY_eval_level                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 15))
#define MSY_eval_eval                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 16))
#define MSY_eval_float                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 17))
#define MSY_eval_bool                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 18))
#define MSY_eval_reset                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 19))
#define MSY_eval_memory                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 20))
#define MSY_eval_bytes                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 21))
#define MSY_eval_load                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 22))
#define MSY_eval_anames                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 23))
#define MSY_eval_append                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 24))
#define MSY_eval_expr2text             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 25))
#define MSY_eval_ifactor               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 26))
#define MSY_eval_ithprime              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 27))
#define MSY_eval_phi                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 28))
#define MSY_eval_primedivisors         (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 29))
#define MSY_eval_numdivisors           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 30))
#define MSY_eval_ass_mem               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 31))
#define MSY_eval_val                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 32))
#define MSY_eval_error                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 33))
#define MSY_eval_traperror             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 34))
#define MSY_igcd                       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 35))
#define MSY_igcdex                     (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 36))
#define MSY_ilcm                       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 37))
#define MSY_sign                       (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 38))
#define MSY_stack_info                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 39))
#define MSY_arbitrary_args             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 44))
#define MSY_read_string_ass_mem        (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 41))
#define MSY_eval_contains              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 42))
#define MSY_eval_null                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 43))
#define MSY_fact                       (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 44))
#define MSY_round                      (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 45))
#define MSY_trunc                      (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 46))
#define MSY_eval_min                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 47))
#define MSY_eval_max                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 48))
#define MSY_eval_map                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 49))
#define MSY_eval_powermod              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 50))
#define MSY_eval_id                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 51))
#define MSY_eval_igamma                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 52))
#define MSY_eval_isqrt                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 53))
#define MSY_eval_sort                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 54))
#define MSY_eval_sysname               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 55))
#define MSY_eval_testargs              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 56))
#define MSY_eval_input_filter          (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 57))
#define MSY_is_proc_def                (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 58))
#define MSY_number_less                (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 59))
#define MSY_sort                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 60))
#define MSY_eval_zip                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 61))
#define MSY_eval_select                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 62))
#define MSY_eval_getpid                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 63))
#define MSY_eval_has                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 64))
#define MSY_eval_protected             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 65))
#define MSY_eval_split                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MSY+ 66))

#define MSYV_CODE_TAB                  (*(S_Pointer      *) MDM_K(MDM_MSY+ 75))
#define MSYV_EXEC_MOD                  (*(S_Pointer      *) MDM_K(MDM_MSY+ 76))
#define MSYV_EXEC_MODP                 (*(S_Pointer      *) MDM_K(MDM_MSY+ 77))
#define MSYV_EXEC_MODS                 (*(S_Pointer      *) MDM_K(MDM_MSY+ 78))
#define MSYV_TESTARGS                  (*(MUPbool        *) MDM_K(MDM_MSY+ 79))
#define MSYV_STR_PolyExpr              (*(S_Pointer      *) MDM_K(MDM_MSY+ 80))
#define MSYV_STR_RatExpr               (*(S_Pointer      *) MDM_K(MDM_MSY+ 81))

	/*** #010 = MTA *******************************************************/

#define MTA_eval_table_call            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTA+  0))
#define MTA_new_table                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTA+  1))
#define MTA_read_table                 (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MTA+  2))
#define MTA_write_table                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MTA+  3))

	/*** #060 = MTR *******************************************************/

#define MTR_new_code                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+  0))
#define MTR_new_cat_int                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+  1))
#define MTR_new_cat_task               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+  2))
#define MTR_new_cat_set_finite         (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+  3))
#define MTR_new_cat_stat_list          (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+  4))

#define MTR_new_cat_expr               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+  6))
#define MTR_new_cat_exec               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+  7))
#define MTR_new_cat_func_env           (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+  8))
#define MTR_new_user_func              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+  9))
#define MTR_new_sys_expr               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 10))
#define MTR_build_sys_expr             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 11))
#define MTR_new_sys_expr_empty         (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 12))
#define MTR_new_cat_ident_sys          (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 13))
#define MTR_new_cat_string_sys         (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 14))
#define MTR_new_cat_ident              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 15))
#define MTR_new_cat_string             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 16))
#define MTR_set_string                 (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 17))
#define MTR_set_int                    (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 18))
#define MTR_new_cat_bool               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 19))
#define MTR_new_cat_null               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 20))
#define MTR_new_cat_nil                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 21))
#define MTR_new_cat_failed             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 22))
#define MTR_new_tmp_string             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 23))
#define MTR_new_cat_dom                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 24))
#define MTR_new_cat_ext                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 25))

#ifdef MDX
#define MTR_new_cat_debug              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 26))
#endif

#define MTR_new_proc                   (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 27))
#define MTR_new_cat_funcarg            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 28))
#define MTR_shift_sys_expr             (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 29))
#define MTR_new_cat_point              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 30))
#define MTR_new_cat_polygon            (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MTR+ 31))

#define MTRV_SYS_FUNC_LIST             (*(S_Pointer*     *) MDM_K(MDM_MTR+ 50))
#define MTRV_sysfunc_name           ( (MTRT_sysfunc_name *) MDM_K(MDM_MTR+ 51))

	/*** #050 = MUP *******************************************************/

#define MUP_send_signal                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MUP+  0))
#define MUP_mainexit                   (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUP+  1))

#define MUPV_INTERACTIV                (*(short          *) MDM_K(MDM_MUP+ 15))
#define MUPV_PROC_DEPTH                (*(long           *) MDM_K(MDM_MUP+ 16))
#define MUPV_IN_PROC_DEPTH             (*(long           *) MDM_K(MDM_MUP+ 17))
#define MUPV_NEXT_PARALLEL_NUMBER      (*(long           *) MDM_K(MDM_MUP+ 18))
#define MUPV_PARALLEL_NUMBER           (*(long           *) MDM_K(MDM_MUP+ 19))
#define MUPV_PARALLEL_ENV              (*(S_Pointer      *) MDM_K(MDM_MUP+ 20))
#define MUPV_no_history                (*(MUPbool        *) MDM_K(MDM_MUP+ 21))
#define MUPV_CAT_NULL                  (*(S_Pointer      *) MDM_K(MDM_MUP+ 22))
#define MUPV_CAT_NIL                   (*(S_Pointer      *) MDM_K(MDM_MUP+ 23))
#define MUPV_CAT_BOOL_TRUE             (*(S_Pointer      *) MDM_K(MDM_MUP+ 24))
#define MUPV_CAT_BOOL_FALSE            (*(S_Pointer      *) MDM_K(MDM_MUP+ 25))
#define MUPV_STARTUP                   (*(MUPbool        *) MDM_K(MDM_MUP+ 26))
#define session_begin                  (*(long           *) MDM_K(MDM_MUP+ 27))
#define MUPV_CAT_FAILED                (*(S_Pointer      *) MDM_K(MDM_MUP+ 28))
#define MUPV_CAT_BOOL_UNKNOWN          (*(S_Pointer      *) MDM_K(MDM_MUP+ 29))
#define MUPV_INITIALIZING              (*(MUPbool        *) MDM_K(MDM_MUP+ 30))

#ifndef MACINTOSH
#define XMUPAD                         (*(int            *) MDM_K(MDM_MUP+ 40))
#define PPID                           (*(int            *) MDM_K(MDM_MUP+ 41))
#define confile                        (*(char*          *) MDM_K(MDM_MUP+ 42))
#endif

	/*** #045 = MUT *******************************************************/

#define MUT_unique_string              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  0))
#define MUT_new_cat_ident              (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  1))
#define MUT_print_statistics           (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  2))
#define MUT_free_extern_data           (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  3))
#define MUT_clock                      (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  4))
#define MUT_str_cmp                    (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  5))
#define MUT_ins_ext_list               (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  6))
#define MUT_new_ext_list               (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  7))
#define debug                          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  8))
#define MUT_concat_list                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MUT+  9))
#define MUT_read_list                  (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 10))
#define MUT_write_list                 (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 11))
#define MUT_print_error                (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 12))
#define MUT_ident_has_name             (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 13))
#define MUT_is_ident                   (*(MUPbool    (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 14))
#define MUT_similcmp                   (*(int        (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 15))
#define MUT_is_exprseq                 (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 16))
#define MUT_read_exprseq               (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 17))
#define MUT_write_exprseq              (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 18))
#define MUT_read_init_file             (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 19))
#define MUT_user_option                (*(const char*(*)(MDMD_ARGS)) MDM_K(MDM_MUT+ 20))

#define MUT_SYS_INIT_PATH              (*(char*          *) MDM_K(MDM_MUT+ 30))
#define MUT_USER_INIT_PATH             (*(char*          *) MDM_K(MDM_MUT+ 31))
#define MUT_LIB_INIT_PATH              (*(char*          *) MDM_K(MDM_MUT+ 32))
#define MUT_HELP_INDEX_PATH            (*(char*          *) MDM_K(MDM_MUT+ 33))
#define MUT_HELP_PATH                  (*(char*          *) MDM_K(MDM_MUT+ 34))
#define MUT_LIB_PATH                   (*(S_Pointer      *) MDM_K(MDM_MUT+ 35))
#define MUT_READ_PATH                  (*(S_Pointer      *) MDM_K(MDM_MUT+ 36))
#define MUT_WRITE_PATH                 (*(S_Pointer      *) MDM_K(MDM_MUT+ 37))
#define MUT_MODUL_PATH                 (*(char*          *) MDM_K(MDM_MUT+ 38))
#define MUT_PROG_NAME                  (*(char*          *) MDM_K(MDM_MUT+ 39))

	/*** #005 = MXP *******************************************************/

#define MXP_eval_expand                (*(S_Pointer  (*)(MDMD_ARGS)) MDM_K(MDM_MXP+  0))

	/*** #260 = PAR *******************************************************/

#define gtolong                        (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  0))
#define gcmp0                          (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  1))
#define gcmp1                          (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  2))
#define gcmp_1                         (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  3))
#define gegal                          (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  4))
#define etatpile                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  5))
#define dbltor                         (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  6))
#define stoi                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  7))
#define gcmp                           (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  8))
#define gtodouble                      (*(double     (*)(MDMD_ARGS)) MDM_K(MDM_PAR+  9))
#define bigprem                        (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 10))
#define gabs                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 11))
#define gceil                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 12))
#define gfloor                         (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 13))
#define gfrac                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 14))
#define isprime                        (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 15))
#define gach                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 16))
#define gacos                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 17))
#define gash                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 18))
#define gasin                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 19))
#define gatan                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 20))
#define gath                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 21))
#define gch                            (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 22))
#define gcos                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 23))
#define gexp                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 24))
#define glog                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 25))
#define gsh                            (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 26))
#define gsin                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 27))
#define gsqrt                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 28))
#define gtan                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 29))
#define gth                            (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 30))
#define gzeta                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 31))
#define ggamma                         (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 32))
#define gpsi                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 33))
#define gerfc                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 34))
#define eint1                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 35))
#define gmul                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 36))
#define gun                            (*(GEN                     *) MDM_K(MDM_PAR+ 37))
#define addii                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 38))

#define affrr                          (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 42))
#define avma                           (*(MMMulong                   *) MDM_K(MDM_PAR+ 43))
#define overflow                       (*(MMMulong                   *) MDM_K(MDM_PAR+ 44))
#define content                        (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 45))
#define ginv                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 46))
#define err                            (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 47))
#define ghalf                          (*(GEN                     *) MDM_K(MDM_PAR+ 48))
#define gadd                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 49))
#define gaffect                        (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 50))
#define gconj                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 51))
#define gcopy                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 52))
#define gdeux                          (*(GEN                     *) MDM_K(MDM_PAR+ 53))
#define gdiv                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 54))
#define gdivent                        (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 55))
#define gdivgs                         (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 56))
#define gerepile                       (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 57))
#define gexpo                          (*(long       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 58))
#define ggcd                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 59))
#define gi                             (*(GEN                     *) MDM_K(MDM_PAR+ 60))
#define gmod                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 61))
#define gmodulcp                       (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 62))
#define gneg                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 63))
#define gnil                           (*(GEN                     *) MDM_K(MDM_PAR+ 64))
#define gpui                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 65))
#define gpuigs                         (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 66))
#define greal                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 67))
#define gshift                         (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 68))
#define gsqr                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 69))
#define gsub                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 70))
#define gzero                          (*(GEN                     *) MDM_K(MDM_PAR+ 71))
#define hiremainder                    (*(MMMulong                   *) MDM_K(MDM_PAR+ 72))
#define killbloc                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 73))
#define gmulsg                         (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 74))
#define lontyp                         ( (long                    *) MDM_K(MDM_PAR+ 75))
#define lontyp2                        ( (long                    *) MDM_K(MDM_PAR+ 76))
#define mppi                           (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 77))
#define mulii                          (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 78))
#define newbloc                        (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 79))
#define bruteall                       (*(void       (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 80))
#define shifti                         (*(GEN        (*)(MDMD_ARGS)) MDM_K(MDM_PAR+ 81))

	/*** #010 = MDB *******************************************************/

        /*** #100 = MKT *******************************************************/

#define MFprintf1                      (*(int        (*)(MDMD_ARGS)) MDM_K(MDM_MKT+  0))
#define MFstring1                      (*(char*      (*)(MDMD_ARGS)) MDM_K(MDM_MKT+  1))
#define MFcomplex1                     (*(MTcell     (*)(MDMD_ARGS)) MDM_K(MDM_MKT+  2))

#define MFrat1                         (*(MTcell     (*)(MDMD_ARGS)) MDM_K(MDM_MKT+  4))

#define MFvalue1                       (*(MTcell     (*)(MDMD_ARGS)) MDM_K(MDM_MKT+  6))
#define MFnewExpr1                     (*(MTcell     (*)(MDMD_ARGS)) MDM_K(MDM_MKT+  7))
#define MFnewExpr2                     (*(MTcell     (*)(MDMD_ARGS)) MDM_K(MDM_MKT+  8))
#define MFcall1                        (*(MTcell     (*)(MDMD_ARGS)) MDM_K(MDM_MKT+  9))

#define MV_errnum                      (*(long           *) MDM_K(MDM_MKT+ 80))
#define MV_errstr                      (*(char*          *) MDM_K(MDM_MKT+ 81))
#define MV_result                      (*(MTcell         *) MDM_K(MDM_MKT+ 82))

	/*** #000 = END *******************************************************/


#endif /* MDMD_USE_DYNLINK */


/******************************************************************************/
/*** Makros zur Unterstuetzung der Modulprogrammierung (nur fuer ANSI-C)    ***/
/******************************************************************************/
#define OLD_MODULE_STYLE              /*** Aus Gruenden der Kompatibilitaet ***/
#ifdef  OLD_MODULE_STYLE

/*** Die C-Name der Modulfunktionen beginnt stets mit MF_eval_... ! ***********/

#define MDMD_C_NAME(Name)	MD_FUNC_NAME(Name)
#define MDMD_INTERN(Name)	MD_FUNC_NAME(__ ## Name)

/*** Funktionsheader der oeffentlichen Modulfunktionen ************************/

#define MODUL_FUNC(Name)			\
        S_Pointer MDMD_C_NAME(Name) (		\
                  S_Pointer       s,		\
                  long            prev_func,	\
                  long            eval_type,	\
                  S_Pointer       exec		\
        )

#define MODULE_FUNC(Name)			\
	MODUL_FUNC(Name)
#define MODUL_FUNC_INTERN(Name)			\
	MODUL_FUNC(MDMD_INTERN(Name))
#define MODULE_FUNC_INTERN(Name)		\
	MODUL_FUNC(MDMD_INTERN(Name))

#endif /* OLD_MODULE_STYLE */

#endif /* __MDM_mod__ */
