/* $Date: 1997/12/15 15:22:18 $  $Author: olli $  $Revision: 1.9 $ */

/* 
 * Diese Datei enth"alt alle Typendefinitionen, die das Grafik-
 * tool VCam ben"otigt. Einzige Ausnahme dabei sind die Typen,
 * die f"ur das rechnerunabh"angige Zeichnen entworfen wurden.
 * Diese Typen sind im File MdX_types.h definiert.
 *
 * Autor: Thorsten Schulze
 * Datum: 17.04.93
 */

#ifndef __MGR_types__
#define __MGR_types__

#ifdef DOS_FILENAMES
#   include "MGR_cons.h"
#else
#   include "MGR_constants.h"
#endif

#ifndef NO_TICKS_EXTENSION

/**************************************************************************

Bedeutung der Eintraege der Struktur MGRT_Ticks anhand der x-Achse (analog 
fuer die y- und z-Achse, d.h. Index ist 1 bzw. 2):

1) ticks.mode & MGRC_NUM_X :
   ticks.n[0] spezifiziert die Anzahl der Ticks. Die Werte der Ticks werden
   von Vcam automatisch berechnet.
2) ticks.mode & MGRC_DELTA_X :
   ticks.delta[0] spezifiziert die Abstaende zwischen den Ticks, d.h. es werden
   Ticks an den Stellen i * ticks.delta zwischen ticks.min[0] und ticks.max[0]
   gezeichnet. Ist ticks.auto_min_max & MGRC_MIN_X, so wird anstelle von 
   ticks.min[0] der Wert scene.auto_vb.xmin genommen. Ist 
   ticks.auto_min_max & MGRC_MIN_X, so wird anstelle von ticks.min[0] der Wert
   scene.auto_vb.xmax genommen. Jeder Tick wird beschriftet.
   Zwischen je zwei Ticks werden noch ticks.div[0]-1 Ticks gezeichnet, die
   allerdings nicht beschriftet werden.
   Wurden ticks.min[0] und ticks.max[0] explizit vom Benutzer spezifiziert,
   so bewirken diese Werte, dass die ViewingBox vergroessert wird, d.h.
   scene.auto_vb.xmin = min(x.ticks.min, scene.auto_vb.xmin)
   scene.auto_vb.xmax = min(x.ticks.max, scene.auto_vb.xmax)
3) ticks.mode & MGRC_LIST_X :
   Es werden n = ticks.n[0] Ticks an den Stellen ticks.mark[0][0] bis
   ticks.mark[0][n-1] gezeichnet. Ist ticks.label[0][i][0] == MGRC_NO_TEXT,
   so wird der i-te Tick mit seinem Wert beschriftet. Ansonsten wird der Tick
   mit ticks.label[0][i] beschriftet.

4) Sonderfall (Kompatibilitaet zu aelteren Versionen) 
   ticks.mode & MGRC_NUM_XYZ :
   tick spezifiziert die Anzahl der Ticks aller Achsen. Die Werte der Ticks
   werden von Vcam automatisch berechnet.

****************************************************************************/

typedef struct
{   long      mode;

    long      auto_min_max;
    double    min    [3];
    double    max    [3];
    double    delta  [3];
    long      div    [3];

    long      n      [3];

    double    mark [3][MGRC_MAX_TICKS];
    char      label[3][MGRC_MAX_TICKS][MGRC_STRLEN_TICKS];

}   MGRT_Ticks;

/* 
 * Definition des Typs MGRT_Axes.
 * 
 * Die einzelnen Eintr"age haben folgende Bedeutung:
 *
 *   style        : Gibt den Achsenstil an und kann die Werte
 *                  MGRC_NO_AXES,
 *                  MGRC_NORMAL_AXES,
 *                  MGRC_FRAMED_AXES,
 *                  MGRC_BOXED_AXES
 *                  annehmen.
 *   arrows       : Gibt an, ob Achsenpfeile oder nicht und kann
 *                  die Werte
 *                  MGRC_NO_ARROWS,
 *                  MGRC_ARROWS
 *                  annehmen.
 *   labels       : Gibt an, ob Beschriftungen an die Achsen ge-
 *                  zeichnet werden, und kann die Werte
 *                  MGRC_NO_LABELS,
 *                  MGRC_LABELS
 *                  annehmen.
 *   tick         : Gibt die Anzahl der Achsenmarkierungen an und
 *                  kann Werte zwischen MGRC_MIN_TICKS und
 *                  MGRC_MAX_TICKS annehmen.
 *   xlab         : String der x-Achsenbeschriftung.          
 *   ylab         : String der y-Achsenbeschriftung           
 *   zlab         : String der z-Achsenbeschriftung           
 *   origin       : gibt an, ob automatische Achsen oder nicht,
 *                  m"ogliche Werte: MGRC_AUTOMATIC, MGRC_MANUAL
 *   originx      : x-Komponente des Achsenschnittpunktes     
 *   originy      : y-Komponente des Achsenschnittpunktes     
 *   originz      : z-Komponente des Achsenschnittpunktes 
 */

typedef struct                                                
{   long        style;                                           
    long        arrows;                                          
    long        labels;                                          
    long        tick;
    MGRT_Ticks  ticks;                                            
    char        xlab[MGRC_STRLEN_LABELS];                        
    char        ylab[MGRC_STRLEN_LABELS];                        
    char        zlab[MGRC_STRLEN_LABELS];                        
    long        origin;                                          
    double      originx;                                         
    double      originy;                                         
    double      originz;                                         
}   MGRT_Axes;  

#else

typedef struct
{   long     style;                       
    long     arrows;                 
    long     labels;                
    long     tick;                   
    char     xlab[MGRC_STRLEN_LABELS];
    char     ylab[MGRC_STRLEN_LABELS];
    char     zlab[MGRC_STRLEN_LABELS];   
    long     origin;
    double   originx;                   
    double   originy;                   
    double   originz;                   
}   MGRT_Axes;

#endif

/*
 * Definition des Typs MGRT_ViewBox
 *
 * Die einzelnen Eintr"age haben die folgende Bedeutung:
 *
 *  xmin   : Minimale Koordinate der VB in x-Richtung
 *  xmax   : Maximale Koordinate der VB in x-Richtung
 *  ymin   : Minimale Koordinate der VB in y-Richtung
 *  ymax   : Maximale Koordinate der VB in y-Richtung
 *  zmin   : Minimale Koordinate der VB in z-Richtung
 *  zmax   : Maximale Koordinate der VB in z-Richtung
 */

typedef struct 
{   double   xmin, xmax; 
    double   ymin, ymax;               
    double   zmin, zmax;
}   MGRT_Viewbox;


/*
 * Defintion des Typs MGRT_Color
 *
 * Die einzelnen Eintr"age haben die Bedeutung
 *
 *  index  : Gibt den Index der Farbe in der Farbtafel an.
 *           Dieser Eintrag ist nur f"ur X erforderlich.
 *  red    : Gibt den Rot-Wert der Farbe an.
 *  green  : Gibt den Gr"un-Wert der Farbe an.
 *  blue   : Gibt den Blau-Wert der Farbe an.
 */

typedef struct 
{   long    index;
    long    red;                       
    long    green;                     
    long    blue; 
}   MGRT_Color;



/*
 * Definition des Typs DrawStyle
 * 
 * Die einzelnen Eintr"age haben die Bedeutung
 *
 *  powi   : Gibt die Breite eines Punktes beim Zeichnen an.
 *           Kann Werte zwischen MGRC_MIN_POINT_WIDTH und 
 *           MGRC_MAX_POINT_WIDTH annehmen.
 *  post   : Gibt den Stil eines Punktes beim Zeichnen an und
 *           kann Werte aus 
 *           MGRC_FILLED_SQUARE_POINT,
 *           MGRC_SQUARE_POINT,
 *           MGRC_FILLED_CIRCLE_POINT,
 *           MGRC_CIRCLE_POINT,
 *           annehmen.
 *  liwi   : Gibt die Breite der Linien beim Zeichnen an und kann
 *           Werte zwischen MGRC_MIN_LINE_WIDTH und
 *           MGRC_MAX_LINE_WIDTH annehmen.
 *  list   : Gibt den Stil der Linien beim Zeichnen an und kann
 *           Werte aus
 *           MGRC_SOLID_LINE,
 *           MGRC_DASHED_LINE
 *           annehmen.
 */

typedef struct 
{   long     powi; 
    long     post;  
    long     liwi;       
    long     list;   
}   MGRT_DrawStyle;


/* 
 * Definition des Typ MGRT_Font
 *
 * Die einzelnen Eintr"age haben die Bedeutung:
 *
 *  family : Gibt die Familie des Fonts an.
 *  style  : Gibt den Stil des Fonts an.
 *  scale  : Gibt die Gr"osse des Fonts an und kann Werte zwischen
 *           MGRC_MIN_FONT_SIZE und MGRC_MAX_FONT_SIZE annehmen.
 */

typedef struct
{   char     family[MGRC_STRLEN_FONTNAME];
    char     style[MGRC_STRLEN_FONTSTYLE];
    long     scale;
}   MGRT_Font;



/*
 * neu eingef"ugt am 26.11.1993
 * Definition des Typs MGRT_Scal
 * beinhaltet die Skalierungsparameter.
 */

typedef struct
{   double    xmin;
    double    xmax;
    double    ymin;
    double    ymax;
}   MGRT_Scal;


/*
 * Definition des Typs MGRT_Scene
 *
 * Die einzelnen Eintr"age haben die folgenden Bedeutungen:
 *
 *  changes  : Gibt an, ob sich die Szene ge"andert hat, kann
 *             die Werte MGRC_NO_CHANGES oder MGRC_CHANGES annehmen.
 *             Dabei bedeutet MGRC_NOT_CHANGED immer, dass sich die
 *             Szene-Optionen garantiert nicht ge"andert haben. Dieses
 *             bezieht sich aber nur auf die Szene-Optionen.
 *  dime     : Dimension der Szene. M"ogliche Werte sind
 *             MGRC_SCENE_2D und MGRC_SCENE_3D
 *  acob     : Gibt die Nummer des aktuelle Objektes an.
 *  nuob     : Gibt die Anzahl der Objekte an.
 *  rout     : Gibt die Routine an mit der gezeichnet wird.
 *             Ist entweder MGRC_QUICKDRAW oder MGRC_QUALITYDRAW
 *  axes     : Gibt den aktuellen Achsenstil an.
 *  scal     : Gibt die Skalierungsroutine an. M"ogliche Werte:
 *             MGRC_SCAL_UNCONSTRAINED und MGRC_SCAL_CONSTRAINED
 *  auto_scal: Beinhaltet die automatischen Skalierungswerte
 *  manu_scal: Beinhaltet die manuellen Skalierungswerte
 *  back     : Gibt die Hintergrundfarbe an.
 *  fore     : Gibt die Vordergrundfarbe an.
 *  view     : Gibt an, ob automatische VB oder nicht. M"ogliche
 *             Werte MGRC_AUTO_VIEWBOX und MGRC_MANU_VIEWBOX
 *  auto_vb  : Zum Speichern der automatischen VB-Werte.
 *  manu_vb  : Zum Speichern der aktuellen VB-Werte.
 *  pers     : Gibt an, ob automatische Perspektive oder nicht.
 *             M"ogliche Werte MGRC_AUTO_PERS oder MGRC_MANU_PERS 
 *             oder MGRC_MANU_CP oder MGRC_MANU_FP.
 *  camx     : x-Komponente des CameraPoints
 *  camy     : y-Komponente des CameraPoints
 *  camz     : z-Komponente des CameraPoints
 *  focx     : x-Komponente des FocalPoints
 *  focy     : y-Komponente des FocalPoints
 *  focz     : z-Komponente des FocalPoints
 *  radius   : \ 
 *  phi      : - Polarkoordinaten des CameraPoints
 *  theta    : /
 *  ligh     : Gibt an, ob Beleuchtung oder nicht. M"ogliche Werte
 *           : MGRC_NO_LIGHTING und MGRC_LIGHTING.
 *  title    : Gibt den Titel der Szene an.
 *  posx     : Relative x-Koordinate des Szenetitels
 *  posy     : Relative y-Koordinate des Szenetitels
 *  drst     : Gibt die Zeichenstile f"ur Punkte und Linien an.
 *  font     : Gibt den gew"ahlten Font an.
 *  ax_scal  : Gibt an, ob die X- und/oder Y-Achse und/oder Z-Achse 
 *             logarithmisch skaliert ist. Moegliche Werte sind:
 *             MGRC_<x><y><z> , wobei <x>, <y> bzw. <z> Lin oder Log ist.
 */

typedef struct
{   long               changes;
    long               dime;   
    long               acob;
    long               rout;
    long               nuob; 
    MGRT_Axes          axes; 
    long               scal;   
    MGRT_Scal          auto_scal;
    MGRT_Scal          manu_scal;
    MGRT_Color         back;   
    MGRT_Color         fore;
    long               view;   
    MGRT_Viewbox       auto_vb;   
    MGRT_Viewbox       manu_vb;
    long               pers;
    double             camx;                      
    double             camy;  
    double             camz;
    double             focx;
    double             focy; 
    double             focz;
    double             radius;
    long               phi;
    long               theta;
    long               ligh;  
    char               title[MGRC_STRLEN_TITLE];
    double             posx;
    double             posy;
    MGRT_Font          font;
    MGRT_DrawStyle     drst;
    long               ax_scal;
    double             shift_x;
    double             shift_y;
    double             shift_z;
}   MGRT_Scene;



/* 
 * Definition des Typs MGRT_Object
 *
 * Die Eintr"age habe die Bedeutung:
 *
 *  changes  : Gibt an, ob sich die Object-Optionen ge"andert haben,
 *             kann die Werte MGRC_NO_CHANGES oder MGRC_CHANGES an-
 *             men.
 *  title    : Zeichenkette des Objekttitels
 *  posx     : Relative x-Koordinate des Objekttitels
 *  posy     : Relative y-Koordinate des Objekttitels
 *  numb     : Gibt die Objektnummer an, beginnt bei 1.
 *  plotted  : Gibt an, ob das Objekt schon mal geplotted
 *             wurde. M"ogliche Werte sind
 *             MGRC_NOT_PLOTTED und MGRC_PLOTTED.
 *  mode     : Gibt den Modus des Objektes an. M"ogliche Werte
 *             MGRC_MODE_CURVE_2D,
 *             MGRC_MODE_LIST_2D,
 *             MGRC_MODE_CURVE_3D,
 *             MGRC_MODE_CONTOUR_3D,
 *             MGRC_MODE_SURFACE_3D,
 *             MGRC_MODE_LIST_3D.
 *  styl     : Gibt den Stil des Objektes an. M"ogliche Werte
 *             MGRC_STYLE_POINTS,
 *             MGRC_STYLE_LINES,
 *             MGRC_STYLE_LINESPOINTS
 *             MGRC_STYLE_IMPULSES
 *             MGRC_STYLE_WIREFRAME
 *             MGRC_STYLE_HIDDENLINE
 *             MGRC_STYLE_COLORPATCHES
 *             MGRC_STYLE_DEPTHCUEING
 *  mesh     : Gibt an, welche Linien gezeichnet werden. Werte:
 *             MGRC_MESH_ONLY,
 *             MGRC_MESH_AND,
 *             MGRC_MESH_ULINE,
 *             MGRC_MESH_VLINE.
 *  colo     : Gibt an, welche Farbfunktionalit"at gew"ahlt wird.
 *             M"Ogliche Werte:
 *             MGRC_COLO_FLAT,
 *             MGRC_COLO_HEIGHT,
 *             MGRC_COLO_FUNC
 *             MGRC_COLO_PHYS
 *  star     : Gibt die Anfangsfarbe an.
 *  end      : Gibt die Endfarbe an.
 *  fine     : Gibt an, ob die Feinstruktur gew"ahlt wurde. Werte
 *             MGRC_NO_FINE oder MGRC_FINE.
 *  para     : Gibt an, ob ein karthesisches Koordinatensystem 
 *             gew"ahlt wurde, m"ogliche Werte sind 
 *             MGRC_NO_PARA oder MGRC_PARA.
 *  obvb     : Gibt die minimale Box (Quader) an, die (der) das 
 *             Objekt noch ganz enth"alt.
 *  cfun     : Zeichenkette f"ur die vom Benutzer gew"ahlte Farb-
 *             funktion
 *  fun1     : Zeichenkette f"ur die x-Funktion.
 *  fun2     : Zeichenkette f"ur die y-Funktion.
 *  fun3     : Zeichenkette f"ur die z-Funktion.
 *  indet_u  : Zeichenkette f"ur die erste Variable.
 *  indet_v  : Zeichenkette f"ur die zweite Variable.
 *  umin     : Zeichenkette f"ur die minimale Bereichsgrenze von u
 *  umax     : Zeichenkette f"ur die maximale Bereichsgrenze von u
 *  vmin     : Zeichenkette f"ur die minimale Bereichsgrenze von v
 *  vmax     : Zeichenkette f"ur die maximale Bereichsgrenze von v
 *  ugrid    : Anzahl der auszuwertenden St"utzstellen. Werte aus
 *             MGRC_MIN_GRID und MGRC_MAX_GRID.
 *  usmooth  : Anzahl der zus"atzlichen St"utzstellen mit Werten
 *             aus MGRC_MIN_SMOOTH und MGRC_MAX_SMOOTH.
 *  vgrid    : Anzahl der auszuwertenden St"utzstellen. Werte aus
 *             MGRC_MIN_GRID und MGRC_MAX_GRID.
 *  vsmooth  : Anzahl der zus"atzlichen St"utzstellen mit Werten
 *             aus MGRC_MIN_SMOOTH und MGRC_MAX_SMOOTH.
 */

typedef struct 
{   long          changes;
    char          title[MGRC_STRLEN_TITLE];  
    double        posx;                      
    double        posy;                      
    long          numb;                      
    long          plotted;                   
    long          mode;                      
    long          styl;                      
    long          mesh;                      
    long          colo;                      
    MGRT_Color    star;                      
    MGRT_Color    end;                       
    long          fine;                      
    long          para;                      
    MGRT_Viewbox  obvb;                      
    char          cfun[MGRC_STRLEN_FUNCTION];   
    char          fun1[MGRC_STRLEN_FUNCTION];   
    char          fun2[MGRC_STRLEN_FUNCTION];   
    char          fun3[MGRC_STRLEN_FUNCTION];
    char          indet_u[MGRC_STRLEN_VARIABLE];
    char          indet_v[MGRC_STRLEN_VARIABLE];   
    char          umin[MGRC_STRLEN_RANGE];   
    char          umax[MGRC_STRLEN_RANGE];   
    char          vmin[MGRC_STRLEN_RANGE];   
    char          vmax[MGRC_STRLEN_RANGE];   
    long          ugrid;                     
    long          usmooth;                   
    long          vgrid;                     
    long          vsmooth; 
    /*
     * Neu: T.S. 11.08.95 Punkt-Linien-Breite-Stil
     * als Objekt-Attribute eingef"ugt.
     */
    MGRT_DrawStyle drst;                  
}   MGRT_Object;




/*
 * Definition des Typs MGRT_Defaults.
 * 
 * Die einzelnen Eintr"age haben die folgende Bedeutung
 *
 *  rout     : GIbt den Default-Wert der Berechnungsroutine an.
 *             Werte sind MGRC_ROUT_QUICKDRAW oder
 *             MGRC_ROUT_QUALITYDRAW.
 *  colo     : Gibt den Default-Wert der Farbfunktionalit"at an.
 *             Werte sind
 *             MGRC_COLO_FLAT,
 *             MGRC_COLO_HEIGHT,
 *             MGRC_COLO_FUNC
 *             MGRC_COLO_PHYS
 *  scal     : Gibt den Default-Wert der Skalierungsroutine an.
 *             Werte sind aus MGRC_SCAL_CONSTRAINED oder
 *             MGRC_SCAL_UNCONSTRAINED.
 *  axes     : Gibt die Default-Werte f"ur den Achsenstil an.
 *  back     : Gibt den Default-Wert f"ur die Hintergrundfarbe an
 *  fore     : Gibt den Default-Wert f"ur die Vordergrundfarbe an.
 *  mo2d     : Gibt den Default-Wert f"ur den Modus eines 2D-Ob-
 *             jektes an. Werte sind aus
 *             MGRC_MODE_CURVE_2D,
 *             MGRC_MODE_LIST_2D
 *  mo3d     : Gibt den Default-Wert f"ur den Modus eines 3D-Ob-
 *             jektes an. Werte sind aus
 *             MGRC_MODE_CURVE_3D,
 *             MGRC_MODE_CONTOUR_3D,
 *             MGRC_MODE_SURFACE_3D,
 *             MGRC_MODE_LIST_3D
 *  cust     : Gibt den Default-Wert f"ur den Zeichenstil eines
 *             Objektes vom Modus MGRC_MODE_CURVE_(2D/3D) an.
 *             Werte sind aus
 *             MGRC_STYLE_POINTS,
 *             MGRC_STYLE_LINES,
 *             MGRC_STYLE_LINESPOINTS
 *             MGRC_STYLE_IMPULSES
 *  sust     : Gibt den Default-Wert f"ur den Zeichenstil eines
 *             Objektes vom Modus MGRC_MODE_SURFACE_3D an.
 *             Werte sind aus
 *             MGRC_STYLE_POINTS,
 *             MGRC_STYLE_WIREFRAME,
 *             MGRC_STYLE_HIDDENLINE,
 *             MGRC_STYLE_COLORPATCHES,
 *             MGRC_STYLE_DEPTHCUEING.
 *  mesh     : Gibt den Default-Wert f"ur das Mesh eines Objektes
 *             dvom Modus MGRC_NODE_SURFACE_3D an. Werte sind aus
 *             MGRC_MESH_ONLY,
 *             MGRC_MESH_AND,
 *             MGRC_MESH_ULINE,
 *             MGRC_MESH_VLINE.
 *  grid     : Gibt den Default-Wert f"ur die Anzahl der St"utzstellen
 *             an.
 *             Werte liegen zwischen MGRC_MIN_GRID<= grid <= MGRC_MAX_GRID.
 *  smooth   : Gibt den Default-Wert f"ur die Anzahl der interpolierenden
 *             St"utzstellen an. 
 *             Werte liegen zwischen MGRC_MIN_SMOOTH<= smooth <= MGRC_MAX_SMOOTH.
 *  ligh     : Gibt den Default-Wert f"ur die Beleuchtung an. 
 *             Werte sind MGRC_NO_LIGHTING oder MGRC_LIGHTING.
 *  drst     : Default-Wert f"ur den Zeichenstil.
 *  font     : Default-Wert f"ur den beim Zeichnen verwendeten
 *             Font.
 *  indet_u  : Zeichenkette f"ur die erste Default-Variable
 *  indet_v  : Zeichenkette f"ur die zweite Default-Variable
 */
typedef struct 
{   long            rout;                     
    long            scal; 
    long            ligh;                                         
    MGRT_Axes       axes;                     
    MGRT_DrawStyle  drst;  
    MGRT_Font       font;              
    long            mo2d;                     
    long            mo3d; 
    long            cust;                     
    long            sust;                     
    long            mesh;  
    long            grid;
    long            smooth;
    long            colo;   
    char            indet_u[MGRC_STRLEN_VARIABLE];
    char            indet_v[MGRC_STRLEN_VARIABLE];
    MGRT_Color      back;                  
    MGRT_Color      fore;
    MGRT_Color      deco[36];
}   MGRT_Defaults;


/*
 * Definition des Typs MGRT_BoxEdge. Dieser Typ dient
 * im Zoom-Fenster zur Berechnung der neuen ViewingBox-
 * Werte. In den einzelnen Arrays werden jeweils die
 * reellen Koordinaten sowie die zugeh"origen Bildkoor-
 * dinaten einer einzelnen Boxkante gespeichert.
 * Verwendet wird im Zoom-Fenster je nach Dimension der
 * Szene eine Array bestehend aus vier Eintr"agen des Typs
 * MGRT_BoxEdge bei 2D bzw. ein Array bestehend aus 12 Ein-
 * tr"agen des Typs MGRT_BoxEdge
 * 
 * Die Eintr"age haben die Bedeutung
 * 
 *  x     : reelle x-Koordinaten der beiden Endpunkte
 *  y     : reelle y-Koordinaten der beiden Endpunkte
 *  z     : reelle z-Koordinaten der beiden Endpunkte
 *  nx    : x-Bildkoordinaten der beiden Endpunkte.
 *  ny    : y-Bildkoordinaten der beiden Endpunkte.
 */

typedef struct
{   double    x[2];
    double    y[2];
    double    z[2];
    long      nx[2];
    long      ny[2];
}   MGRT_BoxEdge;



/*
 * Definition des Typs MGRT_Point3d.
 * 
 * Dieses ist der alte Typ point3d. 
 *
 * Die einzelnen Eintr"age bestimmen dabei die einzelnen Koor-
 * dinaten des Punktes.
 */

typedef struct
{   double    x;
    double    y;
    double    z;
}   MGRT_Point3d;


/* 
 * Definition des Typs MGRT_Point2d.
 *
 * Diesen struct gab es noch nicht, aber ich nehme an, dass
 * wir ihn sowieso noch ben"otigen. Die einzlenen Eintr"age
 * geben wiederum die einzelnen Koordinaten des 2D-Punktes an.
 */

typedef struct
{   double    x;
    double    y;
}   MGRT_Point2d;


#endif /* __MGR_types__ */
