/* $Date: 1997/07/07 11:57:30 $  $Author: olli $  $Revision: 1.13 $ */

/**************************************************************************/
/* FILE:             MGS_simplify.h                                       */
/* Autor:            Oliver Kluge                                         */
/* Erstellt:         01/07/91                                             */ 
/* Letzte Aenderung: 03/03/97                                             */ 
/*                                                                        */ 
/* Ab Revision 1.9 (25.10.96) gibt es das Preprocessor-Flag NO_SPEC_BITS  */
/* Hiermit werden die Abfragen auf die Special-Bits MINUS_BIT, DIV_BIT    */
/* und nicht mehr durchgef"uhrt, da davon ausgegangen wird, das keine     */
/* Funktion, insbesondere der Parser und der Simplifier, mehr solche Bits */
/* generieren.  (Oliver Kluge)                                            */
/* Ab Revision 1.12 (3.3.97) gibt es nur noch die Version ohne Special-   */ 
/* Bits. Das Flag NO_SPEC_BITS wurde entfernt.                            */
/**************************************************************************/

#ifndef __MGS_simplify__
#define __MGS_simplify__


#ifdef DOS_FILENAMES
#  include "MGS_simh.h" 
#else 
#  include "MGS_simplify_help.h"
#endif


/*** Wichtige Konstanten des Moduls ******************************************/

#define MGSC_FLAT_OP      1L      /* Zur Bedeutung dieser Konstanten siehe:  */
#define MGSC_FLAT_ARG     2L      /* MGS_simplify.c, Routine MGS_flat().     */
#define MGSC_FLAT_NO_ARG  0L
#define MGSC_FLAT_LIST	  4L
#define MGSC_FLAT_ARG_2   8L      /* Muss MGSC_FLAT_ARG beinhalten !	     */

#define MGSC_QUICKSORT   25L      /* Unterhalb dieser Grenze wird per Ein-   */
				  /* fuegen sortiert, dar"uber per Quicksort.*/


/*** Deklaration der Funktionen, die Schnittstelle des Moduls sind ***********/

S_Pointer MGS_eval_set_length     ( S_Pointer s,
                                    long      prev_func,
                                    long      eval_type,
                                    S_Pointer exec );
 
void MGS_init                      ( VOID );
void MGS_exit                      ( VOID );

void MGS_quicksort                 ( S_Pointer *s, long le, long *rp, long op_typ, 
                                     long prev_func, long eval_type, S_Pointer function );

void MGS_quicksort_generic         ( S_Pointer *s, long le, long *rp, long op_typ, 
                                     long (*cmp_func)(S_Pointer,S_Pointer), 
                                     void (*sum_func)(S_Pointer*,long,long,long*,long,long*), 
                                     long prev_func, long eval_type, S_Pointer function );

void MGS_straightinsertion         ( S_Pointer *s, long l, long *r, long op_typ, 
                                     long prev_func, long eval_type, S_Pointer function );

void MGS_straightinsertion_generic ( S_Pointer *s, long l, long *r, long op_typ, 
                                     long (*cmp_func)(S_Pointer, S_Pointer), 
                                     void (*sum_func)(S_Pointer*,long,long,long*,long,long*), 
                                     long prev_func, long eval_type, S_Pointer function );

void MGS_summarize_and_or_union_intersect ( S_Pointer *s, long i, long j, long *r, 
                                            long op, long *dummy );

void MGS_compress                  ( S_Pointer s, long *upperbound, long count_blank, long op_typ, 
                                     long prev_func, long eval_type, S_Pointer function );

long MGS_cmp                       ( long i, long j, S_Pointer sp_i, S_Pointer sp_j, 
                                     long op_typ );

MUPbool MGS_flat                   ( S_Pointer *r, long flat_flag );

MUPbool MGS_combine_constants      ( S_Pointer *s, long lower_const_typ, long upper_const_typ, 
                                     long op_typ, long *new_counter );


long      MGS_cmp_and_or            ( S_Pointer s, S_Pointer t );

MUPbool   MGS_contains_not          (S_Pointer s );

S_Pointer MGS_normalize_not         ( S_Pointer s );

S_Pointer MGS_negate_               ( S_Pointer s, MUPbool Unsimplified );
S_Pointer MGS_reciprocal_           ( S_Pointer s, MUPbool Unsimplified );

S_Pointer MGS_divide_               ( S_Pointer s, S_Pointer t, MUPbool Unsimplified );
S_Pointer MGS_minus_                ( S_Pointer s, S_Pointer t, MUPbool Unsimplified );

/***********************************************************/
/* Berechne 1/s. Das Ergebnis ist kopiert und vereinfacht. */
/***********************************************************/

#define MGS_reciprocal( s )     MGS_reciprocal_( s, _false_ )

/***********************************************************/
/* Berechne -s. Das Ergebnis ist kopiert und vereinfacht. */
/***********************************************************/

#define MGS_negate( s )         MGS_negate_( s, _false_ )

/***********************************************************/
/* Berechne s/t. Das Ergebnis ist kopiert und vereinfacht. */
/***********************************************************/

#define MGS_divide( s, t )      MGS_divide_( s, t, _false_ )

/***********************************************************/
/* Berechne s-t. Das Ergebnis ist kopiert und vereinfacht. */
/***********************************************************/

#define MGS_minus( s, t )       MGS_minus_( s, t, _false_ )


/*** Definition von Makros ***************************************************/

#define MGS_sort_generic(s, l, r, op_typ, cmp_func, sum_func, prev_func, eval_type, func)       \
 {                                                                                              \
   if( *(r)-(l) >= MGSC_QUICKSORT )                                                             \
        MGS_quicksort_generic        (s,l,r,op_typ,cmp_func,sum_func,prev_func,eval_type,func); \
   else MGS_straightinsertion_generic(s,l,r,op_typ,cmp_func,sum_func,prev_func,eval_type,func); \
 }


#define MGS_sort(s, l, r, op_typ, prev_func, eval_type, func)               \
 {                                                                          \
   if( *(r)-(l) >= MGSC_QUICKSORT )                                         \
        MGS_quicksort        (s, l, r, op_typ, prev_func, eval_type, func); \
   else MGS_straightinsertion(s, l, r, op_typ, prev_func, eval_type, func); \
 }


#define MGS_op_is(a, b) \
        MCM_compare_MMMstring_ident(MTRV_SYS_FUNC_LIST[a], b)

#define MGS_flat_allowed(func_nr) \
	MGSV_flat_table[func_nr]


#endif /* __MGS_simplify__ */
