/* $Date: 1997/03/03 12:56:53 $  $Author: olli $  $Revision: 1.18 $ */

/**************************************************************************/
/* FILE:             MGS_simplify_help.h                                  */
/* Autor:            Oliver Kluge                                         */
/* Erstellt:         06/09/91                                             */
/* Letzte Aenderung: 03/03/97                                             */
/*                                                                        */
/* Ab Revision 1.17 (31.10.96) gibt es das Preprocessor-Flag NO_SPEC_BITS */ 
/* Hiermit werden die Abfragen auf die Special-Bits MINUS_BIT, DIV_BIT    */ 
/* und nicht mehr durchgef"uhrt, da davon ausgegangen wird, das keine     */ 
/* Funktion, insbesondere der Parser und der Simplifier, mehr solche Bits */ 
/* generieren.  (Oliver Kluge)                                            */
/* Ab Revision 1.18 (3.3.97) gibt es nur noch die Version ohne Special-   */ 
/* Bits. Das Flag NO_SPEC_BITS wurde entfernt.                            */
/**************************************************************************/

#ifndef __MGS_simplify_help__
#define __MGS_simplify_help__

/*
 * Struktur fuer den Simplifier, die Signatur und Typ (Ausdruck,
 * Konstante) enthaelt.
 */

typedef struct _MGST_sig {
    unsigned long sig;		/* Signatur */
    unsigned char type;		/* Type */
} MGST_sig;


/*** kg:
#define SIZE_OF_CHANGE 5
***/
/************************************************************************/
/* Bestimmt, ab welcher Anzahl von Elementen 'n' des zu sortierenden    */
/* Arrays Straightinsertion angewendet wird. D.h. 'n' < SIZE_OF_CHANGE, */
/* dann Straightinsertion, sonst Quicksort.                             */
/* =====> kg: Wird nicht mehr verwendet, da der Overhead zum Aufruf von */
/* Straightinsertion innerhalb von Quicksort zu gross ist.  Stattdessen */
/* wird eine  Grenze zur Vorab-Auswahl  des Sortierverfahrens verwendet */
/* (vgl. 'MGSC_QUICKSORT' in 'MGS_simplify.h').                         */
/************************************************************************/


long MGS_expr_cmp          ( long i, long j, S_Pointer sp_i, 
                             S_Pointer sp_j, long op_typ );
long MGS_expr_cat_cmp      ( long i, S_Pointer sp_i, 
                             S_Pointer sp_j, long op_typ );
long MGS_radical_cmp       ( S_Pointer sp_i, S_Pointer sp_j );
void MGS_summarize_expr    ( long i, S_Pointer *adr_i, S_Pointer *adr_j,
                             long op_typ, long *count_blank);
void MGS_summarize_expr_cat( long i, S_Pointer *adr_i, S_Pointer *adr_j,
                             long op_typ, long *count_blank );
void MGS_summarize_cat_cat ( long i, S_Pointer *adr_i, S_Pointer *adr_j, 
                             long op_typ, long *count_blank );
void MGS_set_spec          ( S_Pointer a, short spec);
void MGS_pass_through_sign_in_plus( S_Pointer a );


/*** Makros zur Signaturverwaltung **************************************/

#define MGS_get_key(i) \
        (sig_table[i].sig)
 
#define MGS_set_sig(i, j) \
        { sig_table[i] = sig_table[j]; }
 
#define MGS_swap_sig(i,j) \
        { sig_table[0] = sig_table[i]; \
          sig_table[i] = sig_table[j]; \
          sig_table[j] = sig_table[0]; \
        }

#define MGS_fix_sig(i) \
        MGS_set_sig(0, i)
 
#define MGS_reset_sig(i) \
        MGS_set_sig(i, 0)

/*** Mit Hilfe der beiden  obersten Bits  wird Simplifier optimiert. ***/
/*** Die folgenden Masken werden zum Abfragen dieser Bits verwendet. ***/

#define  MGS_MASK_EXPR   0x1
#define  MGS_MASK_CONST  0x2

/*** Liefert Zahl > 0, falls Bit MGS_MASK_EXPR gesetzt ist. Das ***/
/*** Bit wurde gesetzt wenn der S_Pointer vom Typ DOM_EXPR ist.  ***/

#define MGS_is_expr(i) \
        (sig_table[i].type & MGS_MASK_EXPR)

/*** Liefert Zahl > 0, falls MGS_MASK_CONST gesetzt ist. Das     ***/
/*** Bit wurde gesetzt wenn der S_Pointer  oder einer der Soehne ***/
/*** eine numerische Konstante enthaelt.  ***/

#define MGS_contain_constant(i) \
        (sig_table[i].type & MGS_MASK_CONST)

#define MGS_update_key_const(i) \
        (sig_table[i].type = (MGS_MASK_EXPR | MGS_MASK_CONST))

#define MGS_update_key_no_const(adr, i) { \
        sig_table[i].sig = sig_table[i].sig					\
				-MMMsignature(*MMMP(adr, counter-1L, MMMread)); \
	sig_table[i].type ^= MGS_MASK_CONST; }

#define MGS_set_key(i, sp_i, type, op_typ)                                      \
{                                                                               \
  if( type != CAT_EXPR ) {                                                      \
      sig_table[i].sig = MMMsignature(sp_i);                                    \
      sig_table[i].type = '\0';                                                 \
  } else {                                                                      \
      S_Pointer tmp = *MMMP(&sp_i,0L,MMMread);                                  \
                                                                                \
      if( MFisIdent(tmp) && MCM_compare_MMMstring_ident(MTRV_SYS_FUNC_LIST[op_typ+1],tmp) ) { \
          tmp = *MMMP(&sp_i, MMMcounter(sp_i)-1L, MMMread);                     \
          sig_table[i].sig = MMMsignature(sp_i)                                 \
                          -MMMsignature(*MMMP(&sp_i,0L,MMMread));               \
          if( MMMZ(&tmp, CATEGORY, MMMread)->cat < MUPC_num) {                  \
              sig_table[i].sig = sig_table[i].sig -MMMsignature(tmp);           \
              sig_table[i].type = (MGS_MASK_EXPR | MGS_MASK_CONST);             \
          } else {                                                              \
              sig_table[i].type = MGS_MASK_EXPR;                                \
          }                                                                     \
       } else {                                                                 \
           sig_table[i].sig = MMMsignature(sp_i);                               \
           sig_table[i].type = MGS_MASK_EXPR;                                   \
       }                                                                        \
       MMMinactiv(&tmp);                                                        \
       MMMinactiv(&sp_i);                                                       \
   }                                                                            \
}

/************************************************************************/
 
#define MGS_change_s()                                                   \
  if( s_has_not_changed ) {                                              \
      MMMchange(s);                                                      \
      bp_s = MMMP(s, 0L, MMMwrite);                                      \
      s_has_not_changed = FALSE;                                         \
  }

#define MGS_swap_sp(i,j)                                                 \
  { tmp = *(bp_s+(i));                                                   \
    *(bp_s+(i)) = *(bp_s+(j));                                           \
    *(bp_s+(j)) = tmp;                                                   \
  }

#define MGS_change_s_bp_i()                                              \
  if( s_has_not_changed ) {                                              \
      MMMchange(s);                                                      \
      bp_s = MMMP(s, 0L, MMMwrite);                                      \
      bp_i = bp_s + i;                                                   \
      s_has_not_changed = FALSE;                                         \
  }

/************************************************************************/

#define MGS_add_sub_one(c, flag)                                         \
        (flag ? MCA_mm(c) : MCA_pp(c))

/************************************************************************/

#define MGS_expr_to_node(adr)                                            \
  { S_Pointer leave = *MMMP(adr,1L,MMMread);                             \
    MMMinactiv(adr);                                                     \
    MMMcopy(leave);                                                      \
    MMMfree(*adr);                                                       \
    *adr = leave;                                                        \
  }

#define MGS_delete_constant(adr, i)                                      \
  { long counter      = MMMcounter(*adr);                                \
    unsigned long sig = MMMsignature(*adr);                              \
    if( counter > 3L ) {                                                 \
        MGS_update_key_no_const(adr, i);                                 \
        MMMnewcounter(adr, counter-1L);                                  \
        MMMinactiv(adr);                                                 \
        MMMnewsignature(*adr, sig);                                        \
    } else {                                                             \
        MGS_expr_to_node(adr);                                           \
        sig_table[i].sig = MMMsignature(*adr);                           \
        if (MFcat(*adr) == CAT_EXPR) {                                   \
           sig_table[i].type = MGS_MASK_EXPR;                            \
        } else {                                                         \
           sig_table[i].type = '\0';                                     \
        }                                                                \
        MMMinactiv(adr);                                                 \
    }                                                                    \
  }

#define MGS_summarize(i, j, adr_i, adr_j, op_typ, count_blank)           \
  if( MGS_contain_constant(i) )                                          \
      if( MGS_contain_constant(j) )                                      \
          MGS_summarize_expr    (i, adr_i, adr_j, op_typ, count_blank);  \
      else                                                               \
          MGS_summarize_expr_cat(i, adr_i, adr_j, op_typ, count_blank);  \
  else                                                                   \
      if( MGS_contain_constant(j) )                                      \
          MGS_summarize_expr_cat(j, adr_j, adr_i, op_typ, count_blank);  \
      else                                                               \
          MGS_summarize_cat_cat (i, adr_i, adr_j, op_typ, count_blank);


#endif /* __MGS_simplify_help__ */
