/* $Date: 1997/10/10 15:54:32 $  $Author: tonner $  $Revision: 1.34 $ */

/**************************************************************************/
/* FILE:             MIO_io.h                                             */
/* Autor:            Oliver Kluge, Andreas Kemper, Ralf Hillebrand        */ 
/* Erstellt:         27/06/91                                             */ 
/* Letzte Aenderung: 31/08/94                                             */ 
/**************************************************************************/ 
 
#ifndef __MIO_io__
#define __MIO_io__

#ifdef NEWCOMMUNICATION
#  ifdef DOS_FILENAMES
#    include "MPR_prot.h"
#  else
#    include "MPR_protocol.h"
#  endif
#endif

/**************************************************************************/ 
/* Dieses File enthaelt die Routinen zur Ausgabe.                         */  
/**************************************************************************/ 
 
/************** Die offizielle Schnittstelle der Ausgabe ******************/

void     /* Initialisierung der Ausgabe */
MIO_init(VOID);

void     /* Gibt bei Programmende dynamische Strukturen frei */
MIO_exit(VOID);


void     /* Zuruecksetzen der Ausgabe nach Error */
MIO_reset(VOID);


void     /* Ausgabe eines MuPAD-Datums */
MIO_out(S_Pointer);

/* Ausgabe einer Prozedur und gleichzeitiges Einfuegen von
   Debug-Informationen.                                    */
void 
MIO_create_debug(S_Pointer *, MCLT_iostream &, long, long *);


/* Diese beiden Funktionen sollten nicht direkt zugreifbar sein */
void 
MIO_print_string(char *); /* C-String schreiben */

void 
MIO_print_char(char);     /* einen char schreiben */


S_Pointer /* erledigt die Drecksarbeit fuer MSY_eval_expr2text  */
MIO_expr2text(S_Pointer);


/* Diese Variable stellt sicher, dass immer nur ein 
   Prozess in der Ausgabe ist.                          */
//extern MMMTlock MIOlock;
extern S_Pointer MIOlock;



extern char MIOV_BUFFER[];    /* Textpuffer, der zu formatierende Texte */
                              /* zwischenspeichert                      */
	                      /* [ siehe MIO_print_format_number() ]    */

/* Formatierte Bildschirmausgabe der Zahl 'i'.				*/
/* Die Formatanweisung  's' darf genau ein Element besitzen.		*/ 
void 
MIO_print_format_number(char *, long);

void 
MIO_print_format_string(char *, char *);


/* Verwaltung des aktuellen Ausgabetyps MIOV_OUTPUT_TYPE */
#ifdef NEWCOMMUNICATION
#  define MIOC_LOCAL_TYPE MPRTtype MIOV_local_type;
#  define MIOC_PUSH_TYPE(t) { if (XMUPAD) \
			      { MUP_fflush(stdout); \
	MIOV_local_type = MIOV_OUTPUT_TYPE; MIOV_OUTPUT_TYPE = (t); } }
#  define MIOC_POP_TYPE { if (XMUPAD) \
    { MUP_fflush(stdout); MIOV_OUTPUT_TYPE = MIOV_local_type; } }
#else
#  define MIOC_LOCAL_TYPE
#  define MIOC_PUSH_TYPE(t)
#  define MIOC_POP_TYPE
#endif


/**** Die folgenden Funktionen kontrollieren den Zugriff auf die 
      Variable MIOV_PRETTY_PRINT, die IMMER auch im Stack erfolgen
      MUSS.                                                        ***/

/* Schreiben eines neuen Pretty-Werts auf den Stack */
void 
MIO_push_pretty( long pretty );

/* Eine Schachtel mit PRETTY_PRINT = TRUE auf den Stack legen. */
inline  void 
MIO_push_pretty_true(VOID) 
{ MIO_push_pretty(1L); }

/* Eine Schachtel mit PRETTY_PRINT = FALSE auf den Stack legen. */
inline void 
MIO_push_pretty_false(VOID) 
{ MIO_push_pretty(0L); }
     
/* Letzten Pretty-Wert Stack vom Stack nehmen */
void 
MIO_pop_pretty(VOID);



/* dies ist der Typ fuer Ausgabefunktionen fuer Ausdruecke, also
   z.B. fuer alle Anweisungen */
typedef S_Pointer 
(*MIOT_outfunction_pointer)(S_Pointer *, 
			    S_Pointer, 
			    long, 
			    S_Pointer);

/* zuerst kommen die Standard-Ausgabe-Funktionen  */
S_Pointer 
MIO_out_stand_op(      S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_stand_sys_func(S_Pointer *, S_Pointer, long, S_Pointer);

/* nun diejenigen Ausgaberoutinen spezieller Ausdruckstypen */
S_Pointer 
MIO_out_domattr(       S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_index(         S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_not(           S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_divex(         S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_negate(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_invert(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_subtract(      S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_seqgen(        S_Pointer *, S_Pointer, long, S_Pointer);

/* und schliesslich die Ausgaberoutinen der Anweisungen */
S_Pointer 
MIO_out_assign(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_break(         S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_case(          S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_for(           S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_for_down(      S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_for_in(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_for_in_par(    S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_for_par(       S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_if(            S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_next(          S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_parbegin(      S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_quit(          S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_repeat(        S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_seqbegin(      S_Pointer *, S_Pointer, long, S_Pointer);

S_Pointer 
MIO_out_while(         S_Pointer *, S_Pointer, long, S_Pointer);


/**** interne Funktionen, die trotzdem woanders benoetigt werden :-( ****/

/* wird im Macro MUP_fflush benoetigt, bitte nicht direkt benutzen */
void 
MIO_write(char);


/******* Vordefinierte Konstanten fuer Ausgabeprioritaeten *******/

#define MIOC_P_NULL             0
#define MIOC_P_STMTSEQ          1
#define MIOC_P_EXPRSEQ          2
#define MIOC_P_OR               3
#define MIOC_P_AND              4
#define MIOC_P_SEQGEN           5
#define MIOC_P_LESS             6
#define MIOC_P_LEEQUAL          6
#define MIOC_P_EQUAL            6
#define MIOC_P_UNEQUAL          6
#define MIOC_P_RANGE            7
#define MIOC_P_UNION            8
#define MIOC_P_MINUS            9
#define MIOC_P_INTERSECT       10
#define MIOC_P_MOD             11
#define MIOC_P_PLUS            12

/* die beiden folgenden Eintraege sind nur fuer den PRETTY_PRINTer */
#define MIOC_P_UNARY_MINUS     13  
#define MIOC_P_COMPLEX         14

#define MIOC_P_MULT            15
#define MIOC_P_POWER           16
/* hier fehlen eigentlich selbst-def Operatoren */
#define MIOC_P_NOT             17
#define MIOC_P_FCONCAT	       18
#define MIOC_P_CONCAT          19
#define MIOC_P_STMT            20


#define MIOC_P_max             21

#define MIOC_DONT_SPLIT        32
 
#define MIOT_FIELDSIZE long       /* Datenfeldgroesse im Ausgabebaum */

extern S_Pointer MIOV_DOMAIN_ASS_OUTPUT ;    /* CAT_STRING "print" */
extern S_Pointer MIOV_DOMAIN_ASS_EXPR2TEXT ; /* CAT_STRING "expr2text" */

#ifdef NEWCOMMUNICATION
extern MPRTtype MIOV_OUTPUT_TYPE;   /* Typ der Ausgabe bei Kommunikation 
			      * mittels MPR, zulaessige Typen werden in 
		              * MPR_protocol.h definiert. */
#endif


#endif /* __MIO_io__ */
