////////////////////////////////////////////////////////////////////////
//                           MMMstorage_auto.h                        //
// In diesem File sind die Deklarationen f"ur eine Implementation von //
// MAMMUT enthalten, das auf einem automatischen Garbage Collector    //
// aufsetzt, der konservativ arbeitet, d.h. der selbst den Stack des  //
// C-Programms untersucht und entscheidet, welche Speicherzelle noch  //
// ben"otigt werden oder nicht. Diese Implementation von MAMMUT setzt //
// also direkt auf einer Allokationsfunktion auf und ben"otigt keine  //
// Freigabefunktion.                                                  //
// Als unterliegende automatische Garbage Collectoren sind            //
// insbesondere vorgesehen:                                           //
// (1) ftp.di.unipi.it:/pub/project/posso/cmm                         //
// (2) http://reality.sgi.com/employees/boehm_mti/gc.html             //
//                                                                    //
// Autor:              Holger Naundorf                                //
// Erstellungsdatum:   17.09.1996                                     //
// Letzte "Anderung:   Mai 1997, chh, Anpassung  der CMM-Version an   //
//                     parallelen Fall                                //
////////////////////////////////////////////////////////////////////////

#ifndef __MMMstorage_auto__
#define __MMMstorage_auto__

#ifdef MACINTOSH
#pragma once
extern int      MACV_EVENT_TIME;
extern Boolean  gfRegistered;
void   MAC_event(void);
#endif

#ifdef MMMDAUTO
#  define MMMDREFSUNCERTAIN // Auf den Referenzz"ahler ist kein Verla"s
#endif

#include <signal.h>
#include <stdio.h>

#define MEXDSTANDARD
#ifdef PARALLEL
#   define MEXDPARALLEL
#   define MMMDshared
#endif

#ifdef MSDOS
#  include "MDE_decl.h"
#  include "MEX_ext.h"
#else
#  include "MDE_declare.h"
#  include "MEX_extern.h"
#endif


////////////////////////////////////////////////////////////////////////
// Allgemeine Typen                                                   //
////////////////////////////////////////////////////////////////////////
#ifdef MSDOS
typedef unsigned char MUPbool ;
#else
typedef long MUPbool ;
#endif
#define MTbool MUPbool   // Fuer Module

////////////////////////////////////////////////////////////////////////
// Typen der Speicherverwaltung                                       //
////////////////////////////////////////////////////////////////////////
typedef unsigned long MMMulong ;


////////////////////////////////////////////////////////////////////////
// Verschiedenste Konstanten                                          //
////////////////////////////////////////////////////////////////////////
#ifndef true
#   define true  1 /*Der Boolean-Wert true*/
#   define false 0 /*Der Boolean-Wert false*/
#endif
#define _true_  1
#define _false_ 0

#define MMMlocked 1L
#define MMMunlocked 0L

#define NULLL 0

////////////////////////////////////////////////////////////////////////
// MMMTstatus                                                         //
// Jede MAMMUT-Zelle enth"alt Statusinformtionen, die in einer        //
// Instanz dieser Klasse gespeichert sind.                            //
////////////////////////////////////////////////////////////////////////
class MMMTstatus
{
   unsigned int type : 12 ;
   unsigned int system : 1 ;
   unsigned int unmoveable : 1 ;
   unsigned int volatile locked : 1 ;
public:
   MMMTstatus(MMMulong typ, MMMulong sys =0, MMMulong unmv =0) { type = typ ; system = sys ; unmoveable = unmv ; locked = 0 ; } ;
   MMMulong getType() { return (MMMulong)type ; } ;
   void setType(MMMulong typ) { type = (unsigned int) typ ; } ;
   MMMulong getSystem() { return (MMMulong) system ; } ;
   void setSystem(MMMulong sys) { system = (unsigned int) sys ; };
   MMMulong getUnmoveable() { return (MMMulong) unmoveable ; } ;
   void setUnmoveable(MMMulong unmv) { unmoveable = (unsigned int) unmv ; };
   inline void lock();
   //void lock();
   void setlocked(){locked=MMMlocked;};
   MMMulong getlocked(){ return (MMMulong) locked; };
   void unlock(){/* cout << CMMbase::mynumber << ": unlock MMMTstatus " << (unsigned long) this << "\n"; */ locked = MMMunlocked;};
   //void unlock();
} ;


////////////////////////////////////////////////////////////////////////
// MMMTheader                                                         //
// Jede MAMMUT-Zelle enth"alt in einem Header zus"atzliche            //
// Informationen. Dieser Header wird am Anfang der Zelle gespeichert, //
// die vom externen Collector angefordert wurde.                      //
////////////////////////////////////////////////////////////////////////
#ifdef MEXDMMMCMM
class MMMTheader : public GcVarObject
#else
class MMMTheader
#endif
{
   MMMulong signature ;
   MMMulong size ;
   MMMulong count ;
   MMMTstatus status ;
public:
   MMMTheader(MMMulong siz, MMMulong coun, MMMulong typ, MMMulong sys =0, MMMulong unmv =0) : status(typ, sys, unmv) { size = siz ; count = coun ; } ;
#ifdef MEXDMMMCMM
   void traverse() ;
#endif
   MMMulong getSignature() { return signature ; } ;
   void setSignature(MMMulong sig) { signature = sig ; } ;
   MMMulong getSize() { return size ; } ;
   void setSize(MMMulong siz) { size = siz ; } ;
   MMMulong getCount() { return count ; } ;
   void setCount(MMMulong coun) { count = coun ; } ;
   MMMulong getType() {return status.getType() ; } ;
   void setType(MMMulong type) { status.setType(type) ; } ;
   MMMulong getSystem() { return status.getSystem() ; } ;
   void setSystem(MMMulong system) { status.setSystem(system) ; } ;
   MMMulong getUnmoveable() { return status.getUnmoveable() ; } ;
   void setUnmoveable(MMMulong unmv) { status.setUnmoveable(unmv) ; } ;
   void lock() { status.lock() ; } ;
   void unlock() { status.unlock() ; } ;
   void actualize( const MMMTheader& head ) { signature = head.signature ; } ;
} ;

typedef MMMTheader* S_Pointer ;


////////////////////////////////////////////////////////////////////////
// Deklaration "Offentlicher Variablen                                //
////////////////////////////////////////////////////////////////////////
extern const S_Pointer MMMNULL ;
extern const MMMulong MMMCwork ;
extern const MMMulong MMMCglobal ;
extern const MMMulong MMMClocal ;
extern const MMMulong MMMread ;
extern const MMMulong MMMwrite ;

#ifndef PARALLEL

extern MMMulong MMMVused_blocks ;
extern MMMulong MMMVused_memory ;
extern MMMulong MMMVreserved_blocks ;
extern MMMulong MMMVreserved_memory ;
extern MMMulong MMMVfrees ;
extern MMMulong MMMVrealfrees ;
extern MMMulong MMMVmallocs ;
extern void (*MMMVsystemfree)(S_Pointer s) ;
extern void (*MMMVsystemchange)(S_Pointer *s) ;
typedef char *(*MMMTno_memory)(MUPbool) ;
extern MMMTno_memory MMMVno_memory ;
extern MMMulong MMMVnumber_of_processes ;
extern MMMulong MMMVson ;
extern MMMulong MMMVlocal_son ;
extern MMMulong MMMVnode ;
extern MMMulong MMMVnetnode ;
extern S_Pointer MMMVtopology ;

#else

typedef MCLT_iostream MMMThandle ;
typedef pid_t MMMTpid ;
typedef void (*MMMTsystemfree)(S_Pointer) ;
typedef void (*MMMTsystemchange)(S_Pointer *) ;
typedef char *(*MMMTno_memory)(MUPbool) ;

SHARED_DECL(ulong, MMMVused_blocks) ;
SHARED_DECL(ulong, MMMVused_memory) ;
SHARED_DECL(ulong, MMMVreserved_blocks) ;
SHARED_DECL(ulong, MMMVreserved_memory) ;
SHARED_DECL(ulong, MMMVfrees) ;
SHARED_DECL(ulong, MMMVrealfrees) ;
SHARED_DECL(ulong, MMMVmallocs) ;
SHARED_DECL(ulong, MMMVnumber_of_processes) ;
SHARED_DECL(S_Pointer, MMMVtopology);
SHARED_DECL(MMMTpid *, MMMVprocessid) ;
SHARED_DECL(ulong, MMMVlockcount) ;
SHARED_DECL(MMMTlock*, MMMVlocktable) ;

SHARED_DECL( MMMTno_memory, MMMVno_memory) ;
SHARED_DECL( MMMTsystemfree, MMMVsystemfree) ;
SHARED_DECL( MMMTsystemchange, MMMVsystemchange) ;

GLOBAL_DECL(char, *MMMVmalloc_help) ;
GLOBAL_DECL(long, MMMVnetnode) ;
GLOBAL_DECL(long, MMMVnode) ;
GLOBAL_DECL(ulong, MMMVson) ;
GLOBAL_DECL(ulong, MMMVlocal_son) ;

#endif

////////////////////////////////////////////////////////////////////////
// Hilfsvariablen                                                     //
////////////////////////////////////////////////////////////////////////
SHARED_DECL(S_Pointer, MMMVsharedh) ;
SHARED_DECL(S_Pointer, MMMVlocal_sharedh);

////////////////////////////////////////////////////////////////////////
// Deklaration der "offentlichen Schnittstelle                        //
////////////////////////////////////////////////////////////////////////
inline S_Pointer MMMmalloc(MMMulong Memtyp, MMMulong typ, MMMulong size, MMMulong count) ;
inline S_Pointer MMMpalloc(MMMulong Memtyp, MMMulong typ, MMMulong size, MMMulong count) ;
inline S_Pointer MMMcalloc(MMMulong Memtyp, MMMulong typ, MMMulong size, MMMulong count) ;
inline S_Pointer MMMforce_data(MMMulong Memtyp, MMMulong typ, char *data, MMMulong size) ;

inline void MMMfree(S_Pointer s) ;
inline void MMMlfree(S_Pointer s) ;

inline void MMMresize(S_Pointer *s, MMMulong size) ;
inline void MMMnewcounter(S_Pointer *s, MMMulong count) ;
inline void MMMrealloc(S_Pointer *s, MMMulong size, MMMulong count) ;

inline int MMMequal(S_Pointer s, S_Pointer t) ;
inline void MMMset(S_Pointer *p, S_Pointer s) ;
inline void MMMreplace( S_Pointer *p, S_Pointer s ) ;
inline void MMMcopy(S_Pointer s) ;
inline void MMMshift(S_Pointer sour, MMMulong spos, S_Pointer dest, MMMulong dpos, MMMulong count) ;

inline void MMMchange(S_Pointer *s) ;
inline S_Pointer *MMMP(S_Pointer *s, MMMulong i, MMMulong use) ;

inline void MMMinactiv(S_Pointer *s) ;
inline S_Pointer MMM(S_Pointer s) ;

inline MMMulong MMMsize(S_Pointer s) ;
inline MMMulong MMMcounter(S_Pointer s) ;
inline MMMulong MMMtype(S_Pointer s) ;
inline void MMMnewtype(S_Pointer s, MMMulong typ) ;
inline MMMulong MMMsignature(S_Pointer s) ;
inline void MMMnewsignature(S_Pointer s, MMMulong sig) ;
inline void MMMlock(S_Pointer s) ;
inline void MMMunlock(S_Pointer s) ;
inline void MMMunmoveable(S_Pointer s, MMMulong m) ;

inline S_Pointer MMMread_shared(MMMulong i) ;
inline void MMMwrite_shared(MMMulong i, S_Pointer s) ;
inline S_Pointer MMMread_local_shared(MMMulong i) ;
inline void MMMwrite_local_shared(MMMulong i, S_Pointer s) ;
inline void MMMprepare_sleep() ;
inline void MMMsleep() ;
inline void MMMwake_up(MMMulong pid) ;
inline void MMMinsert_global(S_Pointer *p) ;
inline void MMMmark_globals(){} ;
inline void MMMmark_tree(S_Pointer p) {};
inline MUPbool MMMfree_unmarked(S_Pointer p) { return _false_; }
inline void MMMfree_unused() ;

void MMMfirst_init() ;
void MMMinit(char *name) ;
void MMMexit() ;

inline MMMulong MMMrefs(S_Pointer s) ; // Wird eigentlich nicht unterst"utzt.

inline MMMulong MMMbytes(S_Pointer p) { return 0L; }

////////////////////////////////////////////////////////////////////////
// Anforderung von Speicher OHNE Mammut-Objekt                        //
////////////////////////////////////////////////////////////////////////

// Prozessprivater Speicher!
inline char *MMMEmalloc(MMMulong size) ;
inline void MMMEfree(char *) ;

// Um Speicher im gemeinsamen Speicher zur Verfuegung zu stellen,
// gibt es die Funktionen char *MMMEsharemalloc(unsigned long size)
// und void MMMEsharefree(char *)
// Diese sollten aber nur verwendet werden, wenn es unbdingt sein muss,
// da in der parallelen CMM-Version MMMEsharefree nicht wirklich
// Speicher freigibt.

// Allgemein gilt folgende Mammut-Spezifikation:
// MMMEmalloc: Es wird Speicher reserviert, von dem nicht definiert ist,
//             ob er im gemeinsamen Speicher liegt oder nicht. Sollte immer
//             benutzt werden, wenn es egal ist, wo der Speicher liegt.
// MMMEsharemalloc:  Reserviert Speicher im gemeinsamen Speicher. Sollte nur
//                   benutzt werden, wenn wirklich Speicher im gemeinsamen
//                   Speicher benoetigt wird.
//
// Bei dieser Implementation gilt: MMMEsharemalloc und MMMEmalloc sind
// synonym. Beide nehmen Speicher aus dem gemeinsamen Speicher!
//
// Keinesfalls darf in mit MMMEmalloc oder MMMEsharemalloc angefordertem
// Speicher ein Zeiger auf ein Mammutobjekt gespeichert werden.


////////////////////////////////////////////////////////////////////////
// Deklaration der Hilfsfunktionen                                    //
////////////////////////////////////////////////////////////////////////

inline MMMulong MMMadjustSize(MMMulong size) ;
inline void MMMcopyAll(S_Pointer s1, S_Pointer s2) ;
inline char *MMMgetMemAdr(S_Pointer s, MMMulong mv) ;

////////////////////////////////////////////////////////////////////////
// Memberfunktionen von MMMTstatus
////////////////////////////////////////////////////////////////////////

#ifdef MMMDshared
#   ifdef MMMDLOCKSPACE
#      define MMMHlock_for_pointer(l,s) ((l) = ((SHARED_READ(MMMVlocktable)[((unsigned long)(s)) % SHARED_READ(MMMVlockcount)])))
#   else /* MMMDLOCKSPACE */
#      define MMMHlock_for_pointer(l,s) ((l) = (&(SHARED_READ(MMMVlocktable)[((unsigned long)(s)) % SHARED_READ(MMMVlockcount)])))
#   endif /* MMMDLOCKSPACE */
#else /* MMMDshared */
#  define MMMHlock_for_pointer(l,s)
#endif /* MMMDshared */

inline
void MMMTstatus::lock()
{
#ifdef PARALLEL
           // cout << CMMbase::mynumber << ": lock MMMTstatus " << (unsigned long) this << "\n";
           {  VOLATILE(MMMTstatus *) r = this ;
              MMMHdeclare_lock_ptr(MMMVlock) ; 
              MMMHlock_for_pointer(MMMVlock, 1) ; 
              while (true) 
                 {  while (r->getlocked() == MMMlocked) /* cerr << CMMbase::mynumber << ": locked\n"; */;
                    MMMElock ( MMMVlock ) ; 
                    if ( r->getlocked() == MMMlocked ) 
                       MMMEunlock ( MMMVlock ) ; 
                    else 
                       {  r->setlocked() ; 
                          MMMEunlock ( MMMVlock ) ; 
                          //cout << CMMbase::mynumber << ": End lock \n";
                          return ; 
                       } 
                 } 
            }
#else
  locked=MMMlocked;
#endif
}



////////////////////////////////////////////////////////////////////////
// Definitionen der "offentlichen Schnittstelle:                      //
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// Zugriff auf den Inhalt                                             //
////////////////////////////////////////////////////////////////////////
#define MMMZ(s,T,use) \
           ((T *)MMMgetMemAdr(*s, 0))

#define MMMarr(s,i,T,use) \
           ((T *)MMMgetMemAdr(*s, (i)*sizeof(T)))

#define MMMmv(s,mv,T,use) \
           ((T *)MMMgetMemAdr(*s, (mv)))

#define MMMmvarr(s,mv,i,T,use) \
           ((T *)MMMgetMemAdr(*s, (mv) + (i)*sizeof(T)))

inline   
S_Pointer *MMMP(S_Pointer *s, MMMulong i, MMMulong use) 
{ 
   S_Pointer *p = (S_Pointer *)((*s)+1) ; 
   return p+i ; 
} 

////////////////////////////////////////////////////////////////////////
// Allokieren von Speicher                                            //
////////////////////////////////////////////////////////////////////////
inline
S_Pointer MMMmalloc(MMMulong Memtyp, MMMulong type, MMMulong size, MMMulong count)
{
#ifdef MEXDMMMCMM
   MMMulong rsize = MMMadjustSize(size) + count*sizeof(S_Pointer) ;
   S_Pointer s = new(rsize) MMMTheader(size, count, type) ;

   return s ;
#else
   MMMulong rsize = MMMadjustSize(size) + count*sizeof(S_Pointer) + sizeof(MMMTheader);

   S_Pointer s ;
   if ( count == 0 )
      {
         s = GC_malloc_atomic( rsize ) ;
      }
    else
      {
         s = GC_malloc_ignore_off_page( rsize ) ;
      }
   s->MMMTheader(size, count, type) ;

   return s ;
#endif
}

inline
S_Pointer MMMpalloc(MMMulong Memtyp, MMMulong typ, MMMulong size, MMMulong count)
{
   S_Pointer s = MMMmalloc( Memtyp, typ, size, count ) ;
   S_Pointer *q = MMMP( &s, 0, MMMwrite ) ;
   for ( ; count>0 ; count-- , q++ )
      {
         *q = MMMNULL ;
      }
   return s ;
}

inline
S_Pointer MMMcalloc(MMMulong Memtyp, MMMulong typ, MMMulong size, MMMulong count)
{
   S_Pointer s = MMMpalloc( Memtyp, typ, size, count ) ;
   char *p = MMMZ( &s, char, MMMwrite ) ;
   for ( ; size>0 ; size-=sizeof(char) , p++ )
      {
         *p = 0 ;
      }
   return s ;
}

inline
S_Pointer MMMforce_data(MMMulong Memtyp, MMMulong typ, char *data, MMMulong size)
{
   S_Pointer s = MMMmalloc( Memtyp, typ, sizeof(char *), 0 ) ;
   *MMMZ( &s, char *, MMMwrite ) = data ; // Das MMMZ() mu"s ausgef"uhrt
              // werden, bevor s->setSystem(1) aufgerufen wird!!!
   s->setSystem( 1 ) ;
   return s ;
}

////////////////////////////////////////////////////////////////////////
// Freigabe                                                           //
////////////////////////////////////////////////////////////////////////
inline
void MMMfree(S_Pointer s)
{
}

inline
void MMMlfree(S_Pointer s)
{
}


////////////////////////////////////////////////////////////////////////
// Gr"o"senver"anderung                                               //
////////////////////////////////////////////////////////////////////////
inline
void MMMresize(S_Pointer *s, MMMulong size)
{
   MMMulong rsize = MMMadjustSize(size) ;
   if ( rsize != MMMsize(*s) )
      {
         S_Pointer r = MMMmalloc( MMMCglobal, MMMtype( *s ), rsize, MMMcounter( *s ) ) ;
         MMMcopyAll( *s, r ) ;
         MMMnewsignature( r, MMMsignature( *s ) ) ;
         *s = r ;
      }
}

inline
void MMMnewcounter(S_Pointer *s, MMMulong count)
{
   if ( count != MMMcounter( *s ) )
      {
         S_Pointer r = MMMmalloc( MMMCglobal, MMMtype( *s ), MMMsize( *s ), count ) ;
         MMMcopyAll( *s, r ) ; 
         MMMnewsignature( r, MMMsignature( *s ) ) ; 
         *s = r ; 
      }  
}

inline
void MMMrealloc(S_Pointer *s, MMMulong size, MMMulong count)
{
   MMMulong rsize = MMMadjustSize(size) ;
   if ( (rsize != MMMsize(*s)) || (count != MMMcounter( *s )) )
      {
         S_Pointer r = MMMmalloc( MMMCglobal, MMMtype( *s ), size, count ) ;
         MMMcopyAll( *s, r ) ;  
         MMMnewsignature( r, MMMsignature( *s ) ) ;  
         *s = r ;  
      }   
}


////////////////////////////////////////////////////////////////////////
// Vergleichen und Kopieren                                           //
////////////////////////////////////////////////////////////////////////
inline
int MMMequal(S_Pointer s, S_Pointer t)
{
   return s==t ;
}

inline
void MMMset(S_Pointer *p, S_Pointer s)
{
   *p = s ;
}

inline
void MMMreplace( S_Pointer *p, S_Pointer s )
{
   *p = s ;
}

inline
void MMMcopy(S_Pointer s)
{
}

inline
void MMMshift(S_Pointer sour, MMMulong spos, S_Pointer dest, MMMulong dpos, MMMulong count)
{
   S_Pointer *p, *q ;
   MMMulong i ;

   if (spos < dpos)
      {  
         p = MMMP (&sour,spos+count-1,MMMread) ;
         q = MMMP (&dest,dpos+count-1,MMMwrite) ;
         for ( i=count ; i>0L ; i-- )
            {
               *q-- = *p-- ;
            }
      } 
    else
      { 
         p = MMMP (&sour , spos , MMMread) ;
         q = MMMP (&dest , dpos , MMMwrite) ;
         for (i=count ; i>0L ; i-- )
            {
               *q++ = *p++ ;
            }
      }
}

inline
void MMMchange(S_Pointer *s)
{
   S_Pointer t = MMMmalloc( MMMCglobal, MMMtype(*s), (*s)->getSize(), (*s)->getCount() ) ;
   MMMcopyAll(*s, t) ;
   *s = t ;
}

////////////////////////////////////////////////////////////////////////
// Poolverwaltung                                                     //
////////////////////////////////////////////////////////////////////////
template<class T>
inline
void MMMpoolinsert(T **p)
{
}

template<class T>
inline
void MMMpooldelete(T **p)
{
}

inline
void MMMpoolinit()
{
}

////////////////////////////////////////////////////////////////////////
// Ende eines Zugriffs                                                //
//////////////////////////////////////////////////////////////////////// 
inline
void MMMinactiv(S_Pointer *s)
{
}

inline
S_Pointer MMM(S_Pointer s)
{
   return s ;
}

////////////////////////////////////////////////////////////////////////
// Zugriff auf Statusinformationen                                    //
////////////////////////////////////////////////////////////////////////
inline
MMMulong MMMsize(S_Pointer s)
{
   if ( s != MMMNULL )
      return s->getSize() ;
    else
      return 0 ;
}

inline
MMMulong MMMcounter(S_Pointer s)
{
   if ( s != MMMNULL )
      return s->getCount() ;
    else
      return 0 ;
}

inline
MMMulong MMMtype(S_Pointer s)
{
   return s->getType() ;
}

inline
void MMMnewtype(S_Pointer s, MMMulong typ)
{
   s->setType( typ ) ;
}

inline
MMMulong MMMsignature(S_Pointer s)
{
   return s->getSignature() ;
}

inline
void MMMnewsignature(S_Pointer s, MMMulong sig)
{
   s->setSignature( sig ) ;
}

inline
void MMMlock(S_Pointer s)
{
   //cout << CMMbase::mynumber << ": Lock S_Pointer " << (unsigned long) s << "\n";
   s->lock() ;
}

inline
void MMMunlock(S_Pointer s)
{
   //cout << CMMbase::mynumber << ": unlock S_Pointer " << (unsigned long) s << "\n";
   s->unlock() ;
}

inline
void MMMunmoveable(S_Pointer s, MMMulong m)
{
}

////////////////////////////////////////////////////////////////////////
// Kommunikation mit anderen Prozessen                                //
////////////////////////////////////////////////////////////////////////
inline
S_Pointer MMMread_shared(MMMulong i)
{
   return SHARED_READ(MMMVsharedh) ;
}

inline
void MMMwrite_shared(MMMulong i, S_Pointer s)
{
   SHARED_WRITE(MMMVsharedh , s) ;
}

inline
S_Pointer MMMread_local_shared(MMMulong i)
{
   return SHARED_READ(MMMVlocal_sharedh) ;
}

inline
void MMMwrite_local_shared(MMMulong i, S_Pointer s)
{
   SHARED_WRITE(MMMVlocal_sharedh , s) ;
}

extern void MMMFprepare_sleep();
inline
void MMMprepare_sleep()
{
   #ifdef PARALLEL
   MMMFprepare_sleep();
   #endif
}

extern void MMMFsleep();
inline
void MMMsleep()
{
   #ifdef PARALLEL
   MMMFsleep();
   #endif
}

extern void MMMFwake_up(MMMulong son);
inline
void MMMwake_up(MMMulong pid)
{
   #ifdef PARALLEL
   MMMFwake_up(pid);
   #endif
}

inline
void MMMinsert_global(S_Pointer *p)
{
}

inline
void MMMdelete_global(S_Pointer *p)
{
}

inline
void MMMfree_unused()
{
}

////////////////////////////////////////////////////////////////////////
// Nicht offiziell unterst"utzte Funktionen                           //
////////////////////////////////////////////////////////////////////////
inline
MMMulong MMMrefs(S_Pointer s)
{
   return 100 ;
}

inline
char *MMMEmalloc(MMMulong size)
{
#ifdef MEXDMMMCMM
   return (char *)malloc(size) ;
#endif
#ifdef MEXDMMMBoehm
   return (char *)GC_malloc(size) ;
#endif
}

inline
void MMMEfree(char *p)
{
#ifdef MEXDMMMCMM
   free(p) ;
#endif
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// Hilfsfunktionen (nur fuer den internen Gebrauch)                   //
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
inline
MMMulong MMMadjustSize(MMMulong size)
{
   return size ;
}

inline
MMMulong MMMmin(MMMulong i, MMMulong j)
{
   if (i<j)
      return i ;
   return j ;
}

////////////////////////////////////////////////////////////////////////
// void MMMcopyAll(S_Pointer s1, S_Pointer s2)                        //
// Es wird soviel von dem Inhalt von s1 nach s2 kopiert, wie m"oglich //
// ist. Enth"alt s2 einen gr"o"seren Point- oder Mem-Teil als s1,     //
// so wird der "ubrigbleibende Rest nicht initialisiert.              //
// Im Header von s2 m"ussen zumindest die Gr"o"se des Point- und des  //
// Mem-Teils korrekt eingetragen sein.                                //
// Die anderen Eintr"age des Headers werden von der Funktion kopiert. //
////////////////////////////////////////////////////////////////////////
inline
void MMMcopyAll(S_Pointer s1, S_Pointer s2)
{
   MMMulong count1 = s1->getCount() ;
   MMMulong count2 = s2->getCount() ;
   S_Pointer *p1 = MMMP( &s1, 0, MMMread ) ;
   S_Pointer *p2 = MMMP( &s2, 0, MMMwrite ) ;
   for ( MMMulong count = MMMmin(count1, count2) ; count>0 ; count--, p1++, p2++ )
      {
         *p2 = *p1 ;
      }

   MMMulong size1 = s1->getSize() ;
   MMMulong size2 = s2->getSize() ;
   char *q1 = MMMZ( &s1, char, MMMread ) ;
   char *q2 = MMMZ( &s2, char, MMMwrite ) ;
   for ( MMMulong size = MMMmin(size1, size2) ; size>0 ; size--, q1++, q2++ )
      {
         *q2 = *q1 ;
      }

   s2->actualize( *s1 ) ;
}


inline
char *MMMgetMemAdr(S_Pointer s, MMMulong mv)
{
   S_Pointer *q = (S_Pointer *)(s+1) ;
   char *p ;
   if ( s->getSystem() )
      {
         p = *(char **)q ;
      }
    else
      {
         p = (char *)(q+s->getCount()) ;
      }
   return p+mv ;
}

//#define inline

/*************************************************************************
** MMMThandle MMMHopen_file (char *datei)                               **
** Dieser Makro oeffnet die Datei datei und gibt den entsprechenden     **
** Handle zurueck.                                                      **
*************************************************************************/
#define MMMHopen_file(datei) fopen ( datei , MUPC_READ_TEXT )

/*************************************************************************
** void MMMHclose_file (MMMThandle handle)                              **
** Dieser Makro schliesst den Strom handle.                             **
*************************************************************************/
#define MMMHclose_file(handle) fclose ( handle )


////////////////////////////////////////////////
// Makros fuer den INTERNEN Gebrauch bei der
// parallelen Version. Sind hier zwecks einer
// einfacheren Portierbarkeit definiert !!
////////////////////////////////////////////////

#ifdef PARALLEL
#      define MMMDsleep_signal                      SIGUSR1
#      define MMMEkill(processid)                   kill(processid,SIGQUIT)
#      define MMMEget_my_id()                       0L
#      define MMMEexit(i)                           exit(i)
#      define MMMEsigblock()                        sighold(MMMDsleep_signal)
#      define MMMEsigpause()                        sigpause(MMMDsleep_signal)
#      define MMMEwake_up(id)                       kill(id,MMMDsleep_signal)
#      define MMMEinitlockspace(file)               0
#      define MMMEdeletelockspace(file)             0
#endif

////////////////////////////////////////////////
// Kompatibilitaet mit paralleler Version
// Die folgenden Locks sind eigentlich auch nur
// fuer den internen Speicherverwaltungsgebrauch
// gedacht. Tonner benutzt locks allerdings bei
// der Ein-/Ausgabe.
////////////////////////////////////////////////

#ifndef PARALLEL
typedef char MMMTlock ;
inline void MMMElock(MMMTlock *l) { }
inline void MMMEunlock(MMMTlock *l) { }
#define MMMEsharemalloc MMMEmalloc
#endif

#endif /* __MMMstorage_auto__ */
