/* $Date: 1997/12/01 12:53:58 $  $Author: andi $  $Revision: 1.15 $ */

/**************************************************************************
**                            MMMSpeicher.h                              **
** In diesem File werden die Strukturen definiert, die von der Speicher- **
** verwaltung, die in MMMSpeicher.c steht, benutzt werden.               **
** Mit Definements kann man das Verhalten der Speicherverwaltung etwas   **
** aendern:                                                              **
** -Ist bei der Kompilation MMMDEBUG definiert, so kann die Speicherver- **
**  waltung Fehler bei der Freigabe von Speicherbloecken erkennen. Sie   **
**  stuerzt nicht, wie in der normalen Version, bei Freigabe eines       **
**  freien Blockes ab.                                                   **
** -Ist bei der Kompilation MMMSTATISTIC definiert, so fuehrt die        **
**  Speicherverwaltung eine genaue Statistik ueber die Anzahl der        **
**  Belegungen und Freigaben von Speicherplaetzen. Diese Statistik wird  **
**  fuer jede Speicherplatzgroesse einzeln gefuehrt, und nicht nur global**
**  fuer alle zusammen.                                                  **
** -Ist bei der Kompilation MMMMOVE definiert, so verschiebt die         **
**  Verwaltung Speicherbloecke auch dann, wenn der Speicher noch nicht   **
**  aus Platzmangel zusammengeschoben werden muss. Diese Modus kann      **
**  benutzt werden, um zu ueberpruefen, ob die Programme genuegend oft   **
**  aus den Handels C-Zeiger berechnen lassen.                           **
**                                                                       **
** Erstellung      : 20/09/90                                            **
** Letzte Aenderung: 09/04/91                                            **
** Autor           : Holger Naundorf                                     **
**************************************************************************/

#ifndef __MMMstorage_gen__
#define __MMMstorage_gen__


#ifdef MMMMOVE
#define MMMDSPECIAL
#ifndef MMMDenable_move
#define MMMDenable_move 1
#endif
#else
#ifdef MMMDACTIV
#define MMMDSPECIAL
#else
#ifdef MMMDEBUG
#define MMMDSPECIAL
#endif
#endif
#endif


#ifndef MMMD_GEN
#   define MMMD_GEN
#endif


#define shared    
#define NULLL 0L
#define MMMNULL ((S_Pointer)0L)
#define _true_  1
#define _false_ 0
typedef long MUPbool ;
#define MTbool MUPbool   // Fuer Module

typedef char MMMTlock ;

#include <stdio.h>
#ifdef DOS_FILENAMES
#  include "MDE_decl.h"
#  include "MEX_ext.h"
#  include "MMMstort.h"
#else
#  include "MDE_declare.h"
#  include "MEX_extern.h"
#  include "MMMstorage_type.h"
#endif

void MIO_print_string(char *) ; 

/**************************************************************************
** Im folgenden werden Konstanten der Speicherverwaltung definiert:      **
** Konstanten beginnen mit MMMC...                                       **
**************************************************************************/
#define MMMEmalloc(n)              malloc(n)
#define MMMEsharemalloc MMMEmalloc
#define MMMElock(l)
#define MMMEunlock(l)

#define MMMCmaxtype  65535    /*Groester Typ, der zugewiesen werden kann*/
#define MMMCinfinity 32767  /*Groeste durch den S-Zeiger-Zaehler darstellbare
                            Zahl*/
#define MMMCupper_index 63L  /*Groester Index, der in MMM_Belegung auftaucht*/

#define MMMCheaderinit 300L  /*Anzahl der header, die beim Starten der Verwaltung
                              erzeugt wird*/
#define MMMCheaderstep 300L /*Anzahl der header, die neu erzeugt werden, wenn
                             nicht mehr genuegend existieren*/
#define MMMClheaderinit 300L /*Anzahl der lheader, die beim Starten der Verwaltung 
                              erzeugt wird*/ 
#define MMMClheaderstep 100L /*Anzahl der lheader, die neu erzeugt werden, wenn  
                             nicht mehr genuegend existieren*/ 

#define MMMread 0L   /*Konstante die anzeigt, dass aus einem S-Zeiger
                      nur gelesen wird */
#define MMMwrite 1L  /*Konstante, die anzeigt, dass der Inhalt eines
                      S-Zeigers auch gelesen wird */

#define MMMClocal 0L /*Konstante die anzeigt, dass lokaler Speicher angefordert wird*/
#define MMMCglobal 1L /*Konstante die anzeigt, dass globaler Speicher angefordert wrid*/
#define MMMCwork 2L /*Konstante die anzeigt, dass Arbeitsspeicher angefordert wird, d.h. der Speicher sollte im Prozessoreigenem Speicher liegen */

/**************************************************************************
** Im folgenden werden globale Variablen der Speicherverwaltung definiert**
** Globale Variable beginnen mit MMMV...                                 **
**************************************************************************/
extern long MMMVnetnode ;
extern long MMMVnode ;
extern S_Pointer MMMVtopology ;
extern char *(*MMMVno_memory)(MUPbool) ;
extern S_Pointer MMMVsharedh ;
extern S_Pointer MMMVlocal_sharedh ;
extern long MMMVson ;
extern long MMMVlocal_son ;
extern long MMMVnumber_of_processes ;
extern MMMclock_t MMMVcheck ;
extern long MMMV_new_header ; /*Diese Variable gibt an, wieviele header neu erzeugt
                        werden, wenn nicht mehr genuegend vorhanden sind*/
extern long MMMV_new_lheader ;/*Diese Variable gibt an, wiele freie lheader neu
                     erzeugt werden, wenn nicht mehr genuegend vorhanden sind*/
extern long *MMMV_mem ; /*In dieser globalen Variablen wird ein
                   Array von long-Werten gespeichert. Am Anfang zeigt
                   diese Variable auf das Array MMMV_mem_init[], waehrend des
                   Programmes zeigt es auf MMMV_mem_step[]. Naehere
                   Informationen siehe diese Variablen*/
extern long MMMVused_blocks ; /*In dieser Variablen wrid gespeichert, wieviele Bloecke
             zur Speicherung von Daten benutzt werden*/
extern long MMMVused_memory ; /*In dieser Variablen wird gespeichert, wieviele Bytes an
             Speicher benutzt werden */
extern long MMMVreserved_blocks ; /*In dieser Variablen wird gespeichert, wieviele
             Bloecke von der Speicherverwaltung reserviert sind */
extern long MMMVreserved_memory ; /*In dieser Variablen wird gespeichert, wieviele Bytes
             von der Speicherverwaltung reserviert sind. Nicht der ganze
             reservierte Speicher muss auch benutzt werden*/
extern long MMMVmax_reserved_memory ;

extern struct MMMTBelegung MMMVBelegung [MMMCupper_index+1] ; /*In dieser Variablen werden
             die Listen der freien Speicherbloecke verschiedener Groesse verwaltet*/
extern long MMMVrealfrees ; /*In dieser Variablen wird die Anzahl der MMMfree()
             Aufrufe gezaehlt, die den Speicher wirklich freigegeben haben */
extern long MMMVfrees ; /*In dieser Variablen wird die Anzahl der MMMfree() Aufrufe
             gezaehlt, die nicht MMMNULL als Argument haben*/
extern long MMMVmallocs ; /*In dieser Variablen wird die Anzahl aller MMMmalloc()
             Aufrufe gezaehlt. Hierbei werden auch Aufrufe gezaehlt, die
             intern von der Speicherverwaltung durchgefuehrt werden, z.B. bei
             einem MMMresize(). */ 
extern void (*MMMVsystemfree)(S_Pointer p) ; /*Diese Funktion wird
             aufgerufen, wenn der S-Zeiger p (wirklich) freigegeben wird
             und der Inhalt des S-Zeigers in einem Speicherbereich steht,
             der nicht von der Speicherverwaltung verwaltet wird*/

#ifdef MMMMOVE
extern MMMTPoolHeader MMMVpool ;
extern MMMTMemList *MMMVMemList ;
extern long MMMVmoves ;
extern long MMMVenable_move ;
#endif
#ifdef MMMDACTIV
extern long MMMVactiv_pointers ; /*Aktive S=-Zeiger*/
extern long MMMVactiv_pointers1 ;
extern long MMMVactiv ; /*Anzahl der Aktivierungen*/
extern long MMMVactiv_contents ; /*Aktive Inhalte von S-Zeigern*/
extern long MMMVinactiv ; /*Anzahl der Inahktivierungen*/
extern long MMMVreal_activ ; /*Anzahl der wirklichen Aktivierungen*/
extern long MMMVreal_inactiv ; /*Anzahl echter Inaktivierungen*/
extern long MMMVmax_activ_to_content ; /*Max Zahl von Zeigern, die auf den gleichen
                Inhalt gezeigt haben*/
extern long MMMVmax_activ_pointers ;
extern long MMMVmax_activ_contents ;

extern long MMMV_MMM_calls ;
extern long MMMV_MMM_calls_activ ;
#endif

#ifdef DOS_FILENAMES
#  include "MMMstorm.h"  /*Definition der Makros*/
#else
#  include "MMMstorage_macro.h"  /*Definition der Makros*/
#endif

/*************************************************************************
** Es folgen die deklarationen der Funktionen, die in MMMSpeicher.c     **
** definiert sind.                                                      **
*************************************************************************/
#define MMMprepare_sleep()
#define MMMsleep()
#define MMMwake_up(son)
long MMMcounter ( S_Pointer p ) ;
void MMMfreeh ( S_Pointer *p ) ;
void MMMlfreeh (S_Pointer *p) ;
long MMMsize ( S_Pointer p ) ;
long MMMtype ( S_Pointer p ) ;
S_Pointer MMMmalloc (long Memtyp, long typ , long size , long count)  ;
S_Pointer MMMpalloc (long Memtyp, long typ , long size , long count) ;
S_Pointer MMMcalloc (long Memtyp, long typ , long size , long count) ;
void MMMresize (S_Pointer *p , long size) ;
void MMMnewcounter (S_Pointer *p , long counter) ;
void MMMnewtype (S_Pointer p , long typ) ;
void MMMrealloc (S_Pointer *p , long size , long counter) ;
void MMMunmoveable (S_Pointer p , long l) ;
#define MMMfirst_init()
void MMMinit (char *) ;
void MMMexit (VOID) ;
void MMMgarbage (long time) ;
S_Pointer MMMforce_data (long Memtyp, long typ , char *data , long size) ;
void MMMshift(S_Pointer source, long spos, S_Pointer dest, long dpos, long count) ;
void MMMchange (S_Pointer *p) ;
void MMMseth (S_Pointer *p, S_Pointer *q) ;

void MMMupper_anz(S_Pointer) ;


#ifdef MMMDACTIV
S_Pointer MMM (S_Pointer) ;
long MMMactiv (S_Pointer *p) ;
void MMM_activ (S_Pointer *p) ;
void MMM_inactiv (S_Pointer *p) ;
void MMMinactiv (S_Pointer *p) ;
void MMM_loop (VOID) ;
#endif

#ifdef MMMDSPECIAL
S_Pointer *MMMPh(S_Pointer *p , long i, long art) ;
char *MMMZh(S_Pointer *p, long art) ;
char *MMMarrh (S_Pointer *p , long anz , long size, long art) ;
char *MMMmvarrh(S_Pointer *p, long mv, long anz, long size, long art) ;
char *MMMmvh (S_Pointer *p , long mv, long art) ;

#ifdef MMMMOVE
void MMMpoolinserth (char **p) ;
void MMMpooldeleteh (char **p) ;
void MMMpoolinserth1 (char **p) ;
void MMMpooldeleteh1 (char **) ;
void MMMpoolinith(VOID) ;
void MMM_move (VOID) ;
#endif

#endif

#endif /* __MMMstorage_gen__ */
