
#ifndef __MMMstorage_lean__
#define __MMMstorage_lean__

#ifdef MACINTOSH
#pragma once
#endif

#include <signal.h>
#include <stdio.h>
#define MEXDSTANDARD
#ifdef PARALLEL
#  error Sequential only, sorry...
#endif

// diese Speicherverwaltung speichert Funktionszeiger in MMM_CODE Objekten
// im Mem-Teil der S-Pointer, nicht im Header, daher:
#define MMMC_INDIRECT_CODE

#ifdef DOS_FILENAMES
#  include "MDE_decl.h"
#  include "MEX_ext.h"
#else
#  include "MDE_declare.h"
#  include "MEX_extern.h"
#endif


////////////////////////////////////////////////////////////////////////
// Allgemeine Typen                                                   //

#ifdef MSDOS
typedef unsigned char MUPbool;
#else
typedef long MUPbool;
#endif
#define MTbool MUPbool   // Fuer Module

////////////////////////////////////////////////////////////////////////
// Typen der Speicherverwaltung                                       //

typedef unsigned long MMMulong;

////////////////////////////////////////////////////////////////////////
// Verschiedenste Konstanten                                          //

#ifndef true
#   define true  1 /*Der Boolean-Wert true*/
#   define false 0 /*Der Boolean-Wert false*/
#endif
#define _true_  1
#define _false_ 0

////////////////////////////////////////////////////////////////////////
// Debuggen

#ifdef MMMDEBUG
extern void MMMassertionFailed(const char *file, int line);
#define MMMASSERT(e) \
    { if (! (e)) MMMassertionFailed(__FILE__, __LINE__); }
#else
#define MMMASSERT(e)
#endif

////////////////////////////////////////////////////////////////////////
// MMMTstatus                                                         //
// Jede MAMMUT-Zelle enth"alt Statusinformtionen, die in einer        //
// Instanz dieser Klasse gespeichert sind.                            //

class MMMTstatus
{
   unsigned int type : 16;	// MMMM-Typ
   unsigned int mark : 1;	// fuer Markierung
   unsigned int isBlk : 1;	// 1 falls der Block regulaer in einer Page
				// gespeichert ist
   unsigned int blkIdx : 4;	// Block-Index (falls isBlk=1)

public:
   void init(MMMulong typ, MUPbool is_block, int block_idx);
   void initFree(int block_idx);
   MMMulong getType();
   void setType(MMMulong typ);
   MUPbool isMarked();
   void setMark(MMMulong is_marked);
   MUPbool isBlock();
   int getBlockIdx();
};

// Initialisierung fuer MMMmalloc
inline void
MMMTstatus::init(MMMulong typ, MUPbool is_block, int block_idx)
{
    MMMASSERT(this != NULL);
    MMMASSERT(block_idx < 16);
    MMMASSERT(typ < 0x10000);
    isBlk = (unsigned int) is_block;
    blkIdx = (unsigned int) block_idx;
    type = (unsigned int) typ;
    mark = 0;
}

// Initialisierung fuer freie Zellen
inline void
MMMTstatus::initFree(int block_idx)
{
    MMMASSERT(this != NULL);
    MMMASSERT(block_idx < 16);
    isBlk = 1;
    blkIdx = (unsigned int) block_idx;
    type = 0;
    mark = 0;
}

inline MMMulong
MMMTstatus::getType()
{
    MMMASSERT(this != NULL);
    return (MMMulong) type;
}

inline void
MMMTstatus::setType(MMMulong typ)
{
    MMMASSERT(this != NULL);
    MMMASSERT(typ < 0x10000);
    type = (unsigned int) typ;
}

inline MUPbool
MMMTstatus::isMarked()
{
    MMMASSERT(this != NULL);
    return (MUPbool) mark;
}

inline void
MMMTstatus::setMark(MMMulong is_marked)
{
    MMMASSERT(this != NULL);
    mark = (unsigned int) is_marked;
}

inline MUPbool
MMMTstatus::isBlock()
{
    MMMASSERT(this != NULL);
    return (MUPbool) isBlk;
}

inline int
MMMTstatus::getBlockIdx()
{
    MMMASSERT(this != NULL);
    return (int) blkIdx;
}

////////////////////////////////////////////////////////////////////////
// MMMTheader
// Jede MAMMUT-Zelle enth"alt in einem Header Verwaltungsinformationen.
// Dieser Header wird am Anfang der Zelle gespeichert.
// Die Zelle kann frei oder belegt sein. Ist sie frei, so wird in next
// ein Zeiger auf die naechste freie Zelle gespeichert.

class MMMTheader
{
    union {
	struct {
	    MMMulong signature;		// Signatur
	    MMMulong size;		// Groesse des Mem-Teils
	    MMMulong count;		// Anzahl Kinder (S_Pointer)
	    MMMulong refs;		// Referenz-Zaehler
	} sp;
	MMMTheader *next;		// Zeiger auf naechsten freien Block
    };
    MMMTstatus status;

public:
    void init(MMMulong s, MMMulong c, MMMulong t, MUPbool b, int i);
    void initFree(MMMTheader *next_free, int block_idx);
    MMMulong getSignature();
    void setSignature(MMMulong sig);
    MMMulong getSize();
    void setSize(MMMulong s);
    MMMulong getCount();
    void setCount(MMMulong c);
    int release();
    void addRef();
    MMMulong getRefs();
    MMMulong getType();
    void setType(MMMulong t);
    MUPbool isMarked();
    void setMark(MMMulong is_marked=_true_);
    MUPbool isBlock();
    int getBlockIdx();
    MMMTheader *getNextFree();
    void setNextFree(MMMTheader *s);
    MMMulong getLength();
};

// Initialisierung fuer MMMmalloc
inline void
MMMTheader::init(MMMulong siz, MMMulong cnt, MMMulong typ,
			MUPbool is_blk, int blk_idx)
{
    MMMASSERT(this != NULL);

    status.init(typ, is_blk, blk_idx);
    sp.size = siz;
    sp.count = cnt;
    sp.signature = 0;
    sp.refs = 1;
}

// Initialisierung fuer freie S-Pointer
inline void
MMMTheader::initFree(MMMTheader *next_free, int blk_idx)
{
    status.initFree(blk_idx);
    MMMASSERT(this != NULL);
    next = next_free;
}

inline MMMulong
MMMTheader::getSignature()
{
    MMMASSERT(this != NULL);
    return sp.signature;
}

inline void
MMMTheader::setSignature(MMMulong sig)
{
    MMMASSERT(this != NULL);
    sp.signature = sig;
}

inline MMMulong
MMMTheader::getSize()
{
    MMMASSERT(this != NULL);
    return sp.size;
}

inline void
MMMTheader::setSize(MMMulong s)
{
    MMMASSERT(this != NULL);
    sp.size = s;
}

inline MMMulong
MMMTheader::getCount()
{
    MMMASSERT(this != NULL);
    return sp.count;
}

inline void
MMMTheader::setCount(MMMulong c)
{
    MMMASSERT(this != NULL);
    sp.count = c;
}

inline int
MMMTheader::release()
{
    MMMASSERT(this != NULL);
    MMMASSERT(sp.refs != 0);
    if (sp.refs == 0) {
	return _false_;		// object has already been freed,
				// avoid infinite recursion
    } else {
	sp.refs--;
        return (sp.refs == 0);
    }
}

inline void
MMMTheader::addRef()
{
    MMMASSERT(this != NULL);
    sp.refs++;
}

inline MMMulong
MMMTheader::getRefs()
{
    MMMASSERT(this != NULL);
    return sp.refs;
}

inline MMMulong
MMMTheader::getType()
{
    MMMASSERT(this != NULL);
    return status.getType();
}

inline void
MMMTheader::setType(MMMulong t)
{
    MMMASSERT(this != NULL);
    status.setType(t);
}

inline MUPbool
MMMTheader::isMarked()
{
    MMMASSERT(this != NULL);
    return status.isMarked();
}

inline void
MMMTheader::setMark(MMMulong is_marked)
{
    MMMASSERT(this != NULL);
    status.setMark(is_marked);
}

inline MUPbool
MMMTheader::isBlock()
{
    MMMASSERT(this != NULL);
    return status.isBlock();
}

inline int
MMMTheader::getBlockIdx()
{
    MMMASSERT(this != NULL);
    return status.getBlockIdx();
}

inline MMMTheader *
MMMTheader::getNextFree()
{
    MMMASSERT(this != NULL);
    return next;
}

inline void
MMMTheader::setNextFree(MMMTheader *s)
{
    MMMASSERT(this != NULL);
    next = s;
}

inline MMMulong
MMMTheader::getLength()
{
    MMMASSERT(this != NULL);
    return sizeof(MMMTheader) + sp.count * sizeof(MMMTheader *) +
	   sp.size;
}


typedef MMMTheader *S_Pointer;


////////////////////////////////////////////////////////////////////////
// "Offentliche Variablen

extern MMMulong MMMVused_blocks;
extern MMMulong MMMVused_memory;
extern MMMulong MMMVreserved_blocks;
extern MMMulong MMMVreserved_memory;
extern MMMulong MMMVmax_reserved_memory;
extern MMMulong MMMVfrees;
extern MMMulong MMMVrealfrees;
extern MMMulong MMMVmallocs;
extern void (*MMMVsystemfree)(S_Pointer s);
extern void (*MMMVsystemchange)(S_Pointer *s);
typedef char *(*MMMTno_memory)(MUPbool);
extern MMMTno_memory MMMVno_memory;
extern MMMulong MMMVnumber_of_processes;
extern MMMulong MMMVson;
extern MMMulong MMMVlocal_son;
extern MMMulong MMMVnode;
extern MMMulong MMMVnetnode;
extern S_Pointer MMMVtopology;
extern const S_Pointer MMMNULL;
extern const MMMulong MMMCwork;
extern const MMMulong MMMCglobal;
extern const MMMulong MMMClocal;
extern const MMMulong MMMread;
extern const MMMulong MMMwrite;

#define MMMEsharemalloc MMMEmalloc

////////////////////////////////////////////////////////////////////////
// Hilfsvariablen

extern S_Pointer MMMVsharedh;
extern S_Pointer MMMVlocal_sharedh;

////////////////////////////////////////////////////////////////////////
// Deklaration der "offentlichen Schnittstelle

S_Pointer MMMmalloc(MMMulong Memtyp, MMMulong typ,
			   MMMulong size, MMMulong count);
S_Pointer MMMpalloc(MMMulong Memtyp, MMMulong typ,
			   MMMulong size, MMMulong count);
S_Pointer MMMcalloc(MMMulong Memtyp, MMMulong typ,
			   MMMulong size, MMMulong count);

void MMMresize(S_Pointer *s, MMMulong size);
void MMMnewcounter(S_Pointer *s, MMMulong count);
void MMMrealloc(S_Pointer *s, MMMulong size, MMMulong count);
void MMMchange(S_Pointer *s);

inline void MMMfree(S_Pointer s);
inline void MMMlfree(S_Pointer s);

inline char *MMMEmalloc(MMMulong size);
inline void MMMEfree(char *);

inline int MMMequal(S_Pointer s, S_Pointer t);
inline void MMMset(S_Pointer *p, S_Pointer s);
inline void MMMreplace(S_Pointer *p, S_Pointer s);
inline void MMMcopy(S_Pointer s);
inline void MMMshift(S_Pointer sour, MMMulong spos,
		    S_Pointer dest, MMMulong dpos, MMMulong count);

inline S_Pointer *MMMP(S_Pointer *s, MMMulong i, MMMulong use);

inline MMMulong MMMsize(S_Pointer s);
inline MMMulong MMMcounter(S_Pointer s);
inline MMMulong MMMtype(S_Pointer s);
inline void MMMnewtype(S_Pointer s, MMMulong typ);
inline MMMulong MMMsignature(S_Pointer s);
inline void MMMnewsignature(S_Pointer s, MMMulong sig);

inline S_Pointer MMMread_shared(MMMulong i);
inline void MMMwrite_shared(MMMulong i, S_Pointer s);
inline S_Pointer MMMread_local_shared(MMMulong i);
inline void MMMwrite_local_shared(MMMulong i, S_Pointer s);
inline void MMMprepare_sleep();
inline void MMMsleep();
inline void MMMwake_up(MMMulong pid);

void MMMinsert_global(S_Pointer *s);
void MMMmark_globals();
void MMMmark_tree(S_Pointer s);
inline MUPbool MMMfree_unmarked(S_Pointer s);
void MMMfreeUnmarked(S_Pointer s);
void MMMfree_unused();

void MMMfirst_init();
void MMMinit(char *name);
void MMMexit();

inline MMMulong MMMrefs(S_Pointer s);

long MMMbytes(S_Pointer s);
inline MMMulong MMMused_memory();
inline MMMulong MMMreserved_memory();

////////////////////////////////////////////////////////////////////////
// Deklaration der Hilfsfunktionen

inline char *MMMgetMemAdr(S_Pointer s);
inline char *MMMgetMemAdr(S_Pointer s, MMMulong mv);

extern void MMMrecFree(S_Pointer s);
extern void MMMfreeBlock(S_Pointer s);

#if defined MSDOS || defined MACINTOSH
void *MMMsysIrrMalloc(MMMulong len);
void MMMsysIrrFree(void *p);
#else
inline void *MMMsysIrrMalloc(MMMulong len);
inline void MMMsysIrrFree(void *p);
#endif


////////////////////////////////////////////////////////////////////////
// Definitionen der "offentlichen Schnittstelle
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// Zugriff auf den Inhalt

#define MMMZ(s,T,use) \
           ((T *)MMMgetMemAdr(*(s)))

#define MMMarr(s,i,T,use) \
           ((T *)MMMgetMemAdr(*(s), (i)*sizeof(T)))

#define MMMmv(s,mv,T,use) \
           ((T *)MMMgetMemAdr(*(s), (mv)))

#define MMMmvarr(s,mv,i,T,use) \
           ((T *)MMMgetMemAdr(*(s), (mv) + (i)*sizeof(T)))

inline S_Pointer *
MMMP(S_Pointer *s, MMMulong i, MMMulong /*use*/)
{
    MMMASSERT(s != NULL && *s != NULL);
    S_Pointer *p = (S_Pointer *)((*s)+1);
    return p+i;
}

////////////////////////////////////////////////////////////////////////
// Allokieren von Speicher

inline char *
MMMEmalloc(MMMulong size)
{
    char *p = (char *) MMMsysIrrMalloc(size);
    if (p == NULL)
	p = (*MMMVno_memory)(_false_);
    return p;
}

inline void
MMMEfree(char *p)
{
    MMMsysIrrFree((void *) p);
}


////////////////////////////////////////////////////////////////////////
// Freigabe                                                           //
////////////////////////////////////////////////////////////////////////

inline void
MMMfree(S_Pointer s)
{
    if (s != MMMNULL && s->release()) {
	MMMrecFree(s);
    }
}

inline
void MMMlfree(S_Pointer s)
{
    if (s != MMMNULL && s->release()) {
	MMMfreeBlock(s);
    }
}

inline MUPbool
MMMfree_unmarked(S_Pointer p)
{
    if (p == MMMNULL)
	return _true_;
    else
    if (p->isMarked())
	return _false_;
    else {
        MMMfreeUnmarked(p);
        return _true_;
    }
}

////////////////////////////////////////////////////////////////////////
// Vergleichen und Kopieren                                           //
////////////////////////////////////////////////////////////////////////

inline int
MMMequal(S_Pointer s, S_Pointer t)
{
    return s == t;
}

inline void
MMMset(S_Pointer *p, S_Pointer s)
{
    MMMASSERT(p != NULL);
    MMMcopy(s);
    *p = s;
}

inline void
MMMreplace(S_Pointer *p, S_Pointer s)
{
    MMMASSERT(p != NULL);
    MMMcopy(s);
    MMMfree(*p);
    *p = s;
}

inline void
MMMcopy(S_Pointer s)
{
    if (s != NULL) s->addRef();
}

inline void
MMMshift(S_Pointer sour, MMMulong spos, S_Pointer dest, MMMulong dpos,
	 MMMulong count)
{
    MMMASSERT(sour != NULL && dest != NULL);

    S_Pointer *p, *q;
    MMMulong i;
    if (spos < dpos) {
	p = MMMP(&sour, spos+count-1, MMMread);
	q = MMMP(&dest, dpos+count-1, MMMwrite);
	for (i=count; i != 0L; i--) {
	    *q-- = *p--;
	}
    } else {
	p = MMMP(&sour, spos ,MMMread);
	q = MMMP(&dest, dpos ,MMMwrite);
	for (i=count; i != 0L; i--) {
	    *q++ = *p++;
	}
    }
}

////////////////////////////////////////////////////////////////////////
// Poolverwaltung                                                     //
////////////////////////////////////////////////////////////////////////

#define MMMpoolinsert(p)
#define MMMpooldelete(p)

inline void
MMMpoolinit()
{
}

////////////////////////////////////////////////////////////////////////
// Ende eines Zugriffs                                                //
////////////////////////////////////////////////////////////////////////

#define MMMinactiv(p)
#define MMM(p) (p)

////////////////////////////////////////////////////////////////////////
// Zugriff auf Statusinformationen                                    //
////////////////////////////////////////////////////////////////////////

inline MMMulong
MMMsize(S_Pointer s)
{
    return ((s == MMMNULL) ? 0 : s->getSize());
}

inline MMMulong
MMMcounter(S_Pointer s)
{
    return ((s == MMMNULL) ? 0 : s->getCount());
}

inline MMMulong
MMMtype(S_Pointer s)
{
    MMMASSERT(s != NULL);
    return s->getType();
}

inline void
MMMnewtype(S_Pointer s, MMMulong typ)
{
    MMMASSERT(s != NULL);
    s->setType(typ);
}

inline MMMulong
MMMsignature(S_Pointer s)
{
    MMMASSERT(s != NULL);
    return s->getSignature();
}

inline void
MMMnewsignature(S_Pointer s, MMMulong sig)
{
    MMMASSERT(s != NULL);
    s->setSignature(sig);
}

#define MMMlock(p)
#define MMMunlock(p)
#define MMMunmoveable(p,n)

////////////////////////////////////////////////////////////////////////
// Kommunikation mit anderen Prozessen                                //
////////////////////////////////////////////////////////////////////////

inline S_Pointer
MMMread_shared(MMMulong /* i */)
{
    return MMMVsharedh;
}

inline void
MMMwrite_shared(MMMulong /* i */, S_Pointer s)
{
    MMMVsharedh = s;
}

inline S_Pointer
MMMread_local_shared(MMMulong /* i */)
{
    return MMMVlocal_sharedh;
}

inline void
MMMwrite_local_shared(MMMulong /* i */, S_Pointer s)
{
    MMMVlocal_sharedh = s;
}

inline void
MMMprepare_sleep()
{
}

inline void
MMMsleep()
{
}

inline void
MMMwake_up(MMMulong /* pid */)
{
}

////////////////////////////////////////////////////////////////////////
// Statistik
////////////////////////////////////////////////////////////////////////

inline MMMulong MMMused_memory()
{
    return MMMVused_memory;
}

inline MMMulong MMMreserved_memory()
{
    return MMMVreserved_memory;
}

////////////////////////////////////////////////////////////////////////
// "Eigentlich" interne Funktionen
////////////////////////////////////////////////////////////////////////

inline MMMulong
MMMrefs(S_Pointer s)
{
    MMMASSERT(s != NULL);
    return s->getRefs();
}

////////////////////////////////////////////////////////////////////////
// Hilfsfunktionen                                                    //
////////////////////////////////////////////////////////////////////////

#if (! defined MSDOS) && (! defined MACINTOSH)

// MMMsysIrrMalloc -- alloziiert irregulaeren (Non-Page) Bereich
// mit Hilfe der "uebergeordneten" Speicherverwaltung.

inline void *
MMMsysIrrMalloc(MMMulong len)
{
    return malloc(len);
}

inline void
MMMsysIrrFree(void *p)
{
    free(p);
}

#endif

inline char *
MMMgetMemAdr(S_Pointer s)
{
    MMMASSERT(s != NULL);
    S_Pointer *q = (S_Pointer *)(s+1);
    char *p = (char *)(q+s->getCount());
    return p;
}

inline char *
MMMgetMemAdr(S_Pointer s, MMMulong mv)
{
    return MMMgetMemAdr(s) + mv;
}

////////////////////////////////////////////////
// Kompatibilitaet mit paralleler Version
////////////////////////////////////////////////

typedef char MMMTlock;

inline void MMMElock(MMMTlock * /* l */)
{
}

inline void MMMEunlock(MMMTlock * /* l */)
{
}

#endif /* __MMMstorage_auto__ */

