/* $Date: 1997/08/25 11:21:20 $  $Author: kg $  $Revision: 1.6 $ */

/**************************************************************************
**                        MMMSpeicherMakro.h                             **
** In diesem File werden die Makros, die von der Speicherverwaltung      **
** benutzt werden, definiert. Bei der Verwendung dieser Makros ist, wie  **
** immer beider Verwendung von Makros, Vorsicht vor Nebeneffekten        **
** geboten.                                                              **
** Zum grossen Teil ersetzen diese Makros Funktionen, die von der        **
** Speicherverwaltung nach aussen gereicht werden, dies geschieht aus    **
** Geschwindigkeitsgruenden, oder weil die Funktion nicht anders zu      **
** realisieren ist.                                                      **
**                                                                       **
** Erstellung      : 10/10/90                                            **
** Letzte Aenderung: 09/04/91                                            **
** Autor           : Holger Naundorf                                     **
**************************************************************************/

#ifndef __MMMstorage_macro__
#define __MMMstorage_macro__


/**************************************************************************
** In diesem File sind folgende Makros definiert:                        **
** void MMMdeclare_pointer ( S_Pointer p )                               ** 
** long MMMnumber_pointer (S_Pointer p)                                  **
** S_Pointer *MMMP ( S_Pointer p ; long i )                              **
** T *MMMZ ( S-Pointer p ; C-Typ T )                                     **
** T *MMMarr ( S_Pointer p ; long anz ; C-Typ T )                        **
** T *MMMmvarr ( S-Pointer p ; long anz,mv ; C-Typ T )                   **
** T *MMMmv ( S_Pointer p ; long mv ; C-Typ T )                          **
** void MMMchange_data ( S_Pointer *p )                                  **
** void MMMchange_pointer ( S_Pointer *p )                               **
** void MMMchange ( S_Pointer *p )                                       **
** void MMMset ( S_Pointer *p , q )                                      **
** long MMMequal ( S_Pointer p , q )                                     **
** void MMMinactiv ( S_Pointer *s )                                      **
** void MMMnewdata (S_Pointer s )                                        **
** void MMMpoolinit ()                                                   **
** void MMMpoolinsert (T **p)                                            **
** void MMMpooldelete (T **p)                                            **
** MMMsignature (S_Pointer s)                                            **
** MMMdebug S_Pointer s)                                                 **
**************************************************************************/

#define MMMlock(s)
#define MMMunlock(s)


/**************************************************************************
** void MMMwrite_shared(long number, S_Pointer value)                    **
** Dieses Makro schreibt den Wert value in die number-te shared          **
** Variable.                                                             **
**************************************************************************/
#define MMMwrite_shared(number,value)  {  MMMVsharedh = value ; }


/************************************************************************** 
** S_Pointer MMMread_shared(long number)                                 **
** Dieses Makro liest den Wert aus der number-ten shared-Variablen.      **
**************************************************************************/ 
#define MMMread_shared(number) MMMVsharedh


/************************************************************************** 
** void MMMwrite_local_shared(long number, S_Pointer value)              ** 
** Dieses Makro schreibt den Wert value in die number-te shared          ** 
** Variable.                                                             ** 
**************************************************************************/ 
#define MMMwrite_local_shared(number,value) {  MMMVlocal_sharedh = value ; } 


/**************************************************************************  
** S_Pointer MMMread_local_shared(long number)                           ** 
** Dieses Makro liest den Wert aus der number-ten shared-Variablen.      ** 
**************************************************************************/  
#define MMMread_local_shared(number) MMMVlocal_sharedh 
 


/**************************************************************************
** void MMMcheck( long Memtyp, typ, size, count )                        **
** Diese Funktion "uberpr"uft, ob die "ubergebenen Argumente in einem    **
** vern"unftigen Bereich liegen. Ist dies nicht der Fall, so wird        **
** MMMVno_memory() aufgerufen.                                           **
**************************************************************************/
#define MMMcheck(Memtyp,typ,size,count) \
           {  if ( size<0 || size>0x02000000 || count<0 || count>0x02000000 ) \
                 {  MMMVno_memory(_false_) ; } \
           }


/**************************************************************************
** void MMMcheck_pointer ( S_Pointer p )                                 **
** Die Funktion ueberprueft, ob p ein S-Zeiger sein kann. Dies geschieht **
** mit Hilfe eines Eintrages im Header, der einen bestimmten Wert habe   **
** muss.                                                                 **
**************************************************************************/
#ifndef MMMDEBUG
#define MMMcheck_pointer(p)
#else
#define MMMcheck_pointer(p) \
          { char MMMVcheck_pointer ;\
           if (MMMh(p) != MMMNULL ) \
              {  if ( MMMh(p)->check != MMMVcheck ) \
                    {  MIO_print_string ("Zeiger ist kein S-Zeiger\n") ; \
                       MIO_print_string ("Zum Fortfahren RETURN druecken!!"); \
                       scanf("%c",&MMMVcheck_pointer) ; \
                    } \
              } \
           }
#endif


#define MMMinit_check_pointer( ) \
           {  MMMVcheck = 789 ; }
/**************************************************************************
** MMMDnext(S_Pointer s)                                                 **
** Dieser Makro dereferenziert auf den next-Teil eines S-Zeigers.        **
**************************************************************************/
#ifndef MMMDEBUG
#define MMMDnext(s) (MMMh(s)->un.next)
#else
#define MMMDnext(s) (MMMh(s)->nextt)
#endif


/**************************************************************************
** MMMheader *MMM(S_Pointer s)                                           **
** Dieser Makro wandelt den S-Zeiger s so um, dass aus ihm ein Zeiger    **
** auf einen MMMheader wird.                                             **
**************************************************************************/
#ifdef MMMDACTIV
#define MMMh(s) ((S_Pointer)((long)(s) & 0xfffffffe))
#else
#define MMM(s) (s)
#define MMMh(s) (s)
#endif

#ifdef MMMDACTIV
#define MMMactiv1(p) ((S_Pointer)(((long)*(p)) & 1L))
#define MMM_activ1(p) *(p) = (S_Pointer)(((long)*(p)) | 1L)
#endif

/**************************************************************************
** void MMMreplace (S_Pointer *s , value)                                **
** Die Funktion gibt den S-Zeiger, auf den s zeigt frei und setzt an     **
** diese Stelle value. Dieser S-Zeiger wird dem System auch angemeldet.  **
**************************************************************************/
#define MMMreplace(s,value) \
           { S_Pointer MMMVhelpp1 = *s ;\
             S_Pointer MMMVhelpp2 = value ; \
              MMMset (s,MMMVhelpp2) ; \
              MMMfree (MMMVhelpp1 ) ; }


/**************************************************************************
** long MMMrefs (S_Pointer s)                                            **
** Diese Funktion gibt die Anzahl der S-Zeiger, die gleich s sind,       **
** zurueck.                                                              **
**************************************************************************/
#define MMMrefs(s) (MMMh(s)->un.status.anz)
/**************************************************************************
** void MMMfree (S_Pointer p)                                            **
** Die Funktion gibt den S-Zeiger p frei.                                **
**************************************************************************/
#define MMMfree(p) MMMfreeh(&p)
#define MMMlfree(p) MMMlfreeh(&p)

/**************************************************************************
** void MMMdeclare_pointer ( S_Pointer p )                               **
** Die Funktion teilt dem Speicherbereich, der p zugeordnet ist mit,     **
** dass ein neuer S-Zeiger auf ihn zeigt, d.h. der Zaehler wird um 1     **
** erhoeht.                                                              **
**************************************************************************/
#define MMMdeclare_pointer(p) MMMupper_anz(p)

/**************************************************************************
** void MMMcopy ( S_Pointer s )                                          **
** Die Funktion teilt dem Speicherbereich, der p zugeordnet ist mit,     **
** dass ein neuer S-Zeiger auf ihn zeigt, d.h. der Zaehler wird um 1     **
** erhoeht.                                                              **
**************************************************************************/
#define MMMcopy(s) MMMupper_anz(s)


/**************************************************************************
** long MMMnumber_pointer (S_Pointer p)                                  **
** Dieses Makro gibt die Anzahl der S-Zeiger, die auf den Speicher-      **
** bereich, auf den p zeigt, weisen, an.                                 **
**************************************************************************/
#ifndef MMMDEBUG
#   define MMMnumber_pointer(p) ((long)MMM(p)->un.status.anz)
#endif
#ifdef MMMDEBUG
#   define MMMnumber_pointer(p) ((long)(p)->status.anz)
#endif


/**************************************************************************
** S_Pointer *MMMP ( S_Pointer *p ; long i , use)                        **
** Die Funktion liefert einen C-Zeiger auf den (i+1)-ten S-Zeiger, der   **
** S-Zeiger, die dem S-Zeiger p zugeordnet sind.                         **
**************************************************************************/
#ifndef MMMDSPECIAL
#define MMMP(p,i,use) (((S_Pointer *)((*(p))->inh))+(i))
#else
#define MMMP(p,i,use) MMMPh(p,i,use)
#endif

/**************************************************************************
** T *MMMZ ( S_Pointer *p ; C-Typ T ; long use )                         **
** Die Funktion liefert einen C-Zeiger auf das erste Byte des Speicher-  **
** bereiches, der p zugeordnet ist, welches hinter dem Bereich fuer die  **
** S-Zeiger in dem Bereich liegt.                                        **
**************************************************************************/
#ifndef MMMDSPECIAL
#define MMMZ(p,T,use) ((T *)((((S_Pointer *)((*(p))->inh))) + MMMcounter(*(p))))
#else
#define MMMZ(p,T,use) ((T *)MMMZh(p,use))
#endif

/**************************************************************************
** T *MMMarr ( S_Pointer *p ; long anz ; C-Typ T ; long use )            **
** Die Funktion interpretiert den Teil des Speicherbereiches, der p      **
** zugeordnet ist, der hinter den S-Zeigern liegt, als Array mit Ein-    **
** traegen vom Typ T. Die Funktion liefert einen C-Zeiger auf das anz-te **
** Element dieses Arrayss, wobei das erste Element den Index 0 hat.      **
**************************************************************************/
#ifndef MMMDSPECIAL
#define MMMarr(p,anz,T,use) (((T *)(((S_Pointer *)((*(p))->inh)) + MMMcounter(*(p))))+(anz))
#else
#define MMMarr(p,anz,T,use) ((T *)MMMarrh(p,anz,(long) sizeof(T),use))
#endif

/**************************************************************************
** T *MMMmvarr ( S-Pointer *p ; long mv,anz ; C-Typ T ; long use )       **
** Die Funktion nimmt an, dass beim mv-ten Byte des Teiles des Speicher- **
** bereiches, der p zugeordnet ist, der hinter den S-Zeigern liegt, ein  **
** Array mit Eintraegen vom Typ T beginnt. Die Funktion liefert einen    **
** C-Zeiger auf das anz-te Element, wobei das erste Element den Index 0  **
** hat.                                                                  **
**************************************************************************/
#ifndef MMMDSPECIAL
#define MMMmvarr(p,mv,anz,T,use) (((T *)(((char *)(((S_Pointer *)((*(p))->inh)) + MMMcounter(*(p)))) + mv)) + anz )
#else
#define MMMmvarr(p,mv,anz,T,use) ((T *)MMMmvarrh(p,mv,anz,(long) sizeof(T),use))
#endif

/**************************************************************************
** T *MMMmv ( S_Pointer *p ; long mv ; C-Typ T ; long use)               **
** Die Funktion liefert einen C-Zeiger auf das mv-te Byte hinter den     **
** S-Zeigern im Speicherbereich, der p zugeordnet ist.                   **
**************************************************************************/
#ifndef MMMDSPECIAL
#define MMMmv(p,mv,T,use) ((T *) (((char *)(((S_Pointer *)((*(p))->inh)) + MMMcounter(*(p)))) + mv))
#else
#define MMMmv(p,mv,T,use) ((T *)MMMmvh(p,mv,use))
#endif

/**************************************************************************
** void MMMchange_data ( S_Pointer *p )                                  **
** Die Funktion stellt sicher, dass kein anderer S-Zeiger als p auf die  **
** Daten zeigt. Sollten mehr S-Zeiger auf die Daten von p verweisen, so  **
** werden die Daten kopiert und p entsprechend umgebogen.                **
**************************************************************************/
#define MMMchange_data(p) {if (MMMnumber_pointer(*(p))>1) MMMensure(p) ;}

/**************************************************************************
** void MMMchange_pointer ( S_Pointer *p )                               **
** Die Funktion stellt sicher, dass kein anderer S-Zeiger als p auf die  **
** Daten zeigt. Sollten mehr S-Zeiger auf die Daten von p verweisen, so  **
** werden die Daten kopiert und p entsprechend umgebogen.                **
**************************************************************************/
#define MMMchange_pointer(p) {if (MMMnumber_pointer(*(p))>1) MMMensure(p) ;}


/**************************************************************************
** void MMMFehlerbehandlung                                              **
** Dieser Makro uebernimmt die Fehlerbehandlung, solange MMMerror_handler**
** noch nicht angesprungen wird.                                         **
**************************************************************************/
#define MMMFehlerbehandlung {MIO_print_string ("Fehler in der Speicherverwaltung\n\n") ; exit(1) ; }


/**************************************************************************
** long MMMequal ( S_Pointer p , q )                                     **
** Diese Funktion testet die S-Zeiger p und q auf Gleichheit. Sind beide **
** S-Zeiger gleich, so wird 1L zurueckgegeben, sonst 0L                  **
**************************************************************************/
#define MMMequal(p,q) (MMM(p)==MMM(q))

/**************************************************************************
** void MMMset (S_Pointer *p , q)                                        **
** Dieser Makro weist *p den Wert von q zu.                              **
**************************************************************************/
#define MMMset(p,q) MMMseth(p,&q)

#ifndef MMMDACTIV
#define MMMinactiv(s)
#endif

#define MMMnewdata(s)

#ifndef MMMMOVE
#define MMMpoolinit()
#else
#define MMMpoolinit() MMMpoolinith()
#endif

#ifndef MMMMOVE
#define MMMpoolinsert(p)
#define MMMpoolinserth1(p)
#else
#define MMMpoolinsert(p) MMMpoolinserth((char **)p)
#endif

#ifndef MMMMOVE
#define MMMpooldelete(p)
#define MMMpooldeleteh1(p)
#else
#define MMMpooldelete(p) MMMpooldeleteh((char **)p)
#endif                            


/**************************************************************************
** MMMsignature (S_Pointer s)                                            **
** Diese Funktion berechnet einen C-Zeiger sig auf die Signatur von s    **
** und ersetzt den Aufruf der Funktion durch *sig, d.h. mit diesem       **
** Makro kann der Signatur sowohl ein Wert zugewiesen werden, als auch   **
** der Wert der Signatur ausgelesen werden.                              **
**************************************************************************/
#define MMMsignature(s) (MMMh(s)->signature)

#define MMMnewsignature(s,l) ((MMMh(s)->signature) = l )

#ifdef DBX

/**************************************************************************
** MMMdebug (S_Pointer s)                                                **
** Diese Funktion berechnet einen C-Zeiger sig auf die Debug-Information **
** von sund ersetzt den Aufruf der Funktion durch *sig, d.h. mit diesem  **
** Makro kann der Debug-Information sowohl ein Wert zugewiesen werden,   **
** als auch der Wert der Debug-Information ausgelesen werden.            **
**************************************************************************/

#define MMMdebug(s) (MMMh(s)->debug)

#endif

#define MMMinsert_global(variable)
#define MMMmark_globals() 
#define MMMmark_tree(p) 
#define MMMfree_unmarked(p) _false_
#define MMMfree_unused()

#define MMMbytes(p) 0L

#define MMMEmalloc(n)                   malloc(n)
#define MMMEfree(n)                     free(n)


#endif /* __MMMstorage_macro__ */
