/* $Date: 1997/10/07 06:52:13 $  $Author: kg $  $Revision: 1.16 $ */

/**************************************************************************/
/* FILE:             MPO_poly_help.h                                      */
/* Autor:            Klaus Gottheil                                       */
/* Erstellt:         23/04/93                                             */
/* Letzte Aenderung: 20/12/94                                             */
/**************************************************************************/

#ifndef __MPO_poly_help__
#define __MPO_poly_help__


/*----------------------------------------------------------------------
 * Konstanten
 */
 
#define MPOC_EXPR        0L     /* Koeffizienten-Ring Expr */
#define MPOC_INTMOD      1L     /* Koeffizienten-Ring IntMod(...) */
#define MPOC_DOM         2L     /* Koeffizienten-Ring ist ein Domain */
 
#define MPOC_NOORD       0L	/* unbekannte Ordnung */
#define MPOC_LEXORD      1L	/* lexikographische Ordnung */
#define MPOC_DEGORD      2L	/* totale Gradordnung */
#define MPOC_DEGINVLEX   3L	/* umgek. lexikographische Ordnung */

#define MPOC_MAX_NORM	 0L	/* Maximums-Norm */
#define MPOC_ONE_NORM	 1L	/* 1-Norm */
#define MPOC_N_NORM	 2L	/* n-Norm */

#define MPOC_no_name "<<unknown>>"


/*----------------------------------------------------------------------
 * Makros
 */
 
/* aktueller Funktionsname fuer Fehlermeldungen setzen */
#define MPO_fname(s) { MPOV_fname = (s); }
 
/* Makro fuer Fehlerausgaben mit MPOV_fname */
#define MPO_error(e) { MEV_set_error(e, MPOV_fname); }
 

/*
 *      ACHTUNG: Die folgenden Makros aktivieren die betr. S-Pointer.
 */
 
/* Kategorie von 's' */
#define MPO_cat(s) MMMZ(&(s),CATEGORY,MMMread)->cat
 
/* Special-Bits von 's' */
#define MPO_spec(s) MMMZ(&(s),CATEGORY,MMMread)->spec
 
/* TRUE falls Special-Bits von 's' gesetzt sind */
#define MPO_has_sign(s) \
    (MMMZ(&(s),CATEGORY,MMMread)->spec & (MINUS_BIT | DIV_BIT | NOT_BIT))
 
/* TRUE falls '-' oder 'not'-Bits von 's' gesetzt sind */
#define MPO_neg_sign(s) \
    (MMMZ(&(s),CATEGORY,MMMread)->spec & (MINUS_BIT | NOT_BIT))
 
/* TRUE falls 's' eine Zahl ist */
#define MPO_is_number(s) (MMMZ(&(s),CATEGORY,MMMread)->cat < CAT_STRING)
 
/* TRUE falls String 's' derjenige fuer System-Funktion 'f' ist */
#define MPO_is_sys_func(s,f) \
    (MCM_compare_MMMstring_ident(MTRV_SYS_FUNC_LIST[(f)], (s)))
 
/* der long-Wert zum MMM_INT 's' */
#define MPO_int(s) (*MMMZ(&(s),long,MMMread))
 
/* erzeugt aus der PARI-Zahl 'g' eine PARI-Kategorie _ohne_ den PARI-Stack
 * zu loeschen */
#define MPO_make_cat_pari(g) \
    MCA_init_cat_pari((g), MTR_new_category(MMMCglobal,1L), 0L)


/*----------------------------------------------------------------------
 * Typen
 */
 
/* Typ-Struktur fuer Polynome, enthaelt ua. den Koeffizienten-Ring.
 * Die originale Polynom-Struktur wird in diese Struktur umgewandelt, damit
 * ein schnellerer Zugriff auf die Informationen moeglich ist. */
 
typedef struct _MPOT_poly_type {
 
    long nindets;               /* Anzahl der Unbestimmten */
 
    long ring;                  /* der Koeffizienten-Ring */
    S_Pointer dom;              /* der Domain */
    S_Pointer mod;              /* der Modulus bei IntMod */
 
    GEN gmod;                   /* der Modulus als PARI-Zahl */
 
    S_Pointer Zero;             /* Die 0 im Koeffizienten-Ring */
    S_Pointer one;              /* Die 1 im Koeffizienten-Ring */
 
    S_Pointer plus;		/* Dummy-Aufruf der Methode _plus */
    S_Pointer mult;		/* Dummy-Aufruf der Methode _mult */
    S_Pointer power;		/* Dummy-Aufruf der Methode _power */
    S_Pointer negate;		/* Dummy-Aufruf der Methode negate */
    S_Pointer divide;		/* Dummy-Aufruf der Methode divide */
    S_Pointer intmult;		/* Dummy-Aufruf der Methode intmult */
    S_Pointer norm;		/* Dummy-Aufruf der Methode norm */
    S_Pointer convert;		/* Dummy-Aufruf der Methode convert */
    S_Pointer expr;   		/* Dummy-Aufruf der Methode expr */

} MPOT_poly_type;
 
 
/* Struktur mit Zeigern auf die Grund-Operationen fuer Koeffizienten-Ringe. */

typedef struct _MPOT_coeff_ops {
 
    /* Umwandlung von Ausdruecken in Koeffizienten */
    S_Pointer (*expr2coeff)(S_Pointer, MPOT_poly_type *);
    /* Umwandlung von Koeffizienten in Ausdruecken */
    S_Pointer (*coeff2expr)(S_Pointer, MPOT_poly_type *);
    /* Addition von Koeffizienten */
    S_Pointer (*plus)(S_Pointer, S_Pointer, MPOT_poly_type *);
    /* Multiplikation von Koeffizienten */
    S_Pointer (*mult)(S_Pointer, S_Pointer, MPOT_poly_type *);
    /* Division von Koeffizienten */
    S_Pointer (*divide)(S_Pointer, S_Pointer, MPOT_poly_type *);
    /* Potenzieren von Koeffizienten */
    S_Pointer (*power)(S_Pointer, long, MPOT_poly_type *);
    /* Negieren von Koeffizienten */
    S_Pointer (*negate)(S_Pointer, MPOT_poly_type *);

} MPOT_coeff_ops;
 
 
/* Position von Exponenten einer Unbestimmten im Datenteil eines Polynoms */
 
typedef struct _MPOT_expos {
    long exp;                           /* Exponent */
    long pos;                           /* Position im Datenteil */
} MPOT_expos;
 
 
/*----------------------------------------------------------------------
 * lokale Funktionen
 */

/* Funktionen in MPO_poly.c */

S_Pointer MPO_convert_coeffs(S_Pointer,
		S_Pointer (*)(S_Pointer, MPOT_poly_type *),
		MPOT_poly_type *, long, S_Pointer);
S_Pointer MPO_poly(S_Pointer, S_Pointer, MPOT_poly_type *, MUPbool *, S_Pointer);
S_Pointer MPO_nth_coeff_univar(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_nth_coeff_multivar(S_Pointer, long, long, S_Pointer *,
				 MPOT_poly_type *);
S_Pointer MPO_list2poly(S_Pointer, long, long, MPOT_poly_type *);

/* Funktionen in MPO_utils.c */

S_Pointer MPO_parse_indets(S_Pointer, long *);
MUPbool   MPO_is_indet(S_Pointer);
MUPbool   MPO_is_legal_indet(S_Pointer);
S_Pointer MPO_get_indets(S_Pointer *, long *, MUPbool);
long	  MPO_parse_order(S_Pointer);
 
short     MPO_arg2poly(S_Pointer, long *, S_Pointer *, S_Pointer *,
		       MPOT_poly_type *);
short     MPO_2args2poly(S_Pointer, long *, S_Pointer *, S_Pointer *,
			 S_Pointer *, MPOT_poly_type *);
long      MPO_find_lterm(S_Pointer, MPOT_poly_type *, long);
void      MPO_get_type(S_Pointer, MPOT_poly_type *);
void	  MPO_make_expr_type(MPOT_poly_type *);
void	  MPO_make_imod_type(MPOT_poly_type *, S_Pointer);
void      MPO_make_dom_type(MPOT_poly_type *, S_Pointer);
void	  MPO_free_type(MPOT_poly_type *);
S_Pointer MPO_new_poly_from_data(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_new_const_poly_data(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_new_lin_poly_data(long, MPOT_poly_type *);
S_Pointer MPO_new_type_expr(MPOT_poly_type *);
S_Pointer MPO_remove_coeff(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_remove_list_elem(S_Pointer, long);
S_Pointer MPO_new_const_poly(short, S_Pointer);

 
/* Funktionen in MPO_arithm.c */

S_Pointer MPO_divide_univar(S_Pointer, S_Pointer, S_Pointer *, MUPbool,
    			    S_Pointer (*)(S_Pointer, S_Pointer,
					  MPOT_poly_type *),
			    MPOT_poly_type *);
S_Pointer MPO_divide_multivar(S_Pointer, S_Pointer, S_Pointer *, MUPbool,
                              S_Pointer (*)(S_Pointer, S_Pointer,
                                            MPOT_poly_type *),
                              MPOT_poly_type *);
S_Pointer MPO_pdivide_univar(S_Pointer, S_Pointer, long *, S_Pointer *,
			     MUPbool, MPOT_poly_type *);
#ifdef DEADCODE
S_Pointer MPO_pdivide_multivar(S_Pointer, S_Pointer, long *, S_Pointer *,
                               MUPbool, MPOT_poly_type *);
#endif
S_Pointer MPO_evalp_univar(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_evalp_multivar(S_Pointer, S_Pointer, long, S_Pointer *,
			     MPOT_poly_type *);
void      MPO_sort_expos(MPOT_expos *, long, long);
S_Pointer MPO_genpoly(S_Pointer, S_Pointer, long *, MPOT_poly_type *);
S_Pointer MPO_diff_coeffs(S_Pointer, MPOT_poly_type *, S_Pointer, S_Pointer, S_Pointer*);
S_Pointer MPO_diff_terms(S_Pointer, long, MPOT_poly_type *);
 
/* Funktionen in MPO_basics.c */

void	  MPO_init_arithm(MPOT_poly_type *);
 
S_Pointer MPO_dist_plus(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dist_mult(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dist_power(S_Pointer, long, MPOT_poly_type *);
void      MPO_dist_negate(S_Pointer, MPOT_poly_type *);
long	  MPO_mult_long(long, long);

S_Pointer MPO_expr_expr2coeff(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_coeff2expr(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_plus(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_mult(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_divide(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_divide_exact(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_expr_power(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_expr_negate(S_Pointer, MPOT_poly_type *);
 
S_Pointer MPO_imod_expr2coeff(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_coeff2expr(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_plus(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_mult(S_Pointer, S_Pointer, MPOT_poly_type *);
MUPbool   MPO_imod_is_invertible(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_divide(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_imod_power(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_imod_negate(S_Pointer, MPOT_poly_type *);
 
S_Pointer MPO_dom_expr2coeff(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_coeff2expr(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_plus(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_mult(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_intmult(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_divide(S_Pointer, S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_power(S_Pointer, long, MPOT_poly_type *);
S_Pointer MPO_dom_negate(S_Pointer, MPOT_poly_type *);
S_Pointer MPO_dom_norm(S_Pointer, MPOT_poly_type *);

/* werden zZt. nicht benoetigt:
long      MPO_dist_lex_excmp();
long      MPO_dist_deg_excmp();
*/

/*----------------------------------------------------------------------
 * globale Variablen
 */
 
extern S_Pointer MPOV_INDETS;   /* assoziativer Speicher mit den Listen der
                                 * Unbestimmten aller Polynome */

/*
 * Zeiger auf die Grundoperationen der Koeffizienten-Ringe
 *
 * ACHTUNG: Die Position der Array-Elemente muss mit der jew.
 *          Konstante fuer den Ring (zB MPOC_EXPR) uebereinstimmen!
 */

extern MPOT_coeff_ops MPOV_coeff_ops[];

/* Funktionsname fuer Fehlermeldungen */
 
extern char *MPOV_fname;
 
/* MMM-Strings fuer Optionen und Namen von Domain-Attributen */
 
VALUE_DECL( S_Pointer, MSTR_IntMod);
VALUE_DECL( S_Pointer, MSTR_Expr);
VALUE_DECL( S_Pointer, MSTR_LexOrder);
VALUE_DECL( S_Pointer, MSTR_DegreeOrder);
VALUE_DECL( S_Pointer, MSTR_DegInvLexOrder);
VALUE_DECL( S_Pointer, MSTR_Rem);
VALUE_DECL( S_Pointer, MSTR_Quo);
VALUE_DECL( S_Pointer, MSTR_Exact);
 
/* Strings fuer Namen von Domain-Attributen */
 
extern S_Pointer MPOV_str_zero;
extern S_Pointer MPOV_str_one;
extern S_Pointer MPOV_str_norm;
extern S_Pointer MPOV_str_plus;
extern S_Pointer MPOV_str_mult;
extern S_Pointer MPOV_str_divide;
extern S_Pointer MPOV_str_power;
extern S_Pointer MPOV_str_negate;
extern S_Pointer MPOV_str_intmult;
extern S_Pointer MPOV_str_convert;
extern S_Pointer MPOV_str_expr;
extern S_Pointer MPOV_str_diff;
extern S_Pointer MPOV_DOMAIN_ASS_POLY;

/* Identifier (CAT_IDENT) */
 
extern S_Pointer MPOV_id_IntMod;
extern S_Pointer MPOV_id_Expr;
 
/* Expressions (CAT_EXPR) */

extern S_Pointer MPOV_indet_call ;
/* end of file */

#endif /* __MPO_poly_help__ */
