/**************************************************************************
**                         MPR_protocol.h                                **
** In diesem File werden die Typen f"ur das Protokoll MPR... definiert.  **
** Au\3erdem werden die Schnittstellenfunktionen deklariert.             **
**									 **
** Werden nur die Typ-Konstanten fuer die Meldungen benoetigt, so ist    **
** das Makro MPR_TYPE_CONST zu definieren.				 **
**                                                                       **
**   ALLE AENDERUNGEN MUESSEN IN DOC/PAPER/KERNEL/MPR_protocol.tex       **
**   DOKUMENTIERT WERDEN !!!!                                            **
**                                                                       **
**************************************************************************/

#ifndef __MPR_protocol__
#define __MPR_protocol__

/**************************************************************************
** Typ-Konstanten fuer Meldungen                                         **
**************************************************************************/
#define MPRCno_of_messages	  56

#define MPRCillegal                0    /* unzulaessiger Typ fuer Fehlerpruefung */
#define MPRCinput                  1
#define MPRCoutput                 2
#define MPRCsystem                 3
#define MPRCgettextinput           4
#define MPRCgetinput               5
#define MPRCtextwidth              6
#define MPRCprettyprint            7
#define MPRChelp                   8
#define MPRCerror                  9
#define MPRCexample               10
#define MPRCmdxcommand            11
#define MPRCdisplayvariables      12
#define MPRCprompt		  13
#define MPRClongprompt      	  14
#define MPRCdebuggermode    	  15
#define MPRCmdxprompt             16
#define MPRC_mv_graphic_cmd       17
#define MPRC_vx_plot_cmd          18
#define MPRC_vm_plot_cmd          19
#define MPRC_mv_graphic_error     20
#define MPRC_vx_graphic_error     21
#define MPRCsignal                22      /* Nachricht ist Signal (s.u.) */
#define MPRCmdxoutput		  23
#define MPRCmdxerror		  24
#define MPRCsyntaxerror		  25
#define MPRCincompleteinput	  26
#define MPRCgetstatus             27
#define MPRCstop                  28      /* stoppt die aktuelle Berechnung,
                                           * der Kern kehrt wie bei Ctl-C auf interaktive
                                           * Ebene zurueck (zZt nur unter Windows) */
#define MPRCquit                  29      /* beendet den Kern wie mit Ctl-C und 'quit'
                                           * (zZt nur unter Windows) */
#define MPRCconnect               30      /* koordiniert den Zugriff auf den Kern, falls
                                           * gleichzeitig mehrere Clients mit dem Kern
                                           * verbunden sind (zZt. nur unter Windows fuer
                                           * Cornelsen-Version) */

#ifdef DEBUG_GUI

 /* Protokollerweiterung MDB (Mupad DeBugger) */

 #define MPRCmdb_file_pos          31
 #define MPRCmdb_disp_list         32
 #define MPRCmdb_disp_set          33
 #define MPRCmdb_disp_clear        34
 #define MPRCmdb_disp_clearall     35
 #define MPRCmdb_proc_list         36
 #define MPRCmdb_proc_up           37
 #define MPRCmdb_proc_down         38
 #define MPRCmdb_proc_goto         39
 #define MPRCmdb_bkpt_list         40
 #define MPRCmdb_bkpt_set          41
 #define MPRCmdb_bkpt_clear        42
 #define MPRCmdb_bkpt_clearall     43
 #define MPRCmdb_next              44
 #define MPRCmdb_step              45
 #define MPRCmdb_cont              46
 #define MPRCmdb_execute           47
 #define MPRCmdb_quit              48
 #define MPRCmdb_print             49
 #define MPRCmdb_help              50
 #define MPRCmdb_status            51
 #define MPRCmdb_output            52
 #define MPRCmdb_prompt            53
 #define MPRCmdb_error             54
 #define MPRCmdb_start             55

/* Spezifikation:
**
** Typ [Kommunikationsrichtung] Nachrichtenformat
** (M->X = MuPAD-Kernel an Frontend, X->M = andere Richtung, M<>X = beide Richtungen)
**
**
** MPRCmdb_file_pos [M->X] "%s,%ld" z.B. "file.mu,10"
**	Sendet den Namen der aktuellen Datei sowie die aktuelle Zeilennummer an das Frontend,
**	der Name und die Nummer sind durch ein Komma getrennt. Der Dateiname ist kann auch
**	ein URL bzw. ein TAR-URL sein, er muss vom Frontend mit den Kernel Funktionen
**	ge:offnet und gelesen werden.
**
**
** MPRCmdb_disp_list [M->X] "%s\n%s\n..."
**	Sendet die Liste aller Display-Variablen. Die Liste ist ein String durch \n's
**	getrennt. Kann Unter Unix sofort weiterverwendet werden, unter Windows.. ? (\r ?)
**	Diese Liste wird immer gesendet, wenn im Kern eine Ver"anderung eintritt. Eine
**	explizite Abfrage halte ich f"ur unn"otig. Diese Liste ersetzt jede alte Liste!
**	Wird nach jedem Step usw. ausgel"ost! Wenn die Nachricht NULL ist, so ist die
** Liste leer.
**
**
** MPRCmdb_disp_set [X->M] "%s" z.B. "varname"
**	Setzen einer Display-Variablen. Existiert diese Variable nicht so sollte ein
**	Fehler zur"uckgesendet werden ? Enth"alt der String Trennzeichen (\t\n ) so
**	gilt jedes Teilwort als Variablenname.
**	-> l:ost MPRCmdb_disp_list aus !
**
**
** MPRCmdb_disp_clear [X->M] "%s" z.B. "varname"
**	L"oscht die angegebene Variable explizit. Evt. Fehlermeldung, falls nicht vorhanden.
**	(Kann nicht passieren, da das Frontend den Namen aus der Liste nimmt, die der Kern
**	gesendet hat. Existiert der Name nicht, sind Kern,Debugger und Frontend kaputt!)
**	-> l:ost MPRCmdb_disp_list aus !
**
**
** MPRCmdb_disp_clearall [X->M] NULL
**	L"oscht die interne Liste von Display-Variablen.
**	-> l:ost MPRCmdb_disp_list aus !
**
**
** MPRCmdb_proc_list [M->X] "%s\n%s\n%s..."
**	Gibt den Procstack aus. NULL bedeutet einen leeren Procstack.
**
**
** MPRCmdb_proc_up [M<>X] NULL
**	"Cursor-Up" im Procstack. Kann auf beiden Seiten ausgel:ost werden; Wenn das
**	Frontend dies ausl:ost, so sendet der Kern dies jedoch nochmal zur:uck; hier
**	muss man evt. aufpassen beim Programmieren.
**
**
** MPRCmdb_proc_down [M<>X] NULL
**	Identisch mit ..._up
**
**
** MPRCmdb_proc_goto [X->M] "%s" z.B. "procname"
**	MuPAD soll die Prozedur suchen. Dies l:ost dann ein MPRCmdb_file_pos aus,
**	d.h. die Datei mit der entsprechenden Prozedur wird angezeigt.
**	(Prozedur an Zeile 'pos')
**
**
** MPRCmdb_bkpt_list [M->X] "%s\n%s\n%s..."
**	Gibt die Liste der Breakpoints aus. NULL = keine Breakpoints.
**	Evt. Abfrage in R:uckrichtung ?
**
**
** MPRCmdb_bkpt_set [X->M] "%s,%ld[,%s]" z.B. "file.mu,10" oder "file.mu,10,cond=1"
**	Setzt einen Breakpoint, l:ost ein MPRCmdb_bkpt_list aus. Wenn das Frontend eine
**	Eingabezeile f"ur eine 'Condition' hat, so wird diese zus"atzlich :ubertragen.
**
**
** MPRCmdb_bkpt_clear [X->M] "%s,%ld" z.B. "file.mu,10"
**	L:oschen eines Breakpoints, l:ost ein MPRCmdb_bkpt_list aus.
**
**
** MPRCmdb_bkpt_clearall [X->M] NULL
**	L:oscht die komplette Liste von Breakpoints. L:ost ein MPRCmdb_bkpt_list aus,
**	mit Nachricht NULL.
**
**
** MPRCmdb_next [X->M] NULL
**	F:uhre NEXT aus
**
**
** MPRCmdb_step [X->M] NULL
**	F:uhre STEP aus
**
**
** MPRCmdb_cont [X->M] NULL
**	F:uhre CONT aus
**
**
** MPRCmdb_execute [X->M] "%s" z.B. "mupad expression"
**	Die angegebene 'expression' wird ausgef:uhrt. Das Ergebnis wird mit
**	MPRCmdb_output ausgegeben.
**
**
** MPRCmdb_quit [X->M] NULL
**	Beenden des Debuggermodus.
**
**
** MPRCmdb_print [X->M] "%s" z.B. "mupad expression to be printed"
**	Druckt den Inhalt einer 'expression' mit MPRCmdb_output. Ich weiss nicht
**	wo der Unterschied zu 'execute' liegen soll..
**
**
** MPRCmdb_help [X->M] NULL
**	Ruft den internen Hilfetext des Debuggers auf. Besser: 'Bubblehelp' auf
**	allen GUI Elementen ?
**
**
** MPRCmdb_status [M->X] "%s ??" 
**	Kompakte Nachricht mit dem Status des Debuggers (running, waiting, line, file, ..)
**	Evt. nutzlos ?
**
**
** MPRCmdb_output [M->X] "%s" z.B. "stop at line ..."
**	Ausgabefunktion. Text ist steuerzeichenlos bis auf \n und \t.
**	(Frontend schaltet 'Wartecursor' ein..)
**
**
** MPRCmdb_prompt [M->X] "%s" z.B. "\nmdx> "
**	Prompt ausgeben.
**	(Frontend schaltet 'Wartecursor' aus..)
**
** MPRCmdb_start [X->M] "%s" z.B. "functionname"
**	Starten des Debuggermodus.
*/

#endif /* DEBUG_GUI */

/**************************************************************************
** MPRTtype                                                              **
** Instanzen dieses Typs geben an, um was f"ur einen Typ von Nachricht   **
** es sich bei dieser Nachricht handelt.                                 **
**************************************************************************/
typedef unsigned char MPRTtype ;


#ifndef MPR_TYPE_CONST

#define MEXDSTANDARD
#ifdef DOS_FILENAMES
#  include "MEX_ext.h"
#else
#  include "MEX_extern.h"
#endif

typedef int MPRTbool;
typedef	int MPRTstatus;

/**************************************************************************
** MPRTchannel                                                           **
** Auf Instanzen diesen Typs setzen die Funktionen des Protokolls auf.   **
** Eine Instanz besteht aus einem Zeiger auf ein FILE, aus dem die Daten **
** gelesen bzw. auf den sie geschrieben werden, einem Puffer zum         **
** zwischenpuffern von gelesenen Daten. Dieser wird ben"otigt, da einige **
** Bytes nur im Kontext mit den darauffolgenden Bytes interpretiert      **
** werden k"onnen. Au\3erdem beinhaltet die Instanz einen Z"ahler der    **
** angibt, wieviele Bytes im Puffer gespeichert sind.                    **
**************************************************************************/

#ifdef MSDOS
typedef struct _MPRSchannel {
    int dummy;
} MPRSchannel;
#else
typedef struct _MPRSchannel
{
	FILE	*file ;
	char	readbuffer[1] ;
	short	rsize ;
	int	bytes;
} MPRSchannel ;
#endif
typedef MPRSchannel *MPRTchannel ;


/**************************************************************************
** MPRTchar                                                              **
** Dies ist der Typ, der aus einer Instanz des Typs MPRTchannel          **
** herausgelesen wird. Eine Instanz dieses Typs enth"alt mehr            **
** Informationen als nur den Wert des Bytes, den er repr"asentiert.      **
** Sie enth"alt z.B. die Information, ob es sich "uberhaupt um ein Byte  **
** handelt oder um das Ende einer Nachricht. Sie kann auch weitere       **
** Informationen zum Ende der Nachricht enthalten. Es w"are auch         **
** m"oglich, in eine Nachricht Informationen einzuschieben, dies m"u\3te **
** das entsprechende Protokoll dann aber verstehen.                      **
**************************************************************************/
typedef short MPRTchar ;


/**************************************************************************
** Sonstige Konstanten                                                   **
**************************************************************************/
#define MPRCfail		30	  /* ioctl() fail auf Deskribtor */
#define MPRCnone		31	  /* keine bytes auf Deskribtor  */
#define MPRCread		32	  /* bytes auf Deskribtor        */


/*** Moegliche Signale des Typs MPRCsignal *******************************/
#define MPRC_SIG_break            'b' 
#define MPRC_SIG_quit             'q'
#define MPRC_SIG_kill             'k'


/*
 * Konstanten f"ur die Kommunikation mit VCAM // "Anderungen bitte auch in
 *                          DOC/PAPER/KERNEL/MPR_protocol.tex aktualisieren 
 *
 * Dazu einige Erl"auterungen:
 *
 * MPRC_mv_graphic_cmd      Kennzeichnet eine Botschaft von MuPAD an VCam,
 *                          in der ein Grafik-Befehl (= Grafikdaten + Befehl)
 *                          geschickt wird.
 *
 * MPRC_vx_plot_cmd         Kennzeichnet eine Botschaft von VCam an XMuPAD,
 *                          in der ein Plot-Befehl an XMuPAD geschickt wird,
 *                          der ins XMuPAD-Fenster geschrieben werden soll.
 *
 * MPRC_vm_plot_cmd         Kennzeichnet eine Botschaft von VCam an MuPAD,
 *                          in der dem Kern MuPAD ein Plot-Befehl geschickt
 *                          wird. Zur Zeit wird dieser Befehl wohl noch "uber
 *                          XMuPAD geschickt.
 *
 * MPRC_vx_graphic_error    Kennzeichnet eine Botschaft von VCam an XMuPAD, 
 *                          in der XMuPAD mitgeteilt wird, dass das Grafiktool
 *                          Probleme hat (bspw. konnte eine Datei nicht ge"off-
 *                          net werden).
 *
 * MPRC_mv_graphic_error    Kennzeichnet eine Botschaft vom Kern MuPAD an das
 *                          Grafiktool VCam, in der diesem mitgeteilt wird, das
 *                          es bei der Bearbeitung eines Plot-Befehles, welcher
 *                          von VCam an den Kern geschickt wurde, ein Problem
 *                          gab. Bisher kam dann immer die Fehlermeldung: 'An
 *                          error was detected in the definition of the objects.'
 */

/**************************************************************************
** Konstanten des  Protokolls MPR...                                     **
**************************************************************************/


#define MPRCEOM             1000  /* Ende einer Nachricht */
#define MPRCNOCHAR          1001  /* Kein Zeichen auf dem Kanal */

/**************************************************************************
** Deklaration der Funktionalen Schnittstelle des Protokolls MPR...      **
**************************************************************************/
#ifndef MSDOS

MPRTchannel	MPR_createFP	(FILE *fp) ;
void		MPR_delete	(MPRTchannel ch) ;
MPRTtype	MPR_getstart	(MPRTchannel ch) ;
MPRTchar	MPR_get		(MPRTchannel ch) ;
MPRTchar	MPR_try		(MPRTchannel ch) ;
MPRTbool	MPR_EOM		(MPRTchar c) ;
MPRTbool	MPR_nochar	(MPRTchar c) ;
MPRTbool	MPR_ready       (MPRTchannel ch);
unsigned char   MPR_convert	(MPRTchar c) ;
void		MPR_flush	(MPRTchannel ch) ;
MPRTstatus	MPR_checkChannel(MPRTchannel ch) ;
void		MPR_putstart	(MPRTchannel ch, MPRTtype type) ;
void		MPR_putend	(MPRTchannel ch) ;
void		MPR_putchar	(MPRTchannel ch, unsigned char c) ;
void		MPR_putstring	(MPRTchannel ch, char *s) ;
void		MPR_putmessage	(MPRTchannel ch, MPRTtype type, char *s) ;
void		MPR_putsignal	(MPRTchannel ch, char signal) ;
void      MPR_pipemessage(MPRTchannel ich,MPRTchannel och,MPRTtype type);

#else
/* Unter Windows wird nur die Ausgabe-Routine MPR_putmessage verwendet.
 * Eingaben werden vom Windows-Rahmen verarbeitet. Alle Ausgabe gehen auf
 * einen Kanal, nur der Typ ist entscheidend. */

#define MPR_putmessage(ch,type,str) { MSW_pr_puts((type), (str)); }

#endif /* ! MSDOS */

#endif /* ! MPR_TYPE_CONST */
#endif /* __MPR_protocol__ */
