/* $Date: 1997/09/01 13:24:54 $  $Author: olli $  $Revision: 1.8 $ */

/**************************************************************************/
/* FILE:             MSU_eval_subs.h                                      */
/* Autor:            Karsten Morisse                                      */
/* Erstellt:         08/08/91                                             */
/* Letzte Aenderung: 08/09/93                                             */
/*                                                                        */
/* Dieses File enthaelt die Deklarationen des Modules MSU_eval_subs       */
/**************************************************************************/

#ifndef __MSU_eval_subs__
#define __MSU_eval_subs__


#ifdef DOS_FILENAMES
#  include "MSU_evah.h"
#else
#  include "MSU_subs_help.h"
#endif


/*** Einige allgemeine Makros *********************************************/

/**************************************************************************/
/* Wird im Fehlerfalle aufgerufen.					  */
/* Parameter: a - Fehlernummer						  */
/*	      b - String, der als Teil der Fehlermeldung erscheint 	  */
/*	      c - S_Pointer, an den das Ergebnis zugewiesen wird	  */
/**************************************************************************/

#ifdef ERRORLEVEL

#define MSU_set_error(a,b,c)			\
{						\
	switch (GLOBAL_READ(MEVV_ERRORLEVEL)) {	\
	case MEVC_FULL_ERROR_DETECTION:		\
		MEV_set_error(a,b);		\
		break;				\
	default:				\
		c = VALUE_READ(MUPV_CAT_FAILED);\
		MMMcopy(c);			\
		break;				\
	}					\
}

#else

#define MSU_set_error(a,b,c)      \
{                                 \
  c = VALUE_READ(MUPV_CAT_FAILED);\
  MMMcopy(c);                     \
}

#endif /* ERRORLEVEL */

#define MSU_cat(t)   MMMZ(t, CATEGORY, MMMread)->cat

#define MSU_spec(t) MMMZ(t, CATEGORY, MMMread)->spec


/**************************************************************************/
/* Einstufige Vereinfachung und Zuweisung des Ergebnisses.                */
/**************************************************************************/
#define MSU_simplify(a,b,unsimplified)                               		 \
{									 \
	if (unsimplified)				 	 	 \
	   { MMMset(&a, b); }			         		 \
	else								 \
	   { a = MEV_eval(b, MEVC_UNKNOWN,                               \
                          MEVC_SIMPLIFY|MEVC_NO_EVAL_ARGS_BIT); }        \
}


/**************************************************************************/
/* Konvertiert eine CAT_NULL in eine CAT_NIL.				  */
/* Parameter: S_Pointer  *s, t						  */
/**************************************************************************/
#define MSU_null2nil(s,t)				\
{							\
	if (t == MUPV_CAT_NULL || t == MMMNULL)		\
	   { MMMset(s, MUPV_CAT_NIL); }		        \
	else						\
           { MMMset(s, t);  }	                        \
}  


/*** Globale Variable mit ihren moeglichen Werten *************************/

extern long MSUV_GET_OPERAND_INFO ;
#define MSUC_STANDARD           0
#define MSUC_SPECIALBITS        1
#define MSUC_CODE               2


extern S_Pointer MSUV_TMP;
extern S_Pointer MSUV_ass_mem_subs;  /* Dieser assoziative Speicher wird von
                                       der Funktion MSU_eval_subs ben"otigt,
                                       damit auch in einem unendlich tiefen
                                       Baum substituiert werden kann. */
 
/*** Prototypen der Dateien `MSU_eval_subs.c' und `MSU_eval_type.c' *******/

/*** Initialisierungsroutinen ***/

void MSU_init ( VOID );			
void MSU_exit ( VOID );

/*** Routinen fuer 'nops' ***/

S_Pointer MSU_eval_nops (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
long MSU_eval_nops_intern (	S_Pointer	 t,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );

/*** Routinen fuer 'subs' ***/

S_Pointer MSU_eval_subs (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
long MSU_eval_subs_intern (	S_Pointer	 t, 
				S_Pointer	 old, 
				S_Pointer	 neww,
                                long             prev_func,
                                long             eval_type,
				short            *unsimplified);

/*** Routinen fuer 'subsex' ***/

S_Pointer MSU_eval_subsex (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
long MSU_eval_subsex_intern (	S_Pointer	 t,
				S_Pointer	 old,
				S_Pointer	 neww,
				long		 prev_func,
				long		 eval_type,
				short		 *unsimplified);
long MSU_search_and_subs (	S_Pointer	 t,
				S_Pointer	 otmp,
				S_Pointer	 ntmp,
				long		 prev_func,
				long		 eval_type,
				short		 *unsimplified);

/*** Routinen fuer 'subsop' ***/

S_Pointer MSU_eval_subsop (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
void MSU_eval_subsop_int (	S_Pointer	*s, 
				long		 ind,
				S_Pointer	 t, 
				long		*error_flag, 
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec, 
				short		*unsimplified);
S_Pointer MSU_eval_subsop_list(	S_Pointer	*s,
				S_Pointer	 neww,
				S_Pointer	 list,
				long		 ind,
				long		 end,
				long		*error_flag,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec,
				short		*unsimplified );


/*** Routinen fuer 'testtype' ***/

S_Pointer MSU_eval_testtype (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );

/*** Routinen fuer 'type' ***/

S_Pointer MSU_eval_type (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
S_Pointer MSU_eval_type_intern( S_Pointer	 t,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );

/*** Routinen fuer 'domtype' ***/

S_Pointer MSU_eval_domtype (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
S_Pointer MSU_eval_domtype_intern( S_Pointer	 t, 
				S_Pointer	 exec );
				
/*** Routinen fuer 'ext...' ***/

S_Pointer MSU_eval_extnops (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
S_Pointer MSU_eval_extop (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
S_Pointer MSU_eval_extsubsop (	S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );

/*** Prototypen der Datei `MSU_eval_op.c '**********************************/

/*** Routinen fuer 'op' ***/

S_Pointer MSU_eval_op (		S_Pointer	 s,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
S_Pointer MSU_eval_op_intern (	S_Pointer	 t,
				S_Pointer	 ops,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );
S_Pointer MSU_eval_op_list (	S_Pointer	 s,
				S_Pointer	 t,
				long		 prev_func,
				long		 eval_type,
				S_Pointer	 exec );


#endif /* __MSU_eval_subs__ */
